/*
 * Softing CAN PCI Driver
 * Copyright (C) 1994-2006, Softing AG, Werner Knorsch
 * 
 * See file CREDITS for list of people who contributed to this
 * project.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * email: support.automation@softing.com
 * Softing AG, Richard-Reitzner-Allee 6, 85540 Haar, Germany
 */


/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTIONAL_MODULE_DESCRIPTION                                            */
/*                                                                          */
/* FUNCTIONS              CANDR_reset_board                                 */
/*                        CANDR_reset_chip                                  */
/*                        CANDR_initialize_chip                             */
/*                        CANDR_initialize_chip2                            */
/*                        CANDR_set_mode                                    */
/*                        CANDR_set_mode2                                   */
/*                        CANDR_set_output_control                          */
/*                        CANDR_set_output_control2                         */
/*                        CANDR_set_acceptance                              */
/*                        CANDR_set_acceptance2                             */
/*                        CANDR_echo_tx_req                                 */
/*                        CANDR_echo_tx_req2                                */
/*                        CANDR_enable_fifo_transmit_ack                    */
/*                        CANDR_enable_fifo_transmit_ack2                   */
/*                        CANDR_start_chip                                  */
/*                                                                          */
/*                        CANDR_read_ac                                     */
/*                        CANDR_send_data                                   */
/*                        CANDR_send_data2                                  */
/*                        CANDR_send_remote                                 */
/*                        CANDR_send_remote2                                */
/*                                                                          */
/*HEAD_END*******************************************************************/

/*
 * $Id: can_card.c,v 1.6 2003/11/04 12:48:16 kno Exp $
 */

/******************************************************************
**  include files
*******************************************************************/



#include "os.h"
#include "sysdep.h"

#include "BaseTypes.h"
#include "can_def.h"
#include "can_device.h"
#include "shared_usb_mem.h"
#include "card_def.h"

#include "can_card.h"


#include "can_loadcard.h"
#include "canpro.h"
#include "canL2.h"
#include "canprousb.h"



USIGN32 burnFirmware(PCDEVICES pDevice, USIGN8 *pFirmware, USIGN32 u32FirmwareSize);

extern unsigned long CanFwBytes;
extern unsigned char CanFw[];
extern unsigned long layer2UsbBytes;
extern unsigned char layer2Usb[];

static USIGN32 MsgCounter = 0; // for debugging only

USIGN32 counter = 0; // for debugging only

#ifdef _CANPROUSB_
#define FIRMWARE      layer2Usb
#define FIRMWARE_SIZE layer2UsbBytes
#else
#define FIRMWARE      CanFw
#define FIRMWARE_SIZE CanFwBytes
#endif

/* -------  application globals  -------------------------------------------*/

USIGN32 TimeF1[5] = {1,1,5,3,25}; 
USIGN32 TimeF2[5] = {1,1,2,1,4};

#if !defined(CANPRO) && !defined(CANPCIE)
USIGN32 calculate_time(USIGN32* pLowTime, USIGN32* pHighTime, USIGN32 timestampType)
{
  USIGN32 time, time_low, time_high, temp_time;
  
  /* CAN-ACx-PCI or CAN-ACx-104 */
  /* get 32bit timer value of timer with 0.64us resolution */
  time_low   = _readl(pLowTime);
  rmb();
  /* get sum of 1us timer values at 0.64us timer overflows */
  time_high  = _readl(pHighTime);
  rmb();
  /* calculate 1us timer value from          */
  /* 0.64us timer (temp1) value              */
  /* time_1us = time_0.64 * (16/25)          */
  /* --------------------------------------- */
  /* save reminder of division (time_1us/25) */
  temp_time  = time_low % TimeF1[timestampType];
  /* get integer part of division result(time_1us/25) */
  time_low   = time_low / TimeF1[timestampType];
  /* create sum of (integer * 16) + (rest * 16) and */
  /* add 0.64us timer value at the 1.0us timers     */
  /* overflow, write this sum to param struct       */
  time = (time_low << TimeF2[timestampType]) +
      ((temp_time << TimeF2[timestampType]) / TimeF1[timestampType]) + time_high;
      
  return time;  
}
#endif // CANPRO



INT32 canusb_insert_fifo(PCDEVICES pDevice, USBXMT_ITEM *pEntry, T_CANUSBP_SHARED_MEM *pShMem)
{
  USIGN16 wr_xmt_fifo;
  USBXMT_ITEM volatile *pXmtItem;
  //   CAN_ADDRESS irql;
  
  //   irql = sync_get_access(&pDevice->ChLock);
  wr_xmt_fifo = pShMem->can_pro_usb_fifo_mode.xmt.wr_xmt_fifo;
  if (pShMem->can_pro_usb_fifo_mode.xmt.rd_xmt_fifo != wr_xmt_fifo)
  {
    pXmtItem = (USBXMT_ITEM volatile *)&pShMem->can_pro_usb_fifo_mode.xmt.xmt_fifo[wr_xmt_fifo];
    memcpy((PVOID)pXmtItem,pEntry,sizeof(USBXMT_ITEM));
    
    pShMem->can_pro_usb_fifo_mode.xmt.wr_xmt_fifo = (wr_xmt_fifo+1)&XMT_FIFO_USB_MASK;
    
  }
  else
  {
    // FIFO full
    //     sync_free_access(&pDevice->ChLock, irql);
    return CANL2_REQ_PENDING;
  }
  
  //   sync_free_access(&pDevice->ChLock, irql);
  if(pDevice->canprousb.IoChainXmtState == CHAIN_IDLE)
  {
    XmtNextPacket(pDevice);
  }
  
  return CANL2_OK;
}



/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_get_device_id                       */
/* FUNCTIONAL_DESCRIPTION         read device ID of the opened device       */
/* OUTPUTS                        OK_ if device is open                     */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_get_device_id(USIGN32 *pulDeviceId, volatile PCDEVICES pDevice)
{
    _TRACE5("CANDR_get_device_id(%09u)\n", pDevice->Serialnumber);
    
    if ((*pulDeviceId = pDevice->u16DeviceId) != 0)
        return CANL2_OK;
    else
        return CANL2_ERR;
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_reset_board                         */
/* FUNCTIONAL_DESCRIPTION         reset and download of firmware            */
/* INPUTS                         -                                         */
/* OUTPUTS                        OK_:    download ok                       */
/*                                ERROR1: no response of firmware           */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/

int_ CANDR_reset_board(volatile PCDEVICES pDevice)
{
  
  INT32 iRetVal = 0;

  _TRACE5("CANDR_reset_board(%09u)\n", pDevice->Serialnumber);
    
   
  pDevice->SignalsDefined = FALSE;    // Lowspeed-Modul ist nicht initialisiert
    
  
  iRetVal = burnFirmware(pDevice, FIRMWARE, FIRMWARE_SIZE);
    
  if(!iRetVal)
  {
    pDevice->u8FwId = LAYER2_FW; //Layer2
  }

  return (iRetVal);

}



/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_init_signals                        */
/* FUNCTIONAL_DESCRIPTION         init C165 ports for piggyback triggering  */
/* INPUTS                         -                                         */
/* OUTPUTS                        OK_       acknownledged by firmware       */
/*                                ERROR1    Signal initialized/             */
/*                                          no initialisation possible      */
/*                                ERROR2    input-only signal was tried to  */
/*                                          initialize as output            */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_init_signals(USIGN32 ulChannelDirectionMask,
                   USIGN32 ulChannelOutputDefaults, volatile PCDEVICES pDevice)
{
  return (CANL2_NOT_IMPLEMENTED);
}




/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_read_signals                        */
/* FUNCTIONAL_DESCRIPTION         read C165 ports of piggyback triggering   */
/* INPUTS                         -                                         */
/* OUTPUTS                        OK_       acknownledged by firmware       */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_read_signals(USIGN32 *pulChannelRead, volatile PCDEVICES pDevice)
{
  return (CANL2_NOT_IMPLEMENTED);
}
    
 


/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_write_signals                       */
/* FUNCTIONAL_DESCRIPTION         write to C165 ports for piggyback         */
/* INPUTS                         -                                         */
/* OUTPUTS                        OK_       acknownledged by firmware       */
/*                                ERROR2    input-only signal was tried to  */
/*                                          write like an output            */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_write_signals(USIGN32 ulChannelWrite, USIGN32 ulCareMask,
                         volatile PCDEVICES pDevice)
{
  return (CANL2_NOT_IMPLEMENTED);
}





/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_reinitialize                        */
/* FUNCTIONAL_DESCRIPTION         reinitialization of firmware              */
/* OUTPUTS                        OK_                                       */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_reinitialize(volatile PCDEVICES pDevice)
{
  DFCT_STRUCT cmd_in;
  INT32 status;
  
  _TRACE5("CANDR_reinitialize(%09u)\n", pDevice->Serialnumber);
  StopIoChain(pDevice);
  cmd_in.wRequest = FCT_REINITIALIZE;
  cmd_in.wFRWReqRes = FALSE;
  status = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 100);
  
  return (status);
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_reset_rcv_fifo                      */
/* FUNCTIONAL_DESCRIPTION         resets receive fifo                       */
/* OUTPUTS                        OK_                                       */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_reset_rcv_fifo(volatile PCDEVICES pDevice)
{
  DFCT_STRUCT cmd_in;
  INT32 ret = VCARD_OK;
  T_CANUSBP_SHARED_MEM *pShM = (T_CANUSBP_SHARED_MEM *)pDevice->canprousb.pSharedMem;
  
  _TRACE5("CANDR_reset_rcv_fifo(%09u)\n", pDevice->Serialnumber);
  
  pDevice->canprousb.rcv_usb_enable = FALSE;
  
  pDevice->canprousb.RcvCounter = 0;
    
  // reset the host side fifo
  pShM->can_pro_usb_fifo_mode.rcv.rd_rcv_fifo = (pShM->can_pro_usb_fifo_mode.rcv.wr_rcv_fifo-1) & RCV_FIFO_USB_MASK;
  
  // reset the firmware side fifo
  cmd_in.wRequest = FCT_RESET_RCV_FIFO;
  cmd_in.wFRWReqRes = FALSE;
  ret = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 500);
  if(ret) 
  {
    _ERRPRINT("CANDR_reset_rcv_fifo(%09u) error %u\n", pDevice->Serialnumber, ret);
    return ret;
  }
  
  pShM->can_pro_usb_fifo_mode.rcv.rd_rcv_fifo = RCV_FIFO_USB_MASK;
  pShM->can_pro_usb_fifo_mode.rcv.wr_rcv_fifo = 0;
  pShM->lostMsgOnHostFifo = 0;
  
  pDevice->canprousb.rcv_usb_enable = TRUE;
  
  return ret;
  
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_get_version                         */
/* FUNCTIONAL_DESCRIPTION         reads versions                            */
/* INPUTS                         -                                         */
/* OUTPUTS                        OK_       acknownledged by firmware       */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_get_version(PINT32 sw_version, PINT32 fw_version, PINT32 hw_version,
                          PINT32 license, PINT32 can_chip_type, volatile PCDEVICES pDevice)
{
  _TRACE5("CANDR_get_version(%09u)\n", pDevice->Serialnumber);
  
  *sw_version = SW_VERSION;
  *fw_version = pDevice->fw_version;
  *hw_version = pDevice->hw_version;
  *license = pDevice->license;
  *can_chip_type = pDevice->chip_type;
  
  can_chip_type[1] = (pDevice->u8NumOfCh == 2) ? pDevice->chip_type : 0;
 
  return (CANL2_OK);
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_get_version_fw                      */
/* FUNCTIONAL_DESCRIPTION         reads versions                            */
/* INPUTS                         -                                         */
/* OUTPUTS                        OK_       acknownledged by firmware       */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_get_version_fw(PINT32 sw_version, PINT32 fw_version, PINT32 hw_version,
                          PINT32 license, PINT32 can_chip_type, volatile PCDEVICES pDevice)
{
  DFCT_STRUCT cmd_in;
  INT32 status;
  
  _TRACE5("CANDR_get_version_fw(%09u)\n", pDevice->Serialnumber);
  memset(&cmd_in, 0xff, sizeof(cmd_in));
  
  cmd_in.wRequest = FCT_GET_VERSION;
  cmd_in.wFRWReqRes = FALSE_;
  
  status = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 500);
  
  if (status != 0)
  {
    _ERRPRINT("CANDR_get_version_fw(%09u), Error %u\n", pDevice->Serialnumber, status);
    return (status);
  }
  
  *sw_version = SW_VERSION;
  *fw_version = cmd_in.sParam_1;
  *hw_version = ((cmd_in.sParam_2 & 0xFF)<<8) + (cmd_in.sParam_3 & 0xFF);
  *license = cmd_in.sParam_3;
  *can_chip_type = cmd_in.sParam_4;
  can_chip_type[1] = cmd_in.sParam_5;
  
//   printk("CANDR_get_version_fw: fw_version=%u, hw_version=%u, license=%u\n", *fw_version, *hw_version, *license);
  _TRACE5("CANDR_get_version_fw: fw_version=%u, hw_version=%u, license=%u\n", *fw_version, *hw_version, *license);

  return (cmd_in.sRetCode);
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_get_serial_number                   */
/* FUNCTIONAL_DESCRIPTION         reads serial number                       */
/* INPUTS                         -                                         */
/* OUTPUTS                        OK_       acknownledged by firmware       */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_get_serial_number(PUSIGN32 serial_number, volatile PCDEVICES pDevice)
{
  _TRACE5("CANDR_get_serial_number(%09u)\n", pDevice->Serialnumber);
 
  *serial_number = pDevice->Serialnumber;
    
  return (0);
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_get_serial_number_fw                */
/* FUNCTIONAL_DESCRIPTION         reads serial number                       */
/* INPUTS                         -                                         */
/* OUTPUTS                        OK_       acknownledged by firmware       */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_get_serial_number_fw(PUSIGN32 serial_number, volatile PCDEVICES pDevice)
{
  DFCT_STRUCT cmd_in;
  INT32 status;
  
  _TRACE5("CANDR_get_serial_number(%09u)\n", pDevice->Serialnumber);
  cmd_in.wRequest = FCT_GET_SN;
  cmd_in.wFRWReqRes = FALSE_;

  status = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 100);
  
  if (status != OK_)
  {
    _ERRPRINT("CANDR_get_serial_number(%09u), Error %u\n", pDevice->Serialnumber, status);
    return (status);
  }
  
  *(PUSIGN16) serial_number = cmd_in.sParam_1;
  *((PUSIGN16) serial_number + 1) = cmd_in.sParam_2;
    
  return (readw(&(cmd_in.sRetCode)));
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_reset_chip                          */
/* FUNCTIONAL_DESCRIPTION         sw reset of CAN controllers               */
/* INPUTS                         -                                         */
/* OUTPUTS                        OK_       acknownledged by firmware       */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/* CHANGES                                                                  */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_reset_chip(volatile PCDEVICES pDevice)
{
  DFCT_STRUCT cmd_in;
  INT32 status;
  
  USIGN32 DeviceType;
  INT32 sw_version, fw_version, hw_version, license, chip_type[2];
  INT32 ret;
  
  _TRACE5("CANDR_reset_chip(%09u)\n", pDevice->Serialnumber);

  StopIoChain(pDevice);
  
  pDevice->canprousb.rcv_usb_enable = FALSE;
  
  cmd_in.wRequest = FCT_REINITIALIZE;
  cmd_in.wFRWReqRes = FALSE_;
  
  status = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 500);
  
  
  memset(&cmd_in, 0, sizeof(cmd_in));
  DeviceType = pDevice->u16DeviceId;
  
  
  cmd_in.wRequest = FCT_RESET_CHIP;
  cmd_in.wFRWReqRes = FALSE_;
  
  status = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 500);
  
  if (status != OK_)
    return (status);
  
  pDevice->number_of_rcv_objects = 0;
  pDevice->number_of_xmt_objects = 0;
  pDevice->dyn_enable = FALSE;
  pDevice->object_buffer_enable = 1;
  pDevice->bus_state = 0;
  
  pDevice->canprousb.ReceiveFifoEnable = FALSE;
  pDevice->canprousb.TransmitRemoteFifoEnable = FALSE;
  pDevice->canprousb.TransmitAckFifoEnable = FALSE;
  pDevice->canprousb.TransmitReqFifoEnable = FALSE;
  pDevice->canprousb.lost_remotes = 0;
  pDevice->canprousb.enableXmtAck = FALSE;
  
  
//   CanEnableIrq(TRUE, pDevice);
  
  ret = cmd_in.sRetCode;
  
  if(ret) return ret;
  
  ret = CANDR_get_version_fw(&sw_version, &fw_version, &hw_version, &license, chip_type, pDevice);
  
  if(ret) return ret;
  
  pDevice->chip_type = chip_type[0];
  pDevice->fw_version = fw_version;
  pDevice->hw_version = hw_version;
  
  CANDR_reset_lost_msg_counter(pDevice);
  
  return ret;
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_initialize_chip,                    */
/*                                CANDR_initialize_chip2                    */
/* FUNCTIONAL_DESCRIPTION         defines bittiming                         */
/* INPUTS                         presc:                [1..64]             */
/*                                sjw:                  [1..4]              */
/*                                tseg1:                [1..16]             */
/*                                tseg2:                [1..8]              */
/*                                sam:               0: 1 Sample            */
/*                                                   1: 3 Sample            */
/* OUTPUTS                        OK_       acknownledged by firmware       */
/*                                ERROR1    inconsistent parameters         */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/* CHANGES                                                                  */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_initialize_chip(INT32 presc, INT32 sjw, INT32 tseg1, INT32 tseg2, INT32 sam,
                      volatile PCDEVICES pDevice)
{
  DFCT_STRUCT cmd_in;
  INT32 status;
  INT32 div8x = sam, ret;
  
  _TRACE5("CANDR_initialize_chip presc=%u, sjw=%u, tseg1=%u, tseg2=%u, sam=%u\n",presc, sjw, tseg1, tseg2, sam);
  
  cmd_in.wRequest = FCT_INITIALIZE_CHIP;
  cmd_in.sParam_1 = presc;
  cmd_in.sParam_2 = sjw;
  cmd_in.sParam_3 = tseg1;
  cmd_in.sParam_4 = tseg2;
  cmd_in.sParam_5 = div8x;
  cmd_in.wFRWReqRes = FALSE_;
  
  
  status = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 500);
  
  if (status != OK_)
    return (status);
  
  ret = cmd_in.sRetCode;
  
  if(ret)
  {
    _TRACE5("canpro_initialize_chip: presc=%i, sjw=%i, tseg1=%i, tseg2=%i, div8x=%i, ret=%u\n",presc,sjw,tseg1,tseg2,div8x,ret);
  }
  
  if(ret == -1) ret = CANL2_WRONG_PARAM; // mapping firmware to API
  
  return ret;
}



/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_set_mode,CANDR_set_mode2            */
/* FUNCTIONAL_DESCRIPTION         defines syncronisation control            */
/* INPUTS                         SleepMode 0: disable                      */
/*                                          1: enable (wakeup int           */
/*                                                     enabled ! )          */
/*                                SpeedMode 0: resynchronisation            */
/*                                             only with the recessive to   */
/*                                             dominant edge (high speed)   */
/*                                          1: resyn. rec -> dominant       */
/*                                             and dominant -> rec          */
/* OUTPUTS                        OK_       acknownledged by firmware       */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_set_mode(INT32 SleepMode, INT32 SpeedMode, volatile PCDEVICES pDevice)
{
    _TRACE5("CANDR_set_mode(%09u)\n", pDevice->Serialnumber);

    return 0;
}



/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_set_acceptance,                     */
/*                                CANDR_set_acceptance2                     */
/* FUNCTIONAL_DESCRIPTION         defines acceptance filter masks and codes */
/*                                for both standard and extended frames     */
/* INPUTS                         AccCodeStd: acceptance code for std frames*/
/*                                AccMaskStd: accpetance mask for std frames*/
/* INPUTS                         AccCodeXtd: acceptance code for xtd frames*/
/*                                AccMaskXtd: accpetance mask for xtd frames*/
/* OUTPUTS                        OK_       acknownledged by firmware       */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_set_acceptance(USIGN32 AccCodeStd,
                     USIGN32 AccMaskStd,
                     USIGN32 AccCodeXtd,
                     USIGN32 AccMaskXtd, volatile PCDEVICES pDevice)
{
  DFCT_STRUCT cmd_in;
  INT32 status;
  
  _TRACE5("CANDR_set_acceptance(0x%x,0x%x,0x%x,0x%x)\n", AccCodeStd,
          AccMaskStd, AccCodeXtd, AccMaskXtd);
  cmd_in.wRequest = FCT_SET_ACCEPTANCE;
  cmd_in.sParam_1 = AccCodeStd;
  cmd_in.sParam_3 = (USIGN16) (AccCodeXtd >> 0);
  cmd_in.sParam_5 = (USIGN16) (AccCodeXtd >> 16);
  cmd_in.sParam_2 = ~AccMaskStd;
  cmd_in.sParam_4 = (USIGN16) (~AccMaskXtd >> 0);
  cmd_in.sParam_6 = (USIGN16) (~AccMaskXtd >> 16);
  
  cmd_in.wFRWReqRes = FALSE_;
  
  status = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 100);
  
  if (status != OK_)
    return (status);
  
  return (cmd_in.sRetCode);
}



/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_set_output_control,                 */
/*                                CANDR_set_output_control2                 */
/* FUNCTIONAL_DESCRIPTION         writes input/output control register of   */
/*                                CAN controller                            */
/* INPUTS                         OutputControl:   [0..FF]                  */
/* OUTPUTS                        OK_       acknownledged by firmware       */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_set_output_control(INT32 OutputControl, volatile PCDEVICES pDevice)
{
  DFCT_STRUCT cmd_in;
  INT32 status;
  
  _TRACE5("CANDR_set_output_control(%09u)\n", pDevice->Serialnumber);
  
  cmd_in.wRequest = FCT_SET_OUTPUT_CONTROL;
  
  if (OutputControl == -1)
  {
    cmd_in.sParam_1 = 0xFA;
  }
  else
  {
    cmd_in.sParam_1 = OutputControl;
  }
  
  _TRACE5("CANDR_set_output_control(0x%02X)\n", cmd_in.sParam_1);
  cmd_in.wFRWReqRes = FALSE_;
  
  status = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 100);
  
  if (status != OK_)
    return (status);
  
  return (cmd_in.sRetCode);
}



/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_start_chip,CANDR_start_chip2        */
/* FUNCTIONAL_DESCRIPTION         changing the firmware mode from initializ.*/
/*                                to online                                 */
/* INPUTS                         -                                         */
/* OUTPUTS                        OK_       acknownledged by firmware       */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_start_chip(volatile PCDEVICES pDevice)
{
  DFCT_STRUCT cmd_in;
  INT32 status;
  INT32 ret,i;
  
  _TRACE5("CANDR_start_chip(%09u)\n", pDevice->Serialnumber);
  
  StopIoChain(pDevice);
  
  pDevice->bus_state = 0;
  
  if(pDevice->object_buffer_enable)
  {
    if(!pDevice->canprousb.pObjectBuf)
    {
      return CANL2_BOARD_NOT_INITIALIZED;
    }
    
    pDevice->canprousb.enableXmtAck = TRUE;
        
    if(pDevice->dyn_enable)
    {
      // dyn. object buffer
      for(i=0; i<MAX_OBJ_DYN; i++)
      {
        spin_lock_init(&pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.object[i].access);
        spin_lock_init(&pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[i].access);
      }
      
    }
    else
    {
      // static object buffer
      for(i=0; i<ANZ_IDENT; i++)
      {
        spin_lock_init(&pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[i].access);
        spin_lock_init(&pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[i].access);
      }
    }
  }
  
  cmd_in.wRequest = FCT_ENABLE_FIFO_TRANSMIT_ACK;
  cmd_in.wFRWReqRes = FALSE_;
  
  status = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 100);
  
  if (status != OK_)
    return (status);
  
  
  CANDR_reset_rcv_fifo(pDevice);
  CANDR_reset_xmt_fifo(pDevice);

  cmd_in.wRequest = FCT_START_CHIP;
  cmd_in.wFRWReqRes = FALSE_;
  
  status = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 100);
  
  if (status != OK_)
  {
    return (status);
  }
  
  ret = cmd_in.sRetCode;
  
  if(ret) 
  {
    return ret;
  }
  
  StartIoChain(pDevice);
  
  cmd_in.wRequest = FCT_ENABLE_DELAYED_POLL_MODE;
  cmd_in.sParam_1 = (INT16)5;
  cmd_in.wFRWReqRes = FALSE_;
  
  status = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 100);
  
  if (status != OK_)
  {
    return (status);
  }
  
  ret = cmd_in.sRetCode;
  
  return ret;
}



/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_enable_fifo_transmit_ack,           */
/*                                CANDR_enable_fifo_transmit_ack2           */
/* FUNCTIONAL_DESCRIPTION         enables transmit interrupts and messages  */
/*                                to host                                   */
/* INPUTS                         -                                         */
/* OUTPUTS                        OK_                                       */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_enable_fifo_transmit_ack(volatile PCDEVICES pDevice)
{
  
  pDevice->canprousb.enableXmtAck = TRUE;
  
  return 0;
}


#ifndef EMU
/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_enable_dyn_obj_buf                  */
/* FUNCTIONAL_DESCRIPTION         enables dynamic object buffer             */
/* INPUTS                         -                                         */
/* OUTPUTS                        OK_                                       */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_enable_dyn_obj_buf(volatile PCDEVICES pDevice)
{
  DFCT_STRUCT cmd_in;
  INT32 status;
  INT32 i;
  
  _TRACE5("CANDR_enable_dyn_obj_buf(%09u)\n", pDevice->Serialnumber);
  cmd_in.wRequest = FCT_ENABLE_DYN_OBJ_BUF;
  cmd_in.wFRWReqRes = FALSE_;
  
  status = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 100);
  
  if (status != OK_)
    return (status);
  
  pDevice->dyn_enable = TRUE;
  pDevice->object_buffer_enable = TRUE;
  
  pDevice->canprousb.pObjectBuf = kmalloc (sizeof(T_CANUSBP_OBJECT_BUF_MODE), GFP_ATOMIC);
  if(!pDevice->canprousb.pObjectBuf)
  {
    return VCARD_NOTENOUGHMEMORY;
  }
  
  memset(pDevice->canprousb.pObjectBuf, 0, sizeof(T_CANUSBP_OBJECT_BUF_MODE));
  
  for (i=0;i<MAX_IDENT_STD;i++)
  {
    pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.rcv_obj_lookup[i] = STD_OBJ_NOT_VALID;
    pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.xmt_obj_lookup[i] = STD_OBJ_NOT_VALID;
  }
  
  
  for (i=0;i<(MAX_OBJ_DYN+1);i++)
  {
    
    pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.xmt_id_lookup[i] = XTD_OBJ_NOT_VALID;
    pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.rcv_id_lookup[i].id = XTD_OBJ_NOT_VALID;
    
  }
  
  // initialize read and write pointers 
  pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_write_ptr        = 0;
  pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_read_ptr         = 255;
  pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_lost_msg         = 0;
  
  pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.req_fifo_write_ptr    = 0;
  pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.req_fifo_read_ptr     = 255;
  pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.req_fifo_lost_msg     = 0;
  
  pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.rmt_fifo_write_ptr    = 0;
  pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.rmt_fifo_read_ptr     = 255;
  pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.rmt_fifo_lost_remotes = 0;
  
  pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_write_ptr    = 0;
  pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_read_ptr     = 255;
  pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_lost_acks    = 0;
  
  
  
  return (cmd_in.sRetCode);
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_enable_fifo                         */
/* FUNCTIONAL_DESCRIPTION         enables fifo mode                         */
/* INPUTS                         -                                         */
/* OUTPUTS                        OK_                                       */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_enable_fifo(volatile PCDEVICES pDevice)
{
  DFCT_STRUCT cmd_in;
  INT32 status;
  T_CANUSBP_SHARED_MEM *pShM = pDevice->canprousb.pSharedMem;
  
  _TRACE5("CANDR_enable_fifo(%09u)\n", pDevice->Serialnumber);
  
  cmd_in.wRequest = FCT_ENABLE_FIFO;
  cmd_in.wFRWReqRes = FALSE_;
  status = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 100);
  
  if (status != OK_)
    return (status);
  
  pShM->can_pro_usb_fifo_mode.rcv.rd_rcv_fifo = RCV_FIFO_USB_MASK;
  pShM->can_pro_usb_fifo_mode.rcv.wr_rcv_fifo = 0;
  pShM->lostMsgOnHostFifo = 0;
  
  pDevice->canprousb.rcv_usb_enable = TRUE;
  
  pDevice->dyn_enable = FALSE;
  pDevice->object_buffer_enable = 0;
  
  return (cmd_in.sRetCode);
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_enable_error_frame_detection        */
/* FUNCTIONAL_DESCRIPTION         enables signalling of single error frames */
/* INPUTS                         -                                         */
/* OUTPUTS                        OK_                                       */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_enable_error_frame_detection(volatile PCDEVICES pDevice)
{
  DFCT_STRUCT cmd_in;
  INT32 status;
  
  _TRACE5("CANDR_enable_error_frame_detection(%09u)\n", pDevice->Serialnumber);
  cmd_in.wRequest = FCT_ENABLE_ERROR_FRAME;
  cmd_in.wFRWReqRes = FALSE_;
  
  status = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 100);
  
  if (status != OK_)
    return (status);
  
  return (cmd_in.sRetCode);
}

//HEAD_BEG****************************************************************** ///
//                                                                           //
// FUNCTION                       initialize_interface1                       //
// FUNCTIONAL_DESCRIPTION         defines modes for object buffer            //
// INPUTS                         RcvFifoEnable:  signal fifo or poll        //
//                                   rcv objects                             //
//                                RcvPollAll:  poll all std objects          //
//                                   or only defined ones in static obj buf  //
//                                RcvEnableAll:  enable all std rcv objects  //
//                                                   in static object buffer //
//                                RcvIntEnableAll:  signal all std rcv       //
//                                   object messages in static object buffer //
//                                AutoRemoteEnable: automatic retransmission //
//                                   of a data frame after remote frame      //
//                                   reception                               //
//                                TransmitReqFifoEnable: signal fifo or poll //
//                                   xmt objects                             //
//                                TxPollAll: poll all xmt objects            //
//                                   or only defined ones in static obj buf  //
//                                TxAckEnableAll:  signal transmitted        //
//                                   objects in static obj buf               //
//                                TxAckFifoEnableAll:  signal fifo           //
//                                   or poll acks in static object buffer    //
//                                TxRmtFifoEnableAll:  fifo for remote       //
//                                   answeres or poll                        //
// OUTPUTS                        OK                                         //
//                                ERROR4    timeout of firmware              //
//                                                                           //
//HEAD_END****************************************************************** ///
INT32 canpro_initialize_interface(
  PCDEVICES pDevice,
  INT32 RcvFifoEnable,
  INT32 RcvPollAll,
  INT32 RcvEnableAll,
  INT32 RcvIntEnableAll,
  INT32 AutoRemoteEnableAll,
  INT32 TxReqFifoEnable,
  INT32 TxPollAll,
  INT32 TxAckEnableAll,
  INT32 TxAckFifoEnableAll,
  INT32 TxRmtFifoEnableAll
)
{
  
  INT32  Ident;
  
  if (RcvEnableAll)
  {
    pDevice->canprousb.ReceiveEnableAll= TRUE;
    if(pDevice->dyn_enable)
    {
      
      if (RcvIntEnableAll)
      {
        pDevice->canprousb.ReceiveIntEnableAll= TRUE;
        for (Ident=0; Ident < ANZ_IDENT; Ident++)
        {
          pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.object[Ident].ctrl1 |=  HOST_INTERRUPT_ENABLE;
        }
      }
      else
      {
        pDevice->canprousb.ReceiveIntEnableAll= FALSE;
        for (Ident=0; Ident < ANZ_IDENT; Ident++)
        {
          pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.object[Ident].ctrl1 &= (USIGN8)~HOST_INTERRUPT_ENABLE;
        }
      }
    }
    else // dyn_enable
    {
      for (Ident=0; Ident < ANZ_IDENT; Ident++)
      {
        pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[Ident].ctrl1 = RCV_ENABLE;
      }
      
      if (RcvIntEnableAll)
      {
        pDevice->canprousb.ReceiveIntEnableAll= TRUE;
        for (Ident=0; Ident < ANZ_IDENT; Ident++)
        {
          pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[Ident].ctrl1 |=  HOST_INTERRUPT_ENABLE;
        }
      }
      else
      {
        pDevice->canprousb.ReceiveIntEnableAll= FALSE;
        for (Ident=0; Ident < ANZ_IDENT; Ident++)
        {
          pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[Ident].ctrl1 &= (USIGN8)~HOST_INTERRUPT_ENABLE;
        }
      }
      
      // for each receive object the Auto Remote Enable flag is set //
      // only in Static Object Buffer Mode                          //
      if (AutoRemoteEnableAll)
      {
        pDevice->canprousb.auto_remote_enable= TRUE;
        for (Ident=0; Ident<ANZ_IDENT; Ident++)
        {
          pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[Ident].ctrl1 |=  AUTO_REMOTE_ENABLE;
        }
      }
      else
      {
        pDevice->canprousb.auto_remote_enable= FALSE;
        for (Ident=0; Ident<ANZ_IDENT; Ident++)
        {
          pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[Ident].ctrl1 &= (USIGN8)~AUTO_REMOTE_ENABLE;
        }
      }
    }
  }
  else
  {
    pDevice->canprousb.ReceiveEnableAll= FALSE;
    pDevice->canprousb.auto_remote_enable= FALSE;
    if (pDevice->dyn_enable)
    {
      for (Ident=0; Ident<MAX_OBJ_DYN; Ident++)
      {
        // HOST_ACCESS = FALSE //
        pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.object[Ident].ctrl0  = 0;
        // disable all rcv objects //
        pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.object[Ident].ctrl1 = 0;
        // reset lookup table cyclic objects //
        //xmt_cyclic1.object_nr_to_idx[Ident] = NOT_VALID_INT;
      }
    }
    else // dyn_enable
    {
      for (Ident=0; Ident<ANZ_IDENT; Ident++)
      {
        // HOST_ACCESS = FALSE //
        pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[Ident].ctrl0  = 0;
        // disable all rcv1 objects //
        pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[Ident].ctrl1 = 0;
      }
    }
  }
  
  // xmt objects in Dynamic Object Buffer Mode //
  if (pDevice->dyn_enable)
  { 
    for (Ident=0; Ident<MAX_OBJ_DYN; Ident++)
    {
      pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[Ident].ctrl[0]  = 0;
      pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[Ident].ctrl[1]  = 0;
      pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[Ident].ctrl[1]  = 0;
    }
    
    if (TxAckEnableAll)
    {
      for (Ident=0; Ident<MAX_OBJ_DYN; Ident++)
      {
        pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[Ident].ctrl[1] |=  XMT_ACK_ENABLE;
      }
      pDevice->canprousb.TransmitAckEnableAll= 1;
    }
    else
      // no TransmitAckEnableAll 
      {
        for (Ident=0; Ident<MAX_OBJ_DYN; Ident++)
        {
          pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[Ident].ctrl[1] &= (USIGN8)~XMT_ACK_ENABLE;
        }
        pDevice->canprousb.TransmitAckEnableAll= 0;
      }
      
      if (TxAckFifoEnableAll)
      {
        pDevice->canprousb.TransmitAckFifoEnableAll= TRUE;
        for (Ident=0; Ident<MAX_OBJ_DYN; Ident++)
        {
          pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[Ident].ctrl[1] |=  XMT_ACK_FIFO_ENABLE;
        }
      }
      else
      {
        pDevice->canprousb.TransmitAckFifoEnableAll= FALSE;
        for (Ident=0; Ident<MAX_OBJ_DYN; Ident++)
        {
          pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[Ident].ctrl[1] &= ~XMT_ACK_FIFO_ENABLE;
        }
      }
      
  }
  
  // xmt objects in Static Object Buffer Mode //
  else //dyn_enable//
  {
    for (Ident=0; Ident<ANZ_IDENT; Ident++)
    {
      pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[Ident].ctrl[0]  = 0;
      pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[Ident].ctrl[1]  = 0;
      pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[Ident].ctrl[1]  = 0;
    }
    
    if (TxAckEnableAll)
    {
      for (Ident=0; Ident<ANZ_IDENT; Ident++)
      {
        pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[Ident].ctrl[1] |=  XMT_ACK_ENABLE;
      }
      
      pDevice->canprousb.TransmitAckEnableAll= 1;
    }
    else
      // no TransmitAckEnableAll //
      {
        for (Ident=0; Ident<ANZ_IDENT; Ident++)
        {
          pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[Ident].ctrl[1] &= (USIGN8)~XMT_ACK_ENABLE;
        }
        pDevice->canprousb.TransmitAckEnableAll= 0;
      }
      
      if (TxAckFifoEnableAll)
      {
        pDevice->canprousb.TransmitAckFifoEnableAll= TRUE;
        for (Ident=0; Ident<ANZ_IDENT; Ident++)
        {
          pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[Ident].ctrl[1] |=  XMT_ACK_FIFO_ENABLE;
        }
      }
      else
      {
        pDevice->canprousb.TransmitAckFifoEnableAll= FALSE;
        for (Ident=0; Ident<ANZ_IDENT; Ident++)
        {
          pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[Ident].ctrl[1] &= (USIGN8)~XMT_ACK_FIFO_ENABLE;
        }
      }
      
      
      // initialize read and write pointers //
      pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.fifo_write_ptr        = 0;
      pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.fifo_read_ptr         = 255;
      pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.fifo_lost_msg         = 0;
      
      pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.req_fifo_write_ptr    = 0;
      pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.req_fifo_read_ptr     = 255;
      pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.req_fifo_lost_msg     = 0;
      
      pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.rmt_fifo_write_ptr    = 0;
      pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.rmt_fifo_read_ptr     = 255;
      pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.rmt_fifo_lost_remotes = 0;
      
      pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.ack_fifo_write_ptr    = 0;
      pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.ack_fifo_read_ptr     = 255;
      pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.ack_fifo_lost_acks    = 0;
      
      // firmware check object to send! xmt_poll_list1[0] = END_MARK_WORD;
  }
  
  //global flags//
  if (RcvFifoEnable)
  {
    pDevice->canprousb.ReceiveFifoEnable= TRUE;
  }
  else
  {
    pDevice->canprousb.ReceiveFifoEnable= FALSE;
  }
  
  if (RcvPollAll)
  {
    pDevice->canprousb.ReceivePollAll= TRUE;
  }
  else
  {
    pDevice->canprousb.ReceivePollAll= FALSE;
    // initialize poll_list //
    if(pDevice->dyn_enable)
    {
      //rcv_dpram1.dyn_obj_buf.poll_list[0] = END_MARK_WORD;
    }
    else
    {
      pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.poll_list[0] = END_MARK_WORD;
    }
  }
  
  if (TxReqFifoEnable)
  {
    pDevice->canprousb.TransmitReqFifoEnable= TRUE;
  }
  else
  {
    pDevice->canprousb.TransmitReqFifoEnable= FALSE;
  }
  
  if (TxRmtFifoEnableAll)
  {
    pDevice->canprousb.TransmitRemoteFifoEnable= TRUE;
  }
  else
  {
    pDevice->canprousb.TransmitRemoteFifoEnable= FALSE;
  }
  
  /*
   *  Firmware; not used for host side!
   *  
   *  if (TxPollAll)
   *  {
   *    pDevice->canprousb.TransmitPollAll= TRUE;
   }
   else
   {
     pDevice->canprousb.TransmitPollAll= FALSE;
   }
   */
  pDevice->canprousb.number_of_xtd_rcv_objects= 0;
  pDevice->canprousb.number_of_std_rcv_objects= 0;
  pDevice->canprousb.number_of_xtd_xmt_objects= 0;
  pDevice->canprousb.number_of_std_xmt_objects= 0;
  pDevice->number_of_xmt_objects= 0;
  pDevice->canprousb.rcv_poll_count= 0;
  pDevice->canprousb.xmt_poll_count= 0;
  pDevice->bus_state     = 0;
  pDevice->canprousb.lost_remotes  = 0;
  
  return(OK);
  
}



/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_initialize_interface                */
/* FUNCTIONAL_DESCRIPTION         defines modes for object buffer           */
/* INPUTS                         ReceiveFifoEnable:  signal fifo or poll   */
/*                                   rcv objects                            */
/*                                ReceivePollAll:  poll all std objects     */
/*                                   or only defined ones                   */
/*                                ReceiveEnableAll:  enable all std rcv objs*/
/*                                ReceiveIntEnableAll:  signal all std rcv  */
/*                                   object messages                        */
/*                                AutoRemoteEnable: automatic retransmission*/
/*                                   of a data frame after remote frame     */
/*                                   reception                              */
/*                                TransmitReqFifoEnable: signal fifo or poll*/
/*                                   xmt objects                            */
/*                                TransmitPollAll: poll all xmt objects     */
/*                                   or only defined ones                   */
/*                                TransmitAckEnableAll:  signal transmitted */
/*                                   objects                                */
/*                                TransmitAckFifoEnableAll:  signal fifo    */
/*                                   or poll acks                           */
/*                                TransmitRmtFifoEnableAll:  fifo for remote*/
/*                                   answeres or poll                       */
/* OUTPUTS                        OK_                                       */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_initialize_interface(INT32 ReceiveFifoEnable,
                           INT32 ReceivePollAll,
                           INT32 ReceiveEnableAll,
                           INT32 ReceiveIntEnableAll,
                           INT32 AutoRemoteEnableAll,
                           INT32 TransmitReqFifoEnable,
                           INT32 TransmitPollAll,
                           INT32 TransmitAckEnableAll,
                           INT32 TransmitAckFifoEnableAll,
                           INT32 TransmitRmtFifoEnableAll, 
                           volatile PCDEVICES pDevice)
{
  DFCT_STRUCT cmd_in;
  INT32 status;
  
  _TRACE5("CANDR_initialize_interface(%09u)\n", pDevice->Serialnumber);
  
  if (!pDevice->object_buffer_enable)
    return (CANL2_WRONG_PARAM);
  
  if (pDevice->dyn_enable && ReceiveEnableAll)
    return (CANL2_WRONG_PARAM);
    
  cmd_in.wRequest = FCT_INITIALIZE_INTERFACE;
  cmd_in.sParam_1 = ReceiveFifoEnable;
  cmd_in.sParam_2 = ReceivePollAll;
  cmd_in.sParam_3 = ReceiveEnableAll;
  cmd_in.sParam_4 = ReceiveIntEnableAll;
  cmd_in.sParam_5 = AutoRemoteEnableAll;
  cmd_in.sParam_6 = TransmitReqFifoEnable;
  cmd_in.sParam_7 = TransmitPollAll;
  cmd_in.sParam_8 = TransmitAckEnableAll;
  cmd_in.sParam_9 = TransmitAckFifoEnableAll;
  cmd_in.sParam_10 = TransmitRmtFifoEnableAll;
  
  _TRACE5("CANDR_initialize_interface(%x,%x,%x,%x,%x,%x,%x,%x,%x,%x)\n",
          cmd_in.sParam_1,
          cmd_in.sParam_2,
          cmd_in.sParam_3,
          cmd_in.sParam_4,
          cmd_in.sParam_5,
          cmd_in.sParam_6,
          cmd_in.sParam_7,        
          cmd_in.sParam_8,
          cmd_in.sParam_9,
          cmd_in.sParam_10
          
  );
  
  cmd_in.wFRWReqRes = FALSE_;
  
  status = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 100);
  
  if (status != OK_)
    return (status);
  
  
  if(!pDevice->dyn_enable && pDevice->object_buffer_enable)
  {
    if(pDevice->canprousb.pObjectBuf)
    {
      kfree(pDevice->canprousb.pObjectBuf);
      pDevice->canprousb.pObjectBuf = NULL;
    }
    
    pDevice->canprousb.pObjectBuf = kmalloc (sizeof(T_CANUSBP_OBJECT_BUF_MODE), GFP_ATOMIC);
    if(!pDevice->canprousb.pObjectBuf)
    {
      return VCARD_NOTENOUGHMEMORY;
    }
    memset(pDevice->canprousb.pObjectBuf, 0, sizeof(T_CANUSBP_OBJECT_BUF_MODE));
  }
  
  
  status = canpro_initialize_interface(
    pDevice,
    ReceiveFifoEnable,
    ReceivePollAll,
    ReceiveEnableAll,
    ReceiveIntEnableAll,
    AutoRemoteEnableAll,
    TransmitReqFifoEnable,
    TransmitPollAll,
    TransmitAckEnableAll,
    TransmitAckFifoEnableAll,
    TransmitRmtFifoEnableAll);
  
  
  
  return (status);
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_define_object, CANDR_define_object2 */
/* FUNCTIONAL_DESCRIPTION         definition of an objekts and modes        */
/* INPUTS                         Ident:    Identifier                      */
/*                                Type:     Transferdirection    0: read    */
/*                                                               1: write   */
/*                                ReceiveIntEnable: generation  of a signal */
/*                                   to the PC after a frame reception      */
/*                                AutoRemoteEnable: automatic retransmission*/
/*                                   of a data frame after remote frame     */
/*                                   reception                              */
/*                                TransmitAckEnable: generation of a signal */
/*                                   to the PC after transmission of a frame*/
/* OUTPUTS                        OK_                                       */
/*                                ERROR1   parameter  error                 */
/*                                ERROR2   dynamic object buffer not enabled*/
/*                                CANL2_FW_TIMEOUT   too many objects already defined */
/*                                                                          */
/*HEAD_END*******************************************************************/
INT32 define_object(PCDEVICES pDevice, 
                    USIGN32 Id, 
                    INT16 Type, 
                    INT16 ReceiveIntEnable, 
                    INT16 AutoRemoteEnable, 
                    INT16 TransmitAckEnable, 
                    INT16 ObjectNumber)
  
{
  INT16 i;
  INT16 j;
  
  
   _TRACE5("CANDR_define_object(%09u)\n", pDevice->Serialnumber);
   
  // function part for Dynamic Object Buffer Mode //
  // -------------------------------------------- //
   if (pDevice->dyn_enable)
   {
     switch (Type)
     {
       case 0: //std rcv object
         pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.rcv_obj_lookup[Id] = pDevice->canprousb.number_of_xtd_rcv_objects +  pDevice->canprousb.number_of_std_rcv_objects++;
         pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.object[ObjectNumber].ctrl1 &= (USIGN8)~XTD_RCV;
         pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.object[ObjectNumber].ctrl1 |= RCV_ENABLE;
         break;
       case 1: //std xmt object
         pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.xmt_obj_lookup[Id] = pDevice->number_of_xmt_objects;
         pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.xmt_id_lookup[pDevice->number_of_xmt_objects] = Id;
         pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[ObjectNumber].ctrl[0] &= (USIGN8)~XTD_XMT;
         break;
         
         // xtd rcv object 
       case 2:
         
         for (i=pDevice->canprousb.number_of_xtd_rcv_objects; (i>0) &&
           (Id<pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.rcv_id_lookup[i-1].id); i--)
         {
           pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.rcv_id_lookup[i].obj = pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.rcv_id_lookup[i-1].obj;
           pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.rcv_id_lookup[i].rmt_obj = pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.rcv_id_lookup[i-1].rmt_obj;
           pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.rcv_id_lookup[i].id = pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.rcv_id_lookup[i-1].id;
         }
         pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.rcv_id_lookup[i].id = Id;
         pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.rcv_id_lookup[i].obj = pDevice->canprousb.number_of_std_rcv_objects +
         pDevice->canprousb.number_of_xtd_rcv_objects++;
         pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.rcv_id_lookup[i].rmt_obj = (USIGN8)XTD_OBJ_NOT_VALID;
         pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.object[ObjectNumber].ctrl1 |= XTD_RCV;
         for (j=0;j<pDevice->number_of_xmt_objects;j++)
           if (Id==pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.xmt_id_lookup[j] && (pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[j].ctrl[0] & XTD_XMT))
           {
             pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.rcv_id_lookup[i].rmt_obj = j;
             break;
           }
           
           break;
         
       case 3: //xtd xmt object
         pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.xmt_id_lookup[pDevice->number_of_xmt_objects] = Id;
         pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[ObjectNumber].ctrl[0] |= XTD_XMT;
         //////////////////////////////////////////////////
         /*
          *         for (i=pDevice->canprousb.number_of_xtd_xmt_objects1; (i>0) &&
          *           (Id<xmt_id_lookup1[i-1].id); i--)
          *         {
          *           xmt_id_lookup1[i].obj = xmt_id_lookup1[i-1].obj;
          *           xmt_id_lookup1[i].rmt_obj = xmt_id_lookup1[i-1].rmt_obj;
          *           xmt_id_lookup1[i].id = xmt_id_lookup1[i-1].id;
          }
          xmt_id_lookup1[i].id = Id;
          xmt_id_lookup1[i].obj = ObjectNumber; //number_of_std_xmt_objects1 + number_of_xtd_xmt_objects1;
          */
         pDevice->canprousb.number_of_xtd_xmt_objects++;
         //////////////////////////////////////////////////
         for (j=0;j<pDevice->canprousb.number_of_xtd_rcv_objects;j++)
           if (Id==pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.rcv_id_lookup[j].id)
           {
             pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.rcv_id_lookup[j].rmt_obj = ObjectNumber;
             break;
           }
           break;
       default: return -6;
       // not reached  break;
     }
     if (Type == 0 || Type == 2) //rcv object
     {
       pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.object[ObjectNumber].ctrl1 |= RCV_ENABLE;
       
       if (ReceiveIntEnable)
         pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.object[ObjectNumber].ctrl1 |=  HOST_INTERRUPT_ENABLE;
       else
         pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.object[ObjectNumber].ctrl1 &= (USIGN8)~HOST_INTERRUPT_ENABLE;
       
       if (AutoRemoteEnable)
       {
         pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.object[ObjectNumber].ctrl1 |=  AUTO_REMOTE_ENABLE;
       }
       else
         pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.object[ObjectNumber].ctrl1 &= (USIGN8)~AUTO_REMOTE_ENABLE;
     }
     else 
       if (Type == 1 || Type == 3) //xmt object
       {
         if (TransmitAckEnable)
         {
           pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[ObjectNumber].ctrl[1] |=  XMT_ACK_ENABLE;
         }
         else
           pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[ObjectNumber].ctrl[1] &= (USIGN8)~XMT_ACK_ENABLE;
       }
       
       
   }
   // function part for Static Object Buffer Mode //
   // ------------------------------------------- //
   else // dyn_enable//
   {
     if (Id > MAX_IDENT_XTD)
       return(CANL2_WRONG_PARAM);
     
     if (Type == 0)
     {
       pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[Id].ctrl1 |= RCV_ENABLE;
       // pObjectData->can_pro_usb_stat_obj_buf.rcv_static.object[Id].ctrl0 |= HOST_ACCESS;
       
       if (ReceiveIntEnable)
       {
         pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[Id].ctrl1 |=  HOST_INTERRUPT_ENABLE;
       }
       else
       {
         pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[Id].ctrl1 &= (USIGN8)~HOST_INTERRUPT_ENABLE;
         pDevice->canprousb.ReceiveIntEnableAll = FALSE;
       }
       
       if (AutoRemoteEnable)
       {
         pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[Id].ctrl1 |=  AUTO_REMOTE_ENABLE;
       }
       else
         pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[Id].ctrl1 &= (USIGN8)~AUTO_REMOTE_ENABLE;
       
       pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.poll_list[pDevice->canprousb.rcv_poll_count] = (USIGN16)Id;
       pDevice->canprousb.rcv_poll_count++;
       pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.poll_list[pDevice->canprousb.rcv_poll_count] = END_MARK_WORD;
     }
     else // (Type == 1)//
     {
       if (TransmitAckEnable)
       {
         pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[Id].ctrl[1] |=  XMT_ACK_ENABLE;
       }
       else
       {
         pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[Id].ctrl[1] &= (USIGN8)~XMT_ACK_ENABLE;
       }
       
       pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.xmt_poll_list[pDevice->canprousb.xmt_poll_count] = (USIGN16)Id;
       pDevice->canprousb.xmt_poll_count++;
       pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.xmt_poll_list[pDevice->canprousb.xmt_poll_count] = END_MARK_WORD;
       
     }
   }
   
   
   return(OK);
   
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_define_object, CANDR_define_object2 */
/* FUNCTIONAL_DESCRIPTION         definition of an objekts and modes        */
/* INPUTS                         Ident:    Identifier                      */
/*                                Type:     Transferdirection    0: read    */
/*                                                               1: write   */
/*                                ReceiveIntEnable: generation  of a signal */
/*                                   to the PC after a frame reception      */
/*                                AutoRemoteEnable: automatic retransmission*/
/*                                   of a data frame after remote frame     */
/*                                   reception                              */
/*                                TransmitAckEnable: generation of a signal */
/*                                   to the PC after transmission of a frame*/
/* OUTPUTS                        OK_                                       */
/*                                ERROR1   parameter  error                 */
/*                                ERROR2   dynamic object buffer not enabled*/
/*                                CANL2_FW_TIMEOUT   too many objects already defined */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_define_object(USIGN32 Ident,
                         PINT32 pObjectNumber,
                         INT32 Type,
                         INT32 ReceiveIntEnable,
                         INT32 AutoRemoteEnable,
                         INT32 TransmitAckEnable, 
                         volatile PCDEVICES pDevice)
{
  DFCT_STRUCT cmd_in;
  INT32 ret;
  
  _TRACE5("canusb_define_object: \n");
  
  if(!pDevice->canprousb.pObjectBuf)
  {
    return CANL2_BOARD_NOT_INITIALIZED;
  }
  
  
  cmd_in.wRequest = FCT_DEFINE_OBJECT;
  cmd_in.wFRWReqRes = FALSE;
  
  cmd_in.sParam_1 = (USIGN16)(Ident >>  0);
  cmd_in.sParam_2 = (USIGN16)(Ident >> 16);
  cmd_in.sParam_3 = (USIGN16)Type;
  cmd_in.sParam_4 = (USIGN16)ReceiveIntEnable;
  cmd_in.sParam_5 = (USIGN16)AutoRemoteEnable;
  cmd_in.sParam_6 = (USIGN16)TransmitAckEnable;
  
  
  if (pDevice->dyn_enable)
  {
    if (Type == 0 || Type == 2) /*rcv object*/
    {
      if (pDevice->number_of_rcv_objects >= MAX_OBJ_DYN)
      {
        return(CANL2_WRONG_PARAM);    //all possible Identifiers are defined
      }
      *pObjectNumber = pDevice->number_of_rcv_objects; //pDevice->number_of_rcv_objects++;
      ret = define_object(pDevice, Ident, (INT16)Type, (INT16)ReceiveIntEnable, (INT16)AutoRemoteEnable, (INT16)TransmitAckEnable, (INT16)*pObjectNumber);
      if(ret) return ret;
      
      cmd_in.sParam_7 = (USIGN16)*pObjectNumber;
      ret = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 100);
      if(ret) return ret;
      pDevice->number_of_rcv_objects++;
    }
    else
    {
      if (Type == 1 || Type == 3) /*xmt object*/
      {
        if (pDevice->number_of_xmt_objects >= MAX_OBJ_DYN)
        {
          return(CANL2_WRONG_PARAM);    /*all possible Identifiers are defined*/
        }
        *pObjectNumber = pDevice->number_of_xmt_objects; //pDevice->number_of_rcv_objects++;
        
        ret = define_object(pDevice, Ident, (INT16)Type, (INT16)ReceiveIntEnable, (INT16)AutoRemoteEnable, (INT16)TransmitAckEnable, (INT16)*pObjectNumber);
        if(ret) return ret;
        
        cmd_in.sParam_7 = (USIGN16)*pObjectNumber;
        ret = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 100);
        if(ret) return ret;
        pDevice->number_of_xmt_objects++;
      }
      else /* Type != 0 or 1 (or 2 or 3 for xtd)*/
      {
        return(CANL2_WRONG_PARAM);
      }
    }
  }
  else
  {
    if (Type != 0 && Type != 1) /*no std rcv or xmt object*/
    {
      return(CANL2_WRONG_PARAM);
    }
    *pObjectNumber = (INT32)Ident;
    cmd_in.sParam_7 = (USIGN16)*pObjectNumber;
    ret = define_object(pDevice, Ident, (INT16)Type, (INT16)ReceiveIntEnable, (INT16)AutoRemoteEnable, (INT16)TransmitAckEnable, (INT16)*pObjectNumber);
    if(ret) return ret;
    ret = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 100);
  }
  
  return ret;
}


/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_define_cyclic, CANDR_define_cyclic2 */
/* FUNCTIONAL_DESCRIPTION         defines cyclic transmission of objects    */
/* INPUTS                         ObjectNumber, Rate in ms, limit of cycles */
/* OUTPUTS                        OK_                                       */
/*                                ERROR1   object number not valid          */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_define_cyclic(INT32 ObjectNumber, USIGN32 Rate, USIGN32 Cycles,
                    volatile PCDEVICES pDevice)
{
  DFCT_STRUCT cmd_in;
  INT32 status;
  
    _TRACE5("CANDR_define_cyclic(%09u)\n", pDevice->Serialnumber);
    cmd_in.wRequest = FCT_DEFINE_CYCLIC;
    cmd_in.sParam_1 = ObjectNumber;
    cmd_in.sParam_2 = Rate;
    cmd_in.sParam_3 = Cycles;
    cmd_in.wFRWReqRes = FALSE_;
    
    status = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 100);
    
    if (status != OK_)
      return (status);
    
    return (cmd_in.sRetCode);
}


/*HEAD_BEG********************************************************************/
/*                                                                           */
/* FUNCTION                       send_remote_object,send remote_object2     */
/* FUNCTIONAL_DESCRIPTION         transmits a remote frame                   */
/* INPUTS                         Can: CAN handle                            */
/*                                Ident:     object number                   */
/*                                DataLength: dlc in remote frame            */
/*                                                                           */
/* OUTPUTS                        OK                                         */
/*                                ERROR1: last request still pending         */
/*                                ERROR3: fifo full                          */
/*                                                                           */
/*HEAD_END********************************************************************/
INT32 canpro_send_remote_object_stat(PCDEVICES pDevice, INT32 Ident, INT32 DataLength)
{
  volatile XMT_OBJECT *pXmtObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[Ident];
  USIGN8 wrt_ptr;
  CAN_ADDRESS xmtIrql;
  
  /* part for static object buffer mode */
  /* ---------------------------------- */
  /* last request still pending */
  if (pXmtObject->ctrl[0] & XMT_REQ_REMOTE)
  {
    return(CANL2_REQ_PENDING);
  }
  
  xmtIrql = sync_get_access((spinlock_t*)&pXmtObject->access);
  
  /* part for static object buffer mode */
  /* ---------------------------------- */
  /* last request still pending */
  if (pXmtObject->ctrl[0] & XMT_REQ_REMOTE)
  {
    sync_free_access((spinlock_t*)&pXmtObject->access, xmtIrql);
    return(CANL2_REQ_PENDING);
  }
  
  _TRACE5("CANBUS canpro_send_remote_object_stat: 0x%x\n",Ident);
  
  if (pDevice->canprousb.TransmitReqFifoEnable)
  {
    volatile XMT_STATIC_STRUCT *pStaticStruct = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static;
    
    wrt_ptr = pStaticStruct->req_fifo_write_ptr;
    
    if (wrt_ptr == pStaticStruct->req_fifo_read_ptr)
    {
      /* fifo full */
      
      sync_free_access((spinlock_t*)&pXmtObject->access, xmtIrql);
      return (CANL2_XMT_REQ_OVR);
    }
    else
    {
      pXmtObject->len_remote = DataLength;
      pStaticStruct->req_fifo[wrt_ptr] = Ident | RTR_MASK;
      pXmtObject->ctrl[0] |=  (USIGN8)XMT_REQ_REMOTE;
      wrt_ptr++;
      pStaticStruct->req_fifo_write_ptr = wrt_ptr;
      
    }
  }
  else
  {
    //  pXmtObject->ctrl[1] &= ~XMT_STATUS;
    pXmtObject->ctrl[0] |=  (USIGN8)XMT_REQ_REMOTE; 
    pXmtObject->len_remote = DataLength;
  }
  
  
  sync_free_access((spinlock_t*)&pXmtObject->access, xmtIrql);
  
  return(CANL2_OK);
  
}

/*HEAD_BEG********************************************************************/
/*                                                                           */
/* FUNCTION                       send_remote_object_dyn                     */
/* FUNCTIONAL_DESCRIPTION         transmits a remote frame                   */
/* INPUTS                         Can: CAN handle                            */
/*                                Ident:     object number                   */
/*                                DataLength: dlc in remote frame            */
/*                                                                           */
/* OUTPUTS                        OK                                         */
/*                                ERROR1: last request still pending         */
/*                                ERROR3: fifo full                          */
/*                                                                           */
/*HEAD_END********************************************************************/

INT32 canpro_send_remote_object_dyn(PCDEVICES pDevice, INT32 Ident, INT32 DataLength)
{
  volatile XMT_OBJECT *pXmtObject;
  CAN_ADDRESS xmtIrql;
  
  if ((
    Ident > MAX_OBJ_DYN) || 
    (pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.xmt_id_lookup[Ident] == XTD_OBJ_NOT_VALID)
  )
    return(CANL2_WRONG_PARAM);
    
    pXmtObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[Ident];
    
    xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
    
    if (pXmtObject->ctrl[0] & XMT_REQ_REMOTE)
    {
      /* last request still pending */
      sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
      return(CANL2_REQ_PENDING);
    }
    
    if (pDevice->canprousb.TransmitReqFifoEnable)
    {
      
      volatile XMT_DYN_STRUCT *pDynXmtStruct = &pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn;
      
      //fifoirql = sync_get_access(&pDevice->ChLock);
      if (pDynXmtStruct->req_fifo_write_ptr == pDynXmtStruct->req_fifo_read_ptr)
      {
        /* request fifo full */
        sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
        return (CANL2_XMT_REQ_OVR);
      }
      else
      {
        
        pXmtObject->len_remote = DataLength;
        pXmtObject->ctrl[0] |= (USIGN8)XMT_REQ_REMOTE;
        
        
        pDynXmtStruct->req_fifo[pDynXmtStruct->req_fifo_write_ptr] = (USIGN8)Ident | RTR_MASK;
        pDynXmtStruct->req_fifo_write_ptr++;
      }
    }
    else
    {
      
      pXmtObject->len_remote = DataLength;
      pXmtObject->ctrl[0] |= (USIGN8)XMT_REQ_REMOTE;
      
    }
    
    sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
    
    return(CANL2_OK);
    
}



/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       send_remote_object,send remote_object2    */
/* FUNCTIONAL_DESCRIPTION         transmits a remote frame                  */
/* INPUTS                         Ident:     object number                  */
/*                                DataLength: dlc in remote frame           */
/* OUTPUTS                        OK                                        */
/*                                ERROR1: last request still pending        */
/*                                ERROR3: fifo full                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_send_remote_object(INT32 ObjectNumber, INT32 DataLength, volatile PCDEVICES pDevice)
{
//   DFCT_STRUCT cmd_in;
//   INT32 status;
  USBXMT_ITEM fifoItem;
  INT32 ret = VCARD_OK;
 
  _TRACE5("CANDR_send_remote_object(%09u)\n", pDevice->Serialnumber);
  
  if(!pDevice->canprousb.pObjectBuf)
  {
    return CANL2_BOARD_NOT_INITIALIZED;
  }
  
  if (pDevice->object_buffer_enable)
  {
    if (pDevice->dyn_enable)
    {
      ret = canpro_send_remote_object_dyn(pDevice, ObjectNumber, (INT8) DataLength);
      if(!ret)
      {
        fifoItem.can_cmd = XMT_REM_OBJ;
        fifoItem.can_id = ObjectNumber;
        fifoItem.can_len = (INT8)DataLength;
        fifoItem.u32Counter = MsgCounter++;
        ret = canusb_insert_fifo(pDevice, &fifoItem, pDevice->canprousb.pSharedMem);
      }
    }
    else
    {
      ret = canpro_send_remote_object_stat(pDevice, ObjectNumber, (INT8) DataLength);
      if(!ret)
      {
        fifoItem.can_cmd = XMT_REM_OBJ;
        fifoItem.can_id = ObjectNumber;
        fifoItem.can_len = (INT8)DataLength;
        fifoItem.u32Counter = MsgCounter++;
        ret = canusb_insert_fifo(pDevice, &fifoItem, pDevice->canprousb.pSharedMem);
      }
    }
  }
  else
  {
    ret = CANL2_OB_NOT_DEFINED;
  }
  
  return ret;
  
}

/*HEAD_BEG********************************************************************/
/*                                                                           */
/* FUNCTION                       canpro_supply_object_data_stat             */
/* FUNCTIONAL_DESCRIPTION         writes data into xmt object buffer without */
/*                                transmitting a frame                       */
/* INPUTS                         Can:       channnel handle                 */
/*                                Ident:     object number                   */
/*                                DataLength:number of xmt data bytes        */
/*                                pData:     pointer to data                 */
/*                                                                           */
/* OUTPUTS                        OK                                         */
/*                                ERROR1: last request still pending         */
/*                                                                           */
/*HEAD_END********************************************************************/

INT32 canpro_supply_object_data_stat(volatile PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PBYTE pData)
{
  volatile XMT_OBJECT *pXmtObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[Ident];
  CAN_ADDRESS xmtIrql;
  
  /* function part for Static Object Buffer Mode */
  /* ------------------------------------------- */
  
  
  xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
  
  _TRACE5("CANBUS canpro_supply_object_data_stat: 0x%x\n",Ident);
  
  /* function part for Static Object Buffer Mode */
  /* ------------------------------------------- */
  
  
  pXmtObject->len_data = DataLength;
  
  /* write data in object buffer */
  *((USIGN64 *)pXmtObject->data) = *((USIGN64 *)pData);
  
  pXmtObject->ctrl[1] |= XMT_DATA_VALID;
  
  //sync_free_access(&pObject->host_access);
  sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
  
  return(CANL2_OK);
}



/*HEAD_BEG********************************************************************/
/*                                                                           */
/* FUNCTION                       canpro_supply_object_data_dyn              */
/* FUNCTIONAL_DESCRIPTION         writes data into xmt object buffer without */
/*                                transmitting a frame                       */
/* INPUTS                         Can:       channnel handle                 */
/*                                Ident:     object number                   */
/*                                DataLength:number of xmt data bytes        */
/*                                pData:     pointer to data                 */
/*                                                                           */
/* OUTPUTS                        OK                                         */
/*                                ERROR1: last request still pending         */
/*                                                                           */
/*HEAD_END********************************************************************/

INT32 canpro_supply_object_data_dyn(volatile PCDEVICES pDevice, INT32 ObjectNumber, INT32 DataLength, PBYTE pData)

{
  
  //  INT32 return_value;
  volatile XMT_OBJECT *pXmtObject;
  CAN_ADDRESS xmtIrql;
  
  // function part for Dynamic Object Buffer Mode +/
  // -------------------------------------------- +/
  
  if(ObjectNumber > MAX_OBJ_DYN)
  {
    return(CANL2_WRONG_PARAM);
  }
  
  if(pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.xmt_id_lookup[ObjectNumber] == XTD_OBJ_NOT_VALID)
  {
    return(CANL2_WRONG_PARAM);
  }
  
  pXmtObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[ObjectNumber];
  
  xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
  
  pXmtObject->len_data = DataLength;
  
  // write data in object buffer +/
  *((USIGN64 *)pXmtObject->data) = *(USIGN64 *)pData;
  pXmtObject->ctrl[1] |= XMT_DATA_VALID;
  
  sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
  
  return(CANL2_OK);
  
}


/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       supply_object_data,supply_object_data2    */
/* FUNCTIONAL_DESCRIPTION         writes data into xmt object buffer without*/
/*                                transmitting a frame                      */
/* INPUTS                         ObjectNumber:     object number           */
/*                                DataLength:number of xmt data bytes       */
/*                                pData:     pointer to data                */
/* OUTPUTS                        OK                                        */
/*                                ERROR1: last request still pending        */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_supply_object_data(INT32 ObjectNumber, INT32 DataLength, PUSIGN8 pData,
                         volatile PCDEVICES pDevice)
{
  INT32 ret = VCARD_OK;
  USBXMT_ITEM fifoItem;
  
  _TRACE5("CANBUS canusb_supply_object: 0x%x\n",ObjectNumber);
  
  if(!pDevice->canprousb.pObjectBuf)
  {
    return CANL2_BOARD_NOT_INITIALIZED;
  }
  
  if (pDevice->object_buffer_enable)
  {
    if (pDevice->dyn_enable)
    {
      ret = canpro_supply_object_data_dyn(pDevice, ObjectNumber, (INT8) DataLength, pData);
      if(!ret)
      {
        fifoItem.can_cmd = SUP_OBJ;
        fifoItem.can_id = ObjectNumber;
        memcpy((PVOID)fifoItem.data,pData,8);
        fifoItem.can_len = (INT8)DataLength;
        fifoItem.u32Counter = MsgCounter++;
        ret = canusb_insert_fifo(pDevice, &fifoItem, pDevice->canprousb.pSharedMem);
      }
    }
    else
    {
      ret = canpro_supply_object_data_stat(pDevice, ObjectNumber, (INT8) DataLength, pData);
      if(!ret)
      {
        fifoItem.can_cmd = SUP_OBJ;
        fifoItem.can_id = ObjectNumber&0x7FF;
        memcpy((PVOID)fifoItem.data,pData,8);
        fifoItem.can_len = (INT8)DataLength;
        fifoItem.u32Counter = MsgCounter++;
        ret = canusb_insert_fifo(pDevice, &fifoItem, pDevice->canprousb.pSharedMem);
      }
    }
  }
  else
  {
    ret = CANL2_OB_NOT_DEFINED;
  }
  
  
  return ret;
}

/*HEAD_BEG********************************************************************/
/*                                                                           */
/* FUNCTION                    canpro_supply_rcv_object_data_stat            */
/* FUNCTIONAL_DESCRIPTION      initializes rcv object data                   */
/* INPUTS                      Can:       channnel handle                    */
/*                             Ident:     object number                      */
/*                             DataLength:data length                        */
/*                             pData:     pointer to array of data           */
/* OUTPUTS                     OK                                            */
/*                                                                           */
/*HEAD_END********************************************************************/

INT32 canpro_supply_rcv_object_data_stat(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PBYTE pData)
{
  /* function part for Static Object Buffer Mode */
  /* ------------------------------------------- */
  
  volatile RCV_OBJECT *pObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[Ident];
  CAN_ADDRESS rcvIrql;
  
  //sync_get_access(&pObject->host_access, &pObject->fw_access);
  rcvIrql = sync_get_access((spinlock_t *)&pObject->access);
  
  /* write datalength to object -Ident1- */             
  pObject->DataLength = (USIGN8)DataLength; 
  /* write data to object -Ident1- */      
  *(USIGN64 *)&pObject->data[0] = *(USIGN64  *)((USIGN8  *)&pData[0]);
  
  //sync_free_access(&pObject->host_access);
  sync_free_access((spinlock_t *)&pObject->access, rcvIrql);
  
  return(CANL2_OK);
}

/*HEAD_BEG********************************************************************/
/*                                                                           */
/* FUNCTION                    supply_rcv_object_data_dyn                    */
/* FUNCTIONAL_DESCRIPTION      initializes rcv object data                   */
/* INPUTS                      Can:       channnel handle                    */
/*                             Ident:     object number                      */
/*                             DataLength:data length                        */
/*                             pData:     pointer to array of data           */
/*                                                                           */
/* OUTPUTS                     OK                                            */
/*                                                                           */
/*HEAD_END********************************************************************/

INT32 canpro_supply_rcv_object_data_dyn(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PBYTE pData)
{
  volatile RCV_OBJECT *pXmtObject;
  CAN_ADDRESS xmtIrql;
  
  if (Ident > MAX_OBJ_DYN) 
    return(CANL2_WRONG_PARAM);
  
  pXmtObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.object[Ident];
  
  xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
  
  /* write datalength to object -Ident1- */             
  pXmtObject->DataLength = (USIGN8)DataLength; 
  *(USIGN64 *)&pXmtObject->data[0] = *(USIGN64 *)(pData+0);
  
  sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
  
  return(CANL2_OK);
  
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                    supply_rcv_object_data,supply_rcv_obect_data2*/
/* FUNCTIONAL_DESCRIPTION      initializes rcv object data                  */
/* INPUTS                      Ident:     object number                     */
/*                             DataLength:dummy                             */
/*                             pData:     pointer to array of data          */
/* OUTPUTS                     OK                                           */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_supply_rcv_object_data(INT32 ObjectNumber, INT32 DataLength, PUSIGN8 pData,
                             volatile PCDEVICES pDevice)
{
  INT32 ret = VCARD_OK;
  
  _TRACE5("canusb_supply_rcv_object: \n");
  
  if(!pDevice->canprousb.pObjectBuf)
  {
    return CANL2_BOARD_NOT_INITIALIZED;
  }
  
  if (pDevice->object_buffer_enable)
  {
    if (pDevice->dyn_enable)
    {
      ret = canpro_supply_rcv_object_data_dyn(pDevice, ObjectNumber, DataLength, pData);
    }
    else
    {
      ret = canpro_supply_rcv_object_data_stat(pDevice, ObjectNumber, DataLength, pData);
    }
  }
  else
  {
    ret = CANL2_OB_NOT_DEFINED;
  }
  
  return ret;

}

/*HEAD_BEG********************************************************************/
/*                                                                           */
/* FUNCTION                       canpro_send_object_stat                    */
/* FUNCTIONAL_DESCRIPTION         transmits data previously written in       */
/*                                object buffer by supply_object_data or     */
/*                                write_object                               */
/* INPUTS                         Can:       channnel handle                 */
/*                                Ident:     object number                   */
/*                                DataLength: number of transmit data        */
/*                                                                           */
/* OUTPUTS                        OK                                         */
/*                                ERROR1: request overrun                    */
/*                                ERROR3: fifo full                          */
/*                                                                           */
/*HEAD_END********************************************************************/

INT32 canpro_send_object_stat(PCDEVICES pDevice, INT32 Ident, INT32 DataLength)
{
  volatile XMT_OBJECT *pXmtObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[Ident];
  CAN_ADDRESS xmtIrql;
  
  
  if(!(pXmtObject->ctrl[1] & XMT_DATA_VALID))
  {
    return CANL2_NOT_ACTIVE;
  }
  
  if (pXmtObject->ctrl[0] & XMT_REQ_DATA)
  {
    /* request overrun */
    return(CANL2_REQ_PENDING);
  }
  
  xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
  
  if (pXmtObject->ctrl[0] & XMT_REQ_DATA)
  {
    /* request overrun */
    sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
    return(CANL2_REQ_PENDING);
  }
  
  _TRACE5("CANBUS canpro_send_object_stat: 0x%x\n",Ident);
  
  if (pDevice->canprousb.TransmitReqFifoEnable)
  {
    volatile XMT_STATIC_STRUCT *pStaticStruct = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static;
    
    //fifoirql = sync_get_access(&pDevice->ChLock);
    if (pStaticStruct->req_fifo_write_ptr == pStaticStruct->req_fifo_read_ptr)
    {
      /* fifo full */
      //sync_free_access(&pObject->host_access);
      sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
      return (CANL2_XMT_REQ_OVR);
    }
    else
    {
      pXmtObject->ctrl[0] |= (USIGN8)XMT_REQ_DATA;
      pXmtObject->len_data = DataLength;
      
      pStaticStruct->req_fifo[pStaticStruct->req_fifo_write_ptr] = (USIGN16)Ident;
      pStaticStruct->req_fifo_write_ptr++;
    }
  }
  else
  {
    pXmtObject->ctrl[0] |= (USIGN8)XMT_REQ_DATA;
    pXmtObject->len_data = DataLength;
  }
  
  //sync_free_access(&pObject->host_access);
  sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
  
  return 0;
  
}



/*HEAD_BEG********************************************************************/
/*                                                                           */
/* FUNCTION                       canpro_send_object_dyn                     */
/* FUNCTIONAL_DESCRIPTION         transmits data previously written in       */
/*                                object buffer by supply_object_data or     */
/*                                write_object                               */
/* INPUTS                         Can:       channnel handle                 */
/*                                Ident:     object number                   */
/*                                DataLength: number of transmit data        */
/*                                                                           */
/* OUTPUTS                        OK                                         */
/*                                ERROR1: request overrun                    */
/*                                ERROR3: fifo full                          */
/*                                                                           */
/*HEAD_END********************************************************************/
INT32 canpro_send_object_dyn(PCDEVICES pDevice, INT32 Ident, INT32 DataLength)
{
  volatile XMT_OBJECT *pXmtObject;
  CAN_ADDRESS xmtIrql;
  
  /* function part for Dynamic Object Buffer Mode */
  /* -------------------------------------------- */
  
  if ((
    Ident > MAX_OBJ_DYN) || 
    (pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.xmt_id_lookup[Ident] == XTD_OBJ_NOT_VALID)
  )
    return(CANL2_WRONG_PARAM);
    
    pXmtObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[Ident];
    
    if(!(pXmtObject->ctrl[1] & XMT_DATA_VALID))
    {
      return CANL2_NOT_ACTIVE;
    }
    
    if (pXmtObject->ctrl[0] & XMT_REQ_DATA)
      /* request overrun */
      return(CANL2_REQ_PENDING);
    
    if (pDevice->canprousb.TransmitReqFifoEnable)
    {
      volatile XMT_DYN_STRUCT *pDynStruct = &pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn;
      
      //fifoirql = sync_get_access(&pDevice->ChLock);
      
      if (pDynStruct->req_fifo_write_ptr == pDynStruct->req_fifo_read_ptr)
      {
        /* fifo full */
        return (CANL2_XMT_REQ_OVR);
      }
      else
      {
        xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
        
        
        pXmtObject->ctrl[0] |= (USIGN8)XMT_REQ_DATA;
        
        pXmtObject->len_data = DataLength;
        
        pDynStruct->req_fifo[pDynStruct->req_fifo_write_ptr] = (USIGN8)Ident;
        pDynStruct->req_fifo_write_ptr++;
        
        //sync_free_access(&pXmtObject->host_access);
        sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
      }
    }
    else
    {
      xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
      
      
      pXmtObject->ctrl[0] |= (USIGN8)XMT_REQ_DATA;
      
      pXmtObject->len_data = DataLength;
      
      //sync_free_access(&pXmtObject->host_access);
      sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
    }
    
    return(CANL2_OK);
    
}


/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       send_object,send_object2                  */
/* FUNCTIONAL_DESCRIPTION         transmits data previously written in      */
/*                                object buffer by supply_object_data or    */
/*                                write_object                              */
/* INPUTS                         Ident:      Identifier                    */
/*                                DataLength: number of transmit data       */
/* OUTPUTS                        OK                                        */
/*                                ERROR1: request overrun                   */
/*                                ERROR3: fifo full                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_send_object(INT32 ObjectNumber, INT32 DataLength, volatile PCDEVICES pDevice)
{  
  USBXMT_ITEM fifoItem;
  INT32 ret = VCARD_OK;
  
  _TRACE5("CANBUS canusb_send_object: 0x%x\n",ObjectNumber);
  if(!pDevice->canprousb.pObjectBuf)
  {
    return CANL2_BOARD_NOT_INITIALIZED;
  }
  
  if (pDevice->object_buffer_enable)
  {
    if (pDevice->dyn_enable)
    {
      ret = canpro_send_object_dyn(pDevice, ObjectNumber, (INT8) DataLength);
      if(!ret)
      {
        fifoItem.can_cmd = XMT_OBJ;
        fifoItem.can_id = ObjectNumber;
        fifoItem.can_len = (INT8)DataLength;
        fifoItem.u32Counter = MsgCounter++;
        ret = canusb_insert_fifo(pDevice, &fifoItem, pDevice->canprousb.pSharedMem);
      }
    }
    else
    {
      ret = canpro_send_object_stat(pDevice, ObjectNumber, (INT8) DataLength);
      if(!ret)
      {
        fifoItem.can_cmd = XMT_OBJ;
        fifoItem.can_id = ObjectNumber;
        fifoItem.can_len = (INT8)DataLength;
        fifoItem.u32Counter = MsgCounter++;
        ret = canusb_insert_fifo(pDevice, &fifoItem, pDevice->canprousb.pSharedMem);
      }
    }
  }
  else
  {
    ret = CANL2_OB_NOT_DEFINED;
  }
  
  return ret;
  
}

/*HEAD_BEG********************************************************************/
/*                                                                           */
/* FUNCTION                       write_object                               */
/* FUNCTIONAL_DESCRIPTION         writes data in object buffer and requests  */
/*                                data frame for transmission                */
/* INPUTS                         Can:       channnel handle                 */
/*                                Ident:     object number                   */
/*                                DataLength:number of data bytes            */
/*                                *pData:    pointer to data                 */
/* OUTPUTS                        OK                                         */
/*                                ERROR1: request overrun                    */
/*                                ERROR3: fifo full                          */
/*                                                                           */
/*HEAD_END********************************************************************/
INT32 canpro_write_object_stat(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PBYTE pData)
{
  
  INT32 return_value = 0;
  CAN_ADDRESS xmtIrql;
  volatile XMT_OBJECT *pXmtObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[Ident];
  
  _TRACE5("CANBUS canpro_write_object_stat: 0x%x\n",Ident);
  
  if (pXmtObject->ctrl[0] & XMT_REQ_DATA)
  {
    /* request overrun */
    _TRACE5("CANBUS canpro_write_object_stat: done! ret=%u\n",CANL2_REQ_PENDING);
    return(CANL2_REQ_PENDING);
  }
  
  xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
  
  /* function part for Static Object Buffer Mode */
  /* ------------------------------------------- */
  //  return_value = supply_no_access2(/*Ident2, DataLength, pData*/);
  if (pXmtObject->ctrl[0] & XMT_REQ_DATA)
  {
    /* request overrun */
    sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
    _TRACE5("CANBUS canpro_write_object_stat: done! ret=%u\n",CANL2_REQ_PENDING);
    return(CANL2_REQ_PENDING);
  }
  
  
  pXmtObject->len_data = DataLength;
  
  // write data in object buffer +/
  *(USIGN64 *)&pXmtObject->data[0] = *(USIGN64  *)(pData);
  pXmtObject->ctrl[1] |= XMT_DATA_VALID;
  
  /////////////////
  
  if (pDevice->canprousb.TransmitReqFifoEnable)
  {
    volatile XMT_STATIC_STRUCT *pStaticStruct = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static;
    //fifoirql = sync_get_access(&pDevice->ChLock);
    
    if (pStaticStruct->req_fifo_write_ptr == pStaticStruct->req_fifo_read_ptr)
    {
      /* fifo full */
      //sync_free_access(&pXmtObject->host_access);
      sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
      _TRACE5("CANBUS canpro_write_object_stat: done! ret=%u\n",CANL2_XMT_REQ_OVR);
      return (CANL2_XMT_REQ_OVR);
    }
    else
    {
      //      pXmtObject->ctrl[1] &= ~XMT_STATUS;
      pXmtObject->ctrl[0] |=  XMT_REQ_DATA;
      pStaticStruct->req_fifo[pStaticStruct->req_fifo_write_ptr] = (USIGN16)Ident;
      pStaticStruct->req_fifo_write_ptr++;
    }
  }
  else
  {
    //  pXmtObject->ctrl[1] &= ~XMT_STATUS;
    pXmtObject->ctrl[0] |=  XMT_REQ_DATA;
  }
  
  //sync_free_access(&pXmtObject->host_access);
  sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
  _TRACE5("CANBUS canpro_write_object_stat: done! ret=%u\n",return_value);
  
  return(return_value);
  
}

/*HEAD_BEG********************************************************************/
/*                                                                           */
/* FUNCTION                       write_object,                              */
/* FUNCTIONAL_DESCRIPTION         writes data in object buffer and requests  */
/*                                data frame for transmission                */
/* INPUTS                         Can:       channnel handle                 */
/*                                Ident:     object number                   */
/*                                DataLength:number of data bytes            */
/*                                *pData:    pointer to data                 */
/*                                                                           */
/* OUTPUTS                        OK                                         */
/*                                ERROR1: request overrun                    */
/*                                ERROR3: fifo full                          */
/*                                                                           */
/*HEAD_END********************************************************************/
INT32 canpro_write_object_dyn(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PBYTE pData)
{
  
  INT32 return_value  = CANL2_OK;
  volatile XMT_OBJECT *pXmtObject;
  CAN_ADDRESS xmtIrql;
  
  /* function part for Dynamic Object Buffer Mode */
  /* -------------------------------------------- */
  
  _TRACE5("CANBUS canpro_write_object_dyn: 0x%x\n",Ident);
  
  if (
    (Ident > MAX_OBJ_DYN) || 
    (pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.xmt_id_lookup[Ident] == XTD_OBJ_NOT_VALID)
  )
  {
    _TRACE5("CANBUS canpro_write_object_dyn: lookup=0x%x\n", pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.xmt_id_lookup[Ident]);
    return(CANL2_WRONG_PARAM);
  }
        
    pXmtObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[Ident];
    
    
    if (pXmtObject->ctrl[0] & XMT_REQ_DATA)
    {
      /* request overrun */
      _TRACE5("CANBUS canpro_write_object_dyn: request overrun\n");
      return CANL2_XMT_REQ_OVR;
    }
        
    xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
    
    if (pDevice->canprousb.TransmitReqFifoEnable)
    {
      
      volatile XMT_DYN_STRUCT *pDynStruct = &pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn;
      
      
      if (pDynStruct->req_fifo_write_ptr == pDynStruct->req_fifo_read_ptr)
      {
        // fifo full 
        _TRACE5("CANBUS canpro_write_object_dyn: fifo full\n");
        sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
        return (CANL2_XMT_REQ_OVR);
      }
      else
      {
        pXmtObject->len_data = DataLength;
        
        *(USIGN64 *)&pXmtObject->data[0] = *(USIGN64 *)pData;
        pXmtObject->ctrl[1] |= XMT_DATA_VALID;
        
        pXmtObject->ctrl[0] |=  XMT_REQ_DATA;
        pDynStruct->req_fifo[pDynStruct->req_fifo_write_ptr] = (USIGN8)Ident;
        pDynStruct->req_fifo_write_ptr++;
      }
    }
    else
    {
      pXmtObject->len_data = DataLength;
      
      *(USIGN64 *)&pXmtObject->data[0] = *(USIGN64 *)pData;
      
      pXmtObject->ctrl[1] |=  XMT_DATA_VALID;
      pXmtObject->ctrl[0] |=  XMT_REQ_DATA;
    }
    
    sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
    return(return_value);
    
}


/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       write_object, write_object2               */
/* FUNCTIONAL_DESCRIPTION         writes data in object buffer and requests */
/*                                data frame for transmission               */
/* INPUTS                         ObjectNumber:     Identifier              */
/*                                DataLength:number of data bytes           */
/*                                *pData:    pointer to data                */
/* OUTPUTS                        OK                                        */
/*                                ERROR1: request overrun                   */
/*                                ERROR3: fifo full                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_write_object(INT32 ObjectNumber, INT32 DataLength, PUSIGN8 pData,
                        volatile PCDEVICES pDevice)
{
  USBXMT_ITEM fifoItem;
  INT32 ret = VCARD_OK;
  
  _TRACE5("CANBUS canusb_write_object: 0x%x\n",ObjectNumber);
  
  if (pDevice->object_buffer_enable)
  {
    if (pDevice->dyn_enable)
    {
      ret = canpro_write_object_dyn(pDevice, ObjectNumber, (INT8) DataLength, pData);
      if(!ret)
      {
        fifoItem.can_cmd = WRT_OBJ;
        fifoItem.can_id = ObjectNumber;
        memcpy((PVOID)fifoItem.data,pData,8);
        fifoItem.can_len = (INT8)DataLength;
        fifoItem.u32Counter = MsgCounter++;
        ret = canusb_insert_fifo(pDevice, &fifoItem, pDevice->canprousb.pSharedMem);
      }
    }
    else
    {
      ret = canpro_write_object_stat(pDevice, ObjectNumber, (INT8) DataLength, pData);
      if(!ret)
      {
        fifoItem.can_cmd = WRT_OBJ;
        fifoItem.can_id = ObjectNumber;
        memcpy((PVOID)fifoItem.data,pData,8);
        fifoItem.can_len = (INT8)DataLength;
        fifoItem.u32Counter = MsgCounter++;
        ret = canusb_insert_fifo(pDevice, &fifoItem, pDevice->canprousb.pSharedMem);
      }
    }
  }
  else
  {
    ret = CANL2_OB_NOT_DEFINED;
  }
  return ret;
}


INT32 map_CANL2_read_rcv_data_error(INT32 s32Error)
{
  INT32 ret=0;

  if(s32Error>=0)
    return s32Error;

  switch(s32Error)
  {
    case -1:
      ret = CANL2_DATA_OVR;
      break;

    case -2:
      ret = CANL2_REMOTE_OVR;
      break;

    case -3:
      ret = CANL2_NOT_ACTIVE;
      break;

    case -7:
      ret = CANL2_FW_TIMEOUT;
      break;

    default:
      ret = CANL2_ERR;
      break;


  }

  return ret;

}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       canpro_read_rcv_data_stat                 */
/* FUNCTIONAL_DESCRIPTION         read object data in stat. obj. buf. mode  */
/*                                                                          */
/* INPUTS                Can:              CAN handle                       */
/*                       Ident:            object number/handle             */
/*                       pData:     :      data buffer                      */
/*                       pRcvObject:       pointer to the object to read    */
/*                       Time:             time stamp                       */
/*                                                                           */
/* OUTPUTS                                                                  */
/*                                                                          */
/*HEAD_END*******************************************************************/

INT32 canpro_read_rcv_data_stat(PCDEVICES pDevice, INT32 Ident, PBYTE pData, PUSIGN32 Time)
{
  CAN_ADDRESS rcvIrql;
  INT32  return_value;
  volatile RCV_OBJECT *pRcvObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[Ident];
  
  //sync_get_access(&pObject->host_access, &pObject->fw_access);
  rcvIrql = sync_get_access((spinlock_t *)&pRcvObject->access);
  
  
  /* function part for Static Object Buffer Mode */
  /* ------------------------------------------- */
  if (pRcvObject->ctrl1 & RCV_ENABLE)
  {
    /* remote frame received ? */
    if (pRcvObject->ctrl1 & RCV_STATUS_REMOTE)
    {
      *Time = pRcvObject->TimestampRem;
      
      /* overrun ? */
      if (pRcvObject->ctrl1 & RCV_STATUS_OVERRUN)
      {
        pRcvObject->ctrl1 &= ~(RCV_STATUS_OVERRUN | RCV_STATUS_REMOTE);
        //sync_free_access(&pRcvObject->host_access);
        sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
        return(CANL2_REMOTE_OVR);
      }
      else
      {
        pRcvObject->ctrl1 &= ~RCV_STATUS_REMOTE;
        //sync_free_access(&pRcvObject->host_access);
        sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
        return(RET_CODE2);
      }
    }
    
    /* read data until no NEW data received in isr */
    return_value = RET_CODE0;
    
    /* data frame received in the meantime ? */
    if (pRcvObject->ctrl1 & RCV_STATUS_DATA)
    {
      /* overrun ? */
      if (pRcvObject->ctrl1 & RCV_STATUS_OVERRUN)
      {
        pRcvObject->ctrl1 &= ~(RCV_STATUS_OVERRUN | RCV_STATUS_DATA);
        return_value = CANL2_DATA_OVR;
      }
      else
      {
        pRcvObject->ctrl1 &= ~RCV_STATUS_DATA;
        if (return_value != CANL2_DATA_OVR)
          return_value = RET_CODE1;
      }
    }
    
    /* read data */
    *(USIGN64 *)&(pData[0]) = *(USIGN64  *)&(pRcvObject->data[0]);
    
    *Time = pRcvObject->Timestamp;
    
    //sync_free_access(&pRcvObject->host_access);
    sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
    return(return_value);
  }
  else
  {
    /* identifier not defined */
    //sync_free_access(&pRcvObject->host_access);
    sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
    return(CANL2_NOT_ACTIVE);
  }
}

/*HEAD_BEG********************************************************************/
/*                                                                           */
/* FUNCTION                       read_rcv_data                              */
/* FUNCTIONAL_DESCRIPTION         reads received data from object buffer     */
/* INPUTS                         Can: CAN handle                            */
/*                                Ident:     Identifier                      */
/*                                *pRCV_Data:pointer to data                 */
/*                                                                           */
/* OUTPUTS                        RET_CODE0: no new data received            */
/*                                RET_CODE1: data frame received             */
/*                                RET_CODE2: remote frame received           */
/*                                ERROR1:    overrun in object buffer by     */
/*                                           data frame                      */
/*                                ERROR2:    overrun in object buffer by     */
/*                                           remote frame                    */
/*                                ERROR3:    object not defined              */
/*                                ERROR7:    timeout in access to receive    */
/*                                           object                          */
/*                                                                           */
/*HEAD_END********************************************************************/
INT32 canpro_read_rcv_data_dyn(PCDEVICES pDevice, INT32 Ident, PBYTE pData, PUSIGN32 Time)
{
  volatile RCV_OBJECT *pRcvObject;
  CAN_ADDRESS rcvIrql;
  
  INT32  return_value;
  
  if (Ident > MAX_OBJ_DYN)
    return(CANL2_WRONG_PARAM);
  
  pRcvObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.object[Ident];
  
  
  /* function part for Dynamic Object Buffer Mode */
  /* -------------------------------------------- */
  rcvIrql = sync_get_access((spinlock_t *)&pRcvObject->access);
  if (pRcvObject->ctrl1 & RCV_ENABLE)
  {
    /* remote frame received ? */
    if (pRcvObject->ctrl1 & RCV_STATUS_REMOTE)
    {
      *Time = pRcvObject->TimestampRem;
      /* overrun ? */
      if (pRcvObject->ctrl1 & RCV_STATUS_OVERRUN)
      {
        pRcvObject->ctrl1 &= ~(RCV_STATUS_OVERRUN | RCV_STATUS_REMOTE);
        sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
        return(CANL2_REMOTE_OVR);
      }
      else
      {
        pRcvObject->ctrl1 &= ~RCV_STATUS_REMOTE;
        sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
        return(RET_CODE2);
      }
    }
    
    /* read data until no NEW data received in isr */
    return_value = CANL2_OK;
    
    /* data frame received ? */
    if (pRcvObject->ctrl1 & RCV_STATUS_DATA)
    {
      /* overrun ? */
      if (pRcvObject->ctrl1 & RCV_STATUS_OVERRUN)
      {
        pRcvObject->ctrl1 &= ~(RCV_STATUS_OVERRUN | RCV_STATUS_DATA);
        return_value = CANL2_DATA_OVR;
      }
      else
      {
        pRcvObject->ctrl1 &= ~RCV_STATUS_DATA;
        return_value = RET_CODE1;
      }
    }
    
    /* read data */
    *(USIGN64 *)&(pData[0]) = *(USIGN64 *)&(pRcvObject->data[0]);
    
    /* time stamp */
    
    
    *Time = pRcvObject->Timestamp;
    
    sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
    return(return_value);
  }
  else
  {
    /* identifier not defined */
    sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
    return(CANL2_OB_NOT_DEFINED);
  }
  
}



/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       read_rcv_data, read_rcv_data2             */
/* FUNCTIONAL_DESCRIPTION         reads received data from object buffer    */
/* INPUTS                         Ident:     Identifier                     */
/*                                *pRCV_Data:pointer to data                */
/* OUTPUTS                        RET_CODE0: no new data received           */
/*                                RET_CODE1: data frame received            */
/*                                RET_CODE2: remote frame received          */
/*                                ERROR1:    overrun in object buffer by    */
/*                                           data frame                     */
/*                                ERROR2:    overrun in object buffer by    */
/*                                           remote frame                   */
/*                                ERROR3:    object not defined             */
/*                                ERROR7:    timeout in access to receive   */
/*                                           object                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_read_rcv_data(INT32 ObjectNumber, PUSIGN8 pData, PUSIGN32 Time,
                         volatile PCDEVICES pDevice)
{
  INT32 ret = VCARD_OK;
  
  _TRACE5("canusb_read_rcv_object: \n");
  
  if(!pDevice->canprousb.pObjectBuf)
  {
    return CANL2_BOARD_NOT_INITIALIZED;
  }
  
  
  if (pDevice->object_buffer_enable)
  {
    if (pDevice->dyn_enable)
    {
      ret = canpro_read_rcv_data_dyn(pDevice, ObjectNumber, pData, Time);
    }
    else
    {
      ret = canpro_read_rcv_data_stat(pDevice, ObjectNumber, pData, Time);
    }
  }
  else
  {
    ret = CANL2_OB_NOT_DEFINED;
  }
  
  
  return ret;
}


/*HEAD_BEG********************************************************************/
/*                                                                           */
/* FUNCTION                       canpro_read_xmt_data_stat                  */
/* FUNCTIONAL_DESCRIPTION         reads transmit object data and checks      */
/*                                transmit acknowledge                       */
/* INPUTS                         Can:       channnel handle                 */
/*                                Ident:     object number                   */
/*                                *pDataLength: return of data length        */
/*                                *pXMT_Data:   return of data               */
/*                                                                           */
/* OUTPUTS                        RET_CODE0:    data not yet transmitted     */
/*                                RET_CODE1:    data already transmitted     */
/*                                ERROR1        acknowledge overrun          */
/*                                                                           */
/*HEAD_END********************************************************************/

INT32 canpro_read_xmt_data_stat(PCDEVICES pDevice, INT32 Ident, PINT32 pDataLength, PBYTE pXMTData)
{
  volatile XMT_OBJECT *pXmtObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[Ident];
  CAN_ADDRESS xmtIrql;
  
  //sync_get_access(&pXmtObject->host_access, &pXmtObject->fw_access);
  xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
  
  /* read data from object buffer */
  *(USIGN64  *)pXMTData = *(USIGN64  *)pXmtObject->data;
  
  *pDataLength = pXmtObject->len_data;
  
  /* transmit acknowledged ? */
  if (pXmtObject->ctrl[1] & XMT_STATUS)
  {
    if (pXmtObject->ctrl[1] & XMT_STATUS_OVERRUN)
    {
      /* overrun */
      pXmtObject->ctrl[1] &= ~(XMT_STATUS | XMT_STATUS_OVERRUN);
      //sync_free_access(&pXmtObject->host_access);
      sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
      return(-1);
    }
    else
    {
      /* no overrun */
      pXmtObject->ctrl[1] &= ~XMT_STATUS;
      //sync_free_access(&pXmtObject->host_access);
      sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
      return(RET_CODE1);
    }
  }
  else
  {
    /* no acknowledge */
    sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
    return(RET_CODE0);
  }
  
}


/*HEAD_BEG********************************************************************/
/*                                                                           */
/* FUNCTION                       canpro_read_xmt_data_dyn                   */
/* FUNCTIONAL_DESCRIPTION         reads transmit object data and checks      */
/*                                transmit acknowledge                       */
/* INPUTS                         Can:       channnel handle                 */
/*                                Ident:     object number                   */
/*                                *pDataLength: return of data length        */
/*                                *pXMT_Data:   return of data               */
/*                                                                           */
/* OUTPUTS                        RET_CODE0:    data not yet transmitted     */
/*                                RET_CODE1:    data already transmitted     */
/*                                ERROR1        acknowledge overrun          */
/*                                                                           */
/*HEAD_END********************************************************************/
INT32 canpro_read_xmt_data_dyn(PCDEVICES pDevice, INT32 Ident, PINT32 pDataLength, PBYTE pXMTData)
{
  volatile XMT_OBJECT *pXmtObject;
  CAN_ADDRESS xmtIrql;
  
  if(Ident > MAX_OBJ_DYN)
  {
    return(CANL2_WRONG_PARAM);
  }
  
  if(pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.xmt_id_lookup[Ident] == XTD_OBJ_NOT_VALID)
  {
    return(CANL2_WRONG_PARAM);
  }
  
  pXmtObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[Ident];
  
  xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
  
  /* read data from object buffer */
  *((USIGN64 *)pXMTData) = *((USIGN64 *)pXmtObject->data);
  
  *pDataLength = pXmtObject->len_data;
  
  /* transmit acknowledged ? */
  if (pXmtObject->ctrl[1] & XMT_STATUS)
  {
    if (pXmtObject->ctrl[1] & XMT_STATUS_OVERRUN)
    {
      /* overrun of transmit acknowledge */
      pXmtObject->ctrl[1] &= ~(XMT_STATUS | XMT_STATUS_OVERRUN);
      sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
      return(CANL2_RXD_XMT_OVR);
    }
    else
    {
      /* no overrun */
      pXmtObject->ctrl[1] &= ~XMT_STATUS;
      sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
      return(RET_CODE1);
    }
  }
  else
  {
    /* no acknowledge */
    sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
    return(RET_CODE0);
  }
  
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       read_xmt_data, read_xmt_data2             */
/* FUNCTIONAL_DESCRIPTION         reads transmit object data and checks     */
/*                                transmit acknowledge                      */
/* INPUTS                         Ident:        identifier                  */
/*                                *pDataLength: return of data length       */
/*                                *pXMT_Data:   return of data              */
/* OUTPUTS                        RET_CODE0:    data not yet transmitted    */
/*                                RET_CODE1:    data already transmitted    */
/*                                ERROR1        acknowledge overrun         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_read_xmt_data(INT32 ObjectNumber, PINT32 pDataLength, PUSIGN8 pXMTData,
                    volatile PCDEVICES pDevice)
{
  INT32 ret = VCARD_OK;
  
  _TRACE5("canusb_read_xmt_object: \n");
  
  if(!pDevice->canprousb.pObjectBuf)
  {
    return CANL2_BOARD_NOT_INITIALIZED;
  }
  
  if (pDevice->object_buffer_enable)
  {
    if (pDevice->dyn_enable)
    {
      ret = canpro_read_xmt_data_dyn(pDevice, ObjectNumber, pDataLength, pXMTData);
    }
    else
    {
      ret = canpro_read_xmt_data_stat(pDevice, ObjectNumber, pDataLength, pXMTData);
    }
  }
  else
  {
    ret = CANL2_OB_NOT_DEFINED;
  }
  
  return ret;
}



/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_get_bus_state                       */
/* FUNCTIONAL_DESCRIPTION         returns bus state                         */
/*                                                                          */
/* INPUTS                         Can: bus 1 or 2                           */
/* OUTPUTS                        0: error active                           */
/*                                1: error passive                          */
/*                                2: bus off                                */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/

int_ CANDR_get_bus_state(volatile PCDEVICES pDevice)
{    
  INT32 ret;
  T_CANUSBP_SHARED_MEM *pShMem = (T_CANUSBP_SHARED_MEM *)pDevice->canprousb.pSharedMem;
  
  _TRACE5("CANDR_get_bus_state(%09u)\n", pDevice->Serialnumber);
  
  if(pShMem->usb_error)
  {
    return BUS_OFF;
  }
  
  ret = pShMem->can_state & 3;
  
  return ret;
}

INT32 canusb_getEventType(USIGN8 cmd)
{
  INT32 ret = 0;
  
  switch(cmd)
  {
    
    case RCV_CMD:
      ret = 1;
      break;
      
    case RCV_CMD_RMT:
      ret = 2;
      break;
      
    case XMT_ACK_CMD:
      ret = 3;
      break;
      
    case 0x20:
      ret = 5;
      break;
      
      
    case XMT_ACK_CMD_RMT:
      ret = 8;
      break;
      
    case RCV_CMD_XTD:
      ret = 9;
      break;
      
    case XMT_ACK_CMD_XTD:
      ret = 10;
      break;
      
    case XMT_ACK_CMD_XTD_RMT:
      ret = 11;
      break;
      
    case RCV_CMD_XTD_RMT:
      ret = 12;
      break;
      
    case ERR_CMD:
      ret = 15;
      break;
      
      
    default:
      ret = 0;
      break;
  }
  
  return ret;
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       read_object                               */
/* FUNCTIONAL_DESCRIPTION         read object                               */
/*                                                                          */
/* INPUTS                Can:              CAN handle                       */
/*                       param     :       param struct                     */
/*                       pRcvObject:       pointer to the object to read    */
/*                       pStaticRcvStruct: static obj. buf. infrastructure  */
/*                       Ident:            object number/handle             */
/* OUTPUTS                                                                  */
/*                                                                          */
/*HEAD_END*******************************************************************/

INT32 read_object(PCDEVICES pDevice, PARAM_STRUCT *param, volatile RCV_OBJECT *pRcvObject, volatile RCV_STATIC_STRUCT *pStaticRcvStruct, USIGN32 Ident)
{
  
  INT32 loop_var;
  CAN_ADDRESS rcvIrql;
  
  //sync_get_access(&pRcvObject->host_access, &pRcvObject->fw_access);
  rcvIrql = sync_get_access((spinlock_t *)&pRcvObject->access);
  
  /*************************************/
  /* data frame received ?             */
  /*************************************/
  if (pRcvObject->ctrl1 & RCV_STATUS_DATA)
  {
    /* read data until no new received data */
    /* overrun ? */
    if (pRcvObject->ctrl1 & RCV_STATUS_OVERRUN)
    {
      param->RecOverrun_flag = 1;
    }
    else
    {
      param->RecOverrun_flag = 0;
    }
    
    /* reset status bits */
    pRcvObject->ctrl1 &= ~(RCV_STATUS_OVERRUN | RCV_STATUS_DATA);
    
    /* read data length */
    param->DataLength = pRcvObject->DataLength;
    
    /* read data */
    
    for(loop_var=0;loop_var < pRcvObject->DataLength;loop_var++) 
      param->RCV_data[loop_var] = pRcvObject->data[loop_var];
    
    /* time stamp */
    
    param->Time  = pRcvObject->Timestamp;
    
    
    param->RCV_fifo_lost_msg = pStaticRcvStruct->fifo_lost_msg;
    //sync_free_access(&pRcvObject->host_access);
    sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
    
    return (RET_CODE1);
  }
  else
  {
    /**************************************/
    /* remote frame received ?            */
    /**************************************/
    if (pRcvObject->ctrl1 & RCV_STATUS_REMOTE)
    {
      
      if (pRcvObject->ctrl1 & RCV_STATUS_OVERRUN)
        param->RecOverrun_flag = 1;
      else
        param->RecOverrun_flag = 0;
      
      /* reset status bits */
      pRcvObject->ctrl1 &= ~(RCV_STATUS_OVERRUN | RCV_STATUS_REMOTE);
      
      
      /* read data length */
      param->DataLength = pRcvObject->DataLengthRem;
      
      
      /* time stamp */
      param->Time = pRcvObject->TimestampRem;
      
      param->RCV_fifo_lost_msg = pStaticRcvStruct->fifo_lost_msg;
      
      //sync_free_access(&pRcvObject->host_access);
      sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
      
      return (RET_CODE2);
    }
    else
    {
      //sync_free_access(&pRcvObject->host_access);
      sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
      return (RET_CODE0); /*neither data nor remote frame*/
    }
  }
}

/*HEAD_BEG********************************************************************/
/*                                                                           */
/* FUNCTION                       canpro_read_ac_dyn                         */
/* FUNCTIONAL_DESCRIPTION         checks new events in object buffer and     */
/*                                gives messages to host                     */
/*                                                                           */
/* INPUTS                         Can: CAN handle                            */
/*                                param: pointer to the param structure      */
/*                                                                           */
/*                                                                           */
/* OUTPUTS                        O: no new event from firmware              */
/*                                1: std data frame received                 */
/*                                2: std remote frame received               */
/*                                3: acknowledge of a transmitted            */
/*                                   std data frame                          */
/*                                4: not implemented                         */
/*                                5: change of bus state                     */
/*                                8: acknowledge of a transmitted            */
/*                                   std remote frame                        */
/*                                9: xtd data frame received                 */
/*                                10:acknowledge of a transmitted            */
/*                                   xtd data frame                          */
/*                                11:acknowledge of a transmitted            */
/*                                   xtd remote frame                        */
/*                                12:xtd remote frame received               */
/*                                                                           */
/*HEAD_END********************************************************************/



INT32 canprousb_read_ac_stat(PCDEVICES pDevice, PARAM_STRUCT *param, USIGN8 can_state, USIGN32 time)
{
  INT32 Ident;
  INT32 poll_index;
  CAN_ADDRESS xmtIrql;
  INT32 ret;
  
  volatile RCV_STATIC_STRUCT *pStaticRcvStruct = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static;
  volatile XMT_STATIC_STRUCT *pStaticXmtStruct = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static;
  volatile USIGN16 *pXmtPollList = pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.xmt_poll_list;
  
  _TRACE5("CANBUS canpro_read_ac_stat: \n");
  /* static object buffer mode is only possible with CAN2 */
  param->Can = 1;
  
  /**********************/
  /* Bus-Status pruefen */
  /**********************/
  if (can_state != pDevice->bus_state)
  {
    param->Bus_state = can_state;
    pDevice->bus_state = can_state;
    
    param->Time = time;
    _TRACE5("CANBUS canpro_read_ac_stat: done! ret=%u\n",RET_CODE5);
    return (RET_CODE5);
  }
  
  /* check channels in Static Object Buffer Mode */
  /* ------------------------------------------- */
  if (pDevice->canprousb.ReceiveFifoEnable)
  {
    if ((USIGN8)pStaticRcvStruct->fifo_write_ptr !=
      (USIGN8)(pStaticRcvStruct->fifo_read_ptr + (USIGN8)1))
    {
      /* via RCV_FIFO */
      volatile RCV_OBJECT *pRcvObject;
      
      Ident =  pStaticRcvStruct->fifo[(USIGN8)++pStaticRcvStruct->fifo_read_ptr];
      pRcvObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[Ident];
      param->Ident = Ident;
      ret = read_object(pDevice, param, pRcvObject, pStaticRcvStruct, Ident);
      _TRACE5("CANBUS canpro_read_ac_stat: done! ret=%u\n",ret);
      return(ret);
    }
  }
  else
  {
    /* via polling */
    /* poll complete static object buffer       */
    /* (not possible with dynamic object buffer)*/
    if (pDevice->canprousb.ReceivePollAll) 
    {
      for (Ident=0; Ident < ANZ_IDENT; Ident++)
      {
        volatile RCV_OBJECT *pRcvObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[Ident];
        
        if (pRcvObject->ctrl1 & RCV_STATUS_DATA_OR_REMOTE)
        {
          param->Ident = Ident;
          ret = read_object(pDevice, param, pRcvObject, pStaticRcvStruct, Ident);
          _TRACE5("CANBUS canpro_read_ac_stat: done! ret=%u\n",ret);
          return(ret);
        }
      }
    }
    else
    {
      /* poll only poll_list */
      for (poll_index = 0; (Ident = pStaticRcvStruct->poll_list[poll_index]) != END_MARK_WORD;poll_index++)
      {
        volatile RCV_OBJECT *pRcvObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.rcv_static.object[Ident];
        if (pRcvObject->ctrl1 & RCV_STATUS_DATA_OR_REMOTE)
        {
          param->Ident = Ident;
          ret = read_object(pDevice, param, pRcvObject, pStaticRcvStruct, Ident);
          _TRACE5("CANBUS canpro_read_ac_stat: done! ret=%u\n",ret);
          return(ret);
        }
      }
    }
  }
  
  /********************************************************************/
  /* check transmit acknowledges                                      */
  /********************************************************************/
  // if (xmt_ack_enable)
  {
    if (pDevice->canprousb.TransmitAckFifoEnableAll)
    {
      if ((USIGN8)pStaticXmtStruct->ack_fifo_write_ptr !=
        (USIGN8)(pStaticXmtStruct->ack_fifo_read_ptr + (USIGN8)1))
      {
        volatile XMT_OBJECT *pXmtObject;
        USIGN8 read_ptr = pStaticXmtStruct->ack_fifo_read_ptr+1;
        
        /* get identifier from fifo */
        Ident = pStaticXmtStruct->ack_fifo[read_ptr];
        param->Ident = Ident;
        pXmtObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[Ident];
        
        param->XMT_ack_fifo_lost_acks = pStaticXmtStruct->ack_fifo_lost_acks;
        
        // sync_get_access(&pXmtObject->host_access, &pXmtObject->fw_access);
        xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
        
        
        /* overrun ? */
        if (pXmtObject->ctrl[1] & XMT_STATUS_OVERRUN)
          param->AckOverrun_flag = 1;
        else
          param->AckOverrun_flag = 0;
        
        
        if(pXmtObject->ctrl[1] & XMT_ACK_RMT)
        {
          pXmtObject->ctrl[1] &= ~XMT_ACK_RMT;
          
          
          param->Time = pXmtObject->TimestampRemote;
          
          param->DataLength = pXmtObject->len_remote;
          
          /* reset status bits */
          pXmtObject->ctrl[1] &= ~XMT_ACK_RMT;
          pStaticXmtStruct->ack_fifo_read_ptr = read_ptr;
          
          //sync_free_access(&pXmtObject->host_access);
          sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
          _TRACE5("CANBUS canpro_read_ac_stat: done! ret=%u\n",RET_CODE8);
          return(RET_CODE8);
        }
        else
        {
          
          param->Time = pXmtObject->Timestamp;
          
          param->DataLength = pXmtObject->len_data;
          memcpy((PVOID)param->RCV_data, (PVOID)pXmtObject->data, 8);
          
          /* reset status bits */
          pXmtObject->ctrl[1] &= ~XMT_STATUS;
          
          pStaticXmtStruct->ack_fifo_read_ptr = read_ptr;
          //sync_free_access(&pXmtObject->host_access);
          sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
          _TRACE5("CANBUS canpro_read_ac_stat: done! ret=%u\n",RET_CODE3);
          return(RET_CODE3);
        }
      }
    }
    else if (pDevice->canprousb.TransmitAckEnableAll)
    {
      /* poll all transmit objects */
      for (Ident=0;Ident < ANZ_IDENT; Ident++)
      {
        volatile XMT_OBJECT *pXmtObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[Ident];
        
        if (pXmtObject->ctrl[1] & (XMT_STATUS | XMT_ACK_RMT))
        {
          // sync_get_access(&pXmtObject->host_access, &pXmtObject->fw_access);
          xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
          param->Ident = Ident;
          /* overrun ? */
          if (pXmtObject->ctrl[1] & XMT_STATUS_OVERRUN)
            param->AckOverrun_flag = 1;
          else
            param->AckOverrun_flag = 0;
          
          pXmtObject->ctrl[1] &= ~XMT_STATUS_OVERRUN;
          
          param->XMT_ack_fifo_lost_acks = 0;
          
          if (pXmtObject->ctrl[1] & XMT_ACK_RMT)
          {
            param->Time = pXmtObject->TimestampRemote;
            
            param->DataLength = pXmtObject->len_remote;
            /* reset status bits */
            pXmtObject->ctrl[1] &= ~XMT_ACK_RMT;
            //  pXmtObject->ctrl[1] |= XMT_ACK_ENABLE;
            
            //sync_free_access(&pXmtObject->host_access);
            sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
            _TRACE5("CANBUS canpro_read_ac_stat: done! ret=%u\n",RET_CODE8);
            return(RET_CODE8);
          }
          else
          {
            param->Time = pXmtObject->Timestamp;
            
            param->DataLength = pXmtObject->len_data;
            memcpy((PVOID)param->RCV_data, (PVOID)pXmtObject->data, 8);
            /* reset status bits */
            pXmtObject->ctrl[1] &= ~XMT_STATUS;
            // pXmtObject->ctrl[1] |= XMT_ACK_ENABLE;
            
            //sync_free_access(&pXmtObject->host_access);
            sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
            _TRACE5("CANBUS canpro_read_ac_stat: done! ret=%u\n",RET_CODE3);
            return(RET_CODE3);
          }
        }
      }
    }
    else /*TransmitAckEnableAll*/
    {
      /* poll transmit objects in xmt poll list */
      for (poll_index=0; (Ident = pXmtPollList[poll_index]) != END_MARK_WORD; poll_index++)
      {
        volatile XMT_OBJECT *pXmtObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_stat_obj_buf.xmt_static.object[Ident];
        
        if (pXmtObject->ctrl[1] & (XMT_STATUS | XMT_ACK_RMT))
        {
          //sync_get_access(&pXmtObject->host_access, &pXmtObject->fw_access);
          xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
          param->Ident = Ident;
          /* overrun ? */
          if (pXmtObject->ctrl[1] & XMT_STATUS_OVERRUN)
            param->AckOverrun_flag = 1;
          else
            param->AckOverrun_flag = 0;
          
          pXmtObject->ctrl[1] &= ~XMT_STATUS_OVERRUN;
          
          param->XMT_ack_fifo_lost_acks = 0;
          
          if (pXmtObject->ctrl[1] & XMT_ACK_RMT)
          {
            pXmtObject->ctrl[1] &= ~XMT_ACK_RMT;
            
            
            param->Time = pXmtObject->TimestampRemote;
            
            param->DataLength = pXmtObject->len_remote;
            /* reset status bits */
            pXmtObject->ctrl[1] &= XMT_ACK_RMT;
            
            // sync_free_access(&pXmtObject->host_access);
            sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
            _TRACE5("CANBUS canpro_read_ac_stat: done! ret=%u\n",RET_CODE8);
            return(RET_CODE8);
          }
          else
          {
            xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
            param->Time = pXmtObject->Timestamp;
            
            param->DataLength = pXmtObject->len_data;
            memcpy((PVOID)param->RCV_data, (PVOID)pXmtObject->data, 8);
            
            /* reset status bits */
            pXmtObject->ctrl[1] &= ~XMT_STATUS;
            
            // sync_free_access(&pXmtObject->host_access);
            sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
            _TRACE5("CANBUS canpro_read_ac_stat: done! ret=%u\n",RET_CODE3);
            return(RET_CODE3);
          }
        }
      }
    }
  }
  
  /****************************************/
  /* check lost remote transmit requests  */
  /****************************************/
  if (pStaticXmtStruct->rmt_fifo_lost_remotes > pDevice->canprousb.lost_remotes)
  {
    param->XMT_rmt_fifo_lost_remotes = pStaticXmtStruct->rmt_fifo_lost_remotes;
    pDevice->canprousb.lost_remotes = pStaticXmtStruct->rmt_fifo_lost_remotes;
    _TRACE5("CANBUS canpro_read_ac_stat: done! ret=%u\n",RET_CODE4);
    return (RET_CODE4);
  }
  
  _TRACE5("CANBUS canpro_read_ac_stat: done! ret=%u\n",RET_CODE0);
  return (RET_CODE0);
  
}




/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       read_object_dyn                           */
/* FUNCTIONAL_DESCRIPTION         read object in dyn. obj. buffer mode      */
/*                                                                          */
/* INPUTS                param     :       param struct                     */
/*                       pRcvObject:       pointer to the object to read    */
/*                       pStaticRcvStruct: static obj. buf. infrastructure  */
/*                       Ident:            object number/handle             */
/* OUTPUTS                                                                  */
/*                                                                          */
/*HEAD_END*******************************************************************/
INT32 read_object_dyn(PARAM_STRUCT *param, 
                      volatile RCV_OBJECT *pRcvObject, 
                      RCV_DYN_STRUCT *pDynRcvStruct, 
                      USIGN32 Ident)
{
  
  INT32 loop_var;
  CAN_ADDRESS rcvIrql;
  
  /*************************************/
  /* data frame received ?             */
  /*************************************/
  if (pRcvObject->ctrl1 & RCV_STATUS_DATA)
  {
    
    param->RecOverrun_flag = FALSE;
    
    /* overrun ? */
    if (pRcvObject->ctrl1 & RCV_STATUS_OVERRUN)
      param->RecOverrun_flag = TRUE;
    
    rcvIrql = sync_get_access((spinlock_t *)&pRcvObject->access);
    
    
    /* read data length */
    param->DataLength = pRcvObject->DataLength;
    
    /* read data */
    for(loop_var=0;loop_var < pRcvObject->DataLength;loop_var++) 
      param->RCV_data[loop_var] = pRcvObject->data[loop_var];
    
    
    /* time stamp */
    
    param->Time = pRcvObject->Timestamp;
    
    
    param->RCV_fifo_lost_msg = pDynRcvStruct->fifo_lost_msg;
    
    if (pRcvObject->ctrl1 & XTD_RCV)
    {
      /* reset status bits */
      pRcvObject->ctrl1 &= ~(RCV_STATUS_OVERRUN | RCV_STATUS_DATA);
      
      sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
      return (RET_CODE9);
    }
    else
    {
      /* reset status bits */
      pRcvObject->ctrl1 &= ~(RCV_STATUS_OVERRUN | RCV_STATUS_DATA);
      
      //sync_free_access(&pRcvObject->host_access);
      sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
      return (RET_CODE1);
    }
  }
  else
  {
    /**************************************/
    /* remote frame received ?            */
    /**************************************/
    if (pRcvObject->ctrl1 & RCV_STATUS_REMOTE)
    {
      
      param->RecOverrun_flag = FALSE;
      
      /* overrun ? */
      if (pRcvObject->ctrl1 & RCV_STATUS_OVERRUN)
        param->RecOverrun_flag = TRUE;
      
      rcvIrql = sync_get_access((spinlock_t *)&pRcvObject->access);
      
      
      /* read data length */
      param->DataLength = pRcvObject->DataLengthRem;
      
      /* time stamp */
      
      param->Time = pRcvObject->TimestampRem;
      
      param->RCV_fifo_lost_msg = pDynRcvStruct->fifo_lost_msg;
      
      if (pRcvObject->ctrl1 & XTD_RCV)
      {
        /* reset status bits */
        pRcvObject->ctrl1 &= ~(RCV_STATUS_OVERRUN | RCV_STATUS_REMOTE);
        
        sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
        return (RET_CODE12);
      }
      else
      {
        /* reset status bits */
        pRcvObject->ctrl1 &= ~(RCV_STATUS_OVERRUN | RCV_STATUS_REMOTE);
        
        sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
        return (RET_CODE2);
      }
    }
    else
    {
      return (RET_CODE0); /* neither data nor remote frame */
    }
  }
}

/*HEAD_BEG********************************************************************/
/*                                                                           */
/* FUNCTION                       canpro_read_ac_dyn                         */
/* FUNCTIONAL_DESCRIPTION         checks new events in object buffer and     */
/*                                gives messages to host                     */
/*                                                                           */
/* INPUTS                         Can: CAN handle                            */
/*                                param: pointer to the param structure      */
/*                                                                           */
/* OUTPUTS                        O: no new event from firmware              */
/*                                1: std data frame received                 */
/*                                2: std remote frame received               */
/*                                3: acknowledge of a transmitted            */
/*                                   std data frame                          */
/*                                4: not implemented                         */
/*                                5: change of bus state                     */
/*                                8: acknowledge of a transmitted            */
/*                                   std remote frame                        */
/*                                9: xtd data frame received                 */
/*                                10:acknowledge of a transmitted            */
/*                                   xtd data frame                          */
/*                                11:acknowledge of a transmitted            */
/*                                   xtd remote frame                        */
/*                                12:xtd remote frame received               */
/*                                                                           */
/*HEAD_END********************************************************************/


INT32 canprousb_read_ac_dyn(PCDEVICES pDevice, PARAM_STRUCT *param, USIGN8 can_state, USIGN32 time)
{
  
  INT32 Ident;
  INT32 poll_index;
  INT32 next;
  RCV_DYN_STRUCT *pDynRcvStruct = &pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn;
  XMT_DYN_STRUCT *pDynXmtStruct = &pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn;
  volatile RCV_OBJECT *pRcvObject;
  volatile XMT_OBJECT *pXmtObject;
  CAN_ADDRESS xmtIrql;
  
  
  
  param->Can = 1;
  
  /**********************/
  /* Bus-Status pruefen */
  /**********************/
  if (can_state != pDevice->bus_state)
  {
    param->Bus_state = can_state;
    pDevice->bus_state = can_state;
    
    param->Time = time;
    
    return (RET_CODE5);
  }
  
  
  /* check channels in Dynamic Object Buffer Mode */
  /* -------------------------------------------- */
  
  if (pDevice->canprousb.ReceiveFifoEnable)
  {
    USIGN8 read_ptr = pDynRcvStruct->fifo_read_ptr+1;
    
    if ((USIGN8) pDynRcvStruct->fifo_write_ptr != read_ptr)
    {
      /* via RCV_FIFO */
      /* get object number */
      Ident =  pDynRcvStruct->fifo[read_ptr]; 
      param->Ident = Ident;
      pRcvObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.object[Ident];
      next = read_object_dyn(param, pRcvObject, pDynRcvStruct, Ident);
      
      pDynRcvStruct->fifo_read_ptr++;
      return next;
    }
  }
  else
  {
    /* via polling */
    /*CAN 1*/
    for (poll_index = 0; poll_index < pDevice->number_of_rcv_objects; poll_index++)
    {
      pRcvObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.rcv_dyn.object[poll_index];
      if (pRcvObject->ctrl1 & RCV_STATUS_DATA_OR_REMOTE)
      {
        param->Ident = poll_index;
        return(read_object_dyn(param, pRcvObject, pDynRcvStruct, poll_index));
      }
    }
  }
  
  
  /********************************************************************/
  /* check transmit acknowledges                                      */
  /********************************************************************/
  if (pDevice->canprousb.TransmitAckFifoEnable)
  {
    USIGN8 read_ptr = pDynXmtStruct->ack_fifo_read_ptr + (USIGN8)1;
    
    if ((USIGN8) pDynXmtStruct->ack_fifo_write_ptr != read_ptr)
    {
      
      param->Can = 1;
      
      
      
      /* get identifier from fifo */
      Ident = pDynXmtStruct->ack_fifo[read_ptr];
      pXmtObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[Ident];
      
      param->Ident = Ident;
      
      xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
      
      
      *(USIGN64*)&param->RCV_data[0]  = *(USIGN64*)&pXmtObject->data[0];
      
      
      param->XMT_ack_fifo_lost_acks = pDynXmtStruct->ack_fifo_lost_acks;
      
      /* overrun ? */
      if (pXmtObject->ctrl[1] & XMT_STATUS_OVERRUN)
        param->AckOverrun_flag = TRUE;
      else
        param->AckOverrun_flag = FALSE;
      
      
      
      if (pXmtObject->ctrl[0] & XTD_XMT)
      {
        if(pXmtObject->ctrl[1] & XMT_ACK_RMT)
        {
          
          pDynXmtStruct->ack_fifo_read_ptr = read_ptr;
          /* reset status bits */
          pXmtObject->ctrl[1] |= XMT_ACK_ENABLE | XMT_ACK_FIFO_ENABLE;
          pXmtObject->ctrl[1] &= ~XMT_ACK_RMT;
          
          /* time stamp */
          
          param->Time = pXmtObject->TimestampRemote;
          
          param->DataLength =  pXmtObject->len_remote;
          sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
          return(RET_CODE11);
        }
        else
        {
          pDynXmtStruct->ack_fifo_read_ptr = read_ptr;
          /* reset status bits */
          pXmtObject->ctrl[1] |= XMT_ACK_ENABLE | XMT_ACK_FIFO_ENABLE;
          pXmtObject->ctrl[1] &= ~XMT_STATUS;
          
          /* time stamp */
          
          param->Time = pXmtObject->Timestamp;
          
          param->DataLength =  pXmtObject->len_data;
          *(USIGN64*)&param->RCV_data[0]  = *(USIGN64*)&pXmtObject->data[0];
          
          sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
          return(RET_CODE10);
        }
      }
      else
      {
        if(pXmtObject->ctrl[1] & XMT_ACK_RMT)
        {
          /* reset status bits */
          pXmtObject->ctrl[1] |= XMT_ACK_ENABLE | XMT_ACK_FIFO_ENABLE;
          pXmtObject->ctrl[1] &= ~XMT_ACK_RMT;
          
          /* time stamp */
          
          param->Time = pXmtObject->TimestampRemote;
          
          param->DataLength =  pXmtObject->len_remote;
          
          pDynXmtStruct->ack_fifo_read_ptr = read_ptr;
          
          sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
          return(RET_CODE8);
        }
        else 
        {
          /* reset status bits */
          pXmtObject->ctrl[1] |= XMT_ACK_ENABLE | XMT_ACK_FIFO_ENABLE;
          pXmtObject->ctrl[1] &= ~XMT_STATUS;
          
          /* time stamp */
          
          param->Time = pXmtObject->Timestamp;
          
          param->DataLength =  pXmtObject->len_data;
          *(USIGN64*)&param->RCV_data[0]  = *(USIGN64*)&pXmtObject->data[0];
          pDynXmtStruct->ack_fifo_read_ptr = read_ptr;
          sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
          return(RET_CODE3);
        }
      }
    }
    // sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
  }
  else
  {
    /* poll transmit objects */
    /*CAN 1*/
    for (poll_index=0; poll_index < pDevice->number_of_xmt_objects; poll_index++)
    {
      pXmtObject = &pDevice->canprousb.pObjectBuf->can_pro_usb_dyn_obj_buf.xmt_dyn.object[poll_index];
      if (pXmtObject->ctrl[1] & (XMT_STATUS | XMT_ACK_RMT))
      {
        
        param->Can = 1;
        
        param->Ident = poll_index;
        
        xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
        
        param->DataLength =  pXmtObject->len_data;
        /* overrun ? */
        if (pXmtObject->ctrl[1] & XMT_STATUS_OVERRUN)
          param->AckOverrun_flag = TRUE;
        else
          param->AckOverrun_flag = FALSE;
        
        
        pXmtObject->ctrl[1] &= ~XMT_STATUS_OVERRUN;
        
        param->XMT_ack_fifo_lost_acks = 0;
        
        if (pXmtObject->ctrl[0] & XTD_XMT)
        {
          if(pXmtObject->ctrl[1] & XMT_ACK_RMT)
          {
            /* reset status bits */
            pXmtObject->ctrl[1] &= ~XMT_ACK_RMT;
            /* time stamp */
            
            param->Time = pXmtObject->TimestampRemote;
            param->DataLength =  pXmtObject->len_remote;
            
            sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
            return(RET_CODE11);
          }
          else
          {
            /* reset status bits */
            pXmtObject->ctrl[1] &= ~XMT_STATUS;
            /* time stamp */
            
            param->Time = pXmtObject->Timestamp;
            *(USIGN64*)&param->RCV_data[0]  = *(USIGN64*)&pXmtObject->data[0];
            param->DataLength =  pXmtObject->len_data;
            sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
            return(RET_CODE10);
          }
        }
        else
        {
          if(pXmtObject->ctrl[1] & XMT_ACK_RMT)
          {
            /* reset status bits */
            pXmtObject->ctrl[1] &= ~XMT_ACK_RMT;
            /* time stamp */
            
            param->Time = pXmtObject->TimestampRemote;
            param->DataLength =  pXmtObject->len_remote;
            
            //  pXmtObject->ctrl[1] &= ~XMT_ACK_RMT;
            sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
            return(RET_CODE8);
          }
          else      
          { 
            /* reset status bits */
            pXmtObject->ctrl[1] &= ~XMT_STATUS;
            /* time stamp */
            
            param->Time = pXmtObject->Timestamp;
            param->DataLength =  pXmtObject->len_data;
            *(USIGN64*)&param->RCV_data[0]  = *(USIGN64*)&pXmtObject->data[0];
            sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
            return(RET_CODE3);
          }
        }
        // sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql); // because of loop
      }
    }
  }
  
  /****************************************/
  /* check lost remote transmit requests  */
  /****************************************/
  /*CAN 1*/
  if (pDynXmtStruct->rmt_fifo_lost_remotes > pDevice->canprousb.lost_remotes)
  {
    param->XMT_rmt_fifo_lost_remotes = pDynXmtStruct->rmt_fifo_lost_remotes;
    pDevice->canprousb.lost_remotes = pDynXmtStruct->rmt_fifo_lost_remotes;
    
    return (RET_CODE4);
  }
  return(RET_CODE0);
  
}

INT32 canprousb_read_ac(PARAM_STRUCT * param, volatile PCDEVICES pDevice)
{
  INT32 ret = VCARD_OK;
  
  if(!pDevice->canprousb.pObjectBuf)
  {
    return CANL2_BOARD_NOT_INITIALIZED;
  }
  
  if(pDevice->dyn_enable)
  {
    ret = canprousb_read_ac_dyn(
      pDevice, 
      param, 
      pDevice->canprousb.pSharedMem->can_state,
      pDevice->canprousb.pSharedMem->time
    );
  }
  else
  {
    ret = canprousb_read_ac_stat(
      pDevice, 
      param, 
      pDevice->canprousb.pSharedMem->can_state,
      pDevice->canprousb.pSharedMem->time
    );
  }
  
  return ret;
  
}


/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_read_ac                             */
/* FUNCTIONAL_DESCRIPTION         reads and interprets all events signalled */
/*                                by the firmware                           */
/* INPUTS                         -                                         */
/* OUTPUTS                        O: no new event from firmware             */
/*                                1: std data frame received                */
/*                                2: std remote frame received              */
/*                                3: acknowledge of a transmitted std data frame*/
/*                                4: not implemented                        */
/*                                5: change of bus state                    */
/*                                6: firmware reset                         */
/*                                7: error occured                          */
/*                                8: acknowledge of a transmitted std remote frame*/
/*                                9: xtd data frame received                */
/*                                10:acknowledge of a transmitted xtd data frame*/
/*                                11:acknowledge of a transmitted xtd remote frame*/
/*                                12:xtd remote frame received              */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/


int_ CANDR_read_ac(PARAM_STRUCT * param, volatile PCDEVICES pDevice)
{
  
//   DFCT_STRUCT cmd_in;
//   INT32 status;
  USIGN16 rd_rcv_fifo;
  USBRCVITEM volatile *pRcvItem;
  INT32 ret = 0;
  
  _TRACE5("CANDR_read_ac(%09u)\n", pDevice->Serialnumber);
  
  
  if(pDevice->object_buffer_enable)
  {
    ret = canprousb_read_ac(param, pDevice);
    param->Can = 1;
  }
  else
  {
    T_CANUSBP_SHARED_MEM *pShMem = pDevice->canprousb.pSharedMem;
    
    if(pShMem->usb_error)
    {
      return FRW_ERROR_COMMUNICATIONBROKEN;
    }
    
    rd_rcv_fifo = (pShMem->can_pro_usb_fifo_mode.rcv.rd_rcv_fifo + 1) & RCV_FIFO_USB_MASK;
    if (pShMem->can_pro_usb_fifo_mode.rcv.wr_rcv_fifo != rd_rcv_fifo)
    {
      
      
      pRcvItem = (USBRCVITEM volatile *)&pShMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[rd_rcv_fifo];
      
      param->DataLength                 = pRcvItem->u8DataLen;
      *(USIGN64*)param->RCV_data = *(USIGN64*)pRcvItem->u8Data;
      param->RCV_fifo_lost_msg          = pRcvItem->u16LostMsg;
      param->Bus_state                  = pRcvItem->u8Busstate;
      param->Can                        = 1;
      
      ret = canusb_getEventType(pRcvItem->u8Cmd);
      
      param->Ident = pRcvItem->u32Ident;
      
      param->Error_state                = pRcvItem->u8Err | ((USIGN32)pRcvItem->u8RxErrorCounter<<16) | ((USIGN32)pRcvItem->u8TxErrorCounter<<24);
      
      param->Time                       = MkTimestamp((USBRCVITEM *)pRcvItem);
      param->RCV_fifo_lost_msg          = pRcvItem->u16LostMsg;
      
      pShMem->can_pro_usb_fifo_mode.rcv.rd_rcv_fifo = rd_rcv_fifo;
    }
  }
  return ret;
}



/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_send_data, CANDR_send_data2         */
/* FUNCTIONAL_DESCRIPTION         transmit   a data frame                   */
/* INPUTS                         Ident:     Identifier                     */
/*                                DataLength:length of data                 */
/*                                *pData:    pointer to array of data       */
/* OUTPUTS                        0     int: ok                             */
/*                                -1    int: fifo full                      */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_send_data(USIGN32 Ident,
                INT32 xtd, INT32 DataLength, PUSIGN8 pData,
                volatile PCDEVICES pDevice)
{
  USIGN16 wr_xmt_fifo;
  USBXMT_ITEM volatile *pXmtItem;
  T_CANUSBP_SHARED_MEM *pShMem = (T_CANUSBP_SHARED_MEM *)pDevice->canprousb.pSharedMem;
  
  if(!pDevice->canprousb.transferStarted)
  {
    return FRW_ERROR_COMMUNICATIONBROKEN;
  }
  
  wr_xmt_fifo = pShMem->can_pro_usb_fifo_mode.xmt.wr_xmt_fifo;
  if (pShMem->can_pro_usb_fifo_mode.xmt.rd_xmt_fifo != wr_xmt_fifo)
  {
    pXmtItem = (USBXMT_ITEM volatile *)&pShMem->can_pro_usb_fifo_mode.xmt.xmt_fifo[wr_xmt_fifo];
    memset((USBXMT_ITEM *)pXmtItem,0,sizeof(USBXMT_ITEM));
    
    pXmtItem->can_cmd = XMT_CMD;
    pXmtItem->can_id = Ident;
    pXmtItem->can_len = DataLength;
    pXmtItem->xtd = xtd;
    pXmtItem->u32Counter = counter++;
    memset((PVOID)pXmtItem->data, 0, 8);
    memcpy((PVOID)pXmtItem->data, pData, DataLength);
    
    pShMem->can_pro_usb_fifo_mode.xmt.wr_xmt_fifo = (wr_xmt_fifo+1)&XMT_FIFO_USB_MASK;
    
  }
  else
  {
    return -1;
  }
  
  if(pDevice->canprousb.IoChainXmtState == CHAIN_IDLE)
  {
    XmtNextPacket(pDevice);
  }
  
  return 0;  

}



/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_send_remote                         */
/* FUNCTIONAL_DESCRIPTION         transmit   a data frame                   */
/* INPUTS                         Ident:     Identifier                     */
/*                                DataLength: dlc code in remote frame      */
/* OUTPUTS                        0     INT32: ok                             */
/*                                -1    INT32: fifo full                      */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_send_remote(USIGN32 Ident, INT32 xtd, INT32 DataLength,
                       volatile PCDEVICES pDevice)
{
  USIGN16 wr_xmt_fifo;
  USBXMT_ITEM volatile *pXmtItem;
  T_CANUSBP_SHARED_MEM *pShMem = (T_CANUSBP_SHARED_MEM *)pDevice->canprousb.pSharedMem;
  
  _TRACE5("CANDR_send_remote(%09u)\n", pDevice->Serialnumber);
  
  if(!pDevice->canprousb.transferStarted)
  {
    return FRW_ERROR_COMMUNICATIONBROKEN;
  }
  
  wr_xmt_fifo = pShMem->can_pro_usb_fifo_mode.xmt.wr_xmt_fifo;
  
  if (pShMem->can_pro_usb_fifo_mode.xmt.rd_xmt_fifo != wr_xmt_fifo)
  {
    pXmtItem = (USBXMT_ITEM volatile *)&pShMem->can_pro_usb_fifo_mode.xmt.xmt_fifo[wr_xmt_fifo];
    
    pXmtItem->can_cmd = XMT_CMD_RMT;
    pXmtItem->can_id = Ident;
    pXmtItem->can_len = DataLength;
    pXmtItem->xtd = xtd;
    pXmtItem->u32Counter = counter++;
    memset((PVOID)pXmtItem->data, 0, 8);
    
    pShMem->can_pro_usb_fifo_mode.xmt.wr_xmt_fifo = (wr_xmt_fifo+1)&XMT_FIFO_USB_MASK;
    
  }
  else
  {
    return -1;
  }
  
  if(pDevice->canprousb.IoChainXmtState == CHAIN_IDLE)
  {
    _TRACE5("CANDR_send_remote(%09u) restart xmt\n", pDevice->Serialnumber);
    XmtNextPacket(pDevice);
  }
  
  return 0;

}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANPC_get_time                            */
/* FUNCTIONAL_DESCRIPTION         reads 32 bit time from CANcard            */
/* INPUTS                         *time:      time in us                    */
/* OUTPUTS                        0     INT32: no error                       */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_get_time(PUSIGN32 time, volatile PCDEVICES pDevice)
{
  INT32 ret = 0;

  if(pDevice)
  {
    if(pDevice->canprousb.pSharedMem)
    {
      *time = pDevice->canprousb.pSharedMem->time;
    }
  }

  return(ret);
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_reset_xmt_fifo                      */
/* FUNCTIONAL_DESCRIPTION         resets transmit fifo                      */
/* INPUTS                         -                                         */
/* OUTPUTS                        0     INT32: no error                       */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_reset_xmt_fifo(volatile PCDEVICES pDevice)
{
  DFCT_STRUCT cmd_in;
  INT32 ret = VCARD_OK;
  T_CANUSBP_SHARED_MEM *pShM = (T_CANUSBP_SHARED_MEM *)pDevice->canprousb.pSharedMem;
  INT32 i;
    
  _TRACE5("CANDR_reset_xmt_fifo(%09u)\n", pDevice->Serialnumber);
    
    // disable xmt message processing
//     ret = ControlVcardDevice(pDevice, pPRT, VCARD_CONTROL_DISABLE_XMT);
//     if(ret) return ret;
    
  // wait for xmt idle
  for(i=0; i<100; i++)
  {
    if(pDevice->canprousb.IoChainXmtState != CHAIN_IDLE) 
    {
      mdelay(1);
    }
    else
    {
      break;
    }
  }
  
  if(pDevice->canprousb.IoChainXmtState != CHAIN_IDLE)
    return CANL2_FW_TIMEOUT;
  
    // reset the firmware side fifo
  cmd_in.wRequest = FCT_RESET_XMT_FIFO;
  cmd_in.wFRWReqRes = FALSE;
  ret = send_command(pDevice, (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), (BYTE*)&cmd_in, sizeof(DFCT_STRUCT), 500);
  if(ret) return ret;
  
  // reset the host side fifo
  pShM->can_pro_usb_fifo_mode.xmt.rd_xmt_fifo = (pShM->can_pro_usb_fifo_mode.xmt.wr_xmt_fifo-1) & RCV_FIFO_USB_MASK;
  
  pDevice->canprousb.FirmwareXmtFifoLevel = 255;
  
  
  return ret;
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_read_rcv_fifo_level                 */
/* FUNCTIONAL_DESCRIPTION         returns items in receive fifo             */
/* INPUTS                         -                                         */
/* OUTPUTS                        items in receive fifo                     */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_read_rcv_fifo_level(volatile PCDEVICES pDevice)
{
  INT32 fifo_level;
  T_CANUSBP_SHARED_MEM *pShMem = (T_CANUSBP_SHARED_MEM *)pDevice->canprousb.pSharedMem;
  volatile DPRAM_CAN_PRO_USB_FIFO_MODE * pFifo = &pShMem->can_pro_usb_fifo_mode;
  
  _TRACE5("CANDR_read_rcv_fifo_level(%09u)\n", pDevice->Serialnumber);
  
  fifo_level = (pFifo->rcv.wr_rcv_fifo - pFifo->rcv.rd_rcv_fifo - 1) & RCV_FIFO_USB_MASK;
  
  return fifo_level;
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_read_xmt_fifo_level                 */
/* FUNCTIONAL_DESCRIPTION         returns items in transmit fifo            */
/* INPUTS                         -                                         */
/* OUTPUTS                        items in transmit fifo                    */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_read_xmt_fifo_level(volatile PCDEVICES pDevice)
{
  INT32 fifo_level;
  T_CANUSBP_SHARED_MEM *pShMem = (T_CANUSBP_SHARED_MEM *)pDevice->canprousb.pSharedMem;
  volatile DPRAM_CAN_PRO_USB_FIFO_MODE * pFifo = &pShMem->can_pro_usb_fifo_mode;
  
  _TRACE5("CANDR_read_xmt_fifo_level(%09u)\n", pDevice->Serialnumber);
  
  fifo_level = ((pFifo->xmt.wr_xmt_fifo - pFifo->xmt.rd_xmt_fifo - 1) & XMT_FIFO_USB_MASK) + (XMT_FW_USB_ITEMS-pDevice->canprousb.FirmwareXmtFifoLevel);
  
  return fifo_level;
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANDR_reset_lost_msg_counter              */
/* FUNCTIONAL_DESCRIPTION         resets lost msg counter                   */
/* INPUTS                         -                                         */
/* OUTPUTS                        0     INT32: no error                       */
/* CHANGES                        -                                         */
/*                                                                          */
/*HEAD_END*******************************************************************/
int_ CANDR_reset_lost_msg_counter(volatile PCDEVICES pDevice)
{
    _TRACE5("CANDR_reset_lost_msg_counter(%09u)\n", pDevice->Serialnumber);
    
    pDevice->canprousb.pSharedMem->can_pro_usb_fifo_mode.rcv.lost_msg = 0;
//     channel[Can].dpram_data.can_pro_fifo_mode.prcv_fifo->lost_msg = 0;
    
    return (0);
}









#endif //not EMU
