/*
 * Softing CANpro USB Driver
 * Copyright (C) 1994-2013, Softing AG, Werner Knorsch
 * 
 * See file CREDITS for list of people who contributed to this
 * project.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * email: support.automation@softing.com
 * Softing AG, Richard-Reitzner-Allee 6, 85540 Haar, Germany
 */

/*
 * $Id: can_pci.c,v 1.3 2003/10/31 15:16:37 kno Exp $
 */
/******************************************************************
 * *  include files
 *******************************************************************/

#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/version.h>
#include <linux/init.h>
#include <linux/fs.h>           /* everything... */

#include <linux/cdev.h>


#include <linux/sched.h>



#include <linux/kernel.h>       /* printk() */
#include <linux/errno.h>        /* error codes */
#include <linux/delay.h>        /* udelay */
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/mman.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <linux/sched.h>

#include <linux/pci.h>

#include <linux/netdevice.h>
#include <linux/usb.h>
#include <linux/spinlock.h>
#include <linux/kthread.h>

#include <asm/io.h>

#include "BaseTypes.h"

#include "sysdep.h"

/* candriver header files */


#include "can_device.h"
#include "os.h"
#include "shared_usb_mem.h"
#include "card_def.h"
#include "canprousb.h"


#include "can_def.h"

#include "can_loadcard.h"


#include "can_card.h"

#include "canL2.h"
#include "can_ioctl.h"
#include "can_proc.h"


// canprousb endpoint type
#define BULK_TRANSFER_USED

#ifdef BULK_TRANSFER_USED
#define CANUSBP_ENDPOINT_TYPE USB_ENDPOINT_TYPE_BULK
#define CANUSBP_DAT_IN_ENDPOINT    0x81
#define CANUSBP_DAT_OUT_ENDPOINT   0x01
#define CANUSBP_CMD_IN_ENDPOINT    0x82
#define CANUSBP_CMD_OUT_ENDPOINT   0x02
#define CANUSBP_ENDPOINT1_HS_SIZE  0x200
#define CANUSBP_ENDPOINT2_HS_SIZE  0x200
#define CANUSBP_ENDPOINT1_FS_SIZE  0x40
#define CANUSBP_ENDPOINT2_FS_SIZE  0x40
#else
#define CANUSBP_ENDPOINT_TYPE USB_ENDPOINT_TYPE_INTERRUPT
#define CANUSBP_ENDPOINT1_HS_SIZE  0x1200
#define CANUSBP_ENDPOINT2_HS_SIZE  0x200
#define CANUSBP_ENDPOINT1_FS_SIZE  0x40
#define CANUSBP_ENDPOINT2_FS_SIZE  0x40
#endif

#ifdef CONFIG_USB_DYNAMIC_MINORS
#define USB_CAN_MINOR_BASE       0
#else
#define USB_CAN_MINOR_BASE       176
#endif



/*****************************************************************
**  externals
******************************************************************/
extern struct file_operations can_fops;
extern struct semaphore cDeviceListSemaphore;
extern struct cdev *driver_object;

extern unsigned long layer2UsbBytes;
extern unsigned char layer2Usb[];
  

/******************************************************************
**  variables
*******************************************************************/

extern int can_trace_level;

/*===========================================================================*/
int canprousb_probe(struct usb_interface *intf, const struct usb_device_id *id);
void canprousb_disconnect(struct usb_interface *intf);
PCDEVICES FindDevice(CAN_ADDRESS Dpram);
static void on_complete_rcv(struct urb *urb);
static void on_complete_xmt(struct urb *urb);
static void check_dyn_rcv_object_fifo(PCDEVICES pDevice, USBRCVITEM *pfifo_entry);
static void check_stat_rcv_object_fifo(PCDEVICES pDevice, USBRCVITEM *pfifo_entry);
static int canprousb_suspend(struct usb_interface *intf, pm_message_t msg);
static int canprousb_resume(struct usb_interface *intf);

#define SOFTING_USB_VENDOR 0x077E
#define CANPROUSB          0x000F

static struct usb_device_id canprousb_ids [] = {
         { USB_DEVICE(SOFTING_USB_VENDOR, CANPROUSB) },
         { }                                             /* Terminating entry */
 };
 
 /*
  * usb class driver info in order to get a minor number from the usb core,
  * and to have the device registered with the driver core
  */
 char *canprousb_name(struct device *dev, umode_t mode);
 
 static struct usb_class_driver canpro_usb_class = 
 {
   .name =         "canpro_usb_%d",
   //.devnode =      canprousb_name,
   .fops =         &can_fops,
   .minor_base =   USB_CAN_MINOR_BASE,
 };
 
 
MODULE_DEVICE_TABLE (usb, canprousb_ids);

static struct usb_driver canprousb_driver = 
{
  name: "canpro_usb",
  probe: canprousb_probe, 
  disconnect: canprousb_disconnect, 
  suspend: canprousb_suspend,
  resume: canprousb_resume,
 // fops: &can_fops,
  id_table: canprousb_ids,
};



int major = 0;
static int cmd_count = 0; // will be incremented on every command over the USB; this counter is part of the command.


/******************************************************************
**  macros
*******************************************************************/


MODULE_AUTHOR("Werner Knorsch");
MODULE_DESCRIPTION("Softing CANpro USB driver");

MODULE_VERSION(CAN_DRIVER_VERSION);
MODULE_LICENSE("GPL");


/****************************************************************************
**  prototypes
****************************************************************************/

PCDEVICES SecureFindDevice(CAN_ADDRESS u32Dpram);
PCDEVICES InsertDevice(PCDEVICES pNewDevice);
VOID RemoveDevice(CAN_ADDRESS Dpram);
INT32 FreeResources(PCDEVICES pDevice);


/*****************************************************************/
/*                                                               */
/* CRC LOOKUP TABLE                                              */
/* ================                                              */
/* The following CRC lookup table was generated automagically    */
/* by the Rocksoft^tm Model CRC Algorithm Table Generation       */
/* Program V1.0 using the following model parameters:            */
/*                                                               */
/*    Width   : 2 bytes.                                         */
/*    Poly    : 0x1021                                           */
/*    Reverse : FALSE.                                           */
/*                                                               */
/* For more information on the Rocksoft^tm Model CRC Algorithm,  */
/* see the document titled "A Painless Guide to CRC Error        */
/* Detection Algorithms" by Ross Williams                        */
/* (ross@guest.adelaide.edu.au.). This document is likely to be  */
/* in the FTP archive "ftp.adelaide.edu.au/pub/rocksoft".        */
/*                                                               */
/*****************************************************************/

USIGN16 crctable[256] =
{
  0x0000, 0x1021, 0x2042, 0x3063, 0x4084, 0x50A5, 0x60C6, 0x70E7,
  0x8108, 0x9129, 0xA14A, 0xB16B, 0xC18C, 0xD1AD, 0xE1CE, 0xF1EF,
  0x1231, 0x0210, 0x3273, 0x2252, 0x52B5, 0x4294, 0x72F7, 0x62D6,
  0x9339, 0x8318, 0xB37B, 0xA35A, 0xD3BD, 0xC39C, 0xF3FF, 0xE3DE,
  0x2462, 0x3443, 0x0420, 0x1401, 0x64E6, 0x74C7, 0x44A4, 0x5485,
  0xA56A, 0xB54B, 0x8528, 0x9509, 0xE5EE, 0xF5CF, 0xC5AC, 0xD58D,
  0x3653, 0x2672, 0x1611, 0x0630, 0x76D7, 0x66F6, 0x5695, 0x46B4,
  0xB75B, 0xA77A, 0x9719, 0x8738, 0xF7DF, 0xE7FE, 0xD79D, 0xC7BC,
  0x48C4, 0x58E5, 0x6886, 0x78A7, 0x0840, 0x1861, 0x2802, 0x3823,
  0xC9CC, 0xD9ED, 0xE98E, 0xF9AF, 0x8948, 0x9969, 0xA90A, 0xB92B,
  0x5AF5, 0x4AD4, 0x7AB7, 0x6A96, 0x1A71, 0x0A50, 0x3A33, 0x2A12,
  0xDBFD, 0xCBDC, 0xFBBF, 0xEB9E, 0x9B79, 0x8B58, 0xBB3B, 0xAB1A,
  0x6CA6, 0x7C87, 0x4CE4, 0x5CC5, 0x2C22, 0x3C03, 0x0C60, 0x1C41,
  0xEDAE, 0xFD8F, 0xCDEC, 0xDDCD, 0xAD2A, 0xBD0B, 0x8D68, 0x9D49,
  0x7E97, 0x6EB6, 0x5ED5, 0x4EF4, 0x3E13, 0x2E32, 0x1E51, 0x0E70,
  0xFF9F, 0xEFBE, 0xDFDD, 0xCFFC, 0xBF1B, 0xAF3A, 0x9F59, 0x8F78,
  0x9188, 0x81A9, 0xB1CA, 0xA1EB, 0xD10C, 0xC12D, 0xF14E, 0xE16F,
  0x1080, 0x00A1, 0x30C2, 0x20E3, 0x5004, 0x4025, 0x7046, 0x6067,
  0x83B9, 0x9398, 0xA3FB, 0xB3DA, 0xC33D, 0xD31C, 0xE37F, 0xF35E,
  0x02B1, 0x1290, 0x22F3, 0x32D2, 0x4235, 0x5214, 0x6277, 0x7256,
  0xB5EA, 0xA5CB, 0x95A8, 0x8589, 0xF56E, 0xE54F, 0xD52C, 0xC50D,
  0x34E2, 0x24C3, 0x14A0, 0x0481, 0x7466, 0x6447, 0x5424, 0x4405,
  0xA7DB, 0xB7FA, 0x8799, 0x97B8, 0xE75F, 0xF77E, 0xC71D, 0xD73C,
  0x26D3, 0x36F2, 0x0691, 0x16B0, 0x6657, 0x7676, 0x4615, 0x5634,
  0xD94C, 0xC96D, 0xF90E, 0xE92F, 0x99C8, 0x89E9, 0xB98A, 0xA9AB,
  0x5844, 0x4865, 0x7806, 0x6827, 0x18C0, 0x08E1, 0x3882, 0x28A3,
  0xCB7D, 0xDB5C, 0xEB3F, 0xFB1E, 0x8BF9, 0x9BD8, 0xABBB, 0xBB9A,
  0x4A75, 0x5A54, 0x6A37, 0x7A16, 0x0AF1, 0x1AD0, 0x2AB3, 0x3A92,
  0xFD2E, 0xED0F, 0xDD6C, 0xCD4D, 0xBDAA, 0xAD8B, 0x9DE8, 0x8DC9,
  0x7C26, 0x6C07, 0x5C64, 0x4C45, 0x3CA2, 0x2C83, 0x1CE0, 0x0CC1,
  0xEF1F, 0xFF3E, 0xCF5D, 0xDF7C, 0xAF9B, 0xBFBA, 0x8FD9, 0x9FF8,
  0x6E17, 0x7E36, 0x4E55, 0x5E74, 0x2E93, 0x3EB2, 0x0ED1, 0x1EF0
};


PCDEVICES GetDeviceFromMinorNumber(USIGN32 u32MinorNumber);

/*****************************************************************************/
/* DumpASCII and DumpMem                                                     */
/*****************************************************************************/
/* FUNCTIONAL DESCRIPTION                                                    */
/*   functions for debugging to print a memory region into the logfile       */
/*                                                                           */
/*****************************************************************************/
/* PARAMETER DECRIPTION                                                      */
/*                                                                           */
/*   pChar : pointer to the memory                                           */
/*   size  : size of the memory region                                       */
/*   offset: offset inside the memory                                        */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/

VOID DumpASCII (PBYTE pChar, USIGN32 size, USIGN32 offset)
{
  USIGN32 i;
  
  if(offset < size)
    return;
  
  for(i=0; i<size; i++)
  {
    if((pChar[i] >= ' ') && (pChar[i] <= 127))
    {
      printk("%c",pChar[i]);
    }
    else
    {
      printk(".");
    }
    
  }
  
}

/**
 * *****************************************************************
 * @brief  dump memory  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pMem:  pointer to the memory, which should be dumped
 * @param  size:  size of the memory
 * @param  pText: headline of the dump output
 * @return 0, if successful, an error code otherwise
 * 
 *
 ******************************************************************
 */

void DumpMem(void *pMem, USIGN32 size, INT8 *pText)
{
  USIGN32 i, lineLength = 0x10;
  USIGN8 *pVal = pMem;
  
  printk("************************* %s *******************************\n",pText);
  printk("length=%u\n",size);
  
  for(i=0; i<size; i++)
  {
    if(!(i%lineLength))
    {
      DumpASCII(&pVal[i-lineLength], lineLength, i);
      printk("\n0x%08lx: ",i+(CAN_ADDRESS)pMem);
    }
    
    printk("%02x ",pVal[i]);
    
  }
  printk("\n");
  
}

char *canprousb_name(struct device *dev, umode_t mode)
{
  return "canprousb";
}

/**
 * *****************************************************************
 * @brief  enter a critical section  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  access:   spinlock to guarantee atomic operation
 * @return IrqFlags: must be used on sync_free_access()
 * 
 *
 ******************************************************************
 */

CAN_ADDRESS sync_get_access(spinlock_t *access)
{
  CAN_ADDRESS IrqFlags;
  spin_lock_irqsave(access, IrqFlags);
  return IrqFlags;
}

/**
 * *****************************************************************
 * @brief  leave a critical section  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  access:   spinlock to guarantee atomic operation
 * @param  IrqFlags: data which is needed for leaving the critical section
 * @return nothing
 * 
 *
 ******************************************************************
 */
void sync_free_access(spinlock_t *access, CAN_ADDRESS IrqFlags)
{
  spin_unlock_irqrestore(access, IrqFlags);
}

/**
 * *****************************************************************
 * @brief  send a command over the command pipe and wait for the response.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pDevice:    device data
 * @param  pOutBuffer: command data, which will be sent.
 * @param  OutBufSize: size of the pOutBuffer
 * @param  pInBuffer:  buffer which holds the answer of the CANpro USB on the command.
 * @param  InBufSize:  size of pInBuffer
 * @param  timeout:    timeout for the transfer
 * @return error code
 *
 ******************************************************************
 */
int send_command(PCDEVICES pDevice, void *pOutBuffer, int OutBufSize, void *pInBuffer, int InBufSize, int timeout)
{
  int status;
  struct usb_device *usb_dev = interface_to_usbdev(pDevice->canprousb.device.intf);
  int len;
	
	// sending the Command on the OUT Command pipe
  
  memcpy(pDevice->canprousb.device.pcmd_out_buf, pOutBuffer, OutBufSize);
  
  status = usb_bulk_msg(usb_dev, usb_sndbulkpipe(usb_dev, CANUSBP_CMD_OUT_ENDPOINT), pDevice->canprousb.device.pcmd_out_buf, OutBufSize, &len, timeout);
  
  if(status)
  {
    _TRACE5("sending over cmd OUT pipe returned error %u\n", status);
    return status;
  }
  
  if(len != OutBufSize)
  {
    _TRACE5("sending over cmd OUT pipe: only %u bytes from %u bytes sent\n",len,OutBufSize);
    return -1;
  }
  
  // getting the response on the command - using the IN response Pipe
  
  status = usb_bulk_msg(usb_dev, usb_rcvbulkpipe(usb_dev, CANUSBP_CMD_IN_ENDPOINT), pDevice->canprousb.device.pcmd_in_buf, InBufSize, &len, timeout);
  
  if(status)
  {
    _TRACE5("sending over cmd IN pipe returned error %u\n", status);
    return status;
  }
  
  memcpy(pInBuffer, pDevice->canprousb.device.pcmd_in_buf, InBufSize);
  
  if(len != InBufSize)
  {
    _TRACE5("receiving from cmd IN pipe: only %u bytes from %u bytes received\n",len,InBufSize);
    return -1;
  }
  
  _TRACE5("command successfully sent\n");
  return 0;
}


/*****************************************************************************/
/* CanResetCard                                                              */
/*****************************************************************************/
/* FUNCTIONAL DESCRIPTION                                                    */
/*   halts and starts the PCI CAN device                                     */
/*   returns 0 if successful, else -1                                        */
/*****************************************************************************/
/* PARAMETER DECRIPTION                                                      */
/*    BOOL bDoReset    : ==0 => start, == 1 => halt                          */
/*    PCDEVICES pDevice: pointer to the device                               */
/*                                                                           */
/*****************************************************************************/

/**
 * *****************************************************************
 * @brief  should generate a reset on other boards, on CANpro USB the firmware goes back to the bootblock  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  bDoReset: if true, then jump back to the bootblock
 * @param  pDevice: device data
 * @return 0, if successful, an error code otherwise
 * 
 *
 ******************************************************************
 */

// Reset
USIGN16 CanResetCard(BOOL bDoReset, PVOID pDevice)
{
  _TRACE1("CanResetCard\n");
  if(bDoReset)
  {
    StopFirmware(pDevice);
  }
  return 0;
}

/**
 * *****************************************************************
 * @brief  not used for CANpro USB  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  bDoEnable: 
 * @param  pDevice: 
 * @return 0, if successful, an error code otherwise
 * 
 *
 ******************************************************************
 */

USIGN16 CanEnableIrq(BOOL bDoEnable, PVOID pDevice)
{
  return 0;
}

/**
 * *****************************************************************
 * @brief  calculates the crc check sum  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pval:    address of the buffer for which the crc code should be calculated
 * @param  blk_len: size of the buffer for which the crc code should be calculated
 * @return crc check sum
 *
 ******************************************************************
 */

#define INIT 0
#define XOROUT 0

USIGN16 calcCRC16r (USIGN8 *blk_adr,USIGN32 blk_len)
{
  USIGN32 crc = INIT;
  while (blk_len--)
    crc = crctable[((crc>>8) ^ *blk_adr++) & 0xFFL] ^ (crc << 8);
  return (USIGN16) ((crc ^ XOROUT) & 0xFFFF);
}

/*
#warning unused function ?
INT32 canpro_initialize_chip(INT32 presc, INT32 sjw, INT32 tseg1, INT32 tseg2, INT32 sam,
                      volatile PCDEVICES pDevice)
{
  
  return 0;
}
*/

/**
 * *****************************************************************
 * @brief  sends a control request.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pDevice:    device data
 * @param  request: request to send.
 * @param  flags:   flags
 * @param  pBuffer: buffer which holds the answer of the CANpro USB on the command.
 * @param  bufSize: size of pBuffer
 * @return error code
 *
 ******************************************************************
 */

static int ReceiveCtrlData (struct usb_device *dev, USIGN8 request, USIGN32 flags, BYTE *pBuffer, USIGN32 bufSize)
{
  int retval;
  
  retval = usb_control_msg(
    dev,
    usb_rcvctrlpipe(dev, 0),
                           request,
                           flags | USB_DIR_IN,
                           0, //value
                           0, //index
                           pBuffer,
                           bufSize,
                           100); //ms
  
  if(retval==bufSize)
  {
    retval = 0;
  }
  
  return retval;
} 

/**
 * *****************************************************************
 * @brief  frees the allocated resources.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pcDevice:  device data
 * @return error code
 *
 ******************************************************************
 */

void freeRes(PCDEVICES pcDevice)
{
  struct usb_device *udev = interface_to_usbdev(pcDevice->canprousb.device.intf);
  
  if(pcDevice->canprousb.device.pcmd_in_buf)
  {
    usb_free_coherent(udev, 
                      sizeof(FW_PACKET), 
                      pcDevice->canprousb.device.pcmd_in_buf, 
                      pcDevice->canprousb.device.cmd_in_urb->transfer_dma);
    
    pcDevice->canprousb.device.pcmd_in_buf = NULL;
  }
  
  if(pcDevice->canprousb.device.pcmd_out_buf)
  {
    usb_free_coherent(udev, 
                      sizeof(FW_PACKET), 
                      pcDevice->canprousb.device.pcmd_out_buf, 
                      pcDevice->canprousb.device.cmd_out_urb->transfer_dma);
    
    pcDevice->canprousb.device.pcmd_out_buf = NULL;
  }
  
  if(pcDevice->canprousb.device.pdat_in_buf)
  {
    usb_free_coherent(udev, 
                      MAX_BULK_SIZE, 
                      pcDevice->canprousb.device.pdat_in_buf, 
                      pcDevice->canprousb.device.dat_in_urb->transfer_dma);
    
    pcDevice->canprousb.device.pdat_in_buf = NULL;
  }
  
  if(pcDevice->canprousb.device.pdat_out_buf)
  {
    usb_free_coherent(udev, 
                      MAX_BULK_SIZE, 
                      pcDevice->canprousb.device.pdat_out_buf, 
                      pcDevice->canprousb.device.dat_out_urb->transfer_dma);
    
    pcDevice->canprousb.device.pdat_out_buf = NULL;
  }
  
  if(pcDevice->canprousb.device.cmd_in_urb) usb_free_urb(pcDevice->canprousb.device.cmd_in_urb);
  pcDevice->canprousb.device.cmd_in_urb = NULL;
  if(pcDevice->canprousb.device.cmd_out_urb) usb_free_urb(pcDevice->canprousb.device.cmd_out_urb);
  pcDevice->canprousb.device.cmd_out_urb = NULL;
  if(pcDevice->canprousb.device.dat_in_urb) usb_free_urb(pcDevice->canprousb.device.dat_in_urb);
  pcDevice->canprousb.device.dat_in_urb = NULL;
  if(pcDevice->canprousb.device.dat_out_urb) usb_free_urb(pcDevice->canprousb.device.dat_out_urb);
  pcDevice->canprousb.device.dat_out_urb = NULL;
  
  if(pcDevice->canprousb.pSharedMem) kfree(pcDevice->canprousb.pSharedMem);
  pcDevice->canprousb.pSharedMem = NULL;
  
}

/**
 * *****************************************************************
 * @brief  atomic increment  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pval:     pointer to the result of the operation
 * @param  sum:      summand 
 * @param  Spinlock: spinlock to guarantee an atomic operation 
 * @return void
 *
 ******************************************************************
 */

inline void InterlockedIncrement(volatile INT32 volatile *pval, spinlock_t *Spinlock)
{
  unsigned long IrqFlags;
  spin_lock_irqsave(Spinlock, IrqFlags);
  *pval += 1;
//   printk("InterlockedIncrement: level=%u\n",*pval);
  spin_unlock_irqrestore(Spinlock, IrqFlags);
}

/**
 * *****************************************************************
 * @brief  atomic addition  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pval:     pointer to the result of the operation
 * @param  sum:      summand 
 * @param  Spinlock: spinlock to guarantee an atomic operation 
 * @return void
 *
 ******************************************************************
 */

inline void InterlockedExchangeAdd(volatile INT32 volatile *pval, INT32 sum, spinlock_t *Spinlock)
{
  unsigned long IrqFlags;
  spin_lock_irqsave(Spinlock, IrqFlags);
  *pval += sum;
  spin_unlock_irqrestore(Spinlock, IrqFlags);
  
}

#ifndef CONFIG_X86_64
#warning "32 Bit System __udivdi3 defined by module!"
USIGN64 Div64(USIGN64 divident, USIGN64 divisor)
{
  USIGN64 u64divi = divident;
  USIGN64 u64help2 = divident;
  USIGN64 u64divisor = divisor;
  USIGN64 ret = 0;
  USIGN64 u64help;
  int i;
  
  i=0;
  
  if(divisor==0)
    return 0;
  
  while(u64divi >= (2*divisor))
  {
    i++;
    u64divi = u64divi>>1;
  }
  
  do
  {
    u64divi = u64help2;
    
    u64help = u64divisor << i;
    u64divi = u64divi - u64help;
    
    u64help2 = u64divi;
    
    
    ret = (ret<<1) | 1; 
    
    i--;
    
    u64help = u64divi>>i;
    
    while((u64help < divisor) && (i>0))
    {
      i--;
      u64help = u64divi>>i;
      ret = ret << 1;
    } 
    
    if(i==0)
    {
      ret = (u64help >= divisor) ? (ret<<1) | 1 : ret<<1;     
    }
    
  } while (i > 0);
  
  
  return ret;
  
}
#endif

/**
 * *****************************************************************
 * @brief  calculates the timestamp in ms from a received message buffer.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pMesg:  pointer to the message buffer
 * @return timestamp in us.
 *
 ******************************************************************
 */

USIGN32 MkTimestamp(volatile USBRCVITEM *pMesg)
{
  u64 time = pMesg->u32TimeLow;
  u64 ret = 0;
  USIGN32 hightime = pMesg->u32TimeHigh % 3; // wrong after 5.95 years
  
  switch(hightime)
  {
    case 0:
      ret = (time*2+1)/3;
      break;
      
    case 1:
      ret = (time*2+1)/3 + 0xAAAAAAAB;
      break;
      
    case 2:
      ret = (time*2+1)/3 + 0x55555555;
      break;
  }
/*
switch(hightime)
{
  case 0:
    ret = u64Div((time*2+1),3);
    break;
    
  case 1:
    ret = u64Div((time*2+1),3) + 0xAAAAAAAB;
    break;
    
  case 2:
    ret = u64Div((time*2+1),3) + 0x55555555;
    break;
}
*/
  
  return (USIGN32)ret;
  
}

/**
 * *****************************************************************
 * @brief  copies the received message buffer into the receive FIFO.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pdx:  pointer to the device data (device extension)
 * @param  buffer:  received message buffer
 * @param  size:  number of messages inside the buffer
 * @return true, if something was copied.
 *
 ******************************************************************
 */
BOOL copyReceiveDataBuffer(CAN_PRO_USB *pdx, volatile USBRCVITEM *buffer, unsigned long size)
{
  USIGN16 wr1 = pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.wr_rcv_fifo;
  USIGN16 rd;
  USIGN16 wr2;
  USIGN32 lost;
  BOOL ret = FALSE;
  
  for(wr2=0; wr2<size; wr2++)
  {
    if(buffer[wr2].u8Cmd!=TIMER_CMD)
    { 
      // not a timestamp
      unsigned char cmd = buffer[wr2].u8Cmd;
      
      _TRACE3("copyReceiveDataBuffer: cmd 0x%02x received!\n", (unsigned int)(cmd&0xFF));
      if(buffer[wr2].u8Cmd & CMD_ACK_MASK)
      { 
        // ack received ==> 1 message successfully sent
        InterlockedIncrement(&pdx->FirmwareXmtFifoLevel, &pdx->Spinlock);
        buffer[wr2].u8Cmd &= ~CMD_ACK_MASK;
      }
      
      if(pdx->RcvCounter != buffer[wr2].counter)
      {        
        _TRACE3("CANPROUSB copyReceiveDataBuffer: test2 cmd:0x%x lost msg 0x%x got msg: 0x%x entry: %u size: %lu\n", buffer[wr2].u8Cmd, pdx->RcvCounter, buffer[wr2].counter, wr2, size);
        _TRACE3("CANPROUSB copyReceiveDataBuffer: test2 id 0x%x\n", buffer[wr2].u32Ident);
        pdx->pSharedMem->lostMsgOnHostFifo = (pdx->pSharedMem->lostMsgOnHostFifo<0xFFFF) ? pdx->pSharedMem->lostMsgOnHostFifo++ : pdx->pSharedMem->lostMsgOnHostFifo;
      }
      pdx->RcvCounter++;
      
      if(pdx->enableXmtAck || !(buffer[wr2].u8Cmd & XMT_ACK_MASK))
      {
        
        if(pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rd_rcv_fifo == wr1)
        { 
          // fifo full
          _TRACE3("copyReceiveDataBuffer: FIFO overrun!\n");
          pdx->pSharedMem->lostMsgOnHostFifo = (pdx->pSharedMem->lostMsgOnHostFifo<0xFFFF) ? pdx->pSharedMem->lostMsgOnHostFifo++ : pdx->pSharedMem->lostMsgOnHostFifo;
          pdx->pSharedMem->time = MkTimestamp(&buffer[wr2]);
          pdx->pSharedMem->can_state = buffer[wr2].u8Busstate;
          lost = (USIGN32)buffer[wr2].u16LostMsg + (USIGN32)pdx->pSharedMem->lostMsgOnHostFifo;
          pdx->pSharedMem->lost_messages = (USIGN16)((lost>0xFFFF) ? 0xFFFF : lost);
          
        }
        else
        {
          pdx->pSharedMem->time = MkTimestamp(&buffer[wr2]);
          pdx->pSharedMem->can_state = buffer[wr2].u8Busstate;
          lost = (USIGN32)buffer[wr2].u16LostMsg + (USIGN32)pdx->pSharedMem->lostMsgOnHostFifo;
          pdx->pSharedMem->lost_messages = (USIGN16)((lost>0xFFFF) ? 0xFFFF : lost);
          
          memcpy((void*)&pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr1],(void*)&buffer[wr2],sizeof(USBRCVITEM));
          
          
          wr1 = (wr1+1) & RCV_FIFO_USB_MASK;
          pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.wr_rcv_fifo = wr1;
          ret = TRUE;
        }
      }
      else
      {
        pdx->pSharedMem->time = MkTimestamp(&buffer[wr2]);
        buffer[wr2].u8Cmd = 0;
        if(pdx->pSharedMem->can_state != buffer[wr2].u8Busstate)
        {
          pdx->pSharedMem->can_state = buffer[wr2].u8Busstate;
          ret = TRUE;
        }
        lost = (USIGN32)buffer[wr2].u16LostMsg + (USIGN32)pdx->pSharedMem->lostMsgOnHostFifo;
        pdx->pSharedMem->lost_messages = (USIGN16)((lost>0xFFFF) ? 0xFFFF : lost);
      }
    }
    else
    {
      pdx->pSharedMem->time = MkTimestamp(&buffer[wr2]);
      buffer[wr2].u8Cmd = 0;
      if(pdx->pSharedMem->can_state != buffer[wr2].u8Busstate)
      {
        pdx->pSharedMem->can_state = buffer[wr2].u8Busstate;
        ret = TRUE;
      }
      lost = (USIGN32)buffer[wr2].u16LostMsg + (USIGN32)pdx->pSharedMem->lostMsgOnHostFifo;
      pdx->pSharedMem->lost_messages = (USIGN16)((lost>0xFFFF) ? 0xFFFF : lost);
      
    }
    
  }
  
  
  rd = (pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rd_rcv_fifo + 1) & RCV_FIFO_USB_MASK;
    
  return ret;
}

/**
 * *****************************************************************
 * @brief  remove the timestamp messages from the received packet and copy the messages,  
 * @brief  if neccessary, and adjust the write pointer of the receive FIFO   
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pSharedMem: pointer to the structure which contains the receive FIFO
 * @param  size:       size of the received message data 
 * @return void
 *
 ******************************************************************
 */
// increment the write pointer of the RCV FIFO
BOOL setReceiveDataBuffer(CAN_PRO_USB *pdx, unsigned long size)
{
  unsigned int i;
  volatile USIGN16 wr1 = pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.wr_rcv_fifo;
  volatile USIGN16 wr2 = wr1;
  USIGN16 rd;
  static int test=0;
  USIGN32 lost;
  BOOL ret = FALSE;
  
  for(i=0; i<size; i++)
  {
    
    if(pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u8Cmd!=TIMER_CMD)
    {
      // not a timestamp
      //KdPrint(("CANPROUSB setReceiveDataBuffer: cmd: 0x%x\n", pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u8Cmd));
      pdx->pSharedMem->time = MkTimestamp(&pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2]);
      pdx->pSharedMem->can_state = pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u8Busstate;
      pdx->pSharedMem->lost_messages = pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u16LostMsg;
      lost = (USIGN32)pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u16LostMsg + (USIGN32)pdx->pSharedMem->lostMsgOnHostFifo;
      pdx->pSharedMem->lost_messages = (USIGN16)((lost>0xFFFF) ? 0xFFFF : lost);
      
      
      if(pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u8Cmd & CMD_ACK_MASK)
      { 
        // ack received ==> 1 message successfully sent
        if(pdx->FirmwareXmtFifoLevel < XMT_FW_USB_ITEMS)
        {
          InterlockedIncrement(&pdx->FirmwareXmtFifoLevel, &pdx->Spinlock);
        }
        pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u8Cmd &= ~CMD_ACK_MASK;
        //KdPrint(("CANPROUSB setReceiveDataBuffer: cmd ack: 0x%x\n", pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u8Cmd));
      }
      
      if(pdx->RcvCounter != pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].counter)
      {
        _TRACE3("CANPROUSB setReceiveDataBuffer: lost msg cnt_soll:%u cnt_ist:%u cnt_last:%u lastCmd:0x%x\n", 
               pdx->RcvCounter, 
               pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].counter,
               pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[(wr2-1)&RCV_FIFO_USB_MASK].counter,
               pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[(wr2-1)&RCV_FIFO_USB_MASK].u8Cmd
        );
        pdx->pSharedMem->lostMsgOnHostFifo = (pdx->pSharedMem->lostMsgOnHostFifo<0xFFFF) ? pdx->pSharedMem->lostMsgOnHostFifo++ : pdx->pSharedMem->lostMsgOnHostFifo;
      }
      pdx->RcvCounter++;
      
      
      if(pdx->enableXmtAck || !(pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u8Cmd & XMT_ACK_MASK))
      {
        
        if(pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u8Cmd == 0)
        {
          test++;
        }
                
        ret = TRUE;
        if(wr1 != wr2)
        {
          memcpy((void*)&pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr1],(void*)&pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2],sizeof(USBRCVITEM));
        }
        wr1 = (wr1+1) & RCV_FIFO_USB_MASK;
        wr2 = (wr2+1) & RCV_FIFO_USB_MASK;
      }
      else
      {
        
        pdx->pSharedMem->time = MkTimestamp(&pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2]);
        pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u8Cmd = 0;
        if(pdx->pSharedMem->can_state != pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u8Busstate)
        {
          pdx->pSharedMem->can_state = pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u8Busstate;
          ret = TRUE;
        }
        lost = (USIGN32)pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u16LostMsg + (USIGN32)pdx->pSharedMem->lostMsgOnHostFifo;
        pdx->pSharedMem->lost_messages = (USIGN16)((lost>0xFFFF) ? 0xFFFF : lost);
        
        wr2 = (wr2+1) & RCV_FIFO_USB_MASK;
      }
    }
    else
    {
      pdx->pSharedMem->time = MkTimestamp(&pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2]);
      pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u8Cmd = 0;
      if(pdx->pSharedMem->can_state != pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u8Busstate)
      {
        pdx->pSharedMem->can_state = pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u8Busstate;
        ret = TRUE;
      }
      lost = (USIGN32)pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rcv_fifo[wr2].u16LostMsg + (USIGN32)pdx->pSharedMem->lostMsgOnHostFifo;
      pdx->pSharedMem->lost_messages = (USIGN16)((lost>0xFFFF) ? 0xFFFF : lost);
      wr2 = (wr2+1) & RCV_FIFO_USB_MASK;
    }
    
  }
  
  pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.wr_rcv_fifo = wr1;
  rd = (pdx->pSharedMem->can_pro_usb_fifo_mode.rcv.rd_rcv_fifo + 1) & RCV_FIFO_USB_MASK;
  
  
  
  return ret;
}


/**
 * *****************************************************************
 * @brief  called inside the completion routine, to inform the application about an CAN event,   
 * @brief  and to update the objects in object buffer mode.   
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pDevice:   device data 
 * @return void
 *
 ******************************************************************
 */

VOID InterruptCallback(PCDEVICES pDevice)
{
  if(pDevice->object_buffer_enable)
  {
    T_CANUSBP_SHARED_MEM *pSh = pDevice->canprousb.pSharedMem;
    
    _TRACE3("InterruptCallback: \n");
    if(pDevice->dyn_enable)
    {
      USIGN16 rd;
      
      rd = (pSh->can_pro_usb_fifo_mode.rcv.rd_rcv_fifo+1)&RCV_FIFO_USB_MASK;
      while(rd != pSh->can_pro_usb_fifo_mode.rcv.wr_rcv_fifo)
      {
        check_dyn_rcv_object_fifo(pDevice, (USBRCVITEM*)&pSh->can_pro_usb_fifo_mode.rcv.rcv_fifo[rd]);
        pSh->can_pro_usb_fifo_mode.rcv.rd_rcv_fifo = rd;
        rd = (pSh->can_pro_usb_fifo_mode.rcv.rd_rcv_fifo+1)&RCV_FIFO_USB_MASK;
      }
    }
    else
    {
      USIGN16 rd;
      
      _TRACE3("InterruptCallback: stat. obj. buf.\n");
      rd = (pSh->can_pro_usb_fifo_mode.rcv.rd_rcv_fifo+1)&RCV_FIFO_USB_MASK;
      while(rd != pSh->can_pro_usb_fifo_mode.rcv.wr_rcv_fifo)
      {
        _TRACE3("InterruptCallback: calling check_stat_rcv_object_fifo\n");
        check_stat_rcv_object_fifo(pDevice, (USBRCVITEM*)&pSh->can_pro_usb_fifo_mode.rcv.rcv_fifo[rd]);
        pSh->can_pro_usb_fifo_mode.rcv.rd_rcv_fifo = rd;
        rd = (pSh->can_pro_usb_fifo_mode.rcv.rd_rcv_fifo+1)&RCV_FIFO_USB_MASK;
      }
    }
    if(pDevice->bus_state != pSh->can_state)
    {
      pDevice->can_info.irq_flag = 1;
      wake_up_interruptible(&pDevice->can_info.read_queue);     
    }
  }
  else
  {
    pDevice->can_info.irq_flag = 1;
    wake_up_interruptible(&pDevice->can_info.read_queue);     
  }
  
  return;
}

/**
 * *****************************************************************
 * @brief  start the next IN transfer to receive the next packet from the firmware.   
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pDevice:   device data 
 * @return void
 *
 ******************************************************************
 */

int RcvNextPacket(PCDEVICES pDevice)
{
  int retval;
  struct usb_device *usb_dev = interface_to_usbdev(pDevice->canprousb.device.intf);
  
  pDevice->canprousb.IoChainRcvState = CHAIN_RUNNING;
  
  usb_fill_bulk_urb(pDevice->canprousb.device.dat_in_urb,
                    usb_dev,
                    usb_rcvbulkpipe(usb_dev, CANUSBP_DAT_IN_ENDPOINT), 
                    pDevice->canprousb.device.pdat_in_buf,
                    MAX_BULK_SIZE,
                    on_complete_rcv,
                    pDevice);
  
  retval = usb_submit_urb(pDevice->canprousb.device.dat_in_urb, GFP_ATOMIC);
  
  mod_timer(&pDevice->canprousb.rcv_timer, jiffies + msecs_to_jiffies(1000) );
  
  return retval;
}


/**
 * *****************************************************************
 * @brief  receive completion routine, called after an IN transfer.   
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  urb:   USB request block 
 * @return void
 *
 ******************************************************************
 */
static void on_complete_rcv(struct urb *urb)
{
  static int counter = 0;
  static int packets = 0;
  static int requests = 0;
  PCDEVICES pDevice = (PCDEVICES) urb->context;
  BYTE *pBuffer = NULL;
  int retval;
  
  counter++;
  if(!(counter%20000))
  {
    _TRACE3("%u messages received\n", counter); 
  }
  
  if(urb->status)
  {
    pDevice->canprousb.IoChainRcvState = CHAIN_STOPPED_ON_ERROR;
    _TRACE3("on_complete_rcv: error %u\n", urb->status);
    return;
  }
  
  if(pDevice->canprousb.IoChainRcvState != CHAIN_RUNNING)
  {
    _TRACE3("on_complete_rcv: stop cmd received %u\n", pDevice->canprousb.IoChainRcvState);
    pDevice->canprousb.IoChainRcvState = CHAIN_STOPPED;
    return;
  }
  
  if(urb->actual_length !=  0)
  {
    pBuffer = urb->transfer_buffer;
    
    if(pBuffer)
    {
      USIGN32 numOfMsg = urb->actual_length/32;
      BOOL msg_available = FALSE;
            
      // wrap arround buffer was used!
      if(pDevice->canprousb.rcv_usb_enable)
      {
        msg_available = copyReceiveDataBuffer(&pDevice->canprousb, (USBRCVITEM*)pBuffer, numOfMsg);
        requests--;
      }
      
      if(msg_available)
      {
        pDevice->canprousb.pSharedMem->WrapAroundBuffer.numOfEntries = numOfMsg;
        _TRACE3("on_complete_rcv: calling InterruptCallback %u msg. must be handled!\n", numOfMsg);
        InterruptCallback(pDevice);
      }
      packets++;
      
    }    
  }
  

  retval = RcvNextPacket(pDevice);
  
  if(retval)
  {
    pDevice->canprousb.IoChainRcvState = CHAIN_STOPPED_ON_ERROR;
  }
}

/**
 * *****************************************************************
 * @brief  called after successful transmission, to adjust the xmt FIFO read pointer.   
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pSharedMem: pointer to the structure which contains the transmit FIFO
 * @param  size:       size of the transmitted message data 
 * @return void
 *
 ******************************************************************
 */

// increment the read pointer of the XMT FIFO
void setTransmitDataBuffer(T_CANUSBP_SHARED_MEM *pSharedMem, unsigned long size)
{
  unsigned int i;
  
  for(i=0; i<size; i++)
  {
    volatile unsigned short rd = pSharedMem->can_pro_usb_fifo_mode.xmt.rd_xmt_fifo;
    
    rd = (rd+1) & XMT_FIFO_USB_MASK;
    
    if(rd == pSharedMem->can_pro_usb_fifo_mode.xmt.wr_xmt_fifo)
    {
      return;
    }
    pSharedMem->can_pro_usb_fifo_mode.xmt.rd_xmt_fifo = rd;
  }
  return;
}

/**
 * *****************************************************************
 * @brief  returns a pointer to the next message, which should be transmitted with the next OUT packet.   
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  devdata: pointer to the structure which contains the transmit FIFO
 * @param  pSize:   out: pointer to the size of the message data 
 * @return pointer to an FIFO entry
 *
 ******************************************************************
 */

// get pointer to transmit data buffer in the XMT FIFO and the data size
BYTE *getTransmitDataBuffer(CAN_PRO_USB *devdata, USIGN32 *pSize)
{
  INT32 rd = ((INT32)devdata->pSharedMem->can_pro_usb_fifo_mode.xmt.rd_xmt_fifo & XMT_FIFO_USB_MASK);
  INT32 wr = ((INT32)devdata->pSharedMem->can_pro_usb_fifo_mode.xmt.wr_xmt_fifo & XMT_FIFO_USB_MASK);
  INT32 sz;
  
  rd = (rd+1) & XMT_FIFO_USB_MASK;
  
  if((rd == wr) || (devdata->FirmwareXmtFifoLevel<=0))
  { 
    // empty fifo
    // if FirmwareXmtFifoLevel<0 then the sending of real data should be stoped
    // because a reset of the xmt fifo is pending
    // after the reset the FirmwareXmtFifoLevel will be set to 255 == empty fifo
    // if FirmwareXmtFifoLevel == 0 then the xmt fifo in the firmware is full
    
    *pSize = 0;
    return NULL;
  }
  else
  {
    if(rd > wr)
    {
      sz = (XMT_FIFO_ITEMS_USB_PRO - rd);
    }
    else
    {
      sz = wr-rd;
    }
    
    if(sz>devdata->FirmwareXmtFifoLevel)
    {
      sz = devdata->FirmwareXmtFifoLevel;
    }
    
    InterlockedExchangeAdd(&devdata->FirmwareXmtFifoLevel, (sz*-1), &devdata->Spinlock);
    
    *pSize = sz*32;
    
    return (BYTE*)&devdata->pSharedMem->can_pro_usb_fifo_mode.xmt.xmt_fifo[rd];
  }
}


/**
 * *****************************************************************
 * @brief  starts the next OUT transfer, if a packet to transmit is available.   
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pDevice:   device data 
 * @return void
 *
 ******************************************************************
 */
int XmtNextPacket(PCDEVICES pDevice)
{
  BYTE *pBuffer = NULL;
  INT32 size;
  int retval;
  struct usb_device *usb_dev = interface_to_usbdev(pDevice->canprousb.device.intf);
  
  pBuffer = getTransmitDataBuffer(&pDevice->canprousb, &size);
  
  if(pBuffer)
  {
    pDevice->canprousb.IoChainXmtState = CHAIN_RUNNING;
    if(size > MAX_BULK_SIZE)
    {
      size = MAX_BULK_SIZE;
    }
    pDevice->canprousb.XmtPending = TRUE;
    pDevice->canprousb.XmtIdleCounter = 0;
    memcpy(pDevice->canprousb.device.pdat_out_buf, pBuffer, size);
  }
  else
  {
    pDevice->canprousb.XmtIdleCounter++;
    if(pDevice->canprousb.XmtIdleCounter<MAXIMUM_DUMMY_TRANSFERS)
    {
      size = 2;
      pDevice->canprousb.XmtPending = TRUE;
      memset(pDevice->canprousb.device.pdat_out_buf,0,2);
      pDevice->canprousb.device.pdat_out_buf[0] = 99; 
    }
    else
    {
//       printk("XmtNextPacket: fifo is empty for a long time: going to sleep\n");
      pDevice->canprousb.IoChainXmtState = CHAIN_IDLE;
      pDevice->canprousb.XmtPending = FALSE;
      return 0;
    }
    
  }
  _TRACE3("XmtNextPacket: preparing urb\n");
  
  usb_fill_bulk_urb(pDevice->canprousb.device.dat_out_urb,
                    usb_dev,
                    usb_sndbulkpipe(usb_dev, CANUSBP_DAT_OUT_ENDPOINT), 
                    pDevice->canprousb.device.pdat_out_buf,
                    size,
                    on_complete_xmt,
                    pDevice);
  
  retval = usb_submit_urb(pDevice->canprousb.device.dat_out_urb, GFP_ATOMIC);
  
  mod_timer(&pDevice->canprousb.xmt_timer, jiffies + msecs_to_jiffies(1000) );
  
  return retval;
}

/**
 * *****************************************************************
 * @brief  transmit completion routine, called after an OUT transfer.   
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  urb:   USB request block 
 * @return void
 *
 ******************************************************************
 */
static void on_complete_xmt(struct urb *urb)
{
  static int counter = 0;

	PCDEVICES pDevice = (PCDEVICES) urb->context;
  int retval;
  INT32 size;
  
  counter++;
  if(!(counter%20000))
  {
    _TRACE3("%u messages transmitted\n", counter); 
  }
  
  if(urb->status)
  {
    pDevice->canprousb.IoChainXmtState = CHAIN_STOPPED_ON_ERROR;
    _TRACE3("on_complete_xmt: error %u\n", urb->status);
    return;
  }
  
  if(pDevice->canprousb.IoChainXmtState != CHAIN_RUNNING)
  {
    _TRACE3("on_complete_xmt: stop cmd received %u\n", pDevice->canprousb.IoChainXmtState);
    pDevice->canprousb.IoChainXmtState = CHAIN_STOPPED;
    return;
  }
  
  size = urb->actual_length/sizeof(USBXMT_ITEM);
  
  if(size > 0)
  {
    // set the read pointer inside the transmit fifo
    setTransmitDataBuffer(pDevice->canprousb.pSharedMem, size);
  }
  
  retval = XmtNextPacket(pDevice);
  
  if(retval)
  {
    pDevice->canprousb.IoChainXmtState = CHAIN_STOPPED_ON_ERROR;
  }
  
}

/**
 * *****************************************************************
 * @brief  checks the firmware version and crc code.   
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pDevice:   device data 
 * @param  version:   requested firmware version 
 * @param  checksum:  crc check sum of the firmware
 * @return 0, if successful, an error code otherwise.
 *
 ******************************************************************
 */

static int CheckFirmwareFlash ( IN  PCDEVICES pDevice, IN  USIGN16 version, IN  USIGN16 checksum ) 
{
  int retval;
  volatile FW_PACKET fw_packet;
  volatile FW_PACKET fw_ret;
  
  
  memset((char*)&fw_packet, 1, sizeof(fw_packet));
  
  fw_packet.size = sizeof(fw_packet);
  fw_packet.address = 0xC20000;
  fw_packet.type = CMD_QUERY_FIRMWARE;
  fw_packet.status = VCARD_OK;
  fw_packet.count = cmd_count++;
  fw_packet.data.fw.version = version;
  fw_packet.data.fw.build = 1;
  fw_packet.data.fw.checksum = checksum;
  
  retval = send_command(pDevice, (BYTE*)&fw_packet, sizeof(FW_PACKET), (BYTE*)&fw_ret, sizeof(FW_PACKET), 500);
  
  if(retval) return retval;
  
  if(fw_ret.status)
  {
    _TRACE5("firmware status error: %u\n",fw_ret.status);
    return fw_ret.status;
  }
  
  return retval;
  
}

/**
 * *****************************************************************
 * @brief  causes the Bootblock firmware to change to the CAN firmware   
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pDevice:   device data 
 * @return 0, if successful, an error code otherwise.
 *
 ******************************************************************
 */

static int StartFirmware(IN  PCDEVICES pDevice) 
{
  int status = VCARD_OK;
  FW_PACKET fw_packet;
  FW_PACKET fw_ret;
  ULONG addr = 0xC20000;
  
  _TRACE5("StartFirmware:\n");
  
  memset((char*)&fw_packet, 0, sizeof(fw_packet));
  
  fw_packet.size = 0;
  fw_packet.address = addr;
  fw_packet.type = CMD_START_FIRMWARE;
  fw_packet.status = VCARD_OK;
  fw_packet.count = cmd_count++;
  fw_packet.data.fw.version = 501;
  status = send_command(pDevice, (BYTE*)&fw_packet, sizeof(FW_PACKET), (BYTE*)&fw_ret, sizeof(FW_PACKET), 500);
  
  mdelay(100); // wait for firmware startup
  
  return status;
}

/**
 * *****************************************************************
 * @brief  formats the output of GetFirmwareState() into a readable string   
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
  * @param  state:   type of firmware wich is active 
 * @return textual representation of the firmware state.
 *
 ******************************************************************
 */

char *getFwStateText(USIGN16 state)
{
  static char l2[]="CAN Firmware";
  static char boot[]="Bootblock";
  static char unknown[]="Unknown";
  
  switch(state)
  {
    case STATE_L2:
      return l2;
      break;
      
    case STATE_BOOTBLOCK:
      return boot;
      break;
      
    default:
      return unknown;
      break;
  }
}

/**
 * *****************************************************************
 * @brief  returns, if the running firware is the "CAN firmware" or the "Bootblock firmware"   
 * @brief  one of the vendor specific device requests defined in canprousb.h.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pDevice: device data 
 * @param  state:   type of firmware wich is active 
 * @return 0, if successful, an error code otherwise.
 *
 ******************************************************************
 */

int GetFirmwareState ( IN  PCDEVICES pDevice,  USIGN16 *state)
{
  int status = 0; 
  struct usb_device *dev = interface_to_usbdev(pDevice->canprousb.device.intf);
  
  _TRACE5("GetFirmwareState:\n");
  
  status = ReceiveCtrlData(dev, USBREQ_FW_STATE, USB_TYPE_VENDOR | USB_RECIP_DEVICE, (BYTE *)state, sizeof(USIGN16)); 
  
  if(status)
  { 
    //--- USBREQ_INFO was not successfull.
    
    //   USBD_STATUS ustatus = URB_STATUS(&urbControl);
    _TRACE5("CANUSBP, %s: failed! status=0x%x \n", __FUNCTION__, status);
    
    //   DBG_TRIGGER(0x32);
    
  } //--- USBREQ_INFO was not successfull.
  else
  { 
    //--- On successfull USBREQ_INFO.
    _TRACE5("CANUSBP, %s: success!\n", __FUNCTION__); 
    
  }
  
  
  return status;
} //--- ResetCanusbFrw
///////////////////////////////////////////////////////////////////////////////

/**
 * *****************************************************************
 * @brief  causes the CAN firmware to jump back to the bootblock   
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pDevice:   device data 
 * @return 0, if successful, an error code otherwise.
 *
 ******************************************************************
 */

int StopFirmware (IN  PCDEVICES pDevice)
{
  int status = VCARD_OK;
  FW_PACKET fw_packet;
  FW_PACKET fw_ret;
  USIGN32 addr = 0xC20000;
  USIGN16 firmwareState = 99;
  
  GetFirmwareState(pDevice, &firmwareState);
//   firmwareState = STATE_L2;
    
    
  _TRACE5("CANUSBP StopFirmware: actual state=%s\n", getFwStateText(firmwareState));
    
    if(firmwareState == STATE_BOOTBLOCK)
    {
      _TRACE5("CANUSBP StopFirmware: error resetting pipe!\n");
//       DBG_TRIGGER(0x31);
      // status = SendCmd(fdo, deviceHandle, pdx, CMD_START_FIRMWARE);
      fw_packet.size = 0;
      fw_packet.address = addr;
      fw_packet.type = CMD_STOP_FIRMWARE;
      fw_packet.status = VCARD_OK;
      fw_packet.count = cmd_count++;
      status = send_command(pDevice, (BYTE*)&fw_packet, sizeof(FW_PACKET), (BYTE*)&fw_ret, sizeof(FW_PACKET), 500);  
      
      
      /*
       *   ResetPipe(pdx, &pDevice->canprousb.Pipes[RESPONSE]);
       *   ResetPipe(pdx, &pdx->Pipes[REQUEST]);
       */
    }
    else
    {
      
      fw_packet.size = 0;
      fw_packet.address = addr;
      fw_packet.type = CMD_STOP_FIRMWARE;
      fw_packet.status = VCARD_OK;
      fw_packet.count = cmd_count++;
      status = send_command(pDevice, (BYTE*)&fw_packet, sizeof(FW_PACKET), (BYTE*)&fw_ret, sizeof(FW_PACKET), 500);  
    }
    
    return status;
}

/**
 * *****************************************************************
 * @brief  checks the firmware on the device and burns a new firmware, if   
 * @brief  the firmware on the board is outdated.   
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pDevice:   device data 
 * @param  pFirmware: pointer to the buffer which contains the firmware 
 * @param  u32FirmwareSize:   size of the firmware in bytes  
 * @return 0, if successful, an error code otherwise.
 *
 ******************************************************************
 */

USIGN32 burnFirmware(PCDEVICES pDevice, USIGN8 *pFirmware, USIGN32 u32FirmwareSize)
{
  USIGN16 checksum;
  int bytes2send=u32FirmwareSize,i;
  int status = 0;
  char *pFw = pFirmware;
  USIGN32 addr = 0xC20000;
  volatile FW_PACKET fw_packet;
  volatile FW_PACKET fw_ret;
  
  
  checksum = calcCRC16r(pFw, u32FirmwareSize);
  
  if(CheckFirmwareFlash(pDevice, ((USIGN16*)pFirmware)[318], checksum))
  {
    // firmware must be flashed (again?)
    while((bytes2send>0) && (status == 0))
    {
      for(i=0; ((i<128) && (bytes2send-i >= 0)); i++)
      {
        fw_packet.data.packet[i] = *pFw++;
      }
      fw_packet.size = (USIGN16)i;
      fw_packet.address = addr;
      fw_packet.checksum = calcCRC16r((USIGN8 *)&fw_packet.data.packet[0], i);
      fw_packet.type = CMD_WRT_FLASH;
      fw_packet.status = VCARD_OK;
      fw_packet.count = cmd_count++;
      status = send_command(pDevice, (BYTE*)&fw_packet, sizeof(FW_PACKET), (BYTE*)&fw_ret, sizeof(FW_PACKET), 500);
      addr += i;
      bytes2send -= i;
    }
    
    if(status) return status;
  }
  else
  {
    _TRACE5("CANUSBP CheckFirmwareFlash: firmware is ok!\n");
  }
  
  status = StartFirmware(pDevice);
  
  if(!status) 
  {
    _TRACE5("Firmware is started!\n");
  }
  
  return status;
}


/**
 * *****************************************************************
 * @brief  thread which manages the data stream for sending and receiving data from the CANpro USB.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pDevice:  device data
 * @return void
 *
 ******************************************************************
 */
int CheckTransferThread(void *ctx)
{
  int rcv_errorCounter = 0;
  int xmt_errorCounter = 0;
  int rcv_counter=0;
  int xmt_counter=0;
  int status = 0;
  volatile INT32 rd;
  volatile INT32 wr;
  
  PCDEVICES pDevice = ctx;
  T_CANUSBP_SHARED_MEM  *pSh = pDevice->canprousb.pSharedMem;
  
  pDevice->canprousb.transferStarted = TRUE;
  pDevice->canprousb.IoChainRcvState = CHAIN_IDLE;
  pDevice->canprousb.IoChainXmtState = CHAIN_IDLE;
  pDevice->canprousb.FirmwareXmtFifoLevel = 255;
  pDevice->canprousb.rcv_usb_enable = TRUE;
  
  _TRACE5("CheckTransferThread: \n");
  
  while(!kthread_should_stop())
  {
    switch(pDevice->canprousb.IoChainRcvState)
    {
      case CHAIN_IDLE:
        rcv_errorCounter = 0;
        
        status = RcvNextPacket(pDevice);
        if(status)
        {
          pDevice->canprousb.IoChainXmtState = (pDevice->canprousb.IoChainXmtState != CHAIN_STOPPED) ? CHAIN_STOP_CMD : CHAIN_STOPPED;
        }
        else
        {
          _TRACE5("CheckTransferThread: rcv chain started.\n");
        }
        break;
        
      case CHAIN_STOPPED_ON_ERROR:
        // chain stopped on error
        rcv_errorCounter++;
        rcv_counter = 0;
        if(rcv_errorCounter<4)
        {
          status = RcvNextPacket(pDevice);
          if(status)
          {
            pDevice->canprousb.IoChainXmtState = (pDevice->canprousb.IoChainXmtState != CHAIN_STOPPED) ? CHAIN_STOP_CMD : CHAIN_STOPPED;
          }
          else
          {
            _TRACE5("CheckTransferThread: rcv chain restarted.\n");
          }
        }
        else
        {
          pDevice->canprousb.IoChainXmtState = (pDevice->canprousb.IoChainXmtState != CHAIN_STOPPED) ? CHAIN_STOP_CMD : CHAIN_STOPPED;
          pSh->usb_error = USB_RCV_ERROR;
        }
        break;
        
      case CHAIN_STOP_CMD:
      case CHAIN_STOPPED:
      default:
        // not recoverable error happened or stop command received
        // the thread terminates in this case too
        pDevice->canprousb.IoChainXmtState = (pDevice->canprousb.IoChainXmtState != CHAIN_STOPPED) ? CHAIN_STOP_CMD : CHAIN_STOPPED;
        usb_kill_urb(pDevice->canprousb.device.cmd_in_urb);
        
        break;
        
      case CHAIN_RUNNING:
        if(rcv_errorCounter) rcv_counter++;
        if(rcv_counter>=10000)
        {
          rcv_errorCounter = 0;
          rcv_counter = 0;
        }
        // chain is running -> nothing to do
        break;
        
    }
    
    switch(pDevice->canprousb.IoChainXmtState)
    {
      case CHAIN_IDLE:
        // chain stopped because fifo empty.
        // the chain must be restarted.
        xmt_errorCounter = 0;
        rd = (INT32)((pDevice->canprousb.pSharedMem->can_pro_usb_fifo_mode.xmt.rd_xmt_fifo+1) & XMT_FIFO_USB_MASK);
        wr = (INT32)(pDevice->canprousb.pSharedMem->can_pro_usb_fifo_mode.xmt.wr_xmt_fifo & XMT_FIFO_USB_MASK);
        status = 0;
        if(rd != wr)
        { 
          // xmt fifo is not empty
          pDevice->canprousb.XmtIdleCounter = 0;
/*          if(down_interruptible(&pDevice->cDpramSemaphore))
          {
            // signal received!
            
          }
          else
          {
            status = 0;
            // may be it's restarted from API Function CANL2_send_data(), CANL2_send_remote() ??
            if(pDevice->canprousb.IoChainXmtState != CHAIN_RUNNING)
            {
              status = XmtNextPacket(pDevice);
            }
            
            up(&pDevice->cDpramSemaphore);
          }
  */        
          if(status)
          {
            pDevice->canprousb.IoChainRcvState = (pDevice->canprousb.IoChainRcvState != CHAIN_STOPPED) ? CHAIN_STOP_CMD : CHAIN_STOPPED;
          }
          else
          {
            _TRACE5("CheckTransferThread: xmt chain started. rd=%u wr=%u level=%u\n", rd, wr, pDevice->canprousb.FirmwareXmtFifoLevel);
          }
        }
        break;
        
      case CHAIN_STOPPED_ON_ERROR:
        // chain stopped on error
       //VendorResetPipe(pdx, GET_PIPE_INDEX(CANUSBP_REQ_ENDPOINT));
        xmt_errorCounter++;
        xmt_counter = 0;
        if(xmt_errorCounter<4)
        {
          pDevice->canprousb.XmtIdleCounter = 0;
          if(down_interruptible(&pDevice->cDpramSemaphore))
          {
            // signal received!
            
          }
          else
          {
            status = 0;
            // may be it's restarted from API Function CANL2_send_data(), CANL2_send_remote() ??
            if(pDevice->canprousb.IoChainXmtState != CHAIN_RUNNING)
            {
              status = XmtNextPacket(pDevice);
            }
            
            up(&pDevice->cDpramSemaphore);
          }
          
          if(status)
          {
            pDevice->canprousb.IoChainRcvState = (pDevice->canprousb.IoChainRcvState != CHAIN_STOPPED) ? CHAIN_STOP_CMD : CHAIN_STOPPED;
          }
          else
          {
            _TRACE5("CheckTransferThread: xmt chain restarted after error.\n");
          }
          
        }
        else
        {
          pDevice->canprousb.IoChainRcvState = (pDevice->canprousb.IoChainRcvState != CHAIN_STOPPED) ? CHAIN_STOP_CMD : CHAIN_STOPPED;
          pSh->usb_error = USB_XMT_ERROR;
        }
        break;
        
      case CHAIN_STOP_CMD:
      case CHAIN_STOPPED:
      default:
        // not recoverable error happened or stop command received
        // the thread terminates in this case too
        pDevice->canprousb.IoChainRcvState = (pDevice->canprousb.IoChainRcvState != CHAIN_STOPPED) ? CHAIN_STOP_CMD : CHAIN_STOPPED;
        break;
        
      case CHAIN_RUNNING:
        if(xmt_errorCounter) xmt_counter++;
        if(xmt_counter>=10000)
        {
          xmt_errorCounter = 0;
          xmt_counter = 0;
        }
        
        // chain is running -> nothing to do
        break;
    }
    
     msleep_interruptible(10);
  }
  
  pDevice->canprousb.IoChainRcvState = CHAIN_STOP_CMD;
  
  usb_kill_urb(pDevice->canprousb.device.dat_in_urb);
  usb_kill_urb(pDevice->canprousb.device.dat_out_urb);
  
  pDevice->canprousb.transferStarted = FALSE;
  _TRACE5("CheckTransferThread: terminating\n");
  
  
  return 0;
}

/**
 * *****************************************************************
 * @brief  starts the continuous data stream for sending and receiving data from the CANpro USB.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pDevice:  device data
 * @return void
 *
 ******************************************************************
 */

void StartIoChain(PCDEVICES pDevice)
{
  char name[30];
  
  sprintf(name, "softing_%u",pDevice->Serialnumber);
  
  if(!pDevice->canprousb.transferStarted)
  {
    pDevice->canprousb.CheckTransferThread = kthread_run(CheckTransferThread, pDevice,name);
  }
}

/**
 * *****************************************************************
 * @brief  stops the continuous data stream for sending and receiving data from the CANpro USB.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pDevice:  device data
 * @return void
 *
 ******************************************************************
 */
void StopIoChain(PCDEVICES pDevice)
{
  int i;
  
  if(pDevice->canprousb.transferStarted)
  {
    kthread_stop(pDevice->canprousb.CheckTransferThread);
  }
  
  for(i=0; i<100; i++)
  {
    if(!pDevice->canprousb.transferStarted)
    {
      break;
    }
    else
    {
      mdelay(1); // wait a further ms for terminating.
    }
  }
}

/**
 * *****************************************************************
 * @brief  sends a vendor request to the device and receives information about the device.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  dev:  USB device data
 * @param  info: received information.
 * @return void
 *
 ******************************************************************
 */
int GetDeviceInfo (struct usb_device *dev, DEVICEINFO *info)
{
  int retval;
  
  retval = ReceiveCtrlData(dev, USBREQ_INFO, USB_TYPE_VENDOR | USB_RECIP_DEVICE, (BYTE *)info, sizeof(info));  
  
  printk("serial = %u, pigyback=%u, version=%u\n", info->serialnumber, info->piggyback1, info->version);
  return retval;
} 

/**
 * *****************************************************************
 * @brief  calculates the object number from the CAN identifier.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  Ident:  CAN Identifier
 * @param  lookup: lookup table.
 * @return void
 *
 ******************************************************************
 */
USIGN32 ident_to_objectnr(USIGN32 Ident, sorted_lookup * lookup)
{
  USIGN16 step = (MAX_OBJ_DYN+1) / 2;
  USIGN16 index=0;
  INT16   sign = 1;
  
  do
  {
    index = (sign > 0) ? (index+step>MAX_OBJ_DYN) ? MAX_OBJ_DYN : index+step : (index<step) ? 0 : index-step;
    step = (step+1) >> 1;
    if(lookup[index].id > Ident)
    {  // greater
    sign = -1;
    }
    else
    {
      if(lookup[index].id < Ident)
      {  // less
      sign = 1;
      }
      else
      {  //equal
      return lookup[index].obj;
      }
    }
    
  } while(step > 1);
  
  if(index+1 <= MAX_OBJ_DYN)
  {
    if(lookup[index+1].id == Ident)
      return lookup[index+1].obj;
  }
  
  if(index != 0)
  {
    if(lookup[index-1].id == Ident)
      return lookup[index-1].obj;
  }
  
  return XTD_OBJ_NOT_VALID;
}

/**
 * *****************************************************************
 * @brief  calculates the object number from the CAN identifier.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  Ident:  CAN Identifier
 * @param  lookup: lookup table.
 * @return void
 *
 ******************************************************************
 */
USIGN32 xmt_ident_to_objectnr(USIGN32 Ident, USIGN32 * lookup)
{
  USIGN16 step = (MAX_OBJ_DYN+1) / 2;
  USIGN16 index=0;
  INT16   sign = 1;
  
  do
  {
    index = (sign > 0) ? (index+step>MAX_OBJ_DYN) ? MAX_OBJ_DYN : index+step : (index<step) ? 0 : index-step;
    step = (step+1) >> 1;
    if(lookup[index] > Ident)
    {  // greater
    sign = -1;
    }
    else
    {
      if(lookup[index] < Ident)
      {  // less
      sign = 1;
      }
      else
      {  //equal
      return index;
      }
    }
    
  } while(step > 1);
  
  if(index+1 <= MAX_OBJ_DYN)
  {
    if(lookup[index+1] == Ident)
      return index+1;
  }
  
  if(index != 0)
  {
    if(lookup[index-1] == Ident)
      return index-1;
  }
  
  return XTD_OBJ_NOT_VALID;
}


/**
 * *****************************************************************
 * @brief  checks receive fifo, filled by the ISR and.  
 * @brief  and updates the objects data. (dynamic object buffer only!)
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pDevice:     device data
 * @param  pfifo_entry: FIFO entry, which must be handled.
 * @return void
 *
 ******************************************************************
 */
static void check_dyn_rcv_object_fifo(PCDEVICES pDevice, USBRCVITEM *pfifo_entry)
{
  volatile RCV_OBJECT *pRcvObject;
  volatile XMT_OBJECT *pXmtObject;
  T_CANUSBP_OBJECT_BUF_MODE *pObjectData;
  CAN_ADDRESS rcvIrql,xmtIrql;
  USIGN16  u16ObjectNr;
  
  if(!pDevice || !pfifo_entry)
  {
    return;
  }
  
  pObjectData = (T_CANUSBP_OBJECT_BUF_MODE *)pDevice->canprousb.pObjectBuf;
  
  if(!pObjectData)
  {
    return;
  }
  
  // first check the event type
  switch(pfifo_entry->u8Cmd)
  {
    case RCV_CMD:
      u16ObjectNr = (USIGN16)pfifo_entry->u32Ident;
      _TRACE3("CANBUS check_dyn_rcv_object_fifo: RCV_CMD(0x%x)\n",u16ObjectNr);
      if(u16ObjectNr <= MAX_OBJ_DYN)
      {
        pRcvObject = (RCV_OBJECT *)&pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.object[u16ObjectNr];
        rcvIrql = sync_get_access((spinlock_t *)&pRcvObject->access);
        
        *(USIGN64*)&pRcvObject->data[0] = *(USIGN64*)&pfifo_entry->u8Data[0];
        
        pRcvObject->Timestamp = MkTimestamp(pfifo_entry);
        pRcvObject->DataLength = pfifo_entry->u8DataLen;
        
        
        if(pRcvObject->ctrl1 & RCV_STATUS_DATA)
        {
          pRcvObject->ctrl1 |= RCV_STATUS_OVERRUN;
        }
        pRcvObject->ctrl1 |= RCV_STATUS_DATA;
        
        if(pRcvObject->ctrl1 & HOST_INTERRUPT_ENABLE)
        {
          if(pDevice->canprousb.ReceiveFifoEnable)
          {
            if(pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_read_ptr == pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_write_ptr)
            { 
              // fifo overrun
              if(pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_lost_msg<0xFFFF)
                pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_lost_msg++;
            }
            else
            {
              pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo[pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_write_ptr] = u16ObjectNr;
              pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_write_ptr++;
            }
          }
          pDevice->can_info.irq_flag = 1;
          wake_up_interruptible(&pDevice->can_info.read_queue);     
        }
        // release object access
        sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
      }
      break;
      
    case RCV_CMD_XTD:
      u16ObjectNr = (USIGN16)pfifo_entry->u32Ident;
      _TRACE3("CANBUS check_dyn_rcv_object_fifo: RCV_CMD_XTD(0x%x)\n",u16ObjectNr);
      if(u16ObjectNr != XTD_OBJ_NOT_VALID)
      {
        
        pRcvObject = (RCV_OBJECT *)&pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.object[u16ObjectNr];
        
        rcvIrql = sync_get_access((spinlock_t *)&pRcvObject->access);
        
        *(USIGN64*)&pRcvObject->data[0] = *(USIGN64*)&pfifo_entry->u8Data[0];
        
        pRcvObject->Timestamp = MkTimestamp(pfifo_entry);
        pRcvObject->DataLength = pfifo_entry->u8DataLen;
        
        
        if(pRcvObject->ctrl1 & RCV_STATUS_DATA)
        {
          pRcvObject->ctrl1 |= RCV_STATUS_OVERRUN;
        }
        pRcvObject->ctrl1 |= RCV_STATUS_DATA;
        
        if(pRcvObject->ctrl1 & HOST_INTERRUPT_ENABLE)
        {
          if(pDevice->canprousb.ReceiveFifoEnable)
          {
            if(pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_read_ptr == pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_write_ptr)
            { 
              // fifo overrun
              if(pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_lost_msg<0xFFFF)
                pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_lost_msg++;
            }
            else
            {
              pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo[pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_write_ptr] = u16ObjectNr;
              pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_write_ptr++;
            }
          }
          pDevice->can_info.irq_flag = 1;
          wake_up_interruptible(&pDevice->can_info.read_queue);     
        }
        
        
        // release object access
        sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
      }
      
      break;
        
    case RCV_CMD_RMT:
      u16ObjectNr = (USIGN16)pfifo_entry->u32Ident;
      _TRACE3("CANBUS check_dyn_rcv_object_fifo: RCV_CMD_RMT(0x%x)\n",u16ObjectNr);
      if(u16ObjectNr != STD_OBJ_NOT_VALID)
      {
        pRcvObject = (RCV_OBJECT *)&pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.object[u16ObjectNr];
        pXmtObject = (XMT_OBJECT *)&pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.object[u16ObjectNr];
        
        rcvIrql = sync_get_access((spinlock_t *)&pRcvObject->access);
        xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
        pRcvObject->DataLengthRem = pfifo_entry->u8DataLen;
        
        if(pRcvObject->ctrl1 & RCV_STATUS_REMOTE)
        {
          pRcvObject->ctrl1 |= RCV_STATUS_OVERRUN;
        }
        
        if(pRcvObject->ctrl1 & AUTO_REMOTE_ENABLE)
        {
          if(pDevice->canprousb.TransmitRemoteFifoEnable)
          {
            if(pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.rmt_fifo_read_ptr == pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.rmt_fifo_write_ptr)
            { 
              // fifo overrun
              if(pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.rmt_fifo_lost_remotes<0xFFFF)
                pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.rmt_fifo_lost_remotes++;
            }
            else
            {
              if(pXmtObject->ctrl[1] & XMT_DATA_VALID)
              {
                pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.rmt_fifo[pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.rmt_fifo_write_ptr] = u16ObjectNr;
                pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.rmt_fifo_write_ptr++;
              }
            } 
            // fifo overrun
          } // TransmitRemoteFifoEnable1
          
        } // auto remote
        else
        {
          pRcvObject->TimestampRem = MkTimestamp(pfifo_entry);
          pRcvObject->ctrl1 |= RCV_STATUS_REMOTE;
          
          if(pRcvObject->ctrl1 & HOST_INTERRUPT_ENABLE)
          {
            if(pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_read_ptr == pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_write_ptr)
            { 
              // fifo overrun
              if(pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_lost_msg<0xFFFF)
                pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_lost_msg++;
            }
            else
            {
              pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo[pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_write_ptr] = u16ObjectNr;
              pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_write_ptr++;
            }
            pDevice->can_info.irq_flag = 1;
            wake_up_interruptible(&pDevice->can_info.read_queue);     
          }
          
        }
        // release object access
        sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
        sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
      }
      break;
          
    case RCV_CMD_XTD_RMT:
      u16ObjectNr = (USIGN16)pfifo_entry->u32Ident;
      _TRACE3("CANBUS check_dyn_rcv_object_fifo: RCV_CMD_XTD_RMT(0x%x)\n",u16ObjectNr);
      if(u16ObjectNr != XTD_OBJ_NOT_VALID)
      {
        pRcvObject = (RCV_OBJECT *)&pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.object[u16ObjectNr];
        pXmtObject = (XMT_OBJECT *)&pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.object[u16ObjectNr];
        
        
        rcvIrql = sync_get_access((spinlock_t *)&pRcvObject->access);
        xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
        
        pRcvObject->DataLengthRem = pfifo_entry->u8DataLen;
        
        if(pRcvObject->ctrl1 & RCV_STATUS_REMOTE)
        {
          pRcvObject->ctrl1 |= RCV_STATUS_OVERRUN;
        }
        
        if(pRcvObject->ctrl1 & AUTO_REMOTE_ENABLE)
        {
          if(pDevice->canprousb.TransmitRemoteFifoEnable)
          {
            if(pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.rmt_fifo_read_ptr == pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.rmt_fifo_write_ptr)
            { 
              // fifo overrun
              if(pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.rmt_fifo_lost_remotes<0xFFFF)
                pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.rmt_fifo_lost_remotes++;
            }
            else
            {
              if(pXmtObject->ctrl[1] & XMT_DATA_VALID)
              {
                pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.rmt_fifo[pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.rmt_fifo_write_ptr] = u16ObjectNr;
                pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.rmt_fifo_write_ptr++;
              }
            } // fifo overrun
          } // TransmitRemoteFifoEnable1
                    
        } // auto remote
        else
        {
          pRcvObject->TimestampRem = MkTimestamp(pfifo_entry);
          pRcvObject->ctrl1 |= RCV_STATUS_REMOTE;
          
          if(pRcvObject->ctrl1 & HOST_INTERRUPT_ENABLE)
          {
            if(pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_read_ptr == pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_write_ptr)
            { 
              // fifo overrun
              if(pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_lost_msg<0xFFFF)
                pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_lost_msg++;
            }
            else
            {
              pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo[pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_write_ptr] = u16ObjectNr;
              pObjectData->can_pro_usb_dyn_obj_buf.rcv_dyn.fifo_write_ptr++;
            }
            pDevice->can_info.irq_flag = 1;
            wake_up_interruptible(&pDevice->can_info.read_queue);     
          }
          
        }
        
        
        // release object access
        sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
        sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
      }
      break;
            
            
    case XMT_ACK_CMD:
      u16ObjectNr = (USIGN16)pfifo_entry->u32Ident;
      _TRACE3("CANBUS check_dyn_rcv_object_fifo: XMT_ACK_CMD(0x%x)\n",u16ObjectNr);
      if(u16ObjectNr != STD_OBJ_NOT_VALID)
      {
        pXmtObject = (XMT_OBJECT *)&pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.object[u16ObjectNr];
        xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
        
        // ACK timestamp
        pXmtObject->Timestamp = MkTimestamp(pfifo_entry);
        
        if(pXmtObject->ctrl[1] & XMT_STATUS)
        {
          // ACK overrun
          pXmtObject->ctrl[1] |= XMT_STATUS_OVERRUN;
        }
        else
        {
          //no overrun
          pXmtObject->ctrl[1] |= XMT_STATUS;
        }
        
        if(pXmtObject->ctrl[1] & XMT_ACK_FIFO_ENABLE)
        { 
          //--- ACK Fifo
          
          if(pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_write_ptr !=  pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_read_ptr)
          { 
            //--- Write to ACK fifo.
            pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo[pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_write_ptr] = u16ObjectNr;
            pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_write_ptr++;
            
            // if using ACK fifo generate interrupt
            //  SET_HOST_INT1;
            
          } //--- Write to ACK fifo.
          
          else
          { 
            //--- ACK fifo overrun.
            if (pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_lost_acks < 0xFFFFFFFF)
            { 
              //--- Increment overrun counter
              pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_lost_acks++;              
              // on increment overrun counter, generate interrupt
              //  SET_HOST_INT1;
            } //--- Increment overrun counter
            
          } //--- ACK fifo overrun.
          
        } //--- ACK Fifo
        
        pDevice->can_info.irq_flag = 1;
        wake_up_interruptible(&pDevice->can_info.read_queue);     
        
        // release object access
        sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
      }
      break;
      
              
   case XMT_ACK_CMD_XTD:
     u16ObjectNr = (USIGN16)pfifo_entry->u32Ident;
     _TRACE3("CANBUS check_dyn_rcv_object_fifo: XMT_ACK_CMD_XTD(0x%x)\n",u16ObjectNr);
     if(u16ObjectNr != XTD_OBJ_NOT_VALID)
     {
       
       pXmtObject = (XMT_OBJECT *)&pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.object[u16ObjectNr];
       xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
       
       // ACK timestamp
       pXmtObject->Timestamp = MkTimestamp(pfifo_entry);
       
       if(pXmtObject->ctrl[1] & XMT_STATUS)
         // ACK overrun
       pXmtObject->ctrl[1] |= XMT_STATUS_OVERRUN;
       else
         //no overrun
         pXmtObject->ctrl[1] |= XMT_STATUS;
       
       if(pXmtObject->ctrl[1] & XMT_ACK_FIFO_ENABLE)
       { 
         //--- ACK Fifo
         
         if(pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_write_ptr !=  pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_read_ptr)
         { 
           //--- Write to ACK fifo.
           pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo[pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_write_ptr] = u16ObjectNr;
           pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_write_ptr++;
           
           // if using ACK fifo generate interrupt
           // SET_HOST_INT1;
         } //--- Write to ACK fifo.
         
         else
         { 
           //--- ACK fifo overrun.
           if (pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_lost_acks < 0xFFFFFFFF)
           { 
             //--- Increment overrun counter
             pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_lost_acks++;              
             // on increment overrun counter, generate interrupt
             //  SET_HOST_INT1;
           } //--- Increment overrun counter
           
         } //--- ACK fifo overrun.
         
       } //--- ACK Fifo
       
       pDevice->can_info.irq_flag = 1;
       wake_up_interruptible(&pDevice->can_info.read_queue);     
       
       // release object access
       sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
     }
     break;
     
    case XMT_ACK_CMD_RMT:
      u16ObjectNr = (USIGN16)pfifo_entry->u32Ident;
      _TRACE3("CANBUS check_dyn_rcv_object_fifo: XMT_ACK_CMD_RMT(0x%x)\n",u16ObjectNr);
      if(u16ObjectNr != STD_OBJ_NOT_VALID)
      {
        
        pXmtObject = (XMT_OBJECT *)&pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.object[u16ObjectNr];
        xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
        
        // ACK timestamp
        pXmtObject->TimestampRemote = MkTimestamp(pfifo_entry);
        //  pXmtObject->len_remote = pfifo_entry->u8DataLen;
        
        
        // set XMT ACK status
        if(pXmtObject->ctrl[1] & XMT_ACK_RMT)
          // ACK overrun
        pXmtObject->ctrl[1] |= XMT_STATUS_OVERRUN;
        // else
        //no overrun
        //  pXmtObject->ctrl[1] |= XMT_STATUS;
        
        // set XMT ACK rmt status
        pXmtObject->ctrl[1] |= XMT_ACK_RMT;
        
        
        if(pXmtObject->ctrl[1] & XMT_ACK_FIFO_ENABLE)
        { //--- ACK Fifo
        
        if(pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_write_ptr !=  pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_read_ptr)
        { 
          //--- Write to ACK fifo.
          pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo[pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_write_ptr] = u16ObjectNr;
          pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_write_ptr++;
          
          // if using ACK fifo generate interrupt
          // SET_HOST_INT1;
          
        } //--- Write to ACK fifo.
        
        else
        { 
          //--- ACK fifo overrun.
          if (pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_lost_acks < 0xFFFFFFFF)
          { 
            //--- Increment overrun counter
            pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_lost_acks++;              
            // on increment overrun counter, generate interrupt
            // SET_HOST_INT1;
          } //--- Increment overrun counter
          
        } //--- ACK fifo overrun.
        
        } //--- ACK Fifo
        pDevice->can_info.irq_flag = 1;
        wake_up_interruptible(&pDevice->can_info.read_queue);     
        
        // release object access
        sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
      }
      break;
      
    case XMT_ACK_CMD_XTD_RMT:
      u16ObjectNr = (USIGN16)pfifo_entry->u32Ident;
      _TRACE3("CANBUS check_dyn_rcv_object_fifo: XMT_ACK_CMD_XTD_RMT(0x%x)\n",u16ObjectNr);
      if(u16ObjectNr != XTD_OBJ_NOT_VALID)
      {
        
        pXmtObject = (XMT_OBJECT *)&pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.object[u16ObjectNr];
        xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
        
        // ACK timestamp
        pXmtObject->TimestampRemote = MkTimestamp(pfifo_entry);
        //  pXmtObject->len_remote = pfifo_entry->u8DataLen;
        
        
        // set XMT ACK status
        if(pXmtObject->ctrl[1] & XMT_ACK_RMT)
          // ACK overrun
        pXmtObject->ctrl[1] |= XMT_STATUS_OVERRUN;
        //   else
        //no overrun
        //  pXmtObject->ctrl[1] |= XMT_STATUS;
        
        // set XMT ACK rmt status
        pXmtObject->ctrl[1] |= XMT_ACK_RMT;
        
        if(pXmtObject->ctrl[1] & XMT_ACK_FIFO_ENABLE)
        { 
          //--- ACK Fifo
          
          if(pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_write_ptr !=  pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_read_ptr)
          { //--- Write to ACK fifo.
          pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo[pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_write_ptr] = u16ObjectNr;
          pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_write_ptr++;
          
          // if using ACK fifo generate interrupt
          // SET_HOST_INT1;
          
          } //--- Write to ACK fifo.
          
          else
          { 
            //--- ACK fifo overrun.
            if (pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_lost_acks < 0xFFFFFFFF)
            { 
              //--- Increment overrun counter
              pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.ack_fifo_lost_acks++;              
              // on increment overrun counter, generate interrupt
              // SET_HOST_INT1;
            } //--- Increment overrun counter
            
          } //--- ACK fifo overrun.
          
        } //--- ACK Fifo
        
        pDevice->can_info.irq_flag = 1;
        wake_up_interruptible(&pDevice->can_info.read_queue);     
        
        // release object access
        sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
      }
      break;
      
    case XMT_OBJ:
    case WRT_OBJ:
      
      u16ObjectNr = (USIGN16)pfifo_entry->u32Ident;
      _TRACE3("CANBUS check_dyn_rcv_object_fifo: DATA_ACK(0x%x)\n",u16ObjectNr);
      
      pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.req_fifo_read_ptr++;
      
      if(u16ObjectNr != XTD_OBJ_NOT_VALID)
      {
        
        pXmtObject = (XMT_OBJECT *)&pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.object[u16ObjectNr];
        
        xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
        // enable next request - unlock requests
        pXmtObject->ctrl[0] &=  ~XMT_REQ_DATA;
        // release object access
        sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
      }
      break;
      
    case SUP_OBJ:
      u16ObjectNr = (USIGN16)pfifo_entry->u32Ident;
      _TRACE3("CANBUS check_dyn_rcv_object_fifo: SUP_OBJ(0x%x)\n",u16ObjectNr);
      break;
      
    case XMT_REM_OBJ:
      
      u16ObjectNr = (USIGN16)pfifo_entry->u32Ident;
      _TRACE3("CANBUS check_dyn_rcv_object_fifo: XMT_REM_OBJ(0x%x)\n",u16ObjectNr);
      if(u16ObjectNr != XTD_OBJ_NOT_VALID)
      {
        
        pXmtObject = (XMT_OBJECT *)&pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.object[u16ObjectNr];
        pObjectData->can_pro_usb_dyn_obj_buf.xmt_dyn.req_fifo_read_ptr++;
        
        xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
        // enable next remote request - unlock requests
        pXmtObject->ctrl[0] &= ~XMT_REQ_REMOTE;
        // release object access
        sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
        
      }
      break;
      
    default:
      _TRACE3("CANBUS check_dyn_rcv_object_fifo: UNKNOWN CMD(0x%x)\n",pfifo_entry->u8Cmd);
      break;
      
  }
}


/**
 * *****************************************************************
 * @brief  checks receive fifo, filled by the ISR and.  
 * @brief  and updates the objects in the DPRAM. (static object buffer only!)
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  pDevice:     device data
 * @param  pfifo_entry: FIFO entry, which must be handled.
 * @return void
 *
 ******************************************************************
 */
static void check_stat_rcv_object_fifo(PCDEVICES pDevice, USBRCVITEM *pfifo_entry)
{
  RCV_OBJECT *pRcvObject;
  XMT_OBJECT *pXmtObject;
  T_CANUSBP_OBJECT_BUF_MODE *pObjectData;
  INT32 i;
  CAN_ADDRESS rcvIrql,xmtIrql;
  USIGN16  u16ObjectNr;
  
  
  if(!pDevice || !pfifo_entry)
  {
    return;
  }
  
  pObjectData = (T_CANUSBP_OBJECT_BUF_MODE *)pDevice->canprousb.pObjectBuf;
  
  _TRACE3("check_stat_rcv_object_fifo: cmd 0x%02x\n", pfifo_entry->u8Cmd);
  // first check the event type
  switch(pfifo_entry->u8Cmd)
  {
    case RCV_CMD: 
      pRcvObject = (RCV_OBJECT FAR *)&pObjectData->can_pro_usb_stat_obj_buf.rcv_static.object[pfifo_entry->u32Ident];
      if((pRcvObject->ctrl1 & RCV_ENABLE) ^ RCV_ENABLE)
      {
        break;
      }
      
      rcvIrql = sync_get_access((spinlock_t *)&pRcvObject->access);
      
      for(i=0; i<8; i++)
      {
        pRcvObject->data[i] = pfifo_entry->u8Data[i];
      }
      
      pRcvObject->Timestamp = MkTimestamp(pfifo_entry);
      
      pRcvObject->DataLength = pfifo_entry->u8DataLen;
      
      if(pRcvObject->ctrl1 & RCV_STATUS_DATA)
      {
        pRcvObject->ctrl1 |= RCV_STATUS_OVERRUN;
      }
      pRcvObject->ctrl1 |= RCV_STATUS_DATA;
      
      if(pRcvObject->ctrl1 & HOST_INTERRUPT_ENABLE)
      {
        if(pDevice->canprousb.ReceiveFifoEnable)
        {
          if(pObjectData->can_pro_usb_stat_obj_buf.rcv_static.fifo_read_ptr == pObjectData->can_pro_usb_stat_obj_buf.rcv_static.fifo_write_ptr)
          { // fifo overrun
          if(pObjectData->can_pro_usb_stat_obj_buf.rcv_static.fifo_lost_msg<0xFFFF)
            pObjectData->can_pro_usb_stat_obj_buf.rcv_static.fifo_lost_msg++;
          }
          else
          {
            pObjectData->can_pro_usb_stat_obj_buf.rcv_static.fifo[pObjectData->can_pro_usb_stat_obj_buf.rcv_static.fifo_write_ptr] = (USIGN16)pfifo_entry->u32Ident;
            pObjectData->can_pro_usb_stat_obj_buf.rcv_static.fifo_write_ptr++;
          }
        }
        pDevice->can_info.irq_flag = 1;
        wake_up_interruptible(&pDevice->can_info.read_queue);     
      }
      // release object access
      sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
      break;
      
    case RCV_CMD_RMT:
      pRcvObject = (RCV_OBJECT FAR *)&pObjectData->can_pro_usb_stat_obj_buf.rcv_static.object[pfifo_entry->u32Ident];
      pXmtObject = (XMT_OBJECT FAR *)&pObjectData->can_pro_usb_stat_obj_buf.xmt_static.object[pfifo_entry->u32Ident];
      
      rcvIrql = sync_get_access((spinlock_t *)&pRcvObject->access);    
      xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
      
      
      pRcvObject->DataLengthRem = pfifo_entry->u8DataLen;
      
      if(pRcvObject->ctrl1 & RCV_STATUS_REMOTE)
      {
        pRcvObject->ctrl1 |= RCV_STATUS_OVERRUN;
      }
      
      if(pRcvObject->ctrl1 & AUTO_REMOTE_ENABLE)
      {
        
      } // auto remote
      else
      {
        pRcvObject->TimestampRem = MkTimestamp(pfifo_entry);
        
        pRcvObject->ctrl1 |= RCV_STATUS_REMOTE;
        
        if(pRcvObject->ctrl1 & HOST_INTERRUPT_ENABLE)
        {
          if(pObjectData->can_pro_usb_stat_obj_buf.rcv_static.fifo_read_ptr == pObjectData->can_pro_usb_stat_obj_buf.rcv_static.fifo_write_ptr)
          { 
            // fifo overrun
            if(pObjectData->can_pro_usb_stat_obj_buf.rcv_static.fifo_lost_msg<0xFFFF)
              pObjectData->can_pro_usb_stat_obj_buf.rcv_static.fifo_lost_msg++;
          }
          else
          {
            pObjectData->can_pro_usb_stat_obj_buf.rcv_static.fifo[pObjectData->can_pro_usb_stat_obj_buf.rcv_static.fifo_write_ptr] = (USIGN16)pfifo_entry->u32Ident;
            pObjectData->can_pro_usb_stat_obj_buf.rcv_static.fifo_write_ptr++;
          }
          pDevice->can_info.irq_flag = 1;
          wake_up_interruptible(&pDevice->can_info.read_queue);     
        }
        
      }
      // release object access
      sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
      sync_free_access((spinlock_t *)&pRcvObject->access, rcvIrql);
      break;
      
    case XMT_ACK_CMD: 
      pXmtObject = (XMT_OBJECT FAR *)&pObjectData->can_pro_usb_stat_obj_buf.xmt_static.object[pfifo_entry->u32Ident];
      
      _TRACE3("XMT_ACK_CMD\n");
      if(pXmtObject->ctrl[1] & XMT_ACK_ENABLE)
      {
        _TRACE3("XMT_ACK_CMD will be handled!\n");
        xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
        // ACK timestamp
        pXmtObject->Timestamp = MkTimestamp(pfifo_entry);
        
        if(pXmtObject->ctrl[1] & XMT_STATUS)
          // ACK overrun
        pXmtObject->ctrl[1] |= XMT_STATUS_OVERRUN;
        else
          //no overrun
          pXmtObject->ctrl[1] |= XMT_STATUS;
        
        if(pXmtObject->ctrl[1] & XMT_ACK_FIFO_ENABLE)
        { //--- ACK Fifo
        
        if(pObjectData->can_pro_usb_stat_obj_buf.xmt_static.ack_fifo_write_ptr !=  pObjectData->can_pro_usb_stat_obj_buf.xmt_static.ack_fifo_read_ptr)
        { 
          //--- Write to ACK fifo.
          pObjectData->can_pro_usb_stat_obj_buf.xmt_static.ack_fifo[pObjectData->can_pro_usb_stat_obj_buf.xmt_static.ack_fifo_write_ptr] = (USIGN16)pfifo_entry->u32Ident;
          pObjectData->can_pro_usb_stat_obj_buf.xmt_static.ack_fifo_write_ptr++;
          
          // if using ACK fifo generate interrupt
          pDevice->can_info.irq_flag = 1;
          wake_up_interruptible(&pDevice->can_info.read_queue);     
        } //--- Write to ACK fifo.
        else
        { 
          //--- ACK fifo overrun.
          if (pObjectData->can_pro_usb_stat_obj_buf.xmt_static.ack_fifo_lost_acks < 0xFFFFFFFF)
          { 
            //--- Increment overrun counter
            pObjectData->can_pro_usb_stat_obj_buf.xmt_static.ack_fifo_lost_acks++;              
            // on increment overrun counter, generate interrupt
            pDevice->can_info.irq_flag = 1;
            wake_up_interruptible(&pDevice->can_info.read_queue);     
          } //--- Increment overrun counter
          
        } //--- ACK fifo overrun.
        
        } //--- ACK Fifo
        else
        {
          if(pDevice->canprousb.ReceiveIntEnableAll)
          {
            pDevice->can_info.irq_flag = 1;
            wake_up_interruptible(&pDevice->can_info.read_queue);     
          }
        }
        // release object access
        sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
      }
      break;
      
    case XMT_ACK_CMD_RMT:
      pXmtObject = (XMT_OBJECT FAR *)&pObjectData->can_pro_usb_stat_obj_buf.xmt_static.object[pfifo_entry->u32Ident];
      
      if(pXmtObject->ctrl[1] & XMT_ACK_ENABLE)
      {
        xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
        // ACK timestamp
        pXmtObject->TimestampRemote = MkTimestamp(pfifo_entry);
        //  pXmtObject->len_remote = pfifo_entry->u8DataLen;
        
        
        // set XMT ACK status
        if(pXmtObject->ctrl[1] & XMT_ACK_RMT)
          // ACK overrun
        pXmtObject->ctrl[1] |= XMT_STATUS_OVERRUN;
        //  else
        //no overrun
        //  pXmtObject->ctrl[1] |= XMT_STATUS;
        
        // set XMT ACK rmt status
        pXmtObject->ctrl[1] |= XMT_ACK_RMT;
        
        
        if(pXmtObject->ctrl[1] & XMT_ACK_FIFO_ENABLE)
        { //--- ACK Fifo
        
        if(pObjectData->can_pro_usb_stat_obj_buf.xmt_static.ack_fifo_write_ptr !=  pObjectData->can_pro_usb_stat_obj_buf.xmt_static.ack_fifo_read_ptr)
        { 
          //--- Write to ACK fifo.
          pObjectData->can_pro_usb_stat_obj_buf.xmt_static.ack_fifo[pObjectData->can_pro_usb_stat_obj_buf.xmt_static.ack_fifo_write_ptr] = (USIGN16)pfifo_entry->u32Ident;
          pObjectData->can_pro_usb_stat_obj_buf.xmt_static.ack_fifo_write_ptr++;
          
          // if using ACK fifo generate interrupt
          pDevice->can_info.irq_flag = 1;
          wake_up_interruptible(&pDevice->can_info.read_queue);     
        } //--- Write to ACK fifo.
        
        else
        { 
          //--- ACK fifo overrun.
          if (pObjectData->can_pro_usb_stat_obj_buf.xmt_static.ack_fifo_lost_acks < 0xFFFFFFFF)
          { 
            //--- Increment overrun counter
            pObjectData->can_pro_usb_stat_obj_buf.xmt_static.ack_fifo_lost_acks++;              
            // on increment overrun counter, generate interrupt
            pDevice->can_info.irq_flag = 1;
            wake_up_interruptible(&pDevice->can_info.read_queue);     
          } //--- Increment overrun counter
          
        } //--- ACK fifo overrun.
        
        } //--- ACK Fifo
        else
        {
          if(pDevice->canprousb.ReceiveIntEnableAll)
          {
            pDevice->can_info.irq_flag = 1;
            wake_up_interruptible(&pDevice->can_info.read_queue);     
          }
        }
        
        // release object access
        sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
      }
      break;
      
    case XMT_OBJ:
    case WRT_OBJ:
        
      u16ObjectNr = (USIGN16)pfifo_entry->u32Ident;
      if(u16ObjectNr <= MAX_IDENT_STD)
      {
        
        pXmtObject = (XMT_OBJECT *)&pObjectData->can_pro_usb_stat_obj_buf.xmt_static.object[u16ObjectNr];
        
        xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
        
        pObjectData->can_pro_usb_stat_obj_buf.xmt_static.req_fifo_read_ptr++;
        
        // enable next request - unlock requests
        pXmtObject->ctrl[0] &=  ~XMT_REQ_DATA;
        _TRACE3("CANBUS check_stat_rcv_object_fifo: DataCmdAck(0x%x)\n",u16ObjectNr);
        // release object access
        sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
      }
      break;
      
    case SUP_OBJ:
      break;
      
  case XMT_REM_OBJ:
      
    u16ObjectNr = (USIGN16)pfifo_entry->u32Ident;
    if(u16ObjectNr <= MAX_IDENT_STD)
    {
      pXmtObject = (XMT_OBJECT *)&pObjectData->can_pro_usb_stat_obj_buf.xmt_static.object[u16ObjectNr];
      
      xmtIrql = sync_get_access((spinlock_t *)&pXmtObject->access);
      pObjectData->can_pro_usb_stat_obj_buf.xmt_static.req_fifo_read_ptr++;
      // enable next remote request - unlock requests
      pXmtObject->ctrl[0] &= ~XMT_REQ_REMOTE;
      _TRACE3("CANBUS check_stat_rcv_object_fifo: RemCmdAck(0x%x)\n",u16ObjectNr);
      // release object access
      sync_free_access((spinlock_t *)&pXmtObject->access, xmtIrql);
      
    }
    break;
    
    
  }
  
}

/**
 * *****************************************************************
 * @brief  called, when a transfer over the IN pipe is canceled.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  ctx:  transfer context.
 * @return void
 *
 ******************************************************************
 */

void on_cancel_rcv(CAN_ADDRESS ctx)
{
  PCDEVICES pDevice = (PCDEVICES)ctx;
  
  if(pDevice->canprousb.IoChainRcvState != CHAIN_IDLE)
  {
    _TRACE3("on_cancel_rcv\n");
    pDevice->canprousb.IoChainRcvState = CHAIN_STOPPED_ON_ERROR;
  }
}

/**
 * *****************************************************************
 * @brief  called, when a transfer over the OUT pipe is canceled.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  ctx:  transfer context.
 * @return void
 *
 ******************************************************************
 */

void on_cancel_xmt(CAN_ADDRESS ctx)
{
  PCDEVICES pDevice = (PCDEVICES)ctx;
  
  if(pDevice->canprousb.IoChainXmtState != CHAIN_IDLE)
  {
    _TRACE3("on_cancel_xmt state %u\n", pDevice->canprousb.IoChainXmtState);
    pDevice->canprousb.IoChainXmtState = CHAIN_STOPPED_ON_ERROR;
  }
  
}



/**
 * *****************************************************************
 * @brief  called, when a USB device is plugged.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  intf:  interface data of the unplugged device.
 * @param  id:    device id: information about the device.
 * @return 0 in case of success, an error code in case of an error.
 *
 ******************************************************************
 */


int canprousb_probe(struct usb_interface *intf, const struct usb_device_id *id)
{ 
  struct usb_device *udev = interface_to_usbdev(intf);
  struct usb_host_interface *iface_desc;
  struct usb_endpoint_descriptor *endpoint;
  int i;
  int retval = -ENOMEM;
  CDEVICES *pcDevice = NULL;
  USIGN8 *pu8FifoBuffer = NULL;
  char name[100];
  DEVICEINFO info;
  
  _TRACE1("canprousb_probe: minor=%u name=%s speed=%u\n",intf->minor, intf->dev.init_name, udev->speed);
  
  // check vendor id and device id
  if((id->idVendor != SOFTING_USB_VENDOR) || (id->idProduct != CANPROUSB))
    return -ENODEV;
  
  if(udev->speed != USB_SPEED_HIGH)
  {
    _TRACE1("canprousb_probe: this driver supports only USB high speed!\n");
    return -ENODEV;
  }
  
  pcDevice = kmalloc(sizeof(CDEVICES), GFP_ATOMIC);
  
  if (!pcDevice)
  {
    _ERRPRINT("not enough memory module: %s, line: %u\n", __FILE__, __LINE__); 
    return -ENOMEM;
  }
  
  memset(pcDevice, 0, sizeof(CDEVICES));
  
  
  // simulation of the shared memory similar to the DPRAM based Softing CAN devices.
  // the memory will be allocated in the kernel mode.
  
  pu8FifoBuffer = kmalloc(sizeof(T_CANUSBP_SHARED_MEM), GFP_ATOMIC);
  
  if (!pu8FifoBuffer)
  {
    kfree(pcDevice);
    _ERRPRINT("not enough memory module: %s, line: %u\n", __FILE__,
              __LINE__);
    return -ENOMEM;
  }
  
  memset(pu8FifoBuffer,0,sizeof(T_CANUSBP_SHARED_MEM));
  
  pcDevice->canprousb.pSharedMem = (T_CANUSBP_SHARED_MEM*)pu8FifoBuffer;
  
  pcDevice->canprousb.device.isValid = FALSE;
  
  pcDevice->u32MinorNumber = MINORUNDEFINED;
  pcDevice->u32User = MINORUNDEFINED;
  
  pcDevice->u16DeviceId = id->idProduct;
  pcDevice->u16Socket = 0;
  pcDevice->can_irq = 0;
  pcDevice->can_dpram_physaddress = (CAN_ADDRESS)&pcDevice->canprousb.device; // to make FindDevice() work!
  pcDevice->can_base = 0;
  pcDevice->can_info.irq_flag = 0;
  pcDevice->u8PhysCh = 1;
  pcDevice->u8FwId = 0;
  pcDevice->u8NumOfCh = 1;
  
  // initilization of the FIFO's inside the "simulated shared memory"
  pcDevice->canprousb.pSharedMem->can_pro_usb_fifo_mode.rcv.rd_rcv_fifo = RCV_FIFO_USB_MASK;
  pcDevice->canprousb.pSharedMem->can_pro_usb_fifo_mode.rcv.wr_rcv_fifo = 0;
  pcDevice->canprousb.pSharedMem->can_pro_usb_fifo_mode.xmt.rd_xmt_fifo = XMT_FIFO_USB_MASK;
  pcDevice->canprousb.pSharedMem->can_pro_usb_fifo_mode.xmt.wr_xmt_fifo = 0;
  pcDevice->canprousb.device.intf = intf;
  spin_lock_init(&pcDevice->canprousb.Spinlock);
  
  // initialization of the timers for controlling the USB transfers
  setup_timer(&pcDevice->canprousb.rcv_timer, on_cancel_rcv, (CAN_ADDRESS)&pcDevice->canprousb);
  setup_timer(&pcDevice->canprousb.xmt_timer, on_cancel_xmt, (CAN_ADDRESS)&pcDevice->canprousb);
  
  
  _TRACE2("init semaphore\n");
  sema_init(&pcDevice->cDpramSemaphore, 1);   // only 1 task may hold the semaphore
  _TRACE2("init_waitqueue_head\n");
  init_waitqueue_head(&pcDevice->can_info.read_queue);
  
  pcDevice->bIsStarted = TRUE;
  
  pcDevice->pOtherDevice = NULL;
  
  iface_desc = intf->cur_altsetting;
  
  // setting up of the USB endpoints
  // ep1 IN for receiving async. events: for example receiving CAN messages or time stamps.
  // ep1 OUT for sendung async. commands: for example sending CAN messages.
  // ep2 OUT for sending commands, only one command is possible to a device; command is finished, 
  //         when the response over ep2 IN has been received.
  // ep2 IN for receiving the response on a command.
  
  _TRACE1("canprousb_probe: set up %u endpoints\n",iface_desc->desc.bNumEndpoints);
  
  for (i=0; i<iface_desc->desc.bNumEndpoints; ++i)
  {
    endpoint = &iface_desc->endpoint[i].desc;
    _TRACE1("length=%u, type=0x%x, addr=0x%x, attr=0x%x, size=%u\n",
           endpoint->bLength,
           endpoint->bDescriptorType,
           endpoint->bEndpointAddress,
           endpoint->bmAttributes,
           endpoint->wMaxPacketSize);
    
    if(endpoint->bEndpointAddress==CANUSBP_DAT_IN_ENDPOINT)
    {
      if(endpoint->wMaxPacketSize != MAX_BULK_SIZE) 
      {
        _TRACE1("canprousb_probe: packet size is to small!\n");
        kfree(pu8FifoBuffer);
        kfree(pcDevice);
        return -ENODEV;
      }
      pcDevice->canprousb.device.dat_in_ep = endpoint;
    }
    
    if(endpoint->bEndpointAddress==CANUSBP_DAT_OUT_ENDPOINT)
    {
      if(endpoint->wMaxPacketSize != MAX_BULK_SIZE) 
      {
        _TRACE1("canprousb_probe: packet size is to small!\n");
        kfree(pu8FifoBuffer);
        kfree(pcDevice);
        return -ENODEV;
      }      
      pcDevice->canprousb.device.dat_out_ep = endpoint;
    }
    
    if(endpoint->bEndpointAddress==CANUSBP_CMD_IN_ENDPOINT)
    {
      if(endpoint->wMaxPacketSize != MAX_BULK_SIZE) 
      {
        _TRACE1("canprousb_probe: packet size is to small!\n");
        kfree(pu8FifoBuffer);
        kfree(pcDevice);
        return -ENODEV;
      }
      
      pcDevice->canprousb.device.cmd_in_ep = endpoint;
    }
    
    if(endpoint->bEndpointAddress==CANUSBP_CMD_OUT_ENDPOINT)
    {
      if(endpoint->wMaxPacketSize != MAX_BULK_SIZE) 
      {
        _TRACE1("canprousb_probe: packet size is to small!\n");
        kfree(pu8FifoBuffer);
        kfree(pcDevice);
        return -ENODEV;
      }
      
      pcDevice->canprousb.device.cmd_out_ep = endpoint;
    }
  }
  
  if(!pcDevice->canprousb.device.dat_in_ep || !pcDevice->canprousb.device.dat_out_ep || !pcDevice->canprousb.device.cmd_in_ep || !pcDevice->canprousb.device.cmd_out_ep)
  {
    _TRACE1("error some ep missing!\n");
    kfree(pu8FifoBuffer);
    kfree(pcDevice);
    return -ENODEV;
  }
  
  pcDevice->canprousb.device.cmd_in_urb = usb_alloc_urb(0, GFP_ATOMIC);
  if (!pcDevice->canprousb.device.cmd_in_urb) 
  {
    _TRACE1("Couldn't allocate cmd_in_urb\n");
    kfree(pu8FifoBuffer);
    kfree(pcDevice);
    return -ENODEV;
  }
  
  pcDevice->canprousb.device.pcmd_in_buf = 
  usb_alloc_coherent(udev,
                     sizeof(FW_PACKET),
                     GFP_ATOMIC, 
                     &pcDevice->canprousb.device.cmd_in_urb->transfer_dma);
  
  if (!pcDevice->canprousb.device.pcmd_in_buf) 
  {
    _TRACE1("Couldn't allocate cmd_out_urb\n");
    freeRes(pcDevice);
    kfree(pcDevice);
    pcDevice = NULL;
    return -ENODEV;
  }
  
  
  
  
  
  pcDevice->canprousb.device.cmd_out_urb = usb_alloc_urb(0, GFP_ATOMIC);
  if (!pcDevice->canprousb.device.cmd_out_urb) 
  {
    _TRACE1("Couldn't allocate cmd_out_urb\n");
    freeRes(pcDevice);
    kfree(pcDevice);
    return -ENODEV;
  }
  
  pcDevice->canprousb.device.pcmd_out_buf = 
  usb_alloc_coherent(udev,
                     sizeof(FW_PACKET),
                     GFP_ATOMIC, 
                     &pcDevice->canprousb.device.cmd_out_urb->transfer_dma);
  
  if (!pcDevice->canprousb.device.pcmd_out_buf) 
  {
    _TRACE1("Couldn't allocate pcmd_out_buf\n");
    freeRes(pcDevice);
    kfree(pcDevice);
    return -ENODEV;
  }
  
  
  pcDevice->canprousb.device.dat_in_urb = usb_alloc_urb(0, GFP_ATOMIC);
  if (!pcDevice->canprousb.device.dat_in_urb) 
  {
    _TRACE1("Couldn't allocate dat_in_urb\n");
    freeRes(pcDevice);
    kfree(pcDevice);
    return -ENODEV;
  }
  
  pcDevice->canprousb.device.pdat_in_buf = 
  usb_alloc_coherent(udev,
                     MAX_BULK_SIZE,
                     GFP_ATOMIC, 
                     &pcDevice->canprousb.device.dat_in_urb->transfer_dma);
  
  
  pcDevice->canprousb.device.dat_out_urb = usb_alloc_urb(0, GFP_ATOMIC);
  if (!pcDevice->canprousb.device.dat_out_urb) 
  {
    _TRACE1("Couldn't allocate dat_out_urb\n");
    freeRes(pcDevice);
    kfree(pcDevice);
    return -ENODEV;
  }
  
  pcDevice->canprousb.device.pdat_out_buf = 
  usb_alloc_coherent(udev,
                     MAX_BULK_SIZE,
                     GFP_ATOMIC, 
                     &pcDevice->canprousb.device.dat_out_urb->transfer_dma);
  
  
  
  usb_set_intfdata(intf, pcDevice);
  
  retval = usb_register_dev(intf, &canpro_usb_class);
  if (retval) 
  {
    /* something prevented us from registering this driver */
    _TRACE1("Not able to get a minor for this device.\n");
    freeRes(pcDevice);
    kfree(pcDevice);
    return -ENODEV;
  }
  
  pcDevice->u32MinorNumber = intf->minor;
  major = USB_MAJOR;
  
  if(!InsertDevice(pcDevice))
  {
    freeRes(pcDevice);
    kfree(pcDevice);
    return -ENODEV;
  }
  
  
  /* let the user know what node this device is now attached to */
  _TRACE1("CANpro USB Device #%d now attached to major %d minor %d\n",
         (intf->minor - USB_CAN_MINOR_BASE), USB_MAJOR, intf->minor);
  
  GetDeviceInfo(udev, &info);
  pcDevice->Serialnumber = info.serialnumber;
  sprintf(name,"%09u_%u",pcDevice->Serialnumber, pcDevice->u8PhysCh); 
  
  if(procDir)
  {
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0)
    pcDevice->procFile = create_proc_read_entry(name, S_IRUGO, procDir, ReadProc, pcDevice);
#else
    pcDevice->procFile = create_proc_file(name, procDir, pcDevice);
#endif
  }
  else
  {
    freeRes(pcDevice);
    kfree(pcDevice);
    
    return -EIO;
  }
  
  return 0;
  
}



/**
 * *****************************************************************
 * @brief  called, when a USB device is unplugged.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  intf:  interface data of the unplugged device.
 * @return void
 *
 ******************************************************************
 */

void canprousb_disconnect(struct usb_interface *intf)
{
  
  int minor;
  CDEVICES *pcDevice = (CDEVICES *)usb_get_intfdata(intf);
  
  _TRACE1("canprousb_disconnect minor=%u\n",intf->minor);
  
  if(pcDevice)
  {
    // free application from waiting for CAN events, if the channel is used.
    // and signal an error -> appl. must terminate now!
    pcDevice->can_info.irq_flag = 2; // signal an error
    pcDevice->l2_state = ST_CANL2_INIT; // signal that the channel is no more in configured state.
    wake_up_interruptible(&pcDevice->can_info.read_queue); // and wake the application         
    
    pcDevice->bIsStarted = FALSE;
    
    // stop the tranfer thread, if it is already started.
    StopIoChain(pcDevice);
  }
  
  usb_set_intfdata(intf, NULL);
  
  minor = intf->minor;
  
  /* give back our minor */
  usb_deregister_dev(intf, &canpro_usb_class);
  
  if(pcDevice)
  {  
    del_timer(&pcDevice->canprousb.rcv_timer);
    del_timer(&pcDevice->canprousb.xmt_timer);
    freeRes(pcDevice);  
    if(pcDevice->procFile)
    {
      USIGN8 name[100];
      sprintf(name, "%09u_%u",pcDevice->Serialnumber, 1);
      _TRACE2("removing /proc/%s\n",name);
      remove_proc_entry(name, procDir);
    }
    RemoveDevice(pcDevice->can_dpram_physaddress);
  }
  
  _TRACE1("CANpro USB Device #%d now disconnected\n",
           (minor - USB_CAN_MINOR_BASE));
  

  return;
}

/**
 * *****************************************************************
 * @brief  called, when a USB device is suspended.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  intf:  interface data of the suspended device.
 * @param  msg:   powermanagement message.
 * @return void
 *
 ******************************************************************
 */
static int canprousb_suspend(struct usb_interface *intf, pm_message_t msg)
{
  CDEVICES *pcDevice = (CDEVICES *)usb_get_intfdata(intf);
  int rc = 0;
//   USIGN16 firmwareState = 99;
//   rc = usb_port_suspend(intf, msg);
  
//   GetFirmwareState(pcDevice, &firmwareState);

  if(pcDevice->bIsOpen)
  {
    rc = -1; // prevent supend, if device is in use.
    // free application from waiting for CAN events, if the channel is used.
    // and signal an error -> appl. must terminate now!
//     pcDevice->can_info.irq_flag = 2; // signal an error
//     pcDevice->l2_state = ST_CANL2_INIT; // signal that the channel is no more in configured state.
//     wake_up_interruptible(&pcDevice->can_info.read_queue); // and wake the application   
//     StopIoChain(pcDevice);
//     StopFirmware(pcDevice);
  }
  
  return rc;
}


/**
 * *****************************************************************
 * @brief  called, when a USB device is resumed.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  intf:  interface data of the resumed device.
 * @return void
 *
 ******************************************************************
 */
static int canprousb_resume(struct usb_interface *intf)
{
//   CDEVICES *pcDevice = (CDEVICES *)usb_get_intfdata(intf);
  int rc = 0;
  
//   rc = usb_port_resume(udev, msg);
  
  return rc;
}



/*****************************************************************************/
/* can_init                                                                  */
/*****************************************************************************/
/* FUNCTIONAL DESCRIPTION                                                    */
/*   will be called first when the driver is loaded                          */
/*****************************************************************************/
/* PARAMETER DECRIPTION                                                      */
/*                                                                           */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/

/**
 * *****************************************************************
 * @brief  will be called, on loading of the driver.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @return error code
 *
 ******************************************************************
 */

int can_init(VOID)
{
    _TRACE1("can_init\n");
    procDir = NULL;
    
    _TRACE1("*************************************************************************************\n");    
    _TRACE1("SOFTING CAN PRO USB DRIVER Version %s -- Compiled %s %s\n",CAN_DRIVER_VERSION,__DATE__, __TIME__);
    _TRACE1("*************************************************************************************\n");    

    sema_init(&cDeviceListSemaphore, 1);        // only 1 task may hold the semaphore
    
    procDir = proc_mkdir("softing_canprousb",NULL);
    
    /* register USB driver */
    if (usb_register(&canprousb_driver))
    {
      if(procDir)
      {
        remove_proc_entry("softing_canprousb",NULL);
      }
      
      return -1;
    }
    
   
    
    
    _TRACE1("leaving can_init:\n");
    
    return 0;
}



/*****************************************************************************/
/* can_cleanup                                                               */
/*****************************************************************************/
/* FUNCTIONAL DESCRIPTION                                                    */
/*   will be called first when the driver is unloaded                        */
/*****************************************************************************/
/* PARAMETER DECRIPTION                                                      */
/*                                                                           */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/

/**
 * *****************************************************************
 * @brief  called when the driver is unloaded.  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @return void
 *
 ******************************************************************
 */

VOID  can_cleanup(VOID)
{
  _TRACE1("can_cleanup\n");
  
  usb_deregister (&canprousb_driver);
  
  if(procDir)
  {
    remove_proc_entry("softing_canprousb",NULL);
  }
  _TRACE1("can_cleanup leaving");
}


module_init(can_init);
module_exit(can_cleanup);




