/*
 * Softing CAN PC 104 Driver
 * Copyright (C) 1994-2006, Softing AG, Werner Knorsch
 * 
 * See file CREDITS for list of people who contributed to this
 * project.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * email: support.automation@softing.com
 * Softing AG, Richard-Reitzner-Allee 6, 85540 Haar, Germany
 */


/*
 * $Id: can_device.c,v 1.4 2003/10/31 15:16:37 kno Exp $
 */



/******************************************************************
**  include files
*******************************************************************/
#include "os.h"

#include "sysdep.h"
#include "BaseTypes.h"

#include "can_def.h"

#include "can_device.h"

#include "card_def.h"

#include "fbcard.h"
#include "fldcard.h"
#include "fcancard.h"
#include "fcansja.h"
#include "fboot_cp.h"
#include "fldboard.h"
#include "fcanpci.h"

#include "fbcard2.h"
#include "fldcard2.h"
#include "fcancrd2.h"

#include "fboot104.h"
#include "fld104.h"
#include "fcanpc10.h"

#ifdef CANACXPCI
#include "can_pci.h"
#endif

/******************************************************************
** protoypes  
*******************************************************************/

INT32 CANDR_reinitialize(PCDEVICES pDevice);
INT32 CANDR_reset_chip(PCDEVICES pDevice);


/******************************************************************
**  externals
*******************************************************************/

extern DEVICE_LOAD_DATA dldDeviceLoadData_pcmcia8;
extern DEVICE_LOAD_DATA dldDeviceLoadData_pcmcia16;
extern DEVICE_LOAD_DATA dldDeviceLoadData_pci;
extern DEVICE_LOAD_DATA dldDeviceLoadData_pc104;


/******************************************************************
**  constants
*******************************************************************/


const DEVICE_ID_TABLE16 ditDeviceList[] = {
    {                           // CANcard
     &dldDeviceLoadData_pcmcia8,
     bbcardData,
     bldcardData,
     bcancardData,
     "LOADCARD.INI",            // *StandardIniFile;
     "BCARD.SBN",               // *BootFileDefault;
     "LDCARD.SBN",              // *LoadFileDefault;
     "CANCARD.SBN",             // *AppFileDefault;
     NULL,
     CanResetCard,
     CanEnableIrq,
     TST_OLD,     
     },
    {                           // CANcard-SJA
     &dldDeviceLoadData_pcmcia8,
     bbcardData,
     bldcardData,
     bcansjaData,
     "LOADCARD.INI",            // *StandardIniFile;
     "BCARD.SBN",               // *BootFileDefault;
     "LDCARD.SBN",              // *LoadFileDefault;
     "CANSJA.SBN",              // *AppFileDefault;
     NULL,
     CanResetCard,
     CanEnableIrq,
     TST_20MHZ,     
     },
    {                           // EDICcard-C
     &dldDeviceLoadData_pcmcia8,
     bbcardData,
     bldcardData,
     bcancardData,
     "LOADCARD.INI",            // *StandardIniFile;
     "BCARD.SBN",               // *BootFileDefault;
     "LDCARD.SBN",              // *LoadFileDefault;
     "CANCARD.SBN",             // *AppFileDefault;
     NULL,
     CanResetCard,
     CanEnableIrq,
     TST_OLD,     
     },
    {                           // CANcard2
     &dldDeviceLoadData_pcmcia16,
     bbcard2Data,
     bldcard2Data,
     bcancrd2Data,
     "LOADCARD.INI",            // *StandardIniFile;
     "BOOT_CP.SBN",             // *BootFileDefault;
     "FLDBOARD.SBN",            // *LoadFileDefault;
     "FCANPCI.SBN",             // *AppFileDefault;
     NULL,
     CanResetCard,
     NULL,
     TST_24MHZ,
     },
    {                           // CAN-AC2-PCI
     &dldDeviceLoadData_pci,
     bboot_cpData,
     bldboardData,
     bcanpciData,
     "LOADCARD.INI",            // *StandardIniFile;
     "BOOT_CP.SBN",             // *BootFileDefault;
     "FLDBOARD.SBN",            // *LoadFileDefault;
     "FCANPCI.SBN",             // *AppFileDefault;
     NULL,
     CanResetCard,
     CanEnableIrq,
     TST_25MHZ,
    },
    {                           // CAN-AC2-104
     &dldDeviceLoadData_pc104,
     bboot104Data,
     bld104Data,
     bcanpc104Data,
     "LDCAN104.INI",            // *StandardIniFile;
     "BOOT104.SBN",             // *BootFileDefault;
     "LD104.SBN",               // *LoadFileDefault;
     "CANPC104.SBN",            // *AppFileDefault;
     NULL,
     CanResetCard,
     NULL,
     TST_25MHZ,
     },
};




/*****************************************************************************/
/*
   MODULE:             iGet..Type  Funtions accessing device list

   MODULE_DESCRIPTION  returns the index in the device list for a given 
                               device type and NULL, if device type is unknown
                                                                             */
/*****************************************************************************/
USIGN32 GetIndex(PCDEVICES pDevice)
{
    switch (pDevice->u16DeviceId)
    {
    case PNPDEVICE_CANCARD:
    case CANCARD_DEV_VECTOR:
        return 0;
        break;

    case PNPDEVICE_CANCARD_SJA:
    case CANCARD_SJA_DEV_VECTOR:
        return 1;
        break;

    case PNPDEVICE_EDICCARD_C:
        return 2;
        break;

    case PNPDEVICE_CANCARD_2:
    case CANCARD_2_VECTOR:
    case PNPDEVICE_EDICCARD_2:
        return 3;
        break;

    case SUBDEVICE_ID_CANPCI:
    case SUBDEVICE_ID_CANPCI_DEVNET:
        return 4;
        break;
        
        
    case PNPDEVICE_CAN_AC2_104:
        return 5;
        break;


    default:
        return DEVL_NULL;
        break;

    }
    return DEVL_NULL;
}


USIGN32 GetDeviceId(PVOID pDevice)
{
    USIGN32 ret;
    PCDEVICES pMyDevice = pDevice;
    ret = GetIndex(pMyDevice);
    return ret;
}

PDEVICE_LOAD_DATA
GetDeviceLoadDataPointer(PVOID pDevice)
{
    if (GetDeviceId(pDevice) != DEVL_NULL)
        return ditDeviceList[GetDeviceId(pDevice)].pldLoadData;
    return NULL;
}

SCHAR FAR *
GetSBNBufBootDataPointer(PVOID pDevice)
{
    _TRACE5("GetSBNBufBootDataPointer:\n");
    if (GetDeviceId(pDevice) != DEVL_NULL)
        return ditDeviceList[GetDeviceId(pDevice)].bufSBNBootFile;
    return NULL;
}

SCHAR FAR *
GetSBNBufLoadDataPointer(PVOID pDevice)
{
    if (GetDeviceId(pDevice) != DEVL_NULL)
        return ditDeviceList[GetDeviceId(pDevice)].bufSBNLoadFile;
    return NULL;
}

SCHAR FAR *
GetSBNBufAppDataPointer(PVOID pDevice)
{
    if (GetDeviceId(pDevice) != DEVL_NULL)
        return ditDeviceList[GetDeviceId(pDevice)].bufSBNAppFile;
    return NULL;
}

SCHAR FAR *
GetSBNBufIniFileName(PVOID pDevice)
{
    if (GetDeviceId(pDevice) != DEVL_NULL)
        return ditDeviceList[GetDeviceId(pDevice)].strStandardIniFile;
    return NULL;
}

SCHAR FAR *
GetSBNBufBootDataFileName(PVOID pDevice)
{
    if (GetDeviceId(pDevice) != DEVL_NULL)
        return ditDeviceList[GetDeviceId(pDevice)].strBootFileName;
    return NULL;
}

SCHAR FAR *
GetSBNBufLoadDataFileName(PVOID pDevice)
{
    if (GetDeviceId(pDevice) != DEVL_NULL)
        return ditDeviceList[GetDeviceId(pDevice)].strLoadFileName;
    return NULL;
}

SCHAR FAR *
GetSBNBufAppDataFileName(PVOID pDevice)
{
    if (GetDeviceId(pDevice) != DEVL_NULL)
        return ditDeviceList[GetDeviceId(pDevice)].strAppFileName;
    return NULL;
}

enum enTimeStampType
GetTimeStampType(PVOID pDevice)
{
    if (GetDeviceId(pDevice) != DEVL_NULL)
    {
        _TRACE5("GetTimeStampType returns %x\n",ditDeviceList[GetDeviceId(pDevice)].tstTimeStampType);
        return ditDeviceList[GetDeviceId(pDevice)].tstTimeStampType;
    }
    _ERRPRINT("GetTimeStampType: error, return code = %x\n", TST_NOT_DEFINED);
    return TST_NOT_DEFINED;
}



USIGN16 GetAssignedIRQ_Nr(PVOID pDevice)
{
  PCDEVICES pActDevice = pDevice;
  return pActDevice->can_irq;
}









/*****************************************************************************/
/*
   MODULE:             CS_EnableInterrupt

                       returns 0 if successful
                                                                             */
/*****************************************************************************/
USIGN16 CS_EnableInterrupt(CSHANDLE notUsed, BOOL bDoEnable,
                   USIGN8 _far * pAttrBase, PVOID pDevice)
{
    USIGN16 iRet = 0;

    if (GetDeviceId(pDevice) != DEVL_NULL)
        if (ditDeviceList[GetDeviceId(pDevice)].EnableIrqFkt)
            iRet =
                (ditDeviceList[GetDeviceId(pDevice)].EnableIrqFkt) (bDoEnable,
                                                                    pDevice);
    return iRet;
}

/*****************************************************************************/
/*
   MODULE:             CS_InitCnfReg

                       returns 0 if successful
                                                                             */
/*****************************************************************************/
USIGN16 CS_InitCnfReg(CSHANDLE notUsed, LPBYTE pAttrBase, PCDEVICES pDevice)
{
    USIGN16 iRet = 0;

    if (GetDeviceId(pDevice) != DEVL_NULL)
        if (ditDeviceList[GetDeviceId(pDevice)].InitFkt)
            iRet =
                (ditDeviceList[GetDeviceId(pDevice)].InitFkt) (TRUE, pDevice);
    return iRet;
}


 /*****************************************************************************/
/*
   MODULE:             CS_SetCanCardReset

                       returns 0 if successful
                                                                             */
/*****************************************************************************/
USIGN16 CS_SetCanCardReset(CSHANDLE notUsed, BOOL bDoReset,
                   USIGN8 _far * pAttrBase, PCDEVICES pDevice)
{
    USIGN16 iRet = 0xFFFF;

    if (GetDeviceId(pDevice) != DEVL_NULL)
        if (ditDeviceList[GetDeviceId(pDevice)].ResetFkt)
            iRet =
                (ditDeviceList[GetDeviceId(pDevice)].ResetFkt) (bDoReset,
                                                                pDevice);
    return iRet;

}

/*

  from ini10416.c

*/

PCDEVICES
LookForFreeDevice(PCDEVICES pcCanDevices)
{
    PCDEVICES pCheckDevice = pcCanDevices;
    while (pCheckDevice)
    {
        if (!pCheckDevice->bIsOpen)
        {
            _TRACE1("LookForFreeDevice: device at %lx found!\n",
                    (CAN_ADDRESS) pCheckDevice);
            return pCheckDevice;
        }
        pCheckDevice = pCheckDevice->pNextDevice;

    }
    _TRACE1("LookForFreeDevice: no CAN device found!\n");
    return NULL;

}

INT32 INIDR_close_board(PCDEVICES pDevice)
{
  PCDEVICES pOtherDevice = pDevice->pOtherDevice;
    
  _TRACE5("INIDR_close_board(0x%lx)\n", (CAN_ADDRESS)pDevice);
  if(pOtherDevice)
  {
    _TRACE5("closing channel2\n");
    if(!pOtherDevice->bIsOpen)
      CanEnableIrq(FALSE, pDevice);
  }


  if (pDevice->u8FwId == LAYER2_FW)
  {
    CANDR_reinitialize(pDevice);
    CANDR_reset_chip(pDevice);
  }
    
  pDevice->u8FwId = 0;

  if(pOtherDevice)
  {
    if(!pOtherDevice->bIsOpen)
      CanResetCard(TRUE, pDevice);
  }
     
  return (OK_);
}




/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       INIDR_initialize_board                    */
/* FUNCTIONAL_DESCRIPTION         configuration of hw addresses of dpram    */
/*                                in address area of PC                     */
/* INPUTS                         address                                   */
/* OUTPUTS                        address valid                             */
/* CHANGES                                                                  */
/*                                                                          */
/*HEAD_END*******************************************************************/

int
INIDR_initialize_board(PCDEVICES pDevice)
{
    USIGN8 *dpram_ptr;

    _TRACE5("INIDR_initialize_board(0x%lx)\n",(CAN_ADDRESS)pDevice);


    if (pDevice->can_dpram_address == 0)
    {
        /* no success in allocating a DPRAM pointer */
        INIDR_close_board(pDevice);
        return (INIPC_IB_ERR_ACCESS_DPRAM);
    }


    dpram_ptr = (SCHAR *) pDevice->can_dpram_address;
    _TRACE5("INIDR_initialize_board: DPRAM at: 0x%lx\n",(CAN_ADDRESS)dpram_ptr);

    pDevice->dpram_data.dpram4.prcv_fifo =
        (RCV_FIFO_STRUCT far *) ((CAN_ADDRESS) dpram_ptr + RCV_FIFO_BASE);
    pDevice->dpram_data.dpram4.pxmt_fifo =
        (XMT_FIFO_STRUCT far *) ((CAN_ADDRESS) dpram_ptr + XMT_FIFO_BASE);

    pDevice->fct = (FCT_STRUCT far *) ((CAN_ADDRESS) dpram_ptr + FCT_STRUCT_BASE);

    pDevice->info =
        (INFO_STRUCT far *) ((CAN_ADDRESS) dpram_ptr + INFO_STRUCT_BASE);
    pDevice->int_regs =
        (INT_STRUCT far *) ((CAN_ADDRESS) pDevice->can_dpram_card_address + INT_STRUCT_BASE);

    return (OK_);
}
