/*
    can_proc.c
    handles the proc file access for the CAN-PCI devices.
    Copyright (C) 2015  kno@softing.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/version.h>
#include <linux/init.h>
#include <linux/fs.h>           /* everything... */

#include <linux/cdev.h>


#include <linux/sched.h>



#include <linux/kernel.h>       /* printk() */
#include <linux/errno.h>        /* error codes */
#include <linux/delay.h>        /* udelay */
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/mman.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
//#include <linux/tqueue.h>
#include <linux/sched.h>

#include <linux/pci.h>

#include <linux/netdevice.h>

#include <asm/io.h>

#include "BaseTypes.h"

#include "sysdep.h"

#include <linux/file.h>

/* candriver header files */


#include "can_device.h"
#include "os.h"
#ifdef _CANPROUSB_
#include "shared_usb_mem.h"
#endif
#include "card_def.h"


#include "can_def.h"

#include "can_loadcard.h"
#include "can_card.h"

#include "canlay2.h"
#include "can_ioctl.h"
#include "can_pci.h"
#include "can_proc.h"

extern int major;
struct proc_dir_entry *procDir = NULL, *procFile = NULL;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,10,0)
struct file_operations proc_fops = {
  owner:THIS_MODULE,
  read: ReadProc,
  llseek:SeekProc,
};
#endif



/*****************************************************************************/
/* ReadProc                                                                  */
/*****************************************************************************/
/* FUNCTIONAL DESCRIPTION                                                    */
/*   is called, when someone reads the proc file in /proc/softing_can_pcmcia */
/*   returns the number of bytes read                                        */
/*****************************************************************************/
/* PARAMETER DECRIPTION                                                      */
/*    *buf:   user buffer where the data is stored                           */
/*    offset: start offset in the virtual file                               */
/*    size:   size of the user buffer                                        */
/*    eof:    end of file (true, if all bytes stored in the user buffer)     */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/

#if LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0)

/**
 * *****************************************************************
 * @brief  is called, when someone reads the proc file in /proc/softing_can_usb  
 * @author Werner Knorsch, Softing Industrial Automation GmbH
 * @date   2013-09-18
 *
 * @param  *buf:   user buffer where the data is stored 
 * @param  offset: start offset in the virtual file 
 * @param  size:   size of the user buffer  
 * @param  eof:    end of file (true, if all bytes stored in the user buffer) 
 * @return returns the number of bytes read
 *
 ******************************************************************
 */

int ReadProc(char *buf, char **start, off_t offset, int size, int *eof, void *data)
{
  CDEVICES *pDevice = (CDEVICES *)data;
  USIGN8 PciBuffer[200], channel;
  
  channel = (pDevice->can_dpram_card_address == pDevice->can_dpram_address) ? 1 : 2;
  
  sprintf(PciBuffer, "major: %u minor: %u channel: %u open: %u firmware: %u type: %u irq: %u ioport: 0x%lx dpram: 0x%lx\n", 
          major, 
          pDevice->u32MinorNumber, 
          channel,
          pDevice->bIsOpen,
          pDevice->u8FwId,
          pDevice->u16DeviceId,
          pDevice->can_irq,
          pDevice->can_base,
          pDevice->can_dpram_physaddress);
  
  if(size+offset >= (strlen(PciBuffer)+1))
  { // paÃÂÃÂÃÂÃÂt rein
    if(size+offset <= (strlen(PciBuffer)+1))
    { // size+offset == strlen
      memcpy(buf, &PciBuffer[offset], size);
      *eof = 1;
    }
    else
    {
      memcpy(buf, &PciBuffer[offset], strlen(PciBuffer)+1-offset);
      *eof = 1;
      
    }
  }
  else
  {
    memcpy(buf, PciBuffer, size);
    if(size>0)
      buf[size-1] = 0;
  }
  
  
  return strlen(buf)+1;
}

#else
ssize_t ReadProc(struct file *filp, char *buf, size_t count, loff_t *offset)
{
  CDEVICES *pDevice = (CDEVICES *)PDE_DATA(file_inode(filp));
  USIGN8 PciBuffer[200], channel;

  if(!(pDevice)) 
  {
    _ERRPRINT("no access to device data; module: %s, line: %u\n", __FILE__,__LINE__);
    return 0;
  }    
  channel = (pDevice->can_dpram_card_address == pDevice->can_dpram_address) ? 1 : 2;
  
  sprintf(PciBuffer, "major: %u minor: %u channel: %u open: %u firmware: %u type: %u irq: %u ioport: 0x%lx dpram: 0x%lx\n", 
          major, 
          pDevice->u32MinorNumber, 
          channel,
          pDevice->bIsOpen,
          pDevice->u8FwId,
          pDevice->u16DeviceId,
          pDevice->can_irq,
          pDevice->can_base,
          pDevice->can_dpram_physaddress);
  
//   printk("%lu bytes to copy; offset=0x%lx\n", count, (unsigned long)filp->f_pos);
  
  if(filp->f_pos>strlen(PciBuffer))
    return 0; //eof
  
  if(count >= (strlen(PciBuffer)+1))
  {
    count = strlen(PciBuffer)+1;
  }
  
//   printk("%lu bytes to copy; offset=0x%lx\n", count, *((unsigned long*)offset));
  
  copy_to_user(buf,PciBuffer,count);
  
  *offset = count;
  

  return count;
}



loff_t SeekProc(struct file *filp, loff_t off, int whence)
{
  loff_t newpos;
  
  switch(whence)
  {
    case 0:
      newpos = off;
      break;
      
    case 1:
      newpos = filp->f_pos + off;
      break;
      
    case 2:
      newpos = 199;
      break;
      
    default:
      return -EINVAL;
  }
  
  if(newpos < 0) return -EINVAL;
  
  filp->f_pos = newpos;
  
//   printk("setting offset to %llu\n",newpos);
  return newpos;
}

struct proc_dir_entry *create_proc_file(USIGN8 * name, struct proc_dir_entry *parent, void *data)
{
  struct proc_dir_entry *can_proc_file;
  
  can_proc_file = proc_create_data(name,S_IRUGO, parent, &proc_fops, data);
  
  return can_proc_file;
}

#endif
