/*
 * Softing CAN PCI Driver
 * Copyright (C) 1994-2006, Softing AG, Werner Knorsch
 * 
 * See file CREDITS for list of people who contributed to this
 * project.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * email: support.automation@softing.com
 * Softing AG, Richard-Reitzner-Allee 6, 85540 Haar, Germany
 */

/*
 * $Id: can_device.c,v 1.4 2003/10/31 15:16:37 kno Exp $
 */



/******************************************************************
**  include files
*******************************************************************/


#include "os.h"
#include "sysdep.h"

#include "BaseTypes.h"

#include "can_def.h"

#include "can_device.h"
#ifdef _CANPROUSB_
#include "shared_usb_mem.h"
#endif // _CANPROUSB_
#include "card_def.h"

#include "canL2.h"


/******************************************************************
** protoypes  
*******************************************************************/

INT32 CANDR_reinitialize(PCDEVICES pDevice);
INT32 CANDR_reset_chip(PCDEVICES pDevice);


/******************************************************************
**  externals
*******************************************************************/



/******************************************************************
**  constants
*******************************************************************/

enum enTimeStampType
    GetTimeStampType(void *pDevice)
{
  switch(((PCDEVICES)pDevice)->u16DeviceId)
  {
    case CANCARD2:
    case EDICCARD2:
    case CANPROUSB:
      return TST_24MHZ;
      break;
      
    case CANPCI:
    case CANPCIDN:
    case CAN_104:
      return TST_25MHZ;
      break;
      
    case CANPROPCIE:
    case CAN104PLUS:
    case CANCARDSJA:
      return TST_20MHZ;
      break;
      
    default:
      
      break;
      
      
  }
  
  return TST_NOT_DEFINED;
}


INT32 INIDR_close_board(PCDEVICES pDevice)
{
  PCDEVICES pOtherDevice = pDevice->pOtherDevice;
    
  if(pOtherDevice)
  {
    if(!pOtherDevice->bIsOpen)
      CanEnableIrq(FALSE, pDevice);
  }


  if (pDevice->u8FwId == LAYER2_FW)
  {
    CANDR_reinitialize(pDevice);
    CANDR_reset_chip(pDevice);
  }
    
  pDevice->u8FwId = 0;

  if(pOtherDevice)
  {
    if(!pOtherDevice->bIsOpen)
      CanResetCard(TRUE, pDevice);
  }
     
  return (OK_);
}




/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       INIDR_initialize_board                    */
/* FUNCTIONAL_DESCRIPTION         configuration of hw addresses of dpram    */
/*                                in address area of PC                     */
/* INPUTS                         address                                   */
/* OUTPUTS                        address valid                             */
/* CHANGES                                                                  */
/*                                                                          */
/*HEAD_END*******************************************************************/
#if (defined CANPCIE || defined CANPRO)

int INIDR_initialize_board(PCDEVICES pDevice)
{
  USIGN8 *dpram_ptr;

  _TRACE5("INIDR_initialize_board:\n");

  if (pDevice->can_dpram_address == 0)
  {
    /* no success in allocating a DPRAM pointer */
    INIDR_close_board(pDevice);
    return (INIPC_IB_ERR_ACCESS_DPRAM);
  }


  dpram_ptr = (SCHAR *) pDevice->can_dpram_address;

      // fct structure for commands
  pDevice->fct      =  (FCT_STRUCT *) ((CAN_ADDRESS)dpram_ptr + CAN_PRO_FCT_STRUCT_BASE);
        
    // receive fifo
  pDevice->dpram_data.can_pro_fifo_mode.prcv_fifo  =  (RCV_FIFO_CAN_PRO_STRUCT *)((CAN_ADDRESS)dpram_ptr + CAN_PRO_RCV_FIFO_BASE);
    // transmit fifo
  pDevice->dpram_data.can_pro_fifo_mode.pxmt_fifo  =  (XMT_FIFO_CAN_PRO_STRUCT *)((CAN_ADDRESS)dpram_ptr + CAN_PRO_XMT_FIFO_BASE);
    // and for some other features ...
  pDevice->info     =  (INFO_STRUCT *)((CAN_ADDRESS)dpram_ptr + CAN_PRO_INFO_STRUCT_BASE);
        
  pDevice->canpro.pxmt_poll_list = (USIGN16 *)((CAN_ADDRESS)dpram_ptr + CAN_PRO_XMT_POLL_LIST_BASE);

 
  return (OK_);
}

#endif //_CANPROUSB_