/******************************************************************************

SOFTING AG
Richard-Reitzner-Allee 6
D-85540 Haar
Phone: ++49-89-4 56 56-0
Fax: ++49-89-4 56 56-3 99
http://www.softing.com

Copyright (C) SOFTING AG 2000-2006, All Rights Reserved

Version: 3.11.0.XX.release

******************************************************************************/

#ifndef __ACCESS_H__
#define __ACCESS_H__

/******************************************************************************

! ! !  A T T E N T I O N  ! ! !

       This header file needs  B Y T E  structure alignment.

******************************************************************************/

/******************************************************************************
HOST COMMUNICATION
******************************************************************************/

typedef USIGN16   T_BOOTIF_SIGNAL;

#define BOOTIF_SIGNAL_ACKNOWLEDGE                       ((T_BOOTIF_SIGNAL) 0x0000)

/*--- HOST -> CONTROLLER ----------------------------------------------------*/

#define BOOTIF_SIGNAL_SERVICE_GET_DEVICE_INFO           ((T_BOOTIF_SIGNAL) 0x0001)
#define BOOTIF_SIGNAL_SERVICE_SET_LICENCE_DATA          ((T_BOOTIF_SIGNAL) 0x0002)
#define BOOTIF_SIGNAL_SERVICE_GET_LICENCE_DATA          ((T_BOOTIF_SIGNAL) 0x0003)
#define BOOTIF_SIGNAL_SERVICE_PROGRAM_FLASH             ((T_BOOTIF_SIGNAL) 0x0004)
#define BOOTIF_SIGNAL_SERVICE_TEST_INTERRUPT            ((T_BOOTIF_SIGNAL) 0x0005)
#define BOOTIF_SIGNAL_SERVICE_START_BUILT_IN_SELFTEST   ((T_BOOTIF_SIGNAL) 0x0006)
#define BOOTIF_SIGNAL_SERVICE_SET_FEATURES              ((T_BOOTIF_SIGNAL) 0x0007)
#define BOOTIF_SIGNAL_SERVICE_GET_FEATURES              ((T_BOOTIF_SIGNAL) 0x0008)
#define BOOTIF_SIGNAL_SERVICE_CALCULATE_FLASH_CHECKSUM  ((T_BOOTIF_SIGNAL) 0x0009)
#define BOOTIF_SIGNAL_SERVICE_START_BIST_WITH_ASPC2     ((T_BOOTIF_SIGNAL) 0x000A)

#define BOOTIF_SIGNAL_SERVICE_START_BIST_WITH_TWINCAN   ((T_BOOTIF_SIGNAL) 0x000B)
#define BOOTIF_SIGNAL_SERVICE_START_BIST_WITH_LED       ((T_BOOTIF_SIGNAL) 0x000C)


#define BOOTIF_SIGNAL_INITIALIZE_EEPROM                 ((T_BOOTIF_SIGNAL) 0x4B44) /* "DK" */
#define BOOTIF_SIGNAL_READ_EEPROM                       ((T_BOOTIF_SIGNAL) 0x4757) /* "WG" */

#define BOOTIF_SIGNAL_RESET_FIRMWARE                    ((T_BOOTIF_SIGNAL) 0x4000)

/*---------------------------------------------------------------------------*/

#define BOOT_IF_VERSION_NO_ID             0x0000
#define BOOT_IF_VERSION_IF_ID             0x0101
#define BOOT_IF_VERSION_BOOT_IF_ID        BOOT_IF_VERSION_IF_ID
#define BOOT_IF_VERSION_SERVICE_IF_ID     BOOT_IF_VERSION_IF_ID

/******************************************************************************
HOST INTERFACE
******************************************************************************/

/*--- boot select masks -----------------------------------------------------*/

#define START_UPDATE_FLASH_FIRMWARE  0x474F5250uL             /* "PROG" command to start PROGR firmware */
#define START_PROFIBUS_V5X_FIRMWARE  0x874B2D1EuL             /* command to start PROFIBUS V5x firmware */
#define START_FIRMWARE               0x47325044uL             /* command to start firmware */

#define START_BOOTBLOCK_FIRMWARE     0x544F4F42uL             /* "BOOT" command to start Bootblock firmware */

/*--- boot loader mask ------------------------------------------------------*/

#define BOOT_BLOCK_EXIST             0x544F4F42uL             /* "BOOT" Bootblock-Exist-Flag */

/*---------------------------------------------------------------------------*/

#define BOOTBLOCK_VERSION_SIZE                72
#define FIRMWARE_VERSION_SIZE                 128

//#ifdef WIN32
#pragma pack(push,1)
//#endif

#define CAN_LOWSPEED_BUS  0x01
#define CAN_HIGHSPEED_BUS 0x07
#define CAN_UNKNOWN_BUS   0x00

typedef struct _T_BOOT_IF_DESCR   /* 232 bytes */
{
  volatile USIGN32      HostBootSelectMask;             /* must be located at offset 0x00 */

  USIGN8               Reserved1 [22];                 /* !!! DO NOT USE THIS AREA !!! */

  USIGN32               LiveSign;

  volatile T_BOOTIF_SIGNAL    SignalFromHost;
  volatile T_BOOTIF_SIGNAL    SignalFromController;

  USIGN16              IfId;
  USIGN16              BootIfId;
  USIGN16              ServiceIfId;

  USIGN8               Reserved2 [58];
  USIGN8               BusType1;
  USIGN8               BusType2;
  USIGN16              FirmwareVersionNumber;
  USIGN16              NumberOfChannels;
  USIGN32              SerialNumber;
  USIGN16              DeviceId;
  USIGN16              HwRevision;

  volatile USIGN32     BootLoaderMask;                 /* must be located at offset 0x70 */
  USIGN32              BootLoaderVersion;              /* must be located at offset 0x74 */

  INT8                 BootBlockVersionString [BOOTBLOCK_VERSION_SIZE];
  INT8                 FirmwareVersionString [FIRMWARE_VERSION_SIZE]; /* must be located at offset 0xC0 */

} T_BOOT_IF_DESCR;





/*=== SERVICE INTERFACE =====================================================*/

/* softing profibus device idendifiers */
#define DEVICE_ID_PROFICARD1          0x0010
#define DEVICE_ID_PROFICARD2          0x0011
#define DEVICE_ID_CANPROPCIE          0x000D
#define DEVICE_ID_CANPRO104PLUS       0x000E
#define DEVICE_ID_PROFICARDM          0x0018
#define DEVICE_ID_PROFICARD2M         0x0019
#define DEVICE_ID_PROFIBOARD          0x0020
#define DEVICE_ID_PROFI104            0x0021
#define DEVICE_ID_PROFI104S           0x0022
#define DEVICE_ID_PROFIPRO104         0x0023
#define DEVICE_ID_PROFIPCI            0x0028
#define DEVICE_ID_PROFIPCIS           0x0029
#define DEVICE_ID_PROFIPROPCI         0x002A

#define DEVICE_ID_PROFI_IP_MODULE     0x0089

/* softing profibus revision numbers */
#define DEVICE_REV_PROFICARD1_REV1    0x0100
#define DEVICE_REV_PROFICARD1_REV2    0x0101
#define DEVICE_REV_PROFIBOARD_REV1    0x0100
#define DEVICE_REV_PROFIBOARD_REV2    0x0101
#define DEVICE_REV_PROFIBOARD_REV3    0x0102
#define DEVICE_REV_PROFIBOARD_REV4    0x0103

/* includes 104D_KOP and 104D_STD same like PROFI104, but different chip-and addess select modes */
#define STRING_PROFI104_STD           0x44343031uL  /* 104D */

/* includes PBCOM same like PROFI104, but different chip-and addess select modes */
#define STRING_PROFI104_PBCOM         0x4F434250uL  /* PBCO */

typedef struct _T_DEVICE_INFO
{
  USIGN32               SerialNumber;
  USIGN16              DeviceId;
  USIGN16              HwRevision;

} T_DEVICE_INFO;

/*---------------------------------------------------------------------------*/

#define FEATURE_IS_DISABLED           0x00u
#define FEATURE_IS_ENABLED            0x01u
#define FEATURE_DISABLE               0x02u
#define FEATURE_ENABLE                0x03u
#define FEATURE_DONT_CARE             0x04u

typedef struct _T_LICENCE_DATA
{
  USIGN8               dummy[1000];
  USIGN8               LicenceKey [64];
  USIGN16              Handshake;

} T_LICENCE_DATA;

/*---------------------------------------------------------------------------*/

#define PROFIBRD_IRQ_TO_HOST_NO   0x00u
#define PROFIBRD_IRQ_TO_HOST_5    0x10u
#define PROFIBRD_IRQ_TO_HOST_10   0x20u
#define PROFIBRD_IRQ_TO_HOST_11   0x30u
#define PROFIBRD_IRQ_TO_HOST_12   0x40u
#define PROFIBRD_IRQ_TO_HOST_15   0x50u

typedef struct _T_TEST_INTERRUPT
{
  USIGN8               InterruptNumber;

} T_TEST_INTERRUPT;

/*---------------------------------------------------------------------------*/

#define MAX_BINARY_LENGTH         2048u * 6

#define TYPE_PROGRAM_DATA         0
#define TYPE_PROGRAM_END          1
#define TYPE_INITIALIZE_EEPROM    2

typedef struct _T_PROGRAM_FLASH
{
  USIGN16              Type;

  USIGN32               Offset;
  USIGN32               Length;

  USIGN8               BinaryData [MAX_BINARY_LENGTH];

} T_PROGRAM_FLASH;

/*---------------------------------------------------------------------------*/

typedef struct _T_FLASH_CHECKSUM
{
  USIGN16    BootblockFlashChecksum;
  USIGN16    ProtocolFwFlashChecksum;

} T_FLASH_CHECKSUM;

/*---------------------------------------------------------------------------*/

#define TESTCASE_VERIFY_FLASH_CHECKSUM             1u
#define TESTCASE_RAM_TEST                          2u
#define TESTCASE_DPRAM_TEST                        3u
#define TESTCASE_ASPC2_REGISTER_TEST               4u
#define TESTCASE_ASPC2_TEST                        5u
#define TESTCASE_ASPC2_BUS_TEST                    6u

typedef struct _T_BUILTIN_SELF_TEST
{
  USIGN16    LiveSign;

  USIGN16    TestCase;
  USIGN16    Progess;

  USIGN16    RamValue;
  USIGN32    RamAddress;
   
  USIGN16    StatusCode;

} T_BUILTIN_SELF_TEST;


/*---------------------------------------------------------------------------*/

#define BOOTIF_OK                                      0
#define BOOTIF_ERROR_INVALID_SERIAL_NUMBER            -1
#define BOOTIF_ERROR_EEPROM_WRITE_FAILED              -2
#define BOOTIF_ERROR_EEPROM_READ_FAILED               -3
#define BOOTIF_ERROR_INVALID_LENGTH                   -4
#define BOOTIF_ERROR_INVALID_OFFSET                   -5
#define BOOTIF_ERROR_FLASH_ERASE_ERROR                -6
#define BOOTIF_ERROR_FLASH_WRITE_ERROR                -7
#define BOOTIF_ERROR_FLASH_READ_ERROR                 -8
#define BOOTIF_ERROR_CHECK_SUM_ERROR                  -9
#define BOOTIF_ERROR_INVALID_PROGRAM_TYPE             -10
#define BOOTIF_ERROR_INVALID_INTERRUPT_NUMBER         -11
#define BOOTIF_ERROR_SERVICE_NOT_SUPPORTED            -12
#define BOOTIF_ERROR_BOOTBLOCK_CHECK_SUM_ERROR        -13
#define BOOTIF_ERROR_PROTOCOLFW_CHECK_SUM_ERROR       -14
#define BOOTIF_ERROR_RAM_TEST_READ_UPWARD_ERROR       -15
#define BOOTIF_ERROR_RAM_TEST_READ_DOWNWARD_ERROR     -16
#define BOOTIF_ERROR_DPRAM_TEST_READ_UPWARD_ERROR     -17
#define BOOTIF_ERROR_DPRAM_TEST_READ_DOWNWARD_ERROR   -18
#define BOOTIF_ERROR_ASPC2_REGISTER_ACCESS_ERROR      -19
#define BOOTIF_ERROR_ASPC2_INTERRUPT_ERROR            -20
#define BOOTIF_ERROR_ASPC2_NOT_DETECTED               -21
#define BOOTIF_ERROR_ASPC2_NOT_ONLINE                 -22
#define BOOTIF_ERROR_ASPC2_ACCESS_ERROR               -23
#define BOOTIF_ERROR_UNKNOWN_DEVICE_ID                -24
#define BOOTIF_ERROR_ASPC2_BUS_ACCESS_ERROR           -25
#define BOOTIF_ERROR_RAM_TEST_DATABUS_ERROR           -26
#define BOOTIF_ERROR_RAM_TEST_ADDRESSBUS_ERROR        -27
#define BOOTIF_ERROR_RAM_TEST_ERROR                   -28
#define BOOTIF_ERROR_DPR_TEST_DATABUS_ERROR           -29
#define BOOTIF_ERROR_DPR_TEST_ADDRESSBUS_ERROR        -30
#define BOOTIF_ERROR_DPR_TEST_ERROR                   -31
#define BOOTIF_ERROR_HOST_INTERRUPT_ERROR             -32

#define BOOTIF_ERROR_CAN_COMMUNICATION_A              -33
#define BOOTIF_ERROR_CAN_COMMUNICATION_B              -34
#define BOOTIF_ERROR_CAN_MESSAGE_A                    -35
#define BOOTIF_ERROR_CAN_MESSAGE_B                    -36



typedef struct _T_SERVICE_REQ_IF   /* 2060 bytes */
{
  signed short                ReturnCode;

  union
  {
    T_DEVICE_INFO             DeviceInfo;
    T_LICENCE_DATA            LicenceData;
    T_PROGRAM_FLASH           ProgramFlash;
    T_TEST_INTERRUPT          TestInterrupt;
    T_FLASH_CHECKSUM          FlashChecksum;
    T_BUILTIN_SELF_TEST       Bist;

  }                           UseAs;

} T_SERVICE_REQ_IF;


/*=== DEBUG INTERFACE =======================================================*/

typedef struct _T_TRACE
{
  INT8                        FunctionName [60];

} T_TRACE;

/*---------------------------------------------------------------------------*/

typedef struct _T_DEBUG_LOG
{
  INT8                        VariableName [32];
  USIGN32               VariableValue;

} T_DEBUG_LOG;

/*---------------------------------------------------------------------------*/

typedef struct _T_DEBUG_IF    /* 96 bytes */
{
  T_TRACE                     Trace;
  T_DEBUG_LOG                 DebugLog;

} T_DEBUG_IF;


/*=== HOST SHARED MEMORY ====================================================*/

typedef struct _T_HOST_SHARED_MEMORY
{
  T_BOOT_IF_DESCR             BootIfDescr;
  T_SERVICE_REQ_IF            ServiceReq;
  T_DEBUG_IF                  Debug;

} T_HOST_SHARED_MEMORY;


/******************************************************************************
GLOBAL DATA
******************************************************************************/

extern T_HOST_SHARED_MEMORY    FAR HostSharedMemory;

//#ifdef WIN32
#pragma pack(pop)
//#endif

/*****************************************************************************/

#endif /* __ACCESS_H__ */
