/****************************************************************************/
/*                                                                          */
/*   Softing AG            Richard-Reitzner-Allee 6       85540 Haar        */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*   Copyright (C) by Softing AG, 1997 - 2005   All rights reserved.        */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*                               C A N L  2 . H                             */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* MODULE_DESCRIPTION  CANL2.H                                              */
/* VERSION             4                                                    */
/* DATE                1.12.98                                              */
/*                     include file for CAN Layer 2 Library                 */
/*                     function prototypes and return codes                 */
/*                                                                          */
/****************************************************************************/
#ifndef _CANL2_H
#define _CANL2_H 

#include "can_def.h"
             
#ifdef __cplusplus
extern "C" {
#endif //  __cplusplus

#define CAN_HANDLE unsigned long



#define  CANL2_SOCKET_AUTO   0xFFFF    /* Socket automatically searched     */
#define  CANL2_NO_IRQ        0x0       /* no interrupt requested            */
#define  CANL2_IRQ_AUTO      0xFFFF    /* interrupt automatically assigned  */
#define  CANL2_BASE_AUTO     0x0L      /* DPR-Base automatically assigned   */
#define  CANL2_SIZE_AUTO     0x0L      /* DPR-Size automatically assigned   */
#define  CANL2_AI_AUTO       0x0       /* chip does no matter               */
#define  CANL2_AI_DB8        0x01      /* chip is Databook                  */
#define  CANL2_READ_INIFILE  0xFFFE    /* DOS CAN-AC2-104: reads INI-File   */

/* ---- CAN chip-types -----------------------------------------------------*/
#define NEC005_CHIP            5
#define SJA1000_CHIP        1000


/* ---- Softing CAN Baugruppentypen ----------------------------------------*/


    
typedef enum
{
  CANCARDNEC = 1,
  CANCARDSJA = 4,
  CANCARD2,
  CANPCI   = 7,
  CANPCIDN,
  CAN_104,
  CANUSB,
  CAN104ECO = 12,
  CANPROPCIE,
  CAN104PLUS,
  CANPROUSB,
  EDICCARDC=0x101,
  EDICCARD2=0x105,
  GEFANUCPC8=0x8000,
  FG100CAN = 0x10000000L
} DEVICETYPE;


#define GET_FROM_SCIM -1



#ifdef WIN32
#pragma pack(push,1)                    /* pack alignment must be one        */
#else
#pragma pack(1)                        /* pack alignment must be one        */
#endif

#ifndef CANL2_EXPORTS
#define MAXLENCHNAME 80
typedef struct
{
  USIGN32   u32Serial;   /* Serialnumber */
  USIGN32   u32DeviceType; /* device type */
  USIGN32   u32PhysCh; /* physical channel number (1 or 2) */
  USIGN32   u32FwId; /* id of the loaded firmware */
  BOOLEAN         bIsOpen; /* TRUE if opened by a process */
  TCHAR ChannelName[MAXLENCHNAME];        


} CHDSNAPSHOT, *PCHDSNAPSHOT;

#endif 



typedef struct canl2_ch_s {
    CAN_HANDLE     ulChannelHandle;
    USIGN8  sChannelName[80];
} CANL2_CH_STRUCT;

typedef struct  {
                  USIGN32          Ident;
                  INT32            DataLength;
                  INT32            RecOverrun_flag;
                  INT32            RCV_fifo_lost_msg;
                  USIGN8           RCV_data[8];
                  INT32            AckOverrun_flag;
                  INT32            XMT_ack_fifo_lost_acks;
                  INT32            XMT_rmt_fifo_lost_remotes;
                  INT32            Bus_state;
                  INT32            Error_state;
                  CAN_HANDLE       Can;
                  USIGN32  Time;
                } PARAM_STRUCT;
                
#ifdef LINUX
#define EVT_HANDLE unsigned long
#else
#define EVT_HANDLE HANDLE
#endif
		
typedef struct 
{
  double fBaudrate;
  INT32 s32Prescaler;
  INT32 s32Tseg1;
  INT32 s32Tseg2;
  INT32 s32Sjw;
  INT32 s32Sam;
  INT32 s32AccCodeStd;
  INT32 s32AccMaskStd;
  INT32 s32AccCodeXtd;
  INT32 s32AccMaskXtd;
  INT32 s32OutputCtrl;
  BOOL    bEnableAck;
  BOOL    bEnableErrorframe;
  EVT_HANDLE  hEvent;
} L2CONFIG;


#define  INIPC_IB_ERR_VC_IOCANCELLED   0xFE05   /* driver call cancelled      */
#define  INIPC_IB_ERR_ACCESS_DPRAM    0xFFEA    /* DOS: invalid dpram address
                                                   (above 1MB)
                                                       WIN: failed allocating
                                                       selector              */

                                                       /* -- CAN-AC2 return values                                                 */
                                                       /* -- boot-download error                                                   */

                                                       /* ErrorCode = ErrorBase + ErrorDetail                                      */
                                                       /* ErrorBase codes                                                          */
                                                       /* +--- CANPC_reset_board        -------------------------------------------*/
                                                       /* -- CANcard return values                                                 */
                                                       /* -- return values for access with DOS only                                */
#define  CANPC_RB_INI_FILE           -1 /* can't open IniFile              */
#define  CANPC_RB_ERR_FMT_INI        -2 /* format error in INI-file        */
#define  CANPC_RB_ERR_OP_BIN         -3 /* error opening binary-file       */
#define  CANPC_RB_ERR_RD_BIN         -4 /* error reading binary-file       */
#define  CANPC_RB_BIN_TOO_LONG       -5 /* binary-file too long            */

                                                       /* -- return values for access with DOS and WINDOWS (WIN16, WIN32)          */
#define  CANPC_RB_ERR_BIN_FMT        -6 /* binary-data format error        */
#define  CANPC_RB_ERR_BIN_CS         -7 /* binary-data checksum error      */
#define  CANPC_RB_NO_CARD           -16 /* no card present                 */
#define  CANPC_RB_NO_PHYS_MEM       -17 /* no physical memory              */
#define  CANPC_RB_INVLD_IRQ         -18 /* invalid IRQ-number              */
#define  CANPC_RB_ERR_DPRAM_ACCESS  -19 /* error accessing dpram           */
#define  CANPC_RB_ERR_CRD_RESP      -20 /* bad response from card          */
#define  CANPC_RB_ERR_SRAM          -21 /* sram seems to be damaged        */
#define  CANPC_RB_ERR_PRG           -22 /* invalid program start address   */
#define  CANPC_RB_ERR_REC           -23 /* invalid record type             */
#define  CANPC_RB_ERR_NORESP        -24 /* no response after program start */
#define  CANPC_RB_ERR_BADRESP       -25 /* bad response after program start */
#define  CANPC_RB_PCMCIA_NSUPP      -26 /* pcmcia chip not supported       */
#define  CANPC_RB_ERR_RD_PCMCIA     -27 /* error reading ocmcia parameters */
#define  CANPC_RB_INIT_CHIP         -38 /* error initializing chip         */
#define  CANPC_RB_ERR_NO_CANCARD    -39 /* no CANcard plugged in           */

#define  CANPC_RB_ERR_DEV_LD        -40 /* CAN device not properly load    */

                                                       /* -- CAN-AC2 return values                                                 */
                                                       /* -- boot-download error                                                   */

                                                       /* ErrorCode = ErrorBase + ErrorDetail                                      */
                                                       /* ErrorBase codes                                                          */
#define CANPC_RB_BOOT_BASE         -100 /* boot-data errors                */
#define CANPC_RB_LOAD_BASE         -200 /* load-data errors                */
#define CANPC_RB_APP_BASE          -300 /* application-data errors         */
#define CANPC_RB_CPY_BASE          -400 /* copy-data errors                */
#define CANPC_RB_RUN_BASE          -500 /* run-data errors                 */
                                                       /* ErrorDetail codes                                                        */
#define  CANPC_RB_NO_RESPONSE        -1 /* no response from CAN-AC2        */
#define  CANPC_RB_ERR_FMT            -8 /* boot-data format error          */
#define  CANPC_RB_ERR_INT1           -2 /* internal error 1                */
#define  CANPC_RB_ERR_INT2           -3 /* internal error 2                */
#define  CANPC_RB_ERR_INT3           -4 /* internal error 3                */
#define  CANPC_RB_ERR_INT4           -9 /* internal error 4                */
#define  CANPC_RB_ERR_INT5          -10 /* internal error 5                */
#define  CANPC_RB_ERR_OPEN           -5 /* file open error (DOS only)      */
#define  CANPC_RB_ERR_READ           -6 /* file read error (DOS only)      */
#define  CANPC_RB_ERR_CLOSE          -7 /* file close error (DOS only)     */




#ifdef WIN32
#pragma pack(pop)                    /* reset pack alignment to prev. value */
#else
#pragma pack()                       /* reset pack alignment to default     */
#endif

#ifndef _ERROR_BASES_
#define _ERROR_BASES_

#define INFORMATIONAL_BASE          (0x60000000)
#define WARNING_BASE                (0xA0000000)
#define ERROR_BASE                  (0xE0000000)

#endif

#define L2_ERROR_BASE              (0x000B0000)
#define VCARD_ERROR_BASE           (0x000A0000)


#define MK_VCARD_ERROR_CODE(VCARD_ERROR) ((VCARD_ERROR) ? (VCARD_ERROR_BASE | ERROR_BASE | VCARD_ERROR) : 0)


    /*///////////////////////////////////////////////////////////////////////////
    //  VCARD ERROR CODES (Error codes from kernelmode driver)
    */
#ifndef CANL2_EXPORTS

#define VCARD_OK                         (USIGN32)0                        // success
#define VCARD_INTERNALERROR              (USIGN32)0xE00A0001   // internal Error
#define VCARD_GENERALERROR               (USIGN32)0xE00A0002   // general Error
#define VCARD_TIMEOUT                    (USIGN32)0xE00A0003   // Timeout
#define VCARD_IOPENDING                  (USIGN32)0xE00A0004   // pending driver call
#define VCARD_IOCANCELLED                (USIGN32)0xE00A0005   // driver call was canceled
#define VCARD_ILLEGALCALL                (USIGN32)0xE00A0006   // illegal driver call
#define VCARD_NOTSUPPORTED               (USIGN32)0xE00A0007   // not implemented
#define VCARD_VERSIONERROR               (USIGN32)0xE00A0008   // driver interface dll has the wrong version
#define VCARD_DRIVERVERSIONERROR         (USIGN32)0xE00A0009   // wrong driver version
#define VCARD_DRIVERNOTFOUND             (USIGN32)0xE00A000A   // driver not loaded / not installed, or device is not plugged
#define VCARD_NOTENOUGHMEMORY            (USIGN32)0xE00A000B   // out of memory
#define VCARD_TOOMANYDEVICES             (USIGN32)0xE00A000C   // driver can not handle as much devices
#define VCARD_UNKNOWNDEVICE              (USIGN32)0xE00A000D   // unknown device
#define VCARD_DEVICEALREADYEXISTS        (USIGN32)0xE00A000E   // device already exists
#define VCARD_DEVICEACCESSERROR          (USIGN32)0xE00A000F   // access is not possible: device is already open
#define VCARD_RESOURCEALREADYREGISTERED  (USIGN32)0xE00A0010   // ressorce is in use by an other device
#define VCARD_RESOURCECONFLICT           (USIGN32)0xE00A0011   // ressorce conflict
#define VCARD_RESOURCEACCESSERROR        (USIGN32)0xE00A0012   // ressorce can not be accessed
#define VCARD_PHYSMEMORYOVERRUN          (USIGN32)0xE00A0013   // ungueltiger Zugriff auf physikalischen Speicher
#define VCARD_TOOMANYPORTS               (USIGN32)0xE00A0014   // zu viele I/O Ports 20
#define VCARD_INTERRUPTERROR             (USIGN32)0xE00A0015   // error while activating the interrupt
#define VCARD_UNKNOWNRESOURCE            (USIGN32)0xE00A0016   // unknown ressorce
#define VCARD_CREATEDEVFAILED            (USIGN32)0xE00A0017   // IoCreateDevice failed
#define VCARD_ATTACHTODEVSTACKFAILED     (USIGN32)0xE00A0018   // IoAttachDeviceToDeviceStack failed
#define VCARD_SYMBOLICLINKFAILED         (USIGN32)0xE00A0019   // failed to create a symbolic link
        //////////////////////////////////////////////////////////////////////
        ///    Errors which can occur while downloading the firmware
        //////////////////////////////////////////////////////////////////////
#define VCARD_NOCARDSERVICES             (USIGN32)0xE00A001A   // can not access card services under Win '98
#define VCARD_CARDSERVICESVERSION        (USIGN32)0xE00A001B   // wrong version of the card services under Win '98
#define VCARD_CARDSERVICESGETINFO        (USIGN32)0xE00A001C   // error while accessing the card services under Win '98
#define VCARD_DEVICENOTFOUND             (USIGN32)0xE00A001D   // device not found.
#define VCARD_NODPRAM                    (USIGN32)0xE00A001E   // can not get a free address region for DPRAM from system
#define VCARD_CONTROLHWERROR             (USIGN32)0xE00A001F   // Error while accessing hardware
#define VCARD_SBNCHECKSUM                (USIGN32)0xE00A0020   // Checksum error in SBN format (dll binary may be corrupt)
#define VCARD_DPRAMACCESS                (USIGN32)0xE00A0021   // can not access the DPRAM memory
#define VCARD_CARDREACTION               (USIGN32)0xE00A0022   // Loader program for firmware download does no more react.
#define VCARD_NOSTARTADDRESS             (USIGN32)0xE00A0023   // No startaddress defined in SBN (dll binary may be corrupt)
#define VCARD_NOINTERRUPT                (USIGN32)0xE00A0024   // Interrupt does not work
        //////////////////////////////////////////////////////////////////////
        ///    Errors which can occur in the channel driver
        //////////////////////////////////////////////////////////////////////

#define VCARD_DRIVERNOTPRESENT           (USIGN32)0xE00A0025   // Kernelmode driver is not loaded
#define VCARD_DEVICEISOPEN               (USIGN32)0xE00A0026   // Device is already open
#define VCARD_DEVICELOCKINGERROR         (USIGN32)0xE00A0027   // Device can not be locked
#define VCARD_OTHERFWISLOADED            (USIGN32)0xE00A0028   // A other firmware is running on that device (CANalyzer/CANopen/DeviceNet firmware)
#define VCARD_CHANNELNOTOPEN             (USIGN32)0xE00A0029   // channel can not be accessed, because it is not open.

        //////////////////////////////////////////////////////////////////////
        ///    status codes for CANusb Device Driver canusbw.sys
        //////////////////////////////////////////////////////////////////////

#define VCARD_PNPCALLERROR               (USIGN32)0xE00A002A   // lower driver call in PnP process fails
#define VCARD_URBRETERROR                (USIGN32)0xE00A002B   // URB returns USBD_ERROR code
#define VCARD_ERRORDEVICEDESC            (USIGN32)0xE00A002C   // faulty device desc or read failed
#define VCARD_ERRORCONFIGDESC            (USIGN32)0xE00A002D   // faulty config desc or read failed
#define VCARD_ERRORSELECTCONFIG          (USIGN32)0xE00A002E   // unable to select configuration
#define VCARD_ERRORDECONFIG              (USIGN32)0xE00A002F   // unable to deconfigure the device
#define VCARD_PIPEACCESSERROR            (USIGN32)0xE00A0030   // unable to open usb pipe
#define VCARD_COMMUNICATIONBROKEN        (USIGN32)0xE00A0031   // communication via usb pipe broken off

        //////////////////////////////////////////////////////////////////////
        ///    Errors which can occur in the canchd.dll
        //////////////////////////////////////////////////////////////////////
#define VCARD_FILENOTFOUND               (USIGN32)0xE00A0032   // file not found
#define VCARD_ACCESSRIGHT                (USIGN32)0xE00A0033

#define VCARD_OSERROR                    (USIGN32)0xE00A0034   // error in OS call
#define VCARD_DEVICEIDMISMATCH           (USIGN32)0xE00A0035   // wrong device id stored in registry
#define VCARD_MAXNUMOFCHANNELSOPEN       (USIGN32)0xE00A0036   // the maximum number of channels are open
#define VCARD_INVALIDHANDLE              (USIGN32)0xE00A0037   // a invalid handle is specified



#endif

/* ---- FUNCTION PROTOTYPES AND FUNCTION RETURN VALUES  --------------------*/
/* default return values -- valid for all functions (if no other value is 
   described)                                                               */
/* default return values -- valid for all functions                         */
#define  CANL2_OK                          0  /* function successful        */
#define  CANL2_ERR                        -1  /* function not successful    */
#define  CANL2_BOARD_NOT_INITIALIZED     -99  /* board not initialized:
                                                 INIL2_initialize_channel(.) 
                                                     was not yet called, or a 
                                                     INIL2_close_board was done */
#define CANL2_INVALID_CHANNEL_HANDLE     -1000 /* the channel handle is invalid */
#define CANL2_TOO_MANY_CHANNELS          -1002 /* no free channel available */
#define CANL2_VERSION_CONFLICT           -1003 /* wrong dll or driver version */
#define CANL2_FIRMWARE_DOWNLOAD          -1004 /* error while loading the firmware
                                                         may be a DPRAM access error. */

#define CANL2_CAN_NOT_LOAD_CANUSBDLL     -1005 /* the canusbm.dll can not be found/loaded */


#define INIL2_FW_FILE_NOT_FOUND          -2000 /* firmware file not found */
#define INIL2_ERROR_WHILE_FLASHING       -2001 /* flashing error */





                                                         /* ---- CANusb return values                                                */

#define FRW_ERROR_PIPEACCESS_ERROR       -602 /* unable to open usb pipe    */ 
#define FRW_ERROR_COMMUNICATIONBROKEN    -603 /* communication via usb pipe
                                                         broken                     */
#define FRW_ERR_NOVALID_LUPT_ENTRY       -604 /* no valid lookup table entry 
                                                         chosen                     */

                                                         /* ---- INIL2_initialize_board   -------------------------------------------*/

                                                         /* -- CANusb return values                                                  */
#define FRW_IF_ERR_FRWINIT_FAILED        -611 /* CANusb Framework 
                                                         initialization failed      */
       
#define CANL2_LIC_ERR           -101
#define CANL2_WRONG_PARAM         -102  // error in a parameter value
#define CANL2_FW_TIMEOUT          -104  // timout while communicating with the firmware
#define CANL2_HW_ERR            -108  // firmware runs on the wrong hardware
#define CANL2_DC_NO_DYN_OBJBUF        -109  // no object dyn. buffer 
#define CANL2_REQ_PENDING                   -110  // last request pending

#define CANL2_DATA_OVR            -111  // receive data overrun           
#define CANL2_REMOTE_OVR          -112  // rec. rem. frame overrun        
#define CANL2_NOT_ACTIVE          -113  // object not active              
#define CANL2_RXD_XMT_OVR                   -114  // transmit ack. overrun         
#define CANL2_OB_NOT_DEFINED        -115  // object not defined
#define CANL2_XMT_REQ_OVR                   -116  // xmt request fifo overrun
#define CANL2_NOT_IMPLEMENTED               -117  // not supported for this device

// in this implementation the CAN handle is the same as the file descriptor
// for driver access; Please use the following macro for compatibility with
// later versions of this API (see sample program can_tst)
#define CANL2_handle_to_descriptor(handle) handle

PRAEDEF INT32 MIDDEF INIL2_initialize_channel(CAN_HANDLE *pu32ChannelHandle, char *pChannelName);


PRAEDEF INT32 MIDDEF CANL2_reset_board(CAN_HANDLE Can);

#ifdef WIN32
// +--- CANL2_set_interrupt_event ------------------------------------------
PRAEDEF INT32 MIDDEF CANL2_set_interrupt_event(CAN_HANDLE Can, HANDLE hEvent);

// +--- CANL2_set_interrupt_event_name -------------------------------------
PRAEDEF INT32 MIDDEF CANL2_set_interrupt_event_name(CAN_HANDLE Can, TCHAR *eventName);
#endif
// +--- CANL2_reset_chip         -------------------------------------------
PRAEDEF INT32 MIDDEF CANL2_reset_chip(CAN_HANDLE Can);


// +--- CANL2_initialize_chip(CAN_HANDLE Can, 2) -------------------------------------------
PRAEDEF INT32 MIDDEF CANL2_initialize_chip (CAN_HANDLE Can, INT32 presc,
                               INT32 sjw,
                               INT32 tseg1,
                               INT32 tseg2,
                               INT32 sam);



// +--- CANL2_set_mode(CAN_HANDLE Can, 2)        -------------------------------------------
PRAEDEF INT32 MIDDEF CANL2_set_mode (CAN_HANDLE Can, INT32 SleepMode,
                        INT32 SpeedMode);

/* +--- CANL2_set_acceptance(CAN_HANDLE Can, 2)  ------------------------------------------- */
PRAEDEF INT32 MIDDEF CANL2_set_acceptance (CAN_HANDLE Can, USIGN32 AccCodeStd,
                              USIGN32 AccMaskStd,
                              USIGN32 AccCodeXtd,
                              USIGN32 AccMaskXtd);

/* +--- CANL2_set_output_control(CAN_HANDLE Can, 2)  --------------------------------------- */
PRAEDEF INT32 MIDDEF CANL2_set_output_control (CAN_HANDLE Can, INT32 OutputControl);


// +--- CANL2_initialize_interface   ---------------------------------------

PRAEDEF INT32 MIDDEF CANL2_initialize_interface(CAN_HANDLE Can, INT32 ReceiveFifoEnable,
                                   INT32 ReceivePollAll,
                                   INT32 ReceiveEnableAll,
                                   INT32 ReceiveIntEnableAll,
                                   INT32 AutoRemoteEnable,
                                   INT32 TransmitReqFifoEnable,
                                   INT32 TransmitPollAll,
                                   INT32 TransmitAckEnableAll,
                                   INT32 TransmitAckFifoEnable,
                                   INT32 TransmitRmtFifoEnable);

// +--- CANL2_define_object(CAN_HANDLE Can, 2)          ------------------------------------

PRAEDEF INT32 MIDDEF CANL2_define_object (CAN_HANDLE Can, USIGN32 Handle,
                             INT32 FAR * ObjectNumber,
                             INT32 Type,
                             INT32 ReceiveIntEnable,
                             INT32 AutoRemoteEnable,
                             INT32 TransmitAckEnable);


// +--- CANL2_define_cyclic(CAN_HANDLE Can, 2)        --------------------------------------

PRAEDEF INT32 MIDDEF CANL2_define_cyclic (CAN_HANDLE Can, INT32 ObjectNumber,
                             USIGN32 Rate,
                             USIGN32 Cycles);


// +--- CANL2_set_rcv_fifo_size      ---------------------------------------


PRAEDEF INT32 MIDDEF CANL2_set_rcv_fifo_size(CAN_HANDLE Can,  INT32 FifoSize);


// +--- CANL2_enable_fifo            ---------------------------------------

#define CANL2_WRONG_MODE                    -1001
PRAEDEF INT32 MIDDEF CANL2_enable_fifo(CAN_HANDLE Can);

// +--- CANL2_optimize_rcv_speed     ---------------------------------------
// this function has no effect on the CANcard                               

PRAEDEF INT32 MIDDEF CANL2_optimize_rcv_speed(CAN_HANDLE Can);

// +--- CANL2_enable_dyn_obj_buf     ---------------------------------------

PRAEDEF INT32 MIDDEF CANL2_enable_dyn_obj_buf(CAN_HANDLE Can);

// +--- CANL2_enable_fifo_transmit_ack  ------------------------------------

PRAEDEF INT32 MIDDEF CANL2_enable_fifo_transmit_ack (CAN_HANDLE Can);



PRAEDEF INT32 MIDDEF CANL2_initialize_fifo_mode(CAN_HANDLE Can, L2CONFIG *pUserCfg);

PRAEDEF INT32 MIDDEF CANL2_get_all_CAN_channels(USIGN32 u32ProvidedBufferSize,
                                              OUT PUSIGN32 pu32NeededBufferSize,
                                              OUT PUSIGN32 pu32NumOfChannels,
                                              OUT PCHDSNAPSHOT pBuffer);




// +--- CANL2_get_version            ---------------------------------------
// the parameter hw_version is not used with the CAN-AC2                    

PRAEDEF INT32 MIDDEF CANL2_get_version(CAN_HANDLE Can, INT32 FAR * sw_version, 
                                       INT32 FAR * fw_version,
                                       INT32 FAR * hw_version,
                                       INT32 FAR * license,
                                       INT32 FAR * can_chip_type);

// +--- CANL2_get_serial_number      ---------------------------------------
// this function has no effect on the CAN-AC2                               

PRAEDEF INT32 MIDDEF CANL2_get_serial_number(CAN_HANDLE Can, USIGN32 FAR * ser_number);

// +--- CANL2_start_chip             ---------------------------------------
// -- CANusb return values                                            
#define FRW_IOE_ERR_NOTENOUGHMEMORY     -612    // not enough memory  
#define FRW_IOE_ERR_INITFAILED          -613    // failed to initialize FRWOrder/Event   

PRAEDEF INT32 MIDDEF CANL2_start_chip(CAN_HANDLE Can);

// +--- CANL2_send_remote_object(CAN_HANDLE Can, 2)  ---------------------------------------

PRAEDEF INT32 MIDDEF CANL2_send_remote_object (CAN_HANDLE Can, INT32 ObjectNumber,
                                  INT32 DataLength);


// +--- CANL2_supply_object_data(CAN_HANDLE Can, 2)  ---------------------------------------

PRAEDEF INT32 MIDDEF CANL2_supply_object_data (CAN_HANDLE Can, INT32  ObjectNumber,
                                  INT32  DataLength,
                                  USIGN8 FAR * pData);


// +--- CANL2_supply_rcv_object_data(CAN_HANDLE Can, 2)-------------------------------------

PRAEDEF INT32 MIDDEF CANL2_supply_rcv_object_data (CAN_HANDLE Can, INT32  ObjectNumber,
                                      INT32  DataLength,
                                      USIGN8 FAR * pData);


// +--- CANL2_send_object(CAN_HANDLE Can, 2)         ---------------------------------------

PRAEDEF INT32 MIDDEF CANL2_send_object (CAN_HANDLE Can, INT32  ObjectNumber,
                           INT32  DataLength);

// +--- CANL2_write_object(CAN_HANDLE Can, 2)        ---------------------------------------

PRAEDEF INT32 MIDDEF CANL2_write_object (CAN_HANDLE Can, INT32  ObjectNumber,
                            INT32  DataLength,
                            USIGN8 FAR * pData);


 

// +--- CANL2_read_rcv_data(CAN_HANDLE Can, 2)       ---------------------------------------

PRAEDEF INT32 MIDDEF CANL2_read_rcv_data (CAN_HANDLE Can, INT32  ObjectNumber,
                             USIGN8 FAR * pRCV_Data,
                             USIGN32 FAR * Time);



// +--- CANL2_read_xmt_data(CAN_HANDLE Can, 2)       ---------------------------------------

PRAEDEF INT32 MIDDEF CANL2_read_xmt_data (CAN_HANDLE Can, INT32  ObjectNumber,
                             INT32 FAR * pDataLength,
                             USIGN8 FAR * pXMT_Data);



// +--- CANL2_read_ac                ---------------------------------------*/
#define  CANL2_RA_NO_DATA                 0  // no new data received        */    
#define  CANL2_RA_DATAFRAME               1  // std. data frame received    */    
#define  CANL2_RA_REMOTEFRAME             2  // std. remote frame received  */    
#define  CANL2_RA_TX_DATAFRAME            3  // transmission of std. data-
                                             //   frame is confirmed          */    
#define  CANL2_RA_TX_FIFO_OVR             4  // remote tx fifo overrun      */    
#define  CANL2_RA_CHG_BUS_STATE           5  // change of bus status        */    
#define  CANL2_RA_ERR_ADD                 7  // additional error causes     */    
#define  CANL2_RA_TX_REMOTEFRAME          8  // transmission of std. data-
                                             //   frame is confirmed          */    
#define  CANL2_RA_XTD_DATAFRAME           9  // xtd. data frame received    */    
#define  CANL2_RA_XTD_TX_DATAFRAME       10  // transmission of xtd. data-
                                             //   frame is confirmed          */    
#define  CANL2_RA_XTD_TX_REMOTEFRAME     11  // transmission of xtd. remote-
                                             //   frame is confirmed          */    
#define  CANL2_RA_XTD_REMOTEFRAME        12  // xtd. remote frame received 
                                             //   (only CANcard)              */    
#define  CANL2_RA_ERRORFRAME             15  // error frame detected  
                                             //   (only CANcard)              */    

PRAEDEF INT32 MIDDEF CANL2_read_ac(CAN_HANDLE Can, PARAM_STRUCT* param);

// +--- CANL2_send_data              ---------------------------------------*/

PRAEDEF INT32 MIDDEF CANL2_send_data (CAN_HANDLE Can, USIGN32 Ident,
                         INT32 Xtd,
                         INT32 DataLength,
                         USIGN8* pData);



/* +--- CANL2_send_remote            ---------------------------------------*/


PRAEDEF INT32 MIDDEF CANL2_send_remote (CAN_HANDLE Can, USIGN32 Ident,
                           INT32 Xtd,
                           INT32 DataLength);

 

/* +--- CANL2_get_trigger(CAN_HANDLE Can, 2)            ---------------------------------------*/
/* these functions have no effect on the CANcard                               */
PRAEDEF INT32 MIDDEF CANL2_get_trigger (CAN_HANDLE Can, INT32 FAR * level);



/* +--- CANL2_reinitialize           ---------------------------------------*/
PRAEDEF INT32 MIDDEF CANL2_reinitialize(CAN_HANDLE Can);

/* +--- CANL2_get_time               ---------------------------------------*/
PRAEDEF INT32 MIDDEF CANL2_get_time(CAN_HANDLE Can, USIGN32 FAR * time);

/* +--- CANL2_get_bus_state          ---------------------------------------*/
#define  CANL2_GBS_ERROR_ACTIVE            0  /* error active               */    
#define  CANL2_GBS_ERROR_PASSIVE           1  /* error passive              */    
#define  CANL2_GBS_ERROR_BUS_OFF           2  /* bus off                    */    

PRAEDEF INT32 MIDDEF CANL2_get_bus_state(CAN_HANDLE Can);


/* +--- CANL2_reset_rcv_fifo         ---------------------------------------*/
PRAEDEF INT32 MIDDEF CANL2_reset_rcv_fifo(CAN_HANDLE Can);

/* +--- CANL2_reset_xmt_fifo         ---------------------------------------*/
PRAEDEF INT32 MIDDEF CANL2_reset_xmt_fifo(CAN_HANDLE Can);

/* +--- CANL2_reset_lost_msg_counter  --------------------------------------*/
PRAEDEF INT32 MIDDEF CANL2_reset_lost_msg_counter(CAN_HANDLE Can);

/* +--- CANL2_read_rcv_fifo_level    ---------------------------------------*/
/* returns number of items in receive fifo   */
PRAEDEF INT32 MIDDEF CANL2_read_rcv_fifo_level(CAN_HANDLE Can);

/* +--- CANL2_read_xmt_fifo_level    ---------------------------------------*/
/* returns number of items in transmit fifo  */
PRAEDEF INT32 MIDDEF CANL2_read_xmt_fifo_level(CAN_HANDLE Can);


/* +--- INIL2_close_board            ---------------------------------------*/
PRAEDEF INT32 MIDDEF INIL2_close_channel(CAN_HANDLE Can);

/* +--- CANL2_enable_error_frame_detection  --------------------------------*/
/* this function has no effect on the CAN-AC2                               */
PRAEDEF INT32 MIDDEF CANL2_enable_error_frame_detection(CAN_HANDLE Can);

/* +--- CANL2_get_device_id  -----------------------------------------------*/
PRAEDEF INT32 MIDDEF CANL2_get_device_id(CAN_HANDLE Can, USIGN32 *pulDeviceId);

/* +--- CANL2_init_signals/CANL2_read_signals/CANL2_write_signals            */
/* these functions are only available on the CAN-ACx-PCI. They are designed  */
/* for use with the CAN-ACx-PCI and piggy-back boards. These functions are   */
/* not in the standard users manual. For description read the piggy-back     */
/* manuals                                                                   */
PRAEDEF INT32 MIDDEF CANL2_init_signals (CAN_HANDLE Can, USIGN32 ulChannelDirectionMask,
                                         USIGN32 ulChannelOutputDefaults);

PRAEDEF INT32 MIDDEF CANL2_read_signals  (CAN_HANDLE Can, USIGN32 *pulChannelRead);

PRAEDEF INT32 MIDDEF CANL2_write_signals  (CAN_HANDLE Can, USIGN32 pulChannelWrite, 
                                          USIGN32 ulCareMask);

#ifdef __cplusplus
}
#endif //  __cplusplus

#endif  /*_CANL2_H */


