/*
 * Can-Driver for Softing Can-Card
 * Copyright (C) 1994-2003, Softing AG, Gunnar Larisch, Werner Knorsch
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * email: support.automation@softing.com
 * Softing AG, Richard-Reitzner-Allee 6, 85540 Haar, Germany
 */

/*
 * $Id: can_device.h,v 1.3 2003/10/31 15:16:38 kno Exp $
 */

#ifndef _DEVICE_H
#define _DEVICE_H

/******************************************************************
**  include files
*******************************************************************/

#include "canL2.h"

#include "can_loadcard.h"



/******************************************************************
**  macros
*******************************************************************/

// ----------------------------------------------------------------------------
// - DEFINITIONEN: DEVICES
#define PNPDEVICE_UNKNOWN           0x0000

// ----------------------------------------------------------------------------
// - CANcard (CANcard (NEC), CANcard-SJA, CANcard2) Device ID's





// PnP-Typen entsprechend Softing-Baugruppentypen
#define PNPDEVICE_CANCARD           0x0001L     // CANcard
#define PNPDEVICE_CANCARD_SJA       0x0004L     // CANcard-SJA
#define PNPDEVICE_CANCARD_2         0x0005L     // CANcard-2
#define CANCARD_DEV_VECTOR          0x81
#define CANCARD_SJA_DEV_VECTOR      0x84
#define CANCARD_2_VECTOR            0x85


// ----------------------------------------------------------------------------
// - EDICcard-C und EDICcard2 Device ID's

// PnP-Typen entsprechend Softing-Baugruppentypen ("deviceContext->hwData")
#define PNPDEVICE_EDICCARD_C        0x0102L     // EDICcard-C-NEC (PCMCIA)
#define PNPDEVICE_EDICCARD_2        0x0105L     // EDICcard-2 (PCMCIA)

// ----------------------------------------------------------------------------
// - CAN-ACx-104 Device ID's
#define PNP_104_VENDOR_ID               0x0000L // (keine Venor ID existent)

// Baugruppentypen ("deviceContext->hwData")
#define PNPDEVICE_CAN_AC2_104       0x0009L     // CAN-AC2-104

///// PC104 Register Bitmasken

// Registerbyte offset 0
#define R0_MEMCS16SA                 0x40
#define R0_MEMCS16LA                 0x00
#define R0_ENABLE_DPRAM              0x02
#define R0_C165_RESET_RUN            0x01
#define R0_DPR_MASK_A15_14           0x18

// Registerbyte offset 2
#define R2_DPR_MASK_A13_12           0x03

// Masken zur DPR - Maskierung
#define DPR_MASK_A23_16              0x00FF0000L
#define DPR_MASK_A15_14              0x0000C000L
#define DPR_MASK_A13_12              0x00003000L

// PCMCIA CANcard ...

#define PCMCIA_SOFTING              0x168
#define CANCARD_DEV                 0x1
#define CANCARD_SJA_DEV             0x4
#define CANCARD_2                   0x5



#define SUBDEVICE_ID_CANPCI            0x0007   // Device Id of CAN-ACx-PCI == Subsystem ID
#define SUBDEVICE_ID_CANPCI_DEVNET     0x0008   // Device Id of CAN-ACx-PCI/DN == Subsystem ID
#define SUBDEVICE_ID_CAN104PLUS        0x000E   // Softing CAN-PROx-PC104+ Device == Subsystem ID
#define SUBDEVICE_ID_CANPROPCIE        0x000D   // Softing CAN-PROx-PCI Device == Subsystem ID




/******************************************************************
**  types
*******************************************************************/

typedef long CSHANDLE;
typedef CSHANDLE *PCSHANDLE;

// ----------------------------------------------------------------------------
// - Beschreibung eines Devices

enum enTimeStampType
{
    TST_NOT_DEFINED,            // time stamp Methode nicht definiert
    TST_OLD,                    // time stamp Methode fuer NEC
    TST_20MHZ,                  // time stamp Methode 64 Bit Timer, 20 MHz
    TST_24MHZ,                  // time stamp Methode 64 Bit Timer, 24 MHz
    TST_25MHZ                   // time stamp Methode 64 Bit Timer, 25 MHz
};


// - Daten eines geoeffneten Devices
typedef struct device_data_s
{
    USIGN16 usSlot;
    USIGN8 ucPC104IORegister_0; // shadow registers
    USIGN8 ucPC104IORegister_1;
    USIGN8 ucPC104IORegister_2;
}
DEVICE_DATA16;
typedef DEVICE_DATA16 *PDEVICE_DATA16;


typedef USIGN16(*DEVICE_FUNKTION) (BOOL boDoFunction, PVOID pDevice);
typedef struct device_id_table16_s
{
    PDEVICE_LOAD_DATA pldLoadData;      // Ladedaten des Device (f. LOADCARD)
    USIGN8 *bufSBNBootFile;      // SBN-Daten des Booters
    USIGN8 *bufSBNLoadFile;      // SBN-Daten des Loaders
    USIGN8 *bufSBNAppFile;       // SBN-Daten der Applikation
    SCHAR *strStandardIniFile;   // INI-Dateiname
    SCHAR *strBootFileName;      // SBN-Dateiname des Booters
    SCHAR *strLoadFileName;      // SBN-Dateiname des Loaders
    SCHAR *strAppFileName;       // SBN-Dateiname der Applikation
    DEVICE_FUNKTION InitFkt;    // Funktion fuer device initialisation
    DEVICE_FUNKTION ResetFkt;   // Funktion fuer device-Reset
    DEVICE_FUNKTION EnableIrqFkt;       // Funktion zum IRQ-Enable
    enum enTimeStampType tstTimeStampType;      // Time Stamp Methode
}
DEVICE_ID_TABLE16;
typedef DEVICE_ID_TABLE16 *PDEVICE_ID_TABLE16;




// ----------------------------------------------------------------------------
// - Liste aller Devices

enum DeviceListEntries
{
    DEVL_CANCARD,
    DEVL_CANCARD_SJA,
    DEVL_EDICCARDC,
    DEVL_CANCARD2,
    DEVL_CANACXPCI,
    DEVL_CANACX104,
    DEVL_NULL
};




SCHAR FAR *GetSBNBufBootDataPointer(PVOID pDevice);
SCHAR FAR *GetSBNBufLoadDataPointer(PVOID pDevice);
SCHAR FAR *GetSBNBufAppDataPointer(PVOID pDevice);
SCHAR FAR *GetSBNBufIniFileName(PVOID pDevice);
SCHAR FAR *GetSBNBufBootDataFileName(PVOID pDevice);
SCHAR FAR *GetSBNBufLoadDataFileName(PVOID pDevice);
SCHAR FAR *GetSBNBufAppDataFileName(PVOID pDevice);
USIGN32 GetDeviceId(PVOID pDevice);
PDEVICE_LOAD_DATA GetDeviceLoadDataPointer(PVOID pDevice);

enum enTimeStampType GetTimeStampType(PVOID pDevice);

USIGN16 GetAssignedIRQ_Nr(PVOID pDevice);

USIGN16 CS_EnableInterrupt(CSHANDLE notUsed, BOOL bDoEnable,
                        USIGN8 far * pAttrBase, PVOID pDevice);

/* from ini10416.c */

int INIPC_close_board(VOID);

#endif /* _DEVICE_H */

/* EOF */
