/*
 * Can-Driver for Softing Can-Card
 * Copyright (C) 2003, Softing AG, Gunnar Larisch, Werner Knorsch
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * email: support.automation@softing.com
 * Softing AG, Richard-Reitzner-Allee 6, 85540 Haar, Germany
 */

/*
 * $Id: can_ioctl.h,v 1.1.1.1 2003/10/24 16:11:00 la Exp $
 */

#ifndef _CAN_IOCTL_H_
#define _CAN_IOCTL_H_

#include <linux/ioctl.h>
#include "BaseTypes.h"

#include "canL2.h"

/*
 * Ioctl definitions
 */
 

//#ifdef OLD_PARAM
 typedef enum ChipTypeEnum
{ _82365SL_,
    _DB86082_
}
__attribute__ ((packed)) ChipType;



 typedef struct canpc_ressources_s
{
  USIGN16 uSocket;
  USIGN16 uInterrupt;
  USIGN32 ulDPRMemBase;
  USIGN32 ulDPRMemSize;
  ChipType uChip;
  USIGN16 uIOAdress;
  USIGN16 uRegisterBase;


}
__attribute__ ((packed)) CANPC_RESOURCES;

//#endif



extern int *ppram;
extern unsigned long can_ioc_pram[10];
extern unsigned long can_ioc_l;
extern unsigned long *can_ioc_pl;
extern unsigned int *can_ioc_pi;
extern CANPC_RESOURCES can_resources;

extern unsigned long *pCanIocPram;
extern unsigned long *pcan_resources;
extern unsigned long *pcan_ioc_l;


typedef struct
{
    USIGN32 Ident;
    INT32 DataLength;
    INT32 RecOverrun_flag;
    INT32 RCV_fifo_lost_msg;
    USIGN8 RCV_data[8];
    INT32 AckOverrun_flag;
    INT32 XMT_ack_fifo_lost_acks;
    INT32 XMT_rmt_fifo_lost_remotes;
    INT32 Bus_state;
    INT32 Error_state;
    INT32 Can;
    USIGN32 Time;
}
__attribute__ ((packed)) can_ioc_ac_struct;

/* Use 'k' as magic number */
#define CAN_IOC_MAGIC  0xAA

#define CAN_IOCRESET    _IO(CAN_IOC_MAGIC, 0)

/*
 * S means "Set" through a ptr,
 * T means "Tell" directly with the argument value
 * G means "Get": reply by setting through a pointer
 * Q means "Query": response is on the return value
 * X means "eXchange": G and S atomically
 * H means "sHift": T and Q atomically
 */

/*
 * The other entities only have "Tell" and "Query", because they're
 * not printed in the book, and there's no need to have all six.
 * (The previous stuff was only there to show different ways to do it.
 */




#define CAN_IOC_CANL2_reset_board                        _IO(CAN_IOC_MAGIC,   1)

#define CAN_IOC_CANL2_reset_chip                         _IO(CAN_IOC_MAGIC,   2)

#define CAN_IOC_CANL2_initialize_chip                    _IOR(CAN_IOC_MAGIC,  3, pCanIocPram)

#define CAN_IOC_CANL2_initialize_chip2                   _IOR(CAN_IOC_MAGIC,  4, pCanIocPram)

#define CAN_IOC_CANL2_set_mode                           _IOR(CAN_IOC_MAGIC,  5, pCanIocPram)

#define CAN_IOC_CANL2_set_mode2                          _IOR(CAN_IOC_MAGIC,  6, pCanIocPram)

#define CAN_IOC_CANL2_set_acceptance                     _IOR(CAN_IOC_MAGIC,  7, pCanIocPram)

#define CAN_IOC_CANL2_set_acceptance2                    _IOR(CAN_IOC_MAGIC,  8, pCanIocPram)

#define CAN_IOC_CANL2_set_output_control                 _IO(CAN_IOC_MAGIC,   9)

#define CAN_IOC_CANL2_set_output_control2                _IO(CAN_IOC_MAGIC,   11)

#define CAN_IOC_CANL2_initialize_interface               _IOR(CAN_IOC_MAGIC,  12, pCanIocPram)

#define CAN_IOC_CANL2_define_object                      _IOWR(CAN_IOC_MAGIC, 13, pCanIocPram)

#define CAN_IOC_CANL2_define_object2                     _IOWR(CAN_IOC_MAGIC, 14, pCanIocPram)

#define CAN_IOC_CANL2_define_cyclic                      _IOR(CAN_IOC_MAGIC,  15, pCanIocPram)

#define CAN_IOC_CANL2_define_cyclic2                     _IOR(CAN_IOC_MAGIC,  16, pCanIocPram)

#define CAN_IOC_CANL2_enable_fifo                        _IO(CAN_IOC_MAGIC,   17)

#define CAN_IOC_CANL2_optimize_rcv_speed                 _IO(CAN_IOC_MAGIC,   18)

#define CAN_IOC_CANL2_enable_dyn_obj_buf                 _IO(CAN_IOC_MAGIC,   19)

#define CAN_IOC_CANL2_enable_fifo_transmit_ack           _IO(CAN_IOC_MAGIC,   20)

#define CAN_IOC_CANL2_enable_fifo_transmit_ack2          _IO(CAN_IOC_MAGIC,   21)

#define CAN_IOC_CANL2_get_version                        _IOWR(CAN_IOC_MAGIC, 22, pCanIocPram)

#define CAN_IOC_CANL2_get_serial_number                  _IOWR(CAN_IOC_MAGIC, 23, can_ioc_pl)

#define CAN_IOC_CANL2_start_chip                         _IO(CAN_IOC_MAGIC,   24)

#define CAN_IOC_CANL2_send_remote_object                 _IOR(CAN_IOC_MAGIC,  25, pCanIocPram)

#define CAN_IOC_CANL2_send_remote_object2                _IOR(CAN_IOC_MAGIC,  26, pCanIocPram)

#define CAN_IOC_CANL2_supply_object_data                 _IOWR(CAN_IOC_MAGIC, 27, pCanIocPram)

#define CAN_IOC_CANL2_supply_object_data2                _IOWR(CAN_IOC_MAGIC, 28, pCanIocPram)

#define CAN_IOC_CANL2_supply_rcv_object_data             _IOWR(CAN_IOC_MAGIC, 29, pCanIocPram)

#define CAN_IOC_CANL2_supply_rcv_object_data2            _IOWR(CAN_IOC_MAGIC, 30, pCanIocPram)

#define CAN_IOC_CANL2_send_object                        _IOR(CAN_IOC_MAGIC,  31, pCanIocPram)

#define CAN_IOC_CANL2_send_object2                       _IOR(CAN_IOC_MAGIC,  32, pCanIocPram)

#define CAN_IOC_CANL2_write_object                       _IOWR(CAN_IOC_MAGIC, 33, pCanIocPram)

#define CAN_IOC_CANL2_write_object2                      _IOWR(CAN_IOC_MAGIC, 34, pCanIocPram)

#define CAN_IOC_CANL2_read_rcv_data                      _IOWR(CAN_IOC_MAGIC, 35, pCanIocPram)

#define CAN_IOC_CANL2_read_rcv_data2                     _IOWR(CAN_IOC_MAGIC, 36, pCanIocPram)

#define CAN_IOC_CANL2_read_xmt_data                      _IOWR(CAN_IOC_MAGIC, 37, pCanIocPram)

#define CAN_IOC_CANL2_read_xmt_data2                     _IOWR(CAN_IOC_MAGIC, 38, pCanIocPram)

#define CAN_IOC_CANL2_read_ac                            _IOWR(CAN_IOC_MAGIC, 39, can_ioc_ac_struct)

#define CAN_IOC_CANL2_send_data                          _IOWR(CAN_IOC_MAGIC, 40, pCanIocPram)

#define CAN_IOC_CANL2_send_data2                         _IOWR(CAN_IOC_MAGIC, 41, pCanIocPram)

#define CAN_IOC_CANL2_send_remote                        _IOR(CAN_IOC_MAGIC,  42, pCanIocPram)

#define CAN_IOC_CANL2_send_remote2                       _IOR(CAN_IOC_MAGIC,  43, pCanIocPram)

#define CAN_IOC_CANL2_get_trigger                        _IOWR(CAN_IOC_MAGIC, 44, can_ioc_pi)

#define CAN_IOC_CANL2_get_trigger2                       _IOWR(CAN_IOC_MAGIC, 45, can_ioc_pi)

#define CAN_IOC_CANL2_set_trigger                        _IO(CAN_IOC_MAGIC,   46)

#define CAN_IOC_CANL2_set_trigger2                       _IO(CAN_IOC_MAGIC,   47)

#define CAN_IOC_CANL2_reinitialize                       _IO(CAN_IOC_MAGIC,   48)

#define CAN_IOC_CANL2_get_time                           _IOWR(CAN_IOC_MAGIC, 49, can_ioc_pl)

#define CAN_IOC_CANL2_get_bus_state                      _IO(CAN_IOC_MAGIC,   50)

#define CAN_IOC_CANL2_reset_rcv_fifo                     _IO(CAN_IOC_MAGIC,   51)

#define CAN_IOC_CANL2_reset_xmt_fifo                     _IO(CAN_IOC_MAGIC,   52)

#define CAN_IOC_CANL2_reset_lost_msg_counter             _IO(CAN_IOC_MAGIC,   53)

#define CAN_IOC_CANL2_read_rcv_fifo_level                _IO(CAN_IOC_MAGIC,   54)

#define CAN_IOC_CANL2_read_xmt_fifo_level                _IO(CAN_IOC_MAGIC,   55)

#define CAN_IOC_INIL2_initialize_board                   _IOWR(CAN_IOC_MAGIC, 56, pcan_resources)

#define CAN_IOC_INIL2_close_board                        _IO(CAN_IOC_MAGIC,   57)

#define CAN_IOC_CANL2_enable_error_frame_detection       _IO(CAN_IOC_MAGIC,   58)

#define CAN_IOC_CANL2_get_device_id                      _IOWR(CAN_IOC_MAGIC, 59, pcan_ioc_l)

#define CAN_IOC_CANL2_init_signals                       _IOR(CAN_IOC_MAGIC,  60, pCanIocPram)

#define CAN_IOC_CANL2_read_signals                       _IOWR(CAN_IOC_MAGIC, 61, can_ioc_pl)

#define CAN_IOC_CANL2_write_signals                      _IOR(CAN_IOC_MAGIC,  62, pCanIocPram)

#define CAN_IOC_CANL2_enable_timestamps                  _IO(CAN_IOC_MAGIC,   63)
/* - - - - - - */

#define CAN_IOC_START_RECORD                             _IO(CAN_IOC_MAGIC,   64)

#define CAN_IOC_STOP_RECORD                              _IO(CAN_IOC_MAGIC,   65)

#define CAN_IOC_CLEAR_MEM                                _IO(CAN_IOC_MAGIC,   66)

#define CAN_IOC_COUNT_RECORDS                            _IO(CAN_IOC_MAGIC,   67)

#define CAN_IOC_SET_SPEED                                _IO(CAN_IOC_MAGIC,   68)

#define CAN_IOC_GET_SPEED                                _IO(CAN_IOC_MAGIC,   69)

#define CAN_IOC_CANL2_trace_level                        _IOR(CAN_IOC_MAGIC,   70, pCanIocPram)

#define CAN_IOC_CANL2_get_phys_channel                   _IOWR(CAN_IOC_MAGIC, 71, can_ioc_pl)

#define CAN_IOC_CANL2_get_device_type                    _IOWR(CAN_IOC_MAGIC, 72, can_ioc_pl)



/* - - - - - -  */

#define CAN_IOCHARDRESET _IO(CAN_IOC_MAGIC, 71) /* debugging tool */

#define CAN_IOC_MAXNR 72

#endif /* _CAN_H_ */
