/*
 * Can-Driver for Softing Can-Card
 * Copyright (C) 1994-2003, Softing AG, Gunnar Larisch, Werner Knorsch
 * 
 * See file CREDITS for list of people who contributed to this
 * project.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * email: support.automation@softing.com
 * Softing AG, Richard-Reitzner-Allee 6, 85540 Haar, Germany
 */

/*
 * $Id: can_pci.h,v 1.3 2003/10/31 15:16:38 kno Exp $
 */





#ifndef __CANACXPCI_H__
#define __CANACXPCI_H__

#ifndef __KERNEL__
#  define __KERNEL__
#endif
#ifndef MODULE
#  define MODULE
#endif

/******************************************************************
**  include files
*******************************************************************/

#include <linux/pci.h>

/******************************************************************
**  macros
*******************************************************************/

#define CAN_NR_PORTS 4          /* use 4 ports by default */
#define CAN_DPRAM_SIZE 0x1000   /* DPRAM size */

// CAN-ACx-PCI
//

#define VENDOR_ID_PLX               0x10B5      // Vendor Id of PLX PCI Chip
#define DEVICE_ID_PLX_9050          0x9050      // Device Id of PLX PCI Chip
#define DEVICE_ID_PLX_9030          0x9030      // Device Id of PLX PCI Chip
#define DEVICE_ID_PLX_86E1          0x86e1      // Device Id of PLX PCI Chip
#define SUBVENDOR_ID_SOFTING        0x14A0      // Vendor Id of Softing == Subsystem Vendor ID
#ifdef CANPRO
#define DRIVERNAME                  "vcanprod"  // Driver name
#endif

#ifdef CANPCIE
#define DRIVERNAME                  "vcanpcie"  // Driver name
#endif

#ifdef CANACXPCI
#define DRIVERNAME                  "vcanpcid"  // Driver name
#endif

#define PNP_PCI_VENDOR_ID            SUBVENDOR_ID_SOFTING
#define PNPDEVICE_CAN_AC2_PCI        0x0001L
#define MAX_PCI_BUSES                256

///// PCI9050 Register Bitmasken

#define INTCSR_LOC_INT_STAT          0x00000004L        // INTCSR_LOC_INT_STAT Local Interrupt 1 status
#define CNTRL_USR0_IO_DATA           0x00000004L        // User I/O Data (interrupt pc->card)


// The following lines are taken from PCI9050.h
// All Local Configuration Registers (LCR) are of DWORD width.
//
// Bit definitions

// Register INTCSR (50h)
#define _9050_50_USER0_DATA       0x00000004    // User I/O 0 data bit in CNTRL register
#define RESET_IRQ                 _9050_50_USER0_DATA   // Special usage in CAN-PCI as reset irq

#define _9050_50_USER1_DATA       0x00000020    // User I/O 1 data bit in CNTRL register
#define RUN_C165                  _9050_50_USER1_DATA   // Special usage in CAN-PCI as reset output

#define _9050_50_USER0_DATA       0x00000004    // User I/O 0 data bit in CNTRL register
#define INT_TO_C165	              _9050_50_USER0_DATA       // Special usage as Interrupt output

// Register INTCSR (4Ch)
#define _9050_4C_LOC_INT1_STAT    0x00000040    // Local Interrupt 1 INTCSR register
#define ENAB_IRQ_PC	              _9050_4C_LOC_INT1_STAT    // Enable IRQ (card to host-PC)

#define PCI_PORT_INTCSR         0x004CL // offset Port Interrupt Control/Status Register
#define PCI_PORT_CNTRL          0x0050L // offset Port User I/O, PCI Target Response, EEPROM,

// 9030 Offsets of PCI Local Configuration Registers (LCR) defined in DWORD units
#define _9030_INTCSR              0x4C    // Interrupt Control/Status register
#define _9030_CNTRL               0x50    // PCI Target Response, EEPROM,
#define _9030_GPIOC               0x54    // User I/O


// Bit definitions in LCRs
#define _9030_EN_IRQ_0        0x00000001   // EN-IRQ-BIT channel 0 in INTCSR register
#define _9030_RESET_XC161_0   0x00000020   // RST-BIT channel 0 in GPIOC register
#define _9030_INT_TO_XC161_0  0x00000004   // IRQ-BIT channel 0 in GPIOC register

// PCIE
#define _8111_INT_TO_XC161_0  0x00010000   // IRQ-BIT channel 1 in GPIOC register
#define _8111_EN_IRQ_0        0x00000900   // EN-IRQ-BIT channel 0 in INTCSR register
#define _8111_INTCSR              0x68


/******************************************************************
**  externals
*******************************************************************/

// PCI Device Id Table. 
// This Table defines the ID's for which the driver will be called from OS.
// ID == 0 means that all possible ID's will match.


// Device Table for CAN-ACx-PCI (subsystem Id 7)

extern struct pci_device_id HwPciIdTable[];


/*---------------------------------------------------------------------------*/
// PCI Driver Structure for hotplugging.

extern struct pci_driver HwPci;

// Device Table for CAN-ACx-PCI/DEV: CAN-ACx-PCI with DeviceNet connector (subsystem Id 8)
extern struct pci_device_id HwPciIdTableDevNet[];
// PCI Driver Structure (DeviceNet) for hotplugging.

extern struct pci_driver HwPciDevNet;
extern PCDEVICES pcCanDevices;


/******************************************************************
** protoypes  
*******************************************************************/
int hw_pci_probe(struct pci_dev *, const struct pci_device_id *);
VOID hw_pci_remove(struct pci_dev *);

int hw_pci_probe(IN struct pci_dev *PciDev,
                 IN const struct pci_device_id *PciId);
VOID hw_pci_remove(IN struct pci_dev *PciDev);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
irqreturn_t can_pci_interrupt(int irq, void *dev_id, struct pt_regs *regs);
#else
    irqreturn_t can_pci_interrupt(int irq, void *dev_id);
#endif
    INT32 CanInstallInterrupt(PCDEVICES pDevice);
int can_intCanDriver(PCDEVICES pDevice);
int can_init(VOID);
VOID can_cleanup(VOID);


#endif // __CANACXPCI_H__
