/*****************************************************************************/
/*                                                                           */
/*   Softing AG           Richard-Reitzner-Allee 6       85540 Haar          */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*      Copyright (C) by Softing AG, 1999-2007   All rights reserved.        */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*                               caneco.h                                    */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/* PROJECT:                  Softing CAN Layer2                              */
/*                                                                           */
/* SWKE:                    <SWKE_identifier>                                */
/*                                                                           */
/* COMPONENT:                CAN-ECOx specific part                          */
/*                                                                           */
/* AUTHOR:                     Softing AG                                    */
/*                                                                           */
/* GENERATION_DATE:            21.05.07                                      */
/*                                                                           */
/* FUNCTIONAL_DESCRIPTION:    CAN-ECOx specific part                         */
/*                                                                           */
/*****************************************************************************/

#define CAN_DPRAM_SIZE 0x1000   /* DPRAM size */
#define DRIVERNAME     "vcanacd"  // Driver name

#define GN_LED_ON(a) *a  |= 2;
#define GN_LED_OFF(a) *a &= ~2;
#define RED_LED_ON(a) *a  |= 1;
#define RED_LED_OFF(a) *a  &= ~1;

INT32 caneco_enable_fifo(PCDEVICES pDevice);
INT32 caneco_enable_fifo_transmit_ack(PCDEVICES pDevice);
INT32 caneco_enable_error_frame_detection(PCDEVICES pDevice);
INT32 caneco_start_chip(PCDEVICES pDevice);
INT32 caneco_get_time(PCDEVICES pDevice, PUSIGN32 time);
INT32 caneco_reset_rcv_fifo(RCV_FIFO_CAN_ECO_STRUCT *pRcvFifo);
INT32 caneco_reset_xmt_fifo(XMT_FIFO_CAN_ECO_STRUCT *pXmtFifo);
INT32 caneco_read_rcv_fifo_level(RCV_FIFO_CAN_ECO_STRUCT *pRcvFifo);
INT32 caneco_read_xmt_fifo_level(XMT_FIFO_CAN_ECO_STRUCT *pXmtFifo);
INT32 caneco_send_remote (PCDEVICES pDevice, USIGN32 Ident, INT32 Xtd, INT32 DataLength);
INT32 caneco_enable_dyn_obj_buf(PCDEVICES pDevice);
INT32 caneco_read_ac(PCDEVICES pDevice, PARAM_STRUCT *param);

INT32 caneco_define_object (PCDEVICES pDevice,
                          USIGN32 Ident,
                          PINT32 ObjectNumber,
                          INT32 Type,
                          INT32 ReceiveIntEnable,
                          INT32 AutoRemoteEnable,
                          INT32 TransmitAckEnable);

INT32 caneco_define_cyclic (PCDEVICES pDevice,
                          INT32          ObjectNumber,
                          unsigned int Rate,
                          unsigned int Cycles);

INT32 caneco_initialize_interface(PCDEVICES pDevice,
                                INT32 ReceiveFifoEnable,
                                INT32 ReceivePollAll,
                                INT32 ReceiveEnableAll,
                                INT32 ReceiveIntEnableAll,
                                INT32 AutoRemoteEnableAll,
                                INT32 TransmitReqFifoEnable,
                                INT32 TransmitPollAll,
                                INT32 TransmitAckEnableAll,
                                INT32 TransmitAckFifoEnableAll,
                                INT32 TransmitRmtFifoEnableAll);


INT32 caneco_send_remote_object(PCDEVICES pDevice, INT32 Ident, INT32 DataLength);
INT32 caneco_supply_object_data(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PUSIGN8 pData);
INT32 caneco_supply_rcv_object_data(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PUSIGN8 pData);
INT32 caneco_write_object(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PUSIGN8 pData);
INT32 caneco_read_rcv_data(PCDEVICES pDevice, INT32 Ident, PUSIGN8 pData, PUSIGN32 Time);
INT32 caneco_read_xmt_data(PCDEVICES pDevice, INT32 Ident, PINT32 pDataLength, PUSIGN8 pXMTData);
INT32 caneco_init_signals (PCDEVICES pDevice, USIGN32 ulChannelDirectionMask, USIGN32 ulChannelOutputDefaults);

/*
INT32 caneco_set_green_led_feature(PCDEVICES pDevice, LED_FEATURE *pFeature);
INT32 caneco_set_red_led_feature(PCDEVICES pDevice, LED_FEATURE *pFeature);
*/




USIGN32 canecodrv_get_time(CAN_ECO *pCaneco);



void reset_xmt_fifo(XMT_FIFO_CAN_ECO_STRUCT *pXmtFifo);
void reset_rcv_fifo(RCV_FIFO_CAN_ECO_STRUCT *pRcvFifo);




INT32 caneco_send_data (PCDEVICES pDevice, 
                      USIGN32 Ident,
                      INT32 Xtd,
                      INT32 DataLength,
                      PUSIGN8 pData);


INT32 caneco_reset_chip(PCDEVICES pDevice);

INT32 caneco_initialize_chip (PCDEVICES pDevice,
                            INT32 presc,
                            INT32 sjw,
                            INT32 tseg1,
                            INT32 tseg2,
                            INT32 sam);

INT32 caneco_set_acceptance(PCDEVICES pDevice, 
                          unsigned int AccCodeStd,
                          unsigned int AccMaskStd,
                          USIGN32 AccCodeXtd,
                          USIGN32 AccMaskXtd);

INT32 caneco_set_output_control(PCDEVICES pDevice, INT32 OutputControl);



INT32 caneco_read_object_dyn(PARAM_STRUCT *param, 
           volatile RCV_OBJECT *pRcvObject, 
           volatile RCV_DYN_STRUCT *pDynRcvStruct, 
           USIGN32 Ident);

INT32 caneco_read_ac_dyn(PCDEVICES pDevice, PARAM_STRUCT *param);
INT32 caneco_read_ac_stat(PCDEVICES pDevice, PARAM_STRUCT *param);
INT32 caneco_read_rcv_data_stat(PCDEVICES pDevice, INT32 Ident, PUSIGN8 pData, PUSIGN32 Time);
INT32 caneco_read_rcv_data_dyn(PCDEVICES pDevice, INT32 Ident, PUSIGN8 pData, PUSIGN32 Time);
INT32 caneco_send_remote_object_stat(PCDEVICES pDevice, INT32 Ident, INT32 DataLength);
INT32 caneco_send_remote_object_dyn(PCDEVICES pDevice, INT32 Ident, INT32 DataLength);
INT32 caneco_supply_object_data_dyn(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PUSIGN8 pData);
INT32 caneco_supply_object_data_stat(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PUSIGN8 pData);
INT32 caneco_supply_rcv_object_data_stat(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PUSIGN8 pData);
INT32 caneco_supply_rcv_object_data_dyn(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PUSIGN8 pData);
INT32 caneco_send_object(PCDEVICES pDevice, INT32 Ident, INT32 DataLength);
INT32 caneco_send_object_stat(PCDEVICES pDevice, INT32 Ident, INT32 DataLength);
INT32 caneco_write_object_dyn(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PUSIGN8 pData);
INT32 caneco_write_object_stat(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PUSIGN8 pData);
INT32 caneco_read_xmt_data_stat(PCDEVICES pDevice, INT32 Ident, PINT32 pDataLength, PUSIGN8 pXMTData);
INT32 caneco_read_xmt_data_dyn(PCDEVICES pDevice, INT32 Ident, PINT32 pDataLength, PUSIGN8 pXMTData);











