/*
 * Can-Driver for Softing Can-Card
 * Copyright (C) 1997-2003, Softing AG, Gunnar Larisch, Werner Knorsch
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * email: support.automation@softing.com
 * Softing AG, Richard-Reitzner-Allee 6, 85540 Haar, Germany
 */

/****************************************************************************/
/*                                                                          */
/*                               C A N L A Y 2 . H                          */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* MODULE_DESCRIPTION  CANLAY2.H                                            */
/* VERSION             4                                                    */
/* DATE                1.12.98                                              */
/*                     include file for CAN Layer 2 Library                 */
/*                     function prototypes and return codes                 */
/*                                                                          */
/****************************************************************************/

/*
 * $Id: canlay2.h,v 1.2 2003/10/30 17:38:51 kno Exp $
 */

#ifndef _CANLAY2_H
#define _CANLAY2_H


#include "can_def.h"



//#define TCHAR char
#define _tcscpy strcpy
#define _tcsncpy strncpy
#define _stprintf sprintf
#define _tcslen strlen
#define _tcscat strcat
#define _tcscmp strcmp
#define _T(wert) wert
#define _tprintf printk
#define noUnicode




#define  CANPC_SOCKET_AUTO   0xFFFF     /* Socket automatically searched     */
#define  CANPC_NO_IRQ        0x0        /* no interrupt requested            */
#define  CANPC_IRQ_AUTO      0xFFFF     /* interrupt automatically assigned  */
#define  CANPC_BASE_AUTO     0x0L       /* DPR-Base automatically assigned   */
#define  CANPC_SIZE_AUTO     0x0L       /* DPR-Size automatically assigned   */
#define  CANPC_AI_AUTO       0x0        /* chip does no matter               */
#define  CANPC_AI_DB8        0x01       /* chip is Databook                  */
#define  CANPC_READ_INIFILE  0xFFFE     /* DOS CAN-AC2-104: reads INI-File   */

/* ---- CAN chip-types -----------------------------------------------------*/
#define NEC005_CHIP            5
#define SJA1000_CHIP        1000


/* ---- Softing CAN Baugruppentypen ----------------------------------------*/

#define    DEV_CANCARD         0x01680001L
#define    DEV_CANCARD_SJA     0x01680004L
#define    DEV_CANCARD2        0x01680005L
#define    DEV_EDICCARDC       0x01680102L
#define    DEV_EDICCARD2       0x01680105L
#define    DEV_CANACPCI        0x14A00001L
#define    DEV_CANAC104        0x00000009L
#define    DEV_CANAC           0x00000001L

/*
 typedef enum ChipTypeEnum
{ _82365SL_,
    _DB86082_
}
__attribute__ ((packed)) ChipType;



 typedef struct canpc_ressources_s
{
  unsigned short uSocket;
  unsigned short uInterrupt;
  USIGN32 ulDPRMemBase;
  USIGN32 ulDPRMemSize;
  ChipType uChip;
  unsigned short uIOAdress;
  unsigned short uRegisterBase;


}
__attribute__ ((packed)) CANPC_RESOURCES;
*/


typedef struct
{
    USIGN32 Ident;
    INT32 DataLength;
    INT32 RecOverrun_flag;
    INT32 RCV_fifo_lost_msg;
    USIGN8 RCV_data[8];
    INT32 AckOverrun_flag;
    INT32 XMT_ack_fifo_lost_acks;
    INT32 XMT_rmt_fifo_lost_remotes;
    INT32 Bus_state;
    INT32 Error_state;
    CAN_HANDLE Can;
    USIGN32 Time;
}
__attribute__ ((packed)) param_struct;



/* ---- FUNCTION PROTOTYPES AND FUNCTION RETURN VALUES  --------------------*/
/* default return values -- valid for all functions (if no other value is
   described)                                                               */
/* default return values -- valid for all functions                         */

#define  CANPC_OK                          0    /* function successful        */
#define  CANPC_ERR                        -1    /* function not successful    */
#define  CANPC_ERR_DPR                    -3    /* error accessing dpram (only
                                                   CAN-AC2)                   */
#define  CANPC_ACCESS_TIMEOUT             -4    /* firmware communication
                                                   timeout                    */
#ifdef LINUX
#define  CANPC_ERR_OPEN                   -80   /* open  can-device failed */
#define  CANPC_ERR_IOCTL                  -81   /* ioctl can-device failed */
#define  CANPC_ERR_CLOSE                  -82   /* close can-device failed */
#endif
#define  CANPC_BOARD_NOT_INITIALIZED     -99    /* board not initialized:
                                                   INIPC_initialize_board(.)
                                                   was not yet called, or a
                                                   INIPC_close_board was done */

/* ---- INIPC_initialize_board   -------------------------------------------*/

/* -- CANcard return values                                                 */
/* --  return codes for card services access with WINDOWS 3.11              */

#define  INIPC_IB_ERR_WINSYSRES       0xFFFF    /* not enough windows system
                                                   resources                  */

/* --  return codes for card services access with DOS or WINDOWS 3.11       */

/* return values between 0x0201 and 0x3622: card services call failed       */

#define  INIPC_IB_ERR_NO_CARD_FOUND   0xFFEF    /* no pcmcia card found       */
#define  INIPC_IB_ERR_NO_CS_INSTALLED 0xFFEE    /* no card services installed */
#define  INIPC_IB_ERR_WRONG_VERSION   0xFFED    /* card services version not
                                                   supported                  */
#define  INIPC_IB_ERR_IRQ             0xFFEC    /* interrupt number invalid
                                                   or in use                  */
#define  INIPC_IB_ERR_DPRAM           0xFFEB    /* failed allocating dpr-window:
                                                   out of resources or in use */
#define  INIPC_IB_ERR_ACCESS_DPRAM    0xFFEA    /* DOS: invalid dpram address
                                                   (above 1MB)
                                                   WIN: failed allocating
                                                   selector              */

/* -- CANcard and CAN-AC2 return values                                     */
/* --  return codes for access with WIN32 (WINDOWS-NT 4.0 and WINDOWS 95)   */
/* return values between 0xFE0201 and 0x3622: card services call failed     */
#define  INIPC_IB_PNP_NO_DEVICE_FOUND  0xFE00   /* no can device found        */
#define  INIPC_IB_ERR_VC_INTERNALERROR 0xFE01   /* internal error             */
#define  INIPC_IB_ERR_VC_GENERALERROR  0xFE02   /* general error              */
#define  INIPC_IB_ERR_VC_TIMEOUT       0xFE03   /* Timeout                    */
#define  INIPC_IB_ERR_VC_IOPENDING     0xFE04   /* driver call pending        */
#define  INIPC_IB_ERR_VC_IOCANCELLED   0xFE05   /* driver call cancelled      */
#define  INIPC_IB_ERR_VC_ILLEGALCALL   0xFE06   /* illegal driver call        */
#define  INIPC_IB_ERR_VC_NOTSUPPORTED  0xFE07   /* driver call not supported  */
#define  INIPC_IB_ERR_VC_VERSIONERROR  0xFE08   /* wrong driver-dll version   */
#define  INIPC_IB_ERR_VC_DRIVERVERSIONERROR   0xFE09    /*wrong driver version */
#define  INIPC_IB_ERR_VC_DRIVERNOTFOUND   0xFE0A        /* driver not found        */
#define  INIPC_IB_ERR_VC_NOTENOUGHMEMORY  0xFE0B        /* not enough memory       */
#define  INIPC_IB_ERR_VC_TOOMANYDEVICES   0xFE0C        /* too many devices        */
#define  INIPC_IB_ERR_VC_UNKNOWNDEVICE    0xFE0D        /* unknown device          */
#define  INIPC_IB_ERR_VC_DEVICEALREADYEXISTS 0xFE0E     /* Device ardy exists   */
#define  INIPC_IB_ERR_VC_DEVICEACCESSERROR   0xFE0F     /* device ardy open     */
#define  INIPC_IB_ERR_VC_RESOURCEALREADYREGISTERED 0xFE10       /* Resource in use */
#define  INIPC_IB_ERR_VC_RESOURCECONFLICT 0xFE11        /* Resource-conflict       */
#define  INIPC_IB_ERR_VC_RESOURCEACCESSERROR 0xFE12     /* Resource access error */
#define  INIPC_IB_ERR_VC_PHYSMEMORYOVERRUN 0xFE13       /* invalid phys.mem-access */
#define  INIPC_IB_ERR_VC_TOOMANYPORTS    0xFE14 /* too many I/O ports   */
#define  INIPC_IB_ERR_VC_UNKNOWNRESOURCE  0xFE15        /* unknown resource        */
#define  INIPC_IB_ERR_LOADDDLL_DEV 0xFF00       /* device or driver DLL not found */
#define  INIPC_IB_ERR_CANT_DEVREGISTER 0xFF01   /* cannot register device     */


//int INIPC_initialize_board(CANPC_RESOURCES * pcrResources);

/* +--- CANPC_reset_board        -------------------------------------------*/
/* -- CANcard return values                                                 */
/* -- return values for access with DOS only                                */
#define  CANPC_RB_INI_FILE           -1 /* can't open IniFile              */
#define  CANPC_RB_ERR_FMT_INI        -2 /* format error in INI-file        */
#define  CANPC_RB_ERR_OP_BIN         -3 /* error opening binary-file       */
#define  CANPC_RB_ERR_RD_BIN         -4 /* error reading binary-file       */
#define  CANPC_RB_BIN_TOO_LONG       -5 /* binary-file too long            */

/* -- return values for access with DOS and WINDOWS (WIN16, WIN32)          */
#define  CANPC_RB_ERR_BIN_FMT        -6 /* binary-data format error        */
#define  CANPC_RB_ERR_BIN_CS         -7 /* binary-data checksum error      */
#define  CANPC_RB_NO_CARD           -16 /* no card present                 */
#define  CANPC_RB_NO_PHYS_MEM       -17 /* no physical memory              */
#define  CANPC_RB_INVLD_IRQ         -18 /* invalid IRQ-number              */
#define  CANPC_RB_ERR_DPRAM_ACCESS  -19 /* error accessing dpram           */
#define  CANPC_RB_ERR_CRD_RESP      -20 /* bad response from card          */
#define  CANPC_RB_ERR_SRAM          -21 /* sram seems to be damaged        */
#define  CANPC_RB_ERR_PRG           -22 /* invalid program start address   */
#define  CANPC_RB_ERR_REC           -23 /* invalid record type             */
#define  CANPC_RB_ERR_NORESP        -24 /* no response after program start */
#define  CANPC_RB_ERR_BADRESP       -25 /* bad response after program start */
#define  CANPC_RB_PCMCIA_NSUPP      -26 /* pcmcia chip not supported       */
#define  CANPC_RB_ERR_RD_PCMCIA     -27 /* error reading ocmcia parameters */
#define  CANPC_RB_INIT_CHIP         -38 /* error initializing chip         */
#define  CANPC_RB_ERR_NO_CANCARD    -39 /* no CANcard plugged in           */

#define  CANPC_RB_ERR_DEV_LD        -40 /* CAN device not properly load    */

/* -- CAN-AC2 return values                                                 */
/* -- boot-download error                                                   */

/* ErrorCode = ErrorBase + ErrorDetail                                      */
/* ErrorBase codes                                                          */
#define CANPC_RB_BOOT_BASE         -100 /* boot-data errors                */
#define CANPC_RB_LOAD_BASE         -200 /* load-data errors                */
#define CANPC_RB_APP_BASE          -300 /* application-data errors         */
#define CANPC_RB_CPY_BASE          -400 /* copy-data errors                */
#define CANPC_RB_RUN_BASE          -500 /* run-data errors                 */
/* ErrorDetail codes                                                        */
#define  CANPC_RB_NO_RESPONSE        -1 /* no response from CAN-AC2        */
#define  CANPC_RB_ERR_FMT            -8 /* boot-data format error          */
#define  CANPC_RB_ERR_INT1           -2 /* internal error 1                */
#define  CANPC_RB_ERR_INT2           -3 /* internal error 2                */
#define  CANPC_RB_ERR_INT3           -4 /* internal error 3                */
#define  CANPC_RB_ERR_INT4           -9 /* internal error 4                */
#define  CANPC_RB_ERR_INT5          -10 /* internal error 5                */
#define  CANPC_RB_ERR_OPEN           -5 /* file open error (DOS only)      */
#define  CANPC_RB_ERR_READ           -6 /* file read error (DOS only)      */
#define  CANPC_RB_ERR_CLOSE          -7 /* file close error (DOS only)     */



INT32 CANPC_reset_board(void);

/* +--- CANPC_reset_chip         -------------------------------------------*/
INT32 CANPC_reset_chip(void);


/* +--- CANPC_initialize_chip(2) -------------------------------------------*/
#define  CANPC_IC_PARAM_ERR               -1    /* Parameter error            */
INT32 CANPC_initialize_chip(INT32 presc, INT32 sjw, INT32 tseg1, INT32 tseg2, INT32 sam);

INT32 CANPC_initialize_chip2(INT32 presc, INT32 sjw, INT32 tseg1, INT32 tseg2, INT32 sam);

/* +--- CANPC_set_mode(2)        -------------------------------------------*/
INT32 CANPC_set_mode(INT32 SleepMode, INT32 SpeedMode);

INT32 CANPC_set_mode2(INT32 SleepMode, INT32 SpeedMode);

/* +--- CANPC_set_acceptance(2)  -------------------------------------------*/
INT32 CANPC_set_acceptance(USIGN32 AccCodeStd,
                         USIGN32 AccMaskStd,
                         USIGN32 AccCodeXtd, USIGN32 AccMaskXtd);

INT32 CANPC_set_acceptance2(USIGN32 AccCodeStd,
                          USIGN32 AccMaskStd,
                          USIGN32 AccCodeXtd, USIGN32 AccMaskXtd);

/* +--- CANPC_set_output_control(2)  ---------------------------------------*/
INT32 CANPC_set_output_control(INT32 OutputControl);

INT32 CANPC_set_output_control2(INT32 OutputControl);

/* +--- CANPC_initialize_interface   ---------------------------------------*/
#define  CANPC_II_REA_CONFLICT            -6    /* Parameter conflict:
                                                   ReceiveEnableAll with dyn.
                                                   obj. buffer or fifo mode   */
#define  CANPC_II_DPR_STATIC              -5    /* DPRAM size conflict:
                                                   DPRAM is not 64k, but
                                                   static object buffer is
                                                   requested(only CAN-AC2)    */

INT32 CANPC_initialize_interface(INT32 ReceiveFifoEnable,
                               INT32 ReceivePollAll,
                               INT32 ReceiveEnableAll,
                               INT32 ReceiveIntEnableAll,
                               INT32 AutoRemoteEnable,
                               INT32 TransmitReqFifoEnable,
                               INT32 TransmitPollAll,
                               INT32 TransmitAckEnableAll,
                               INT32 TransmitAckFifoEnable,
                               INT32 TransmitRmtFifoEnable);

/* +--- CANPC_define_object(2)          ------------------------------------*/
#define  CANPC_DO_PARAM_ERR               -1    /* Parameter error            */
#define  CANPC_DO_NO_DYNOBJ               -2    /* dyn. obj. buf. not enabled
                                                   (only CANPC_define_object2) */
#define  CANPC_DO_DPR_STATIC              -5    /* DPRAM size conflict:
                                                   DPRAM is not 64k, but
                                                   static object buffer is
                                                   requested(only CAN-AC2)    */
#define  CANPC_DO_TOO_MANY_OBJECTS        -6    /* too many objects defined   */

INT32 CANPC_define_object(unsigned long Handle,
                        INT32 *ObjectNumber,
                        INT32 Type,
                        INT32 ReceiveIntEnable,
                        INT32 AutoRemoteEnable, INT32 TransmitAckEnable);

INT32 CANPC_define_object2(unsigned long Handle,
                         INT32 *ObjectNumber,
                         INT32 Type,
                         INT32 ReceiveIntEnable,
                         INT32 AutoRemoteEnable, INT32 TransmitAckEnable);

/* +--- CANPC_define_cyclic(2)        --------------------------------------*/
#define  CANPC_DC_INVLD_OBJ_NR            -1    /* invalid object number      */
#define  CANPC_DC_NO_DYN_OBJBUF           -2    /* no object dyn. buffer
                                                   enabled (only CAN-AC2)     */

INT32 CANPC_define_cyclic(INT32 ObjectNumber,
                        USIGN32 Rate, USIGN32 Cycles);

INT32 CANPC_define_cyclic2(INT32 ObjectNumber,
                         USIGN32 Rate, USIGN32 Cycles);

/* +--- CANPC_enable_fifo            ---------------------------------------*/

INT32 CANPC_enable_fifo(void);

/* +--- CANPC_optimize_rcv_speed     ---------------------------------------*/
/* this function has no effect on the CANcard                               */
#define  CANPC_ORS_PARAM_ERR              -1    /* Parameter error            */

INT32 CANPC_optimize_rcv_speed(void);

/* +--- CANPC_enable_dyn_obj_buf     ---------------------------------------*/

INT32 CANPC_enable_dyn_obj_buf(void);

/* +--- CANPC_enable_timestamps      ---------------------------------------*/
/* this function has no effect on the CANcard                               */
#define  CANPC_ET_NO_FIFO                 -6    /* fifo not enabled
                                                   (only CAN-AC2)             */

INT32 CANPC_enable_timestamps(void);
/* +--- CANPC_enable_fifo_transmit_ack  ------------------------------------*/

INT32 CANPC_enable_fifo_transmit_ack(void);

INT32 CANPC_enable_fifo_transmit_ack2(void);

/* +--- CANPC_get_version            ---------------------------------------*/
/* the parameter hw_version is not used with the CAN-AC2                    */

INT32 CANPC_get_version(INT32 *sw_version,
                      INT32 *fw_version,
                      INT32 *hw_version, INT32 *license, INT32 *can_chip_type);

/* +--- CANPC_get_serial_number      ---------------------------------------*/
/* this function has no effect on the CAN-AC2                               */

INT32 CANPC_get_serial_number(USIGN32 *ser_number);

/* +--- CANPC_start_chip             ---------------------------------------*/

INT32 CANPC_start_chip(void);

/* +--- CANPC_send_remote_object(2)  ---------------------------------------*/
#define  CANPC_SRO_PEND                   -1    /* last request still pending */
#define  CANPC_SRO_TX_FIFO_FULL           -3    /* transmit fifo full         */

INT32 CANPC_send_remote_object(INT32 ObjectNumber, INT32 DataLength);

INT32 CANPC_send_remote_object2(INT32 ObjectNumber, INT32 DataLength);

/* +--- CANPC_supply_object_data(2)  ---------------------------------------*/
#define  CANPC_SOD_REQ_OVR                -1    /* request overrun            */

INT32 CANPC_supply_object_data(INT32 ObjectNumber,
                             INT32 DataLength, USIGN8 *pData);

INT32 CANPC_supply_object_data2(INT32 ObjectNumber,
                              INT32 DataLength, USIGN8 *pData);

/* +--- CANPC_supply_rcv_object_data(2)-------------------------------------*/

INT32 CANPC_supply_rcv_object_data(INT32 ObjectNumber,
                                 INT32 DataLength, USIGN8 *pData);

INT32 CANPC_supply_rcv_object_data2(INT32 ObjectNumber,
                                  INT32 DataLength, USIGN8 *pData);

/* +--- CANPC_send_object(2)         ---------------------------------------*/


#define  CANPC_SO_REQ_OVR                 -1    /* request overrun            */
#define  CANPC_SO_FIFO_FULL               -3    /* job fifo full              */

INT32 CANPC_send_object(INT32 ObjectNumber, INT32 DataLength);

INT32 CANPC_send_object2(INT32 ObjectNumber, INT32 DataLength);

/* +--- CANPC_write_object(2)        ---------------------------------------*/
#define  CANPC_WO_REQ_OVR                 -1    /* request overrun            */
#define  CANPC_WO_FIFO_FULL               -3    /* transmit fifo full         */

INT32 CANPC_write_object(INT32 ObjectNumber,
                       INT32 DataLength, USIGN8 *pData);

INT32 CANPC_write_object2(INT32 ObjectNumber,
                        INT32 DataLength, USIGN8 *pData);

/* +--- CANPC_read_rcv_data(2)       ---------------------------------------*/
#define  CANPC_RRD_NO_DATA                 0    /* no new data received       */
#define  CANPC_RRD_DATAFRAME               1    /* data frame received        */
#define  CANPC_RRD_REMOTEFRAME             2    /* remote frame received      */
#define  CANPC_RRD_RCV_OVR                -1    /* receive data overrun       */
#define  CANPC_RRD_RF_OVR                 -2    /* rec. rem. frame overrun    */
#define  CANPC_RRD_OBJ_INACTIVE           -3    /* object not active          */
#define  CANPC_RDD_ACCESS_TIMEOUT         -7    /* firmware communication
                                                   timeout (only CAN-AC2)     */

INT32 CANPC_read_rcv_data(INT32 ObjectNumber,
                        USIGN8 *pRCV_Data, USIGN32 *Time);

INT32 CANPC_read_rcv_data2(INT32 ObjectNumber,
                         USIGN8 *pRCV_Data, USIGN32 *Time);


/* +--- CANPC_read_xmt_data(2)       ---------------------------------------*/
#define  CANPC_RXD_NO_DATA                 0    /* no message was transmitted */
#define  CANPC_RXD_DATAFRAME               1    /* message was transmitted    */
#define  CANPC_RXD_XMT_OVR                -1    /* transmit ack. overrun      */
#define  CANPC_RXD_NO_DYN_OBJBUF          -3    /* no dyn. obj-buffer enabled
                                                   (only CAN-AC2/527)         */

INT32 CANPC_read_xmt_data(INT32 ObjectNumber,
                        INT32 *pDataLength, USIGN8 *pXMT_Data);

INT32 CANPC_read_xmt_data2(INT32 ObjectNumber,
                         INT32 *pDataLength, USIGN8 *pXMT_Data);

/* +--- CANPC_read_ac                ---------------------------------------*/
#define  CANPC_RA_NO_DATA                 0     /* no new data received        */
#define  CANPC_RA_DATAFRAME               1     /* std. data frame received    */
#define  CANPC_RA_REMOTEFRAME             2     /* std. remote frame received  */
#define  CANPC_RA_TX_DATAFRAME            3     /* transmission of std. data-
                                                   frame is confirmed          */
#define  CANPC_RA_TX_FIFO_OVR             4     /* remote tx fifo overrun      */
#define  CANPC_RA_CHG_BUS_STATE           5     /* change of bus status        */
#define  CANPC_RA_ERR_ADD                 7     /* additional error causes     */
#define  CANPC_RA_TX_REMOTEFRAME          8     /* transmission of std. data-
                                                   frame is confirmed          */
#define  CANPC_RA_XTD_DATAFRAME           9     /* xtd. data frame received    */
#define  CANPC_RA_XTD_TX_DATAFRAME       10     /* transmission of xtd. data-
                                                   frame is confirmed          */
#define  CANPC_RA_XTD_TX_REMOTEFRAME     11     /* transmission of xtd. remote-
                                                   frame is confirmed          */
#define  CANPC_RA_XTD_REMOTEFRAME        12     /* xtd. remote frame received
                                                   (only CANcard)              */
#define  CANPC_RA_ERRORFRAME             15     /* error frame detected
                                                   (only CANcard)              */

INT32 CANPC_read_ac(param_struct * param);

/* +--- CANPC_send_data              ---------------------------------------*/
#define  CANPC_SD_FIFO_FULL               -1    /* transmit fifo full         */

INT32 CANPC_send_data(USIGN32 Ident,
                    INT32 Xtd, INT32 DataLength, USIGN8 *pData);

INT32 CANPC_send_data2(USIGN32 Ident,
                     INT32 Xtd, INT32 DataLength, USIGN8 *pData);

/* +--- CANPC_send_remote            ---------------------------------------*/
#define  CANPC_SR_FIFO_FULL               -1    /* transmit fifo full         */
INT32 CANPC_send_remote(USIGN32 Ident, INT32 Xtd, INT32 DataLength);

INT32 CANPC_send_remote2(USIGN32 Ident, INT32 Xtd, INT32 DataLength);


/* +--- CANPC_get_trigger(2)            ---------------------------------------*/
/* these functions have no effect on the CANcard                               */
INT32 CANPC_get_trigger(INT32 *level);

INT32 CANPC_get_trigger2(INT32 *level);

/* +--- CANPC_set_trigger            ---------------------------------------*/
#define  CANPC_ST_FIFO_FULL               -1    /* transmit fifo full         */

INT32 CANPC_set_trigger(INT32 level);

INT32 CANPC_set_trigger2(INT32 level);

/* +--- CANPC_reinitialize           ---------------------------------------*/
INT32 CANPC_reinitialize(void);

/* +--- CANPC_get_time               ---------------------------------------*/
INT32 CANPC_get_time(USIGN32 *time);

/* +--- CANPC_get_bus_state          ---------------------------------------*/
#define  CANPC_GBS_ERROR_ACTIVE            0    /* error active               */
#define  CANPC_GBS_ERROR_PASSIVE           1    /* error passive              */
#define  CANPC_GBS_ERROR_BUS_OFF           2    /* bus off                    */

INT32 CANPC_get_bus_state(INT32 Can);

INT32 CANPC_get_bus_state(INT32 Can);


/* +--- CANPC_reset_rcv_fifo         ---------------------------------------*/
INT32 CANPC_reset_rcv_fifo(void);

/* +--- CANPC_reset_xmt_fifo         ---------------------------------------*/
INT32 CANPC_reset_xmt_fifo(void);

/* +--- CANPC_reset_lost_msg_counter  --------------------------------------*/
INT32 CANPC_reset_lost_msg_counter(void);

/* +--- CANPC_read_rcv_fifo_level    ---------------------------------------*/
/* returns number of items in receive fifo   */
INT32 CANPC_read_rcv_fifo_level(void);

/* +--- CANPC_read_xmt_fifo_level    ---------------------------------------*/
/* returns number of items in transmit fifo  */
INT32 CANPC_read_xmt_fifo_level(void);

/* +--- CANPC_set_path               ---------------------------------------*/
#define  CANPC_SP_STRING_TOO_LONG        -1     /* path string too long        */
INT32 CANPC_set_path(TCHAR * path);

/* +--- INIPC_close_board            ---------------------------------------*/
INT32 INIPC_close_board(void);

#define  INIPC_CB_ERR                     -1    /* error unlocking rerssources */

/* +--- CANPC_enable_error_frame_detection  --------------------------------*/
/* this function has no effect on the CAN-AC2                               */
INT32 CANPC_enable_error_frame_detection(void);

/* +--- CANPC_get_device_id  -----------------------------------------------*/
/* this function has no effect on the CAN-AC2                               */
INT32 CANPC_get_device_id(USIGN32 *pulDeviceId);

/* +--- CANPC_init_signals/CANPC_read_signals/CANPC_write_signals            */
/* these functions are only available on the CAN-ACx-PCI. They are designed  */
/* for use with the CANcard and piggy-back boards. These functions are   */
/* not in the standard users manual. For description read the piggy-back     */
/* manuals                                                                   */
INT32 CANPC_init_signals(USIGN32 ulChannelDirectionMask,
                       USIGN32 ulChannelOutputDefaults);

INT32 CANPC_read_signals(USIGN32 *pulChannelRead);

INT32 CANPC_write_signals(USIGN32 pulChannelWrite,
                        USIGN32 ulCareMask);

INT32 CANPC_trace_level(USIGN32 level);



#ifdef LINUX
INT32 CANPC_get_fd(unsigned long *pfd);
#endif

// Acceptance filter
#define ACCEPT_MASK_1           0x0000
#define ACCEPT_CODE_1           0x0000
#define ACCEPT_MASK_XTD_1       0x00000000L
#define ACCEPT_CODE_XTD_1       0x00000000L

#endif  /*_CANLAY2_H */
