/*****************************************************************************/
/*                                                                           */
/*   Softing AG           Richard-Reitzner-Allee 6       85540 Haar          */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*      Copyright (C) by Softing AG, 1999-2007   All rights reserved.        */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*                               canpro.h                                    */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/* PROJECT:                  Softing CAN Layer2                              */
/*                                                                           */
/* SWKE:                    <SWKE_identifier>                                */
/*                                                                           */
/* COMPONENT:                CAN-PROx specific part                          */
/*                                                                           */
/* AUTHOR:                     Softing AG                                    */
/*                                                                           */
/* GENERATION_DATE:            19.07.06                                      */
/*                                                                           */
/* FUNCTIONAL_DESCRIPTION:    CAN-PROx specific part                         */
/*                                                                           */
/*****************************************************************************/


#define CAN_HANDLE unsigned long

#define ANZ_IDENT               2048
#define MAX_OBJ_DYN             200

#define  RTR_MASK               0x0800
#define END_MARK_WORD           0xFFFF

#define NOT_VALID               0xFFFFFFFF



/* RCV_CTRL_1 */
#define  RCV_STATUS_DATA        0x01
#define  RCV_STATUS_REMOTE      0x02
#define  RCV_STATUS_DATA_OR_REMOTE 0x03
#define  RCV_STATUS_OVERRUN     0x04
#define  XTD_RCV                0x10
#define  AUTO_REMOTE_ENABLE     0x20
#define  HOST_INTERRUPT_ENABLE  0x40
#define  RCV_ENABLE             0x80



/* XMT_CTRL_1 now  XMT_CTRL_0*/
#define  XTD_XMT                0x10
#define  XMT_REQ_DATA           0x40
#define  XMT_REQ_REMOTE         0x20

/* XMT_CTRL_2 now XMT_CTRL_1*/
#define  XMT_STATUS             0x01
#define  XMT_STATUS_OVERRUN     0x02
#define  XMT_DATA_VALID         0x10
#define  XMT_ACK_ENABLE         0x40
#define  XMT_ACK_FIFO_ENABLE    0x80

/* XMT_CTRL_3 now XMT_CTRL_1*/
#define  RMT_XMT_REQ_DATA       0x04
#define  XMT_ACK_RMT            0x08



typedef struct
{
  unsigned char live_time; // number of 200ms ticks
  BOOL          is_led_on; // should the led be on or off for the live time?

} LED_SUBSTATE;

typedef struct
{
  unsigned short number_of_substates;

  LED_SUBSTATE substate[15];

} LED_FEATURE;


int canpro_initialize_chip (PCDEVICES pDevice,
                            INT32 presc,
                            INT32 sjw,
                            INT32 tseg1,
                            INT32 tseg2,
                            INT32 div8x);

INT32 canpro_read_object_dyn(PARAM_STRUCT *param, 
					 volatile RCV_OBJECT *pRcvObject, 
					 volatile RCV_DYN_STRUCT *pDynRcvStruct, 
					 USIGN32 Ident);

INT32 canpro_read_ac_dyn(PCDEVICES pDevice, PARAM_STRUCT * param);
INT32 canpro_read_ac_stat(PCDEVICES pDevice, PARAM_STRUCT * param);
INT32 canpro_read_rcv_data_stat(PCDEVICES pDevice, INT32 Ident, PBYTE pData, PUSIGN32 Time);
INT32 canpro_read_rcv_data_dyn(PCDEVICES pDevice, INT32 Ident, PBYTE pData, PUSIGN32 Time);
INT32 canpro_send_remote_object_stat(PCDEVICES pDevice, INT32 Ident, INT32 DataLength);
INT32 canpro_send_remote_object_dyn(PCDEVICES pDevice, INT32 Ident, INT32 DataLength);
INT32 canpro_supply_object_data_dyn(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PBYTE pData);
INT32 canpro_supply_object_data_stat(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PBYTE pData);
INT32 canpro_supply_rcv_object_data_stat(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PBYTE pData);
INT32 canpro_supply_rcv_object_data_dyn(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PBYTE pData);
INT32 canpro_send_object_dyn(PCDEVICES pDevice, INT32 Ident, INT32 DataLength);
INT32 canpro_send_object_stat(PCDEVICES pDevice, INT32 Ident, INT32 DataLength);
INT32 canpro_write_object_dyn(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PBYTE pData);
INT32 canpro_write_object_stat(PCDEVICES pDevice, INT32 Ident, INT32 DataLength, PBYTE pData);
INT32 canpro_read_xmt_data_stat(PCDEVICES pDevice, INT32 Ident, PINT32 pDataLength, PBYTE pXMTData);
INT32 canpro_read_xmt_data_dyn(PCDEVICES pDevice, INT32 Ident, PINT32 pDataLength, PBYTE pXMTData);
INT32 canpro_set_green_led_feature(PCDEVICES pDevice, LED_FEATURE *pFeature);
INT32 canpro_set_red_led_feature(PCDEVICES pDevice, LED_FEATURE *pFeature);
INT32 canpro_read_ac(PCDEVICES pDevice, PARAM_STRUCT * param);



USIGN32 Mk20MhzTime(USIGN32 u32TimeLow, USIGN32 u32TimeHigh);











