

#ifndef __CANPROUSB__
#define __CANPROUSB__

#define MAX_BULK_SIZE 512
#define USBREQ_INFO       1
#define USBREQ_RESET_PIPE 2
#define USBREQ_FW_STATE   3

#define STATE_BOOTBLOCK 0
#define STATE_L2        1

#define MAXIMUM_DUMMY_TRANSFERS 1
#define SET_HOST_INT1 



typedef struct
{
  USIGN32 serialnumber; // serial number of the device
  BYTE    piggyback1;
  BYTE    piggyback2; // CAN Highspeed ...
  BYTE    numOfChannels; // number of channels on the device
  BYTE    version; // bootblock version
  
} DEVICEINFO;




typedef enum
{
  CMD_WRT_FLASH = 0x100,
  CMD_START_FIRMWARE,
  CMD_STOP_FIRMWARE,
  CMD_WRT_EEPROM,
  CMD_RD_EEPROM,
  CMD_QUERY_FIRMWARE
} BOOTCMD;

#pragma pack(push,1)

/////////////////////////////////////////////////////////////////////////////
//  IOCTL_VCARD_SENDCMD
/////////////////////////////////////////////////////////////////////////////


typedef struct _FCT_STRUCT
{
  volatile   USIGN16  wRequest;   // framework request type, see above
  volatile   INT16           sParam_1;   // may be used for parametres
  volatile   INT16           sParam_2;
  volatile   INT16           sParam_3;
  volatile   INT16           sParam_4;
  volatile   INT16           sParam_5;
  volatile   INT16           sParam_6;
  volatile   INT16           sParam_7;
  volatile   INT16           sParam_8;
  volatile   INT16           sParam_9;   // currently not used
  volatile   INT16           sParam_10;
  volatile   INT16           sParam_11;
  volatile   INT16           sResult_1;  // may be used for return values
  volatile   INT16           sResult_2;
  volatile   INT16           sResult_3;
  volatile   INT16           sResult_4;
  volatile   INT16           sResult_5;
  volatile   INT16           sResult_6;
  volatile   INT16           sResult_7;
  volatile   INT16           sResult_8;
  volatile   INT16           sRetCode;   // return code of FRW_Req/FRW_Res, see above
  volatile   BYTE            sDummy42;
  volatile   USIGN16  wFRWReqRes; // framework handshake flag:
  // set FALSE by host, set TRUE by device
  volatile   BYTE            sDummy45;
  volatile   INT16           sDummy46;
  volatile   INT16           sDummy48;
  volatile   INT16           sDummy50;
  volatile   INT16           sDummy52;
  volatile   INT16           sDummy54;
  volatile   INT16           sDummy56;
  volatile   INT16           sDummy58;
  volatile   INT16           sDummy60;
  volatile   USIGN16  wCounter;
}   DFCT_STRUCT, *PDFCT_STRUCT;


typedef struct
{
  volatile USIGN16 version;
  volatile USIGN16 build;
  volatile USIGN16 checksum;
} FW_TYPE;

typedef union
{
  volatile char    packet[128]; // 128 Bytes
  volatile FW_TYPE fw;     // firmware version
} FW_PACKET_DATA;


typedef struct _FW_PACKET_
{
  volatile USIGN16       type;       // type of the packet
  volatile FW_PACKET_DATA    data; // 128 Bytes
  volatile USIGN32       address;    // adress inside the flash
  volatile USIGN16       checksum;   // checksum
  volatile USIGN16       size;       // size of the filled packet (<=128 bytes)
  volatile USIGN16       status;     // status code;
  volatile USIGN16       count;      // command counter
} volatile FW_PACKET;

typedef USIGN16 FW_RESULT;


#pragma pack(pop)

int send_command(PCDEVICES pDevice, void *pOutBuffer, int OutBufSize, void *pInBuffer, int InBufSize, int timeout);
int StopFirmware (IN  PCDEVICES pDevice);
void StartIoChain (PCDEVICES pDevice);
void StopIoChain (PCDEVICES pDevice);
USIGN32 MkTimestamp(volatile USBRCVITEM *pMesg);
int XmtNextPacket(PCDEVICES pDevice);
CAN_ADDRESS sync_get_access(spinlock_t *access);
void sync_free_access(spinlock_t *access, CAN_ADDRESS IrqFlags);


#endif // __CANPROUSB__
