/*
 * Can-Driver for Softing Can-Card
 * Copyright (C) 1994-2003, Softing AG, Gunnar Larisch, Werner Knorsch
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * email: support.automation@softing.com
 * Softing AG, Richard-Reitzner-Allee 6, 85540 Haar, Germany
 */

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* PROJECT_NAME           PCMCIA                                            */
/* MODULE                 CARD_DEF.H                                        */
/* COMPONENT_LIBRARY                                                        */
/* AUTHOR                 S.Gemperlein                                      */
/* VERSION                1.0                                               */
/* DATE                   08.08.94                                          */
/*                                                                          */
/* MODULE_DESCRIPTION     header file for all source modules                */
/*                                                                          */
/*HEAD_END*******************************************************************/

/*
 * $Id: card_def.h,v 1.7 2003/11/04 12:48:17 kno Exp $
 */
/******************************************************************
**  include files
*******************************************************************/

#ifndef _CARD_DEF_H_
#define _CARD_DEF_H_

#include "sja1000.h"
#include "channel.h"



/******************************************************************
**  compiler directives
*******************************************************************/

/* Project dependend defines */
//#define EMU         //CANalyzer
//#define PARA
//#define DEVNET      //DeviceNet
//#define DB_BUG 
//NEC PD72005 or Intel 82527
/******************************************************************
**  macros
*******************************************************************/

/* Linux compatibility */
#define SCHAR SCHAR


#define SW_VERSION 520 // used in get version
#define CAN_DRIVER_VERSION "5.20.1 for Kernel 3.x"


#define CAN_HALT TRUE
#define CAN_RUN FALSE


#define int_         PRAEDEF int MIDDEF

#ifdef LINUX
#define U_PTR
#define F_PTR
#define N_PTR
#endif



/*---------------------------------------*/
/*             constants                 */
/*---------------------------------------*/

#define  LAYER2_FW              0x01

#define  OK_                    0

#define  ERROR1                -1
#define  ERROR2                -2
/*
#define  ERROR3                -3
#define  ERROR4                -4
#define  ERROR5                -5
#define  ERROR6                -6
#define  ERROR7                -7
*/

#define  RET_CODE0              0
#define  RET_CODE1              1
#define  RET_CODE2              2
#define  RET_CODE3              3
#define  RET_CODE4              4
#define  RET_CODE5              5
#define  RET_CODE6              6
#define  RET_CODE7              7
#define  RET_CODE8              8
#define  RET_CODE9              9
#define  RET_CODE10             10
#define  RET_CODE11             11
#define  RET_CODE12             12
#define  RET_CODE13             13
#define  RET_CODE14             14
#define  RET_CODE15             15

#define  SET_                   0x01
#define  RESET_                 0x00

#define  TRUE_                  0x01
#define  FALSE_                 0x00

#define  ERROR_ACTIVE           0
#define  ERROR_PASSIVE          1
#define  BUS_OFF                2

#define  MAX_IDENT              0x1FFFFFFFL
#ifdef PARA
#define  MAX_OBJ_DYN            100
#else
#define  MAX_OBJ_DYN            200
#endif

//command bytes
/* Bit  7      6         5      4     3     2     1     0    */
/*    CAN2 HIGH_TIME HALF_TICK       RMT   XTD XMT_ACK XMT   */
#define CAN2_MASK               0x80
#define HIGH_TIME_MASK          0x40
#define HALF_TICK_MASK          0x20
#define RMT_MASK                0x08
#define XTD_MASK                0x04
#define XMT_ACK_MASK            0x02
#define XMT_MASK                0x01

#define RCV_CMD                 0x00
#define RCV_CMD_XTD             RCV_CMD | XTD_MASK
#define RCV_CMD_RMT             RCV_CMD | RMT_MASK
#define RCV_CMD_XTD_RMT         RCV_CMD | XTD_MASK | RMT_MASK

#define RCV_CMD2                0x00                              | CAN2_MASK
#define RCV_CMD_XTD2            RCV_CMD | XTD_MASK                | CAN2_MASK
#define RCV_CMD_RMT2            RCV_CMD | RMT_MASK                | CAN2_MASK
#define RCV_CMD_XTD_RMT2        RCV_CMD | XTD_MASK | RMT_MASK     | CAN2_MASK

#define XMT_CMD                 XMT_MASK
#define XMT_CMD_XTD             XMT_CMD | XTD_MASK
#define XMT_CMD_RMT             XMT_CMD | RMT_MASK
#define XMT_CMD_XTD_RMT         XMT_CMD | XTD_MASK | RMT_MASK

#define XMT_CMD2                XMT_MASK                          | CAN2_MASK
#define XMT_CMD_XTD2            XMT_CMD | XTD_MASK                | CAN2_MASK
#define XMT_CMD_RMT2            XMT_CMD | RMT_MASK                | CAN2_MASK
#define XMT_CMD_XTD_RMT2        XMT_CMD | XTD_MASK | RMT_MASK     | CAN2_MASK

#define XMT_ACK_CMD             XMT_ACK_MASK
#define XMT_ACK_CMD_XTD         XMT_ACK_CMD | XTD_MASK
#define XMT_ACK_CMD_RMT         XMT_ACK_CMD | RMT_MASK
#define XMT_ACK_CMD_XTD_RMT     XMT_ACK_CMD | XTD_MASK | RMT_MASK

#define XMT_ACK_CMD2            XMT_ACK_MASK                      | CAN2_MASK
#define XMT_ACK_CMD_XTD2        XMT_ACK_CMD | XTD_MASK            | CAN2_MASK
#define XMT_ACK_CMD_RMT2        XMT_ACK_CMD | RMT_MASK            | CAN2_MASK
#define XMT_ACK_CMD_XTD_RMT2    XMT_ACK_CMD | XTD_MASK | RMT_MASK | CAN2_MASK

#define ERR_CMD                 0x10
#define ERR_CMD2                ERR_CMD                           | CAN2_MASK
#define BUSLOAD_CMD             0x11
#define BUSLOAD_CMD2            BUSLOAD_CMD                       | CAN2_MASK
#define TRIGGER_CMD             0x14
#define TRIGGER_CMD2            TRIGGER_CMD                       | CAN2_MASK
#define TIMER_CMD               0x15
#define IO_CMD                  0x16    /* fuer CANalyzer IF1_I/01...IF2_I/02 
                                           piggyback Ansteuerung H/L-Speed  LeSe 990224 */

#ifndef _CANPROUSB_
#define ERR_BUS_STATE           0x17
#endif //_CANPROUSB_

#define  CHANNEL2_OFFSET         0x0800
#define  CANECO_CHANNEL2_OFFSET  0x0200

#ifdef DTS
#define  FAST_FIFO_BASE          0x0210L
#endif // DTS


#define  DPRAM_BASE_0_L         0xD0000000L
#define  DPRAM_BASE_0           0xD000
#define  DPRAM_SIZE             0x1000
#define  ATTRIBUTE_MEMORY_SIZE  0x1000


#define _PRINT_ERROR     0x1    // errors
#ifdef CANDEBUG

#define _PRINT_DISABLE   0x0    // no debug output
#define _PRINT_TRACE     0x2    // Ablaufverfolgung
#define _PRINT_MSG       0x4    // message
#define _PRINT_INT       0x8    // interrupt debugging
#define _PRINT_IO        0x10   // ioctl debugging
#define _PRINT_API       0x20   // API debugging


#ifndef CANACXPCI
#ifndef CANCARD
#ifndef CANAC104
#ifndef CANECO104
#ifndef CANPRO
#ifndef CANPCIE
#define _TRACE(String, Arguments...) TRACEPRINT(KERN_INFO " Softing: " String, ##Arguments)
#define _ERRPRINT(String, Arguments...)  if(can_trace_level & _PRINT_ERROR) TRACEPRINT(KERN_INFO " Softing: " String, ##Arguments)
#define _TRACE1(String, Arguments...) if(can_trace_level & _PRINT_TRACE) TRACEPRINT(KERN_ERR " Softing: " String, ##Arguments)
#define _TRACE2(String, Arguments...) if(can_trace_level & _PRINT_MSG) TRACEPRINT(KERN_INFO " Softing: " String, ##Arguments)
#define _TRACE3(String, Arguments...) if(can_trace_level & _PRINT_INT) TRACEPRINT(KERN_INFO " Softing: " String, ##Arguments)
#define _TRACE4(String, Arguments...) if(can_trace_level & _PRINT_IO) TRACEPRINT(KERN_INFO " Softing: " String, ##Arguments)
#define _TRACE5(String, Arguments...) if(can_trace_level & _PRINT_API) TRACEPRINT(KERN_INFO " Softing: " String, ##Arguments)
#endif // CANPCIE
#endif // CANPRO
#endif // CANECO104
#endif // CANAC104
#endif // CANCARD
#endif // CANACXPCI

#ifdef CANCARD
#define _TRACE(String, Arguments...) TRACEPRINT(KERN_INFO " CANcard: " String, ##Arguments)
#define _ERRPRINT(String, Arguments...) if(can_trace_level & _PRINT_ERROR) TRACEPRINT(KERN_INFO " CANcard: " String, ##Arguments)
#define _TRACE1(String, Arguments...) if(can_trace_level & _PRINT_TRACE) TRACEPRINT(KERN_ERR " CANcard: " String, ##Arguments)
#define _TRACE2(String, Arguments...) if(can_trace_level & _PRINT_MSG) TRACEPRINT(KERN_INFO " CANcard: " String, ##Arguments)
#define _TRACE3(String, Arguments...) if(can_trace_level & _PRINT_INT) TRACEPRINT(KERN_INFO " CANcard: " String, ##Arguments)
#define _TRACE4(String, Arguments...) if(can_trace_level & _PRINT_IO) TRACEPRINT(KERN_INFO " CANcard: " String, ##Arguments)
#define _TRACE5(String, Arguments...) if(can_trace_level & _PRINT_API) TRACEPRINT(KERN_INFO " CANcard: " String, ##Arguments)

#endif // CANCARD

#ifdef CANPRO
#define _TRACE(String, Arguments...) TRACEPRINT(KERN_INFO " CANcard: " String, ##Arguments)
#define _ERRPRINT(String, Arguments...) if(can_trace_level & _PRINT_ERROR) TRACEPRINT(KERN_INFO " CAN-PRO: " String, ##Arguments)
#define _TRACE1(String, Arguments...) if(can_trace_level & _PRINT_TRACE) TRACEPRINT(KERN_ERR " CAN-PRO: " String, ##Arguments)
#define _TRACE2(String, Arguments...) if(can_trace_level & _PRINT_MSG) TRACEPRINT(KERN_INFO " CAN-PRO: " String, ##Arguments)
#define _TRACE3(String, Arguments...) if(can_trace_level & _PRINT_INT) TRACEPRINT(KERN_INFO " CAN-PRO: " String, ##Arguments)
#define _TRACE4(String, Arguments...) if(can_trace_level & _PRINT_IO) TRACEPRINT(KERN_INFO " CAN-PRO: " String, ##Arguments)
#define _TRACE5(String, Arguments...) if(can_trace_level & _PRINT_API) TRACEPRINT(KERN_INFO " CAN-PRO: " String, ##Arguments)
#endif // CANPRO

#ifdef CANPCIE
#define _TRACE(String, Arguments...) TRACEPRINT(KERN_INFO " CANcard: " String, ##Arguments)
#define _ERRPRINT(String, Arguments...) if(can_trace_level & _PRINT_ERROR) TRACEPRINT(KERN_INFO " CAN-PRO-PCIe: " String, ##Arguments)
#define _TRACE1(String, Arguments...) if(can_trace_level & _PRINT_TRACE) TRACEPRINT(KERN_ERR " CAN-PRO-PCIe: " String, ##Arguments)
#define _TRACE2(String, Arguments...) if(can_trace_level & _PRINT_MSG) TRACEPRINT(KERN_INFO " CAN-PRO-PCIe: " String, ##Arguments)
#define _TRACE3(String, Arguments...) if(can_trace_level & _PRINT_INT) TRACEPRINT(KERN_INFO " CAN-PRO-PCIe: " String, ##Arguments)
#define _TRACE4(String, Arguments...) if(can_trace_level & _PRINT_IO) TRACEPRINT(KERN_INFO " CAN-PRO-PCIe: " String, ##Arguments)
#define _TRACE5(String, Arguments...) if(can_trace_level & _PRINT_API) TRACEPRINT(KERN_INFO " CAN-PRO-PCIe: " String, ##Arguments)
#endif // CANPCIE

#ifdef CANACXPCI
#define _TRACE(String, Arguments...) TRACEPRINT(KERN_INFO " CAN-ACx-PCI: " String, ##Arguments)
#define _ERRPRINT(String, Arguments...) if(can_trace_level & _PRINT_ERROR) TRACEPRINT(KERN_ERR " CAN-ACx-PCI: " String, ##Arguments)
#define _TRACE1(String, Arguments...) if(can_trace_level & _PRINT_TRACE) TRACEPRINT(KERN_INFO " CAN-ACx-PCI: " String, ##Arguments)
#define _TRACE2(String, Arguments...) if(can_trace_level & _PRINT_MSG) TRACEPRINT(KERN_INFO " CAN-ACx-PCI: " String, ##Arguments)
#define _TRACE3(String, Arguments...) if(can_trace_level & _PRINT_INT) TRACEPRINT(KERN_INFO " CAN-ACx-PCI: " String, ##Arguments)
#define _TRACE4(String, Arguments...) if(can_trace_level & _PRINT_IO) TRACEPRINT(KERN_INFO " CAN-ACx-PCI: " String, ##Arguments)
#define _TRACE5(String, Arguments...) if(can_trace_level & _PRINT_API) TRACEPRINT(KERN_INFO " CAN-ACx-PCI: " String, ##Arguments)

#endif //CANAC104

#ifdef CANAC104
#define _TRACE(String, Arguments...) TRACEPRINT(KERN_INFO " CAN-ACx-104: " String, ##Arguments)
#define _ERRPRINT(String, Arguments...) if(can_trace_level & _PRINT_ERROR) TRACEPRINT(KERN_ERR " CAN-ACx-104: " String, ##Arguments)
#define _TRACE1(String, Arguments...) if(can_trace_level & _PRINT_TRACE) TRACEPRINT(KERN_INFO " CAN-ACx-104: " String, ##Arguments)
#define _TRACE2(String, Arguments...) if(can_trace_level & _PRINT_MSG) TRACEPRINT(KERN_INFO " CAN-ACx-104: " String, ##Arguments)
#define _TRACE3(String, Arguments...) if(can_trace_level & _PRINT_INT) TRACEPRINT(KERN_INFO " CAN-ACx-104: " String, ##Arguments)
#define _TRACE4(String, Arguments...) if(can_trace_level & _PRINT_IO) TRACEPRINT(KERN_INFO " CAN-ACx-104: " String, ##Arguments)
#define _TRACE5(String, Arguments...) if(can_trace_level & _PRINT_API) TRACEPRINT(KERN_INFO " CAN-ACx-104: " String, ##Arguments)

#endif //CANAC104

#ifdef CANECO104
#define _TRACE(String, Arguments...) TRACEPRINT(KERN_INFO " CAN-ECOx-104: " String, ##Arguments)
#define _ERRPRINT(String, Arguments...) if(can_trace_level & _PRINT_ERROR) TRACEPRINT(KERN_ERR " CAN-ECOx-104: " String, ##Arguments)
#define _TRACE1(String, Arguments...) if(can_trace_level & _PRINT_TRACE) TRACEPRINT(KERN_INFO " CAN-ECOx-104: " String, ##Arguments)
#define _TRACE2(String, Arguments...) if(can_trace_level & _PRINT_MSG) TRACEPRINT(KERN_INFO " CAN-ECOx-104: " String, ##Arguments)
#define _TRACE3(String, Arguments...) if(can_trace_level & _PRINT_INT) TRACEPRINT(KERN_INFO " CAN-ECOx-104: " String, ##Arguments)
#define _TRACE4(String, Arguments...) if(can_trace_level & _PRINT_IO) TRACEPRINT(KERN_INFO " CAN-ECOx-104: " String, ##Arguments)
#define _TRACE5(String, Arguments...) if(can_trace_level & _PRINT_API) TRACEPRINT(KERN_INFO " CAN-ECOx-104: " String, ##Arguments)

#endif //CANAC104

#else
#define _TRACE(String, Arguments...)
#define _ERRPRINT(String, Arguments...) TRACEPRINT(KERN_ERR " CAN-ACx-104: " String, ##Arguments)
#define _TRACE1(String, Arguments...)
#define _TRACE2(String, Arguments...)
#define _TRACE3(String, Arguments...)
#define _TRACE4(String, Arguments...)
#define _TRACE5(String, Arguments...)

#endif //CANDEBUG
#define CAN_DPRAM_SIZE 0x1000
#define MINORUNDEFINED 0xFFFFFFFF

#define DOWN(sema)    if (down_interruptible(sema))                             \
              {                                         \
                  _ERRPRINT                                   \
                        ("CAN_IOC_CANPC_reset_board: down_interruptible() was interrupted\n");  \
                  atomic_dec(&pDevice->atomInUseCount);                     \
                  return (0x10000 - INIPC_IB_ERR_VC_IOCANCELLED);                 \
              }
#define UP(sema) up(sema)


#define ANZ_IDENT 2048



/******************************************************************
**  externals
*******************************************************************/




//extern SCHAR *dpram_ptr;
extern int can_trace_level;     
extern PCDEVICES pcCanDevices;

/******************************************************************
** protoypes  
*******************************************************************/

int ReadIniFileData(DEVICE_DATA16 * pddIniFileData, PCDEVICES pDevice);

int INIDR_initialize_board(PCDEVICES pDevice);

USIGN16 CanResetCard(BOOL bDoReset, PVOID pDevice);
USIGN16 CanEnableIrq(BOOL bDoReset, PVOID pDevice);
USIGN16 CanInit(BOOL bDoReset, PVOID pDevice);


USIGN16 pciInit(BOOL boNotUsed, PVOID pDevice);


USIGN16 CS_InitCnfReg(CSHANDLE notUsed, LPBYTE pAttrBase, PCDEVICES pDevice);
USIGN16 CS_SetCanCardReset(CSHANDLE notUsed, BOOL bDoReset,
                        USIGN8 far * pAttrBase, PCDEVICES pDevice);





#endif
