
/*
 * Softing CAN Driver
 * Copyright (C) 1994-2015, Softing AG, Werner Knorsch
 * 
 * See file CREDITS for list of people who contributed to this
 * project.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * email: support.automation@softing.com
 * Softing Industrial Automation GmbH, Richard-Reitzner-Allee 6, 85540 Haar, Germany
 */

#ifndef __CHANNEL_H__
#define __CHANNEL_H__


#define ANZ_IDENT 2048
#define MAX_OBJ_DYN 200


/* fifo constants */
#define RCV_FIFO_ITEM_LEN       32
#define RCV_FIFO_ITEMS          16
#define RCV_WRAP_AROUND_MASK    (RCV_FIFO_ITEMS-1)
#define XMT_FIFO_ITEM_LEN       16
#define XMT_FIFO_ITEMS          32
#define XMT_WRAP_AROUND_MASK    (XMT_FIFO_ITEMS-1)

/* fifo constants 512 Kb DPRAM*/
#define RCV_FIFO_ITEMS_CANPRO          4096  
#define RCV_WRAP_AROUND_MASK_CANPRO    RCV_FIFO_ITEMS_CANPRO-1
#define XMT_FIFO_ITEMS_CANPRO          4096
#define XMT_WRAP_AROUND_MASK_CANPRO    XMT_FIFO_ITEMS_CANPRO-1

/* fifo constants 256 Kb FIFO Buffer*/
#define RCV_FIFO_ITEMS_CANECO          2048  
#define RCV_WRAP_AROUND_MASK_CANECO    RCV_FIFO_ITEMS_CANECO-1
#define XMT_FIFO_ITEMS_CANECO          2048
#define XMT_WRAP_AROUND_MASK_CANECO    XMT_FIFO_ITEMS_CANECO-1

#define RCV_FIFO_ITEMS_PC8             2048  
#define RCV_WRAP_AROUND_MASK_PC8       RCV_FIFO_ITEMS_PC8   -1
#define XMT_FIFO_ITEMS_PC8             2048
#define XMT_WRAP_AROUND_MASK_PC8       XMT_FIFO_ITEMS_PC8   -1


/* base addresses in dpram */
#define  RCV_FIFO_BASE          0x0000L
#define  XMT_FIFO_BASE          0x0300L
#define  FCT_STRUCT_BASE        0x0210L
#define  INFO_STRUCT_BASE       0x0240L
#define  INT_STRUCT_BASE        0x07FEL

/* CAN-PRO base addresses in DPRAM */
#define  CAN_PRO_RCV_FIFO_BASE          0x00000000L
#define  CAN_PRO_XMT_FIFO_BASE          0x00020100L
#define  CAN_PRO_FCT_STRUCT_BASE        0x00020010L
#define  CAN_PRO_INFO_STRUCT_BASE       0x00020040L
#define  CAN_PRO_XMT_POLL_LIST_BASE     0x0003EFF0L

/* CAN-ECO base addresses in FIFO Buffer */
#define  CAN_ECO_RCV_FIFO_BASE          0x00000000L
#define  CAN_ECO_XMT_FIFO_BASE          0x00010080L
#define  CAN_ECO_INFO_STRUCT_BASE       0x00010020L
#define  CAN_ECO_XMT_POLL_LIST_BASE     0x0001F7F8L

#define  PC8_RCV_FIFO_BASE          0x00000000L
#define  PC8_XMT_FIFO_BASE          0x00010080L
#define  PC8_INFO_STRUCT_BASE       0x00010020L
#define  PC8_XMT_POLL_LIST_BASE     0x0001F7F8L

/*---------------------------------------*/
/*        structures in dpram            */
/*---------------------------------------*/
#pragma pack(push,1)
typedef struct
{
    USIGN8 rcv_fifo[RCV_FIFO_ITEMS][RCV_FIFO_ITEM_LEN];
    USIGN8 dummy1;
    USIGN16 rd_rcv_fifo;


    USIGN16 dummy2;
    USIGN16 wr_rcv_fifo;
    USIGN16 dummy3;
    USIGN16 lost_msg;
    USIGN8 dummy4;
}
__attribute__ ((packed)) RCV_FIFO_STRUCT;

typedef struct
{
    USIGN8 xmt_fifo[XMT_FIFO_ITEMS][XMT_FIFO_ITEM_LEN];
    USIGN8 dummy1;
    USIGN16 rd_xmt_fifo;
    USIGN16 dummy2;
    USIGN16 wr_xmt_fifo;
    USIGN8 dummy3;
}
__attribute__ ((packed)) XMT_FIFO_STRUCT;

#ifndef _CANPROUSB_
typedef struct
   {
   USIGN32 id;
   USIGN16 obj;
   USIGN16 rmt_obj;
   }  sorted_lookup;




                         /* Static Object Buffer mode RCV-BUFFER-RAM struct */
                         /* ----------------------------------------------- */

typedef struct
{
  // synchronization USIGN8s
  volatile USIGN8  host_access;
  volatile USIGN8  fw_access;
  
  // control bytes of receive objects +/
  volatile USIGN8  ctrl0;
  volatile USIGN8  ctrl1;
  
  // CAN data bytes
  volatile USIGN8  data[8];
  
  // Timestamp (data frame)
  volatile USIGN32 TimestampLow;
  volatile USIGN32 TimestampHigh; 

    // Timestamp (remote frame)
  volatile USIGN32 TimestampLowRem;
  volatile USIGN32 TimestampHighRem; 


  volatile USIGN8 DataLength;
  volatile USIGN8 DataLengthRem;
  
} RCV_OBJECT;



typedef struct  {
  volatile RCV_OBJECT object[ANZ_IDENT];
  
  // receive fifo +/
  volatile USIGN16   fifo[256];
  volatile USIGN8  fifo_read_ptr;
  volatile USIGN8  fifo_write_ptr;
  volatile USIGN16   fifo_lost_msg;
  
  // receive poll list +/
  volatile USIGN16   poll_list[ANZ_IDENT + 1];
  
}   RCV_STATIC_STRUCT;


                         /* Static Object Buffer mode XMT-BUFFER-RAM struct */
                         /* ----------------------------------------------- */



typedef struct 
{
   // data bytes
   volatile USIGN8 data[8];
   // timestamp for ack
   volatile USIGN32 TimestampLow;
   volatile USIGN32 TimestampHigh;
   // timestamp for remotes
   volatile USIGN32 TimestampLowRemote;
   volatile USIGN32 TimestampHighRemote;

   // data len ack
   volatile USIGN8 len_data;
   // data len remotes
   volatile USIGN8 len_remote;
                  
   // control and data bytes of transmit objects 
   volatile USIGN8 ctrl[2];

   // synchronization bytes
   volatile USIGN8  host_access;
   volatile USIGN8  fw_access;
}  XMT_OBJECT;


typedef struct  {
  
  // transmission request fifo 
  volatile USIGN16   req_fifo[256];
  volatile USIGN8  req_fifo_read_ptr;
  volatile USIGN8  req_fifo_write_ptr;
  volatile USIGN16   req_fifo_lost_msg;
  
  // transmit remote fifo 
  volatile USIGN16   rmt_fifo[256];
  volatile USIGN8  rmt_fifo_read_ptr;
  volatile USIGN8  rmt_fifo_write_ptr;
  volatile USIGN16   rmt_fifo_lost_remotes;
  
  // transmission acknowledge fifo 
  volatile USIGN16   ack_fifo[256];
  volatile USIGN8  ack_fifo_read_ptr;
  volatile USIGN8  ack_fifo_write_ptr;
  volatile USIGN16   ack_fifo_lost_acks;
  
  
  volatile XMT_OBJECT object[ANZ_IDENT];
  
  
}   XMT_STATIC_STRUCT;



                        /* Dynamic Object Buffer mode RCV-BUFFER-RAM struct */
                        /* ------------------------------------------------ */

typedef struct
{
  volatile RCV_OBJECT object[MAX_OBJ_DYN];
  
  // receive fifo +/
  volatile USIGN16   fifo[256];
  volatile USIGN8  fifo_read_ptr;
  volatile USIGN8  fifo_write_ptr;
  volatile USIGN16   fifo_lost_msg;
  
  volatile USIGN16 rcv_obj_lookup[ANZ_IDENT+1];
  volatile  sorted_lookup rcv_id_lookup[MAX_OBJ_DYN+1];
    
} RCV_DYN_STRUCT;



typedef struct
{
  volatile XMT_OBJECT object[MAX_OBJ_DYN];
  /* transmission request fifo */
  volatile USIGN16   req_fifo[256];
  volatile USIGN8  req_fifo_read_ptr;
  volatile USIGN8  req_fifo_write_ptr;
  volatile USIGN16   req_fifo_lost_msg;

  /* transmit remote fifo */
  volatile USIGN16   rmt_fifo[256];
  volatile USIGN8  rmt_fifo_read_ptr;
  volatile USIGN8  rmt_fifo_write_ptr;
  volatile USIGN16   rmt_fifo_lost_remotes;

  /* transmission acknowledge fifo */
  volatile USIGN16   ack_fifo[256];
  volatile USIGN8  ack_fifo_read_ptr;
  volatile USIGN8  ack_fifo_write_ptr;
  volatile USIGN16   ack_fifo_lost_acks;
  /* transmit poll list */
  volatile USIGN16  poll_list[MAX_OBJ_DYN + 1];

  volatile USIGN16  xmt_obj_lookup[ANZ_IDENT+1];
  volatile USIGN32 xmt_id_lookup[MAX_OBJ_DYN+1];

} XMT_DYN_STRUCT;

#endif // (defined(CANPRO) || defined(CANPCIE))
#pragma pack(pop)


// structures for cards with 512 KBytes shared RAM
typedef struct
{
  USIGN32 u32TimeLow; // time stamp low part
  USIGN32 u32TimeHigh;// time stamp high part
  USIGN32 u32Ident;   // CAN Identifier
  USIGN8 u8Data[8];   // CAN data bytes
  USIGN32 u32Info;    
  USIGN8 u8DataLen;   // data length (0..8)
  USIGN8 u8Xtd;       // extended or standard message
  USIGN8 u8Cmd;       // command byte (data received, ack received, bus state change ...)
  USIGN8 u8Busstate;  // actual bus state
  USIGN8 u8RxErrorCounter; // CAN receive error counter
  USIGN8 u8TxErrorCounter; // transmit error counter
  USIGN8 u8Err;
  USIGN8 u8dummy; // for future use.
} RCV_FIFO_ITEM;


typedef struct
{
  USIGN32 Ident;    // CAN Identifier
  USIGN8 u8Data[8]; // CAN data bytes
  USIGN8 u8DataLen; // data length (0..8)
  USIGN8 u8Xtd;     // extended or standard message
  USIGN8 u8Cmd;     // command byte (send data ...)
  USIGN8 u8Dummy;   // for future use.

} XMT_FIFO_ITEM;

typedef struct
   {
   volatile RCV_FIFO_ITEM rcv_fifo[RCV_FIFO_ITEMS_CANPRO]; // FIFO data
   volatile USIGN16  rd_rcv_fifo; // FIFO read pointer
   volatile USIGN16  wr_rcv_fifo; // FIFO write pointer
   volatile USIGN16  lost_msg;    // number of lost messages
   } RCV_FIFO_CAN_PRO_STRUCT;


   typedef struct
   {
     volatile XMT_FIFO_ITEM xmt_fifo[XMT_FIFO_ITEMS_CANPRO]; // FIFO data
     volatile USIGN16  rd_xmt_fifo; // FIFO read pointer
     volatile USIGN16  wr_xmt_fifo; // FIFO write pointer
   } XMT_FIFO_CAN_PRO_STRUCT;

   typedef struct
   {
     volatile RCV_FIFO_ITEM rcv_fifo[RCV_FIFO_ITEMS_CANECO]; // FIFO data
     volatile USIGN16  rd_rcv_fifo; // FIFO read pointer
     volatile USIGN16  wr_rcv_fifo; // FIFO write pointer
     volatile USIGN16  lost_msg;    // number of lost messages
   } RCV_FIFO_CAN_ECO_STRUCT;


   typedef struct
   {
     volatile XMT_FIFO_ITEM xmt_fifo[XMT_FIFO_ITEMS_CANECO]; // FIFO data
     volatile USIGN16  rd_xmt_fifo; // FIFO read pointer
     volatile USIGN16  wr_xmt_fifo; // FIFO write pointer
   } XMT_FIFO_CAN_ECO_STRUCT;
   
   typedef struct
   {
     volatile XMT_FIFO_ITEM xmt_fifo[XMT_FIFO_ITEMS_CANECO]; // FIFO data
     volatile USIGN16  rd_xmt_fifo; // FIFO read pointer
     volatile USIGN16  wr_xmt_fifo; // FIFO write pointer
   } XMT_FIFO_PC8_STRUCT;

   typedef struct
   {
     volatile RCV_FIFO_ITEM rcv_fifo[RCV_FIFO_ITEMS_CANECO]; // FIFO data
     volatile USIGN16  rd_rcv_fifo; // FIFO read pointer
     volatile USIGN16  wr_rcv_fifo; // FIFO write pointer
     volatile USIGN16  lost_msg;    // number of lost messages
   } RCV_FIFO_PC8_STRUCT;


   typedef struct
   {
     INT16 index;
     INT16 param_1;
     INT16 param_2;
     INT16 param_3;
     INT16 param_4;
     INT16 param_5;
     INT16 param_6;
     INT16 param_7;
     INT16 param_8;
     INT16 param_9;
     INT16 param_10;
     INT16 param_11;
     INT16 param_12;
     INT16 param_13;
     INT16 param_14;
     INT16 param_15;
     INT16 param_16;
     INT16 param_17;
     INT16 param_18;

     /* param_19 is used for time_wrap_10_real value,  */
     /* C165 platforms hand this parameter to the host */

     INT16 param_19;

     INT16 return_value;
     USIGN8 dummy1;
     USIGN16 host_access;
     USIGN8 dummy2;
   }
   __attribute__ ((packed)) FCT_STRUCT;


typedef struct
{   
  
  volatile USIGN16 bus_state;
  volatile USIGN32 change_bus_state_time_low; //4
  USIGN8           dummy1;
  USIGN8           dummy2;
  USIGN8           dummy3; //8
  volatile USIGN16 error_state; //9
  USIGN16          dummy4;
  volatile USIGN16 error_state2;
  USIGN16          dummy5;
  volatile USIGN16 reset;
  USIGN16          dummy6;
  volatile USIGN16 clear_rcv_fifo;
  volatile USIGN32 change_bus_state_time_high;
  USIGN16          dummy8;
  volatile USIGN16 time_reset;
  volatile USIGN8  enableAck; // enable ack - CAN-ECOx only
  volatile USIGN32 time;
  volatile USIGN32 time_wrap;
  volatile USIGN8  wr_start;
  volatile USIGN8  wr_end;
  volatile USIGN8  enableErrorFrameDetection; // enable error frame detect. - CAN-ECOx only
  USIGN16          dummy12;
  volatile USIGN16 AccCodeStd; // acceptance code std - CAN-ECOx only
  volatile USIGN16 AccMaskStd; // acceptance mask std - CAN-ECOx only
  volatile USIGN16 reset_rcv_fifo;
  USIGN8           dummy14;
  volatile USIGN8  reset_xmt_fifo;
  volatile USIGN8  read_fifo_levels;
  volatile USIGN16 rcv_fifo_level;
  volatile USIGN16 xmt_fifo_level;
  volatile USIGN8  criticalFw;
  volatile USIGN8  criticalHost;
  volatile USIGN32 AccCodeXtd;
  volatile USIGN32 AccMaskXtd;
#ifdef FANUC_PC8
  volatile USIGN64 Pc8StartTime;
  volatile USIGN64 Pc8TicksPerSecond;
  volatile USIGN64 Pc8UserStartTime;
  
#else
  volatile USIGN64 CanEcoStartTime;
  volatile USIGN64 CanEcoTicksPerSecond;
  volatile USIGN64 CanEcoUserStartTime;
#endif // FANUC_PC8

}  
__attribute__ ((packed)) INFO_STRUCT;


typedef struct
{
    USIGN8 int_to_host;
    USIGN8 int_to_card;
}
__attribute__ ((packed)) INT_STRUCT;

typedef enum
{
  ST_CANL2_INIT,
  ST_CANL2_RUN
} CANL2_STATE;



typedef union
{
    USIGN32 ul[3];
    USIGN16 ui[6];
    USIGN8 ch[12];
    /* bit addressable struct for pin_mask1 */
}
__attribute__ ((packed)) BITMASK_UNION;

typedef struct __can_async_struct__
{
    wait_queue_head_t read_queue;
    int irq_flag;
} CAN_ASYNC_STRUCT;


typedef union _SJA1000_
{
  SJA1000 sja1000;
} CAN_CTRL;

// DPRAM definition for cards with 4KBytes DPRAM
typedef struct
{
  volatile RCV_FIFO_STRUCT       *prcv_fifo; // pointer to the receive FIFO
  volatile XMT_FIFO_STRUCT       *pxmt_fifo; // pointer to the transmit FIFO
} DPRAM_4K;

// definitions for cards with 512 KBytes DPRAM


// used in FIFO mode
typedef struct
{
  volatile RCV_FIFO_CAN_PRO_STRUCT       *prcv_fifo; // pointer to the receive FIFO
  volatile XMT_FIFO_CAN_PRO_STRUCT       *pxmt_fifo; // pointer to the transmit FIFO
} DPRAM_CAN_PRO_FIFO_MODE;

// used in static object buffer mode
typedef struct
{
  volatile RCV_STATIC_STRUCT         *prcv_static; // receive struct contains the receive objects
  volatile XMT_STATIC_STRUCT         *pxmt_static; // transmit struct contains the transmit objects
} DPRAM_CAN_PRO_STAT_OBJ_BUF;

// used in dynamic object buffer mode
typedef struct
{
  volatile RCV_DYN_STRUCT         *prcv_dyn; // receive struct contains the receive objects
  volatile XMT_DYN_STRUCT         *pxmt_dyn; // transmit struct contains the transmit objects
} DPRAM_CAN_PRO_DYN_OBJ_BUF;

// used in FIFO mode
typedef struct
{
  volatile RCV_FIFO_CAN_ECO_STRUCT       *prcv_fifo; // pointer to the receive FIFO
  volatile XMT_FIFO_CAN_ECO_STRUCT       *pxmt_fifo; // pointer to the transmit FIFO
} DPRAM_CAN_ECO_FIFO_MODE;

typedef struct
{
  volatile RCV_FIFO_PC8_STRUCT       *prcv_fifo; // pointer to the receive FIFO
  volatile XMT_FIFO_PC8_STRUCT       *pxmt_fifo; // pointer to the transmit FIFO
} DPRAM_PC8_FIFO_MODE;

typedef union
{
  DPRAM_4K                    dpram4;
  DPRAM_CAN_PRO_FIFO_MODE     can_pro_fifo_mode;
  DPRAM_CAN_PRO_STAT_OBJ_BUF  can_pro_stat_obj_buf;
  DPRAM_CAN_PRO_DYN_OBJ_BUF   can_pro_dyn_obj_buf;
  DPRAM_CAN_ECO_FIFO_MODE     can_eco_fifo_mode;
  DPRAM_PC8_FIFO_MODE         pc8_fifo_mode;

} DPRAM;

typedef struct
{
  CAN_CTRL   *pCanCntrl; // CAN controller for cards without an own microprocessor
  USIGN8     *pFifoBuffer;
  USIGN32    AccCodeStd; // acceptance code std - CAN-ECOx and Fanuc PC8 only
  USIGN32    AccMaskStd; // acceptance mask std - CAN-ECOx and Fanuc PC8 only
  USIGN32    AccCodeXtd; // acceptance code xtd - CAN-ECOx and Fanuc PC8 only
  USIGN32    AccMaskXtd; // acceptance mask xtd - CAN-ECOx and Fanuc PC8 only
  USIGN8     enableErrorFrameDetection; // enable error frame detect. - CAN-ECOx and Fanuc PC8 only
  USIGN8     enableAck; // enable ack - CAN-ECOx only
  // global from now on 21.1.08
 // USIGN16    hw_version; // hardware version
  spinlock_t Spinlock;  // for synchronization between interrupt and ioctl.
  USIGN64    CanEcoStartTime; 
  
} CAN_ECO;

#define NO_COMMAND 0

typedef struct
{
  USIGN32 Ident;
  USIGN8  Xtd;
  USIGN8  Length;
  USIGN8  data[8];
  USIGN8  command;
  
} CAN_MESSAGE;


typedef struct
{
  CAN_CTRL    *pCanCntrl; // CAN controller for cards without an own microprocessor
  USIGN8      *pFifoBuffer;
  USIGN32     AccCodeStd; // acceptance code std - CAN-ECOx and Fanuc PC8 only
  USIGN32     AccMaskStd; // acceptance mask std - CAN-ECOx and Fanuc PC8 only
  USIGN32     AccCodeXtd; // acceptance code xtd - CAN-ECOx and Fanuc PC8 only
  USIGN32     AccMaskXtd; // acceptance mask xtd - CAN-ECOx and Fanuc PC8 only
  USIGN8      enableErrorFrameDetection; // enable error frame detect. - CAN-ECOx and Fanuc PC8 only
  USIGN8      enableAck; // enable ack - PC8 only
  spinlock_t  Spinlock;  // for synchronization between interrupt and ioctl.
  USIGN64     Pc8StartTime; 
  CAN_MESSAGE Message;
  
} FANUCPC8;

typedef struct
{
  USIGN32   lost_remotes; // lost remotes
  BOOLEAN   TransmitReqFifoEnable; // some configuration values of the object buffer, see API documentation
  BOOLEAN   ReceiveFifoEnable;
  BOOLEAN   ReceivePollAll;
  BOOLEAN   TransmitAckFifoEnableAll;
  BOOLEAN   TransmitAckEnableAll;
  USIGN16   *pxmt_poll_list; // transmit poll list, used in object buffer
} CAN_PRO;

#ifdef _CANPROUSB_
typedef enum
{
  CHAIN_STOPPED,
  CHAIN_STOP_CMD,
  CHAIN_STOPPED_ON_ERROR,
  CHAIN_IDLE,
  CHAIN_RUNNING
} IO_CHAIN_STATE;


typedef struct
{
  BYTE                      RcvCounter;
  BOOL                      XmtPending;
  BOOL                      RcvPending;
  BOOL                      CmdPending;
  BOOL                      enableXmtAck;
  BOOL                      transferStarted;
  BOOL                      ReceiveFifoEnable;
  BOOL                      TransmitRemoteFifoEnable;
  BOOL                      TransmitAckFifoEnable;
  BOOL                      TransmitReqFifoEnable;
  BOOL                      ReceiveEnableAll;
  BOOL                      ReceiveIntEnableAll;
  BOOL                      auto_remote_enable;
  BOOL                      TransmitAckEnableAll;
  BOOL                      TransmitAckFifoEnableAll;
  BOOL                      ReceivePollAll;
  INT32                     number_of_xtd_rcv_objects;
  INT32                     number_of_std_rcv_objects;
  INT32                     number_of_xtd_xmt_objects;
  INT32                     number_of_std_xmt_objects;
  INT32                     rcv_poll_count;
  INT32                     xmt_poll_count;
  
  BOOL                      lost_remotes;
  BOOL                      rcv_usb_enable; // 
  INT32                     FirmwareXmtFifoLevel;
  INT32                     XmtIdleCounter;
  IO_CHAIN_STATE            IoChainRcvState;
  IO_CHAIN_STATE            IoChainXmtState;
  T_CANUSBP_OBJECT_BUF_MODE *pObjectBuf;
  struct timer_list         rcv_timer;
  struct timer_list         xmt_timer;
  spinlock_t                Spinlock;  // for synchronization between rcv and xmt completion routine
  struct task_struct        *CheckTransferThread;
  
  CANPROUSB_DEV             device;
  T_CANUSBP_SHARED_MEM      *pSharedMem; // shared memory (receive FIFO, transmit FIFO) only in Windows it's a real shared memory!
}CAN_PRO_USB;
#endif


// this structure describes one device
typedef struct __DEVICEDATA__
{
  CAN_ADDRESS can_dpram_card_address;        // DPRAM address of the card
  CAN_ADDRESS can_dpram_address;      // virtual DPRAM address of the channel

  CAN_ADDRESS can_dpram_physaddress;      // physical DPRAM address 
  USIGN32     can_dpram_size;             // size of the DPRAM in Bytes
  CAN_ADDRESS can_base;                   // IoPort base used only with CAN-ACx-104
  INT32       can_irq;                  // interrupt number
  CAN_ADDRESS can_config_space;       // virtual configuration space address
  CAN_ADDRESS PciPorts[2];            // virtual address of PCI CNTRL and INTCSR register
  BOOLEAN     bIsOpen;                // TRUE, if device is opened by the application
  BOOLEAN     bIsStarted;             // device is ready for communication
  USIGN32     u32User;                // minor number of the user 
  USIGN32     u32MinorNumber;         // minor number
  PVOID       pFile;        // filepointer
  DEVICE_DATA16 ddDeviceData;     // device data for compatibility
  PVOID       pNextDevice;                  // pointer to next device
  PVOID       pOtherDevice;                 // pointer to the other channel of the same card
  USIGN32     Serialnumber; 
  struct proc_dir_entry* procFile;
  USIGN16     u16DeviceId;            // device ID (CANcard=1, CANcard SJA=4, CANcard2=5, CAN-ACx-PCI=7, CAN-ACx-PCI/DN=8, 
                                        // CAN-ACx-104=9, EDICcardC=0x102, EDICcard2=0x105)
  USIGN16     u16Socket;              // Socket or slot where the device is pluged
  USIGN8      au8IrqName[30];          // name shown in /proc/interrupts
  atomic_t    atomInUseCount;      // shows if an IO operation runs at that moment
  FCT_STRUCT  *fct;                // control structur for communication with the firmware
  INFO_STRUCT *info;              // info structure
  INT_STRUCT  *int_regs;           // int regs structure
  DPRAM       dpram_data;
  USIGN32     bus_state;              // CAN bus state (ERROR ACTIVE, ERROR PASSIVE, or BUS OFF)
 
  struct semaphore cDpramSemaphore;   // sequentialize DPRAM access

  USIGN8 old_status_reg;

  INT32 number_of_rcv_objects;    // number of defined receive objects on channel 1
  INT32 number_of_xmt_objects;    // number of defined transmit objects on channel 1
  INT32 dyn_enable;         // dynamic object buffer is enabled if this variable is TRUE
  INT32 object_buffer_enable;     // (dynamic or static) object buffer is enabled if this variable is TRUE
  USIGN8 int_reg;
  USIGN8 SignalsDefined;        // CAN lowspeed module is configured if TRUE
  BITMASK_UNION pb;         // configuration data of CAN lowspeed module
  timespec ToutStartTime;        // used for timout calculation for communication with the firmware

#ifdef CANCARD
#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,16) 
  dev_link_t *link;               // PCMCIA structure which describes the card 
#else
  struct pcmcia_device *link;
#endif
#endif
  CAN_ASYNC_STRUCT can_info;      // wait queue for interrupt handling (user mode process waits on this queue)
  USIGN8           u8FwId;                    // Firmware Id: // 0=no Firmware loaded; Layer2=0x1; CANalyzer=0x20; CANopen=0x40; DeviceNet=0x70
  USIGN8           u8NumOfCh;                 // number of channels located on this card
  USIGN8           u8PhysCh;                  // physical channel number
#ifdef FANUC_PC8
  FANUCPC8         pc8;
#else // FANUC_PC8
  CAN_ECO          caneco;
  CAN_PRO          canpro;
#endif // FANUC_PC8
  
#ifdef _CANPROUSB_
  CAN_PRO_USB      canprousb;
#endif
  
  INT32            hw_version; // hardware version
  INT32            fw_version; // firmware version
  INT32            bt_version; // bootblock version
  INT32            license;
  INT32            chip_type;
  CANL2_STATE      l2_state;

}
CDEVICES, *PCDEVICES;



#endif //__CHANNEL_H__
