

/*
    shared_usb_mem.h
    handles the shared memory access for the CANpro USB devices.
    Copyright (C) 2015  kno@softing.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

// host fifo definition
#define RCV_FIFO_ITEMS_USB_PRO 4096
#define RCV_FIFO_USB_MASK (RCV_FIFO_ITEMS_USB_PRO-1)
#define XMT_FIFO_ITEMS_USB_PRO 4096
#define XMT_FIFO_USB_MASK (XMT_FIFO_ITEMS_USB_PRO-1)

// firmware fifo definitions
#define XMT_FW_USB_ITEMS 255
#define RCV_FW_USB_ITEMS 255

/* Bit  7      6         5      4     3     2     1     0    */
/*   SUP_OB CMD_ACK  BUS STATE ERR   RMT   XTD XMT_ACK XMT   */
#define RMT_MASK                0x08
#define XTD_MASK                0x04
#define XMT_ACK_MASK            0x02
#define XMT_MASK                0x01
#define CMD_ACK_MASK            0x40
#define SUP_OB_MASK             0x80

#define INT_TST_CMD             0x0FF
#define RCV_CMD                 0x00
#define RCV_CMD_XTD             RCV_CMD | XTD_MASK
#define RCV_CMD_RMT             RCV_CMD | RMT_MASK
#define RCV_CMD_XTD_RMT         RCV_CMD | XTD_MASK | RMT_MASK


#define XMT_CMD                 XMT_MASK
#define XMT_CMD_XTD             XMT_CMD | XTD_MASK
#define XMT_CMD_RMT             XMT_CMD | RMT_MASK
#define XMT_CMD_XTD_RMT         XMT_CMD | XTD_MASK | RMT_MASK

#define XMT_ACK_CMD             XMT_ACK_MASK
#define XMT_ACK_CMD_XTD         XMT_ACK_CMD | XTD_MASK
#define XMT_ACK_CMD_RMT         XMT_ACK_CMD | RMT_MASK
#define XMT_ACK_CMD_XTD_RMT     XMT_ACK_CMD | XTD_MASK | RMT_MASK


#define ERR_CMD                 0x10
#ifdef ERR_BUS_STATE
#undef ERR_BUS_STATE
#endif
#define ERR_BUS_STATE           0x20
#define TIMER_CMD               0x15

// object buffer
#define XMT_OBJ                 XMT_MASK
#define SUP_OBJ                 SUP_OB_MASK
#define WRT_OBJ                 SUP_OB_MASK | XMT_MASK
#define XMT_REM_OBJ             XMT_MASK    | RMT_MASK



#define FCT_RESET_CHIP                  0
#define FCT_INITIALIZE_CHIP             1
#define FCT_REINITIALIZE                2
#define FCT_SET_MODE                    3
#define FCT_RESET_LOST_MSG_COUNTER      4
#define FCT_SET_OUTPUT_CONTROL          5
#define FCT_ENABLE_DELAYED_POLL_MODE    6
#define FCT_SET_ACCEPTANCE              7
#define FCT_ECHO_TX_REQ                 9
#define FCT_START_CHIP                  11
#define FCT_GET_VERSION                 12
#define FCT_ENABLE_FIFO_TRANSMIT_ACK    13
#define FCT_ENABLE_EMU_CHECK            15
#define FCT_ENABLE_DYN_OBJ_BUF          16
#define FCT_INITIALIZE_INTERFACE        17
#define FCT_DEFINE_OBJECT               18
#define FCT_READ_AC                     20
#define FCT_SEND_REMOTE_OBJECT          21
#define FCT_SUPPLY_OBJECT_DATA          23
#define FCT_SUPPLY_RCV_OBJECT_DATA      25
#define FCT_SEND_OBJECT                 27
#define FCT_WRITE_OBJECT                29
#define FCT_READ_RCV_DATA               31
#define FCT_READ_XMT_DATA               33
#define FCT_ENABLE_DDE_CHECK            35
#define FCT_ENABLE_FIFO                 36
#define FCT_SET_TIMER_RATE              37
#define FCT_ENABLE_J1939_CHECK          38
#define FCT_DEFINE_CYCLIC               39
#define FCT_ENABLE_OE_CHECK             41
#define FCT_ENABLE_IVM_CHECK            42
#define FCT_GET_SN                      43
#define FCT_ENABLE_L1_CHECK             47
#define FCT_ENABLE_L2_CHECK             48
#define FCT_SET_L1_LIC                  49
#define FCT_SET_L2_LIC                  50
#define FCT_ENABLE_ERROR_FRAME          51
#define FCT_OBJECT_FILTER               52

#define FCT_INIT_SIGNALS                53
#define FCT_READ_SIGNALS                54
#define FCT_WRITE_SIGNALS               55


#define FCT_SET_IRQ_NUM                 58


#define FCT_READ_FIFO_LEVELS            72
#define FCT_RESET_RCV_FIFO				73
#define FCT_RESET_XMT_FIFO				74


#define FCT_SET_RED_LED_FEATURES        75
#define FCT_SET_GREEN_LED_FEATURES      76


#define FCT_NO_CMD                      99

#ifdef MAX_IDENT
#undef MAX_IDENT
#endif
#define MAX_IDENT_STD           2047
#define MAX_IDENT_XTD           0x1FFFFFFFL
#define ANZ_IDENT               2048
// to do change back, when card with 512 KB Dpram is usable 2048
#define MAX_OBJ_FIFO            100
#define MAX_OBJ_DYN             200
#define XTD_OBJ_NOT_VALID               0xFFFFFFFFL
#define STD_OBJ_NOT_VALID               0xFFFFL

#define NOT_VALID_INT           -1
#define NOT_VALID_UI            0xFFFF
#define END_MARK_WORD           0xFFFF

#define  RTR_BIT                0x0010
#define  RTR_MASK               0x0800

/* RCV_CTRL_0 */
#define  HOST_ACCESS            0x01

/* RCV_CTRL_1 */
#define  RCV_STATUS_DATA        0x01
#define  RCV_STATUS_REMOTE      0x02
#define  RCV_STATUS_DATA_OR_REMOTE 0x03
#define  RCV_STATUS_OVERRUN     0x04
#define  XTD_RCV                0x10
#define  AUTO_REMOTE_ENABLE     0x20
#define  HOST_INTERRUPT_ENABLE  0x40
#define  RCV_ENABLE             0x80

/* XMT_CTRL_0 removed*/

/* XMT_CTRL_1 now  XMT_CTRL_0*/
#define  DATA_LENGTH_0          0x01
#define  DATA_LENGTH_1          0x02
#define  DATA_LENGTH_2          0x04
#define  DATA_LENGTH_3          0x08
#define  DATA_LENGTH            0x0F
#define  XTD_XMT                0x10
#define  XMT_REQ_REMOTE         0x20
#define  XMT_REQ_DATA           0x40

/* XMT_CTRL_2 now XMT_CTRL_1*/
#define  XMT_STATUS             0x01
#define  XMT_STATUS_OVERRUN     0x02
#define  XMT_DATA_VALID         0x10
#define  XMT_ACK_ENABLE         0x40
#define  XMT_ACK_FIFO_ENABLE    0x80

/* XMT_CTRL_3 now XMT_CTRL_1*/
//#define  RMT_XMT_REQ_DATA       0x01
//#define  XMT_ACK_RMT            0x02
#define  RMT_XMT_REQ_DATA       0x04
#define  XMT_ACK_RMT            0x08

#define OK                               0


#define  RET_CODE0              0
#define  RET_CODE1              1
#define  RET_CODE2              2
#define  RET_CODE3              3
#define  RET_CODE4              4
#define  RET_CODE5              5
#define  RET_CODE6              6
#define  RET_CODE7              7
#define  RET_CODE8              8
#define  RET_CODE9              9
#define  RET_CODE10             10
#define  RET_CODE11             11
#define  RET_CODE12             12

#define  ERROR_ACTIVE           0
#define  ERROR_PASSIVE          1
#define  BUS_OFF                2

#ifdef LINUX
#define KSPIN_LOCK spinlock_t
#endif

#pragma pack(push,1)

typedef struct
{
	volatile USIGN32 u32TimeLow; // time stamp low part
	volatile USIGN32 u32TimeHigh;// time stamp high part
	volatile USIGN32 u32Ident;   // CAN Identifier
	volatile USIGN8 u8Data[8];   // CAN data bytes
  volatile USIGN16 u16LostMsg; // lost msg in fifo mode
  volatile USIGN16 u16Info;    // not used.
	//USIGN32 u32Info;    
	volatile USIGN8 u8DataLen;   // data length (0..8)
	volatile USIGN8 u8Xtd;       // extended or standard message
	volatile USIGN8 u8Cmd;       // command byte (data received, ack received, bus state change ...)
	volatile USIGN8 u8Busstate;  // actual bus state
	volatile USIGN8 u8RxErrorCounter; // CAN receive error counter
	volatile USIGN8 u8TxErrorCounter; // transmit error counter
	volatile USIGN8 u8Err;
	volatile USIGN8 counter; // for future use.
} USBRCVITEM;

typedef struct
{
  volatile USBRCVITEM entry[16];
  volatile USIGN32 numOfEntries;
} USB_RECEIVE_BUFFER;


typedef struct
{
  volatile USIGN16 can_cmd;
  volatile USIGN32 can_id;
  volatile USIGN8  data[8];
  volatile USIGN8  can_len;
  volatile USIGN8  xtd;
  volatile USIGN8  dummy[12];
  volatile USIGN32 u32Counter;
} USBXMT_ITEM;



typedef struct
{
	volatile USBRCVITEM rcv_fifo[RCV_FIFO_ITEMS_USB_PRO]; // FIFO data
	volatile USIGN16  rd_rcv_fifo; // FIFO read pointer
	volatile USIGN16  wr_rcv_fifo; // FIFO write pointer
	volatile USIGN16  lost_msg;    // number of lost messages
} rcv_fifo_can_pro_usb_struct;

#define ANZ_IDENT               2048
#define MAX_OBJ_DYN             200

#define  RTR_MASK               0x0800
#define END_MARK_WORD           0xFFFF

#define NOT_VALID               0xFFFFFFFF



/* RCV_CTRL_1 */
#define  RCV_STATUS_DATA        0x01
#define  RCV_STATUS_REMOTE      0x02
#define  RCV_STATUS_DATA_OR_REMOTE 0x03
#define  RCV_STATUS_OVERRUN     0x04
#define  XTD_RCV                0x10
#define  AUTO_REMOTE_ENABLE     0x20
#define  HOST_INTERRUPT_ENABLE  0x40
#define  RCV_ENABLE             0x80




/* XMT_CTRL_1 now  XMT_CTRL_0*/
#define  XTD_XMT                0x10
#define  XMT_REQ_DATA           0x40
#define  XMT_REQ_REMOTE         0x20

/* XMT_CTRL_2 now XMT_CTRL_1*/
#define  XMT_STATUS             0x01
#define  XMT_STATUS_OVERRUN     0x02
#define  XMT_DATA_VALID         0x10
#define  XMT_ACK_ENABLE         0x40
#define  XMT_ACK_FIFO_ENABLE    0x80

/* XMT_CTRL_3 now XMT_CTRL_1*/
#define  RMT_XMT_REQ_DATA       0x04
#define  XMT_ACK_RMT            0x08



#ifndef CANL2_DLL
typedef struct
   {
   USIGN32 id;
   USIGN16 obj;
   USIGN16 rmt_obj;
   }  sorted_lookup;

typedef struct
{
  KSPIN_LOCK  access;
  
  // control bytes of receive objects +/
  volatile USIGN8  ctrl0;
  volatile USIGN8  ctrl1;
  
  // CAN data bytes
  volatile USIGN8  data[8];
  
  // Timestamp (data frame)
  volatile USIGN32 Timestamp;

    // Timestamp (remote frame)
  volatile USIGN32 TimestampRem;


  volatile USIGN8 DataLength;
  volatile USIGN8 DataLengthRem;
  
} RCV_OBJECT;

                        /* Dynamic Object Buffer mode RCV-BUFFER-RAM struct */
                        /* ------------------------------------------------ */
#define ANZ_IDENT               2048

typedef struct
{
  RCV_OBJECT object[MAX_OBJ_DYN];
  
  // receive fifo +/
  volatile USIGN16   fifo[256];
  volatile USIGN8  fifo_read_ptr;
  volatile USIGN8  fifo_write_ptr;
  volatile USIGN16   fifo_lost_msg;
  
  volatile USIGN16 rcv_obj_lookup[ANZ_IDENT+1];
	volatile 	sorted_lookup rcv_id_lookup[MAX_OBJ_DYN+1];
    
} RCV_DYN_STRUCT;




typedef struct  {
  RCV_OBJECT object[ANZ_IDENT];
  
  // receive fifo +/
  volatile USIGN16   fifo[256];
  volatile USIGN8  fifo_read_ptr;
  volatile USIGN8  fifo_write_ptr;
  volatile USIGN16   fifo_lost_msg;
  
  // receive poll list +/
  volatile USIGN16   poll_list[ANZ_IDENT + 1];
  
}   RCV_STATIC_STRUCT;

typedef struct 
{
   // data bytes
   volatile USIGN8 data[8];
   // timestamp for ack
   volatile USIGN32 Timestamp;
   // timestamp for remotes
   volatile USIGN32 TimestampRemote;

   // data len ack
   volatile USIGN8 len_data;
   // data len remotes
   volatile USIGN8 len_remote;
                  
   // control and data bytes of transmit objects 
   volatile USIGN8 ctrl[2];
   KSPIN_LOCK  access;
}  XMT_OBJECT;

typedef struct
{
  XMT_OBJECT object[MAX_OBJ_DYN];
  /* transmission request fifo */
  volatile USIGN16   req_fifo[256];
  volatile USIGN8  req_fifo_read_ptr;
  volatile USIGN8  req_fifo_write_ptr;
  volatile USIGN16   req_fifo_lost_msg;

  /* transmit remote fifo */
  volatile USIGN16   rmt_fifo[256];
  volatile USIGN8  rmt_fifo_read_ptr;
  volatile USIGN8  rmt_fifo_write_ptr;
  volatile USIGN16   rmt_fifo_lost_remotes;

  /* transmission acknowledge fifo */
  volatile USIGN16   ack_fifo[256];
  volatile USIGN8  ack_fifo_read_ptr;
  volatile USIGN8  ack_fifo_write_ptr;
  volatile USIGN16   ack_fifo_lost_acks;
  /* transmit poll list */
  volatile USIGN16  poll_list[MAX_OBJ_DYN + 1];

  volatile USIGN16  xmt_obj_lookup[ANZ_IDENT+1];
  volatile USIGN32 xmt_id_lookup[MAX_OBJ_DYN+1];

} XMT_DYN_STRUCT;


typedef struct  {
  
  // transmission request fifo 
  volatile USIGN16   req_fifo[256];
  volatile USIGN8    req_fifo_read_ptr;
  volatile USIGN8    req_fifo_write_ptr;
  volatile USIGN16   req_fifo_lost_msg;
  
  // transmit remote fifo 
  volatile USIGN16   rmt_fifo[256];
  volatile USIGN8    rmt_fifo_read_ptr;
  volatile USIGN8    rmt_fifo_write_ptr;
  volatile USIGN16   rmt_fifo_lost_remotes;
  
  // transmission acknowledge fifo 
  volatile USIGN16   ack_fifo[256];
  volatile USIGN8    ack_fifo_read_ptr;
  volatile USIGN8    ack_fifo_write_ptr;
  volatile USIGN16   ack_fifo_lost_acks;
  
  volatile USIGN16  xmt_poll_list[ANZ_IDENT + 1];

  XMT_OBJECT object[ANZ_IDENT];
  
  
}   XMT_STATIC_STRUCT;


// used in static object buffer mode
typedef struct
{
	 RCV_STATIC_STRUCT         rcv_static; // receive struct contains the receive objects
	 XMT_STATIC_STRUCT         xmt_static; // transmit struct contains the transmit objects
} DPRAM_CAN_PRO_USB_STAT_OBJ_BUF;

// used in dynamic object buffer mode
typedef struct
{
	RCV_DYN_STRUCT         rcv_dyn; // receive struct contains the receive objects
	XMT_DYN_STRUCT         xmt_dyn; // transmit struct contains the transmit objects
} DPRAM_CAN_PRO_USB_DYN_OBJ_BUF;


typedef union
{
  DPRAM_CAN_PRO_USB_STAT_OBJ_BUF  can_pro_usb_stat_obj_buf;
  DPRAM_CAN_PRO_USB_DYN_OBJ_BUF   can_pro_usb_dyn_obj_buf;
} T_CANUSBP_OBJECT_BUF_MODE;

#endif // CANL2_DLL


/////////////////////////////////////////////////////////////////////////////

typedef struct
{
	volatile USBXMT_ITEM xmt_fifo[XMT_FIFO_ITEMS_USB_PRO]; // FIFO data
	volatile USIGN16  rd_xmt_fifo; // FIFO read pointer
	volatile USIGN16  wr_xmt_fifo; // FIFO write pointer
} xmt_fifo_can_pro_usb_struct;


// used in FIFO mode
typedef struct
{
 volatile xmt_fifo_can_pro_usb_struct xmt;
 volatile rcv_fifo_can_pro_usb_struct rcv;
} DPRAM_CAN_PRO_USB_FIFO_MODE;

#define USB_OK        0
#define USB_XMT_ERROR 1
#define USB_RCV_ERROR 2

typedef struct
{
  volatile DPRAM_CAN_PRO_USB_FIFO_MODE     can_pro_usb_fifo_mode;
  volatile USIGN32 time;
  volatile USIGN16 lost_messages; // lost messages inside the rcv fifo of the firmware
  volatile USIGN16 lostMsgOnHostFifo; // lost messages inside the rcv fifo of the host side
//   volatile INT32   FirmwareXmtFifoLevel;
  volatile USIGN8  can_state;
  volatile USIGN8  usb_error;
  volatile USB_RECEIVE_BUFFER WrapAroundBuffer;
} T_CANUSBP_SHARED_MEM;



#pragma pack(pop)

typedef struct
{
  struct usb_endpoint_descriptor *cmd_out_ep;
  struct urb                     *cmd_out_urb;
  char                           *pcmd_out_buf;
  struct usb_endpoint_descriptor *cmd_in_ep;
  struct urb                     *cmd_in_urb;
  char                           *pcmd_in_buf;
  struct usb_endpoint_descriptor *dat_out_ep;
  struct urb                     *dat_out_urb;
  char                           *pdat_out_buf;
  INT32                          dat_out_buf_size;
  struct usb_endpoint_descriptor *dat_in_ep;
  struct urb                     *dat_in_urb;
  char                           *pdat_in_buf;
  INT32                          dat_in_buf_size;
  struct usb_interface           *intf;
  BOOLEAN                        isValid;
} CANPROUSB_DEV;

