/***************************************************************************/
/*                                                                         */
/*   Softing GmbH          Richard-Reitzner-Allee 6       85540 Haar       */
/*                                                                         */
/***************************************************************************/
/*                                                                         */
/*   Copyright (C) by Softing GmbH, 1998, All rights reserved.             */
/*                                                                         */
/***************************************************************************/
/*                                                                         */
/*                     sja1000.h                                           */
/*                                                                         */
/***************************************************************************/
/*                                                                         */
/* PROJECT:            CANCARD FIRMWARE                                    */
/*                                                                         */
/* AUTHOR:             Lars Verbesek                                       */
/*                                                                         */
/* MODULE_DESCRIPTION  definition of SJA1000 related structures and        */
/*                     constants                                           */
/*                                                                         */
/***************************************************************************/

/* ----------------------------------------------------------------------- */
/*   SJA1000 register and internal RAM structure (PeliCAN-mode) definition */
/* ----------------------------------------------------------------------- */

#ifndef __SJA1000_H__
#define __SJA1000_H__

#pragma pack(push,1)

typedef struct {
                                                          /* Mode Register */
    unsigned char Mode;
                                                       /* Command Register */
    unsigned char Command;
                                                        /* Status Register */
    unsigned char Status;
                                                     /* Interrupt Register */
    unsigned char Int;
                                              /* Interrupt Enable Register */
    unsigned char IntEnab;
                                       /* reserved register for future use */
    unsigned char Resvd1;
                                                  /* Bus Timing Register 0 */
    unsigned char BusTime0;
                                                  /* Bus Timing Register 1 */
    unsigned char BusTime1;
                                                /* Output Control Register */
    unsigned char OutContr;
                                         /* Test Register (no description) */
    unsigned char Test;
                                       /* reserved register for future use */
    unsigned char Resvd2;
                                      /* Arbitration Lost Capture Register */
    unsigned char ArbLostCapt;
                                            /* Error Code Capture Register */
    unsigned char ErrCodeCapt;
                                           /* Error Warning Limit Register */
    unsigned char ErrWarnLimit;
                                              /* RX Error Counter Register */
    unsigned char RxErrCount;
                                              /* TX Error Counter Register */
    unsigned char TxErrCount;

                           /* Union respresenting the same addresses for   */
                           /* RX buffer, TX buffer, Acceptance Code/Mask   */
                           /* register                                     */ 
                           /* Access:                                      */
                           /*  - RX-Buffer - operating mode + read access  */
                           /*  - TX Buffer - operating mode + write access */
                           /*  - Accept. Reg. - reset mode + read/write    */
    union RXTXAccBuf 
    {
      struct RX_Mess_struct {
        unsigned char Dummy[13];                /* only filling stuff !!   */
        unsigned char  Count;                   /* Receive Message Counter */
        } RX_Mess;

      struct RX_struct {
        unsigned char Buffer[13];
        } RX;
                                                              /* TX buffer */
      struct TX_struct {
        volatile unsigned char Buffer[13];
        } TX;
                                                   /* Acceptance registers */
      struct Accept_struct {
        unsigned char Code[4];
        unsigned char Mask[4];
        unsigned char Resvd3[5];
        } Accept;

      } u;
    /* !!! NOTICE: RxMessageCounter is in the union, because of the even
       alignment rules of the compiler cc166 !!!                           */
    
                                           /* Receive Buffer Start Address */
    unsigned char RxBufStartAdd;
                                                          /* Clock Divider */
    unsigned char ClockDiv;

    unsigned char RxFIFO[64];
    unsigned char TxBuffer[64];
    } SJA1000;

#pragma pack(pop)
                                       /* CAN controller internal RAM */
/*
struct internalRAM 
{
    unsigned char RxFIFO[64];
    unsigned char TxBuffer[64];
};


typedef struct can_dpram 
{
    struct registers regs;
    struct internalRAM intRAM; 
} SJA1000;
*/


/* ----------------------------------------------------------------------- */
/*                                              SJA1000 register bit masks */
/* ----------------------------------------------------------------------- */

                                                    /* Reset Mode Register */
                                                    /* ------------------- */

                                                             /* Sleep Mode */
#define SM                 0x10
                                                 /* Acceptance Filter Mode */
#define AFM                0x08
                                                         /* Self Test Mode */
#define STM                0x04
                                                       /* Listen Only Mode */
#define LOM                0x02
                                                             /* Reset Mode */
#define RM                 0x01

                                                       /* Command Register */
                                                       /* ---------------- */

                                                 /* Self Reception Request */
#define SRR                0x10
                                                     /* Clear Data Overrun */
#define CDO                0x08
                                                 /* Release Receive Buffer */
#define RRB                0x04
                                                     /* Abort Transmission */
#define ABT                0x02
                                                   /* Transmission Request */
#define TR                 0x01

                                                        /* Status Register */
                                                        /* --------------- */

                                                             /* Bus Status */
#define BS                 0x80
                                                           /* Error Status */
//#define ES                 0x40
                                                        /* Transmit Status */
#define TS                 0x20
                                                         /* Receive Status */
#define RS                 0x10
                                           /* Transmission Complete Status */
#define TCS                0x08
                                                 /* Transmit Buffer Status */
#define TBS                0x04
                                                    /* Data Overrun Status */
#define DOS                0x02
                                                  /* Receive Buffer Status */
#define RBS                0x01

                                                     /* Interrupt Register */
                                                     /* ------------------ */

                                                    /* Bus Error Interrupt */
#define BEI                0x80
                                             /* Arbitration Lost Interrupt */
#define ALI                0x40
                                                /* Error Passive Interrupt */
#define EPI                0x20
                                                      /* Wake-Up Interrupt */
#define WUI                0x10
                                                 /* Data Overrun Interrupt */
#define DOI                0x08
                                                /* Error Warning Interrupt */
#define EI                 0x04
                                                     /* Transmit Interrupt */
#define TI                 0x02
                                                      /* Receive Interrupt */
#define RI                 0x01

                                              /* Interrupt Enable Register */
                                              /* ------------------------- */

                                             /* Bus Error Interrupt Enable */
#define BEIE               0x80
                                      /* Arbitration Lost Interrupt Enable */ 
#define ALIE               0x40
                                         /* Error Passive Interrupt Enable */
#define EPIE               0x20
                                               /* Wake-Up Interrupt Enable */
#define WUIE               0x10
                                          /* Data Overrun Interrupt Enable */
#define DOIE               0x08
                                         /* Error Warning Interrupt Enable */
#define EIE                0x04
                                              /* Transmit Interrupt Enable */
#define TIE                0x02
                                               /* Receive Interrupt Enable */
#define RIE                0x01

                                                /* Output Control Register */
                                                /* ----------------------- */

                                       /* TX1 Output Driver characteristic */
                                       /* of transistor connected to VDD   */
#define OCTP1              0x80
                                       /* TX1 Output Driver characteristic */
                                       /* of transistor connected to VSS   */
#define OCTN1              0x40
                                       /* TX1 Output Driver polarity       */
#define OCPOL1             0x20
                                       /* TX0 Output Driver characteristic */
                                       /* of transistor connected to VDD   */
#define OCTP0              0x10
                                       /* TX0 Output Driver characteristic */
                                       /* of transistor connected to VSS   */
#define OCTN0              0x08
                                       /* TX0 Output Driver polarity       */
#define OCPOL0             0x04
                                       /* TX Output Mode Bit1              */
#define OCMODE1            0x02
                                       /* TX Output Mode Bit0              */
#define OCMODE0            0x01
                                                /*  RX frame info. Reg.    */
                                                /* ----------------------- */
                                       /* frame format (set=EFF, Reset=SFF)*/ 
#define FF                 0x80
                                       /* remote (set) or data (reset)     */
#define RTR                0x40
                                       /* data length code information     */
#define DLC                0x0F
                          
#define ID2_0              0x00E0      /* mask for Identifier 2 to 0 (std) */


                                                /*  Clock Divider Register */
                                                /* ----------------------- */
                                       /* CAN mode definition bit 
                                                   (BasicCAN or PeliCAN)   */
#define CAN_MODE           0x80
                                       /* CAN comparator bypass control bit*/
#define CBP                0x40
                                       /* pin TX1 used for rec. interrupt  */
#define RXINTERN           0x20
                                       /* pin TX1 used for rec. interrupt  */
                                       /* clock off bit, 
                                            control of the CLK OUT pin     */
#define CLKOFF             0x08
                                       /* CKLOUT = oscillator frequency    */
#define DIVBY1             0x00        
                                       /* CKLOUT = 1/2 oscillator frequency*/
#define DIVBY2             0x07       

#endif //__SJA1000_H__

