/****************************************************************************/
/*                                                                          */
/*   Softing Industrial Automation GmbH   Richard-Reitzner-Allee 6   85540 Haar        */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*  Copyright (c) Softing Industrial Automation GmbH, All rights reserved.  */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*                               C M A D E F . H                            */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* PROJECT:            CANopen Client API                                   */
/*                                                                          */
/* MODULE:             CMADEF.H                                             */
/*                                                                          */
/* AUTHOR:             Softing Industrial Automation GmbH                              */
/*                                                                          */
/* MODULE_DESCRIPTION: macro definitions, struktures and prototypes for     */
/*                     CANopen Client API functions                         */
/*                                                                          */
/* USAGE: Macro        Description                                          */                         
/*                                                                          */
/*        WIN32        define for building 32-bit applications              */   
/*                     (WIN-NT 4.0 and WIN 95)                              */
/*                                                                          */
/*        __cplusplus  define for C++ applications (normally defined        */
/*                     automatically by the compiler)                       */
/*                                                                          */
/* DATE                01.04.2012                                           */
/*                                                                          */
/****************************************************************************/
#ifndef _CMADEF_H
#define _CMADEF_H

#include "BaseTypes.h"

#ifdef __cplusplus
extern "C" {
#endif //  __cplusplus


/**** defines for API applications ******************************************/

#define ID11BIT    0x00
#define ID29BIT    0x01
#define DATA_FRAME 0x00
#define RTR_FRAME  0x02

/**** structure declarations ************************************************/
#define COL_API
#define CALLING_CONV

#pragma pack(push,1)                        /* pack alignment must be one        */

/* ---- event structure for function CMA_ReadEvent ------------------------ */
typedef struct 
  {
  BYTE	ucEventType;
  USIGN32	ulTimeStamp;
  BYTE	ucaEventData[16];
  USIGN32 ulDataLenSegmented;
  } CMA_EventStruct;

/* ---- configuration structure for function CMA_AddNode ------------------ */
typedef struct 
  {
  BYTE  ucNodeID;
  USIGN16 usGuardTime;
  BYTE  ucRetryFactor;
  USIGN32  ulHeartBeatInterval;
  BYTE  ucErrAction;
  } CMA_NodeConfig;

/* ---- configuration structure for function CMA_ConfigCANChannel --------- */
typedef struct 
  {
  BYTE ucBaudRate;
  BYTE ucPresc;
  BYTE ucSjw;
  BYTE ucTSeg1;
  BYTE ucTSeg2;
  BYTE ucAccCode;
  BYTE ucAccMask;
  } CMA_CAN_CONFIG;

/* ---- configuration structure for function CMA_ConfigSyncMan ------------ */
typedef struct
  {
  BYTE ucSendMode;
  BYTE ucConsumeMode;
  USIGN32 ulSyncID;
  USIGN32 ulSyncCycle;
  } CMA_SyncManConfig;

/* ---- configuration structure for function CMA_InstallPDO_E -------------- */
typedef struct                                                   /*Ewtg V1.20*/
  {
  BYTE  ucPDOType;
  USIGN32  ulCOBID;
  BYTE  ucTransType;
  BYTE  ucAccType;
  BYTE  ucEventNotif;
  BYTE  ucaDefVal[8];
  USIGN16 usEventTimer;
  BYTE  ucNrOfByte;        //number of data byte per PDO
  BYTE  ucNoInitTrans;     //inhibit flag for initial PDO transmission
  } CMA_PDOConfig_E;                                             /*Dflt TxPdo*/

/* ---- output data structure for function CMA_GetVersion ------------------ */
typedef struct 
  {
  USIGN32  ulHWSerialNumber;    //hardware serial number
  BYTE  ucHWMainRevision;    //hardware main revision
  BYTE  ucHWSubRevision;     //hardware sub revision
  BYTE  ucHWIdentString[18]; //hardware identification string

  BYTE  ucSWMajorVersion;    //major version number of DLL & FW
  BYTE  ucSWMinorVersion;    //minor version number of DLL & FW
  BYTE  ucSWBuild;           //build number of DLL & FW
  } CMA_VersionStruct;

#ifndef COL_EXPORTS
#define MAXLENCHNAME 80
typedef struct
{
  USIGN32   u32Serial;   // Serialnumber
  USIGN32   u32DeviceType; // device type
  USIGN32   u32PhysCh; // physical channel number (1 or 2)
  USIGN32   u32FwId; // id of the loaded firmware
  BOOLEAN bIsOpen; // TRUE if opened by a process
  TCHAR ChannelName[MAXLENCHNAME];        


} CHDSNAPSHOT, *PCHDSNAPSHOT;

#endif 

#pragma pack(pop)


/**** prototypes of CANopen Client API functions ************************** */


COL_API short CALLING_CONV CMA_GetAllCANChannels(USIGN32 u32ProvidedBufferSize,
		 USIGN32 *pu32NeededBufferSize,
		 USIGN32 *pu32NumOfChannels,
		 CHDSNAPSHOT *pBuffer);


COL_API short CALLING_CONV CMA_InitializeCard(USIGN32 ulDeviceIndex,
                                         BYTE ucCANFunctionality,
                                         BYTE *ucpCANHandle,
                                         BYTE ucLicence,
                                         BYTE *pChannelName);


COL_API short CALLING_CONV CMA_SetIntEvent(BYTE ucChannelHandle,HANDLE hEvent);


COL_API short CALLING_CONV CMA_ConfigCANChannel (BYTE ucChannelHandle,
                                           CMA_CAN_CONFIG stCANConfig);

COL_API short CALLING_CONV CMA_InitSDOMan (BYTE ucChannelHandle, 
                                     USIGN16 usSDOTimeout, 
                                     USIGN16 usApplTimeoutFactor);

COL_API short CALLING_CONV CMA_InitNodeMan (BYTE ucChannelHandle, 
                                      BYTE ucNodeID);

                                                                /*Ewtg NmtSlv*/
COL_API short CALLING_CONV CMA_Startup (BYTE ucChannelHandle);

COL_API short CALLING_CONV CMA_Shutdown (BYTE ucChannelHandle);

COL_API short CALLING_CONV CMA_ConfigSyncMan (BYTE ucChannelHandle, 
                                        CMA_SyncManConfig stSyncManConfig);

COL_API short CALLING_CONV CMA_AddNode (BYTE ucChannelHandle,
						          CMA_NodeConfig stNodeConfig);

COL_API short CALLING_CONV CMA_RemoveNode (BYTE ucChannelHandle,
								     BYTE ucNodeID,
						             BYTE ucState); 

COL_API short CALLING_CONV CMA_ChangeState (BYTE ucChannelHandle,
						              BYTE ucNodeID,
									  BYTE ucState);

COL_API short CALLING_CONV CMA_RequestGuarding (BYTE ucChannelHandle,
					                      BYTE ucNodeID,
						                  BYTE ucReqGuard);

COL_API short CALLING_CONV CMA_AbortSDOTransmission (BYTE ucChannelHandle, 
                                               BYTE ucNodeID);

COL_API short CALLING_CONV CMA_InitDownload (BYTE  ucChannelHandle, 
                                       BYTE  ucNodeID,
                                       USIGN16 usIndex,
                                       BYTE  ucSubIndex,
                                       USIGN32  ulTotalDataSize,
                                       BYTE  *ucpData);

COL_API short CALLING_CONV CMA_InitUpload (BYTE  ucChannelHandle, 
                                     BYTE  ucNodeID,
                                     USIGN16 usIndex,
                                     BYTE  ucSubIndex);

COL_API short CALLING_CONV CMA_InitBlockUpload (BYTE  ucChannelHandle, 
                                     BYTE  ucNodeID,
                                     USIGN16 usIndex,
                                     BYTE  ucSubIndex,
                                     BYTE  ucBlockSize);

COL_API short CALLING_CONV CMA_InitBlockDownload (BYTE  ucChannelHandle, 
                                       BYTE  ucNodeID,
                                       USIGN16 usIndex,
                                       BYTE  ucSubIndex,
                                       USIGN32  ulTotalDataSize,
                                       BYTE  *ucpData);

COL_API short CALLING_CONV CMA_ReadData (BYTE  ucChannelHandle, 
                                                BYTE  ucNodeID,
                                                BYTE  ucSubIndex,
                                                USIGN16 usIndex,
                                                USIGN32 ulBufferSize,
                                                BYTE *pData);


COL_API short CALLING_CONV CMA_InstallPDO_E (BYTE    ucChannelHandle,
                                       CMA_PDOConfig_E  stPDOConfig, 
                                       USIGN16   *uspPDOHandle);

COL_API short CALLING_CONV CMA_RemovePDO  (BYTE  ucChannelHandle, 
                                     USIGN16 usPDOHandle);

COL_API short CALLING_CONV CMA_WritePDO  (BYTE  ucChannelHandle, 
                                    USIGN16 usPDOHandle,
                                    BYTE  ucDataOffset, 
                                    BYTE  ucDataLength, 
                                    BYTE  *ucpData);

COL_API short CALLING_CONV CMA_WritePDOBit  (BYTE  ucChannelHandle, 
                                       USIGN16 usPDOHandle,
                                       BYTE  ucBitPosition, 
                                       BYTE  ucBitValue);

COL_API short CALLING_CONV CMA_ReadPDO (BYTE  ucChannelHandle, 
                                  USIGN16 usPDOHandle,
                                  BYTE  *ucpData);

COL_API short CALLING_CONV CMA_SendRemotePDO (BYTE  ucChannelHandle, 
                                        USIGN16 usPDOHandle);

COL_API short CALLING_CONV CMA_CloseCard (BYTE ucCANHandle);

COL_API short CALLING_CONV CMA_ReadEvent (BYTE ucChannelHandle,
                                    CMA_EventStruct *stpEvent);

COL_API short CALLING_CONV CMA_SendL2Message  (BYTE  ucChannelHandle, 
                                         USIGN32  ulCanID,
                                         BYTE  ucConfig,
                                         BYTE  ucDLC,
                                         BYTE  *ucpData); 

COL_API short CALLING_CONV CMA_GetVersion (BYTE     ucChannelHandle,
                                     CMA_VersionStruct *stpVersion);

COL_API short CALLING_CONV LSS_SwitchModeGloabal(BYTE ucChannelHandle, 
                                                 BYTE ucMode);

COL_API short CALLING_CONV LSS_SwitchModeSelective(BYTE ucChannelHandle, 
                                                   USIGN32 ulVendorId,
                                                   USIGN32 ulProductCode,
                                                   USIGN32 ulRevisionNumber,
                                                   USIGN32 ulSerialNumber);

COL_API short CALLING_CONV LSS_ConfigureNodeID(BYTE ucChannelHandle, BYTE ucNodeId);

COL_API short CALLING_CONV LSS_ConfigureBitTiming(BYTE ucChannelHandle, 
                                                  BYTE ucTable,
                                                  BYTE ucIndex);

COL_API short CALLING_CONV LSS_ActivateBitTiming(BYTE ucChannelHandle, 
                                                  USIGN16 usDelay);

COL_API short CALLING_CONV LSS_StoreConfiguration(BYTE ucChannelHandle);

COL_API short CALLING_CONV LSS_InquireVendorID(BYTE ucChannelHandle, 
                                               USIGN32 *pVendorId);

COL_API short CALLING_CONV LSS_InquireProductCode(BYTE ucChannelHandle,
                                                  USIGN32 *pProductCode);

COL_API short CALLING_CONV LSS_InquireSerialNr(BYTE ucChannelHandle,
                                               USIGN32 *pSerialnumber);

COL_API short CALLING_CONV LSS_InquireRevNr(BYTE ucChannelHandle,
                                            USIGN32 *pRevisionNr);

COL_API short CALLING_CONV LSS_InquireNodeID(BYTE ucChannelHandle,
                                             BYTE *pNodeId);

COL_API short CALLING_CONV LSS_IdentityRemoteSlaves(BYTE ucChannelHandle, 
                                                   USIGN32 ulVendorId,
                                                   USIGN32 ulProductCode,
                                                   USIGN32 ulRevisionNumberLow,
                                                   USIGN32 ulRevisionNumberHigh,
                                                   USIGN32 ulSerialNumberLow,
                                                   USIGN32 ulSerialNumberHigh);

COL_API short CALLING_CONV LSS_IdentifyNonConfiguredSlaves(BYTE ucChannelHandle);

COL_API short CALLING_CONV LSS_InitLSSMan(BYTE ucChannelHandle, USIGN16 usLssTimeout);











#ifdef __cplusplus
}
#endif //  __cplusplus

#endif // _CMADEF_H
