
  /* This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of
  * the License, or (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  * MA 02111-1307, USA.
  *
  * email: support.automation@softing.com
  * Softing AG, Richard-Reitzner-Allee 6, 85540 Haar, Germany
  */


#include "BaseTypes.h"

#ifdef _DEBUG
#define COPRINT(_x_) printf _x_
#else 
#define COPRINT(_x_) 
#endif

#define COL_API

// OS specific defines
#define WAIT_FOR_EVENT(event,timeout) 0
#define SET_EVENT(a) pthread_mutex_unlock(&a)
#define SLEEP(ms) usleep(ms*1000)
#define WAIT_FOR_EVENTS(a,b,c,d)
#define CREATE_SEMAPHORE(a,b,c,d) CreateLinuxMutex(a)
#define CloseHandle(a)

// little/big endian
#define WRT16(a,b) {(a)[1] = b/256; (a)[0] = b & 0xFF;}


#define WRT32(a,b) { \
(a)[3] = (USIGN8)  b/0x01000000; \
(a)[2] = (USIGN8) (b/0x00010000) & 0xFF; \
(a)[1] = (USIGN8) (b/0x00000100) & 0xFF; \
(a)[0] = (USIGN8) b & 0xFF; \
}

#define Rd16(a,b) *(a)=(b)[0]+(b)[1]*0x100

#define Rd32(a,b) *(a)=(b)[0]+(b)[1]*0x100+(b)[2]*0x10000+(b)[3]*0x1000000



#define VENDOR_ID       0x1000
#define PRODUCT_CODE    0x3000
#define REV_NR          0x0100

#define UNDEF           0x57F

 
#define MAX_BAUD_RATE 8

// to do error handling must be introduced
#define MAP_ERROR(a) map_error(a)

#define MAX_CHANNEL         4
#define MAX_NODE            127
#define MAX_XMT_TRIALS      3

#define MAX_MALLOCSIZE      128000
#define MAX_PDO             1024

#define FIFOITEMS           256
#define FIFOMASK            (FIFOITEMS-1)

// NMT starup
#define ISMASTER            1
#define NOOPERATIONAL       4
#define APPSTARTNODE        16


#define ST_STOPP            4
#define ST_START            5
#define ST_RESET_NODE       6
#define ST_RESET_COM        7
#define ST_PRE_OPERATIONAL  127

#define INT_EVENT 0
#define END_EVENT 1


#define NMT            0x000
#define SYNC_EMRC      0x080
#define TIME_STAMP     0x100

#define PDO1_RX        0x180
#define PDO1_TX        0x200
#define PDO2_RX        0x280
#define PDO2_TX        0x300
#define PDO3_RX        0x380
#define PDO3_TX        0x400
#define PDO4_RX        0x480
#define PDO4_TX        0x500

#define RX_PDO         0
#define TX_PDO         1

#define SDO_TX         0x580
#define SDO_RX         0x600

#define EV_EMCY                0x0B;
#define EV_STARTUP             0x22;
#define EV_ERROR               0x0F
#define EV_PDO                 0x20
#define EV_UPLOAD_COMPLETE     0x05
#define EV_DOWNLOAD_COMPLETE   0x03
#define EV_SHUTDOWN            0x21
#define EV_BOOTUP              0x0C

#define ERROR_HANDLING 0x700

// 2 Bits reserved to prevent overun on calculation
#define TIMEHACK 0x3FFFFFFF
#define MK_TIME(a) (a & TIMEHACK)


/* ---- Event Type Defines ------------------------------------------------- */
#define  MAX_DAT                20  

#define  NO_EVENT                0 //=0x00
#define  DOWNLOAD_COMPLETED      3 //=0x03
#define  UPLOAD_COMPLETED        5 //=0x05
#define  EMCY_RECEIVED          11 //=0x0B
#define  ERROR_EVENT            15 //=0x0F
#define  NEXT_DOWNLOAD_BUNDLE   30 //=0x1E
#define  NEXT_UPLOAD_BUNDLE     31 //=0x1F
#define  PDO_RECEIVED           32 //=0x20
#define  SHUTDOWN_COMPLETED     33 //=0x21
#define  STARTUP_COMPLETED      34 //=0x22
#define  SYNC_OBJECT_SENT       35 //=0x23                        /*Ewtg 1.20*/


#define SW_MAJ_VERSION 1
#define SW_MIN_VERSION 1
#define SW_BUILD 1

#define HW_STRING_PCIDN "CAN-ACx-PCI/DN"
#define HW_STRING_PCI   "CAN-ACx-PCI"
#define HW_STRING_CC2   "CANcard2"
#define HW_STRING_ED2   "EDICcard2"
#define HW_STRING_104   "CAN-ACx-104"
#define HW_STRING_USB   "CANusb"
#define HW_STRING_PCIE   "CAN-PROx-PCIe"
#define HW_STRING_104P   "CAN-PROx-104+"
#define HW_STRING_IP    "FG-100/CAN"


// SDO Transfer 
#define ERR_API                1
#define ERR_CAN                2
#define ERR_SDO                3
#define ERR_GUARD              4
#define ERR_LSS                5

#define CS_ABORT               0x80

// download
#define TX_SDO_BASE            0x580
#define RX_SDO_BASE            0x600

#define NUMOFBYTES(a)          ((4-a)<<2)
#define SEGMENTED              0x01
#define EXPEDITED              0x03
#define UNSPECIFIED            0x02

#define LAST_SEGMENT           0x01
#define TOGGLE_BIT             0x10


// response
#define SCS_INIT_DOWNLOAD       0x60
#define SEGMENT_RES             0x40

// upload
#define CS_INIT_UPLOAD          0x40
#define CS_INIT_DOWNLOAD        0x20 
#define CS_NEXT_SEGMENT         0x60

#define CS_INIT_BLOCKUPLOAD     0xA0
#define SCS_INIT_BLOCKUPLOAD    0xC0
#define CS_INIT_BLOCKDOWNLOAD   0xC0
#define SCS_INIT_BLOCKDOWNLOAD  0xA0
#define SCS_END_BLOCKUPLOAD     0xC0
#define CCS_END_BLOCKUPLOAD     0xA1

// crc support=4, no crc support = 0
#define SUPPORT_CRC_CODE       0x04
#define SIZE_VALID             0x02

#define TOGGLE_BIT_UPLOAD      0x10
#define NO_FALLBACK            0x00

// LSS services

#define LSS_SWITCH_GLOBAL_MODE      04
#define LSS_SWITCH_MODE_VID         64
#define LSS_SWITCH_MODE_PC          65
#define LSS_SWITCH_MODE_RN          66
#define LSS_SWITCH_MODE_SN          67
#define LSS_CONFIGURE_NODE_ID       17
#define LSS_CONFIGURE_BIT_TIMING    19
#define LSS_ACTIVATE_BIT_TIMING     21
#define LSS_STORE_CONFIGURATION     23
#define LSS_INQ_ID_VID              90
#define LSS_INQ_ID_PC               91
#define LSS_INQ_ID_RN               92
#define LSS_INQ_ID_SN               93
#define LSS_INQ_ID_NODE_ID          94
#define LSS_IDENTITY_VID            70
#define LSS_IDENTITY_PC             71
#define LSS_IDENTITY_RNL            72
#define LSS_IDENTITY_RNH            73
#define LSS_IDENTITY_SNL            74
#define LSS_IDENTITY_SNH            75
#define LSS_IDENTIFY_NONCFGSLVS     76

#define LSS_RES_IDENTITY            79
#define LSS_RES_IDENTITY_NONCFGSLVS 80





#define LSS_NO_SERVICE         0


// LSS responses

#define LSS_RES_SWITCH_MODE    68

#define LSS_MASTER_ID          2021
#define LSS_RESPONSE           2020

typedef struct
{
  USIGN32 id[5];
} IDENTIFY;

typedef struct
{
  USIGN32 u32DevType_1000;
  USIGN8  u8ErrorRegister_1001;
  USIGN32 u32SyncCobId_1005;
  USIGN32 u32ComCyclePeriode_1006;
  USIGN8  DeviceName_1008[80];
  USIGN8  HwVersion_1009[80];
  USIGN8  SwVersion_100A[80];
  USIGN16 u16GuardTime_100C;
  IDENTIFY Identify_1018;
  USIGN32 NMTStartup_1F80;
}CO_OD;

typedef enum
{
    BLK_TRANSFER_INIT = 0,
    BLK_TRANSFER_DATA,
    BLK_TRANSFER_BLK_CONFIRM,
    BLK_TRANSFER_END
    
} BLOCKTRANSFER_STATE;


typedef enum
{
  TOGGLE_ERR = 0,
  TIMEOUT_ERR,
  CLIENT_SERVER_ERR,
  BLOCK_SIZE_ERR,
  SEQ_NUMBER_ERR,
  CRC_CODE_ERR,
  OUT_OF_MEMORY_ERR,
  OBJ_ACC_NOT_SUP_ERR,
  WRITEONLY_ERR,
  READONLY_ERR,
  NO_OBJECT_ERR,
  MAPPING_ERR,
  PDO_LEN_ERR,
  GENERAL_PARA_ERR,
  INCOMPATIBLE_ERR,
  HARDWARE_ERR,
  LENGTH_ERR,
  LEN_TOO_HIGH_ERR,
  LEN_TOO_LOW_ERR,
  SUBINDEX_ERR,
  VALUE_RANGE_ERR,
  VAL_TOO_HIGH_ERR,
  VAL_TOO_LOW_ERR,
  MINMAX_ERR,
  GENERAL_ERR,
  DATA_ACCESS_ERR,
  DEVICE_STATE_ERR,
  NO_OD_ERR

} SDO_ERROR;

typedef enum
{
  INITIALISATION = 0,
  RESET,
  STOPPED = 4,
  OPERATIONAL,
  PREOPERATIONAL = 127

} CO_STATE;

typedef struct
{
  USIGN8  data[8];
  USIGN8  data252[8];
  USIGN32 u16CobId;
  BOOLEAN isUsed;
  BOOLEAN bEvent;
  BOOLEAN bSndReq;
  BOOLEAN bSendAtNextSync;
  USIGN8  u8Type; // rx or tx
  USIGN8  u8DataLen;
  USIGN8  u8XmtType;
  USIGN8  u8NumOfSyncs;

} PDO, *PPDO;

typedef struct
{
  PPDO   pPdo;
  PVOID  pNext;

} PDOLIST, *PPDOLIST;

typedef struct
{
  USIGN32 u32BufferPtr;
  USIGN32 u32Size;
  USIGN8  *pBuffer;
  BOOLEAN isAllocated;

} CO_BUFFER, *PCO_BUFFER;

typedef enum
{
  DOWNLOAD,
  UPLOAD,
  BLOCKDOWNLOAD,
  BLOCKUPLOAD
} SDO_DIRECTION;

typedef struct
{
  USIGN16             u16CobId;
  USIGN8              u8NodeId;
  USIGN8              u8OldDataByte0;
  USIGN32             u32LastEvent;
  USIGN32             u32BlockStart;
  CO_BUFFER           DataBuffer;
  SDO_DIRECTION       Direction;
  CAN_BOOL            isInit;
  CAN_BOOL            isUsed;
  CAN_BOOL            isMasterInvolved;
  CAN_BOOL            isWaiting;
  CAN_BOOL            isLocked;
  CAN_BOOL            isCrcProvided;
  USIGN8              u8SubIndex;
  USIGN8              u8BlockSeqNum;
  USIGN8              u8BlockSize;
  USIGN16             u16CrcCode;
  USIGN16             u16Index;
  BLOCKTRANSFER_STATE SdoBlockState;


} SDO, *PSDO;

typedef struct
{
  USIGN8   u8NodeId;
  USIGN8   u8RetryFactor;
  USIGN8   u8Retries;
  USIGN8   u8ToggleBit;
  USIGN8   u8ErrorAction;
  CAN_BOOL isFirstGuardingReq;
  USIGN16  u16GuardTime;
  USIGN32  u32GuardReqTime;
  USIGN32  u32GuardResTime;
  USIGN32  u32LastHeartbeat;
  USIGN32  u32HeartbeatInterval;
  USIGN8   u8GuardingState;
  CAN_BOOL isDefined;
  CAN_BOOL isHeartbeatChecked;
  CO_STATE state;
  SDO      Sdo;
  
} CO_NODE, *PCO_NODE;

typedef struct
{
  USIGN8 u8NodeId;
  CAN_BOOL bRemove;
  PVOID  pNext;
} GUARDLIST, *PGUARDLIST;


#pragma pack(push, 1)
typedef struct 
  {
  USIGN8	ucEventType;
  USIGN32	ulTimeStamp;
  USIGN8	ucaEventData[20];
  } CO_EVENTSTRUCT, *PCO_EVENTSTRUCT;

typedef struct 
  {
  USIGN8	ucEventType;
  USIGN32	ulTimeStamp;
  USIGN8	ucaEventData[16];
  USIGN32 ulDataLenSegmented;
  } CO_EVENTSTRUCT_E;


#pragma pack(pop)

typedef struct
{
  CO_EVENTSTRUCT cStr;
  PVOID  pNext;
} EV_QUEUE, *PEV_QUEUE;

typedef struct
{

  pthread_mutex_t hLSSEvent;
  BOOLEAN    isInitialized;
  USIGN8     u8LSSPendingService;
  USIGN8     u8NodeId;
  USIGN32    u32Vid;
  USIGN32    u32Pc;
  USIGN32    u32Rn;
  USIGN32    u32Sn;
  USIGN32    u32Timeout;
} LSS, *PLSS;

typedef struct
{
  CO_EVENTSTRUCT event[256];
  USIGN16 rd;
  USIGN16 wr;

} USEREVENTS;

typedef enum 
{
  RUNNING = 0,
  STOP_REQ,
  STOP_ACK
  
} THREAD_STATE;

typedef struct
{
  BOOLEAN         isDefined;
  pthread_mutex_t ChSemaphore;
  CAN_HANDLE      channel;
  BOOLEAN         isXtd;
  BOOLEAN         isSyncConsumer;
  CO_STATE        state;
  USIGN32         MasterNodeId;
  CO_NODE         node[MAX_NODE+1];
  PGUARDLIST       pGuardList;

  pthread_t        hRcvThread;
  THREAD_STATE     bStopRcvThread;


  HANDLE          hStopTimerThread;
  pthread_t       hTimerThread;
  THREAD_STATE    bStopTimerThread;

  USIGN32         u32SdoTimeout;
  USIGN32         u32ApplTimeout;

  USIGN32         u32LastSyncTime;
  

  PEV_QUEUE       pcEvQueue;
  pthread_mutex_t hQueueEvent;
  pthread_cond_t  QueueCondition;
  int             conditionMet;
  
  INT16           s16WaitRet;

  PDO             pdo[1024];

  PPDOLIST        pPdoList;
  void            *pObjects[0x3FFF];
 // USIGN32    u32ObSize[0x3FFF];
  CO_OD           ObjDict;
  LSS             Lss;
  USIGN16         u16CobInt2Ext[0x800];
  USIGN16         u16CobExt2Int[0x800];
  USEREVENTS      UserEvents;
  BOOLEAN         bSendPdo;
  

} CHANNELS, *PCHANNELS;


 USIGN16 calcCRC16r (USIGN8 *blk_adr,USIGN32 blk_len);
