/****************************************************************************/
/*                                                                          */
/*   Softing GmbH          Richard-Reitzner-Allee 6       85540 Haar        */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*   Copyright (C) by Softing GmbH, 1997,1998,1999   All rights reserved.   */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*                               C M A D E F . H                            */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* PROJECT:            CANopen Master API                                   */
/*                                                                          */
/* MODULE:             CMADEF.H                                             */
/*                                                                          */
/* AUTHOR:             Softing GmbH                                         */
/*                                                                          */
/* MODULE_DESCRIPTION: macro definitions, struktures and prototypes for     */
/*                     CANopen Master API functions                         */
/*                                                                          */
/* USAGE: Macro        Description                                          */                         
/*                                                                          */
/*        WIN32        define for building 32-bit applications              */   
/*                     (WIN-NT 4.0 and WIN 95)                              */
/*                                                                          */
/*        __cplusplus  define for C++ applications (normally defined        */
/*                     automatically by the compiler)                       */
/*                                                                          */
/* DATE                25.08.1999                                           */
/*                                                                          */
/****************************************************************************/

  /* This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of
  * the License, or (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  * MA 02111-1307, USA.
  *
  * email: support.automation@softing.com
  * Softing AG, Richard-Reitzner-Allee 6, 85540 Haar, Germany
  */

#ifndef _CMADEF_H
#define _CMADEF_H


#ifdef __cplusplus
extern "C" {
#endif //  __cplusplus


/**** defines for API applications ******************************************/

#define ID11BIT    0x00
#define ID29BIT    0x01
#define DATA_FRAME 0x00
#define RTR_FRAME  0x02

/**** structure declarations ************************************************/



#ifdef WIN32
#define CALLING_CONV _stdcall 
#ifdef COL_EXPORTS
#define COL_API __declspec(dllexport)
#else
#define COL_API __declspec(dllimport)
#endif
#else
#define COL_API
#define CALLING_CONV
#define WINAPI
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE !FALSE
#endif


#pragma pack(push,1)                        /* pack alignment must be one        */

/* ---- event structure for function CMA_ReadEvent ------------------------ */
typedef struct 
  {
  USIGN8	ucEventType;
  USIGN32	ulTimeStamp;
  USIGN8	ucaEventData[16];
  USIGN32 ulDataLenSegmented;
  } CMA_EventStruct;

/* ---- configuration structure for function CMA_AddNode ------------------ */
typedef struct 
  {
  USIGN8  ucNodeID;
  USIGN16 usGuardTime;
  USIGN8  ucRetryFactor;
  USIGN32  ulHeartBeatInterval;
  USIGN8  ucErrAction;
  } CMA_NodeConfig;

/* ---- configuration structure for function CMA_ConfigCANChannel --------- */
typedef struct 
  {
  USIGN8 ucBaudRate;
  USIGN8 ucPresc;
  USIGN8 ucSjw;
  USIGN8 ucTSeg1;
  USIGN8 ucTSeg2;
  USIGN8 ucAccCode;
  USIGN8 ucAccMask;
  } CMA_CAN_CONFIG;

/* ---- configuration structure for function CMA_ConfigSyncMan ------------ */
typedef struct
  {
  USIGN8 ucSendMode;
  USIGN8 ucConsumeMode;
  USIGN32 ulSyncID;
  USIGN32 ulSyncCycle;
  } CMA_SyncManConfig;

/* ---- SDO segment structure for function CMA_ReadNextSegment ------------ */
typedef struct
  {
  USIGN8  ucNodeID;
  USIGN16 usIndex;
  USIGN8  ucSubIndex;
  USIGN32  ulTotalDataSize;
  USIGN32  ulSegNum;
  USIGN8  ucValidDataBytes;
  USIGN8  ucaData[7];
  } CMA_SDOSeg;

/* ---- configuration structure for function CMA_InstallPDO --------------- */
typedef struct 
  {
  USIGN8  ucPDOType;
  USIGN32  ulCOBID;
  USIGN8  ucTransType;
  USIGN8  ucAccType;
  USIGN8  ucEventNotif;
  USIGN8  ucaDefVal[8];
  USIGN16 usEventTimer;
  } CMA_PDOConfig;

/* ---- configuration structure for function CMA_InstallPDO_E -------------- */
typedef struct                                                   /*Ewtg V1.20*/
  {
  USIGN8  ucPDOType;
  USIGN32  ulCOBID;
  USIGN8  ucTransType;
  USIGN8  ucAccType;
  USIGN8  ucEventNotif;
  USIGN8  ucaDefVal[8];
  USIGN16 usEventTimer;
  USIGN8  ucNrOfByte;        //number of data byte per PDO
  USIGN8  ucNoInitTrans;     //inhibit flag for initial PDO transmission
  } CMA_PDOConfig_E;                                             /*Dflt TxPdo*/

/* ---- output data structure for function CMA_GetVersion ------------------ */
typedef struct 
  {
  USIGN32  ulHWSerialNumber;    //hardware serial number
  USIGN8  ucHWMainRevision;    //hardware main revision
  USIGN8  ucHWSubRevision;     //hardware sub revision
  USIGN8  ucHWIdentString[18]; //hardware identification string

  USIGN8  ucSWMajorVersion;    //major version number of DLL & FW
  USIGN8  ucSWMinorVersion;    //minor version number of DLL & FW
  USIGN8  ucSWBuild;           //build number of DLL & FW
  } CMA_VersionStruct;

#pragma pack(pop)


/**** prototypes of CANopen Master API functions ************************** */

COL_API short CALLING_CONV CMA_InitializeCard(USIGN32 ulDeviceIndex,
                                         USIGN8 ucCAN1Functionality,
                                      //   USIGN8 ucCAN2Functionality,
                                         USIGN8 *ucpCAN1Handle,
                                      //   USIGN8 *ucpCAN2Handle,
                                         USIGN8 ucLicence,
                                         USIGN8 *pChannelName);


COL_API short CALLING_CONV CMA_SetIntEvent(USIGN8 ucChannelHandle,EVT_HANDLE hEvent);


COL_API short CALLING_CONV CMA_ConfigCANChannel (USIGN8 ucChannelHandle,
                                           CMA_CAN_CONFIG stCANConfig);

COL_API short CALLING_CONV CMA_InitSDOMan (USIGN8 ucChannelHandle, 
                                     USIGN16 usSDOTimeout, 
                                     USIGN16 usApplTimeoutFactor);

COL_API short CALLING_CONV CMA_InitNodeMan (USIGN8 ucChannelHandle, 
                                      USIGN8 ucNodeID);

                                                                /*Ewtg NmtSlv*/
#ifdef NMT_SLV
COL_API short CALLING_CONV CMA_InitNodeMan_E (USIGN8 ucChannelHandle, 
                                        USIGN8 ucNodeID,
                                        USIGN8 ucNmtState,
                                        USIGN8 ucRestartMode);
#endif //NMT_SLV

COL_API short CALLING_CONV CMA_Startup (USIGN8 ucChannelHandle);

COL_API short CALLING_CONV CMA_Shutdown (USIGN8 ucChannelHandle);

COL_API short CALLING_CONV CMA_ConfigSyncMan (USIGN8 ucChannelHandle, 
                                        CMA_SyncManConfig stSyncManConfig);

COL_API short CALLING_CONV CMA_SendSyncObject (USIGN8 ucChannelHandle, 
                                         CMA_SyncManConfig stSyncManConfig);

COL_API short CALLING_CONV CMA_AddNode (USIGN8 ucChannelHandle,
						          CMA_NodeConfig stNodeConfig);

COL_API short CALLING_CONV CMA_RemoveNode (USIGN8 ucChannelHandle,
								     USIGN8 ucNodeID,
						             USIGN8 ucState); 

COL_API short CALLING_CONV CMA_ChangeState (USIGN8 ucChannelHandle,
						              USIGN8 ucNodeID,
									  USIGN8 ucState);

COL_API short CALLING_CONV CMA_RequestGuarding (USIGN8 ucChannelHandle,
					                      USIGN8 ucNodeID,
						                  USIGN8 ucReqGuard);

COL_API short CALLING_CONV CMA_AbortSDOTransmission (USIGN8 ucChannelHandle, 
                                               USIGN8 ucNodeID);

COL_API short CALLING_CONV CMA_InitDownload (USIGN8  ucChannelHandle, 
                                       USIGN8  ucNodeID,
                                       USIGN16 usIndex,
                                       USIGN8  ucSubIndex,
                                       USIGN32  ulTotalDataSize,
                                       USIGN8  *ucpData);

COL_API short CALLING_CONV CMA_WriteNextSegment (USIGN8  ucChannelHandle, 
                                           USIGN8  ucNodeID,
                                           USIGN16 usIndex,
                                           USIGN8  ucSubIndex,
                                           USIGN8  ucDataSize,
                                           USIGN8  *ucpData);

COL_API short CALLING_CONV CMA_InitUpload (USIGN8  ucChannelHandle, 
                                     USIGN8  ucNodeID,
                                     USIGN16 usIndex,
                                     USIGN8  ucSubIndex);

COL_API short CALLING_CONV CMA_InitBlockUpload (USIGN8  ucChannelHandle, 
                                     USIGN8  ucNodeID,
                                     USIGN16 usIndex,
                                     USIGN8  ucSubIndex,
                                     USIGN8  ucBlockSize);

COL_API short CALLING_CONV CMA_InitBlockDownload (USIGN8  ucChannelHandle, 
                                       USIGN8  ucNodeID,
                                       USIGN16 usIndex,
                                       USIGN8  ucSubIndex,
                                       USIGN32  ulTotalDataSize,
                                       USIGN8  *ucpData);


COL_API short CALLING_CONV CMA_ReadNextSegment (USIGN8  ucChannelHandle, 
                                          CMA_SDOSeg     *stpSDOSeg);

COL_API short CALLING_CONV CMA_ReadData (USIGN8  ucChannelHandle, 
                                                USIGN8  ucNodeID,
                                                USIGN8  ucSubIndex,
                                                USIGN16 usIndex,
                                                USIGN32 ulBufferSize,
                                                USIGN8 *pData);


COL_API short CALLING_CONV CMA_InstallPDO (USIGN8  ucChannelHandle,
                                     CMA_PDOConfig  stPDOConfig, 
                                     USIGN16 *uspPDOHandle);

COL_API short CALLING_CONV CMA_InstallPDO_E (USIGN8    ucChannelHandle,
                                       CMA_PDOConfig_E  stPDOConfig, 
                                       USIGN16   *uspPDOHandle);

COL_API short CALLING_CONV CMA_RemovePDO  (USIGN8  ucChannelHandle, 
                                     USIGN16 usPDOHandle);

COL_API short CALLING_CONV CMA_WritePDO  (USIGN8  ucChannelHandle, 
                                    USIGN16 usPDOHandle,
                                    USIGN8  ucDataOffset, 
                                    USIGN8  ucDataLength, 
                                    USIGN8  *ucpData);

COL_API short CALLING_CONV CMA_WritePDOBit  (USIGN8  ucChannelHandle, 
                                       USIGN16 usPDOHandle,
                                       USIGN8  ucBitPosition, 
                                       USIGN8  ucBitValue);

COL_API short CALLING_CONV CMA_ReadPDO (USIGN8  ucChannelHandle, 
                                  USIGN16 usPDOHandle,
                                  USIGN8  *ucpData);

COL_API short CALLING_CONV CMA_SendRemotePDO (USIGN8  ucChannelHandle, 
                                        USIGN16 usPDOHandle);

COL_API short CALLING_CONV CMA_CloseCard (USIGN8 ucCAN1Handle /*,
                                    USIGN8 ucCAN2Handle */);

COL_API short CALLING_CONV CMA_ReadEvent (USIGN8 ucChannelHandle,
                                    CMA_EventStruct *stpEvent);

COL_API short CALLING_CONV CMA_SendL2Message  (USIGN8  ucChannelHandle, 
                                         USIGN32  ulCanID,
                                         USIGN8  ucConfig,
                                         USIGN8  ucDLC,
                                         USIGN8  *ucpData); 

COL_API short CALLING_CONV CMA_GetVersion (USIGN8     ucChannelHandle,
                                     CMA_VersionStruct *stpVersion);

COL_API short CALLING_CONV LSS_SwitchModeGlobal(USIGN8 ucChannelHandle, 
                                                 USIGN8 ucMode);

COL_API short CALLING_CONV LSS_SwitchModeSelective(USIGN8 ucChannelHandle, 
                                                   USIGN32 ulVendorId,
                                                   USIGN32 ulProductCode,
                                                   USIGN32 ulRevisionNumber,
                                                   USIGN32 ulSerialNumber);

COL_API short CALLING_CONV LSS_ConfigureNodeID(USIGN8 ucChannelHandle, USIGN8 ucNodeId);

COL_API short CALLING_CONV LSS_ConfigureBitTiming(USIGN8 ucChannelHandle, 
                                                  USIGN8 ucTable,
                                                  USIGN8 ucIndex);

COL_API short CALLING_CONV LSS_ActivateBitTiming(USIGN8 ucChannelHandle, 
                                                  USIGN16 usDelay);

COL_API short CALLING_CONV LSS_StoreConfiguration(USIGN8 ucChannelHandle);

COL_API short CALLING_CONV LSS_InquireVendorID(USIGN8 ucChannelHandle, 
                                               USIGN32 *pVendorId);

COL_API short CALLING_CONV LSS_InquireProductCode(USIGN8 ucChannelHandle,
                                                  USIGN32 *pProductCode);

COL_API short CALLING_CONV LSS_InquireSerialNr(USIGN8 ucChannelHandle,
                                               USIGN32 *pSerialnumber);

COL_API short CALLING_CONV LSS_InquireRevNr(USIGN8 ucChannelHandle,
                                            USIGN32 *pRevisionNr);

COL_API short CALLING_CONV LSS_InquireNodeID(USIGN8 ucChannelHandle,
                                             USIGN8 *pNodeId);

COL_API short CALLING_CONV LSS_IdentityRemoteSlaves(USIGN8 ucChannelHandle, 
                                                   USIGN32 ulVendorId,
                                                   USIGN32 ulProductCode,
                                                   USIGN32 ulRevisionNumberLow,
                                                   USIGN32 ulRevisionNumberHigh,
                                                   USIGN32 ulSerialNumberLow,
                                                   USIGN32 ulSerialNumberHigh);

COL_API short CALLING_CONV LSS_IdentifyNonConfiguredSlaves(USIGN8 ucChannelHandle);

COL_API short CALLING_CONV LSS_InitLSSMan(USIGN8 ucChannelHandle, USIGN16 usLssTimeout);


COL_API short CALLING_CONV CMA_WaitForEvent(USIGN8 ucChannelHandle);


USIGN8 getPdoType(USIGN16 u16CobId);





#ifdef __cplusplus
}
#endif //  __cplusplus

#endif // _CMADEF_H
