/* PROJECT_NAME         CAN_Test                                   
   MODULE               CAN_TEST.C                                      
   COMPANY              SOFTING AG Muenchen                                
   VERSION              $Id: Can_test.c,v 1.2 2006/09/19 09:15:41 kno Exp $
   FUNCTIONS            Main program routine
                        Customer inputs
                        Error evaluation routine

                        This is a test- and demo program to make the first
                        steps with the CAN API V4.05.                           
                        It does initialization, offers changing of various
                        modes by command line parameters, and comes to a  
                        quasi user interface loop:
                        - with hotkeys library functions can be called
                        - in the background CANPC_read fetches messages from the board,     
                          interprets and prints them either by polling or interrupt. 
                          This job may also be disabled by hotkey. 
                        The source code is written to be usable independant of the
                        applied interface type.
                        
                        Good luck running it !        

***************************************************************/

#if defined (WIN32)
#include <windows.h>
#endif
#include <stdlib.h>
#include <string.h>
#ifndef linux
#include <conio.h>
#include  <io.h>
#endif
#include  <stdio.h>

/* CAN LAYER 2 LIBRARY INCLUDE FILES */
#include "BaseTypes.h"
#include "can_def.h"

#include "canlay2.h"

#ifdef LINUX
#include "errno.h"
#include "kbhit.h"
#define CAN_DEVICE_NAME "/dev/can_00"
#endif

/*--------------------------------------------------------------------------*/
/*  **** defines                                                            */
/*--------------------------------------------------------------------------*/
// Bit timing (1MBaud)
#define PRESCALER_1             1
#define SYNC_JMP_WIDTH_1        3
#define TIME_SEG1_1             4
#define TIME_SEG2_1             3
#define SAMPLE_1                0

#define PRESCALER_2             1
#define SYNC_JMP_WIDTH_2        3
#define TIME_SEG1_2             4
#define TIME_SEG2_2             3
#define SAMPLE_2                0


// Acceptance filter
#define ACCEPT_MASK_1           0x0000
#define ACCEPT_CODE_1           0x0000
#define ACCEPT_MASK_XTD_1       0x00000000L
#define ACCEPT_CODE_XTD_1       0x00000000L

#define ACCEPT_MASK_2           0x0000
#define ACCEPT_CODE_2           0x0000
#define ACCEPT_MASK_XTD_2       0x00000000L
#define ACCEPT_CODE_XTD_2       0x00000000L


// Resynchronization
#define SLEEPMODE_1             0
#define SPEEDMODE_1             0

#define SLEEPMODE_2             0
#define SPEEDMODE_2             0


// Phys. layer (-1: default - CAN High Speed)
#define OUTPUT_CONTROL_1        -1
#define OUTPUT_CONTROL_2        -1

// Linked operational mode specific functions
// FIFO mode
INT32 InitializeFIFO(void);
INT32 UserRequestFIFO(unsigned char custom);
INT32 ReadBusEventFIFO(void);
// Dyn. obj. buffer mode
INT32 InitializeDynObjBuf(void);
INT32 UserRequestDynObjBuf(unsigned char custom);
INT32 ReadBusEventDynObjBuf(void);
// Stat. obj. buffer mode
// Do not used with CAN-AC2/527!!!!!
INT32 InitializeStatObjBuf(void);
INT32 ReadBusEventStatObjBuf(void);
INT32 UserRequestStatObjBuf(char custom);

// Linked interrupt handling functions
#ifdef LINUX
INT32 init_pthread(void);
void exit_pthread(void);
#elif defined(WIN32)
INT32 InitInterruptHandling32(void);
void ExitInterruptHandling32(void);
#else
#ifdef DOS
void disable_interrupt(void);
void enable_interrupt(void);
#endif
INT32 InitInterruptHandling16(void);
void ExitInterruptHandling16(void);
#endif


// Globals
static char op_mode;            // Customers choice operational mode
static INT32 can_chip_type[2];

/***********************************************************
  printInitErrorText
  Routine to evaluate error codes of CANPC_initialize_board
************************************************************/
void
printInitErrorText(INT32 iErrorCode)
{
    printf("ErrorCode=%x=%d  Error Text:", iErrorCode, iErrorCode);
    switch ((USIGN32) iErrorCode)
    {
    case INIPC_IB_PNP_NO_DEVICE_FOUND:
        printf("no can device found ");
        break;
    case INIPC_IB_ERR_VC_INTERNALERROR:
        printf("internal error ");
        break;
    case INIPC_IB_ERR_VC_GENERALERROR:
        printf("general error ");
        break;
    case INIPC_IB_ERR_VC_TIMEOUT:
        printf("Timeout ");
        break;
    case INIPC_IB_ERR_VC_IOPENDING:
        printf("driver call pending ");
        break;
    case INIPC_IB_ERR_VC_IOCANCELLED:
        printf("driver call cancelled ");
        break;
    case INIPC_IB_ERR_VC_ILLEGALCALL:
        printf("illegal driver call ");
        break;
    case INIPC_IB_ERR_VC_NOTSUPPORTED:
        printf("driver call not supported ");
        break;
    case INIPC_IB_ERR_VC_VERSIONERROR:
        printf("wrong driver-dll version ");
        break;
    case INIPC_IB_ERR_VC_DRIVERVERSIONERROR:
        printf("wrong driver version ");
        break;
    case INIPC_IB_ERR_VC_DRIVERNOTFOUND:
        printf("driver not found ");
        break;
    case INIPC_IB_ERR_VC_NOTENOUGHMEMORY:
        printf("not enough memory ");
        break;
    case INIPC_IB_ERR_VC_TOOMANYDEVICES:
        printf("too many devices ");
        break;
    case INIPC_IB_ERR_VC_UNKNOWNDEVICE:
        printf("unknown device ");
        break;
    case INIPC_IB_ERR_VC_DEVICEALREADYEXISTS:
        printf("Device ardy exists ");
        break;
    case INIPC_IB_ERR_VC_DEVICEACCESSERROR:
        printf("device ardy open ");
        break;
    case INIPC_IB_ERR_VC_RESOURCEALREADYREGISTERED:
        printf("Resource in use");
        break;
    case INIPC_IB_ERR_VC_RESOURCECONFLICT:
        printf("Resource-conflict ");
        break;
    case INIPC_IB_ERR_VC_RESOURCEACCESSERROR:
        printf("Resource access error");
        break;
    case INIPC_IB_ERR_VC_PHYSMEMORYOVERRUN:
        printf("invalid phys.mem-access");
        break;
    case INIPC_IB_ERR_VC_TOOMANYPORTS:
        printf("too many I/O ports ");
        break;
    case INIPC_IB_ERR_VC_UNKNOWNRESOURCE:
        printf("unknown resource ");
        break;
#ifdef LINUX
    case CANPC_ERR_OPEN:
        printf("open devicefile returned: %s", strerror(errno));
        break;
    case CANPC_ERR_IOCTL:
        printf("ioctl on devicefile returned: %s", strerror(errno));
        break;
    case CANPC_ERR_CLOSE:
        printf("close on devicefile returned: %s", strerror(errno));
        break;
#endif
    default:
        printf("unknown return value !");
        break;
    }
    printf("\n");
}



/***********************************************************
  printResetErrorText
  Routine to evaluate error codes of CANPC_reset_board
************************************************************/
void
printResetErrorText(INT32 iErrorCode)
{
    printf("ErrorCode=%d  Error Text:", iErrorCode);
    switch (iErrorCode)
    {
    case CANPC_RB_INI_FILE:
        printf(" can't open IniFile       ");
    case CANPC_RB_ERR_FMT_INI:
        printf(" format error in INI-file ");
    case CANPC_RB_ERR_OP_BIN:
        printf(" error opening binary-file");
    case CANPC_RB_ERR_RD_BIN:
        printf(" error reading binary-file");
    case CANPC_RB_BIN_TOO_LONG:
        printf(" binary-file too long     ");

    case CANPC_RB_ERR_BIN_FMT:
        printf(" binary-data format error ");
        break;
    case CANPC_RB_ERR_BIN_CS:
        printf(" binary-data checksum error      ");
        break;
    case CANPC_RB_NO_CARD:
        printf(" no card present          ");
        break;
    case CANPC_RB_NO_PHYS_MEM:
        printf(" no physical memory       ");
        break;
    case CANPC_RB_INVLD_IRQ:
        printf(" invalid IRQ-number       ");
        break;
    case CANPC_RB_ERR_DPRAM_ACCESS:
        printf(" error accessing dpram    ");
        break;
    case CANPC_RB_ERR_CRD_RESP:
        printf(" bad response from card   ");
        break;
    case CANPC_RB_ERR_SRAM:
        printf(" sram seems to be damaged ");
        break;
    case CANPC_RB_ERR_PRG:
        printf(" invalid program start address   ");
        break;
    case CANPC_RB_ERR_REC:
        printf(" invalid record type      ");
        break;
    case CANPC_RB_ERR_NORESP:
        printf(" no response after program start ");
        break;
    case CANPC_RB_ERR_BADRESP:
        printf(" bad response after program start");
        break;
    case CANPC_RB_PCMCIA_NSUPP:
        printf(" pcmcia chip not supported");
        break;
    case CANPC_RB_ERR_RD_PCMCIA:
        printf(" error reading ocmcia parameters ");
        break;
    case CANPC_RB_INIT_CHIP:
        printf(" error initializing chip  ");
        break;
    default:
        printf("return value ");
        break;
    }
    printf("\n");
}




/***********************************************************
  Initialize_can_parameter
  Reset and initialization of CAN chips
************************************************************/
INT32
Initialize_can_parameter(void)
{
    INT32 i;
    INT32 sw_version;
    INT32 fw_version;
    INT32 hw_version;
    INT32 license;
    USIGN32 SerialNumber;

    // Reset CAN chips of CAN 1 and 2
    i = CANPC_reset_chip();
    if (i != 0)
    {
        printf("-->Error in CANPC_reset_chip \n");
#ifdef LINUX
        if (i == CANPC_ERR_OPEN)
            printf("Error: open \"%s\" failed errno=%d: %s\n",
                   CAN_DEVICE_NAME, errno, strerror(errno));
        if (i == CANPC_ERR_IOCTL)
            printf("Initialize_can_parameter: ioctl failed errno=%d: %s\r\n",
                   errno, strerror(errno));
#endif
        return (-1);
    }

    // Display SW and HW version, chip type and licence (optionally)
    if (CANPC_get_serial_number(&SerialNumber) != 0)
    {
        printf("-->Error in CANPC_get_serial number \n\n\n");
        return (-1);
    }
    if (CANPC_get_version
        (&sw_version, &fw_version, &hw_version, &license, can_chip_type) != 0)
    {
        printf("-->Error in CANPC_get_version \n\n\n");
        return (-1);
    }
    else
    {
        printf("Software version: %u.%02u \n",
               sw_version / 100, sw_version % 100);
        printf("Firmware version: %u.%02u \n",
               fw_version / 100, fw_version % 100);
        printf("Hardware version: %x.%02x \n",
               hw_version % 0x100, hw_version / 0x100);
        for (i = 0; i <= 1; i++)
        {
            switch (can_chip_type[i])
            {
            case 5:
                printf("Chiptype CAN%1d: NEC72005\n", i + 1);
                break;
            case 1000:
                printf("Chiptype CAN%1d: SJA1000\n", i + 1);
                break;
            case 527:
                printf("Chiptype CAN%1d: 82527\n", i + 1);
                break;
            case 200:
                printf("Chiptype CAN%1d: 82C200\n", i + 1);
                break;
            default:
                break;
            }
        }
        if (license & 0x02)
            printf("CANcard licensed for CANalyzer \n");
        if (license & 0x04)
            printf("CANcard licensed for DDE Server \n");
    }
    printf("Serial#: %d\n", SerialNumber);


    // Set Bit timing for CAN 1 and 2
    if (CANPC_initialize_chip
        (PRESCALER_1, SYNC_JMP_WIDTH_1, TIME_SEG1_1, TIME_SEG2_1,
         SAMPLE_1) != 0)
    {
        printf("-->Error in CANPC_initialize_chip \n");
        return (-1);
    }

    if (CANPC_initialize_chip2
        (PRESCALER_2, SYNC_JMP_WIDTH_2, TIME_SEG1_2, TIME_SEG2_2,
         SAMPLE_2) != 0)
    {
        printf("-->Error in CANPC_initialize_chip2 \n");
        return (-1);
    }

    // Set acceptance filter for CAN 1 and 2   
    if (CANPC_set_acceptance
        (ACCEPT_MASK_1, ACCEPT_CODE_1, ACCEPT_MASK_XTD_1,
         ACCEPT_CODE_XTD_1) != 0)
    {
        printf("-->Error in CANPC_set_acceptance \n");
        return (-1);
    }

    if (CANPC_set_acceptance2
        (ACCEPT_MASK_2, ACCEPT_CODE_2, ACCEPT_MASK_XTD_2,
         ACCEPT_CODE_XTD_2) != 0)
    {
        printf("-->Error in CANPC_set_acceptance2 \n");
        return (-1);
    }

    // Set resynchronization
    if (CANPC_set_mode(SLEEPMODE_1, SPEEDMODE_1) != 0)
    {
        printf("-->Error in CANPC_set_mode \n");
        return (-1);
    }
    if (CANPC_set_mode2(SLEEPMODE_2, SPEEDMODE_2) != 0)
    {
        printf("-->Error in CANPC_set_mode2 \n");
        return (-1);
    }

    // Set output control (phys. layer -1: default  CAN High Speed)                               
    if (CANPC_set_output_control(OUTPUT_CONTROL_1) != 0)
    {
        printf("-->Error in CANPC_set_output_control \n");
        return (-1);
    }
    if (CANPC_set_output_control2(OUTPUT_CONTROL_2) != 0)
    {
        printf("-->Error in CANPC_set_output_control2 \n");
        return (-1);
    }

    return (0);
}                               // End of function


INT32
ReadBusEvent(void)
{
    INT32 ret;
    switch (op_mode)
    {
    case 'd':
        ret = ReadBusEventDynObjBuf();
        break;

    case 's':
        ret = ReadBusEventStatObjBuf();
        break;

    default:
        ret = ReadBusEventFIFO();
        break;
    }                           // switch

    if (ret < 0)
    {
        printf("-->Error polling CANPC_read_ac \n");
        INIPC_close_board();
        return (-1);
    }
    return ret;
}                               // end of fct

void
at_exit()
{
    close_keyboard();
}

/***********************************************************
  Main
  Program control and user inputs
************************************************************/
INT32
main(void)
{
    CANPC_RESOURCES crResources;
    char custom;

    INT32 ret;
    BOOL bInterruptFlag;

// Introduction screen
    printf("\n");
    printf("***************************************************************\n");
    printf("*                        SOFTING AG 1998-2003                 *\n");
    printf("*                                                             *\n");
    printf("*                                                             *\n");
    printf("*                           Testprogram                       *\n");
    printf("*                           for CAN API                       *\n");
    printf("*                              V4.05.001                      *\n");
    printf("*                                                             *\n");
    printf("*                         Type 'h' for help                   *\n");
    printf("*                                                             *\n");
    printf("***************************************************************\n");
    printf("\n");
    printf("Initialization started...\n");


// Not required for WIN32 APIs
#if defined (DOS) || defined (WIN16)
    /* only needed for DOS and WIN16 */
    crResources.uInterrupt = CANPC_IRQ_AUTO;
    crResources.ulDPRMemBase = CANPC_BASE_AUTO;
    crResources.ulDPRMemSize = CANPC_SIZE_AUTO;
    crResources.uSocket = CANPC_SOCKET_AUTO;
    /* Check for CAN-AC2 */
    if ((_access("ac_load.ac", 0)) != -1)
    {
        printf("\nDPRAM Adress of CAN-AC2 [0..7]:");
        scanf("%d", &crResources.ulDPRMemBase);
    }                           //Ac2 needs DPRAM area in 16bit OS
    /* Check for CAN-AC2-104 */
    if ((_access("Ldcan104.ini", 0)) != -1)
    {
        crResources.uSocket = 0xFFFE;   // Resource access with parametr of ldcan104.ini
    }

#else
// WIN32
    crResources.uSocket = CANPC_SOCKET_AUTO;
#endif
   // printf("Socket number: ");
  //  scanf("%hi", &crResources.uSocket);
  //  printf("\n");

#ifdef LINUX
    init_keyboard();
    atexit(at_exit);
#endif



// Init ressources 
    if ((ret = INIPC_initialize_board(&crResources)) != 0)
    {
        printf("-->Error in INIPC_initialize_board: ");
        printInitErrorText(ret);
        printf("The 2 CAN channels must be named Softing1 and Softing2 by using the SCIM or by editing /etc/softing/candevices.conf\n");
        // INIPC_close_board();
        return (-1);
    }
  //  printf("uSocket      =   %8u\n", crResources.uSocket);
  //  printf("uInterrupt   =   %8u\n", crResources.uInterrupt);
  //  printf("ulDPRMemBase = 0x%08X\n", crResources.ulDPRMemBase);
  //  printf("ulDPRMemSize =   %8u\n", crResources.ulDPRMemSize);
    printf("uChip        = %s\n",
           (crResources.uChip == NEC005_CHIP) ? "NEC uPD72005" : "SJA1000");

// Load firmware
    if ((ret = CANPC_reset_board()) != 0)
    {
        printf("-->Error in CANPC_reset_board: ");
        printResetErrorText(ret);
        INIPC_close_board();
        return (-1);
    }


    do                          //      Outer loop for reinitialization
    {

        //3.    Initialization of the CAN parameters for CAN channel 1 and 2
        if (Initialize_can_parameter())
        {
            printf("-->Initialization failed \n");
            INIPC_close_board();
            return (-1);
        }


        // Selection of operational mode
        printf("\n\nSelect DPRAM operational mode: ");
        printf("\n\n");
        printf("    Operational mode        Selection\n\n");
        printf("         FIFO                    f (default)\n");
        printf("   Dyn. object buffer            d\n");
        if (can_chip_type[0] != 527)
            printf("  Stat. object buffer            s\n");     // not supported bei Ac2/527  
        printf("\n");
        printf("Your selection:");
        do
        {
            op_mode = getchar();
        } while (!((op_mode == 'f') || (op_mode == 'd') || (op_mode == 's')));
        printf("\n");
        if ((can_chip_type[1] == 0x527) && (op_mode == 's'))
            op_mode = 'f';      // only for Ac2/527

        // Selction of polling or interrupt mode
        if (op_mode == 'f')
        {
            printf("\n\nSelect mode of reading events (CANPC_read_ac): ");
            printf("\n\n");
            printf("    Event mode           Selection\n\n");
            printf("    Interrupt                i (default)\n");
            printf("    Polling                  p\n");
            printf("\n");
            printf("Your selection:");
            do
            {
                custom = getchar();
            } while (!((custom == 'i') || (custom == 'p')));
            printf("\n");
        }
        else
            custom = 'p';
        if (custom == 'p')
            bInterruptFlag = FALSE;
        else
            bInterruptFlag = TRUE;

        //       DPRAM initialization of the chosen operational mode
        switch (op_mode)
        {
        case 'd':
            printf("DPRAM chosen in dynamic object buffer mode. Wait...\n");
            if (InitializeDynObjBuf() != 0)
            {
                printf("-->Initialization of dynamic object buffer mode failed \n");
                INIPC_close_board();
                return (-1);
            }
            break;

            // Not used with CAN-AC2/527
        case 's':
            printf("DPRAM chosen in static object buffer mode.\n");

            if (InitializeStatObjBuf() != 0)
            {
                printf("-->Initialization of dynamic object buffer mode failed \n");
                INIPC_close_board();
                return (-1);
            }
            break;

        default:
            printf("DPRAM chosen in FIFO mode.\n");
            if (InitializeFIFO() != 0)
            {
                printf("-->Initialization of FIFO mode failed \n");
                INIPC_close_board();
                return (-1);
            }
            break;

        }                       // switch

                // Start of operation
        if (CANPC_start_chip() != 0)
        {
          printf("-->Error in CANPC_start_chip \n");
          INIPC_close_board();
          return (-1);
        }
        else
          printf("Initialization successfull\nChip is running.\n\n\n");

        // Initialize interrupt handling
        if (bInterruptFlag)
        {
#ifdef LINUX
            if (init_pthread())
            {
                printf("-->Initialization Interrupt failed \n");
                INIPC_close_board();
                return (-1);
            }
#elif defined(WIN32)
            if (InitInterruptHandling32())
            {
                printf("-->Initialization Interrupt 32bit failed \n");
                INIPC_close_board();
                return (-1);
            }
#else
            if (InitInterruptHandling16())
            {
                printf("-->Initialization Interrupt 16bit failed \n");
                INIPC_close_board();
                return (-1);
            }
#endif
        }




        do                      // Inner loop
        {

#ifdef LINUX
            if (!bInterruptFlag)        /* No Keyboard polling - Just block in getch */
#endif
                // Loop till user request
                while (!kbhit())
                {
                    if (!bInterruptFlag)        //Polling mode chosen --> Poll CANPC_read_ac
                        ReadBusEvent();
                }               // end while

            custom = getch();

            // User request 
#ifdef DOS                      // Only for DOS interrupt
            if (bInterruptFlag)
                disable_interrupt();
#endif
            switch (op_mode)
            {
            case 'd':
                ret = UserRequestDynObjBuf(custom);
                break;

            case 's':
                ret = UserRequestStatObjBuf(custom);
                break;

            default:
                ret = UserRequestFIFO(custom);
                break;
            }                   // End switch

            if (ret << 0)
            {
                printf("-->User request failed \n");
                INIPC_close_board();
                return (-1);
            }
#ifdef DOS                      // Only for DOS interrupt
            if (bInterruptFlag)
                enable_interrupt();
#endif
        }
        while ((custom) != 'q' && custom != 'i');       // End inner loop
        CANPC_reinitialize();
#ifdef LINUX
        if (bInterruptFlag)
            exit_pthread();
#elif def WIN32
        if (bInterruptFlag)
            ExitInterruptHandling32();
#else
        if (bInterruptFlag)
            ExitInterruptHandling16();
#endif
    }
    while (custom == 'i');      // End outer loop
    INIPC_close_board();
    return (0);
}

//  End of main
