/* PROJECT_NAME         Example                                   
   MODULE               DYNOBUF.C                                      
   COMPANY              SOFTING AG Muenchen                                
   VERSION              $Id: Dynobuf.c,v 1.1 2006/09/07 12:36:42 kno Exp $
*/

#if defined (WIN32)
#include <windows.h>
#endif
#include <stdlib.h>
#include <string.h>
#ifndef linux
#include <conio.h>
#endif
#include <stdio.h>

/* CAN LAYER 2 LIBRARY INCLUDE FILES */

#include "can_def.h"
#include "canlay2.h"

#define MAX_OBJECTS 20

INT32 arObjectTableRx1[MAX_OBJECTS];
INT32 arObjectTableRx2[MAX_OBJECTS];
INT32 arObjectTableTx1[MAX_OBJECTS];
INT32 arObjectTableTx2[MAX_OBJECTS];

INT32 iObjectIndexRx1 = 0;
INT32 iObjectIndexRx2 = 0;
INT32 iObjectIndexTx1 = 0;
INT32 iObjectIndexTx2 = 0;

static INT32 Xtd;
static INT32 read_enabled;
static INT32 can_chip_type[2];

INT32
InitializeDynObjBuf(void)
{

    INT32 ReceiveFifoEnable;
    INT32 ReceivePollAll;
    INT32 ReceiveEnableAll;
    INT32 ReceiveIntEnableAll;
    INT32 AutoRemoteEnableAll;
    INT32 TransmitReqFifoEnable;
    INT32 TransmitPollAll;
    INT32 TransmitAckEnableAll;
    INT32 TransmitAckFifoEnableAll;
    INT32 TransmitRmtFifoEnableAll;

    INT32 Type;
    INT32 ReceiveIntEnable;
    INT32 AutoRemoteEnable;
    INT32 TransmitAckEnable;

    unsigned long Identifier;

    INT32 j;

    INT32 sw_version;
    INT32 fw_version;
    INT32 hw_version;
    INT32 license;
    // Just for board type detection
    if (CANPC_get_version
        (&sw_version, &fw_version, &hw_version, &license, can_chip_type) != 0)
    {
        printf("-->Error in CANPC_get_version \n\n\n");
        return (-1);
    }

    //1.  Enable dynamic object buffer mode
    if (CANPC_enable_dyn_obj_buf() != 0)
    {
        printf("-->Error in CANPC_enable_dyn_obj_buf \n\n");
        INIPC_close_board();
        return (-1);
    }

    // Configuration of the object buffer
    ReceiveFifoEnable = 1;
    ReceivePollAll = 0;
    ReceiveEnableAll = 0;
    ReceiveIntEnableAll = 0;
    AutoRemoteEnableAll = 0;
    TransmitReqFifoEnable = 1;
    TransmitPollAll = 0;
    TransmitAckEnableAll = 0;
    TransmitAckFifoEnableAll = 1;
    TransmitRmtFifoEnableAll = 1;
    if (CANPC_initialize_interface(ReceiveFifoEnable,
                                   ReceivePollAll,
                                   ReceiveEnableAll,
                                   ReceiveIntEnableAll,
                                   AutoRemoteEnableAll,
                                   TransmitReqFifoEnable,
                                   TransmitPollAll,
                                   TransmitAckEnableAll,
                                   TransmitAckFifoEnableAll,
                                   TransmitRmtFifoEnableAll) != 0)
    {
        printf("-->Error in CANPC_initialize_interface \n\n");
        INIPC_close_board();
        return (-1);
    }



    // 2.   Define receive objects
    for (j = 0; j < MAX_OBJECTS; j++)
    {
        Identifier = (unsigned long) j + 1;
        ReceiveIntEnable = 1;
        AutoRemoteEnable = 1;
        TransmitAckEnable = 1;


        Type = 0;
        if ((j >= MAX_OBJECTS / 2) && (can_chip_type[0] != 200))
            Type = 2;
        // for CAN 1
        if (CANPC_define_object(Identifier,
                                &arObjectTableRx1[j],
                                Type,
                                ReceiveIntEnable,
                                AutoRemoteEnable, TransmitAckEnable) != 0)
        {
            printf("-->Error in CANPC_define_object \n\n");
            INIPC_close_board();
            return (-1);
        }

        // for CAN 2
        if (CANPC_define_object2(Identifier,
                                 &arObjectTableRx2[j],
                                 Type,
                                 ReceiveIntEnable,
                                 AutoRemoteEnable, TransmitAckEnable) != 0)
        {
            printf("-->Error in CANPC_define_object2 \n\n");
            INIPC_close_board();
            return (-1);
        }


        // 3.     Define transmit objects *********/
        Type = 1;
        if ((j >= MAX_OBJECTS / 2) && (can_chip_type[0] != 200))
            Type = 3;

        // for CAN 1
        if (CANPC_define_object(Identifier,
                                &arObjectTableTx1[j],
                                Type,
                                ReceiveIntEnable,
                                AutoRemoteEnable, TransmitAckEnable) != 0)
        {
            printf("-->Error in CANPC_define_object \n\n");
            INIPC_close_board();
            return (-1);
        }

        // for CAN 2
        if (CANPC_define_object2(Identifier,
                                 &arObjectTableTx2[j],
                                 Type,
                                 ReceiveIntEnable,
                                 AutoRemoteEnable, TransmitAckEnable) != 0)
        {
            printf("-->Error in CANPC_define_object2 \n\n");
            INIPC_close_board();
            return (-1);
        }
    }

    read_enabled = 1;
    Xtd = 0;
    return (0);
}



INT32
UserRequestDynObjBuf(unsigned char custom)
{

    USIGN8 p_RCV_data[8];
    USIGN8 p_XMT_data[8];


    INT32 DataLength;
    INT32 Length;
    INT32 i, frc;
    static USIGN32 Time;


    switch (custom)
    {
    case 'h':                  // Output of the help menu
        printf("\n");
        printf("                   COMMUNICATION KEYS\n\n");
        printf("          CAN 1                              CAN2\n");
        printf("s: supply and send object can1      y: supply and send object can2\n");
        printf("v: supply transmit object can1      x: supply transmit object can2\n");
        printf("n: supply receive object can1       a: supply receive object can2\n");
        printf("t: send object can1                 r: send object can2\n");
        printf("z: send remote object can1          f: send remote object can2\n");
        printf("b: read transmit object can1        c: read transmit object can2\n");
        printf("e: read receive object can1         w: read receive object can2\n");
        printf("D: start cyclic transmission can1   U: start cyclic transmission can2\n");
        printf("F: stop cyclic transmission can1    I: stop cyclic transmission can2\n\n");
        if (can_chip_type[0] != 200)
            printf("\n    p: toggle identifier type standard/extended\n");
        printf("\n\n               ADMINISTRATION KEYS\n\n");

        printf("B: get bus state can1               N: get bus state can2\n");
        printf("h: help menu                        d: disable/enable CANPC_read()\n");
        printf("i: reinitialize                     q: quit\n");
        break;

    case 't':                  // Send object on CAN 1
        iObjectIndexTx1 = (iObjectIndexTx1 < 7) ? iObjectIndexTx1 + 1 : 0;

        DataLength = 8;
        frc =
            CANPC_send_object(arObjectTableTx1
                              [iObjectIndexTx1 + Xtd * MAX_OBJECTS / 2],
                              DataLength);
        printf("XMT     CAN1 Obj%7x Dlc%1x                              Return Code%2d\n",
             arObjectTableRx1[iObjectIndexTx1 + Xtd * MAX_OBJECTS / 2],
             DataLength, frc);
        break;

    case 'r':                  // Send object on CAN 2
        iObjectIndexTx2 = (iObjectIndexTx2 < 7) ? iObjectIndexTx2 + 1 : 0;

        DataLength = 8;
        frc =
            CANPC_send_object2(arObjectTableTx2
                               [iObjectIndexTx2 + Xtd * MAX_OBJECTS / 2],
                               DataLength);
        printf("XMT     CAN2 Obj%7x Dlc%1x                               Return Code%2d\n",
             arObjectTableRx1[iObjectIndexTx2 + Xtd * MAX_OBJECTS / 2],
             DataLength, frc);
        break;

    case 'z':                  // Send  remote object on CAN 1
        iObjectIndexTx1 = (iObjectIndexTx1 < 7) ? iObjectIndexTx1 + 1 : 0;
        DataLength = 0;

        frc =
            CANPC_send_remote_object(arObjectTableTx1
                                     [iObjectIndexTx1 +
                                      Xtd * MAX_OBJECTS / 2], DataLength);
        printf("XMT     CAN1 Obj%7x Dlc%1x REMOTE                      Return Code%2d\n",
             arObjectTableTx1[iObjectIndexTx1 + Xtd * MAX_OBJECTS / 2],
             DataLength, frc);
        break;

    case 'f':                  // Send  remote object on CAN 2

        iObjectIndexTx2 = (iObjectIndexTx2 < 7) ? iObjectIndexTx2 + 1 : 0;
        DataLength = 0;

        frc =
            CANPC_send_remote_object2(arObjectTableTx2
                                      [iObjectIndexTx2 +
                                       Xtd * MAX_OBJECTS / 2], DataLength);
        printf("XMT     CAN2 Obj%7x Dlc%1x REMOTE                      Return Code%2d\n",
             arObjectTableTx2[iObjectIndexTx2 + Xtd * MAX_OBJECTS / 2],
             DataLength, frc);
        break;

    case 's':                  // Supply and send object on CAN 1

        iObjectIndexTx1 = (iObjectIndexTx1 < 7) ? iObjectIndexTx1 + 1 : 0;

        /* set data bytes */
        for (i = 0; i <= 7; i++)
            p_XMT_data[i] = i + 0x10 + iObjectIndexTx1;
        DataLength = 8;

        frc =
            CANPC_write_object(arObjectTableTx1
                               [iObjectIndexTx1 + Xtd * MAX_OBJECTS / 2],
                               DataLength, p_XMT_data);
        printf("XMT     CAN1 Obj%7x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             arObjectTableRx1[iObjectIndexTx1 + Xtd * MAX_OBJECTS / 2],
             DataLength, p_XMT_data[0], p_XMT_data[1], p_XMT_data[2],
             p_XMT_data[3], p_XMT_data[4], p_XMT_data[5], p_XMT_data[6],
             p_XMT_data[7], frc);
        break;

    case 'y':                  // Supply and send object on CAN 2
        iObjectIndexTx2 = (iObjectIndexTx2 < 7) ? iObjectIndexTx2 + 1 : 0;

        /* set data bytes */
        for (i = 0; i <= 7; i++)
            p_XMT_data[i] = i + 0x20 + iObjectIndexTx2;
        DataLength = 8;

        frc =
            CANPC_write_object2(arObjectTableTx2
                                [iObjectIndexTx2 + Xtd * MAX_OBJECTS / 2],
                                DataLength, p_XMT_data);
        printf("XMT     CAN2 Obj%7x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             arObjectTableTx2[iObjectIndexTx2 + Xtd * MAX_OBJECTS / 2],
             DataLength, p_XMT_data[0], p_XMT_data[1], p_XMT_data[2],
             p_XMT_data[3], p_XMT_data[4], p_XMT_data[5], p_XMT_data[6],
             p_XMT_data[7], frc);
        break;



    case 'v':                  // Supply transmit object CAN 1
        iObjectIndexTx1 = (iObjectIndexTx1 < 7) ? iObjectIndexTx1 + 1 : 0;

        /* set data bytes */
        for (i = 0; i <= 7; i++)
            p_XMT_data[i] = i + 0x30 + iObjectIndexTx1;
        DataLength = 8;

        frc =
            CANPC_supply_object_data(arObjectTableTx1
                                     [iObjectIndexTx1 +
                                      Xtd * MAX_OBJECTS / 2], DataLength,
                                     p_XMT_data);
        printf("XMT SPL CAN1 Obj%7x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             arObjectTableTx1[iObjectIndexTx1 + Xtd * MAX_OBJECTS / 2],
             DataLength, p_XMT_data[0], p_XMT_data[1], p_XMT_data[2],
             p_XMT_data[3], p_XMT_data[4], p_XMT_data[5], p_XMT_data[6],
             p_XMT_data[7], frc);
        break;

    case 'x':                  // Supply transmit object CAN 2
        iObjectIndexTx2 = (iObjectIndexTx2 < 7) ? iObjectIndexTx2 + 1 : 0;

        /* set data bytes */
        for (i = 0; i <= 7; i++)
            p_XMT_data[i] = i + 0x40 + iObjectIndexTx2;
        DataLength = 8;

        frc =
            CANPC_supply_object_data2(arObjectTableTx2
                                      [iObjectIndexTx2 +
                                       Xtd * MAX_OBJECTS / 2], DataLength,
                                      p_XMT_data);
        printf("XMT SPL CAN2 Obj%7x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             arObjectTableTx2[iObjectIndexTx2 + Xtd * MAX_OBJECTS / 2],
             DataLength, p_XMT_data[0], p_XMT_data[1], p_XMT_data[2],
             p_XMT_data[3], p_XMT_data[4], p_XMT_data[5], p_XMT_data[6],
             p_XMT_data[7], frc);
        break;
    case 'n':                  // Supply receive object CAN 1
        iObjectIndexRx1 = (iObjectIndexRx1 < 7) ? iObjectIndexRx1 + 1 : 0;


        for (i = 0; i <= 7; i++)
            p_XMT_data[i] = i + 0x50 + iObjectIndexRx1;
        DataLength = 8;

        frc =
            CANPC_supply_rcv_object_data(arObjectTableRx1[iObjectIndexRx1],
                                         DataLength, p_XMT_data);
        printf("RCV SPL CAN1 Obj%7x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             arObjectTableRx1[iObjectIndexRx1], DataLength, p_XMT_data[0],
             p_XMT_data[1], p_XMT_data[2], p_XMT_data[3], p_XMT_data[4],
             p_XMT_data[5], p_XMT_data[6], p_XMT_data[7], frc);
        break;


    case 'a':                  // Supply receive object CAN 2
        iObjectIndexRx2 = (iObjectIndexRx2 < 7) ? iObjectIndexRx2 + 1 : 0;

        for (i = 0; i <= 7; i++)
            p_XMT_data[i] = i + 0x50 + iObjectIndexRx2;
        DataLength = 8;

        frc =
            CANPC_supply_rcv_object_data2(arObjectTableRx2[iObjectIndexRx2],
                                          DataLength, p_XMT_data);
        printf("RCV SPL CAN2 Obj%7x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             arObjectTableRx2[iObjectIndexRx2], DataLength, p_XMT_data[0],
             p_XMT_data[1], p_XMT_data[2], p_XMT_data[3], p_XMT_data[4],
             p_XMT_data[5], p_XMT_data[6], p_XMT_data[7], frc);
        break;

    case 'b':                  // Read data of transmit object of CAN 1

        iObjectIndexTx1 = (iObjectIndexTx1 < 7) ? iObjectIndexTx1 + 1 : 0;

        frc =
            CANPC_read_xmt_data(arObjectTableTx1
                                [iObjectIndexTx1 + Xtd * MAX_OBJECTS / 2],
                                &Length, p_RCV_data);
        printf("XMT RD  CAN1 Obj%7x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             arObjectTableTx1[iObjectIndexTx1 + Xtd * MAX_OBJECTS / 2],
             Length, p_RCV_data[0], p_RCV_data[1], p_RCV_data[2],
             p_RCV_data[3], p_RCV_data[4], p_RCV_data[5], p_RCV_data[6],
             p_RCV_data[7], frc);
        break;

    case 'c':                  // Read data of transmit object of CAN 2
        iObjectIndexTx2 = (iObjectIndexTx2 < 7) ? iObjectIndexTx2 + 1 : 0;

        frc =
            CANPC_read_xmt_data2(arObjectTableTx2
                                 [iObjectIndexTx2 + Xtd * MAX_OBJECTS / 2],
                                 &Length, p_RCV_data);
        printf("XMT RD  CAN2 Obj%7x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             arObjectTableTx2[iObjectIndexTx2 + Xtd * MAX_OBJECTS / 2],
             Length, p_RCV_data[0], p_RCV_data[1], p_RCV_data[2],
             p_RCV_data[3], p_RCV_data[4], p_RCV_data[5], p_RCV_data[6],
             p_RCV_data[7], frc);
        break;

    case 'e':                  // Read data of receive object of CAN 1
        iObjectIndexRx1 = (iObjectIndexRx1 < 7) ? iObjectIndexRx1 + 1 : 0;

        frc =
            CANPC_read_rcv_data(arObjectTableRx1[iObjectIndexRx1], p_RCV_data,
                                &Time);
        printf("RCV RD  CAN1 Obj%7x Dlc? Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             arObjectTableRx1[iObjectIndexRx1], p_RCV_data[0], p_RCV_data[1],
             p_RCV_data[2], p_RCV_data[3], p_RCV_data[4], p_RCV_data[5],
             p_RCV_data[6], p_RCV_data[7], frc);
        break;

    case 'w':                  // Read data of receive object of CAN 2
        iObjectIndexRx2 = (iObjectIndexRx2 < 7) ? iObjectIndexRx2 + 1 : 0;

        frc =
            CANPC_read_rcv_data2(arObjectTableRx2[iObjectIndexRx2],
                                 p_RCV_data, &Time);
        printf("RCV RD  CAN2 Obj%7x Dlc? Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             arObjectTableRx2[iObjectIndexRx2], p_RCV_data[0], p_RCV_data[1],
             p_RCV_data[2], p_RCV_data[3], p_RCV_data[4], p_RCV_data[5],
             p_RCV_data[6], p_RCV_data[7], frc);
        break;



    case 'D':                  // Start cyclic transmission CAN 1
        iObjectIndexTx1 = (iObjectIndexTx1 < 7) ? iObjectIndexTx1 + 1 : 0;
        /* set data bytes */
        for (i = 0; i <= 7; i++)
            p_XMT_data[i] = i + 0x80 + iObjectIndexTx1;
        DataLength = 8;

        if (CANPC_define_cyclic
            (arObjectTableTx1[iObjectIndexTx1 + Xtd * MAX_OBJECTS / 2], 1000,
             0) != 0)
        {
            printf("-->Error in CANPC_define_cyclic \n\n\n");
            INIPC_close_board();
            return (-1);
        }
        frc =
            CANPC_write_object(arObjectTableTx1
                               [iObjectIndexTx1 + Xtd * MAX_OBJECTS / 2],
                               DataLength, p_XMT_data);
        printf("XMT     CAN1 Obj%7x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             arObjectTableRx1[iObjectIndexTx1 + Xtd * MAX_OBJECTS / 2],
             DataLength, p_XMT_data[0], p_XMT_data[1], p_XMT_data[2],
             p_XMT_data[3], p_XMT_data[4], p_XMT_data[5], p_XMT_data[6],
             p_XMT_data[7], frc);
        break;

    case 'U':                  // Start cyclic transmission CAN2

        iObjectIndexTx2 = (iObjectIndexTx2 < 7) ? iObjectIndexTx2 + 1 : 0;

        /* set data bytes */
        for (i = 0; i <= 7; i++)
            p_XMT_data[i] = i + 0x90 + iObjectIndexTx2;
        DataLength = 8;

        if (CANPC_define_cyclic2
            (arObjectTableTx2[iObjectIndexTx2 + Xtd * MAX_OBJECTS / 2], 1000,
             0) != 0)
        {
            printf("-->Error in CANPC_define_cyclic2 \n\n\n");
            INIPC_close_board();
            return (-1);
        }
        frc =
            CANPC_write_object2(arObjectTableTx2
                                [iObjectIndexTx2 + Xtd * MAX_OBJECTS / 2],
                                DataLength, p_XMT_data);
        printf("XMT     CAN2 Obj%7x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             arObjectTableRx2[iObjectIndexTx2 + Xtd * MAX_OBJECTS / 2],
             DataLength, p_XMT_data[0], p_XMT_data[1], p_XMT_data[2],
             p_XMT_data[3], p_XMT_data[4], p_XMT_data[5], p_XMT_data[6],
             p_XMT_data[7], frc);
        break;

    case 'F':                  // Stop cyclic transmission CAN 1

        iObjectIndexTx1 = (iObjectIndexTx1 < 7) ? iObjectIndexTx1 + 1 : 0;

        if (CANPC_define_cyclic
            (arObjectTableTx1[iObjectIndexTx1 + Xtd * MAX_OBJECTS / 2], 0,
             0) != 0)
        {
            printf("-->Error in CANPC_define_cyclic \n\n\n");
            INIPC_close_board();
            return (-1);
        }
        break;

    case 'I':                  // Stop cyclic transmission CAN2

        iObjectIndexTx2 = (iObjectIndexTx2 < 7) ? iObjectIndexTx2 + 1 : 0;


        if (CANPC_define_cyclic2
            (arObjectTableTx2[iObjectIndexTx2 + Xtd * MAX_OBJECTS / 2], 0,
             0) != 0)
        {
            printf("-->Error in CANPC_define_cyclic \n\n\n");
            INIPC_close_board();
            return (-1);
        }
        break;


    case 'B':
        printf("bus state CAN1:%d \n", CANPC_get_bus_state(1));
        break;

    case 'N':
        printf("bus state CAN2:%d \n", CANPC_get_bus_state(2));
        break;

    case 'd':
        read_enabled = 1 - read_enabled;
        if (read_enabled)
            printf("CANPC_read_ac() enabled\n");
        else
            printf("CANPC_read_ac() disabled\n");
        break;

    case 'p':
        if (can_chip_type[0] != 200)    // not for Ac2 with 82C200
        {
            Xtd = 1 - Xtd;
            if (Xtd)
                printf("Extented identifier enabled\n");
            else
                printf("Standard identifier enabled\n");
        }
        break;

    default:
        break;
    }

    return (0);
}


INT32
ReadBusEventDynObjBuf(void)
{
    param_struct param;
    INT32 frc;

    if (read_enabled)
    {
        do
        {
            switch (frc = CANPC_read_ac(&param))
            {
            case 1:
                printf("RCV STD CAN%1u Obj%7x Dlc? Data %2x %2x %2x %2x %2x %2x %2x %2x\n",
                     param.Can, param.Ident, param.RCV_data[0],
                     param.RCV_data[1], param.RCV_data[2], param.RCV_data[3],
                     param.RCV_data[4], param.RCV_data[5], param.RCV_data[6],
                     param.RCV_data[7]);
                break;
            case 2:
                printf("RCV STD CAN%1u Obj%7x Dlc? REMOTE\n",
                       param.Can, param.Ident);
                break;
            case 9:
                printf("RCV XTD CAN%1u Obj%7x Dlc? Data %2x %2x %2x %2x %2x %2x %2x %2x\n",
                     param.Can, param.Ident, param.RCV_data[0],
                     param.RCV_data[1], param.RCV_data[2], param.RCV_data[3],
                     param.RCV_data[4], param.RCV_data[5], param.RCV_data[6],
                     param.RCV_data[7]);
                break;
            case 12:
                printf("RCV XTD CAN%1u Obj%7x Dlc? REMOTE\n",
                       param.Can, param.Ident);
                break;
            case 3:
                printf("ACK STD CAN%1u Obj%7x Dlc?\n",
                       param.Can, param.Ident);
                break;
            case 8:
                printf("ACK STD CAN%1u Obj%7x Dlc0 REMOTE\n",
                       param.Can, param.Ident);
                break;
            case 10:
                printf("ACK XTD CAN%1u Obj%7x Dlc?\n",
                       param.Can, param.Ident);
                break;
            case 11:
                printf("ACK XTD CAN%1u Obj%7x Dlc0 REMOTE\n",
                       param.Can, param.Ident);
                break;
            case 5:
                printf("ERROR  CAN%1u Bus state: %1d\n",
                       param.Can, param.Bus_state);

                break;
            default:
                break;
            }
        }
        while (frc != 0);
    }                           // if (read_enabled)
    return (0);
}
