/* PROJECT_NAME         Example                                   
   MODULE               FIFO.C                                      
   COMPANY              SOFTING AG Muenchen                                
   VERSION              $Id: Fifo.c,v 1.1 2006/09/07 12:36:42 kno Exp $
*/

#if defined (WIN32)
#include <windows.h>
#endif
#include <stdlib.h>
#include <string.h>
#ifndef linux
#include <conio.h>
#endif
#include <stdio.h>


/* CAN LAYER 2 LIBRARY INCLUDE FILES */

#include "can_def.h"
#include "canlay2.h"

#ifdef LINUX
#include <errno.h>
#endif

static USIGN32 diff_time, old_time, Time; // Time of event
static INT32 read_enabled;
static INT32 Xtd;                 // 0:11 bit Identifier,   1: 29bit Identifier
static INT32 can_chip_type[2];


INT32
InitializeFIFO(void)
{
    INT32 sw_version;
    INT32 fw_version;
    INT32 hw_version;
    INT32 license;

    if (CANPC_get_version
        (&sw_version, &fw_version, &hw_version, &license, can_chip_type) != 0)
    {
        printf("-->Error in CANPC_get_version \n\n\n");
        return (-1);
    }

    if (CANPC_enable_fifo() != 0)
    {
        printf("-->Error in CANPC_enable_fifo \n\n");
        return (-1);
    }
    if (CANPC_enable_timestamps() != 0)
    {
        printf("-->Error in CANPC_enable_timestamps \n\n");
        return (-1);
    }
    if (CANPC_enable_fifo_transmit_ack() != 0)
    {
        printf("-->Error in CANPC_enable_fifo_transmit_ack \n\n");
        return (-1);
    }
    if (CANPC_enable_fifo_transmit_ack2() != 0)
    {
        printf("-->Error in CANPC_enable_fifo_transmit_ack2 \n\n");
        return (-1);
    }
    read_enabled = 1;
    Xtd = 0;
    return (0);
}




INT32
UserRequestFIFO(char custom)
{

    static USIGN32 Identifier = 0;    // Identifier 
    INT32 DataLength;             // Datalength of the data frames
    unsigned char p_XMT_data[8];        // Array of data bytes to be sent
    INT32 frc;                    // Function return code


    INT32 k = 0, i;               // Loop variables


    switch (custom)
    {
    case 'h':                  // Output of the help menu
        printf("\n               COMMUNICATION KEYS\n\n");
        printf("        CAN 1                     CAN2\n");
        printf("t: transmit data can1         z: transmit data can2\n");
        printf("r: transmit remote can1       f: transmit remote can2\n");
        if (can_chip_type[0] != 200)    // not for Ac2/200
            printf("\n    p: toggle identifier type standard/extended\n");
        printf("\n\n               ADMINISTRATION KEYS\n\n");

        printf("l: get time                   k: reset fifos\n");
        printf("G: get fifo levels            L: reset lost msg counter\n");
        printf("B: get bus state can1         N: get bus state can2\n");
        printf("h: help menu                  d: disable/enable CANPC_read()\n");
        printf("i: reinitialize               q: quit\n\n");
        break;

    case 't':                  // Send data frame on CAN 1
      Identifier = (Identifier+1) & 0x7L;
        for (i = 0; i <= 7; i++)
            p_XMT_data[i] = (unsigned char) (Identifier + i + k++);
        DataLength = (INT32) (8 - Identifier);

        frc = CANPC_send_data(Identifier, Xtd, DataLength, p_XMT_data);
        printf("XMT     CAN1 Id%8x Dlc%1x Data%2x %2x %2x %2x %2x %2x %2x %2x return Code%2d\n",
             Identifier, DataLength, p_XMT_data[0], p_XMT_data[1],
             p_XMT_data[2], p_XMT_data[3], p_XMT_data[4], p_XMT_data[5],
             p_XMT_data[6], p_XMT_data[7], frc);
        break;

    case 'r':                  // Send remote frame on CAN 1
      Identifier = (Identifier+1) & 0x7L;
        DataLength = 0;

        frc = CANPC_send_remote(Identifier, Xtd, DataLength);
        printf("XMT     CAN1 Id%8x Dlc%1x REMOTE                      return Code%2d\n",
             Identifier, DataLength, frc);
        break;


    case 'z':                  // Send data frame on CAN 2
      Identifier = (Identifier+1) & 0x7L;
        for (i = 0; i <= 7; i++)
            p_XMT_data[i] = (unsigned char) (Identifier + i + k++);
        DataLength = (INT32) (8 - Identifier);
        frc = CANPC_send_data2(Identifier, Xtd, DataLength, p_XMT_data);
        printf("XMT     CAN2 Id%8x Dlc%1x Data%2x %2x %2x %2x %2x %2x %2x %2x return Code%2d\n",
             Identifier, DataLength, p_XMT_data[0], p_XMT_data[1],
             p_XMT_data[2], p_XMT_data[3], p_XMT_data[4], p_XMT_data[5],
             p_XMT_data[6], p_XMT_data[7], frc);
        break;

    case 'f':                  // Send remote frame on CAN 2
      Identifier = (Identifier+1) & 0x7L;
        DataLength = 0;
        frc = CANPC_send_remote2(Identifier, Xtd, DataLength);
        printf("XMT     CAN2 Id%8x Dlc%1x REMOTE                      return Code%2d\n",
             Identifier, DataLength, frc);
        break;


    case 'l':
        CANPC_get_time(&Time);
        diff_time = Time - old_time;
        printf("time:                                                    T%8x D%8x\n",
             Time, diff_time);
        old_time = Time;
        break;

    case 'k':
        CANPC_reset_rcv_fifo();
        CANPC_reset_xmt_fifo();
        printf("reset fifos\n");
        break;

    case 'L':
        CANPC_reset_lost_msg_counter();
        printf("reset lost msg counter\n");
        break;

    case 'G':                  // Read transmit and receive fifos and the lost messages counter
        printf("rcv fifo level:%d xmt fifo level:%d\n",
               CANPC_read_rcv_fifo_level(), CANPC_read_xmt_fifo_level());
        break;

    case 'B':
        printf("bus state CAN1:%d \n", CANPC_get_bus_state(1));
        break;

    case 'N':
        printf("bus state CAN2:%d \n", CANPC_get_bus_state(2));
        break;

    case 'd':
        read_enabled = 1 - read_enabled;
        if (read_enabled)
            printf("CANPC_read_ac() enabled\n");
        else
            printf("CANPC_read_ac() disabled\n");
        break;

    case 'p':
        if (can_chip_type[0] != 200)    // not for Ac2 with 82C200
        {
            Xtd = 1 - Xtd;
            if (Xtd)
                printf("Extented identifier enabled\n");
            else
                printf("Standard identifier enabled\n");
        }
        break;

    case '\n':
        printf("\n");
        break;

    default:
        break;
    }

    return (0);
}


INT32
ReadBusEventFIFO(void)
{
    param_struct param;
    INT32 frc;

    if (read_enabled)
    {
        do
        {
            switch (frc = CANPC_read_ac(&param))
            {
            case 1:
                diff_time = param.Time - old_time;
                
                   printf("RCV STD CAN%1u Id%8x Dlc%1x Data%2x %2x %2x %2x %2x %2x %2x %2x T%8x D%8x\n",
                   param.Can, param.Ident, param.DataLength,
                   param.RCV_data[0], param.RCV_data[1], param.RCV_data[2],
                   param.RCV_data[3], param.RCV_data[4], param.RCV_data[5],
                   param.RCV_data[6], param.RCV_data[7], param.Time,
                   diff_time);
                 
                old_time = param.Time;

                break;
            case 2:
                diff_time = param.Time - old_time;
                printf("RCV STD CAN%1u Id%8x Dlc%1x REMOTE                      T%8x D%8x\n",
                     param.Can, param.Ident, param.DataLength, param.Time,
                     diff_time);
                old_time = param.Time;
                break;
            case 9:            // not valid with AC2 with 82C200
                diff_time = param.Time - old_time;
                printf("RCV XTD CAN%1u Id%8x Dlc%1x Data%2x %2x %2x %2x %2x %2x %2x %2x T%8x D%8x\n",
                     param.Can, param.Ident, param.DataLength,
                     param.RCV_data[0], param.RCV_data[1], param.RCV_data[2],
                     param.RCV_data[3], param.RCV_data[4], param.RCV_data[5],
                     param.RCV_data[6], param.RCV_data[7], param.Time,
                     diff_time);
                old_time = param.Time;
                break;
            case 12:           // not valid with AC2 with 82C200
                diff_time = param.Time - old_time;
                printf("RCV XTD CAN%1u Id%8x Dlc%1x REMOTE                      T%8x D%8x\n",
                     param.Can, param.Ident, param.DataLength, param.Time,
                     diff_time);
                old_time = param.Time;
                break;
            case 3:
                diff_time = param.Time - old_time;
                
                   printf("ACK STD CAN%1u Id%8x Dlc%1x Data%2x %2x %2x %2x %2x %2x %2x %2x T%8x D%8x\n",
                   param.Can, param.Ident, param.DataLength,
                   param.RCV_data[0], param.RCV_data[1], param.RCV_data[2],
                   param.RCV_data[3], param.RCV_data[4], param.RCV_data[5],
                   param.RCV_data[6], param.RCV_data[7], param.Time,
                   diff_time);
                 
                old_time = param.Time;
                break;
            case 8:
                diff_time = param.Time - old_time;
                printf("ACK STD CAN%1u Id%8x Dlc%1x REMOTE                      T%8x D%8x\n",
                     param.Can, param.Ident, param.DataLength, param.Time,
                     diff_time);
                old_time = param.Time;
                break;
            case 10:           // not valid with AC2 with 82C200
                diff_time = param.Time - old_time;
                printf("ACK XTD CAN%1u Id%8x Dlc%1x Data%2x %2x %2x %2x %2x %2x %2x %2x T%8x D%8x\n",
                     param.Can, param.Ident, param.DataLength,
                     param.RCV_data[0], param.RCV_data[1], param.RCV_data[2],
                     param.RCV_data[3], param.RCV_data[4], param.RCV_data[5],
                     param.RCV_data[6], param.RCV_data[7], param.Time,
                     diff_time);
                old_time = param.Time;
                break;
            case 11:           // not valid with AC2 with 82C200
                diff_time = param.Time - old_time;
                printf("ACK XTD CAN%1u Id%8x Dlc%1x REMOTE                      T%8x D%8x\n",
                     param.Can, param.Ident, param.DataLength, param.Time,
                     diff_time);
                old_time = param.Time;
                break;
            case 5:
                diff_time = param.Time - old_time;
                printf("ERROR  CAN%1u Bus state: %1d                      T%8x D%8x\n",
                     param.Can, param.Bus_state, param.Time, diff_time);
                old_time = param.Time;
                break;
            default:
                break;
            }
        }
        while (frc != 0);
    }                           // if (read_enabled)
    return (0);
}
