/* PROJECT_NAME         Can_test                                   
   MODULE               Intexmpl.C                                      
   COMPANY              SOFTING AG Muenchen                                
   VERSION              $Id: Intexmpl.c,v 1.1 2006/09/07 12:36:42 kno Exp $
   FUNCTIONS            Initialization of 16/32bit interrupt
                        handling
                        Interrupt service routines and threads
                        Reallocation of interrupt resources

***************************************************************/
#ifdef WIN32
#include <windows.h>
#include <process.h>
#elif defined(linux)
#else
#include <dos.h>
#endif
#include <stdlib.h>
#include <string.h>
#ifndef linux
#include <conio.h>
#endif
#include <stdio.h>

/* CAN LAYER 2 LIBRARY INCLUDE FILES */

#include "can_def.h"
#include "canlay2.h"

#ifdef LINUX
#include <errno.h>
#include <fcntl.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/poll.h>
#endif

#define CAN_HANDLE unsigned long

INT32 ReadBusEvent(void);


#ifdef LINUX

/* define CONFIG_SELECT to use select */
/* #define CONFIG_SELECT           // */

/* define CONFIG_POLL to use poll */
#define CONFIG_POLL

/**********************************************************

  Interrupt handling routines for LINUX

***********************************************************/

#define CAN_DEVICE_NAME "/dev/can_00"

pthread_t interrupt_pthread1;
pthread_t interrupt_pthread2;
CAN_HANDLE can1, can2;


/**********************************************************
  Interrupt service thread 1 for the first channel
**********************************************************/
void interruptThread1(void *context)
{

  CAN_HANDLE can_channel = *(CAN_HANDLE *)context;
  INT32 ret;
  
  struct pollfd can_poll;
  

  
  while(1)
  {
    
    can_poll.fd = can_channel;
    can_poll.events = POLLIN | POLLHUP;
    
    do
    {
      ret = poll(&can_poll, 1, -1);
    //  printf("interruptThread2: poll returned %u, events=0x%x. revents=0x%x, POLLHUP=0x%x\n",ret,can_poll.events,can_poll.revents,POLLHUP);
      
      if(can_poll.revents & POLLHUP)
      {
        printf("terminating interruptThread2\n");
        return;
      }
      
      
      if (ret == -1 && errno != EINTR)
      {
        printf("Error: poll \"\" failed: (%d) %s\n",
               errno, strerror(errno));
      }
    } while (ret <= 0);

    while(ReadBusEvent() != 0);
 
  } // while(!end)
  return;
}
/**********************************************************
  Interrupt service thread 2 for the second channel
**********************************************************/
void interruptThread2(LPVOID context)
{

  CAN_HANDLE can_channel = *(CAN_HANDLE *)context;
  INT32 ret;
  
  struct pollfd can_poll;
  

  
  while(1)
  {
    can_poll.fd = can_channel;
    can_poll.events = POLLIN | POLLHUP;

 
    do
    {
      ret = poll(&can_poll, 1, -1);
    //  printf("interruptThread2: poll returned %u, events=0x%x. revents=0x%x, POLLHUP=0x%x\n",ret,can_poll.events,can_poll.revents,POLLHUP);
      
      if(can_poll.revents & POLLHUP)
      {
        printf("terminating interruptThread2\n");
        return;
      }
      
      
      if (ret == -1 && errno != EINTR)
      {
        printf("Error: poll \"\" failed: (%d) %s\n",
               errno, strerror(errno));
      }
    } while (ret <= 0);

    while(ReadBusEvent() != 0);
 
  } // while(!end)
  return;
}


/**********************************************************
    Initialize interrupt handling
**********************************************************/
INT32
    init_pthread(/*CAN_HANDLE *can1, CAN_HANDLE *can2*/)
{
  INT32 ret;

  ret = CANPC_get_fd(&can1, &can2);
  
  if(ret)
    return ret;
  
  if ((ret =
       pthread_create(&interrupt_pthread1, 0, (void *) interruptThread1, &can1)))
  {
    return (-1);
  }
  
  if ((ret =
       pthread_create(&interrupt_pthread2, 0, (void *) interruptThread2, &can2)))
  {
    return (-1);
  }
  
  
  
  
  return (0);
}

/**********************************************************
    Release interrupt handling ressources
**********************************************************/
void
    exit_pthread(void)
{
  pthread_cancel(interrupt_pthread1);
  pthread_cancel(interrupt_pthread2);
}


/* End of Linux interrupt handling code */
/**********************************************************/

#elif defined(WIN32)
/**********************************************************

  Interrupt handling routines for WIN32

***********************************************************/
#define INT_EVENT 0
#define END_EVENT 1

struct _threadData
{
    HANDLE events[2];
};

struct _threadData threadData;
HANDLE intThread;
DWORD tid;


/**********************************************************
    Interrupt service thread
**********************************************************/
DWORD WINAPI
interruptThread(LPVOID context)
{

    DWORD ret;
    BOOL end = FALSE;
    struct _threadData *td = (struct _threadData *) context;
    while (!end)
    {
        ret = WaitForMultipleObjects(2, td->events, FALSE, INFINITE);
        switch (ret - WAIT_OBJECT_0)
        {
        case INT_EVENT:
            // Place interrupt service code here
            ReadBusEvent();
            break;
        case END_EVENT:
            end = TRUE;
            printf("terminating thread\n");
            break;
        default:
            //               end = TRUE;
            printf("bad wait result 0x%X -> terminating thread\n", ret);
            break;
        }
    }
    return 0;
}


/**********************************************************
    Initialize interrupt handling
**********************************************************/
INT32
InitInterruptHandling32(void)
{
    INT32 ret;
    if (NULL ==
        (threadData.events[INT_EVENT] =
         CreateEvent(NULL, FALSE, FALSE, NULL)))
    {
        printf("-->Error creating interrupt event 0x%X\n", GetLastError());
        return (-1);
    }

    ResetEvent(threadData.events[INT_EVENT]);

    if (NULL ==
        (threadData.events[END_EVENT] =
         CreateEvent(NULL, FALSE, FALSE, NULL)))
    {
        printf("-->Error creating thread end event 0x%X\n", GetLastError());
        CloseHandle(threadData.events[INT_EVENT]);
        return (-1);
    }

    if (CANPC_OK !=
        (ret = CANPC_set_interrupt_event(threadData.events[INT_EVENT])))
    {
        printf("-->Error set interrupt event 0x%X\n", ret);
        CloseHandle(threadData.events[INT_EVENT]);
        CloseHandle(threadData.events[END_EVENT]);
        return (-1);
    }

    if (NULL ==
        (intThread =
         _beginthreadex(NULL, 0, interruptThread, (void *) &threadData, 0,
                        &tid)))
    {
        printf("-->Error create interrupt thread 0x%X\n", GetLastError());
        INIPC_close_board();
        CloseHandle(threadData.events[INT_EVENT]);
        CloseHandle(threadData.events[END_EVENT]);
        return (-1);
    }
    return (0);
}



/**********************************************************
    Release interrupt handling ressources
**********************************************************/
void
ExitInterruptHandling32(void)
{
    SetEvent(threadData.events[END_EVENT]);
    WaitForSingleObject(intThread, INFINITE);
    CloseHandle(threadData.events[INT_EVENT]);
    CloseHandle(threadData.events[END_EVENT]);
}

/*  End of Win32 interrupt handling code                  */
/**********************************************************/




#else
/**********************************************************

  Interrupt handling routines for 16bit

***********************************************************/
typedef struct
{
    INT32 Used;
    INT32 IRQNr;
    INT32 OriginalEnabled;
// void (interrupt far *OriginalISR) ();
    INT32 OriginalISR;
    void (*UserISR) ();
}
IntrType;

#define _IRQIntNr_(IRQ)   (IRQ  +  (IRQ>7? (0x70-0x08):0x08))
#define _IRQBit_(IRQ)     (0x01 << (IRQ>7? IRQ-8:IRQ))
#define _IRQCW1_(IRQ)              (IRQ>7? 0xa1:0x21)
#define _IRQCW2_(IRQ)              (IRQ>7? 0xa0:0x20)
#define _IRQIntEnd_  0x20

static IntrType InterruptInfo = { 0 };
INT32 int_line;


#ifdef WIN
void
setvect(unsigned INT32 intnum, void (__cdecl __interrupt __far * handler) ())
{
    _asm
    {
    push ds
            lds dx, handler
            mov al, byte ptr intnum mov ah, 25 h INT32 21 h pop ds}
}
#endif                          //WIN

/**************************************************************/
/* Name      : enable_interrupt                               */
/* Funktion  : enable initialized interrupt                   */
/* Parameter : /                                              */
/* Returncode: void                                           */
/**************************************************************/
void
enable_interrupt(void)
{
    if (!InterruptInfo.Used)
        return;

    outp(_IRQCW1_(InterruptInfo.IRQNr),
         inp(_IRQCW1_(InterruptInfo.IRQNr)) & ~_IRQBit_(InterruptInfo.IRQNr));
}

/**************************************************************/
/* Name      : disable_interrupt                              */
/* Funktion  : disable initialized interrupt                  */
/* Parameter : /                                              */
/* Returncode: void                                           */
/**************************************************************/
void
disable_interrupt(void)
{
    if (!InterruptInfo.Used)
        return;

    outp(_IRQCW1_(InterruptInfo.IRQNr),
         inp(_IRQCW1_(InterruptInfo.IRQNr)) | _IRQBit_(InterruptInfo.IRQNr));

}

/**************************************************************/
/* Name      : interrupt_service                              */
/* Function  : Interruptserviceroutine for any hardware       */
/*             interrupt. Calls user service routine and      */
/*             services the interruptcontroller.              */
/* Parameter : none                                           */
/* Returncode: void                                           */
/**************************************************************/
static void interrupt _far
interrupt_service(void)
{
    /* call user routine */
    InterruptInfo.UserISR();

    if (InterruptInfo.IRQNr > 7)
    {
        /* service secondary interruptcontroller */
        outp(_IRQCW2_(InterruptInfo.IRQNr), _IRQIntEnd_);
        /* service primary interruptcontroller */
        outp(_IRQCW2_(2), _IRQIntEnd_);
    }
    else
    {
        /* service primary interruptcontroller */
        outp(_IRQCW2_(InterruptInfo.IRQNr), _IRQIntEnd_);
    }
}

/**************************************************************/
/* Name      : install_interrupt                              */
/* Function  : Installation of an interrupt service routine.  */
/* Parameter : IRQNr 0..15                                    */
/*             UserISR: user interrupt service routine        */
/* Returncode: 0 : successful                                 */
/*             -1: error ( interrupt_number invalid or already*/
/*                         in use)                            */
/**************************************************************/
INT32
install_interrupt(INT32 IRQNr, void (*UserISR) (void))
{
    if ((IRQNr <= 0) || (IRQNr > 15))
        return (-1);

    if (InterruptInfo.Used)
        return (-1);

    /* store IRQNr */
    InterruptInfo.IRQNr = IRQNr;

    /* check old interrupt enabled */
    InterruptInfo.OriginalEnabled = !(inp(_IRQCW1_(IRQNr)) & _IRQBit_(IRQNr));

    /* disable interrupt */
    outp(_IRQCW1_(IRQNr), inp(_IRQCW1_(IRQNr)) | _IRQBit_(IRQNr));

    /* store old vector */
    InterruptInfo.OriginalISR = _dos_getvect(_IRQIntNr_(IRQNr));

    /* set new vector */
#ifdef WIN
    setvect(_IRQIntNr_(IRQNr), interrupt_service);
#else
    _dos_setvect(_IRQIntNr_(IRQNr), interrupt_service);
#endif

    /* install user service */
    InterruptInfo.UserISR = UserISR;

    InterruptInfo.Used = 1;

    return (0);
}

/**************************************************************/
/* Name      : deinstall_interrupt                            */
/* Function  : Deinstallation of interrupt.                   */
/* Parameter : /                                              */
/* Returncode: void                                           */
/**************************************************************/
void
deinstall_interrupt(void)
{
    if (!InterruptInfo.Used)
        return;

    disable_interrupt();

    /* set old vector */
#ifdef WIN
    setvect(_IRQIntNr_(InterruptInfo.IRQNr), InterruptInfo.OriginalISR);
#else
    _dos_setvect(_IRQIntNr_(InterruptInfo.IRQNr), InterruptInfo.OriginalISR);
#endif

    /* restore old interrupt enabled */
    if (InterruptInfo.OriginalEnabled)
        enable_interrupt();

    InterruptInfo.Used = 0;     /* free internal resource */
}




/**********************************************************
    Interrupt service routine
**********************************************************/
void
can_isr(void)
{
    // Place interrupt service code here
    disable_interrupt();
    ReadBusEvent();
    enable_interrupt();

    return;
}


/**********************************************************
    Initialize interrupt handling
**********************************************************/
int
InitInterruptHandling16(void)
{
    do
    {
        printf("Input interrupt line:");
        scanf("%i", &int_line);
    }
    while ((int_line <= 2) || (int_line >= 16));

    if (install_interrupt(int_line, can_isr))
    {
        printf("-->Error installing interrupt \n\n");
        return (-1);
    }
    else
        printf("-->Success installing interrupt \n\n");
    enable_interrupt();
    return (0);
}

/**********************************************************
    Release interrupt handling ressources
**********************************************************/
void
ExitInterruptHandling16(void)
{
    disable_interrupt();
    deinstall_interrupt();
}
#endif //16 interrupt
