/* PROJECT_NAME         Example                                   
   MODULE               STATOBUF.C                                      
   COMPANY              SOFTING AG Muenchen                                
   VERSION              $Id: Statobuf.c,v 1.1 2006/09/07 12:36:42 kno Exp $
*/


#if defined (WIN32)
#include <windows.h>
#endif
#include <stdlib.h>
#include <string.h>
#ifndef linux
#include <conio.h>
#endif
#include <stdio.h>

/* CAN LAYER 2 LIBRARY INCLUDE FILES */
#include "can_def.h"
#include "canlay2.h"

static INT32 Xtd;
static INT32 read_enabled;

INT32
InitializeStatObjBuf(void)
{
    INT32 ReceiveFifoEnable;
    INT32 ReceivePollAll;
    INT32 ReceiveEnableAll;
    INT32 ReceiveIntEnableAll;
    INT32 AutoRemoteEnableAll;
    INT32 TransmitReqFifoEnable;
    INT32 TransmitPollAll;
    INT32 TransmitAckEnableAll;
    INT32 TransmitAckFifoEnableAll;
    INT32 TransmitRmtFifoEnableAll;


    // Configuration of the object buffer
    ReceiveFifoEnable = 1;
    ReceivePollAll = 0;
    ReceiveEnableAll = 1;
    ReceiveIntEnableAll = 1;
    AutoRemoteEnableAll = 1;
    TransmitReqFifoEnable = 1;
    TransmitPollAll = 0;
    TransmitAckEnableAll = 1;
    TransmitAckFifoEnableAll = 1;
    TransmitRmtFifoEnableAll = 1;
    if (CANPC_initialize_interface(ReceiveFifoEnable,
                                   ReceivePollAll,
                                   ReceiveEnableAll,
                                   ReceiveIntEnableAll,
                                   AutoRemoteEnableAll,
                                   TransmitReqFifoEnable,
                                   TransmitPollAll,
                                   TransmitAckEnableAll,
                                   TransmitAckFifoEnableAll,
                                   TransmitRmtFifoEnableAll) != 0)
    {
        printf("-->Error in CANPC_initialize_interface \n\n");
        return (-1);
    }




/* The definition of the transmit and receive objects is not necessary,
 since all identifiers with their objects are present in the static object
 buffer.
 Using the CANPC_define_object in static object buffer mode provides 
 an individuell configuration of several objects (see manual). 
 It is not shown here, but can be studied in the dynamic object buffer definition.
  
*/

    if (CANPC_enable_fifo_transmit_ack2() != 0)
    {
        printf("-->Error in CANPC_enable_fifo_transmit_ack2 \n\n");
        return (-1);
    }


    Xtd = 0;
    read_enabled = 1;
    return (0);
}



INT32
UserRequestStatObjBuf(unsigned char custom)
{
    unsigned char p_RCV_data[8];
    unsigned char p_XMT_data[8];

    static USIGN32 Identifier = 0;
    static INT32 Objectnumber = 0;
    INT32 DataLength;
    INT32 Length;
    INT32 i, frc;
    static USIGN32 Time;


    switch (custom)
    {
    case 'h':                  // Output of the help menu
        printf("\n");
        printf("                   COMMUNICATION KEYS\n\n");
        printf("          CAN 1                              CAN2\n");
        printf("s: supply and send object can1        r: transmit data frame on can2\n");
        printf("z: send remote object can1            f: transmit remote frame on can2\n");
        printf("v: supply transmit object can1\n");
        printf("n: supply receive object can1\n");
        printf("t: send object can1\n");
        printf("b: read transmit object can1\n");
        printf("e: read receive object can1\n");

        printf("\n\n               ADMINISTRATION KEYS\n\n");

        printf("G: get fifo levels                   k: reset fifos\n");
        printf("B: get bus state can1                N: get bus state can2\n");
        printf("h: help menu                         d: disable/enable CANPC_read()\n");
        printf("i: reinitialize                      q: quit\n");
        break;

    case 't':                  // Send object on CAN 1
      Objectnumber = (Objectnumber+1) & 0x7;
        for (i = 0; i <= 7; i++)
            p_XMT_data[i] = (unsigned char) (Objectnumber + i);
        DataLength = (INT32) (8 - Objectnumber);

        frc = CANPC_send_object(Objectnumber, DataLength);
        printf("XMT     CAN1 Obj%7x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             Objectnumber, DataLength, p_XMT_data[0], p_XMT_data[1],
             p_XMT_data[2], p_XMT_data[3], p_XMT_data[4], p_XMT_data[5],
             p_XMT_data[6], p_XMT_data[7], frc);
        break;

    case 'r':                  // Send data frame on CAN 2
      Identifier = (Identifier+1) & 0x7;
        for (i = 0; i <= 7; i++)
            p_XMT_data[i] = (unsigned char) (Identifier + i);
        DataLength = (INT32) (8 - Identifier);
        frc = CANPC_send_data2(Identifier, Xtd, DataLength, p_XMT_data);
        printf("XMT     CAN2 Id%8x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             Identifier, DataLength, p_XMT_data[0], p_XMT_data[1],
             p_XMT_data[2], p_XMT_data[3], p_XMT_data[4], p_XMT_data[5],
             p_XMT_data[6], p_XMT_data[7], frc);
        break;


    case 'z':                  // Send  remote object on CAN 1
      Objectnumber = (Objectnumber+1) & 0x7;
        DataLength = 0;

        frc = CANPC_send_remote_object(Objectnumber, DataLength);
        printf("XMT     CAN1 Obj%7x Dlc%1x REMOTE                       Return Code%2d\n",
             Objectnumber + 100 * Xtd, DataLength, frc);
        break;

    case 'f':                  // Send remote frame on CAN 2
      Identifier = (Identifier+1) & 0x7L;
        DataLength = 0;
        frc = CANPC_send_remote2(Identifier, Xtd, DataLength);
        printf("XMT     CAN2 Id%8x Dlc%1x REMOTE                       Return Code%2d\n",
             Identifier, DataLength, frc);
        break;

    case 's':                  // Supply and send object on CAN 1
      Objectnumber = (Objectnumber+1) & 0x7;
        for (i = 0; i <= 7; i++)
            p_XMT_data[i] = (unsigned char) (Objectnumber + i);
        DataLength = (INT32) (8 - Objectnumber);
        frc = CANPC_write_object(Objectnumber, DataLength, p_XMT_data);
        printf("XMT     CAN1 Obj%7x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             Objectnumber + 100 * Xtd, DataLength, p_XMT_data[0],
             p_XMT_data[1], p_XMT_data[2], p_XMT_data[3], p_XMT_data[4],
             p_XMT_data[5], p_XMT_data[6], p_XMT_data[7], frc);
        break;




    case 'v':                  // Supply transmit object CAN 1
      Objectnumber = (Objectnumber+1) & 0x7;
        for (i = 0; i <= 7; i++)
            p_XMT_data[i] = (unsigned char) (Objectnumber + i);
        DataLength = (INT32) (8 - Objectnumber);
        frc = CANPC_supply_object_data(Objectnumber, DataLength, p_XMT_data);
        printf("XMT SPL CAN1 Obj%7x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             Objectnumber + 100 * Xtd, DataLength, p_XMT_data[0],
             p_XMT_data[1], p_XMT_data[2], p_XMT_data[3], p_XMT_data[4],
             p_XMT_data[5], p_XMT_data[6], p_XMT_data[7], frc);
        break;



    case 'n':                  // Supply receive object CAN 1
      Objectnumber = (Objectnumber+1) & 0x7;
        for (i = 0; i <= 7; i++)
            p_XMT_data[i] = (unsigned char) (Objectnumber + i);
        DataLength = (INT32) (8 - Objectnumber);
        frc =
            CANPC_supply_rcv_object_data(Objectnumber, DataLength,
                                         p_XMT_data);
        printf("RCV SPL CAN1 Obj%7x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             Objectnumber + 100 * Xtd, DataLength, p_XMT_data[0],
             p_XMT_data[1], p_XMT_data[2], p_XMT_data[3], p_XMT_data[4],
             p_XMT_data[5], p_XMT_data[6], p_XMT_data[7], frc);
        break;




    case 'b':                  // Read data of transmit object of CAN 1
      Objectnumber = (Objectnumber+1) & 0x7;
        frc = CANPC_read_xmt_data(Objectnumber, &Length, p_RCV_data);
        printf("XMT RD  CAN1 Obj%7x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             Objectnumber + 100 * Xtd, Length, p_RCV_data[0], p_RCV_data[1],
             p_RCV_data[2], p_RCV_data[3], p_RCV_data[4], p_RCV_data[5],
             p_RCV_data[6], p_RCV_data[7], frc);
        break;



    case 'e':                  // Read data of receive object of CAN 1
      Objectnumber = (Objectnumber+1) & 0x7;
        frc = CANPC_read_rcv_data(Objectnumber, p_RCV_data, &Time);
        printf("RCV RD  CAN1 Obj%7x Dlc? Data %2x %2x %2x %2x %2x %2x %2x %2x Return Code%2d\n",
             Objectnumber + 100 * Xtd, p_RCV_data[0], p_RCV_data[1],
             p_RCV_data[2], p_RCV_data[3], p_RCV_data[4], p_RCV_data[5],
             p_RCV_data[6], p_RCV_data[7], frc);
        break;


    case 'k':                  // Reset transmit and receive FIFO
        CANPC_reset_rcv_fifo();
        CANPC_reset_xmt_fifo();
        break;


    case 'L':                  // Reset the lost message counter
        CANPC_reset_lost_msg_counter();
        break;


    case 'G':                  // Read levels of transmit and receive fifos
        printf("rcv fifo level:%d xmt fifo level:%d\n",
               CANPC_read_rcv_fifo_level(), CANPC_read_xmt_fifo_level());
        break;


    case 'B':
        printf("bus state CAN1:%d \n", CANPC_get_bus_state(1));
        break;

    case 'N':
        printf("bus state CAN2:%d \n", CANPC_get_bus_state(2));
        break;

    case 'd':
        read_enabled = 1 - read_enabled;
        if (read_enabled)
            printf("CANPC_read_ac() enabled\n");
        else
            printf("CANPC_read_ac() disabled\n");
        break;


    default:
        break;
    }

    return (0);
}


INT32
ReadBusEventStatObjBuf(void)
{
    param_struct param;
    INT32 frc;

    if (read_enabled)
    {
        do
        {
            switch (frc = CANPC_read_ac(&param))
            {
            case 1:
                printf("RCV STD CAN%1u Id%8x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x\n",
                     param.Can, param.Ident, param.DataLength,
                     param.RCV_data[0], param.RCV_data[1], param.RCV_data[2],
                     param.RCV_data[3], param.RCV_data[4], param.RCV_data[5],
                     param.RCV_data[6], param.RCV_data[7]);
                break;
            case 2:
                printf("RCV STD CAN%1u Id%8x Dlc%1x REMOTE\n",
                       param.Can, param.Ident, param.DataLength);
                break;
            case 9:
                printf("RCV XTD CAN%1u Id%8x Dlc%1x Data %2x %2x %2x %2x %2x %2x %2x %2x\n",
                     param.Can, param.Ident, param.DataLength,
                     param.RCV_data[0], param.RCV_data[1], param.RCV_data[2],
                     param.RCV_data[3], param.RCV_data[4], param.RCV_data[5],
                     param.RCV_data[6], param.RCV_data[7]);
                break;
            case 12:
                printf("RCV XTD CAN%1u Id%8x Dlc%1x REMOTE\n",
                       param.Can, param.Ident, param.DataLength);
                break;
            case 3:
                printf("ACK STD CAN%1u Id%8x Dlc?\n",
                       param.Can, param.Ident);
                break;
            case 8:
                printf("ACK STD CAN%1u Id%8x Dlc? REMOTE\n",
                       param.Can, param.Ident);
                break;
            case 10:
                printf("ACK XTD CAN%1u Id%8x Dlc?\n",
                       param.Can, param.Ident);
                break;
            case 11:
                printf("ACK XTD CAN%1u Id%8x Dlc? REMOTE\n",
                       param.Can, param.Ident);
                break;
            case 5:
                printf("ERROR  CAN%1u Bus state: %1d\n",
                       param.Can, param.Bus_state);

                break;
            default:
                break;
            }
        }
        while (frc != 0);
    }                           // if (read_enabled)
    return (0);
}
