/*
 * Softing cantest program
 * Copyright (C) 1994-2006, Softing AG, Werner Knorsch
 * 
 * See file CREDITS for list of people who contributed to this
 * project.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * email: support.automation@softing.com
 * Softing AG, Richard-Reitzner-Allee 6, 85540 Haar, Germany
 */

#include <stdio.h>
#include <pthread.h>
#include <errno.h>
#include <sys/poll.h>
#include <stdlib.h>

#include <string.h>
#include <unistd.h>

#include "BaseTypes.h"
#include "can_def.h"
#include "canL2.h"

// output buffer length
#define OUTPUT_BUFFER_LEN 25

// some events used in the interrupt thread
HANDLE hThreadEvents[4];
// thread handles
HANDLE hIntThread[2];
// thread id's
DWORD tid1,tid2;
// number of plugged channels
int ch=0;

// for time stamp calculation ...
USIGN32 Time, diff_time, old_time;

USIGN32   Identifier = 0;   // Identifier 


pthread_t interrupt_pthread1;
pthread_t interrupt_pthread2;

int stop_thread1 = FALSE;
int stop_thread2 = FALSE;


char *FormatData(char *buffer, unsigned char *pData, long dataLen)
{
  int i;

  memset(buffer,0,25);

  for(i=0; i<8; i++)
  {
    if(i<dataLen)
	{
      sprintf(&buffer[i*3], "%3x",pData[i]);
	}
	else
	{
      strcat(buffer,"   ");
	}

  }

  return buffer;
}

/***********************************************************
  ReadBusEventFIFO
  Reads the event data of a received event
************************************************************/
int ReadBusEvent(CAN_HANDLE can, int ch, unsigned long *diff_time, unsigned long *old_time)
{
  PARAM_STRUCT 	param;
  int				    frc     = CANL2_RA_NO_DATA;
  char buffer[25];
  
  param.DataLength = 3;
  
  switch (frc=CANL2_read_ac( can, &param ))
  {
  case CANL2_RA_DATAFRAME:
    
    
    *diff_time =  param.Time - *old_time;
        printf("RCV STD CAN%u Id%8x Dlc%u Data%s T%8x D%10lu\n",
      ch, 
      param.Ident, 
      param.DataLength,
      FormatData(buffer, param.RCV_data, param.DataLength),
      param.Time, 
      *diff_time);
    *old_time = param.Time;
    
    break;
  case CANL2_RA_REMOTEFRAME:
    *diff_time =  param.Time - *old_time;
        printf("RCV STD CAN%u Id%8x Dlc%u REMOTE                       T%8x D%10lu\n",
      ch, param.Ident, param.DataLength,
      param.Time, *diff_time);
    *old_time = param.Time;
    break;
  case CANL2_RA_XTD_DATAFRAME:// not valid with AC2 with 82C200
    *diff_time =  param.Time - *old_time;
          printf("RCV XTD CAN%u Id%8x Dlc%u Data%s T%8x D%10lu\n",
      ch, 
      param.Ident, 
      param.DataLength,
      FormatData(buffer, param.RCV_data, param.DataLength),
      param.Time, 
      *diff_time);
    *old_time = param.Time;
    break;
  case CANL2_RA_XTD_REMOTEFRAME:// not valid with AC2 with 82C200
    *diff_time =  param.Time - *old_time;
            printf("RCV XTD CAN%u Id%8x Dlc%u REMOTE                       T%8x D%10lu\n",
      ch, param.Ident, param.DataLength,
      param.Time, *diff_time);
    *old_time = param.Time;
    break;
  case CANL2_RA_TX_DATAFRAME:
    *diff_time =  param.Time - *old_time;
    
        printf("ACK STD CAN%u Id%8x Dlc%u Data%s T%8x D%10lu\n",
      ch, 
      param.Ident, 
      param.DataLength,
      FormatData(buffer, param.RCV_data, param.DataLength),
      param.Time, 
      *diff_time);
    *old_time = param.Time;
    break;
  case CANL2_RA_TX_REMOTEFRAME:
    *diff_time =  param.Time - *old_time;
        printf("ACK STD CAN%u Id%8x Dlc%u REMOTE                       T%8x D%10lu\n",
      ch, param.Ident, param.DataLength,
      param.Time, *diff_time);
    *old_time = param.Time;
    break;
  case CANL2_RA_XTD_TX_DATAFRAME:// not valid with AC2 with 82C200
    *diff_time =  param.Time - *old_time;
          printf("ACK XTD CAN%u Id%8x Dlc%u Data%s T%8x D%10lu\n",
      ch, 
      param.Ident, 
      param.DataLength,
      FormatData(buffer, param.RCV_data, param.DataLength),
      param.Time, 
      *diff_time);
    *old_time = param.Time;
    break;
  case CANL2_RA_XTD_TX_REMOTEFRAME:// not valid with AC2 with 82C200
    *diff_time =  param.Time - *old_time;
            printf("ACK XTD CAN%u Id%8x Dlc%u REMOTE                       T%8x D%10lu\n",
      ch, param.Ident, param.DataLength,
      param.Time, *diff_time);
    *old_time = param.Time;
    break;
  case CANL2_RA_CHG_BUS_STATE:
    *diff_time =  param.Time - *old_time;
        printf( "ERROR  CAN%u Bus state: %1d                      T%8x D%10lu\n",
      ch, param.Bus_state,
      param.Time, *diff_time);
    *old_time = param.Time;
    break;
  default:
    break;
  }
  return(frc);
}


/**********************************************************
  Interrupt service thread 1 for the first channel
**********************************************************/
int interruptThread1(void *context)
{
  
  unsigned long diff_time=0, old_time=0;
  CAN_HANDLE can_channel = *(CAN_HANDLE *)context;
  int ret,RetCode;
  
  struct pollfd can_poll;
  
  while(!stop_thread1)
  {
    // in this implementation the CAN handle is the same as the file descriptor
    // for driver access; Please use the following macro CANL2_handle_to_descriptor 
    // for compatibility with later versions of this API
    
    can_poll.fd = CANL2_handle_to_descriptor(can_channel);
    can_poll.events = POLLIN | POLLHUP;
    
    do
    {
      ret = poll(&can_poll, 1, -1);
      //  printf("interruptThread1: poll returned %u, events=0x%x. revents=0x%x, POLLHUP=0x%x\n",ret,can_poll.events,can_poll.revents,POLLHUP);
      
      if(can_poll.revents & POLLHUP)
      {
        printf("terminating interruptThread1()\n");
        exit(0);
        return 0;
      }
      
      
      
      if ((ret == -1) && (errno != EINTR))
      {
        printf("Error: poll \"\" failed: (%d) %s\n",
        errno, strerror(errno));
      }
    } while (ret <= 0);
    
    
    do
    {
      RetCode = ReadBusEvent(can_channel, 1, &diff_time, &old_time);
      if(stop_thread1) return 0;
    }
    while(RetCode > 0);
    
    if(RetCode<0)
      stop_thread1 = TRUE;
    
  } // while(!end)
  
  printf("terminating interruptThread1\n");
  return 0;
}
/**********************************************************
  Interrupt service thread 2 for the second channel
**********************************************************/
int interruptThread2(LPVOID context)
{

  unsigned long diff_time=0, old_time=0;
  CAN_HANDLE can_channel = *(CAN_HANDLE *)context;
 // char buf[4096];
  int ret, RetCode;

  struct pollfd can_poll;

  while(!stop_thread2)
  {
    // in this implementation the CAN handle is the same as the file descriptor
    // for driver access; Please use the following macro CANL2_handle_to_descriptor 
    // for compatibility with later versions of this API
  
    can_poll.fd = CANL2_handle_to_descriptor(can_channel);
    can_poll.events = POLLIN | POLLHUP;
  
    do
  {
      ret = poll(&can_poll, 1, -1);
    //  printf("interruptThread2: poll returned %u, events=0x%x. revents=0x%x, POLLHUP=0x%x\n",ret,can_poll.events,can_poll.revents,POLLHUP);
  
      if(can_poll.revents & POLLHUP)
  {
        printf("terminating interruptThread2\n");
        return 0;
  }
  
      
      if (ret == -1 && errno != EINTR)
      {
        printf("Error: poll \"\" failed: (%d) %s\n",
               errno, strerror(errno));
  }
    } while (ret <= 0);
  
      
    
  
    do
  {
      RetCode = ReadBusEvent(can_channel, 2, &diff_time, &old_time);
      if(stop_thread2) return 0;
    }
    while(RetCode > 0);
    
    if(RetCode<0)
      stop_thread2 = TRUE;
      
  
  } // while(!end)
  
  printf("terminating interruptThread2\n");
  
  return 0;
}

// set up interrupt threads and events

// handle the input from the user
int UserRequestFIFO(char custom, CAN_HANDLE can1, CAN_HANDLE can2)		                                                              
{
  
  static unsigned char Xtd = 0;
  int       DataLength=3;   // Datalength of the data frames
  unsigned char	p_XMT_data[8];  // Array of data bytes to be sent
  int				frc;            // Function return code
  char buffer[25];
  int				k=0, i;			// Loop variables
  
  
  switch (custom)
  {
    case 'h': // Output of the help menu
      printf("\n               COMMUNICATION KEYS\n\n");
      printf("        CAN 1                     CAN2\n");
      printf("t: transmit data can1         z: transmit data can2\n");
      printf("r: transmit remote can1       f: transmit remote can2\n");
      printf("\n    p: toggle identifier type standard/extended\n");
      printf("\n\n               ADMINISTRATION KEYS\n\n");
    
      printf("l: get time                   k: reset fifos\n");
      printf("G: get fifo levels            L: reset lost msg counters\n");
      printf("B: get bus state can1         N: get bus state can2\n");
      printf("h: help menu                  q: quit\n\n");
      break;
    
    case 't':  // Send data frame on CAN 1
  
      Identifier = (Identifier > 7) ? 0 : Identifier+1;
      for (i=0; i<=7; i++)	p_XMT_data[i] = (unsigned char)(Identifier + i + k++);
      DataLength = (int)(8 - Identifier);
  
      frc = CANL2_send_data(can1, Identifier, Xtd, DataLength, p_XMT_data);
      printf("XMT     CAN%u Id%8x Dlc%u Data%s return Code%2d\n",
              1,
              Identifier, 
              DataLength,
              FormatData(buffer, p_XMT_data, DataLength),
                        frc);
      break;
  

    case 'r':  // Send remote frame on CAN 1
      Identifier = (Identifier > 7) ? 0 : Identifier+1;
      DataLength = (DataLength+1) & 0x7;
  
      frc = CANL2_send_remote(can1, Identifier, Xtd, DataLength);
      printf("XMT     CAN%u Id%8x Dlc%u REMOTE                      return Code%2d\n",
              1,Identifier, DataLength, frc);
      break;


    case 'z':  // Send data frame on CAN 2

      if(ch > 1)
      {
        Identifier = (Identifier > 7) ? 0 : Identifier+1;
        for (i=0; i<=7; i++)	p_XMT_data[i] = (unsigned char)(Identifier + i + k++);
        DataLength = (int)(8 - Identifier);
        frc = CANL2_send_data(can2, Identifier, Xtd, DataLength, p_XMT_data);
        printf("XMT     CAN%u Id%8x Dlc%u Data%s return Code%2d\n",
                2,
                Identifier, 
                DataLength,
                FormatData(buffer, p_XMT_data, DataLength), 
                          frc);
      }
      else
      {
        printf("The second channel is not available!\n");
      }

      break;

    case 'f':   // Send remote frame on CAN 2
      if(ch > 1)
      {
        Identifier = (Identifier > 7) ? 0 : Identifier+1;
        DataLength = (DataLength+1) & 0x7;
        frc = CANL2_send_remote(can2, Identifier, Xtd, DataLength);
        printf("XMT     CAN%u Id%8x Dlc%u REMOTE                      return Code%2d\n",
                2, Identifier, DataLength, frc);
      }
      else
      {
        printf("The second channel is not available!\n");
      }
      break;                  


    case 'l':
      CANL2_get_time(can1, &Time);
      diff_time = Time - old_time;
      printf("time:                                                    T%8x D%10u\n",
             Time, diff_time);
      old_time = Time;
      break;
    
    case 'k':
      CANL2_reset_rcv_fifo(can1);
      CANL2_reset_xmt_fifo(can1);
      if(ch > 1)
      {
        CANL2_reset_rcv_fifo(can2);
        CANL2_reset_xmt_fifo(can2);
      }
      printf("receive fifos are empty now!\n");
      break;

    case 'L':

      CANL2_reset_lost_msg_counter(can1);
      if(ch > 1)
      {
        CANL2_reset_lost_msg_counter(can2);
        printf("lost message counters cleared\n");
      }
      else
      {
        printf("lost message counter is cleared!\n");
      }

      break;

      case 'G': // Read transmit and receive fifos and the lost messages counter
        printf("CAN%u rcv fifo level:%d xmt fifo level:%d\n",
               1,
               CANL2_read_rcv_fifo_level(can1), 
                                         CANL2_read_xmt_fifo_level(can1));

        if(ch > 1)
        {
          printf("CAN%u rcv fifo level:%d xmt fifo level:%d\n",
                 2,
                 CANL2_read_rcv_fifo_level(can2), 
                                           CANL2_read_xmt_fifo_level(can2));
        }


        break;

  
  
    case 'B':
      printf("bus state CAN%u:%d \n",
             1, CANL2_get_bus_state(can1));
      break;
    
    case 'N':
      if(ch > 1)
      {
        printf("bus state CAN%u:%d \n",
               2, CANL2_get_bus_state(can2));
      }
      else
      {
        printf("The second channel is not available!\n");
      }
      break;
    
    case 'p':
      Xtd=1-Xtd;
      if (Xtd) printf("Extented identifier enabled\n");
      else printf("Standard identifier enabled\n");
      break;
    
    default:  
      break;
  }
  
  return(0);
}

// gets the device name from the device type
char *getDeviceType(int u32DeviceType)
{
  static char u8Type[100];

  switch(u32DeviceType)
  {
    case CANCARD2:
      strcpy(u8Type,"CANcard2");
      break;

    case CANPCI:
      strcpy(u8Type,"CAN-ACx-PCI");
      break;

    case CANPCIDN:
      strcpy(u8Type,"CAN-ACx-PCI/DN");
      break;

    case CAN_104:
      strcpy(u8Type,"CAN-ACx-104");
      break;

    case CANUSB:
      strcpy(u8Type,"CANusb");
      break;

    case EDICCARD2:
      strcpy(u8Type,"EDICcard2");
      break;

    case CANPROPCIE:
      strcpy(u8Type,"CAN-PROx-PCIe");
      break;

    case CAN104PLUS:
      strcpy(u8Type,"CAN-PROx-PC104+");
      break;

    case CAN104ECO:
      strcpy(u8Type,"CAN-ECOx-104");
      break;

    case GEFANUCPC8:
      strcpy(u8Type,"PC8 onboard CAN");
      break;

    default:
      strcpy(u8Type,"UNKNOWN");
      break;

  }
  return u8Type;
}

/**********************************************************
    Initialize interrupt handling
**********************************************************/
int init_pthread(CAN_HANDLE *can1, CAN_HANDLE *can2)
{
  int ret;

  if ((ret =
       pthread_create(&interrupt_pthread1, 0, (void *) interruptThread1, can1)))
  {
    return (-1);
  }
  
  if(ch>1)
  { // 2 channnels available
    if ((ret =
         pthread_create(&interrupt_pthread2, 0, (void *) interruptThread2, can2)))
    {
      return (-1);
    }
  
  }
  
  
  return (0);
}

/**********************************************************
    Release interrupt handling ressources
**********************************************************/

void exit_pthread(CAN_HANDLE *can1, CAN_HANDLE *can2)
{
  void *pRet;

  stop_thread1 = TRUE;  
  CANL2_reset_chip(*can1);
  pthread_join(interrupt_pthread1, &pRet);
  pthread_detach(interrupt_pthread1);


  stop_thread2 = TRUE;
  if(ch>1)
  { // 2 channnels available
    CANL2_reset_chip(*can2);
    pthread_join(interrupt_pthread2, &pRet);
    pthread_detach(interrupt_pthread2);
  }

    

}


int main()
{
  int ret;
  CANL2_CH_STRUCT ch1, ch2;
  CAN_HANDLE can1, can2;
  L2CONFIG L2Config[2];
  char text[100];
  char custom;

  char *pName[2] = {NULL,NULL};
  PCHDSNAPSHOT pBuffer = NULL;
  USIGN32 u32NeededBufferSize, u32NumOfChannels, u32ProvidedBufferSize,i;


  
  // Introduction screen
  printf("\n");
  printf("***************************************************************\n");
  printf("*                        SOFTING AG 1998-2009                 *\n");
  printf("*                                                             *\n");
  printf("*                                                             *\n");
  printf("*                           Testprogram                       *\n");
  printf("*                           for CAN API                       *\n");
  printf("*                              V5.13                          *\n");
  printf("*                                                             *\n");
  printf("*                         Type 'h' for help                   *\n");
  printf("*                                                             *\n");
  printf("***************************************************************\n");
  printf("\n");

  printf ("Analyzing your system ...\n");

  u32ProvidedBufferSize = 0;

  // call the function without a valid buffer size first to get the needed buffersize in "u32NeededBufferSize"
  ret = CANL2_get_all_CAN_channels(0, &u32NeededBufferSize, &u32NumOfChannels, NULL);

  if(!u32NumOfChannels)
  {
    printf("you have no Softing CAN interface card plugged in your Computer!\n");
    printf("plug a interface card first and start this program again after this.\n");
    custom=getchar();
    exit(0);
  }

  if(ret)
  {
    printf("The driver reported a problem: Error Code %x\n", ret);
    custom=getchar();
    exit(0);
  }
  
  
  pBuffer = malloc(u32NeededBufferSize);

  if(!pBuffer)
  {
    printf("Out of memory\n");
    custom=getchar();
    exit(0);
  }

  u32ProvidedBufferSize = u32NeededBufferSize;

    
  ret = CANL2_get_all_CAN_channels(u32ProvidedBufferSize, &u32NeededBufferSize, &u32NumOfChannels, pBuffer);

  if(ret)
  {
    printf("The driver reported a problem: Error Code %x\n", ret);
    custom=getchar();
    free(pBuffer);
    exit(0);
  }

  printf("You have %u Softing CAN channels in your system\n\n", u32NumOfChannels);

  printf("\tname\t\t serialnumber\t type\t chan.\t    open\n");
  printf("--------------------------------------------------------------------\n");
  printf("\n");

  for(i=0; i<u32NumOfChannels; i++)
  {
    PCHDSNAPSHOT pCh = &pBuffer[i];
    

    printf("%26s %12u  %13s %5u %5s\n",
      pCh->ChannelName,
      pCh->u32Serial,
      getDeviceType(pCh->u32DeviceType),
      pCh->u32PhysCh,
      (pCh->bIsOpen) ? "yes" : "no");

    if(!pCh->bIsOpen)
    {
      if(ch<2)
      {
        pName[ch] = (char*)pCh->ChannelName;
        ch++;
      }
    }

  }
  
  if(ch < 2)
  {
    if(ch == 0)
    {
      printf("\nAll CAN channels are used by other applications!\n");
      printf("Type <return> to exit program!\n");
      custom=getchar();
      free(pBuffer);
      exit(0);
    }

    printf("\nYou have only 1 channel plugged, the commands for the second channel will not \n");
    printf("be available!\n");
    printf("Type <return> to proceed!\n");
    custom=getchar();
  }


  
  
  L2Config[0].bEnableAck = GET_FROM_SCIM;
  L2Config[0].bEnableErrorframe = GET_FROM_SCIM;
  L2Config[0].s32AccCodeStd = GET_FROM_SCIM;
  L2Config[0].s32AccCodeXtd = GET_FROM_SCIM;
  L2Config[0].s32AccMaskStd = GET_FROM_SCIM;
  L2Config[0].s32AccMaskXtd = GET_FROM_SCIM;
  L2Config[0].s32OutputCtrl = GET_FROM_SCIM;
  L2Config[0].s32Prescaler = GET_FROM_SCIM;
  L2Config[0].s32Sam = GET_FROM_SCIM;
  L2Config[0].s32Sjw = GET_FROM_SCIM;
  L2Config[0].s32Tseg1 = GET_FROM_SCIM;
  L2Config[0].s32Tseg2 = GET_FROM_SCIM;


 // 1 MBaud settings;
 // "hard programmed values" this is an example if no cfg file was built by SCIM
  /*
     L2Config[0].bEnableAck = 1;
  L2Config[0].bEnableErrorframe = 0;
  L2Config[0].s32AccCodeStd = 0;
  L2Config[0].s32AccCodeXtd = 0;
  L2Config[0].s32AccMaskStd = 0;
  L2Config[0].s32AccMaskXtd = 0;
  L2Config[0].s32OutputCtrl = 0xFA;
  L2Config[0].s32Prescaler = 1;
  L2Config[0].s32Sam = 0;
  L2Config[0].s32Sjw = 2;
  L2Config[0].s32Tseg1 = 4;
  L2Config[0].s32Tseg2 = 3;
  */
  
  
  
  
  

  
  if(ch > 1)
  { // 2 channnels available
    memcpy(&L2Config[1], &L2Config[0], sizeof(L2CONFIG));
    strcpy((char*)ch1.sChannelName, pName[0]);
    strcpy((char*)ch2.sChannelName, pName[1]);    
    printf("\ninitializing %s and %s\n", ch1.sChannelName, ch2.sChannelName);
  }
  else
  {
    strcpy((char*)ch1.sChannelName, pName[0]);
    printf("\ninitializing %s\n", ch1.sChannelName);
  }


  free(pBuffer);
  
  /*
  strcpy((char*)ch1.sChannelName,"Softing1");
  strcpy((char*)ch2.sChannelName,"Softing2");
  */



  ret = INIL2_initialize_channel(&ch1.ulChannelHandle, (char*) ch1.sChannelName);

  if(ret)
  {
    printf("Error %i in INIL2_initialize_channel()\n",ret);
    return ret;
  }

  can1 = ch1.ulChannelHandle;

  if(ch > 1)
  { // 2 channnels available
    
    ret = INIL2_initialize_channel(&ch2.ulChannelHandle, (char*)ch2.sChannelName);
    
    if(ret)
    {
      printf("Error %i in INIL2_initialize_channel()\n",ret);
      return ret;
    }
    
    can2 = ch2.ulChannelHandle;
  }
  else
  {
    can2 = (CAN_HANDLE)0;
  }



  ret = CANL2_initialize_fifo_mode(can1, &L2Config[0]);
  if(ret)
  {
    printf("Error %i in CANL2_initialize_fifo_mode()\n",ret);
    if(ret==258)
    {
      printf("configure the channel by the SCIM first\n");
      // use start->control panel->can for configuration of the card settings!
    }
    return ret;
  }

  if(ch > 1)
  { // 2 channnels available
    ret = CANL2_initialize_fifo_mode(can2, &L2Config[1]);
    if(ret)
    {
      printf("Error %i in CANL2_initialize_fifo_mode()\n",ret);
      if(ret==258)
      {
        printf("configure the channel by the SCIM first\n");
      // use start->control panel->can for configuration of the card settings!
        return ret;
      }
    }
    
  }

  ret = init_pthread(&can1, &can2);

  if(ret)
  {
    printf("Error %i in init_pthread()\n",ret);
    return ret;
  }
    
  printf("The channels are online now!\n\n");
  custom = 'h';
  do	// loop
  {
    // User request	
    ret=UserRequestFIFO(custom, can1, can2);	
      
    if (ret < 0)	
    {
      printf("-->User request failed \n");

      exit_pthread(&can1, &can2);
      INIL2_close_channel(can1);
      if(ch>1)
      {
        INIL2_close_channel(can2);
      }
      return(-1);
    }


    scanf("%s",text);
    custom = text[0];

  }
  while (custom != 'q');  // End inner loop

  //sleep(30);
  
  exit_pthread(&can1, &can2);

  INIL2_close_channel(can1);
  if(ch>1)
  { // 2 channnels available
    INIL2_close_channel(can2);
  }
  
  return ret;
    
}



