/***************************************************************************
 *   Copyright (C) 2007-2015 by Softing Automation GmbH                    *
 *   support.automation@softing.com                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "ch_cfg.h"

/**
 ******************************************************************
 * @brief  Constructor.
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @param  parent: parent Widget
 * @param  name:   name
 * @return -
 *
 ******************************************************************
 */

ch_cfg::ch_cfg(CHANNEL *pChannel)
    :QObject()
{
  // ok button was not pressed.
  m_ok = false;
  
  m_cfg.setupUi(&m_w);
  
  
  connect(m_cfg.buttonOk, SIGNAL(clicked()), this, SLOT(on_ok()));
  connect(m_cfg.buttonCancel, SIGNAL(clicked()), this, SLOT(on_cancel()));
  connect(m_cfg.ed_tseg1, SIGNAL(lostFocus()), this, SLOT(on_tseg1()));
  connect(m_cfg.ed_baudrate, SIGNAL(lostFocus()), this, SLOT(on_baudrate()));
  connect(m_cfg.ed_tseg2, SIGNAL(lostFocus()), this, SLOT(on_tseg2()));
  connect(m_cfg.ed_presc, SIGNAL(lostFocus()), this, SLOT(on_presc()));
  connect(m_cfg.ed_sjw, SIGNAL(lostFocus()), this, SLOT(on_sjw()));
  connect(m_cfg.ed_sam, SIGNAL(lostFocus()), this, SLOT(on_sam()));
  connect(m_cfg.ed_acc_mask, SIGNAL(lostFocus()), this, SLOT(on_acc_mask()));
  connect(m_cfg.ed_acc_code, SIGNAL(lostFocus()), this, SLOT(on_acc_code()));
  connect(m_cfg.ed_acc_xtd_mask, SIGNAL(lostFocus()), this, SLOT(on_acc_xtd_mask()));
  connect(m_cfg.ed_acc_xtd_code, SIGNAL(lostFocus()), this, SLOT(on_acc_xtd_code()));
  connect(m_cfg.ed_output_ctrl, SIGNAL(lostFocus()), this, SLOT(on_output_ctrl()));
  connect(m_cfg.buttonCancel, SIGNAL(clicked()), this, SLOT(on_cancel()));
  connect(m_cfg.buttonOk, SIGNAL(clicked()), this, SLOT(on_ok()));
  connect(m_cfg.ed_channelName, SIGNAL(lostFocus()), this, SLOT(on_channel_name()));
  
  connect(m_cfg.tab_baudrates, SIGNAL(cellClicked(int,int)), this, SLOT(on_cellClicked(int, int)));
  
  

  
  // build the controller settings table.
  
  if(pChannel)
  {
    m_cfg.ed_channelName->setText(pChannel->m_channelName);
    m_channelName = pChannel->m_channelName;
     // first write the valid values to the dialog
    m_fBaudrate = pChannel->m_fBaudrate;
    m_prescaler = pChannel->m_prescaler;
    m_sam = pChannel->m_sam;
    m_sjw = pChannel->m_sjw;
    m_tseg1 = pChannel->m_tseg1;
    m_tseg2 = pChannel->m_tseg2;
    m_u32AccCodeStd = pChannel->m_u32AccCodeStd;
    m_u32AccCodeXtd = pChannel->m_u32AccCodeXtd;
    m_u32AccMaskStd = pChannel->m_u32AccMaskStd;
    m_u32AccMaskXtd = pChannel->m_u32AccMaskXtd;
    m_u32OutputCtrl = pChannel->m_u32OutputCtrl;
    m_cfg.ch_enable_ack->setChecked(pChannel->m_enableAck);
    m_cfg.ch_enable_error_frame->setChecked(pChannel->m_detectErrorframes);
  }
  update_all();
  on_baudrate();
  
}

ch_cfg::~ch_cfg()
{
  deleteAllTabEntries();
}

void ch_cfg::deleteAllTabEntries(void)
{
  int numOfCol = getNumOfCol()-1;
  int i,j;
  QTableWidgetItem *pTableItem;
  
  for(i=numOfCol; i>=0; i--)
  {
    for(j=6; j>=0; j--)
    {
      pTableItem = m_cfg.tab_baudrates->item(i,j);
      delete pTableItem;
    }
  }
}

int ch_cfg::getNumOfCol(void)
{
  int i=0;
  while(m_cfg.tab_baudrates->item(i,0))
  {
    i++;
    if(i>1000)
      return 0;
  }
  return i;
}


void ch_cfg::show_config(void)
{
  QString s;
  
  s = QString("%1").arg(m_fBaudrate,4,'f',1);
  m_cfg.ed_baudrate->setText(s);
  s = QString("%1").arg(m_prescaler);
  m_cfg.ed_presc->setText(s);
  //     m_sam = m_config.m_Div8X/8;
  s = QString("%1").arg(m_sjw);
  m_cfg.ed_sjw->setText(s);
  m_cfg.ed_tseg1->setText(QString("%1").arg(m_tseg1));
  m_cfg.ed_tseg2->setText(QString("%1").arg(m_tseg2));
  m_cfg.ed_acc_code->setText    (QString("0x%1").arg(m_u32AccCodeStd,4,16,QLatin1Char('0')));
  m_cfg.ed_acc_xtd_mask->setText(QString("0x%1").arg(m_u32AccMaskStd,4,16,QLatin1Char('0')));
  
  m_cfg.ed_acc_xtd_code->setText(QString("0x%1").arg(m_u32AccCodeXtd,8,16,QLatin1Char('0')));
  m_cfg.ed_acc_xtd_mask->setText(QString("0x%1").arg(m_u32AccMaskXtd,8,16,QLatin1Char('0')));
  
  m_cfg.ed_output_ctrl->setText (QString("0x%1").arg(m_u32OutputCtrl,2,16,QLatin1Char('0')));
  
  update_all();
  on_baudrate();
  m_w.exec();
}

/**
 ******************************************************************
 * @brief  Shows the baudrate table
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @param  u32MaxEntry: maximum entries in the table
 * @return -
 *
 ******************************************************************
 */

void ch_cfg::PrintTable(USIGN32 u32MaxEntry)
{
  USIGN32 i;
  QString itemText;
  QTableWidgetItem *pTableItem;
  QStringList tableDescription;
  
  tableDescription << "Baudrate[KBaud]" << "Prescaler" << "TSEG1" << "TSEG2" << "Sjw" << "Sam" << "Sample Point[%]";

  m_cfg.tab_baudrates->setRowCount(u32MaxEntry);
  m_cfg.tab_baudrates->setColumnCount(7);
  
  m_cfg.tab_baudrates->setHorizontalHeaderLabels(tableDescription);
  
  for(i=0; i<u32MaxEntry; i++)
  {
    m_aModiTable[i].fBaudrate =
        16E3/(2*m_aModiTable[i].u8Prescaler*(1+m_aModiTable[i].u8Tseg1+m_aModiTable[i].u8Tseg2));
    
    itemText = itemText.sprintf("%.2f",m_aModiTable[i].fBaudrate);
    pTableItem = new QTableWidgetItem();
    
    pTableItem->setText(itemText);
    m_cfg.tab_baudrates->setItem(i,0,pTableItem);
    
    itemText = itemText.sprintf("%u",m_aModiTable[i].u8Prescaler);
    pTableItem = new QTableWidgetItem();
    pTableItem->setText(itemText);
    m_cfg.tab_baudrates->setItem(i,1,pTableItem);
    
    itemText = itemText.sprintf("%u",m_aModiTable[i].u8Tseg1);
    pTableItem = new QTableWidgetItem();
    pTableItem->setText(itemText);
    m_cfg.tab_baudrates->setItem(i,2,pTableItem);
    
    itemText = itemText.sprintf("%u",m_aModiTable[i].u8Tseg2);
    pTableItem = new QTableWidgetItem();
    pTableItem->setText(itemText);
    m_cfg.tab_baudrates->setItem(i,3,pTableItem);
    
    itemText = itemText.sprintf("%u",m_aModiTable[i].u8Sjw);
    pTableItem = new QTableWidgetItem();
    pTableItem->setText(itemText);
    m_cfg.tab_baudrates->setItem(i,4,pTableItem);
    
    itemText = itemText.sprintf("%u",m_aModiTable[i].u8Sam);
    pTableItem = new QTableWidgetItem();
    pTableItem->setText(itemText);
    m_cfg.tab_baudrates->setItem(i,5,pTableItem);
    
    itemText =
    itemText.sprintf(
        "%.2f",
        (1.0+m_aModiTable[i].u8Tseg1)*100.0/(1.0+m_aModiTable[i].u8Tseg1+m_aModiTable[i].u8Tseg2)
                    );
    pTableItem = new QTableWidgetItem();
    pTableItem->setText(itemText);
    m_cfg.tab_baudrates->setItem(i,6,pTableItem);
    
    
  }

}

/**
 ******************************************************************
 * @brief  calculates all possible CAN controller settings for a given sum of tseg1 and tseg2
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @param  Tseg1AndTseg2Int: tseg1+tseg2
 * @return -
 *
 ******************************************************************
 */

void ch_cfg::ShowAll(USIGN32 Tseg1AndTseg2Int)
{
  
  USIGN8 tseg1, tseg2 = 0, sjw, presc, sam, tseg12 = (USIGN8)Tseg1AndTseg2Int;
  USIGN32 u32Entry = 0;
  double fTseg;
  int baud1, baud2;

 // m_Baudrates.DeleteAllItems();
  tseg2 = (USIGN8)(Tseg1AndTseg2Int/2);
  presc = m_prescaler;

  do
  {
    do
    {
      

      if(tseg2 > 8) 
        tseg2 = 8;
      
      if(tseg2 < 1)
        tseg2 = 1;
      
      tseg1 = (USIGN8)(tseg12 - tseg2);
      
      
      if((tseg1 < 1) || (tseg1 > 16))
      { // to do ERROR handling!
        break;
      }
      
      baud1 = (int) (m_fBaudrate * 100000.00);
      baud2 = (int) (16E8/(2*presc*(1+tseg1+tseg2)));

      if(baud1 != baud2)
      {
        break;
      }
      /*
      if(m_fBaudrate != (16E3/(2*presc*(1+tseg1+tseg2))))
      {
        break;
      }
      */
           
      
      if(m_fBaudrate < 100)
        sam = 3;
      else
        sam = 1;
      
      
      for(sjw = 1; sjw<=4; sjw++)
      {
        if((sjw <= tseg2) && ((2*sjw) <= (tseg1+tseg2)))
        {
          m_aModiTable[u32Entry].u8Prescaler  = presc;
          m_aModiTable[u32Entry].u8Tseg1 = tseg1;
          m_aModiTable[u32Entry].u8Tseg2 = tseg2;
          m_aModiTable[u32Entry].u8Sjw   = sjw;
          m_aModiTable[u32Entry].u8Sam   = sam;
          
          u32Entry++;
          
          if(u32Entry >= MAX_MODUS)
          {
            PrintTable(u32Entry-1);
            return; // out of memory, no more modi possible;
          }
        }
      }
      
      tseg2--;
      
    } while(tseg2);

    presc++;
    fTseg = 16E3/(2*presc*m_fBaudrate);
    tseg12 = (USIGN8)(int)fTseg;
    tseg2 = (USIGN8)(tseg12/2);
  } while ((fTseg > 8.0) && (presc <= 32));

  PrintTable(u32Entry-1);

}


/**
 ******************************************************************
 * @brief  reads all text boxes into member variables
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

bool ch_cfg::read_all()
{
  bool isValid;
  
  m_fBaudrate = m_cfg.ed_baudrate->text().toDouble(&isValid);
  if(!isValid) return false;
  
  m_prescaler = m_cfg.ed_presc->text().toUInt(&isValid);
  if(!isValid) return false;
  m_tseg1 = m_cfg.ed_tseg1->text().toUInt(&isValid);
  if(!isValid) return false;
  m_tseg2 = m_cfg.ed_tseg2->text().toUInt(&isValid);
  if(!isValid) return false;
  m_sam = m_cfg.ed_sam->text().toUInt(&isValid);
  if(!isValid) return false;
  m_sjw = m_cfg.ed_sjw->text().toUInt(&isValid);
  
  
  return isValid;
  
}

/**
 ******************************************************************
 * @brief  writes the member variables into the textboxes
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg::update_all()
{
  QString text="1";
  QString str_tooltip = QString("possible CAN controller settings for %1 KBaud").arg(m_fBaudrate);
  
  m_cfg.tab_baudrates->setToolTip(str_tooltip);
  
  text = text.sprintf("%u",m_prescaler);
  m_cfg.ed_presc->setText(text);
  text = text.sprintf("%u",m_tseg1);
  m_cfg.ed_tseg1->setText(text);
  text = text.sprintf("%u",m_tseg2);
  m_cfg.ed_tseg2->setText(text);
  text = text.sprintf("%u",m_sam);
  m_cfg.ed_sam->setText(text);
  text = text.sprintf("%u",m_sjw);
  m_cfg.ed_sjw->setText(text);
  text = text.sprintf("%.2f",m_fBaudrate);
  m_cfg.ed_baudrate->setText(text);

  text = text.sprintf("0x%04X",m_u32AccCodeStd);
  m_cfg.ed_acc_code->setText(text);
  text = text.sprintf("0x%04X",m_u32AccMaskStd);
  m_cfg.ed_acc_mask->setText(text);

  text = text.sprintf("0x%08X",m_u32AccCodeXtd);
  m_cfg.ed_acc_xtd_code->setText(text);
  text = text.sprintf("0x%08X",m_u32AccMaskXtd);
  m_cfg.ed_acc_xtd_mask->setText(text);

  text = text.sprintf("0x%02X",m_u32OutputCtrl);
  m_cfg.ed_output_ctrl->setText(text);

 
  m_CTableText = m_CTableText.sprintf("Possible CAN controller settings for %.2f KBaud", m_fBaudrate);
  m_cfg.tableLabel->setText(m_CTableText);
   
}

/**
 ******************************************************************
 * @brief  called, when the user has changed the baudrate; updates the controller settings.
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg::on_baudrate()
{
  USIGN8 presc, tseg1, tseg2, sjw, sam=0;
  USIGN32 Tseg1AndTseg2Int;
  double samples, BitTime, Tseg1AndTseg2, fOldBaudrate;
  bool isValid;
  
  
  fOldBaudrate = m_fBaudrate;
  
  isValid = read_all();

  //UpdateData(TRUE);

  if((m_fBaudrate < 10.0) || (m_fBaudrate > 1000.0) || !isValid)
  {
    m_fBaudrate = fOldBaudrate;
    update_all();
    return;
  }

  BitTime = 1.0/(m_fBaudrate * 1000);
  samples = BitTime * 16E6;
  presc = (USIGN8)(int)((samples / 48) + 1.0);
  
  if((presc & 1) && (presc > 1))
    presc++;

  if(presc > 32)
    presc = 32;

  Tseg1AndTseg2 = 16E3/(2*presc*m_fBaudrate)-1;

  Tseg1AndTseg2Int = (int)Tseg1AndTseg2;


  tseg2 = (USIGN8)(Tseg1AndTseg2Int/2);

  if((USIGN8)Tseg1AndTseg2Int == 2*tseg2)
    tseg2--;

  if(tseg2 > 8) 
    tseg2 = 8;

  if(tseg2 < 1)
    tseg2 = 1;

  tseg1 = (USIGN8)(Tseg1AndTseg2Int - tseg2);

  if((tseg1 < 1) || (tseg1 > 16))
  { // to do ERROR handling!
    m_fBaudrate = 16E3/(2*m_prescaler*(1+m_tseg1+m_tseg2));
    m_CTableText = m_CTableText.sprintf("Possible CAN controller settings for %.2f KBaud", m_fBaudrate);
    update_all();
   // UpdateData(FALSE);
    return;
  }


  if(m_fBaudrate < 100.0)
    sam = 3;
  else
    sam = 1;


  sjw = (USIGN8)(Tseg1AndTseg2Int/8);

  if(sjw<1)
    sjw = 1;

  m_prescaler = presc;
  m_tseg1 = tseg1;
  m_tseg2 = tseg2;
  m_sam = sam;
  m_sjw = sjw;
  m_fBaudrate = 16E3/(2*m_prescaler*(1+m_tseg1+m_tseg2));
  m_CTableText = m_CTableText.sprintf("Possible CAN controller settings for %.2f KBaud", m_fBaudrate);

  update_all();

  ShowAll(Tseg1AndTseg2Int);

}

/**
 ******************************************************************
 * @brief  on change of tseg1; calculate new baudrate;
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg::on_tseg1()
{
  
  bool isValid;
  unsigned long oldTseg1 = m_tseg1;
  
  isValid = read_all();
  
  if((m_tseg1<1) || (m_tseg1 > 16) || !isValid)
  {
    m_tseg1 = oldTseg1;
    update_all();
    on_baudrate();
    return;
  }

  if(m_tseg1 < m_tseg2)
  {
    m_tseg1 = m_tseg2;
  }

  m_fBaudrate = 16E3/(2*m_prescaler*(1+m_tseg1+m_tseg2));
  m_CTableText = m_CTableText.sprintf("Possible CAN controller settings for %.2f KBaud", m_fBaudrate);
  update_all();
  on_baudrate();
}

/**
 ******************************************************************
 * @brief  on change of tseg2; calculate new baudrate;
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg::on_tseg2()
{
  bool isValid;
  unsigned long tseg2 = m_tseg2;
  
  isValid = read_all();
  if((m_tseg2<1) || (m_tseg2 > 8) || !isValid)
  {
    m_tseg2 = tseg2;
    update_all();
    on_baudrate();
    return;

  }

  m_fBaudrate = 16E3/(2*m_prescaler*(1+m_tseg1+m_tseg2));
  m_CTableText = m_CTableText.sprintf("Possible CAN controller settings for %.2f KBaud", m_fBaudrate);
  update_all();
  on_baudrate();
}

/**
 ******************************************************************
 * @brief  on change of prescaler; calculate new baudrate;
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg::on_presc()
{
  bool isValid;
  unsigned long presc = m_prescaler;
  
  isValid = read_all();
  

  if((m_prescaler<1) || (m_prescaler>32) || !isValid)
  {
    m_prescaler = presc;
    update_all();
    on_baudrate();
    return;

  }

  m_fBaudrate = 16E3/(2*m_prescaler*(1+m_tseg1+m_tseg2));
  m_CTableText = m_CTableText.sprintf("Possible CAN controller settings for %.2f KBaud", m_fBaudrate);
  update_all();
  on_baudrate();

}

/**
 ******************************************************************
 * @brief  on change of sjw; calculate new baudrate;
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg::on_sjw()
{
    
  unsigned long old_val = m_sjw;
  bool isValid;
  
  isValid = read_all();
  if(!isValid) return;
  
  if((m_sjw<1) || (m_sjw > 4))
  {
    m_sjw = old_val;
    update_all();
    return;

  }

}

/**
 ******************************************************************
 * @brief  on change of samples; calculate new baudrate;
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg::on_sam()
{
  unsigned long old_val = m_sam;
  bool isValid;
  
  isValid = read_all();
  
  if(m_sam>1 || !isValid)
  {
    m_sam = old_val;
    update_all();
    return;

  }

}

/**
 ******************************************************************
 * @brief  on change of acceptance mask; check, get and show the new value
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg::on_acc_mask()
{
  bool isValid;
  unsigned long val;
  QString text;
  
  val = m_cfg.ed_acc_mask->text().toULong(&isValid,16);
  
  if(isValid)
  {
    m_u32AccMaskStd = val;
  }
  
  text = text.sprintf("0x%04X",m_u32AccMaskStd);
  m_cfg.ed_acc_mask->setText(text);
}

/**
 ******************************************************************
 * @brief  on change of acceptance code; check, get and show the new value
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg::on_acc_code()
{
  bool isValid;
  unsigned long val;
  QString text;
  
  val = m_cfg.ed_acc_code->text().toULong(&isValid,16);
  
  if(isValid)
  {
    m_u32AccCodeStd = val;
  }
  
  text = text.sprintf("0x%04X",m_u32AccCodeStd);
  m_cfg.ed_acc_code->setText(text);
}

/**
 ******************************************************************
 * @brief  on change of extended acceptance mask; check, get and show the new value
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg::on_acc_xtd_mask()
{
  bool isValid;
  unsigned long val;
  QString text;
  
  val = m_cfg.ed_acc_xtd_mask->text().toULong(&isValid,16);
  
  if(isValid)
  {
    m_u32AccMaskXtd = val;
  }
  
  text = text.sprintf("0x%08X",m_u32AccMaskXtd);
  m_cfg.ed_acc_xtd_mask->setText(text);

}

/**
 ******************************************************************
 * @brief  on change of extended acceptance code; check, get and show the new value
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg::on_acc_xtd_code()
{
  bool isValid;
  unsigned long val;
  QString text;
  
  val = m_cfg.ed_acc_xtd_code->text().toULong(&isValid,16);
  
  if(isValid)
  {
    m_u32AccCodeXtd = val;
  }
  
  text = text.sprintf("0x%08X",m_u32AccCodeXtd);
  m_cfg.ed_acc_xtd_code->setText(text);

}

/**
 ******************************************************************
 * @brief  on change of output control register; check, get and show the new value
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg::on_output_ctrl()
{
  bool isValid;
  unsigned long val;
  QString text;
  
  val = m_cfg.ed_output_ctrl->text().toULong(&isValid,16);
  
  if(isValid)
  {
    m_u32OutputCtrl = val & 0xFF;
  }
  
  text = text.sprintf("0x%02X",m_u32OutputCtrl);
  m_cfg.ed_output_ctrl->setText(text);
}


/**
 ******************************************************************
 * @brief  called, when the user selects a row in the baudrate table; use the new controller settings
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg::on_cellClicked(int row, int col)
{ // ok it works!

  QTableWidgetSelectionRange col_range(row, 0, row, m_cfg.tab_baudrates->columnCount()-1);
  QTableWidgetSelectionRange tab_range(0, 0, m_cfg.tab_baudrates->rowCount()-1, m_cfg.tab_baudrates->columnCount()-1);
  
  QString rw = "Rx PDO";
  
  m_cfg.tab_baudrates->setRangeSelected(tab_range, false);
  m_cfg.tab_baudrates->setRangeSelected(col_range, true);
  m_cfg.tab_baudrates->setCurrentItem(m_cfg.tab_baudrates->item(row,col), QItemSelectionModel::Rows);
  
  if((row > 0) && (row < MAX_MODUS))
  {

    m_prescaler   = m_aModiTable[row].u8Prescaler;
    m_tseg1       = m_aModiTable[row].u8Tseg1;
    m_tseg2       = m_aModiTable[row].u8Tseg2;
    m_sam         = m_aModiTable[row].u8Sam;
    m_sjw         = m_aModiTable[row].u8Sjw;
    
    update_all();
  }
  
}

void ch_cfg::on_tab_selection_changed()
{
  QList<QTableWidgetItem *> list = m_cfg.tab_baudrates->selectedItems();
  
  if(list.count()>0)
  {
    on_cellClicked(list[0]->row(), 0);
  }
  
}

/**
 ******************************************************************
 * @brief  on cancel; signal this to the calling class in the public member variable m_ok
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg::on_cancel()
{
  m_ok = false;
  m_w.hide();
}


/**
 ******************************************************************
 * @brief  on ok; signal this to the calling class in the public member variable m_ok
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg::on_ok()
{
  m_ok = true;
  m_w.hide();
}

void ch_cfg::on_channel_name()
{
  m_channelName = m_cfg.ed_channelName->text();
}

bool ch_cfg::is_ack_enabled(void)
{
  return m_cfg.ch_enable_ack->isChecked();
}

void ch_cfg::set_ack_enabled(bool yesno)
{
  m_cfg.ch_enable_ack->setChecked(yesno);
}

bool ch_cfg::is_errorframe_detection_enabled(void)
{
  return m_cfg.ch_enable_error_frame->isChecked();
}

void ch_cfg::set_error_frame_detection_enabled(bool yesno)
{
  m_cfg.ch_enable_error_frame->setChecked(yesno);
}






















#include "ch_cfg.moc"
