/***************************************************************************
 *   Copyright (C) 2007-2015 by Softing Automation GmbH                    *
 *   support.automation@softing.com                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "ch_cfg_pro.h"



/**
 ******************************************************************
 * @brief  Constructor.
 * @author Werner Knorsch, Softing AG
 * @date   2008-01-29
 *
 * @param  parent: parent Widget
 * @param  name:   name
 * @return -
 *
 ******************************************************************
 */

ch_cfg_pro::ch_cfg_pro(CHANNEL *pChannel)
    :QObject()
{
  // ok button was not pressed.
  m_ok = false;
  
  m_cfg.setupUi(&m_w);
  
  
  connect(m_cfg.buttonOk, SIGNAL(clicked()), this, SLOT(on_ok()));
  connect(m_cfg.buttonCancel, SIGNAL(clicked()), this, SLOT(on_cancel()));
  connect(m_cfg.ed_tseg1, SIGNAL(lostFocus()), this, SLOT(on_tseg1()));
  connect(m_cfg.ed_baudrate, SIGNAL(lostFocus()), this, SLOT(on_baudrate()));
  connect(m_cfg.ed_tseg2, SIGNAL(lostFocus()), this, SLOT(on_tseg2()));
  connect(m_cfg.ed_presc, SIGNAL(lostFocus()), this, SLOT(on_presc()));
  connect(m_cfg.ed_sjw, SIGNAL(lostFocus()), this, SLOT(on_sjw()));
  connect(m_cfg.ed_acc_mask, SIGNAL(lostFocus()), this, SLOT(on_acc_mask()));
  connect(m_cfg.ed_acc_code, SIGNAL(lostFocus()), this, SLOT(on_acc_code()));
  connect(m_cfg.ed_acc_xtd_mask, SIGNAL(lostFocus()), this, SLOT(on_acc_xtd_mask()));
  connect(m_cfg.ed_acc_xtd_code, SIGNAL(lostFocus()), this, SLOT(on_acc_xtd_code()));
  connect(m_cfg.ed_channelName, SIGNAL(lostFocus()), this, SLOT(on_channel_name()));
  connect(m_cfg.tab_baudrates, SIGNAL(cellClicked(int,int)), this, SLOT(on_cellClicked(int, int)));
  connect(m_cfg.ch_div8x, SIGNAL(stateChanged(int)), this, SLOT(on_div8x(int)));
  
  // build the controller settings table.
  
  if(pChannel)
  {
    m_cfg.ed_channelName->setText(pChannel->m_channelName);
    m_channelName = pChannel->m_channelName;
    
     // first write the valid values to the dialog
    m_fBaudrate = pChannel->m_fBaudrate;
    m_prescaler = pChannel->m_prescaler;
    m_sam = pChannel->m_sam;
    m_sjw = pChannel->m_sjw;
    m_tseg1 = pChannel->m_tseg1;
    m_tseg2 = pChannel->m_tseg2;
    m_u32AccCodeStd = pChannel->m_u32AccCodeStd;
    m_u32AccCodeXtd = pChannel->m_u32AccCodeXtd;
    m_u32AccMaskStd = pChannel->m_u32AccMaskStd;
    m_u32AccMaskXtd = pChannel->m_u32AccMaskXtd;
    m_u32OutputCtrl = pChannel->m_u32OutputCtrl;   
    m_Div8X = (pChannel->m_sam) ? (USIGN8) 8 : (USIGN8) 1;
    m_clock_rate = pChannel->getCANClock();
    
    update_all();
//     m_Div8X = (pChannel->m_sam==(USIGN8)8) ? (USIGN8)8 : (USIGN8)1; 
    m_cfg.ch_div8x->setChecked(m_Div8X==8);
    m_cfg.ch_enable_ack->setChecked(pChannel->m_enableAck);
    m_cfg.ch_enable_error_frame->setChecked(pChannel->m_detectErrorframes);

    
  }
  
  
  
  update_all();
  on_baudrate();
  
 
  
}

ch_cfg_pro::~ch_cfg_pro()
{
  deleteAllTabEntries();
}

void ch_cfg_pro::deleteAllTabEntries(void)
{
  int numOfCol = getNumOfCol()-1;
  int i,j;
  QTableWidgetItem *pTableItem;
  
  for(i=numOfCol; i>=0; i--)
  {
    for(j=6; j>=0; j--)
    {
      pTableItem = m_cfg.tab_baudrates->item(i,j);
      delete pTableItem;
    }
  }
}

int ch_cfg_pro::getNumOfCol(void)
{
  int i=0;
  while(m_cfg.tab_baudrates->item(i,0))
  {
    i++;
    if(i>1000)
      return 0;
  }
  return i;
}

void ch_cfg_pro::show_config(void)
{
  QString s;
  
  s = QString("%1").arg(m_fBaudrate,4,'f',1);
  m_cfg.ed_baudrate->setText(s);
  s = QString("%1").arg(m_prescaler);
  m_cfg.ed_presc->setText(s);
//   m_sam = m_cfg.m_Div8X/8;
  s = QString("%1").arg(m_sjw);
  m_cfg.ed_sjw->setText(s);
  m_cfg.ed_tseg1->setText(QString("%1").arg(m_tseg1));
  m_cfg.ed_tseg2->setText(QString("%1").arg(m_tseg2));
  m_cfg.ed_acc_code->setText    (QString("0x%1").arg(m_u32AccCodeStd,4,16,QLatin1Char('0')));
  m_cfg.ed_acc_xtd_mask->setText(QString("0x%1").arg(m_u32AccMaskStd,4,16,QLatin1Char('0')));
  
  m_cfg.ed_acc_xtd_code->setText(QString("0x%1").arg(m_u32AccCodeXtd,8,16,QLatin1Char('0')));
  m_cfg.ed_acc_xtd_mask->setText(QString("0x%1").arg(m_u32AccMaskXtd,8,16,QLatin1Char('0')));
  
  
//   m_cfg.ed_channelName->setText(m_channelName);
 
  update_all();
  on_baudrate();
  m_w.exec();
}


/**
 ******************************************************************
 * @brief  Shows the baudrate table
 * @author Werner Knorsch, Softing AG
 * @date   2008-01-29
 *
 * @param  u32MaxEntry: maximum entries in the table
 * @return -
 *
 ******************************************************************
 */

void ch_cfg_pro::PrintTable(USIGN32 u32MaxEntry)
{
  USIGN32 i;
  QString itemText;
  QTableWidgetItem *pTableItem;
  QStringList tableDescription;
  
  tableDescription << "Baudrate[KBaud]" << "Prescaler" << "TSEG1" << "TSEG2" << "Sjw" << "Div8X" << "Sample Point[%]";

  m_cfg.tab_baudrates->setRowCount(u32MaxEntry);
  m_cfg.tab_baudrates->setColumnCount(7);
  
  m_cfg.tab_baudrates->setHorizontalHeaderLabels(tableDescription);
  
  for(i=0; i<u32MaxEntry; i++)
  {
    m_aModiTable[i].fBaudrate =
        m_clock_rate/(1000*(m_aModiTable[i].u8Prescaler*m_aModiTable[i].u8Div8x*(1+m_aModiTable[i].u8Tseg1+m_aModiTable[i].u8Tseg2)));
    
    itemText = itemText.sprintf("%.2f",m_aModiTable[i].fBaudrate);
    pTableItem = new QTableWidgetItem();
    
    pTableItem->setText(itemText);
    m_cfg.tab_baudrates->setItem(i,0,pTableItem);
    
    itemText = itemText.sprintf("%u",m_aModiTable[i].u8Prescaler);
    pTableItem = new QTableWidgetItem();
    pTableItem->setText(itemText);
    m_cfg.tab_baudrates->setItem(i,1,pTableItem);
    
    itemText = itemText.sprintf("%u",m_aModiTable[i].u8Tseg1);
    pTableItem = new QTableWidgetItem();
    pTableItem->setText(itemText);
    m_cfg.tab_baudrates->setItem(i,2,pTableItem);
    
    itemText = itemText.sprintf("%u",m_aModiTable[i].u8Tseg2);
    pTableItem = new QTableWidgetItem();
    pTableItem->setText(itemText);
    m_cfg.tab_baudrates->setItem(i,3,pTableItem);
    
    itemText = itemText.sprintf("%u",m_aModiTable[i].u8Sjw);
    pTableItem = new QTableWidgetItem();
    pTableItem->setText(itemText);
    m_cfg.tab_baudrates->setItem(i,4,pTableItem);
    
    if(m_aModiTable[i].u8Div8x == 8)
    {
      itemText = "enabled";
    }
    else
    {
      itemText = "disabled";
    }
    
    pTableItem = new QTableWidgetItem();
    pTableItem->setText(itemText);
    m_cfg.tab_baudrates->setItem(i,5,pTableItem);
    
    itemText =
    itemText.sprintf(
        "%.2f",
        (1.0+m_aModiTable[i].u8Tseg1)*100.0/(1.0+m_aModiTable[i].u8Tseg1+m_aModiTable[i].u8Tseg2)
                    );
    pTableItem = new QTableWidgetItem();
    pTableItem->setText(itemText);
    m_cfg.tab_baudrates->setItem(i,6,pTableItem);
    
    
  }

}

/**
 ******************************************************************
 * @brief  calculates all possible CAN controller settings for a given sum of tseg1 and tseg2
 * @author Werner Knorsch, Softing AG
 * @date   2008-01-29
 *
 * @param  Tseg1AndTseg2Int: tseg1+tseg2
 * @return -
 *
 ******************************************************************
 */

void ch_cfg_pro::ShowAll(USIGN32 Tseg1AndTseg2Int)
{
  
 USIGN8 tseg1, tseg2 = 0, sjw, presc, Div8X=1, tseg12 = (USIGN8)Tseg1AndTseg2Int;
 USIGN32 u32Entry = 0;
 double fTseg;
 INT32 baud1, baud2;
 INT32 Div8XStart = 1;
 INT32 prescaler;
 
 if(m_Div8X == 8)
 {
  prescaler = (INT32)m_prescaler*8;
  if(prescaler>64)
   Div8XStart = 8;
 }
 else
 {
  prescaler =(INT32) m_prescaler;
  Div8XStart = 1;
 }

 for(Div8X=(USIGN8)Div8XStart; Div8X<=(USIGN8)8; Div8X += (USIGN8)7)
 {
  tseg12 = (USIGN8)Tseg1AndTseg2Int;
  tseg2 = (USIGN8)(Tseg1AndTseg2Int/2);
  presc = (USIGN8)(prescaler/Div8X);
  
  if(!presc) break;

  do
  {
   do
   {
      

    if(tseg2 > 8) 
     tseg2 = 8;
      
    if(tseg2 < 2)
     tseg2 = 2;
      
    tseg1 = (USIGN8)(tseg12 - tseg2);
      
      
    if((tseg1 < 1) || (tseg1 > 16))
    { // to do ERROR handling!
     break;
    }

    baud1 = (int) (m_fBaudrate * 100000.00);
  // m_clock_rate/((m_prescaler*m_Div8X*1000.0)*(1+m_tseg1+m_tseg2));
    baud2 = (int) (m_clock_rate/((presc*Div8X)*(1+tseg1+tseg2))*100);

    if(baud1 != baud2)
    {
     break;
    }
      

    for(sjw = 1; sjw<=4; sjw++)
    {
     if((sjw <= tseg2) && (sjw <= (tseg1+tseg2)))
     {
      m_aModiTable[u32Entry].u8Prescaler  = presc;
      m_aModiTable[u32Entry].u8Tseg1 = tseg1;
      m_aModiTable[u32Entry].u8Tseg2 = tseg2;
      m_aModiTable[u32Entry].u8Sjw   = sjw;
      m_aModiTable[u32Entry].u8Div8x   = Div8X;
          
      u32Entry++;
          
      if(u32Entry >= MAX_MODUS)
      {
       PrintTable(u32Entry-1);
       return; // out of memory, no more modi possible;
      }
     }
    }
      
    tseg2--;
      
   } while(tseg2 > 1);

   presc = getPresc(++presc,Div8X);
   fTseg = m_clock_rate/(presc*m_fBaudrate*Div8X*1000)-1;
   tseg12 = (USIGN8)(int)fTseg;
   tseg2 = (USIGN8)(tseg12/2);
   
  } while ((fTseg > 8.0) && (presc <= 64));

 }
 
 if(u32Entry > 0)
 {
  PrintTable(u32Entry-1);
 }

}



/**
 ******************************************************************
 * @brief  on change of tseg1; calculate new baudrate;
 * @author Werner Knorsch, Softing AG
 * @date   2008-01-29
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg_pro::on_tseg1()
{
  
 unsigned long oldTseg1 = m_tseg1;
  
 read_all();
  
 if((m_tseg1<1) || (m_tseg1 > 16))
 {
  m_tseg1 = oldTseg1;
  update_all();
  return;
 }

 if(m_tseg1 < m_tseg2)
 {
  m_tseg1 = m_tseg2;
 }

 m_fBaudrate = m_clock_rate/(((double)m_prescaler*(double)m_Div8X*1000.0)*(double)(1+m_tseg1+m_tseg2));
 m_CTableText = m_CTableText.sprintf("Possible CAN controller settings for %.2f KBaud", m_fBaudrate);
 update_all();
 on_baudrate(); 
}

USIGN8 ch_cfg_pro::getPresc(USIGN8 StartPresc, USIGN8 Div8X)
{
 USIGN8 presc = StartPresc;
 USIGN32 IntTesg1AndTeseg2;
 double fTeseg, fTesegRound; 

 if(presc == 0)
  presc++;

 while(presc <= 64)
 {
  fTeseg = (m_clock_rate/1000)/(presc*Div8X*m_fBaudrate)-1;
  IntTesg1AndTeseg2 = (USIGN32) fTeseg;
  fTesegRound = (double)IntTesg1AndTeseg2;

  if(fTesegRound == fTeseg)
   return presc;

  presc++;
 }

 return 0xFF;
}

/**
 ******************************************************************
 * @brief  called, when the user has changed the baudrate; updates the controller settings.
 * @author Werner Knorsch, Softing AG
 * @date   2008-01-29
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg_pro::on_baudrate()
{
 USIGN8 presc, tseg1, tseg2, sjw, Div8X=1;
 USIGN32 Tseg1AndTseg2Int;
 double samples, BitTime, Tseg1AndTseg2, fOldBaudrate;
 bool isValid;
 double baud1;
  
  
 fOldBaudrate = m_fBaudrate;
  
 isValid = read_all();
 baud1 = (m_clock_rate/1000)/((double)m_prescaler*(double)m_Div8X*(double)(1+m_tseg1+m_tseg2));
 if(m_fBaudrate != baud1)
 {

  //UpdateData(TRUE);

  if((m_fBaudrate < 6.0) || (m_fBaudrate > 1000.0) || !isValid)
  {
   m_fBaudrate = fOldBaudrate;
   update_all();
   return;
  }

  BitTime = 1.0/(m_fBaudrate * 1000);
  samples = BitTime * m_clock_rate;
 
   // start value for prescaler
  presc = (USIGN8)(int)(samples / 25);
  
 // as a floating point value the prescaler may be less than 1, but not in the real CAN controller.
  if(presc == 0)
   presc = 1;
  
  presc = 1;

  // presc is a BYTE: may be a prescaler of 255 is not enough to satisfy the following formula
  // to prevent an overrun from 255->0 (endless loop) we go only to max. 65
  while((((m_clock_rate/1000.0)/((double)presc*(double)Div8X*m_fBaudrate)-1) > 24.0) && (presc <= 65)) presc++;
 
  if(presc > 64)
   presc = 64;
 
  if(getPresc(presc,Div8X) == 0xFF)
  { // no fitting prescaler found!
   if(m_fBaudrate < 50.0)
   { // try it with div8x for small baud rates!
    Div8X = 8;
    presc = 1;
    while((((m_clock_rate/1000)/((double)presc*(double)Div8X*m_fBaudrate)-1) > 24.0) && (presc <= 65)) presc++;
                  
    if(presc > 64)
     presc = 64;
                  
                  
    if(getPresc(presc,Div8X) != 0xFF)
    { // ok, a valid prescaler was found: get it!
     presc = getPresc(presc,Div8X);
    }
   }
  }
  else
  { // ok, a valid prescaler was found: get it!
   presc = getPresc(presc,Div8X);
  }

  Tseg1AndTseg2 = (m_clock_rate/1000)/(double)(presc*Div8X*m_fBaudrate)-1;

  Tseg1AndTseg2Int = (int)Tseg1AndTseg2;


  tseg2 = (USIGN8)(Tseg1AndTseg2Int/2);

  if((USIGN8)Tseg1AndTseg2Int == 2*tseg2)
   tseg2--;

  if(tseg2 > 8) 
   tseg2 = 8;

  if(tseg2 < 2)
   tseg2 = 2;

  tseg1 = (USIGN8)(Tseg1AndTseg2Int - tseg2);

  if((tseg1 < 1) || (tseg1 > 16))
  { // to do ERROR handling!
   m_fBaudrate = (m_clock_rate/1000)/(double)(m_prescaler*Div8X*(1+m_tseg1+m_tseg2));
   m_CTableText = m_CTableText.sprintf("Possible CAN controller settings for %.2f KBaud", m_fBaudrate);
   update_all();
   ShowAll(Tseg1AndTseg2Int);
  return; 
  }
  
  
  
  sjw = (USIGN8)(Tseg1AndTseg2Int/8);

  if(sjw<1)
   sjw = 1;

  m_prescaler = presc;
  m_tseg1 = tseg1;
  m_tseg2 = tseg2;
  m_Div8X = Div8X;
  m_sjw = sjw;
  m_fBaudrate = (m_clock_rate/1000)/((double)m_prescaler*(double)Div8X*(double)(1+m_tseg1+m_tseg2));
  m_CTableText = m_CTableText.sprintf("Possible CAN controller settings for %.2f KBaud", m_fBaudrate);

  update_all();
 }
 else
 {
  Tseg1AndTseg2 = (m_clock_rate/1000)/((double)m_prescaler*(double)m_Div8X*m_fBaudrate)-1;
  Tseg1AndTseg2Int = (int)Tseg1AndTseg2;
  
 }

 ShowAll(Tseg1AndTseg2Int);
 
}



/**
 ******************************************************************
 * @brief  on change of prescaler; calculate new baudrate;
 * @author Werner Knorsch, Softing AG
 * @date   2008-01-29
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg_pro::on_presc()
{
 unsigned long presc = m_prescaler;
  
 read_all();

 if((m_prescaler<1) || (m_prescaler>64))
 {
  m_prescaler = presc;
  update_all();
  return;

 }

 m_fBaudrate = (m_clock_rate/1000)/((double)m_prescaler*(double)m_Div8X*(double)(1+m_tseg1+m_tseg2));
 m_CTableText = m_CTableText.sprintf("Possible CAN controller settings for %.2f KBaud", m_fBaudrate);
 update_all();
 on_baudrate();
}


void ch_cfg_pro::on_channel_name()
{
  m_channelName = m_cfg.ed_channelName->text();
}

/**
 ******************************************************************
 * @brief  on change of acceptance mask; check, get and show the new value
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg_pro::on_acc_mask()
{
  bool isValid;
  unsigned long val;
  QString text;
  
  val = m_cfg.ed_acc_mask->text().toULong(&isValid,16);
  
  if(isValid)
  {
    m_u32AccMaskStd = val;
  }
  
  text = text.sprintf("0x%04X",m_u32AccMaskStd);
  m_cfg.ed_acc_mask->setText(text);
}


/**
 ******************************************************************
 * @brief  on change of extended acceptance mask; check, get and show the new value
 * @author Werner Knorsch, Softing AG
 * @date   2008-01-29
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg_pro::on_acc_xtd_mask()
{
 bool isValid;
 unsigned long val;
 QString text;
  
 val = m_cfg.ed_acc_xtd_mask->text().toULong(&isValid,16);
  
 if(isValid)
 {
  m_u32AccMaskXtd = val;
 }
  
 text = text.sprintf("0x%08X",m_u32AccMaskXtd);
 m_cfg.ed_acc_xtd_mask->setText(text);

}

/**
 ******************************************************************
 * @brief  on change of extended acceptance code; check, get and show the new value
 * @author Werner Knorsch, Softing AG
 * @date   2008-01-29
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg_pro::on_acc_xtd_code()
{
 bool isValid;
 unsigned long val;
 QString text;
  
 val = m_cfg.ed_acc_xtd_code->text().toULong(&isValid,16);
  
 if(isValid)
 {
  m_u32AccCodeXtd = val;
 }
  
 text = text.sprintf("0x%08X",m_u32AccCodeXtd);
 m_cfg.ed_acc_xtd_code->setText(text);

}

/**
 ******************************************************************
 * @brief  on change of acceptance mask; check, get and show the new value
 * @author Werner Knorsch, Softing AG
 * @date   2008-01-29
 *
 * @return -
 *
 ******************************************************************
 */

// void ch_cfg_pro::on_add_mask()
// {
//  bool isValid;
//  unsigned long val;
//  QString text;
//   
//  val = m_cfg.ed_acc_mask->text().toULong(&isValid,16);
//   
//  if(isValid)
//  {
//   m_u32AccMaskStd = val;
//  }
//   
//  text = text.sprintf("0x%04lX",m_u32AccMaskStd);
//  m_cfg.ed_acc_mask->setText(text);
// }

/**
 ******************************************************************
 * @brief  on change of acceptance code; check, get and show the new value
 * @author Werner Knorsch, Softing AG
 * @date   2008-01-29
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg_pro::on_acc_code()
{
 bool isValid;
 unsigned long val;
 QString text;
  
 val = m_cfg.ed_acc_code->text().toULong(&isValid,16);
  
 if(isValid)
 {
  m_u32AccCodeStd = val;
 }
  
 text = text.sprintf("0x%04X",m_u32AccCodeStd);
 m_cfg.ed_acc_code->setText(text);

}

/**
 ******************************************************************
 * @brief  on change of sjw; calculate new baudrate;
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg_pro::on_sjw()
{
    
 unsigned long old_val = m_sjw;
  
 read_all();
 if((m_sjw<1) || (m_sjw > 4))
 {
  m_sjw = old_val;
  update_all();
  return;

 }
 m_fBaudrate = (m_clock_rate/1000)/((double)m_prescaler*(double)m_Div8X*(double)(1+m_tseg1+m_tseg2));
 m_CTableText = m_CTableText.sprintf("Possible CAN controller settings for %.2f KBaud", m_fBaudrate);
 update_all();

}

/**
 ******************************************************************
 * @brief  on change of tseg2; calculate new baudrate;
 * @author Werner Knorsch, Softing AG
 * @date   2008-01-29
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg_pro::on_tseg2()
{
 unsigned long tseg2 = m_tseg2;
  
 read_all();
 if((m_tseg2<1) || (m_tseg2 > 8))
 {
  m_tseg2 = tseg2;
  update_all();
  return;

 }

 m_fBaudrate = (m_clock_rate/1000)/((double)m_prescaler*(double)m_Div8X*(double)(1+m_tseg1+m_tseg2));
 m_CTableText = m_CTableText.sprintf("Possible CAN controller settings for %.2f KBaud", m_fBaudrate);
 update_all();
 on_baudrate();
}

/**
 ******************************************************************
 * @brief  reads all text boxes into member variables
 * @author Werner Knorsch, Softing AG
 * @date   2008-01-29
 *
 * @return -
 *
 ******************************************************************
 */

bool ch_cfg_pro::read_all()
{
 bool isValid;
  
 m_fBaudrate = m_cfg.ed_baudrate->text().toDouble(&isValid);
 if(!isValid) return false;
  
 m_prescaler = m_cfg.ed_presc->text().toUInt(&isValid);
 if(!isValid) return false;
 m_tseg1 = m_cfg.ed_tseg1->text().toUInt(&isValid);
 if(!isValid) return false;
 m_tseg2 = m_cfg.ed_tseg2->text().toUInt(&isValid);
 if(!isValid) return false;
 
 m_Div8X = (m_cfg.ch_div8x->isChecked()) ? (USIGN8)8 : (USIGN8)1;
 
 m_sjw = m_cfg.ed_sjw->text().toUInt(&isValid);
 
 m_u32OutputCtrl = (m_cfg.ch_listen_only_mode->isChecked()) ? 1 : 0xFA;
  
  
 return isValid;
  
}

/**
 ******************************************************************
 * @brief  writes the member variables into the textboxes
 * @author Werner Knorsch, Softing AG
 * @date   2008-01-29
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg_pro::update_all()
{
  QString text="1";
  QString str_tooltip = QString("possible CAN controller settings for %1 KBaud").arg(m_fBaudrate);
  
  m_cfg.tab_baudrates->setToolTip(str_tooltip);
  
  text = text.sprintf("%u",m_prescaler);
  m_cfg.ed_presc->setText(text);
  text = text.sprintf("%u",m_tseg1);
  m_cfg.ed_tseg1->setText(text);
  text = text.sprintf("%u",m_tseg2);
  m_cfg.ed_tseg2->setText(text);
  text = text.sprintf("%u",m_sjw);
  m_cfg.ed_sjw->setText(text);
  text = text.sprintf("%.2f",m_fBaudrate);
  m_cfg.ed_baudrate->setText(text);

  text = text.sprintf("0x%04X",m_u32AccCodeStd);
  m_cfg.ed_acc_code->setText(text);
  text = text.sprintf("0x%04X",m_u32AccMaskStd);
  m_cfg.ed_acc_mask->setText(text);

  text = text.sprintf("0x%08X",m_u32AccCodeXtd);
  m_cfg.ed_acc_xtd_code->setText(text);
  text = text.sprintf("0x%08X",m_u32AccMaskXtd);
  m_cfg.ed_acc_xtd_mask->setText(text);

  m_cfg.ch_listen_only_mode->setChecked((m_u32OutputCtrl <= 3));
  m_cfg.ch_div8x->setChecked(m_Div8X==8);
  
  m_CTableText = m_CTableText.sprintf("Possible CAN controller settings for %.2f KBaud", m_fBaudrate);
  m_cfg.tableLabel->setText(m_CTableText);
   
}

/**
 ******************************************************************
 * @brief  called, when the user selects a row in the baudrate table; use the new controller settings
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg_pro::on_row_changed(int row, int col)
{
 col = col;

 if((row > 0) && (row < MAX_MODUS))
 {

  m_prescaler   = m_aModiTable[row].u8Prescaler;
  m_tseg1       = m_aModiTable[row].u8Tseg1;
  m_tseg2       = m_aModiTable[row].u8Tseg2;
  m_sam         = m_aModiTable[row].u8Div8x/8;
  m_Div8X       = m_aModiTable[row].u8Div8x;
  m_sjw         = m_aModiTable[row].u8Sjw;
    
  update_all();
 }

}

void ch_cfg_pro::on_cancel()
{
 m_ok = false;
 m_w.reject();
}

/**
 ******************************************************************
 * @brief  called, when the user selects a row in the baudrate table; use the new controller settings
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

void ch_cfg_pro::on_cellClicked(int row, int col)
{ 
    QTableWidgetSelectionRange col_range(row, 0, row, m_cfg.tab_baudrates->columnCount()-1);
    QTableWidgetSelectionRange tab_range(0, 0, m_cfg.tab_baudrates->rowCount()-1, m_cfg.tab_baudrates->columnCount()-1);
    
    QString rw = "Rx PDO";

    m_cfg.tab_baudrates->setRangeSelected(tab_range, false);
    m_cfg.tab_baudrates->setRangeSelected(col_range, true);
    m_cfg.tab_baudrates->setCurrentItem(m_cfg.tab_baudrates->item(row,col), QItemSelectionModel::Rows);


  if((row > 0) && (row < MAX_MODUS))
  {

    m_prescaler   = m_aModiTable[row].u8Prescaler;
    m_tseg1       = m_aModiTable[row].u8Tseg1;
    m_tseg2       = m_aModiTable[row].u8Tseg2;
    m_Div8X       = m_aModiTable[row].u8Div8x;
    m_sam         = m_aModiTable[row].u8Div8x/8;
    m_sjw         = m_aModiTable[row].u8Sjw;
    
    update_all();
  }
  
}

void ch_cfg_pro::on_tab_selection_changed()
{
  QList<QTableWidgetItem *> list = m_cfg.tab_baudrates->selectedItems();
  
  if(list.count()>0)
  {
    on_cellClicked(list[0]->row(), 0);
  }
  
}


void ch_cfg_pro::on_ok()
{
 m_ok = true;
 m_w.accept();
}

void ch_cfg_pro::on_div8x(int state)
{   
 state = state;
 
 read_all();


 m_Div8X = (m_cfg.ch_div8x->isChecked()) ? (USIGN8)8 : (USIGN8)1;
 m_fBaudrate = m_clock_rate/(((double)m_prescaler*(double)m_Div8X*1000.0)*(double)(1+m_tseg1+m_tseg2));
 
 m_CTableText = m_CTableText.sprintf("Possible CAN controller settings for %.2f KBaud", m_fBaudrate);
 update_all();
 on_baudrate();
  
}

bool ch_cfg_pro::is_ack_enabled(void)
{
  return m_cfg.ch_enable_ack->isChecked();
}

void ch_cfg_pro::set_ack_enabled(bool yesno)
{
  m_cfg.ch_enable_ack->setChecked(yesno);
}

bool ch_cfg_pro::is_errorframe_detection_enabled(void)
{
  return m_cfg.ch_enable_error_frame->isChecked();
}

void ch_cfg_pro::set_error_frame_detection_enabled(bool yesno)
{
  m_cfg.ch_enable_error_frame->setChecked(yesno);
}



#include "ch_cfg_pro.moc"
