/***************************************************************************
 *   Copyright (C) 2007-2015 by Softing Automation GmbH                    *
 *   support.automation@softing.com                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "channel.h"

/**
 ******************************************************************
 * @brief  Constructor of the class CHANNEL.
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

CHANNEL::CHANNEL()
{
    // default settings for 500 kBaud
  m_tseg1 = 8;
  m_tseg2 = 7;
  m_prescaler = 4;
  m_sam = 1;
  m_sjw = 1;
  m_deviceType = UNKNOWN_DEV;
  m_u32AccCodeStd = 0;
  m_u32AccMaskStd = 0;
  m_u32AccCodeXtd = 0;
  m_u32AccMaskXtd = 0;
  m_Error = 0;
  m_u32OutputCtrl = 0xFA;
  m_serialNumber = 0;
  m_physChannel = 1;
  m_fBaudrate = 125.0;
  m_detectErrorframes = false;
  m_enableAck = true;
  m_minor = UNDEFINED_VALUE;
  m_major = UNDEFINED_VALUE;
  m_CAN_clock = 16E6;
}

CHANNEL::CHANNEL(char* line)
{
  char *chName;
  QString serial;
  bool isValid;
  
  m_major = UNDEFINED_VALUE;
  m_minor = UNDEFINED_VALUE;
  
  
  m_prescaler = getNumber(line,"presc:",10);
  m_sam = getNumber(line,"sam:",10);
  m_sjw = getNumber(line,"sjw:",10);
  m_tseg1 = getNumber(line,"tseg1:",10);
  m_tseg2 = getNumber(line,"tseg2:",10);
  m_u32AccCodeStd = getNumber(line,"acccstd:",16);
  m_u32AccMaskStd = getNumber(line,"accmstd:",16);
  m_u32AccCodeXtd = getNumber(line,"acccxtd:",16);
  m_u32AccMaskXtd = getNumber(line,"accmxtd:",16);
  m_u32OutputCtrl = getNumber(line,"ocr:",16);
  m_deviceType = (DEVICETYPE)getNumber(line,"type:",10);
  m_physChannel = getNumber(line,"_",10);
  m_irq = getNumber(line,"irq:",10);
  m_dpram = getNumber(line,"dpram:",16);
  m_ioport = getNumber(line,"ioport:",16);
  
  chName = strstr(line,"name:");
 
  
  m_channelName = &chName[sizeof("name:")-1];
  m_channelName.remove("\n");
  
  m_detectErrorframes = (getNumber(line,"errorframes:",10) == 1);
  m_enableAck = (getNumber(line,"ack:",10) == 1);
  
  m_fBaudrate = getDouble(line,"baud:");
  
  chName = strstr(line,"_");
  *chName = 0;
  serial = line;
  m_serialNumber = serial.toUInt(&isValid,10);
}

CHANNEL::CHANNEL(DEVICETYPE type, QString& chName, QString& ifcName)
{
  bool isValid;
  
  QString serial = chName.right(9);
  
  m_major = UNDEFINED_VALUE;
  m_minor = UNDEFINED_VALUE;
  
  m_serialNumber = serial.toUInt(&isValid,10);
  m_ifcName = ifcName;
  m_channelName = chName;
  m_deviceType = type;
  
}

/**
 * *****************************************************************
 * @brief  copy operator.
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

CHANNEL& CHANNEL::operator=(CHANNEL &ch)
{
   
  this->m_channelName = ch.m_channelName;
  this->m_detectErrorframes = ch.m_detectErrorframes;
  this->m_dpram = ch.m_dpram;
  this->m_deviceType = ch.m_deviceType;
  this->m_driverNode = ch.m_driverNode;
  this->m_enableAck = ch.m_enableAck;
  this->m_Error = ch.m_Error;
  this->m_fBaudrate = ch.m_fBaudrate;
  this->m_ifcName = ch.m_ifcName;
  this->m_ioport = ch.m_ioport;
  this->m_irq = ch.m_irq;
  this->m_isDevFileCreated = ch.m_isDevFileCreated;
  this->m_major = ch.m_major;
  this->m_minor = ch.m_minor;
  this->m_physChannel = ch.m_physChannel;
  this->m_prescaler = ch.m_prescaler;
  this->m_sam = ch.m_sam;
  this->m_serialNumber = ch.m_serialNumber;
  this->m_sjw = ch.m_sjw;
  this->m_tseg1 = ch.m_tseg1;
  this->m_tseg2 = ch.m_tseg2;
  this->m_u32AccCodeStd = ch.m_u32AccCodeStd;
  this->m_u32AccMaskStd = ch.m_u32AccMaskStd;
  this->m_u32AccCodeXtd = ch.m_u32AccCodeXtd;
  this->m_u32AccMaskXtd = ch.m_u32AccMaskXtd;
  this->m_u32OutputCtrl = ch.m_u32OutputCtrl;
  this->m_CAN_clock = ch.m_CAN_clock;
  
  return *this;
}

/**
 ******************************************************************
 * @brief  Destructor of the class CHANNEL.
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @return -
 *
 ******************************************************************
 */

CHANNEL::~CHANNEL()
{
  
}

/**
 ******************************************************************
 * @brief  get the configuration of a CAN channel
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @param  pSerial: pointer to the serial number of the CAN Interface card
 * @param  type:    device type
 * @return -
 *
 ******************************************************************
 */

bool CHANNEL::getChannelConfig(QString Serial, DEVICETYPE type)
{
  char linebuffer[1000];
  char *line = &linebuffer[0];
  QString serial;
  QString qline;
  bool isValid;
  FILE *pDevices = NULL;
  bool bRet = false;
  
  // this information can always be retrived.
  if(Serial.endsWith("1"))
  {
    m_physChannel = 1;
  }
  
  if(Serial.endsWith("2"))
  {
    m_physChannel = 2;
  }

  serial = Serial.left(9);
  m_serialNumber = serial.toUInt(&isValid,10);
 
  pDevices = fopen(CANCONFIGFILE,"r");
  
  if(!pDevices)
    return false;
  
  // this information comes from a config file
  while(!feof(pDevices))
  {
    char *pline = NULL;
    
    pline = fgets(line, 1000, pDevices);
    if(!pline) continue;
    
    qline = line;
    qline = qline.left(11);
    
    // remove \n from string
    if(line[strlen(line)-1] == '\n')
    {
      line[strlen(line)-1] = 0;
    }
    
    //if(!memcmp(line,pSerial,11))
    if (qline == Serial)
    { //9 digits of the serial number + "_1"
      char *chName;
      unsigned long deviceType;

      deviceType = getNumber(line,"type:",10) & 0xFF;

      if(deviceType == type)
      {

        m_prescaler = getNumber(line,"presc:",10);
        m_sam = getNumber(line,"sam:",10);
        m_sjw = getNumber(line,"sjw:",10);
        m_tseg1 = getNumber(line,"tseg1:",10);
        m_tseg2 = getNumber(line,"tseg2:",10);
        m_u32AccCodeStd = getNumber(line,"acccstd:",16);
        m_u32AccMaskStd = getNumber(line,"accmstd:",16);
        m_u32AccCodeXtd = getNumber(line,"acccxtd:",16);
        m_u32AccMaskXtd = getNumber(line,"accmxtd:",16);
        m_u32OutputCtrl = getNumber(line,"ocr:",16);
      
      
        chName = strstr(line,"name:");
        if(!chName)
        {
          fclose(pDevices);
          return false;
        }
        
        m_channelName  = &chName[sizeof("name:")-1];
        m_channelName  = m_channelName.simplified();
      
        m_detectErrorframes = (getNumber(line,"errorframes:",10) == 1);
        m_enableAck = (getNumber(line,"ack:",10) == 1);
      
        m_fBaudrate = getDouble(line,"baud:");
        bRet = true;
      }
      
      
    }
  }
  fclose(pDevices);
  
  return bRet;    
    
}


/**
 ******************************************************************
 * @brief  extracts a integer number out of a text line
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @param  line: text line
 * @param  tok:  the number is after the string tok
 * @param  base: base of the number (hex. decimal ...)
 * @return -
 *
 ******************************************************************
 */

unsigned long CHANNEL::getNumber(char *line, QString tok, int base)
{
  char *pVal = strstr(line,tok.toAscii());
  char *pValEnd;
  bool isValid;
  QString strVal;

  if(!pVal)
    return 0;

  pVal = pVal + strlen(tok.toAscii());
  pValEnd = strstr(pVal," ");

  if(!pValEnd)
  {
    pValEnd = strstr(pVal,"\n");
    if(!pValEnd)
    {
      return 0;
    }
  }


  *pValEnd = 0;
  strVal = pVal;
  *pValEnd = ' ';

  return strVal.toUInt(&isValid,base);

}

/**
 ******************************************************************
 * @brief  extracts a double out of a text line
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @param  line: text line
 * @param  tok:  the double is after the string tok
 * @return -
 *
 ******************************************************************
 */

double CHANNEL::getDouble(char *line, QString tok)
{
  char *pVal = strstr(line,tok.toAscii());
  char *pValEnd = strstr(pVal," ");
  bool isValid;
  QString strVal;

  if(!pVal || !pValEnd)
    return 0.0;

  pVal = pVal + strlen(tok.toAscii());
  *pValEnd = 0;
  strVal = pVal;
  *pValEnd = ' ';

  return strVal.toDouble(&isValid);

}

/**
 ******************************************************************
 * @brief  gets the device name from the device type
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @param  u32DeviceType: device type
 * @return device name as a QString
 *
 ******************************************************************
 */
// gets the device name from the device type
QString CHANNEL::getDeviceName(DEVICETYPE u32DeviceType)
{
  QString DType;
  
  switch(u32DeviceType)
  {
    case CANCARD2:
      DType = "CANcard2";
      break;
      
    case CANPCI:
      DType = "CAN-AC PCI";
      break;
      
    case 8:
      DType = "CAN-ACx-PCI/DN";
      break;
      
    case CAN_104:
      DType = "CAN-AC PC104";
      break;
      
    case CANUSB:
      DType = "CANusb";
      break;
      
    case CAN104ECO:
      DType = "CAN ECO 104";
      break;
      
    case CANPROPCIE:
      DType = "CANpro PCIe";
      break;
      
    case CAN104PLUS:
      DType = "CANpro 104+";
      break;
      
    case CANPROUSB:
      DType = "CANpro USB";
      break;
      
    case 0x105:
      DType = "EDICcard2";
      break;
      
    default:
      DType = "UNKNOWN";
      break; 
      
  }
  return DType;
}

/**
 * *****************************************************************
 * @brief  Constructor of the class CHANNEL.
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @param  pFileName: /proc filename
 * @param  type:      device type
 * @return -
 *
 ******************************************************************
 */
CHANNEL::CHANNEL(char *pFileName, DEVICETYPE type)
{
  FILE *pFile;
  char line[200];
  QString Interface;
  QMessageBox m_on_Error;
  QString h = pFileName;
  QString filename = pFileName;
  
  
  // default settings for 500 kBaud
  m_tseg1 = 8;
  m_tseg2 = 7;
  m_prescaler = 1;
  m_sam = 1;
  m_sjw = 1;
  m_deviceType = type;
  m_u32AccCodeStd = 0;
  m_u32AccMaskStd = 0;
  m_u32AccCodeXtd = 0;
  m_u32AccMaskXtd = 0;
  m_Error = 0;
  m_u32OutputCtrl = 0xFA;
  m_serialNumber = 0;
  m_physChannel = 1;
  m_fBaudrate = 500.0;
  m_detectErrorframes = false;
  m_enableAck = true;
  m_major = UNDEFINED_VALUE;
  m_minor = UNDEFINED_VALUE;
  m_dpram = 0;
  m_irq = 0;
  m_ioport = 0;
  
  
  Interface = getDeviceName(type);
  
  switch(type)
  {
    case CANCARD2:
      filename.prepend(PROC_PCMCIA);
      m_CAN_clock = 16E6;
      break;
    case CANPCI:
      filename.prepend(PROC_PCI);
      m_CAN_clock = 16E6;
      break;
    case CAN_104:
      filename.prepend(PROC_104);
      m_CAN_clock = 16E6;
      break;
      
    case CAN104ECO:
      filename.prepend(PROC_ECO);
      m_CAN_clock = 16E6;
      break;
      
    case CANPROPCIE:
      filename.prepend(PROC_PCIE);
      m_tseg1 = 11;
      m_tseg2 = 8;
      m_prescaler = 6;
      m_sam = 0;
      m_CAN_clock = 40E6;
      break;
      
    case CAN104PLUS:
      filename.prepend(PROC_104PLUS);
      m_tseg1 = 11;
      m_tseg2 = 8;
      m_prescaler = 6;
      m_sam = 0;
      m_CAN_clock = 40E6;
      break;
      
    case CANPROUSB:
      filename.prepend(PROC_USB);
      m_tseg1 = 11;
      m_tseg2 = 8;
      m_prescaler = 6;
      m_sam = 0;
      m_CAN_clock = 80E6;
      break;
      
    default:
      
      m_on_Error.information(
        NULL,
        "CHANNEL",
        "Internal ERROR: unknown interface type",
        QMessageBox::Abort,
        QMessageBox::NoButton,
        QMessageBox::NoButton);
      
      //exit(1); 
      break;
      
      
  }
  
  
  
  pFile = fopen(filename.toAscii(),"r");
  
  if(pFile)
  {
    QString ifc = Interface.simplified();
    char *pline;
    
    
    h = h.simplified();
    
    pline = fgets(line, sizeof(line), pFile);
    
    fclose(pFile);
    if(pline) 
    {
      m_major = getNumber(line,"major: ",10);
      m_minor = getNumber(line,"minor: ",10);
      m_physChannel = getNumber(line,"channel: ",10);
      m_deviceType = (DEVICETYPE) getNumber(line,"type: ",10);
      m_dpram = getNumber(line,"dpram: ",16);
      m_irq = getNumber(line,"irq: ",10);
      m_ioport = getNumber(line,"ioport: ",16);
    }
    
    m_driverNode = Interface;
    
    ifc.append(" #");
    ifc.append(h);
    ifc.truncate(ifc.length()-2);
    
    m_ifcName = ifc;
    m_channelName = m_ifcName;
    if(m_physChannel == 1)
    {      
      m_channelName.append("_1");
    }
    else
    {
      m_channelName.append("_2");
    }
    
  }
  
  getChannelConfig(h, type);
  
  
  
}

double CHANNEL::getCANClock(void)
{
  return m_CAN_clock;
}
