/***************************************************************************
 *   Copyright (C) 2007-2015 by Softing Industrial Automation GmbH         *
 *   support.automation@softing.com                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef CHANNEL_H
#define CHANNEL_H

#include <qstring.h>

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <linux/kdev_t.h>
#include <dirent.h>
#include <errno.h>
#include <QMessageBox>


#include "BaseTypes.h"

#define PROC_PCMCIA  "/proc/softing_can_pcmcia/"
#define PROC_PCI     "/proc/softing_can_pci/"
#define PROC_104     "/proc/softing_can_104/"
#define PROC_104PLUS "/proc/softing_can_pro/"
#define PROC_ECO     "/proc/softing_can_eco/"
#define PROC_PCIE    "/proc/softing_can_pcie/"
#define PROC_USB     "/proc/softing_canprousb/"


/**
 * @short Channel class; contains all configuration information about a CAN channel;
 * @author Werner Knorsch <kno@softing.com>
 * @version 1.0
 */

/******************************************************************
**  macros
*******************************************************************/
#define MAX_MODUS 300


#define CANCONFIGFILE "/etc/softing/candevices.conf"
#define DEVFILEPREFIX "/dev/Softing_AG_"
#define ERROR_CH_NOT_FOUND	1
#define TREE_ROOT 		0
#define TREE_IFC_TYPE 		1
#define TREE_IFC 		2
#define TREE_CHANNEL 		3

#define UNDEFINED_VALUE         -1

/******************************************************************
** protoypes  
*******************************************************************/

typedef enum
{
  UNKNOWN_DEV,
  CANCARDNEC = 1,
  CANCARDSJA = 4,
  CANCARD2,
  CANPCI   = 7,
  CANPCIDN,
  CAN_104,
  CANUSB,
  CAN104ECO = 12,
  CANPROPCIE,
  CAN104PLUS,
  CANPROUSB,
  EDICCARDC=0x101,
  EDICCARD2=0x105,
  FG100CAN = 0x10000000L
  
} DEVICETYPE;


class CHANNEL
{
  public:
  unsigned long m_tseg1;
  unsigned long m_tseg2;
  unsigned long m_prescaler;
  unsigned long m_sam;
  unsigned long m_sjw;
  unsigned long m_u32AccCodeStd;
  unsigned long m_u32AccMaskStd;
  unsigned long m_u32AccCodeXtd;
  unsigned long m_u32AccMaskXtd;
  unsigned long m_u32OutputCtrl;
  unsigned long m_Error;

  double        m_fBaudrate;
  bool          m_detectErrorframes;
  bool          m_enableAck;  
  bool          m_isDevFileCreated;
  QString       m_channelName;
  QString       m_ifcName;
  QString       m_driverNode;
  DEVICETYPE    m_deviceType;
  
  
  
  CHANNEL();
  CHANNEL(char *pFileName, DEVICETYPE type);
  CHANNEL(char *line);
  CHANNEL(DEVICETYPE type, QString& chName, QString& ifcName);
  
  
  CHANNEL& operator=(CHANNEL &ch);
  QString getDeviceName(DEVICETYPE u32DeviceType);
  bool getChannelConfig(QString Serial, DEVICETYPE type);
  unsigned long getNumber(char *line, QString tok, int base);
  double getDouble(char *line, QString tok);
  double getCANClock(void);
  inline int getMinor(void) { return m_minor; }
  inline int getMajor(void) { return m_major; }
  inline unsigned long getDprAddr(void){ return m_dpram; }
  inline unsigned long getSerialnumber(void){ return m_serialNumber; }
  inline void setSerialnumber(unsigned long serial){ m_serialNumber = serial; }
  inline unsigned long getPhysChannel(void){ return m_physChannel; }
  inline unsigned long getIrq(void){ return m_irq; }
  inline unsigned long getIoPort(void){ return m_ioport; }
  
  ~CHANNEL();
  
  private:   
    double        m_CAN_clock;
    int           m_major;
    int           m_minor;
    unsigned long m_dpram;
    unsigned long m_ioport;
    unsigned long m_irq;    
    unsigned long m_physChannel;
    unsigned long m_serialNumber;
    
} ;


// baudrate settings

typedef struct
{
  double fBaudrate;
  USIGN8 u8Prescaler;
  USIGN8 u8Tseg1;
  USIGN8 u8Tseg2;
  USIGN8 u8Sjw;
  union
  {
   USIGN8 u8Sam;
   USIGN8 u8Div8x;
  };
  USIGN32 u32AccCodeStd;
  USIGN32 u32AccMaskStd;
  USIGN32 u32AccCodeXtd;
  USIGN32 u32AccMaskXtd;
  union
  {
   USIGN32 u32OutputCtrl;
   bool    isListenOnly;
  };
  bool    bEnableAck;
  bool    bEnableErrorframe;
  
} CAN_CONTROLER_INIT,*PCAN_CONTROLER_INIT;

#endif // CHANNEL_H
