/***************************************************************************
 *   Copyright (C) 2007-2015 by Softing Automation GmbH                    *
 *   support.automation@softing.com                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "ctreewidgetitem.h"
#include "qstring.h"
#include "qfile.h"

static const char* const img1_scimwidgetbase[] =
{
  "16 16 4 1",
  "b c #008000",
  "# c #00ff00",
  "a c #808080",
  ". c #ffffff",
  "................",
  "................",
  "..............#a",
  "............##ba",
  "...........#baa.",
  "..........#ba...",
  ".........#ba....",
  "........##ba....",
  ".......##ba.....",
  ".###..##ba......",
  ".ab##.#ba.......",
  "..ab##bba.......",
  "..abb#ba........",
  "...abbba........",
  "...abba.........",
  "....aa.........."
};

static const char* const img2_scimwidgetbase[] =
{
  "18 16 2 1",
  ". c #ffffff",
  "# c #ff0000",
  "......######......",
  "....##......##....",
  "...#...####...#...",
  "..#...######...#..",
  "..#...##..##...#..",
  ".#....##..##....#.",
  ".#.......###....#.",
  ".#.......##.....#.",
  ".#......###.....#.",
  ".#......##......#.",
  ".#......##......#.",
  "..#............#..",
  "..#.....##.....#..",
  "...#....##....#...",
  "....##......##....",
  "......######......"
};


cTreeWidgetItem::cTreeWidgetItem(QTreeWidgetItem *tree, DEVICETYPE dev, NODE_TYPE nodeType, QString name) : QTreeWidgetItem(tree)
{
  CHANNEL Ch(dev,name,name);
  
  m_devicetype = dev;
  m_nodeType = nodeType;
//   m_CANChannel.m_deviceType = dev;
//   m_CANChannel.m_channelName = name;
//   m_CANChannel.m_ifcName = name;
  m_CANChannel = Ch;
  m_pac_config = NULL;
  m_ppro_config = NULL;
    
  this->setText(name);

}

cTreeWidgetItem::cTreeWidgetItem(QTreeWidgetItem *tree, CHANNEL *channel, NODE_TYPE nodeType) : QTreeWidgetItem(tree)
{
  QString ch_name = QString("%1").arg(channel->getSerialnumber(),9,10,QLatin1Char('0'));   
  QString ph = QString("%1").arg(channel->getPhysChannel()); 
  QPixmap pixmap_ok = img1_scimwidgetbase;
  QPixmap pixmap_error = img2_scimwidgetbase;
  
  
  ch_name = ch_name + "_" + ph;
  
  m_devicetype = (DEVICETYPE)channel->m_deviceType;
  m_nodeType = nodeType;
  
  channel->getChannelConfig(ch_name, channel->m_deviceType);
  m_CANChannel = *channel;
  m_pac_config = NULL;
  m_ppro_config = NULL;
  
  this->setText(channel->m_channelName);
  
  cTreeWidgetItem *pItem = (cTreeWidgetItem *)tree;
  
  pItem->m_CANChannel.m_Error = channel->m_Error;
  if(channel->m_Error)
  {
    pItem->setPixmap(0,pixmap_error);
    setPixmap(0,pixmap_error);
  }
  else
  {
    pItem->setPixmap(0,pixmap_ok);
    setPixmap(0,pixmap_ok);
  }
  
  if(nodeType == NODE_ROOT)
  {
    QTreeWidget *wholeTree = (QTreeWidget *)tree;
    wholeTree->addTopLevelItem(this);
  }
}

cTreeWidgetItem::cTreeWidgetItem(QTreeWidget *tree) : QTreeWidgetItem(tree)
{
  QIcon icon("softing.ico");
  m_devicetype = UNKNOWN_DEV;
  m_nodeType = NODE_ROOT;
  this->setText("Device Tree");  
  this->setIcon(0,icon);
  m_pac_config = NULL;
  m_ppro_config = NULL;
  tree->takeTopLevelItem(0);
  tree->addTopLevelItem(this);
  
}

cTreeWidgetItem::~cTreeWidgetItem()
{
   int d = this->childCount();
   
  if(d == 3)
  { // for every channel exits a /dev file
    removeDevFile(&this->m_CANChannel);
  }
  
}

bool cTreeWidgetItem::getDialogCanceled(void)
{
  return m_config_canceled;
}

DEVICETYPE cTreeWidgetItem::getDeviceType()
{
  return m_devicetype;
}

NODE_TYPE cTreeWidgetItem::getNodeType()
{
  return m_nodeType;
}

int cTreeWidgetItem::numOfChilds(void)
{
  return this->childCount();
}

void cTreeWidgetItem::setPixmap(int col, QPixmap pixmap)
{
  this->setIcon(col,pixmap);
} 


cTreeWidgetItem* cTreeWidgetItem::itemBelow(void)
{
  return (cTreeWidgetItem*)this->treeWidget()->itemBelow(this);
}

cTreeWidgetItem* cTreeWidgetItem::itemAbove(void)
{
  return (cTreeWidgetItem*)this->treeWidget()->itemAbove(this);
}

cTreeWidgetItem* cTreeWidgetItem::findSelectedNode(void)
{
  QList<QTreeWidgetItem*> list = this->treeWidget()->selectedItems();
  
  if(list.isEmpty())
  {
    return NULL;
  }
  else
  {
    return (cTreeWidgetItem*)list[0];
  }
}

cTreeWidgetItem* cTreeWidgetItem::findItems(QString name, DEVICETYPE dev, NODE_TYPE nodeType)
{
  QList<QTreeWidgetItem*> list = this->treeWidget()->findItems(name,Qt::MatchFixedString | Qt::MatchCaseSensitive | Qt::MatchRecursive, 0);
  
  for(int i=0; i<list.count(); i++)
  {
    cTreeWidgetItem* item = (cTreeWidgetItem*) list[i];
    if((item->getDeviceType() == dev) && (item->getNodeType()==nodeType))
    {
      return item;
    }
  }
  
  return NULL;
  
}

void cTreeWidgetItem::get_config()
{  
  if((m_CANChannel.m_deviceType == CANPROPCIE) || (m_CANChannel.m_deviceType == CAN104PLUS) || (m_CANChannel.m_deviceType == CANPROUSB))
  {
    m_ppro_config = new ch_cfg_pro(&m_CANChannel);
    get_pro_config();
    delete m_ppro_config;
    m_ppro_config = NULL;
  }
  else
  {
    m_pac_config = new ch_cfg(&m_CANChannel);
    get_ac_config();
    delete m_pac_config;
    m_pac_config = NULL;
  }
 
}

void cTreeWidgetItem::get_ac_config()
{  
  m_pac_config->show_config();
  
  if(m_pac_config->m_ok)
  {
    m_config_canceled = false;
    if(m_pac_config->m_channelName != m_CANChannel.m_channelName)
    {
      // channel name has changed!
      QList<QTreeWidgetItem*> list = this->treeWidget()->findItems(m_pac_config->m_channelName,Qt::MatchFixedString | Qt::MatchCaseSensitive | Qt::MatchRecursive, 0);
      
      if(list.count()>0)
      {
        // channel name already exists
        m_on_Error.information(
          this->treeWidget(),
                               "Softing CAN configuration Tool ERROR message",
                               "Sorry, the name is already defined for an other channel!",
                               QMessageBox::Abort,
                               QMessageBox::NoButton,
                               QMessageBox::NoButton
        );
        m_pac_config->m_channelName = m_CANChannel.m_channelName;
        return;
      }
      removeDevFile(&this->m_CANChannel, false);
      m_CANChannel.m_channelName = m_pac_config->m_channelName;
      setText(m_CANChannel.m_channelName);
    }
    m_CANChannel.m_fBaudrate = m_pac_config->m_fBaudrate;
    m_CANChannel.m_prescaler = m_pac_config->m_prescaler;
    //     m_CANChannel.m_sam = m_pac_config->m_Div8X/8;
    m_CANChannel.m_sjw = m_pac_config->m_sjw;
    m_CANChannel.m_tseg1 = m_pac_config->m_tseg1;
    m_CANChannel.m_tseg2 = m_pac_config->m_tseg2;
    m_CANChannel.m_u32AccCodeStd = m_pac_config->m_u32AccCodeStd;
    m_CANChannel.m_u32AccCodeXtd = m_pac_config->m_u32AccCodeXtd;
    m_CANChannel.m_u32AccMaskStd = m_pac_config->m_u32AccMaskStd;
    m_CANChannel.m_u32AccMaskXtd = m_pac_config->m_u32AccMaskXtd;
    m_CANChannel.m_u32OutputCtrl = m_pac_config->m_u32OutputCtrl;
    m_CANChannel.m_detectErrorframes = m_pac_config->is_errorframe_detection_enabled(); 
    m_CANChannel.m_enableAck = m_pac_config->is_ack_enabled();
    
  }
  else
  {
    m_config_canceled = true;
  }
}

void cTreeWidgetItem::get_pro_config()
{  
  m_ppro_config->show_config();
  
  if(m_ppro_config->m_ok)
  {
    m_config_canceled = false;
    if(m_ppro_config->m_channelName != m_CANChannel.m_channelName)
    {
      // channel name has changed!
      QList<QTreeWidgetItem*> list = this->treeWidget()->findItems(m_ppro_config->m_channelName,Qt::MatchFixedString | Qt::MatchCaseSensitive | Qt::MatchRecursive, 0);
      
      if(list.count()>0)
      {
        // channel name already exists
        m_on_Error.information(
          this->treeWidget(),
                               "Softing CAN configuration Tool ERROR message",
                               "Sorry, the name is already defined for an other channel!",
                               QMessageBox::Abort,
                               QMessageBox::NoButton,
                               QMessageBox::NoButton
        );
        m_ppro_config->m_channelName = m_CANChannel.m_channelName;
        return;
      }
      removeDevFile(&this->m_CANChannel, false);
      m_CANChannel.m_channelName = m_ppro_config->m_channelName;
      setText(m_CANChannel.m_channelName);
    }
    m_CANChannel.m_fBaudrate = m_ppro_config->m_fBaudrate;
    m_CANChannel.m_prescaler = m_ppro_config->m_prescaler;
    m_CANChannel.m_sam = m_ppro_config->m_Div8X/8;
    m_CANChannel.m_sjw = m_ppro_config->m_sjw;
    m_CANChannel.m_tseg1 = m_ppro_config->m_tseg1;
    m_CANChannel.m_tseg2 = m_ppro_config->m_tseg2;
    m_CANChannel.m_u32AccCodeStd = m_ppro_config->m_u32AccCodeStd;
    m_CANChannel.m_u32AccCodeXtd = m_ppro_config->m_u32AccCodeXtd;
    m_CANChannel.m_u32AccMaskStd = m_ppro_config->m_u32AccMaskStd;
    m_CANChannel.m_u32AccMaskXtd = m_ppro_config->m_u32AccMaskXtd;
    m_CANChannel.m_u32OutputCtrl = m_ppro_config->m_u32OutputCtrl;
    m_CANChannel.m_detectErrorframes = m_ppro_config->is_errorframe_detection_enabled(); 
    m_CANChannel.m_enableAck = m_ppro_config->is_ack_enabled();
    
  }
  else
  {
    m_config_canceled = true;
  }
}

////////////////////////////////////////////////////////////////
// class cTreeWidgetItem
////////////////////////////////////////////////////////////////




void cTreeWidgetItem::sort(void)
{
  //QListViewItem::sort();
}

/**
 ******************************************************************
 * @brief  set text (virtual overloaded function). Will be called, if the channel name is changing.
 * @brief  changes the device file for a CAN channel and the channel name. 
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @param  column: column of the entry
 * @param  text:   text, which should be set
 * @return -
 *
 ******************************************************************
 */

void cTreeWidgetItem::setText ( const QString & text )
{
  this->m_CANChannel.m_channelName = text.simplified();
  if((this->m_nodeType == NODE_CHANNEL) && !this->m_CANChannel.m_Error)
  {
    if(!existDevFile(&this->m_CANChannel))
    {
      // remove old file
      //removeDevFile(&this->m_CANChannel, false);
      // create new file
      createDevFile(&this->m_CANChannel);
    }
  }
  QTreeWidgetItem::setText(0,this->m_CANChannel.m_channelName);
}

bool cTreeWidgetItem::existDevFile(CHANNEL *pCh)
{
  int exist;
  QString file;
  volatile int major, minor;
  
  file = DEVFILEPREFIX;
  file += pCh->m_channelName;
  struct stat info;
  
  exist = stat(file.toAscii(), &info);
  
  if(exist != 0)
  {
    // no file with this name exists
    return false;
  }
  
  // file exists, but may be major on minor number do not fit?
  major = major(info.st_rdev);
  minor = minor(info.st_rdev);
  
  if((pCh->getMajor() == major) && (pCh->getMinor() == minor))
  {
    //exactly the same file exists
    return true;
  }
  return false;
}

/**
 ******************************************************************
 * @brief  removes a device file.
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @param  pCh: pointer to the channel
 * @return -
 *
 ******************************************************************
 */

int cTreeWidgetItem::removeDevFile(CHANNEL *pCh, bool ignore_errors)
{
  QString file;
  int ret=0;
  bool isOk;
  
  file = DEVFILEPREFIX;
  file += pCh->m_channelName;
  
  if(!QFile::exists(file))
  {
    // nothing to do; file does not exist.
    return 0;
  }

  isOk = QFile::remove(file);
  
  if(ignore_errors)
    return 0;

  if(!isOk)
  {
    ret = errno;
    switch(ret)
    {
      case EACCES:
      case EPERM:
        file.prepend("INVALID ACCESS RIGHTS! Can not remove: ");
        m_on_Error.information(
            NULL,
        "Critical error while removing a device file!",
        file.toAscii(),
        QMessageBox::Abort,
        QMessageBox::NoButton,
        QMessageBox::NoButton);
     
        break;
      
      default:
        ret = 0;
        break;
    }
  }

  return ret;
}
/**
 ******************************************************************
 * @brief  creates a new device file in /dev.
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @param  pCh: pointer to the channel
 * @return -
 *
 ******************************************************************
 */

void cTreeWidgetItem::createDevFile(CHANNEL *pCh)
{
  QString file;
  QString output;
  
  file = DEVFILEPREFIX;
  file += pCh->m_channelName;
  
  remove(file.toAscii());
  if(pCh->getMinor() == UNDEFINED_VALUE || pCh->getMajor() == UNDEFINED_VALUE)
  { 
    // device is not plugged: minor = undefined or major = undefined!
    return;
  }
  
  pCh->m_isDevFileCreated = !mknod(file.toAscii(), S_IFCHR | 0x1ff,  MKDEV(pCh->getMajor(), pCh->getMinor()));
  
  if(!pCh->m_isDevFileCreated)
  {
    //output = file;
    output = file+output.sprintf("\nmajor: %u, minor: %u\n",pCh->getMajor(), pCh->getMinor());
    showMknodError(output);
  }
  
}

/**
 ******************************************************************
 * @brief  shows the text representation of an error
 * @author Werner Knorsch, Softing AG
 * @date   2006-07-25
 *
 * @param  filename: filename of the file which was accessed while the error happend
 * @return -
 *
 ******************************************************************
 */

void cTreeWidgetItem::showMknodError(QString &filename)
{
  QString strErrorText = filename;
    
  switch(errno)
  {
    case EPERM:
      strErrorText += "\noperation not permitted!";
      break;
    case EINVAL:
      strErrorText += "\nwrong device type!";
      break;
    case EEXIST:
      strErrorText += "\ndevice already exists!";
      break;
    case EFAULT:
      strErrorText += "\ninvalid path!";
      break;
    case EACCES:
      strErrorText += "\nWriting is not possible or the program can not be \n";
      strErrorText += "executed, because of invalid access rights!\n";
      strErrorText += "Login as root or type sudo scim! \n";
      break;
    case ENAMETOOLONG:
      strErrorText += "\npathname too long!";
      break;
    case ELOOP:
    case ENOENT:
    case ENOTDIR:
      strErrorText += "\ninvalid path!";
      break;
    case EROFS:
      strErrorText += "\ncan not create a file on a read only file system!";
      break;
    case ENOMEM:
    case ENOSPC:
      strErrorText += "\nout of memory!";
      break;
   
    
    
      default: strErrorText += strErrorText.sprintf("\nerror %u occurred!",errno);
      break;
  }
  
  m_on_Error.information(
      NULL,
  "Critical error while creating a device file",
  strErrorText.toAscii(),
  QMessageBox::Abort,
  QMessageBox::NoButton,
  QMessageBox::NoButton);
    
  
}

