/***************************************************************************
 *   Copyright (C) 2007-2015 by Softing Industrial Automation GmbH         *
 *   support.automation@softing.com                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef CTREEWIDGETITEM_H
#define CTREEWIDGETITEM_H


#include <QTreeWidgetItem>
#include <QMessageBox>

#include "ui_ch_config.h"
#include "ui_ch_config_pro.h"
#include "ch_cfg.h"
#include "ch_cfg_pro.h"


typedef enum
{
  NODE_ROOT,
  NODE_DEVICE_TYPE,
  NODE_DEVICE,
  NODE_CHANNEL
}NODE_TYPE;



class cTreeWidgetItem : public QObject,QTreeWidgetItem
{

   Q_OBJECT
  
public:
    cTreeWidgetItem(QTreeWidgetItem *tree, DEVICETYPE dev, NODE_TYPE nodeType, QString name);
    cTreeWidgetItem(QTreeWidgetItem *tree, CHANNEL *ch, NODE_TYPE nodeType);
    cTreeWidgetItem(QTreeWidget *tree);
    virtual ~cTreeWidgetItem();
    void get_config();
    DEVICETYPE getDeviceType();
    NODE_TYPE getNodeType(void);
    cTreeWidgetItem *itemBelow(void);
    cTreeWidgetItem *itemAbove(void);    
    cTreeWidgetItem *findItems(QString name, DEVICETYPE dev, NODE_TYPE nodeType);    
    cTreeWidgetItem *findSelectedNode(void);
    int numOfChilds(void);
    void setPixmap(int col, QPixmap pixmap);
    bool getDialogCanceled(void);
    
    //kde 3 scim
    virtual void setText ( const QString & text );
    virtual void sort(void);
    
    
    CHANNEL m_CANChannel;
    // end kde3 scim

    
private:
    DEVICETYPE m_devicetype;
    NODE_TYPE  m_nodeType;
    
    
    QTreeWidgetItem *m_pItem;
    
    ch_cfg *m_pac_config;
    ch_cfg_pro *m_ppro_config;
    bool m_config_canceled;
    
    bool existDevFile(CHANNEL *pCh);
    int removeDevFile(CHANNEL *pCh, bool ignore_errors=true);
    void createDevFile(CHANNEL *pCh);
    void showMknodError(QString &filename);
    void get_ac_config();
    void get_pro_config();
    QMessageBox m_on_Error;
    
    
};

#endif // CTREEWIDGETITEM_H
