/***************************************************************************
 *   Copyright (C) 2007-2015 by Softing Automation GmbH                    *
 *   support.automation@softing.com                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "scim.h"
#include <qprocess.h>
#include <sys/stat.h>

extern const char version[];

/**
 ******************************************************************
 * @brief  Constructor
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

SCIM::SCIM()
    : KMainWindow()
{
    // tell the KXmlGuiWindow that this is indeed the main widget


    QWidget* w = new QWidget(this);
    cTreeWidgetItem *pTree;
    QString titleString = QString("Softing CAN Interface Manager V %1").arg(version);

    if(!runningAsRoot())
    {
        ::exit(1);
    }

    setCentralWidget(w);
    m_ui.setupUi(w);
 
    QPixmap sftPixmap;
    
    sftPixmap.load("./softing.ico");

    
//     this->setWindowIcon(sftPixmap);
    this->setWindowTitle(titleString);
    
    m_isVcanacdLoaded = false;
    m_isVcanpcidLoaded = false;
    m_isCancard_csLoaded = false;
    m_isVcanecoLoaded = false;
    m_isVcanproLoaded = false;
    m_isVcanpcieLoaded = false;

    // add context menu entries -> right click on the device tree
    m_ui.treeWidget->setContextMenuPolicy(Qt::ActionsContextMenu);
    m_menu.setTitle("actions");
    m_ac_edit = new QAction("Edit",&m_menu);
    m_ac_refresh = new QAction("Refresh",&m_menu);
    m_ac_remove = new QAction("Remove", &m_menu);

    m_ui.treeWidget->addAction(m_ac_edit);
    m_ui.treeWidget->addAction(m_ac_refresh);
    m_ui.treeWidget->addAction(m_ac_remove);

    m_ac_edit->setDisabled(true);
    m_ac_edit->setVisible(false);
    m_ac_remove->setDisabled(true);
    m_ac_remove->setVisible(false);

    m_ui.bt_edit->setDisabled(true);
    m_ui.bt_remove->setDisabled(true);

    m_ui.bt_refresh->setToolTip("push the button to refresh the device tree!");


    // insert the root of the device tree
    pTree = new cTreeWidgetItem(m_ui.treeWidget);
    pTree = pTree;

    m_ui.treeWidget->topLevelItem(0)->setExpanded(true);

    // connect the signals of the buttons and the context menue to the slots
    connect(m_ui.bt_edit, SIGNAL(clicked(bool)), this, SLOT(on_treeItemRightClicked(bool)));
    connect(m_ui.bt_refresh, SIGNAL(clicked(bool)), this, SLOT(on_scan()));
    connect(m_ui.treeWidget, SIGNAL(itemClicked(QTreeWidgetItem*,int)), this, SLOT(on_treeItemClicked(QTreeWidgetItem*,int)));
    connect(m_ac_edit, SIGNAL(triggered(bool)), this, SLOT(on_treeItemRightClicked(bool)));
    connect(m_ac_refresh, SIGNAL(triggered(bool)), this, SLOT(on_scan()));
    connect(m_ui.bt_about, SIGNAL(clicked(bool)), this, SLOT(on_about()));
    connect(m_ui.treeWidget, SIGNAL(itemSelectionChanged()), this, SLOT(on_treeItemChanged()));
    connect(m_ac_remove, SIGNAL(triggered(bool)), this, SLOT(on_remove_ifc()));

    // scan for installed or configured devices
    on_scan();
}

/**
 ******************************************************************
 * @brief  Destructor
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

SCIM::~SCIM()
{
    delete m_ac_edit;
    delete m_ac_refresh;
    delete m_ac_remove;
}

bool SCIM::runningAsRoot()
{
    struct passwd *pw;
    uid_t uid;

    uid = geteuid();
    pw = getpwuid(uid);

    if((uid!=0) || (pw->pw_gid != 0))
    {
        QString strErrorText = "This program must be run with root privileges,\n";
        strErrorText += "because it must create/rename device files !\n";
        strErrorText += "Please LOGIN AS ROOT, or type sudo scim !\n";

        m_on_Error.information(
            NULL,
            "INVALID ACCESS RIGHTS",
            strErrorText.toAscii(),
            QMessageBox::Abort,
            QMessageBox::NoButton,
            QMessageBox::NoButton);
        return false;
    }

    return true;
}

/**
 ******************************************************************
 * @brief  The slot called, if somebody clicks with the right mouse on the device tree
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void SCIM::on_treeItemRightClicked(bool checked)
{
    cTreeWidgetItem *item = (cTreeWidgetItem*)m_ui.treeWidget->topLevelItem(0);

    // the node, on which the mouse click ist done!
    item = item->findSelectedNode();

    checked=checked;

    // null pointer: we can do nothing.
    if(item == NULL)
        return;

    // only channels can be edited.
    if(item->getNodeType() != NODE_CHANNEL)
    {
        return;
    }

    on_edit_channel();

    return;
}

QString SCIM::getShellOutput(QString cmd)
{
    QProcess *process = new QProcess( );
    QString result;

    process->start(cmd);
    process->closeWriteChannel();

    process->waitForFinished();

    QByteArray qba=process->readAllStandardOutput();
    result = qba;

    return result;
}



void SCIM::on_treeItemClicked(QTreeWidgetItem* treeitem, int col)
{

    cTreeWidgetItem *item = (cTreeWidgetItem*)treeitem; //(cTreeWidgetItem*)m_ui.treeWidget->topLevelItem(0);

    col=col;

    switch(item->getNodeType())
    {
    case NODE_ROOT:
        setRootDetails();
        break;

    case NODE_DEVICE_TYPE:
        setDriverDetails(item);
        break;

    case NODE_DEVICE:
        setDeviceDetails(item);
        break;

    case NODE_CHANNEL:
        setChannelDetails(item);
        break;
    }

    return;
}

void SCIM::on_about()
{
    QDialog* w = new QDialog(this);
    QString versionText, copyrightText;
    QDateTime aktDateTime = QDateTime::currentDateTime();
    QDate aktDate=aktDateTime.date();
    int year = aktDate.year();
    QString strYear;
    
    if (year<2015) year = 2015;
    
    strYear = QString("%1").arg(year);

   
    m_about.setupUi(w);
    versionText = "Softing CAN Interface Manager V " + QString(version);
    m_about.lb_version->setText(versionText);
    copyrightText = "Copyright (C) 2006 - " + strYear;
    m_about.lb_copyright->setText(copyrightText);
    w->exec();

}

void SCIM::on_treeItemChanged(void)
{
    cTreeWidgetItem *item = (cTreeWidgetItem*)m_ui.treeWidget->topLevelItem(0);

    item = item->findSelectedNode();

    if(item==NULL)
        return;

    switch(item->getNodeType())
    {
    case NODE_ROOT:
        m_ac_edit->setDisabled(true);
        m_ac_edit->setVisible(false);
        m_ui.bt_edit->setDisabled(true);
        m_ui.bt_edit->setToolTip("The whole tree cannot be edited. Only a selected channel can be edited!");
        m_ac_remove->setDisabled(true);
        m_ac_remove->setVisible(false);
        m_ui.bt_remove->setToolTip("The whole tree cannot be removed. Only selected Interfaces can be removed!");
        m_ui.bt_remove->setDisabled(true);

        break;

    case NODE_DEVICE_TYPE:
        m_ac_edit->setDisabled(true);
        m_ac_edit->setVisible(false);
        m_ui.bt_edit->setDisabled(true);
        m_ui.bt_edit->setToolTip("The whole Interface type cannot be edited. Only a selected channel can be edited!");
        m_ac_remove->setDisabled(true);
        m_ac_remove->setVisible(false);
        m_ui.bt_remove->setDisabled(true);
        m_ui.bt_remove->setToolTip("You cannot remove an Interface type, only Interfaces can be removed!");
        break;

    case NODE_DEVICE:
        m_ac_edit->setDisabled(true);
        m_ac_edit->setVisible(false);
        m_ui.bt_edit->setDisabled(true);
        m_ui.bt_edit->setToolTip("The whole Interface cannot be edited. Only a selected channel can be edited!");
        m_ac_remove->setDisabled(false);
        m_ac_remove->setVisible(true);
        m_ui.bt_remove->setDisabled(false);
        m_ui.bt_remove->setToolTip("push the button to delete the Interface!");
        break;

    case NODE_CHANNEL:
        m_ac_edit->setDisabled(false);
        m_ac_edit->setVisible(true);
        m_ui.bt_edit->setDisabled(false);
        m_ui.bt_edit->setToolTip("push the button to change baudrate, filter settings and other settings");
        m_ac_remove->setDisabled(true);
        m_ac_remove->setVisible(false);
        m_ui.bt_remove->setDisabled(true);
        m_ui.bt_remove->setToolTip("You cannot remove a channel, only the whole Interfaces can be removed!");
        break;
    }
}

void SCIM::on_remove_ifc(void)
{
    cTreeWidgetItem *item = (cTreeWidgetItem*)m_ui.treeWidget->topLevelItem(0);

    item = item->findSelectedNode();

    if(item == NULL)
        return;

    delete item;

    saveDeviceTree();

}


/**
 ******************************************************************
 * @brief  inserts an interface into the device tree
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-18
 *
 * @param  nodeName: name of the parent node
 * @param  ifcName:  name of the interface
 * @param  pixmap:   pixmap for the new node in the tree
 * @return -
 *
 ******************************************************************
 */

bool SCIM::insertInterfaces(QString nodeName, QString ifcName, DEVICETYPE type, NODE_TYPE nodeType)
{
    cTreeWidgetItem *pTopItem = (cTreeWidgetItem*)m_ui.treeWidget->topLevelItem(0);
    cTreeWidgetItem *pIfcType = pTopItem->findItems(ifcName,type,NODE_DEVICE_TYPE);
    cTreeWidgetItem *pItem = pTopItem->findItems(nodeName,type,NODE_CHANNEL); //findNode(pChannel);
    cTreeWidgetItem *pitem = pTopItem->findItems(nodeName,type,NODE_DEVICE);


    QString name = nodeName;

    if(!pIfcType)
    {
        pIfcType = new cTreeWidgetItem(m_ui.treeWidget->topLevelItem(0), type, NODE_DEVICE_TYPE, ifcName);
    }

    if(nodeType==NODE_DEVICE_TYPE)
        return false;


    if((pitem==NULL) || (pItem != NULL))
    {
        // father node can not be found or node exists already!
        if(pItem)
        {
            // node exists already
//       pItem->setPixmap(0,pixmap);
//       pItem->m_CANChannel.m_Error = 0;
            return false;
        }
        else
        {
            //interface node must be created
            cTreeWidgetItem * newIfc = new cTreeWidgetItem((QTreeWidgetItem *)pIfcType, type, NODE_DEVICE, nodeName);
            newIfc = newIfc;
            return true;
        }
    }

    return false;

}

/**
 ******************************************************************
 * @brief  inserts an channel into the device tree
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-18
 *
 * @param  pChannel: pointer to the channel
 * @param  ifcName:  name of the interface
 * @param  pixmap:   pixmap for the new node in the tree
 * @return -
 *
 ******************************************************************
 */

bool SCIM::insertInterfaces(CHANNEL *pChannel, QString ifcName, DEVICETYPE type, NODE_TYPE nodeType)
{
    cTreeWidgetItem *pTopItem = (cTreeWidgetItem*)m_ui.treeWidget->topLevelItem(0);
    cTreeWidgetItem *pItem = pTopItem->findItems(pChannel->m_channelName,type,NODE_CHANNEL); //findNode(pChannel);
    cTreeWidgetItem *pitem = pTopItem->findItems(ifcName,type,NODE_DEVICE);
    QString name = pChannel->m_channelName;
    QString devFile;

    if(!pitem || pItem)
    {   // father node can not be found or node exists already!
        if(pItem)
        {
//       pItem->setPixmap(0,pixmap);
//       pItem->m_CANChannel.m_Error = 0;
            pItem->setText(pChannel->m_channelName);
        }

        return false;
    }

    pItem = new cTreeWidgetItem((QTreeWidgetItem*)pitem, pChannel, nodeType);

    if(pitem->getNodeType() == NODE_CHANNEL)
    {   //channel
        cTreeWidgetItem *pParent = pItem->itemAbove();
        cTreeWidgetItem *pNode = (cTreeWidgetItem *)pParent;

        pItem->m_CANChannel = *pChannel;
        pItem->m_CANChannel.setSerialnumber(pNode->m_CANChannel.getSerialnumber());

    }
    QFont myFont = m_ui.treeWidget->font();
    myFont.setBold(true);
    m_ui.treeWidget->setFont(myFont);

    pItem->setText(pChannel->m_channelName);
//   pItem->setPixmap(0,pixmap);
//
//   pitem->setPixmap(0,pixmap);
//   pitem->m_CANChannel.m_Error = pItem->m_CANChannel.m_Error;

    m_ui.treeWidget->repaint();

    myFont.setBold(false);
    m_ui.treeWidget->setFont(myFont);

    if((pitem->getNodeType()>NODE_DEVICE_TYPE) && (pItem->getNodeType()>NODE_DEVICE_TYPE))
    {
        if((pitem->m_CANChannel.getSerialnumber()==0) || (pItem->m_CANChannel.getSerialnumber()==0))
        {
            volatile static int test = 0;

            test++;
            if(test>3) test--;
        }
    }



    return true;

}


/**
 ******************************************************************
 * @brief  inserts all plugged devices into the device tree
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-18
 *
 * @param  pProcDir:    directory, where the kernelmode driver stores the information about the int.
 * @param  pInterface:  name of the interface type
 * @param  type:        numerical representation of the interface type
 * @return -
 *
 ******************************************************************
 */

int SCIM::insertPluggedDevices(QString ProcDir, QString Interface, DEVICETYPE type)
{

    struct dirent **namelist;
    int n, min=0;
//   bool is_ifc_inserted, is_ch_inserted;
    int i;

    Interface = Interface;

    n = scandir(ProcDir.toAscii(), &namelist, 0, alphasort);
    if (n < 0)
    {
        return -1;
    }
    else
    {
//     while(n--)
        for(i=0; i<n; i++)
        {

            if(
                strcmp(namelist[i]->d_name,"..") &&
                strcmp(namelist[i]->d_name,".")
            )
            {
                CHANNEL CanCh(namelist[i]->d_name,type);


                insertInterfaces(CanCh.m_ifcName, CanCh.m_driverNode,type, NODE_DEVICE);
                insertInterfaces(&CanCh,CanCh.m_ifcName,type, NODE_CHANNEL);
                min++;

            }

            free(namelist[i]);
        }
        free(namelist);
    }

    return min;

}


/**
 ******************************************************************
 * @brief  clears the device details list view
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-18
 *
 * @return -
 *
 ******************************************************************
 */

void SCIM::clearDeviceDetails(void)
{
    QTableWidget * pTable = m_ui.tableWidget;

    pTable->clearContents();
    pTable->setColumnWidth(1,200);
    pTable->setColumnWidth(0,145);
    pTable->setShowGrid(false);
    pTable->verticalHeader()->setVisible(false);

}

void SCIM::setDeviceDetails(cTreeWidgetItem *pItem)
{
    QString textLine;
    int row = 0;
    QTableWidget * pTable = m_ui.tableWidget;
    QFont myFont = m_ui.tableWidget->font();
    CHANNEL ch;
    QString status;

    myFont.setPointSize(8);
    myFont.setBold(true);
    m_ui.tableWidget->setFont(myFont);


    clearDeviceDetails();

    pTable->setColumnWidth(1,300);
    pTable->setColumnWidth(0,180);

    setTableLine("Interface:", pItem->m_CANChannel.m_ifcName,row++);
    setTableLine(textLine, "",row++);
    setTableLine("serial number:", QString("%1").arg(pItem->m_CANChannel.getSerialnumber(),9,10,QLatin1Char('0')) ,row++);
    if(pItem->m_CANChannel.m_Error)
    {
        setTableLine("hardware access:", "not possible!" ,row++);
    }
    else
    {
        setTableLine("hardware access:", "successful!" ,row++);
    }

}

void SCIM::setRootDetails(void)
{
    QString textLine;
    int row = 0;
    QFont myFont = m_ui.tableWidget->font();
    CHANNEL ch;
    QString status;

    myFont.setPointSize(8);
    myFont.setBold(true);
    m_ui.tableWidget->setFont(myFont);


    clearDeviceDetails();

    setTableLine("Softing CAN Interfaces", "",row++);
    textLine = QString("%1").arg(version);
    setTableLine("SCIM version", textLine,row++);

    if(getShellOutput("modinfo --version").isEmpty())
    {
        setTableLine("problem:", "modinfo not installed!",row++);
        setTableLine("", "Further information not",row++);
        setTableLine("", "available!",row++);
    }
    else
    {
        textLine = getShellOutput("modinfo -F version vcanpcid");
        if(textLine.isEmpty()) textLine = "not installed!";
        setTableLine("CAN-AC PCI mod. version", textLine,row++);

        textLine = getShellOutput("modinfo -F version vcanacd");
        if(textLine.isEmpty()) textLine = "not installed!";
        setTableLine("CAN-AC 104 mod. version", textLine,row++);

        textLine = getShellOutput("modinfo -F version vcanpcie");
        if(textLine.isEmpty()) textLine = "not installed!";
        setTableLine("CANpro PCIe mod. version", textLine,row++);

        textLine = getShellOutput("modinfo -F version vcanprod");
        if(textLine.isEmpty()) textLine = "not installed!";
        setTableLine("CANpro 104+ mod. version", textLine,row++);

        textLine = getShellOutput("modinfo -F version canpro_usb");
        if(textLine.isEmpty()) textLine = "not installed!";
        setTableLine("CANpro USB mod. version", textLine,row++);
    }

}

/**
 ******************************************************************
 * @brief  writes the details of a driver type into the list view
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-18
 *
 * @param  pNode: pointer to the driver node.
 * @return -
 *
 ******************************************************************
 */

void SCIM::setDriverDetails(cTreeWidgetItem *pItem)
{

    QString textLine;
    int row = 0;
    QFont myFont = m_ui.tableWidget->font();
    CHANNEL ch;
    QString status;

    myFont.setPointSize(8);
    myFont.setBold(true);
    m_ui.tableWidget->setFont(myFont);


    clearDeviceDetails();

    if(getShellOutput("modinfo --version").isEmpty())
    {
        status = "modinfo not installed!";
    }
    else
    {
        status = "not installed!";
    }


    if(pItem->getDeviceType() == CANCARD2)
    {
        textLine = getShellOutput("modinfo -F description cancard_cs");
        if(textLine.isEmpty())
        {
            textLine = ch.getDeviceName(pItem->getDeviceType()) + " driver node";
            setTableLine("description:", textLine, row++);
            setTableLine("driver name:", "cancard_cs.ko",row++);
            setTableLine("status:", status,row++);
        }
        else
        {
            setTableLine("description:", textLine,row++);
            setTableLine("driver name:", "cancard_cs.ko",row++);
            textLine = getShellOutput("modinfo -F license cancard_cs");
            setTableLine("license:", textLine,row++);
            textLine = getShellOutput("modinfo -F version cancard_cs");
            setTableLine("version:", textLine,row++);
            textLine = getShellOutput("modinfo -F vermagic cancard_cs");
            setTableLine("", textLine,row++);

            textLine = (QDir(PROC_PCMCIA).exists()) ? "loaded and working!" : "not loaded!";

            setTableLine("status:", textLine,row++);
        }

    }

    if(pItem->getDeviceType() == CANPCI)
    {
        textLine = getShellOutput("modinfo -F description vcanpcid");
        if(textLine.isEmpty())
        {
            textLine = ch.getDeviceName(pItem->getDeviceType()) + " driver node";
            setTableLine("description:", textLine, row++);
            setTableLine("driver name:", "vcanpcid.ko",row++);
            setTableLine("status:", status,row++);
        }
        else
        {
            setTableLine("description:", textLine,row++);
            setTableLine("driver name:", "vcanpcid.ko",row++);
            textLine = getShellOutput("modinfo -F license vcanpcid");
            setTableLine("license:", textLine,row++);
            textLine = getShellOutput("modinfo -F version vcanpcid");
            setTableLine("version:", textLine,row++);
            textLine = getShellOutput("modinfo -F vermagic vcanpcid");
            setTableLine("", textLine,row++);
            textLine = (QDir(PROC_PCI).exists()) ? "loaded and working!" : "not loaded!";
            setTableLine("status:", textLine,row++);
        }

    }

    if(pItem->getDeviceType() == CAN_104)
    {
        textLine = getShellOutput("modinfo -F description vcanacd");
        if(textLine.isEmpty())
        {
            textLine = ch.getDeviceName(pItem->getDeviceType()) + " driver node";
            setTableLine("description:", textLine, row++);
            setTableLine("driver name:", "vcanacd.ko",row++);
            setTableLine("status:", status,row++);
        }
        else
        {
            setTableLine("description:", textLine,row++);
            setTableLine("driver name:", "vcanacd.ko",row++);
            textLine = getShellOutput("modinfo -F license vcanacd");
            setTableLine("license:", textLine,row++);
            textLine = getShellOutput("modinfo -F version vcanacd");
            setTableLine("version:", textLine,row++);
            textLine = getShellOutput("modinfo -F vermagic vcanacd");
            setTableLine("", textLine,row++);
            textLine = (QDir(PROC_104).exists()) ? "loaded and working!" : "not loaded!";
            setTableLine("status:", textLine,row++);
        }
    }

    if(pItem->getDeviceType() == CAN104ECO)
    {
        textLine = getShellOutput("modinfo -F description vcaneco");
        if(textLine.isEmpty())
        {
            textLine = ch.getDeviceName(pItem->getDeviceType()) + " driver node";
            setTableLine("description:", textLine, row++);
            setTableLine("driver name:", "vcaneco.ko",row++);
            setTableLine("status:", status,row++);
        }
        else
        {
            setTableLine("description:", textLine,row++);
            setTableLine("driver name:", "vcaneco.ko",row++);
            textLine = getShellOutput("modinfo -F license vcaneco");
            setTableLine("license:", textLine,row++);
            textLine = getShellOutput("modinfo -F version vcaneco");
            setTableLine("version:", textLine,row++);
            textLine = getShellOutput("modinfo -F vermagic vcaneco");
            setTableLine("", textLine,row++);
            textLine = (QDir(PROC_ECO).exists()) ? "loaded and working!" : "not loaded!";
            setTableLine("status:", textLine,row++);
        }
    }

    if(pItem->getDeviceType() == CANPROPCIE)
    {
        textLine = getShellOutput("modinfo -F description vcanpcie");
        if(textLine.isEmpty())
        {
            textLine = ch.getDeviceName(pItem->getDeviceType()) + " driver node";
            setTableLine("description:", textLine, row++);
            setTableLine("driver name:", "vcanpcie.ko",row++);
            setTableLine("status:", status,row++);
        }
        else
        {
            setTableLine("description:", textLine,row++);
            setTableLine("driver name:", "vcanpcie.ko",row++);
            textLine = getShellOutput("modinfo -F license vcanpcie");
            setTableLine("license:", textLine,row++);
            textLine = getShellOutput("modinfo -F version vcanpcie");
            setTableLine("version:", textLine,row++);
            textLine = getShellOutput("modinfo -F vermagic vcanpcie");
            setTableLine("", textLine,row++);
            textLine = (QDir(PROC_PCIE).exists()) ? "loaded and working!" : "not loaded!";
            setTableLine("status:", textLine,row++);
        }
    }

    if(pItem->getDeviceType() == CAN104PLUS)
    {
        textLine = getShellOutput("modinfo -F description vcanprod");
        if(textLine.isEmpty())
        {
            textLine = ch.getDeviceName(pItem->getDeviceType()) + " driver node";
            setTableLine("description:", textLine, row++);
            setTableLine("driver name:", "vcanprod.ko",row++);
            setTableLine("status:", status,row++);
        }
        else
        {
            setTableLine("description:", textLine,row++);
            setTableLine("driver name:", "vcanprod.ko",row++);
            textLine = getShellOutput("modinfo -F license vcanprod");
            setTableLine("license:", textLine,row++);
            textLine = getShellOutput("modinfo -F version vcanprod");
            setTableLine("version:", textLine,row++);
            textLine = getShellOutput("modinfo -F vermagic vcanprod");
            setTableLine("", textLine,row++);
            textLine = (QDir(PROC_104PLUS).exists()) ? "loaded and working!" : "not loaded!";
            setTableLine("status:", textLine,row++);
        }
    }

    if(pItem->getDeviceType() == CANPROUSB)
    {
        textLine = getShellOutput("modinfo -F description canpro_usb");
        if(textLine.isEmpty())
        {
            textLine = ch.getDeviceName(pItem->getDeviceType()) + " driver node";
            setTableLine("description:", textLine, row++);
            setTableLine("driver name:", "canpro_usb.ko",row++);
            setTableLine("status:", status,row++);
        }
        else
        {
            setTableLine("description:", textLine,row++);
            setTableLine("driver name:", "canpro_usb.ko",row++);
            textLine = getShellOutput("modinfo -F license canpro_usb");
            setTableLine("license:", textLine,row++);
            textLine = getShellOutput("modinfo -F version canpro_usb");
            setTableLine("version:", textLine,row++);
            textLine = getShellOutput("modinfo -F vermagic canpro_usb");
            setTableLine("", textLine,row++);
            textLine = (QDir(PROC_USB).exists()) ? "loaded and working!" : "not loaded!";
            setTableLine("status:", textLine,row++);
        }
    }

}

/**
 ******************************************************************
 * @brief  inserts all devices, which are not plugged, but there is an entry in the config file for it.
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-18
 *
 * @return -
 *
 ******************************************************************
 */

void SCIM::insertUnpluggedDevices()
{
    FILE *pDevices;
    char line[1000];
    CHANNEL pCh;

    insertInterfaces("dummy",pCh.getDeviceName(CANPROUSB),CANPROUSB,NODE_DEVICE_TYPE);
    insertInterfaces("dummy",pCh.getDeviceName(CANPROPCIE),CANPROPCIE,NODE_DEVICE_TYPE);
    insertInterfaces("dummy",pCh.getDeviceName(CAN104PLUS),CAN104PLUS,NODE_DEVICE_TYPE);
    insertInterfaces("dummy",pCh.getDeviceName(CANPCI),CANPCI,NODE_DEVICE_TYPE);
    insertInterfaces("dummy",pCh.getDeviceName(CAN_104),CAN_104,NODE_DEVICE_TYPE);
    // insertInterfaces("dummy","CAN-ECOx-104",CAN104ECO,NODE_DEVICE_TYPE); // not inserted by default, should only appear if pluggd!

    chmod("/etc/softing", 0777);
    chmod(CANCONFIGFILE, 0777);
    pDevices = fopen(CANCONFIGFILE,"r");

    memset(line,0,sizeof(line));
    if(!pDevices)
    {   // file not found, no devices to insert.
        return;
    }

    while(!feof(pDevices))
    {
        QString chName;
        QString serial;
        bool bMustInsert;
        cTreeWidgetItem *pNode;
        cTreeWidgetItem *pcNode;
        char *pline = NULL;

        pline = fgets(line, 1000, pDevices);

        if(!pline) continue;

        pline = line;
        chName = strstr(line,"name:");
        chName = chName.remove(0,strlen("name:"));
        chName = chName.simplified();

        CHANNEL pCh(line);



        if(chName=="")
        {
            fclose(pDevices);
            return;
        }

        pcNode = (cTreeWidgetItem*)m_ui.treeWidget->topLevelItem(0);

        pNode = pcNode->findItems(pCh.m_channelName,(DEVICETYPE)pCh.m_deviceType,NODE_CHANNEL);

        bMustInsert = (pNode) ? pNode->m_CANChannel.m_Error : true;

        if(bMustInsert)
        {
            QString ifc;
            QString h = pCh.getDeviceName(pCh.m_deviceType);

            pCh.m_Error = ERROR_CH_NOT_FOUND;
            ifc = ifc.sprintf("%s #%09lu",
                              h.toAscii().data(),
                              pCh.getSerialnumber());

            insertInterfaces(ifc,h,pCh.m_deviceType,NODE_DEVICE);
            insertInterfaces(&pCh,ifc,pCh.m_deviceType,NODE_CHANNEL);
        }

    }

}

void SCIM::setTableLine(QString col1, QString col2, int row)
{
    QTableWidget * pTable = m_ui.tableWidget;
    QTableWidgetItem *pLine;

    pTable->setRowCount(row+1);

    pLine = new QTableWidgetItem(col1);
    pTable->setItem(row,0,pLine);
    pLine = new QTableWidgetItem(col2);
    pTable->setItem(row,1,pLine);
    pTable->takeVerticalHeaderItem(row);

    pTable->resizeColumnToContents(0);
    pTable->resizeColumnToContents(1);
    pTable->setRowHeight(row,15);

    pTable->horizontalHeader()->setStretchLastSection(true);

    return;
}


/**
 ******************************************************************
 * @brief  writes the details of a channel into the list view
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-18
 *
 * @param  pNode: pointer to the channel node.
 * @return -
 *
 ******************************************************************
 */

void SCIM::setChannelDetails(cTreeWidgetItem *pNode)
{
    QTableWidget * pTable = m_ui.tableWidget;
    QString textLine;
    int row = 0;

    QFont myFont = m_ui.tableWidget->font();
    myFont.setPointSize(8);
    myFont.setBold(true);
    m_ui.tableWidget->setFont(myFont);

    clearDeviceDetails();

    pTable->setShowGrid(false);
    pTable->verticalHeader()->setVisible(false);

    setTableLine("Channel:", pNode->m_CANChannel.m_channelName, row++);
    if(pNode->m_CANChannel.getMajor()==UNDEFINED_VALUE)
    {
        setTableLine("Major number:", "not defined!" ,row++);
    }
    else
    {
        setTableLine("Major number:", QString("%1").arg(pNode->m_CANChannel.getMajor()) ,row++);
    }

    if(pNode->m_CANChannel.getMinor()==UNDEFINED_VALUE)
    {
        setTableLine("Minor number:", "not defined!" ,row++);
    }
    else
    {
        setTableLine("Minor number:", QString("%1").arg(pNode->m_CANChannel.getMinor()) ,row++);
    }


    pTable->takeVerticalHeaderItem(row);


    textLine = textLine.sprintf("%.2f KBaud",pNode->m_CANChannel.m_fBaudrate);
    setTableLine("Baudrate:", textLine, row++);

    pTable->insertRow(row);
    pTable->setRowHeight(row,15);
    pTable->takeVerticalHeaderItem(row++);

    setTableLine("Register settings:","",row++);

    textLine = textLine.sprintf("%lu",pNode->m_CANChannel.m_prescaler);
    setTableLine("Prescaler:", textLine, row++);

    textLine = textLine.sprintf("%lu",pNode->m_CANChannel.m_tseg1);
    setTableLine("TSEG1:", textLine, row++);

    textLine = textLine.sprintf("%lu",pNode->m_CANChannel.m_tseg2);
    setTableLine("TSEG2:", textLine, row++);


    if((pNode->m_CANChannel.m_deviceType == CANPROPCIE) || (pNode->m_CANChannel.m_deviceType == CAN104PLUS) || (pNode->m_CANChannel.m_deviceType == CANPROUSB))
    {
        textLine = textLine.sprintf("%s",(pNode->m_CANChannel.m_sam) ? "enabled" : "disabled");
        setTableLine("Div8X:", textLine, row++);
    }
    else
    {
        textLine = textLine.sprintf("%lu",pNode->m_CANChannel.m_sam);
        setTableLine("Samples:", textLine, row++);
    }


    textLine = textLine.sprintf("%lu",pNode->m_CANChannel.m_sjw);
    setTableLine("Sync. jump width:", textLine, row++);

    if((pNode->m_CANChannel.m_deviceType == CANPROPCIE) || (pNode->m_CANChannel.m_deviceType == CAN104PLUS) || (pNode->m_CANChannel.m_deviceType == CANPROUSB))
    {
        textLine = textLine.sprintf("%s",(pNode->m_CANChannel.m_u32OutputCtrl<=3) ? "enabled" : "disabled");
        setTableLine("Listen only mode:", textLine, row++);
    }
    else
    {
        textLine = textLine.sprintf("0x%02lX",pNode->m_CANChannel.m_u32OutputCtrl);
        setTableLine("Output Ctrl:", textLine, row++);
    }


    pTable->insertRow(row);
    pTable->setRowHeight(row,15);
    pTable->takeVerticalHeaderItem(row);
    row++;

    textLine = textLine.sprintf("0x%04lX",pNode->m_CANChannel.m_u32AccCodeStd);
    setTableLine("Acceptance code std:", textLine, row++);

    textLine = textLine.sprintf("0x%04lX",pNode->m_CANChannel.m_u32AccMaskStd);
    setTableLine("Acceptance mask std:", textLine, row++);

    textLine = textLine.sprintf("0x%08lX",pNode->m_CANChannel.m_u32AccCodeXtd);
    setTableLine("Acceptance code xtd:", textLine, row++);

    textLine = textLine.sprintf("0x%08lX",pNode->m_CANChannel.m_u32AccMaskXtd);
    setTableLine("Acceptance mask xtd", textLine, row++);

    pTable->insertRow(row);
    pTable->setRowHeight(row,15);
    pTable->takeVerticalHeaderItem(row++);


    textLine = textLine.sprintf("%s",(pNode->m_CANChannel.m_enableAck) ? "enabled" : "disabled");
    setTableLine("Confirm transmission:", textLine, row++);

    textLine = textLine.sprintf("%s",(pNode->m_CANChannel.m_detectErrorframes) ? "enabled" : "disabled");
    setTableLine("ERROR FRAME detection:", textLine, row++);


}

/**
 ******************************************************************
 * @brief  called, when the user selects "edit" in the context menu
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-18
 *
 * @return -
 *
 ******************************************************************
 */

void SCIM::on_edit_channel()
{
    cTreeWidgetItem *pNode = (cTreeWidgetItem*)m_ui.treeWidget->topLevelItem(0);
    pNode = pNode->findSelectedNode();

    if(!pNode)
        return;

    if(pNode->getNodeType() != NODE_CHANNEL)
    {
        return;
    }

    pNode->get_config();


    if(!pNode->getDialogCanceled())
    {
        // update the list view
        setChannelDetails(pNode);
        saveDeviceTree();
    }
}



/**
 ******************************************************************
 * @brief  scans for all CAN Interfaces in the system
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-18
 *
 * @return -
 *
 ******************************************************************
 */

void SCIM::on_scan()
{
    DIR *pDir;
    CHANNEL channel;

    cTreeWidgetItem *pNode = (cTreeWidgetItem*)m_ui.treeWidget->topLevelItem(0); //->child(m_ui.treeWidget->topLevelItem(0)->indexOfChild(m_ui.treeWidget->topLevelItem(0)));

    clearDeviceDetails();

    // check wether the directory for the config. file exists.
    pDir = opendir ("/etc/softing");

    if(pDir)
    {
        //directory exists, ok nothing to do!
        closedir (pDir);
    }
    else
    {
        // directory must be created
        mkdir("/etc/softing",0777);
    }


    m_isVcanpcidLoaded   = (insertPluggedDevices(PROC_PCI, channel.getDeviceName(CANPCI), CANPCI) != -1);
    m_isVcanacdLoaded    = (insertPluggedDevices(PROC_104, channel.getDeviceName(CAN_104), CAN_104) != -1);
    m_isVcanecoLoaded    = (insertPluggedDevices(PROC_ECO, channel.getDeviceName(CAN104ECO), CAN104ECO) != -1);
    m_isVcanpcieLoaded    = (insertPluggedDevices(PROC_PCIE, channel.getDeviceName(CANPROPCIE), CANPROPCIE) != -1);
    m_isVcanproLoaded    = (insertPluggedDevices(PROC_104PLUS, channel.getDeviceName(CAN104PLUS), CAN104PLUS) != -1);
    m_isVcanproLoaded    = (insertPluggedDevices(PROC_USB, channel.getDeviceName(CANPROUSB), CANPROUSB) != -1);


    insertUnpluggedDevices();
    m_ui.treeWidget->expandAll();

    saveDeviceTree();

    pNode = pNode->findSelectedNode();
    if(pNode)
    {
        if(pNode->getNodeType() == NODE_CHANNEL)
            setChannelDetails(pNode);

        if(pNode->getNodeType()==NODE_DEVICE)
            setDriverDetails(pNode);

    }
}

/**
 ******************************************************************
 * @brief  writes the configuration values from the device tree leaves to a configuration file
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-18
 *
 * @return -
 *
 ******************************************************************
 */

void SCIM::saveDeviceTree(void)
{
    cTreeWidgetItem *pitem = (cTreeWidgetItem *)m_ui.treeWidget->topLevelItem(0);
    FILE *pDevices = NULL;


    pDevices = fopen(CANCONFIGFILE,"w");

    if(!pDevices)
        return;

    while(pitem)
    {
        if(pitem->getNodeType()==NODE_CHANNEL)
        {   // channel detected
            cTreeWidgetItem *pNode = pitem;
            fprintf(
                pDevices,
                "%09lu_%lu baud:%.2f presc:%lu tseg1:%lu tseg2:%lu sam:%lu sjw:%lu ocr:0x%lx accmstd:0x%04lx acccstd:0x%04lx accmxtd:0x%08lx acccxtd:0x%08lx ack:%u errorframes:%u type:%u irq:%lu ioport:0x%lx dpram:0x%lx name:%s\n",
                pNode->m_CANChannel.getSerialnumber(),
                pNode->m_CANChannel.getPhysChannel(),
                pNode->m_CANChannel.m_fBaudrate,
                pNode->m_CANChannel.m_prescaler,
                pNode->m_CANChannel.m_tseg1,
                pNode->m_CANChannel.m_tseg2,
                pNode->m_CANChannel.m_sam,
                pNode->m_CANChannel.m_sjw,
                pNode->m_CANChannel.m_u32OutputCtrl,
                pNode->m_CANChannel.m_u32AccMaskStd,
                pNode->m_CANChannel.m_u32AccCodeStd,
                pNode->m_CANChannel.m_u32AccMaskXtd,
                pNode->m_CANChannel.m_u32AccCodeXtd,
                pNode->m_CANChannel.m_enableAck,
                pNode->m_CANChannel.m_detectErrorframes,
                pNode->m_CANChannel.m_deviceType,
                pNode->m_CANChannel.getIrq(),
                pNode->m_CANChannel.getIoPort(),
                pNode->m_CANChannel.getDprAddr(),
                pNode->m_CANChannel.m_channelName.toAscii().data()
            );
        }
        pitem = pitem->itemBelow();
    }
    fclose(pDevices);
}




#include "scim.moc"
