/***************************************************************************
 *   Copyright (C) 2007-2015 by Softing Industrial Automation GmbH         *
 *   support.automation@softing.com                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef SCIM_H
#define SCIM_H


#include <KDE/KMainWindow>

#include "ui_scim.h"

#include "ui_about.h"
#include "ctreewidgetitem.h"
#include <qmenu.h>
#include <qdir.h>
#include <qdatetime.h>
#include <sys/types.h>
#include <pwd.h>
#include <unistd.h>



/**
 * This class serves as the main window for SCIM.  It handles the
 * menus, toolbars and status bars.
 *
 * @short Main window class
 * @author Your Name <mail@example.com>
 * @version 0.1
 */
class SCIM : public KMainWindow
{
    Q_OBJECT
public:
    /**
     * Default Constructor
     */
    SCIM();

    /**
     * Default Destructor
     */
    virtual ~SCIM();
    
public slots:
  virtual void on_treeItemRightClicked(bool checked);
  virtual void on_treeItemClicked(QTreeWidgetItem* item, int col);
  virtual void on_edit_channel();
  virtual void on_scan();
  virtual void on_about();
  virtual void on_treeItemChanged(void);
  virtual void on_remove_ifc(void);
  
private:
    // this is the name of the root widget inside our Ui file
    // you can rename it in designer and then change it here
    Ui::mainWidget m_ui;
    
        /*$PRIVATE_FUNCTIONS$*/
    QImage image_ok;
    QImage image_error;
    Ui::aboutDlg m_about;
    QMessageBox m_on_Error;
    bool m_isVcanacdLoaded;
    bool m_isVcanpcidLoaded;
    bool m_isCancard_csLoaded;
    bool m_isVcanecoLoaded;
    bool m_isVcanproLoaded;
    bool m_isVcanpcieLoaded;

    bool runningAsRoot();
    bool insertInterfaces(QString nodeName, QString ifcName, DEVICETYPE type, NODE_TYPE nodeType);
    bool insertInterfaces(CHANNEL *pChannel, QString ifcName, DEVICETYPE type, NODE_TYPE nodeType);
    int insertPluggedDevices(QString ProcDir, QString Interface, DEVICETYPE type);
    cTreeWidgetItem *findNode(char *ifcName);
    cTreeWidgetItem *findNode(CHANNEL *pChannel);
    cTreeWidgetItem *findNode(QString name);
    cTreeWidgetItem *findSelectedNode(void);
    void clearDeviceDetails(void);
    void setRootDetails(void);
    void setDriverDetails(cTreeWidgetItem *pItem);
    void setDeviceDetails(cTreeWidgetItem *pItem);    
    void setChannelDetails(cTreeWidgetItem *pNode);
    
    void saveDeviceTree(void);
    void insertUnpluggedDevices();
    QMenu m_menu;
    QAction *m_ac_edit;
    QAction *m_ac_refresh;
    QAction *m_ac_remove;
    void setTableLine(QString col1, QString col2, int row);
    QString getShellOutput(QString cmd);

};

#endif // _SCIM_H_
