/***************************************************************************
 *   Copyright (C) 2007-2015 by Softing Automation GmbH                    *
 *   support.automation@softing.com                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "clss.h"
#include <qtextedit.h>

/**
 * *****************************************************************
 * @brief  Constructor
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

lss::lss(CAN_HANDLE can, QWidget *parent)
    :QDialog(parent)
{
  m_Can = can;
    m_pw = new QDialog(this);
//   setCentralWidget(w);
    m_ui.setupUi(m_pw);

    connect(m_ui.buttonOk, SIGNAL(clicked()), this, SLOT(accept()));
    connect(m_ui.bt_activate_baud, SIGNAL(clicked()), this, SLOT(OnActivateBaud()));
    connect(m_ui.bt_cfg_baud, SIGNAL(clicked()), this, SLOT(OnCfgBaud()));
    connect(m_ui.bt_cfg_nodeid, SIGNAL(clicked()), this, SLOT(OnCfgNodeid()));
    connect(m_ui.bt_identify_non_cfg_slave, SIGNAL(clicked()), this, SLOT(OnIdentifyNonCfgSlaves()));
    connect(m_ui.bt_identify_rem_slave, SIGNAL(clicked()), this, SLOT(OnIdentifyRemSlaves()));
    connect(m_ui.bt_lss_init, SIGNAL(clicked()), this, SLOT(OnLssInit()));
    connect(m_ui.bt_inq_nodeid, SIGNAL(clicked()), this, SLOT(OnInqNodeId()));
    connect(m_ui.bt_inq_prod_code, SIGNAL(clicked()), this, SLOT(OnInqProdCode()));
    connect(m_ui.bt_inq_serial, SIGNAL(clicked()), this, SLOT(OnInqSerial()));
    connect(m_ui.bt_inquire_rev_nr, SIGNAL(clicked()), this, SLOT(OnInqRevNr()));
    connect(m_ui.bt_store, SIGNAL(clicked()), this, SLOT(OnStore()));
    connect(m_ui.bt_sw_mode_gl, SIGNAL(clicked()), this, SLOT(OnSwModeGlobal()));
    connect(m_ui.bt_sw_mode_sel, SIGNAL(clicked()), this, SLOT(OnSwModeSel()));
    connect(m_ui.bt_inquire_vendor, SIGNAL(clicked()), this, SLOT(OnInquireVendor()));

    m_pw->show();

}

/**
 * *****************************************************************
 * @brief  Destructor
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

lss::~lss(void )
{
    delete m_pw;
}


void lss::OnActivateBaud()
{
    QString errorText;
    unsigned short retCode;

    retCode = LSS_ActivateBitTiming(m_Can, 0x1020);
    if(retCode)
    {
        errorText.sprintf("ERROR: %i in LSS_ActivateBitTiming\n",retCode);
        emit(doPrint(errorText));
    }
}

void lss::OnIdentifyNonCfgSlaves()
{
    QString errorText;
    unsigned short retCode;

    retCode = LSS_IdentifyNonConfiguredSlaves(m_Can);
    if(retCode)
    {
        errorText.sprintf("ERROR: %i in LSS_IdentifyNonConfiguredSlaves\n",retCode);
        emit(doPrint(errorText));
    }

}


void lss::OnCfgNodeid()
{
    QString errorText;
    unsigned short retCode;

    retCode = LSS_ConfigureNodeID(m_Can, m_ui.ed_nodeid->text().toShort());
    if(retCode)
    {
        errorText.sprintf("ERROR: %i in LSS_ConfigureNodeID\n",retCode);
        emit(doPrint(errorText));
    }

}


void lss::OnCfgBaud()
{
    QString errorText;
    unsigned short retCode;

    retCode = LSS_ConfigureBitTiming(m_Can, 0, m_ui.ed_baud->text().toShort());

    if(retCode)
    {
        errorText.sprintf("ERROR: %i in LSS_ConfigureBitTiming\n",retCode);
        emit(doPrint(errorText));
    }

}

void lss::OnInqNodeId()
{
    QString outText;
    unsigned short retCode;
    unsigned char node;

    retCode = LSS_InquireNodeID(m_Can, &node);
    if(retCode)
    {
        outText.sprintf("ERROR: %i in LSS_InquireNodeID\n",retCode);
        emit(doPrint(outText));
        return;
    }

    outText.sprintf("Node ID: %3u\n",node);
    emit(doPrint(outText));

    outText.sprintf("%3u\n",node);
    m_ui.ed_nodeid->setText(outText);

}


void lss::OnLssInit()
{
    QString errorText;
    unsigned short retCode;

    retCode = LSS_InitLSSMan(m_Can,1);
    if(retCode)
    {
        errorText.sprintf("ERROR: %i in LSS_\n",retCode);
        emit(doPrint(errorText));
    }

}


void lss::OnIdentifyRemSlaves()
{
    QString errorText;
    unsigned short retCode;

    retCode = LSS_IdentityRemoteSlaves(
                  m_Can,
                  m_ui.ed_vendor->text().toULong(),
                  m_ui.ed_product_cd->text().toULong(),
                  0,
                  m_ui.ed_rev_nr->text().toULong(),
                  0,
                  m_ui.ed_serial->text().toULong()
              );


    if(retCode)
    {
        errorText.sprintf("ERROR: %i in LSS_IdentityRemoteSlaves\n",retCode);
        emit(doPrint(errorText));
    }

}

void lss::OnInqRevNr()
{
    QString outText;
    USIGN16 retCode;
    USIGN32 revision;

    retCode = LSS_InquireRevNr(m_Can, &revision);
    if(retCode)
    {
        outText.sprintf("ERROR: %i in LSS_RevisionNr\n",retCode);
        emit(doPrint(outText));
    }

    outText.sprintf("Serial Number: %3u\n",revision);
    emit(doPrint(outText));

    outText.sprintf("0x%08x\n",revision);
    emit(doPrint(outText));

}

void lss::OnInquireVendor()
{
    QString outText;
    USIGN16 retCode;
    USIGN32 vid;

    retCode = LSS_InquireVendorID(m_Can, &vid);
    if(retCode)
    {
        outText.sprintf("ERROR: %i in LSS_InquireVendorID\n",retCode);
        emit(doPrint(outText));
    }

    outText.sprintf("Vendor ID: %3u\n",vid);
    emit(doPrint(outText));

    outText.sprintf("0x%08x\n",vid);
    m_ui.ed_vendor->setText(outText);

}

void lss::OnInqSerial()
{
    QString outText;
    USIGN16 retCode;
    USIGN32 serial;

    retCode = LSS_InquireSerialNr(m_Can, &serial);
    if(retCode)
    {
        outText.sprintf("ERROR: %i in LSS_InquireSerialNr\n",retCode);
        emit(doPrint(outText));
    }

    outText.sprintf("Serial Number: %3u\n",serial);
    emit(doPrint(outText));

    outText.sprintf("0x%08x\n",serial);
    m_ui.ed_serial->setText(outText);

}


void lss::OnInqProdCode()
{
    QString outText;
    USIGN16 retCode;
    USIGN32 productCode;

    retCode = LSS_InquireProductCode(m_Can, &productCode);
    if(retCode)
    {
        outText.sprintf("ERROR: %i in LSS_InquireProductCode\n",retCode);
        emit(doPrint(outText));
    }

    outText.sprintf("Product Code: %3u\n",productCode);
    emit(doPrint(outText));

    outText.sprintf("0x%08x\n",productCode);
    m_ui.ed_product_cd->setText(outText);

}

void lss::OnSwModeSel()
{
    QString errorText;
    unsigned short retCode;

    retCode = LSS_SwitchModeSelective(m_Can,
                                      m_ui.ed_vendor->text().toULong(),
                                      m_ui.ed_product_cd->text().toULong(),
                                      m_ui.ed_rev_nr->text().toULong(),
                                      m_ui.ed_serial->text().toULong()
                                     );

    if(retCode)
    {
        errorText.sprintf("ERROR: %i in LSS_SwitchModeSelective\n",retCode);
        emit(doPrint(errorText));
    }

}


void lss::OnSwModeGlobal()
{
    QString errorText;
    unsigned short retCode;
    static unsigned char ucMode = 0;

    retCode = LSS_SwitchModeGlobal(m_Can, ucMode);
    if(retCode)
    {
        errorText.sprintf("ERROR: %i in LSS_SwitchModeGlobal\n",retCode);
        emit(doPrint(errorText));
    }

    ++ucMode &= 1;

}


void lss::OnStore()
{
    QString errorText;
    unsigned short retCode;

    retCode = LSS_StoreConfiguration(m_Can);
    if(retCode)
    {
        errorText.sprintf("ERROR: %i in LSS_StoreConfiguration\n",retCode);
        emit(doPrint(errorText));
    }

}









#include "clss.moc"
