/***************************************************************************
 *   Copyright (C) 2007-2015 by Softing Industrial Automation GmbH         *
 *   support.automation@softing.com                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef CNMT_H
#define CNMT_H

#include "ui_nmt.h"
#include <qspinbox.h>
#include <qtextedit.h>
#include <qlineedit.h>
#include <qstring.h>
#include <qpushbutton.h>
#include <qtimer.h>
#include <qcheckbox.h>
#include <qdialog.h>
#include <qmessagebox.h>

#include "can_def.h"
#include "BaseTypes.h"

#include "canL2.h"
#include "cmadef.h"
#include "cmaret.h"

typedef enum
{
    INITIALISATION,
    RESET,
    STOPPED=4,
    OPERATIONAL,
    UNUSED,
    USED,
    UNKNOWN,
    PREOPERATIONAL = 127

} NODE_STATE;



typedef enum
{
    GUARD_NOTACTIVE,
    GUARD_ACTIVE,
    GUARD_LOST_RESP,
    GUARD_FAILED,
    GUARD_TOGGLE_ERROR

} GUARD_STATE;

#define ST_STOPP            4
#define ST_START            5
#define ST_RESET_NODE       6
#define ST_RESET_COM        7
#define ST_PRE_OPERATIONAL  127

#define MAX_NODE 127



typedef struct _node_
{
    USIGN8  retryFactor;
    USIGN16 guardTime;
    USIGN32 heartbeatInterval;
    USIGN32 heartbeatErrors;
    bool    b_checked;
    bool    b_added;
    NODE_STATE state;
    GUARD_STATE guard_state;

} CO_NODE;

class nmt: public QDialog // d_nmt
{
    Q_OBJECT
public:
    nmt(CAN_HANDLE can=0, QWidget *parent = 0);
    ~nmt(void);
    CAN_HANDLE m_Can;
    CO_NODE m_slave[MAX_NODE+1];
    QDialog* m_pw;


private:
    QTimer m_nmt_timer;
    bool m_enableUpdate;
    Ui::d_nmt m_ui;

    QString& getControlState(GUARD_STATE guardstate);
    QString& getStateText(NODE_STATE state);
    void deleteAllTabEntries(void);
    QMessageBox m_on_Error;


public slots:
    virtual void OnAdd();
    virtual void OnRemove();
    virtual void OnStart();
    virtual void OnResetCom();
    virtual void OnPreoperational();
    virtual void OnErrorCtrl();
    virtual void OnUpdate();
    virtual void OnNodeChanged(int node);
    virtual void OnSync();
    virtual void OnReset();
    virtual void OnStop();
    virtual void OnErrorEvent(unsigned char nodeId, unsigned char eventType, int state);

signals:
    void doPrint(QString toPrint);


};

#endif
