/***************************************************************************
 *   Copyright (C) 2007-2015 by Softing Automation GmbH                    *
 *   support.automation@softing.com                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "cpdo.h"

// columns of the table
#define COB_COL  	0
#define HANDLE_COL 	1
#define DLC_COL         2
#define RXTX_COL  	3
#define TTYPE_COL	4
#define DATA_COL 	5

/**
 * *****************************************************************
 * @brief  Constructor
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

pdo::pdo(CAN_HANDLE can, QWidget *parent)
    :QDialog(parent)
{
    m_pw = new QDialog(this);
    //   setCentralWidget(w);
    m_ui.setupUi(m_pw);
    connect(m_ui.buttonOk, SIGNAL(clicked()), this, SLOT(accept()));
    connect(m_ui.bt_intall_pdo, SIGNAL(clicked()), this, SLOT(OnInstallPdo()));
    connect(m_ui.bt_remove_pdo, SIGNAL(clicked()), this, SLOT(OnRemovePdo()));
    connect(m_ui.bt_read_pdo, SIGNAL(clicked()), this, SLOT(OnReadPdo()));
    connect(m_ui.bt_write_pdo, SIGNAL(clicked()), this, SLOT(OnWritePdo()));

    connect(m_ui.PdoList, SIGNAL(cellClicked(int,int)), this, SLOT(OnTableRowClicked(int, int)));

    m_SelectedRow = 0;


    m_pw->show();
    
    m_installPdo = NULL;
    m_removePdo = NULL;
    m_readPdo = NULL;
    m_writePdo = NULL;
    m_Can = can;

}

/**
 * *****************************************************************
 * @brief  Destructor
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

pdo::~pdo(void )
{
  delete m_pw;
  if(m_installPdo) delete(m_installPdo);
  if(m_removePdo) delete(m_removePdo);
  if(m_readPdo) delete(m_readPdo);
  if(m_writePdo) delete(m_writePdo);
}

/**o
 ******************************************************************
 * @brief  formates the data bytes of a PDO for output.
 * @param  pData: data of the PDO
 * @param  len: data length
 * @param  ret: formated string which represents the PDO data
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return row of the table
 *
 ******************************************************************
 */

QString &pdo::formatPdoData(USIGN8 *pData, USIGN8 len, QString& ret)
{
    USIGN8 *pD = pData;
    int i;

    for(i=0; i<len; i++)
    {
        QString text;
        text = text.sprintf("%02x ", pD[0]);
        ret = ret.append(text);
        pD++;
    }

    return ret;
}

/**o
 ******************************************************************
 * @brief  returns the table row where the PDO information is stored.
 * @param  pdohndl: PDO handle
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return row of the table
 *
 ******************************************************************
 */

int pdo::findTableRow(USIGN32 pdohndl)
{
    int i;
    QTableWidget * pTable = m_ui.PdoList;
    bool isValid;

    for(i=0; i<pTable->rowCount(); i++)
    {
        unsigned int hndl = pTable->item(i,HANDLE_COL)->text().toUInt(&isValid,16);

        if(pdohndl == hndl)
        {
            return i;
        }
    }

    return -1;
}

/**o
 ******************************************************************
 * @brief  called, when the user has clicked at a row inside the table
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void pdo::OnTableRowClicked(int y, int x)
{
    QTableWidgetSelectionRange col_range(y, 0, y, m_ui.PdoList->columnCount()-1);
    QTableWidgetSelectionRange tab_range(0, 0, m_ui.PdoList->rowCount()-1, m_ui.PdoList->columnCount()-1);
    
    QString rw = "Rx PDO";
    x=x;

    m_ui.PdoList->setRangeSelected(tab_range, false);
    m_ui.PdoList->setRangeSelected(col_range, true);
    m_ui.PdoList->setCurrentItem(m_ui.PdoList->item(y,x), QItemSelectionModel::Rows);

}


/**o
 ******************************************************************
 * @brief  update the PDO data inside the PDO table
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void pdo::OnPdoReceived(unsigned long pdohndl, unsigned char len, quint64 data)
{
    QString cob, hndl, dlc, datas;
    QTableWidget * pTable = m_ui.PdoList;
    QTableWidgetItem *pItemCob;
    int row = findTableRow(pdohndl);

    if(row<0) return;

    formatPdoData((unsigned char*)&data, len, datas);

    pItemCob = pTable->item(row,DATA_COL);
    pItemCob->setText(datas);
}

/**
 * *****************************************************************
 * @brief  This slot is called if the install PDO button is pushed.
 * @brief  The routine installs a new PDO.
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void pdo::OnInstallPdo()
{

    if(m_installPdo == NULL)
    {
        m_installPdo = new cinstall;
        connect(m_installPdo, SIGNAL(finished(int)), this, SLOT(OnInstallPdoFinished(int)));
    }
    
    m_installPdo->m_pw->show();
}

/**
 * *****************************************************************
 * @brief  This slot is called if the remove PDO button is pushed.
 * @brief  The routine removes a PDO.
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void pdo::OnRemovePdo()
{
  QTableWidget * pTable = m_ui.PdoList;
  volatile int row = pTable->currentRow();
  
  if(m_removePdo == NULL)
  {
    m_removePdo = new cremove;
    connect(m_removePdo, SIGNAL(finished(int)), this, SLOT(OnRemovePdoFinished(int)));        
  }
  
  if(row>=0)
  {
    m_removePdo->m_ui.ed_pdo_handle->setText((pTable->item(row,HANDLE_COL)->text()));
  }
  
  m_removePdo->m_pw->show();
}

/**
 * *****************************************************************
 * @brief  This slot is called if the read PDO button is pushed.
 * @brief  The routine reads the PDO data and writes it to the output Window.
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void pdo::OnReadPdo()
{
  
  QTableWidget * pTable = m_ui.PdoList;
  volatile int row = pTable->currentRow();
  
  if(m_readPdo == NULL)
  {
    m_readPdo = new cread(m_Can);
    connect(m_readPdo, SIGNAL(finished(int)), this, SLOT(OnReadPdoFinished(int)));
  }
  
  if(row>=0)
  {
    m_readPdo->m_ui.ed_pdo_handle->setText((pTable->item(pTable->currentRow(),HANDLE_COL)->text()));
  }
  
  m_readPdo->m_pw->show();
}

/**
 * *****************************************************************
 * @brief  This slot is called if the write PDO button is pushed.
 * @brief  The routine exchanges the PDO data.
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void pdo::OnWritePdo()
{
  QTableWidget * pTable = m_ui.PdoList;
  volatile int row = pTable->currentRow();
  
  if(m_writePdo == NULL)
  {
    m_writePdo = new cwrite;
    connect(m_writePdo, SIGNAL(finished(int)), this, SLOT(OnWritePdoFinished(int)));  
  }
  
  if(row>=0)
  {
   m_writePdo->m_ui.ed_pdo_handle->setText(pTable->item(pTable->currentRow(),HANDLE_COL)->text()); 
   m_writePdo->m_ui.ed_size->setText(pTable->item(pTable->currentRow(),DLC_COL)->text()); 
  }
  
  m_writePdo->m_pw->show();
  
  
}

/**
 * *****************************************************************
 * @brief  This member function prints a message on the output Window.
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void pdo::Print(QString& msg)
{
    emit(doPrint(msg));
}

void pdo::OnOk(void)
{

}

/**
 ******************************************************************
 * @brief  slot: sdo dialog finished signal received
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void pdo::OnInstallPdoFinished(int ret)
{
    CMA_PDOConfig_E cfg;
    unsigned short handle;
    QString Text;
    short retCode;
    bool isValid;
    QString cob, hndl, dlc, data;
    QTableWidget * pTable = m_ui.PdoList;
    QTableWidgetItem *pItem;
    int rows = pTable->rowCount();

    
   if(ret != 0) // not canceled
   {
     // get the data from the gui and store it to PDO configuration structure
     cfg.ucaDefVal[0] = m_installPdo->m_ui.ed_bt0->text().toShort(&isValid,16);
     cfg.ucaDefVal[1] = m_installPdo->m_ui.ed_bt1->text().toShort(&isValid,16);
     cfg.ucaDefVal[2] = m_installPdo->m_ui.ed_bt2->text().toShort(&isValid,16);
     cfg.ucaDefVal[3] = m_installPdo->m_ui.ed_bt3->text().toShort(&isValid,16);
     cfg.ucaDefVal[4] = m_installPdo->m_ui.ed_bt4->text().toShort(&isValid,16);
     cfg.ucaDefVal[5] = m_installPdo->m_ui.ed_bt5->text().toShort(&isValid,16);
     cfg.ucaDefVal[6] = m_installPdo->m_ui.ed_bt6->text().toShort(&isValid,16);
     cfg.ucaDefVal[7] = m_installPdo->m_ui.ed_bt7->text().toShort(&isValid,16);
     
     
     // PDO size
     cfg.ucNrOfByte = m_installPdo->m_ui.ed_size->text().toShort();
     // flags for PDO behavior
     cfg.ucNoInitTrans = 0;
     cfg.ucAccType = 0;
     cfg.ucEventNotif = 1;
     
     // receive PDO or transmit PDO?
     if(m_installPdo->m_ui.ch_rx_pdo->isChecked())
     {
       // receive PDO
       cfg.ucPDOType = 0;
       cfg.ucTransType = m_installPdo->m_ui.ed_transm_type->text().toShort();
     }
     else
     {
       // transmit PDO
       cfg.ucPDOType = 1;
       cfg.ucTransType = m_installPdo->m_ui.ed_transm_type->text().toShort();
     }
     
     // get the COB ID
     cfg.ulCOBID = m_installPdo->m_ui.ed_cobid->text().toShort(&isValid,16);
     
     if((retCode = CMA_InstallPDO_E(m_Can, cfg, &handle)))
     {
       Text.sprintf("ERROR: %i in CMA_InstallPDO\n", retCode);
       Print(Text);
       m_installPdo->m_pw->hide();
       return;
     }
     
     Text.sprintf("PDO Handle for PDO %x = %x                                                         ",cfg.ulCOBID, handle);
     Print(Text);
     
     hndl = hndl.sprintf("0x%x",handle);
     dlc = dlc.sprintf("%u",cfg.ucNrOfByte);
     cob = cob.sprintf("0x%x",cfg.ulCOBID);
     formatPdoData(cfg.ucaDefVal, cfg.ucNrOfByte, data);
     pTable->setRowCount(rows+1);
     
     pItem = new QTableWidgetItem(cob);
     pTable->setItem(rows,COB_COL,pItem);
     pItem = new QTableWidgetItem(hndl);
     pTable->setItem(rows,HANDLE_COL,pItem);
     pItem = new QTableWidgetItem(dlc);
     pTable->setItem(rows,DLC_COL,pItem);
     pItem = new QTableWidgetItem(cfg.ucPDOType==0? "Rx PDO":"Tx PDO");
     pTable->setItem(rows,RXTX_COL,pItem);
     pItem = new QTableWidgetItem(m_installPdo->m_ui.ed_transm_type->text());
     pTable->setItem(rows,TTYPE_COL,pItem);
     pItem = new QTableWidgetItem(data);
     pTable->setItem(rows,DATA_COL,pItem);
     
     pTable->resizeColumnToContents(COB_COL);
     pTable->resizeColumnToContents(HANDLE_COL);
     pTable->resizeColumnToContents(DLC_COL);
     pTable->resizeColumnToContents(RXTX_COL);
     pTable->resizeColumnToContents(TTYPE_COL);
     pTable->resizeColumnToContents(DATA_COL);
     pTable->sortItems(0);
     
     pTable->horizontalHeader()->setStretchLastSection(true);
     
     OnTableRowClicked(rows, 1);
     
   }
   
   m_installPdo->m_pw->hide();
   
 
}
/**
 ******************************************************************
 * @brief  slot: sdo dialog finished signal received
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void pdo::OnRemovePdoFinished(int ret)
{
    
    QString Text;
    bool isValid;
    int row;
    QTableWidget * pTable = m_ui.PdoList;
    short retCode;

    if(ret != 0)
    {
      // not canceled
      if((retCode = CMA_RemovePDO(m_Can, m_removePdo->m_ui.ed_pdo_handle->text().toShort(&isValid,16))))
      {
	Text.sprintf("ERROR: %i in CMA_RemovePDO\n", retCode);
	Print(Text);
	m_removePdo->m_pw->hide();
	return;
      }
      
      row = findTableRow(m_removePdo->m_ui.ed_pdo_handle->text().toShort(&isValid,16));
      
      if(row<0)
      {
	m_removePdo->m_pw->hide();
	return;
      }
      
      pTable->removeRow(row);
      
      if(pTable->rowCount()>0)
      {
	OnTableRowClicked(0, 0);
      }
    }
    
    m_removePdo->m_pw->hide();
}
/**
 ******************************************************************
 * @brief  slot: sdo dialog finished signal received
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void pdo::OnReadPdoFinished(int ret)
{  
  char ucErrorTextBuffer[100];
  char *ucErrorText = &ucErrorTextBuffer[0];
  short retCode;
  unsigned char data[8];
  bool isValid;
  QString Text;
  
  if(ret != 0)
  {
    if((retCode = CMA_ReadPDO(m_Can, m_readPdo->m_ui.ed_pdo_handle->text().toShort(&isValid,16), data)))
    {
      Text.sprintf(ucErrorText, "ERROR: %i in CMA_RemovePDO\n", retCode);
      Print(Text);
      return;
    }
    Text.sprintf("PDO: %02x %02x %02x %02x %02x %02x %02x %02x                        \n",
		 data[0], data[1], data[2], data[3], data[4], data[5], data[6], data[7]);
    
    Print(Text);
  }
  m_readPdo->m_pw->hide();
}
/**
 ******************************************************************
 * @brief  slot: sdo dialog finished signal received
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void pdo::OnWritePdoFinished(int ret)
{
  QString Text;
  short retCode;
  unsigned char data[8];
  quint64 datas;
  bool isValid;
  int row;
  QTableWidget * pTable = m_ui.PdoList;
  
  if(ret != 0)
  {
    // not canceled
    // copy the data to write
    
    data[0]=m_writePdo->m_ui.ed_bt0->text().toShort(&isValid,16);
    data[1]=m_writePdo->m_ui.ed_bt1->text().toShort(&isValid,16);
    data[2]=m_writePdo->m_ui.ed_bt2->text().toShort(&isValid,16);
    data[3]=m_writePdo->m_ui.ed_bt3->text().toShort(&isValid,16);
    data[4]=m_writePdo->m_ui.ed_bt4->text().toShort(&isValid,16);
    data[5]=m_writePdo->m_ui.ed_bt5->text().toShort(&isValid,16);
    data[6]=m_writePdo->m_ui.ed_bt6->text().toShort(&isValid,16);
    data[7]=m_writePdo->m_ui.ed_bt7->text().toShort(&isValid,16);
    
    if((retCode = CMA_WritePDO(m_Can, m_writePdo->m_ui.ed_pdo_handle->text().toShort(&isValid,16), 0, m_writePdo->m_ui.ed_size->text().toShort(), data)))
    {
      Text.sprintf("ERROR: %i in CMA_RemovePDO\n", retCode);
      Print(Text);
      m_writePdo->m_pw->hide();
      return;
    }
    
    row = findTableRow(m_writePdo->m_ui.ed_pdo_handle->text().toShort(&isValid,16));
    
    if(row<0)
    {
      m_writePdo->m_pw->hide();
      return;
    }
    
    memcpy(&datas,data,8);
    OnPdoReceived(m_writePdo->m_ui.ed_pdo_handle->text().toULong(&isValid,16), pTable->item(row,DLC_COL)->text().toUShort(), datas);
    

  }
  // hide the input widged
  m_writePdo->m_pw->hide();
  
}


#include "cpdo.moc"
