/***************************************************************************
 *   Copyright (C) 2007-2015 by Softing Automation GmbH                    *
 *   support.automation@softing.com                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "csdo.h"

/**
 * *****************************************************************
 * @brief  Constructor
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

sdo::sdo(CAN_HANDLE can, QWidget *parent)
    :QDialog(parent)
{
    m_pw = new QDialog(this);
//   setCentralWidget(w);
    m_ui.setupUi(m_pw);

    m_pw->show();

    connect(m_ui.bt_upload, SIGNAL(clicked()), this, SLOT(OnUpload()));
    connect(m_ui.rd_byte, SIGNAL(clicked()), this, SLOT(OnByte()));
    connect(m_ui.rd_long, SIGNAL(clicked()), this, SLOT(OnLong()));
    connect(m_ui.rd_word, SIGNAL(clicked()), this, SLOT(OnWord()));
    connect(m_ui.rd_double, SIGNAL(clicked()), this, SLOT(OnDouble()));
    connect(m_ui.bt_download, SIGNAL(clicked()), this, SLOT(OnDownload()));
    connect(m_ui.bt_block_download, SIGNAL(clicked()), this, SLOT(OnBlockDownload()));
    connect(m_ui.bt_block_upload, SIGNAL(clicked()), this, SLOT(OnBlockUpload()));
    connect(m_ui.bt_ok, SIGNAL(clicked()), this, SLOT(OnOk()));
    m_Can = can;

}

/**
 * *****************************************************************
 * @brief  Destructor
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

sdo::~sdo(void )
{
    delete m_pw;
}

/**
 * *****************************************************************
 * @brief  This slot is called if the upload button is pushed.
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void sdo::OnUpload()
{
    USIGN16 retCode;
    bool isOk;




    retCode = CMA_InitUpload(m_Can, // channel handle
                             (USIGN8)m_ui.ed_nodeid->text().toShort(), // node id
                             m_ui.ed_index->text().toShort(&isOk,16),  // object index
                             (USIGN8)m_ui.ed_subindex->text().toShort()); // object subindex


    if(retCode)
    {
        QString ErrorText = ErrorText.sprintf("ERROR: %i in CMA_InitUpload", retCode);
        emit(doPrint(ErrorText));
        return;
    }
}

/**
 * *****************************************************************
 * @brief  This slot is called if the download button is pushed.
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void sdo::OnDownload()
{
    USIGN16 retCode;
    USIGN8 data[8];
    bool isOk;
    volatile USIGN16 u16val;
    volatile USIGN32 u32val;
    volatile double dval;

    switch(m_type)
    {
    case 0:
        data[0] = (USIGN8) m_ui.ed_bt0->text().toShort(&isOk,16);
        data[1] = (USIGN8) m_ui.ed_bt1->text().toShort(&isOk,16);
        data[2] = (USIGN8) m_ui.ed_bt2->text().toShort(&isOk,16);
        data[3] = (USIGN8) m_ui.ed_bt3->text().toShort(&isOk,16);
        data[4] = (USIGN8) m_ui.ed_bt4->text().toShort(&isOk,16);
        data[5] = (USIGN8) m_ui.ed_bt5->text().toShort(&isOk,16);
        data[6] = (USIGN8) m_ui.ed_bt6->text().toShort(&isOk,16);
        data[7] = (USIGN8) m_ui.ed_bt7->text().toShort(&isOk,16);
        break;

    case 1:
        u16val = m_ui.ed_word->text().toUShort();
        memcpy(data, (const void*)&u16val, sizeof(u16val));
        break;

    case 2:
        u32val = m_ui.ed_long->text().toULong();
        memcpy(data, (const void*)&u32val, sizeof(USIGN32));
        break;

    case 3:
        dval = m_ui.ed_double->text().toDouble();
        memcpy(data, (const void*)&dval, sizeof(double));
        break;

    default:
        return;
        break;

    }

    retCode = CMA_InitDownload(m_Can, // channel handle
                               (USIGN8)m_ui.ed_nodeid->text().toShort(), // node id
                               m_ui.ed_index->text().toShort(&isOk,16), // index
                               (USIGN8)m_ui.ed_subindex->text().toShort(), // subindex
                               m_ui.ed_datalength->text().toShort(), // data length
                               data); // buffer which contains the data


    if(retCode)
    {
        QString ErrorText = ErrorText.sprintf("ERROR: %i in CMA_InitDownload", retCode);
        emit(doPrint(ErrorText));
        return;
    }

}

/**
 * *****************************************************************
 * @brief  This slot is called if the block download button is pushed.
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void sdo::OnBlockDownload()
{
    USIGN16 retCode;
    USIGN8 data[8];
    bool isOk;
    USIGN16 u16val;
    USIGN32 u32val;
    double dval;


    switch(m_type)
    {
    case 0:
        data[0] = (USIGN8) m_ui.ed_bt0->text().toShort(&isOk,16);
        data[1] = (USIGN8) m_ui.ed_bt1->text().toShort(&isOk,16);
        data[2] = (USIGN8) m_ui.ed_bt2->text().toShort(&isOk,16);
        data[3] = (USIGN8) m_ui.ed_bt3->text().toShort(&isOk,16);
        data[4] = (USIGN8) m_ui.ed_bt4->text().toShort(&isOk,16);
        data[5] = (USIGN8) m_ui.ed_bt5->text().toShort(&isOk,16);
        data[6] = (USIGN8) m_ui.ed_bt6->text().toShort(&isOk,16);
        data[7] = (USIGN8) m_ui.ed_bt7->text().toShort(&isOk,16);
        break;

    case 1:
        u16val = m_ui.ed_word->text().toUShort();
        memcpy(data, &u16val, sizeof(u16val));
        break;

    case 2:
        u32val = m_ui.ed_word->text().toULong();
        memcpy(data, &u32val, sizeof(USIGN32));
        break;

    case 3:
        dval = m_ui.ed_word->text().toDouble();
        memcpy(data, &dval, sizeof(double));
        break;

    default:
        return;
        break;

    }

    retCode = CMA_InitBlockDownload(m_Can, // channel handle
                                    (USIGN8)m_ui.ed_nodeid->text().toShort(), // node id
                                    m_ui.ed_index->text().toShort(&isOk,16),  // index
                                    (USIGN8)m_ui.ed_subindex->text().toShort(), // subindex
                                    m_ui.ed_datalength->text().toShort(), // data length
                                    data); // buffer which contains the data

    if(retCode)
    {
        QString ErrorText = ErrorText.sprintf("ERROR: %i in CMA_InitBlockUpload", retCode);
        emit(doPrint(ErrorText));
        return;

    }
}

/**
 * *****************************************************************
 * @brief  This slot is called if the block upload button is pushed.
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void sdo::OnBlockUpload()
{

    USIGN16 retCode;
    bool isOk;

    retCode = CMA_InitBlockUpload(m_Can, // channel handle
                                  (USIGN8)m_ui.ed_nodeid->text().toShort(), // node id
                                  m_ui.ed_index->text().toShort(&isOk,16),  // index
                                  (USIGN8)m_ui.ed_subindex->text().toShort(), // subindex
                                  m_ui.ed_datalength->text().toShort()); // data length


    if(retCode)
    {
        QString ErrorText = ErrorText.sprintf("ERROR: %i in CMA_InitBlockUpload", retCode);
        emit(doPrint(ErrorText));
        return;
    }

}

/**
 * *****************************************************************
 * @brief  This slot is called if the radio button double is pushed.
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void sdo::OnDouble()
{
    m_type = 3;
    m_ui.ed_datalength->setText("8");
    m_ui.rd_double->setChecked(true);
    m_ui.rd_byte->setChecked(false);
    m_ui.rd_word->setChecked(false);
    m_ui.rd_long->setChecked(false);
}

/**
 * *****************************************************************
 * @brief  This slot is called if the radio button long is pushed.
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void sdo::OnLong()
{
    m_type=2;
    m_ui.ed_datalength->setText("4");
    m_ui.rd_double->setChecked(false);
    m_ui.rd_byte->setChecked(false);
    m_ui.rd_word->setChecked(false);
    m_ui.rd_long->setChecked(true);

}

/**
 * *****************************************************************
 * @brief  This slot is called if the radio button word is pushed.
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void sdo::OnWord()
{
    m_type = 1;
    m_ui.ed_datalength->setText("2");
    m_ui.rd_double->setChecked(false);
    m_ui.rd_byte->setChecked(false);
    m_ui.rd_word->setChecked(true);
    m_ui.rd_long->setChecked(false);

}

/**
 * *****************************************************************
 * @brief  This slot is called if the radio button byte is pushed.
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void sdo::OnByte()
{
    m_type = 0;
    m_ui.ed_datalength->setText("1");
    m_ui.rd_double->setChecked(false);
    m_ui.rd_byte->setChecked(true);
    m_ui.rd_word->setChecked(false);
    m_ui.rd_long->setChecked(false);

}


void sdo::OnOk(void)
{
    done(1);
}

/**o
 ******************************************************************
 * @brief  update the PDO data inside the PDO table
 * @author Werner Knorsch, Softing Automation GmbH
 * @date   2013-02-05
 *
 * @return -
 *
 ******************************************************************
 */

void sdo::OnSdoUploadComplete(unsigned long len, quint64 data)
{
    unsigned char *datas = (unsigned char*)&data;
    unsigned short *pus = (unsigned short*)&data;
    unsigned long *pul = (unsigned long*)&data;
    
    QString text;
    
    m_ui.ed_word->setText(" ");
    m_ui.ed_long->setText(" ");
    
    text = text.sprintf("%02x ", datas[0]);
    m_ui.ed_bt0->setText(text);
    text = text.sprintf("%02x ", datas[1]);
    m_ui.ed_bt1->setText(text);
    text = text.sprintf("%02x ", datas[2]);
    m_ui.ed_bt2->setText(text);
    text = text.sprintf("%02x ", datas[3]);
    m_ui.ed_bt3->setText(text);
    text = text.sprintf("%02x ", datas[4]);
    m_ui.ed_bt4->setText(text);
    text = text.sprintf("%02x ", datas[5]);
    m_ui.ed_bt5->setText(text);
    text = text.sprintf("%02x ", datas[6]);
    m_ui.ed_bt6->setText(text);
    text = text.sprintf("%02x ", datas[7]);
    m_ui.ed_bt7->setText(text);
    
    text = text.sprintf("%lu",len);
    m_ui.ed_datalength->setText(text);
    
    
    
    switch(len)
    {
      case 2:
	text = text.sprintf("%u ", *pus);
        m_ui.ed_word->setText(text);
	m_ui.rd_byte->setChecked(true);
	m_ui.rd_word->setChecked(true);
	m_ui.rd_long->setChecked(false);
        break;
      
      case 4:
	text = text.sprintf("%lu ", *pul);
        m_ui.ed_long->setText(text);	
 	m_ui.rd_byte->setChecked(true);
	m_ui.rd_word->setChecked(false);
	m_ui.rd_long->setChecked(true);	
	break;
	
      default:
	m_ui.rd_byte->setChecked(true);
	m_ui.rd_word->setChecked(false);
	m_ui.rd_long->setChecked(false);	
	
	break;
    }

}






#include "csdo.moc"
