
/***************************************************************************
 *   canlib - Softing Library for API of Can-Cards                         *
 *   Copyright (C) 2007-2015 by Softing Industrial Automation GmbH         *
 *   support.automation@softing.com                                        *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this library; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <linux/kdev_t.h>
#include <dirent.h>

#include <sys/ioctl.h>
#include <unistd.h>

#include <errno.h>
#include <string.h>

#include "BaseTypes.h"
#include "can_def.h"
#include "canL2.h"
#include "can_ioctl.h"

#define CANLIB_FAIL -100
#define INVALID_HANDLE_VALUE -1
#define DEVFILEPREFIX "/dev/Softing_AG_"
#define CAN_CONFIG_FILE "/etc/softing/candevices.conf"
#define LINE_SIZE 300




// get the channel configruration from config file
int getScimConfig(CAN_HANDLE Can, L2CONFIG *pScimCfg);

/* +--- CANL2_init_signals         -------------------------------------------*/

int
CANL2_init_signals(CAN_HANDLE can_channel, USIGN32 ulChannelDirectionMask, USIGN32 ulChannelOutputDefaults)
{
  int rt = CANLIB_FAIL;
    
    
  USIGN32 pram[10] = { (int) ulChannelDirectionMask,
    (int) ulChannelOutputDefaults,
    0, 0, 0, 0, 0, 0, 0, 0
  };

  rt = ioctl(can_channel,
             CAN_IOC_CANL2_init_signals, pram);

  return rt;
}

/* +--- CANL2_read_signals         -------------------------------------------*/

int
    CANL2_read_signals(CAN_HANDLE can_channel, USIGN32 *pulChannelRead)
{
  int rt = CANLIB_FAIL;
    
    
  USIGN32 pram[10] = { *pulChannelRead,
    0,
    0, 0, 0, 0, 0, 0, 0, 0
  };

  rt = ioctl(can_channel,
             CAN_IOC_CANL2_read_signals, pram);
  
  *pulChannelRead = pram[0];

  return rt;
}

/* +--- CANL2_write_signals         -------------------------------------------*/

int
    CANL2_write_signals(CAN_HANDLE can_channel, USIGN32 ulChannelWrite, USIGN32 ulCareMask)
{
  int rt = CANLIB_FAIL;
    
    
  USIGN32 pram[10] = { (int) ulChannelWrite,
    (int) ulCareMask,
    0, 0, 0, 0, 0, 0, 0, 0
  };

  rt = ioctl(can_channel,
             CAN_IOC_CANL2_write_signals, pram);

  return rt;
}


/* +--- CANL2_enable_error_frame_detection         -------------------------------------------*/

int
CANL2_enable_error_frame_detection(CAN_HANDLE can_channel)
{
    int rt = CANLIB_FAIL;

    rt = ioctl(can_channel,
               CAN_IOC_CANL2_enable_error_frame_detection, NULL);

    return rt;
}


/* +--- CANL2_reset_chip         -------------------------------------------*/
int
CANL2_reset_chip(CAN_HANDLE can_channel)
{
    int rt = CANLIB_FAIL;


    rt = ioctl(can_channel,CAN_IOC_CANL2_reset_chip, NULL);

    return rt;
}


/* +--- CANL2_initialize_chip(2) -------------------------------------------*/
int
CANL2_initialize_chip(CAN_HANDLE can_channel,int presc, int sjw, int tseg1, int tseg2, int sam)
{
    int rt = CANLIB_FAIL;
    USIGN32 pram[10];
    L2CONFIG SCIMCfg;
    int ret;
    
    ret = getScimConfig(can_channel, &SCIMCfg);
    memset(pram,0,sizeof(pram));    
    

    if(ret)
    { // cfg not found!
      pram[0] =  (int) presc;
      pram[1] =  (int) sjw;
      pram[2] =  (int) tseg1;
      pram[3] =  (int) tseg2;
      pram[4] =  (int) sam;
    }
    else
    {
      pram[0] =  (int) (presc == -1) ? SCIMCfg.s32Prescaler : presc;
      pram[1] =  (int) (sjw == -1) ?   SCIMCfg.s32Sjw       : sjw;
      pram[2] =  (int) (tseg1 == -1) ? SCIMCfg.s32Tseg1     : tseg1;
      pram[3] =  (int) (tseg2 == -1) ? SCIMCfg.s32Tseg2     : tseg2;
      pram[4] =  (int) (sam == -1) ?   SCIMCfg.s32Sam       : sam;
      
    }
    


    rt = ioctl(can_channel,CAN_IOC_CANL2_initialize_chip, pram);


    return rt;


}



/* +--- CANL2_set_mode(2)        -------------------------------------------*/
int
CANL2_set_mode(CAN_HANDLE can_channel,int SleepMode, int SpeedMode)
{
    int rt = CANLIB_FAIL;
    
    
    USIGN32 pram[10] = { (int) SleepMode,
        (int) SpeedMode,
        0, 0, 0, 0, 0, 0, 0, 0
    };


    rt = ioctl(can_channel,CAN_IOC_CANL2_set_mode, pram);


    return rt;
}




/* +--- CANL2_set_acceptance(2)  -------------------------------------------*/
int
CANL2_set_acceptance(CAN_HANDLE can_channel,USIGN32 AccCodeStd,
                     USIGN32 AccMaskStd,
                     USIGN32 AccCodeXtd, USIGN32 AccMaskXtd)
{
    int rt = CANLIB_FAIL;
    USIGN32 pram[10];
    L2CONFIG SCIMCfg;
    int ret;
    
    ret = getScimConfig(can_channel, &SCIMCfg);
    memset(pram,0,sizeof(pram));    
 
    if(ret)
    { // cfg not found!
    
      pram[0] = (USIGN32) AccCodeStd;
      pram[1] = (USIGN32) AccMaskStd;
      pram[2] = (USIGN32) AccCodeXtd;
      pram[3] = (USIGN32) AccMaskXtd;
    }
    else
    {
      pram[0] = ((long)AccCodeStd == -1) ? SCIMCfg.s32AccCodeStd : AccCodeStd;
      pram[1] = ((long)AccMaskStd == -1) ? SCIMCfg.s32AccMaskStd : AccMaskStd;
      pram[2] = ((long)AccCodeXtd == -1) ? SCIMCfg.s32AccCodeXtd : AccCodeXtd;
      pram[3] = ((long)AccMaskXtd == -1) ? SCIMCfg.s32AccMaskXtd : AccMaskXtd;
      
    }


    rt = ioctl(can_channel,CAN_IOC_CANL2_set_acceptance, pram);


    return rt;
}



/* +--- CANL2_set_output_control(2)  ---------------------------------------*/
int
CANL2_set_output_control(CAN_HANDLE can_channel,int OutputControl)
{

    int rt = CANLIB_FAIL;
    L2CONFIG SCIMCfg;
    int ret;
    
    ret = getScimConfig(can_channel, &SCIMCfg);
    
    if(ret)
    { // no cfg; use value from user!
  
      rt = ioctl(can_channel,CAN_IOC_CANL2_set_output_control,
                 (void *) OutputControl);
    }
    else
    {
      if(OutputControl == -1)
      { // -1 means: get it from SCIM-Cfg.
        rt = ioctl(can_channel,CAN_IOC_CANL2_set_output_control,
                   (void *) SCIMCfg.s32OutputCtrl);
        
      }
      else
      { 
        rt = ioctl(can_channel,CAN_IOC_CANL2_set_output_control,
                   (void *) OutputControl);
        
      }
    }


    return rt;

}



int
CANL2_initialize_interface(CAN_HANDLE can_channel,int ReceiveFifoEnable,
                           int ReceivePollAll,
                           int ReceiveEnableAll,
                           int ReceiveIntEnableAll,
                           int AutoRemoteEnable,
                           int TransmitReqFifoEnable,
                           int TransmitPollAll,
                           int TransmitAckEnableAll,
                           int TransmitAckFifoEnable,
                           int TransmitRmtFifoEnable)
{
    int rt = CANLIB_FAIL;
    USIGN32 pram[10] = { (int) ReceiveFifoEnable,
        (int) ReceivePollAll,
        (int) ReceiveEnableAll,
        (int) ReceiveIntEnableAll,
        (int) AutoRemoteEnable,
        (int) TransmitReqFifoEnable,
        (int) TransmitPollAll,
        (int) TransmitAckEnableAll,
        (int) TransmitAckFifoEnable,
        (int) TransmitRmtFifoEnable
    };


    rt = ioctl(can_channel,CAN_IOC_CANL2_initialize_interface, pram);

    return rt;
}

/* +--- CANL2_define_object(2)          ------------------------------------*/

int
CANL2_define_object(CAN_HANDLE can_channel,USIGN32 Handle,
                    int *ObjectNumber,
                    int Type,
                    int ReceiveIntEnable,
                    int AutoRemoteEnable, int TransmitAckEnable)
{
    int rt = CANLIB_FAIL;
    USIGN32 pram[10] = { (USIGN32) Handle,
        (int) *ObjectNumber,
        (int) Type,
        (int) ReceiveIntEnable,
        (int) AutoRemoteEnable,
        (int) TransmitAckEnable,
        0, 0, 0, 0
    };


    rt = ioctl(can_channel,CAN_IOC_CANL2_define_object, pram);


    *ObjectNumber = (int) pram[1];

    return rt;

}



/* +--- CANL2_define_cyclic(2)        --------------------------------------*/

int
CANL2_define_cyclic(CAN_HANDLE can_channel,int ObjectNumber, USIGN32 Rate, USIGN32 Cycles)
{

    int rt = CANLIB_FAIL;
    USIGN32 pram[10] = { (int) ObjectNumber,
        (USIGN32) Rate,
        (USIGN32) Cycles,
        0, 0, 0, 0, 0, 0, 0
    };


    rt = ioctl(can_channel,CAN_IOC_CANL2_define_cyclic, pram);


    return rt;

}



/* +--- CANL2_enable_fifo            ---------------------------------------*/

int
CANL2_enable_fifo(CAN_HANDLE can_channel)
{

    int rt = CANLIB_FAIL;


    rt = ioctl(can_channel,CAN_IOC_CANL2_enable_fifo, NULL);


    return rt;
}




/* +--- CANL2_enable_dyn_obj_buf     ---------------------------------------*/

int
CANL2_enable_dyn_obj_buf(CAN_HANDLE can_channel)
{

    int rt = CANLIB_FAIL;


    rt = ioctl(can_channel,CAN_IOC_CANL2_enable_dyn_obj_buf, NULL);


    return rt;
}

/* +--- CANL2_enable_timestamps      ---------------------------------------*/
/* this function has no effect on the CANcard                               */

int
CANL2_enable_timestamps(CAN_HANDLE can_channel)
{

    int rt = CANLIB_FAIL;


    rt = ioctl(can_channel,CAN_IOC_CANL2_enable_timestamps, NULL);


    return rt;
}

// CAN_IOC_CANL2_trace_level
int
    CANL2_trace_level(CAN_HANDLE can_channel, USIGN32 level)
{

    int rt = CANLIB_FAIL;
    USIGN32 pram[10] = { (int) level,
        0,
        0, 0, 0, 0, 0, 0, 0, 0
    };

    rt = ioctl(can_channel,CAN_IOC_CANL2_trace_level, pram);


    return rt;
}

/* +--- CANL2_enable_fifo_transmit_ack  ------------------------------------*/

int
CANL2_enable_fifo_transmit_ack(CAN_HANDLE can_channel)
{

    int rt = CANLIB_FAIL;


    rt = ioctl(can_channel,CAN_IOC_CANL2_enable_fifo_transmit_ack, NULL);


    return rt;

}



/* +--- CANL2_get_version            ---------------------------------------*/
/* the parameter hw_version is not used with the CAN-AC2                    */

int
CANL2_get_version(CAN_HANDLE can_channel,int *sw_version,
                  int *fw_version,
                  int *hw_version, int *license, int *can_chip_type)
{
    int rt = CANLIB_FAIL;
    USIGN32 pram[10] = { (int) *sw_version,
        (int) *fw_version,
        (int) *hw_version,
        (int) *license,
        (int) *can_chip_type,
        0, 0, 0, 0, 0
    };


    rt = ioctl(can_channel,CAN_IOC_CANL2_get_version, pram);


    *sw_version = pram[0];
    *fw_version = pram[1];
    *hw_version = pram[2];
    *license = pram[3];
    *can_chip_type = pram[4];
    can_chip_type[1] = pram[5];

    return rt;

}

/* +--- CANL2_get_serial_number      ---------------------------------------*/
/* this function has no effect on the CAN-AC2                               */

int
    CANL2_get_serial_number(CAN_HANDLE can_channel,USIGN32 *ser_number)
{

  int rt = CANLIB_FAIL;



  rt = ioctl(can_channel, CAN_IOC_CANL2_get_serial_number, ser_number);


  return rt;

}
/* +--- CANL2_get_phys_channel      ---------------------------------------*/
/* this function has no effect on the CAN-AC2                               */

int
    CANL2_get_phys_channel(CAN_HANDLE can_channel, USIGN32 *channel_number)
{

  int rt = CANLIB_FAIL;



  rt = ioctl(can_channel, CAN_IOC_CANL2_get_phys_channel, channel_number);


  return rt;

}

/* +--- CANL2_get_device_type      ---------------------------------------*/
/* this function has no effect on the CAN-AC2                               */

int
    CANL2_get_device_type(CAN_HANDLE can_channel, USIGN32 *device_type)
{

  int rt = CANLIB_FAIL;



  rt = ioctl(can_channel, CAN_IOC_CANL2_get_device_type, device_type);


  return rt;

}

/* +--- CANL2_start_chip             ---------------------------------------*/

int
CANL2_start_chip(CAN_HANDLE can_channel)
{

    int rt = CANLIB_FAIL;


    rt = ioctl(can_channel,CAN_IOC_CANL2_start_chip, NULL);


    return rt;

}

/* +--- CANL2_send_remote_object(2)  ---------------------------------------*/

int
CANL2_send_remote_object(CAN_HANDLE can_channel,int ObjectNumber, int DataLength)
{

    int rt = CANLIB_FAIL;
    USIGN32 pram[10] = { (int) ObjectNumber,
        (int) DataLength,
        0, 0, 0, 0, 0, 0, 0, 0
    };


    rt = ioctl(can_channel,CAN_IOC_CANL2_send_remote_object, pram);


    return rt;

}



/* +--- CANL2_supply_object_data(2)  ---------------------------------------*/

int
CANL2_supply_object_data(CAN_HANDLE can_channel,int ObjectNumber,
                         int DataLength, unsigned char *pData)
{
    int rt = CANLIB_FAIL;
    USIGN32 pram[10] = { (int) ObjectNumber,
        (int) DataLength,
        *(USIGN32 *) pData,
        *(((USIGN32 *) pData) + 1),
        0, 0, 0, 0, 0, 0
    };


    rt = ioctl(can_channel,CAN_IOC_CANL2_supply_object_data, pram);

    return rt;


}



/* +--- CANL2_supply_rcv_object_data(2)-------------------------------------*/

int
CANL2_supply_rcv_object_data(CAN_HANDLE can_channel,int ObjectNumber,
                             int DataLength, unsigned char *pData)
{
    int rt = CANLIB_FAIL;
    USIGN32 pram[10] = { (int) ObjectNumber,
        (int) DataLength,
        *(USIGN32 *) pData,
        *(((USIGN32 *) pData) + 1),
        0, 0, 0, 0, 0, 0
    };


    rt = ioctl(can_channel,CAN_IOC_CANL2_supply_rcv_object_data, pram);

    return rt;

}



/* +--- CANL2_send_object(2)         ---------------------------------------*/

int
CANL2_send_object(CAN_HANDLE can_channel,int ObjectNumber, int DataLength)
{

    int rt = CANLIB_FAIL;
    USIGN32 pram[10] = { (int) ObjectNumber,
        (int) DataLength,
        0, 0, 0, 0, 0, 0, 0, 0
    };


    rt = ioctl(can_channel,CAN_IOC_CANL2_send_object, pram);


    return rt;

}



/* +--- CANL2_write_object(2)        ---------------------------------------*/

int
CANL2_write_object(CAN_HANDLE can_channel,int ObjectNumber, int DataLength, unsigned char *pData)
{

    int rt = CANLIB_FAIL;
    USIGN32 pram[10] = { (int) ObjectNumber,
        (int) DataLength,
        *(USIGN32 *) pData,
        *(((USIGN32 *) pData) + 1),
        0, 0, 0, 0, 0, 0
    };


    rt = ioctl(can_channel,CAN_IOC_CANL2_write_object, pram);

    return rt;

}


int
CANL2_read_rcv_data(CAN_HANDLE can_channel,int ObjectNumber,
                    unsigned char *pRCV_Data, USIGN32 *Time)
{
    int rt = CANLIB_FAIL;
    USIGN32 pram[10] = { (int) ObjectNumber,
        0,                      /* (unsigned char)*pRCV_Data */
        0,                      /* (USIGN32)*Time */
        0, 0, 0, 0, 0, 0, 0
    };


    rt = ioctl(can_channel,CAN_IOC_CANL2_read_rcv_data, pram);

    *(USIGN32 *) pRCV_Data = *(USIGN32 *) &pram[1];
    *(USIGN32 *) &pRCV_Data[4] = *(USIGN32 *) &pram[2];
    *Time = (USIGN32) pram[3];

    return rt;

}



int
CANL2_read_xmt_data(CAN_HANDLE can_channel,int ObjectNumber,
                    int *pDataLength, unsigned char *pXMT_Data)
{
    int rt = CANLIB_FAIL;
    USIGN32 pram[10] = { (int) ObjectNumber,
        0,                      /* (unsigned char)*pDataLength */
        0,                      /* (USIGN32)*pXMT_Data */
        0, 0, 0, 0, 0, 0, 0
    };


    rt = ioctl(can_channel,CAN_IOC_CANL2_read_xmt_data, pram);


    *pDataLength = (unsigned char) pram[1];
    *(USIGN32 *) pXMT_Data = *(USIGN32 *) &pram[2];
    *(USIGN32 *) &pXMT_Data[4] = *(USIGN32 *) &pram[3];

    return rt;

}



int
    CANL2_read_ac(CAN_HANDLE can_channel, PARAM_STRUCT * param)
{

    int rt = CANLIB_FAIL;


    rt = ioctl(can_channel,CAN_IOC_CANL2_read_ac, param);
    param->Can = can_channel;

    return rt;

}

/* +--- CANL2_send_data              ---------------------------------------*/

int
CANL2_send_data(CAN_HANDLE can_channel,USIGN32 Ident,
                int Xtd, int DataLength, unsigned char *pData)
{
    int rt = CANLIB_FAIL;
    USIGN32 pram[10] = { (USIGN32) Ident,
        (int) Xtd,
        (int) DataLength,
        *(USIGN32 *) pData,
        *(((USIGN32 *) pData) + 1),
        0, 0, 0, 0, 0
    };


    rt = ioctl(can_channel,CAN_IOC_CANL2_send_data, pram);

    return rt;

}


/* +--- CANL2_send_remote            ---------------------------------------*/

int
CANL2_send_remote(CAN_HANDLE can_channel,USIGN32 Ident, int Xtd, int DataLength)
{
    int rt = CANLIB_FAIL;
    USIGN32 pram[10] = { (USIGN32) Ident,
        (int) Xtd,
        (int) DataLength,
        0, 0, 0, 0, 0, 0, 0
    };


    rt = ioctl(can_channel,CAN_IOC_CANL2_send_remote, pram);


    return rt;

}






/* +--- CANL2_reinitialize           ---------------------------------------*/
int
CANL2_reinitialize(CAN_HANDLE can_channel)
{

    int rt = CANLIB_FAIL;


    rt = ioctl(can_channel,CAN_IOC_CANL2_reinitialize, NULL);


    return rt;

}

/* +--- CANL2_get_time               ---------------------------------------*/
int
CANL2_get_time(CAN_HANDLE can_channel,USIGN32 *time)
{

    int rt = CANLIB_FAIL;
    USIGN32 pram[10] = { (int) *time,
        0, 0, 0, 0, 0, 0, 0, 0, 0
    };


    rt = ioctl(can_channel,CAN_IOC_CANL2_get_time, pram);


    *time = (int) pram[0];

    return rt;
}

/* +--- CANL2_get_bus_state          ---------------------------------------*/
int
CANL2_get_bus_state(CAN_HANDLE can_channel)
{

    int rt = CANLIB_FAIL;
    USIGN32 pram[10] = { (int) 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0
    };


    rt = ioctl(can_channel,CAN_IOC_CANL2_get_bus_state, pram);


    return rt;

}



/* +--- CANL2_reset_rcv_fifo     -------------------------------------------*/
int
CANL2_reset_rcv_fifo(CAN_HANDLE can_channel)
{
    int rt = CANLIB_FAIL;

    rt = ioctl(can_channel,CAN_IOC_CANL2_reset_rcv_fifo, NULL);

    return rt;
}

/* +--- CANL2_reset_xmt_fifo     -------------------------------------------*/
int
CANL2_reset_xmt_fifo(CAN_HANDLE can_channel)
{
    int rt = CANLIB_FAIL;

    rt = ioctl(can_channel,CAN_IOC_CANL2_reset_xmt_fifo, NULL);

    return rt;
}

/* +--- CANL2_reset_lost_msg_counter ---------------------------------------*/
int
CANL2_reset_lost_msg_counter(CAN_HANDLE can_channel)
{
    int rt = CANLIB_FAIL;

    rt = ioctl(can_channel,CAN_IOC_CANL2_reset_lost_msg_counter, NULL);

    return rt;
}

/* +--- CANL2_read_xmt_fifo_level ------------------------------------------*/
int
CANL2_read_xmt_fifo_level(CAN_HANDLE can_channel)
{
    int rt = CANLIB_FAIL;

    rt = ioctl(can_channel,CAN_IOC_CANL2_read_xmt_fifo_level, NULL);

    return rt;
}

/* +--- CANL2_read_xmt_fifo_level ------------------------------------------*/



/* +--- CANL2_read_rcv_fifo_level ------------------------------------------*/
int
CANL2_read_rcv_fifo_level(CAN_HANDLE can_channel)
{
    int rt = CANLIB_FAIL;

    rt = ioctl(can_channel,CAN_IOC_CANL2_read_rcv_fifo_level, NULL);

    return rt;
}

static BOOL getIntVal(char *str, char * format, char *tok, INT32 *pNumber)
{
  char *pStart = strstr(str,tok);
  
  // substring found?
  if(!pStart) return FALSE;
  // read till start of the number
  pStart += strlen(tok);
  
  sscanf(pStart,format,pNumber);
  
  return TRUE;
}

static int getChConfig(USIGN32 u32Serial, USIGN32 u32physChannel, USIGN32 u32DeviceType, char *CfgLine)
{
  USIGN32 u32DeviceType2;
  BOOL bCfgFound = FALSE;
  char lineStart[20];
  FILE *pCfgFile;

  
  memset(CfgLine,0,LINE_SIZE); 
  sprintf(lineStart, "%09u_%u", u32Serial, u32physChannel);
  
 // printf("looking for: %s\n",lineStart);
  
  // open the config file for reading!
  pCfgFile = fopen(CAN_CONFIG_FILE,"r");
  
  if(!pCfgFile)
    return -1;
  
  // looking for the part, which describes that channel
  while(!bCfgFound && !feof(pCfgFile))
  {
    fgets(CfgLine,2*LINE_SIZE,pCfgFile);
   // printf("checking %s\n",CfgLine);
    if(strlen(CfgLine) >= strlen(lineStart))
    {
      if(!memcmp(CfgLine,lineStart,strlen(lineStart)))
      {
        if(!getIntVal(CfgLine,"%u","type:",(INT32*)&u32DeviceType2))
        {
          printf("type not found\n");
          bCfgFound = FALSE;
        }
        else
        {
         // printf("%lu==%lu\n",u32DeviceType2,u32DeviceType);
          bCfgFound = (u32DeviceType2==u32DeviceType);
        }
      }
    }
    
  }
  
  fclose(pCfgFile);
  
  if(!bCfgFound)
  { // channel configuration not found!    
    return -2;
  }
   
  return 0;
  
}

// get the channel configruration from config file
int getScimConfig(CAN_HANDLE Can, L2CONFIG *pScimCfg)
{
  int ret = 0;
  char line[2 * LINE_SIZE];
  INT32 enableAck,enableErrorframe;
  USIGN32 u32Serial, u32physChannel, u32DeviceType1=0;
  
 // get the serial number
  ret = CANL2_get_serial_number(Can, &u32Serial);
  if(ret) return ret;
  
  // get the pysical channel number (1..2)
  ret = CANL2_get_phys_channel(Can, &u32physChannel);
  if(ret) return ret;
  
    // get the device type
  ret = CANL2_get_device_type(Can, &u32DeviceType1);
  if(ret) return ret;

  ret = getChConfig(u32Serial, u32physChannel, u32DeviceType1, line);
  
  if(ret)
  {
   // printf("getChConfig returned %i\n",ret);
    return ret;
  }
      
  // configuration found!
  if(!getIntVal(line,"%u","ack:",&enableAck))
    return -3;
  
  pScimCfg->bEnableAck = (enableAck == 1);
  
  if(!getIntVal(line,"%u","errorframes:",&enableErrorframe))
    return -3;
      
  pScimCfg->bEnableErrorframe = (enableErrorframe == 1);
  
  if(!getIntVal(line,"%x","acccstd:",&pScimCfg->s32AccCodeStd))
    return -3;
  
  if(!getIntVal(line,"%x","accmstd:",&pScimCfg->s32AccMaskStd))
    return -3;
  
  if(!getIntVal(line,"%x","acccxtd:",&pScimCfg->s32AccCodeXtd))
    return -3;
  
  if(!getIntVal(line,"%x","accmxtd:",&pScimCfg->s32AccMaskXtd))
    return -3;
  
  if(!getIntVal(line,"%x","ocr:",&pScimCfg->s32OutputCtrl))
    return -3;
  
  if(!getIntVal(line,"%u","presc:",&pScimCfg->s32Prescaler))
    return -3;
  
  if(!getIntVal(line,"%u","sam:",&pScimCfg->s32Sam))
    return -3;
  
  if(!getIntVal(line,"%u","sjw:",&pScimCfg->s32Sjw))
    return -3;
  
  if(!getIntVal(line,"%u","tseg1:",&pScimCfg->s32Tseg1))
    return -3;
  
  if(!getIntVal(line,"%u","tseg2:",&pScimCfg->s32Tseg2))
    return -3;
  
  return 0;
}


static int checkCfg(L2CONFIG *pCfg, L2CONFIG *pUserCfg)
{
  int ret = 0;

  if(pUserCfg->bEnableAck != GET_FROM_SCIM)
    pCfg->bEnableAck = pUserCfg->bEnableAck;

  if(pUserCfg->bEnableErrorframe != GET_FROM_SCIM)
    pCfg->bEnableErrorframe = pUserCfg->bEnableErrorframe;

  if(pUserCfg->s32AccCodeStd != GET_FROM_SCIM)
    pCfg->s32AccCodeStd = pUserCfg->s32AccCodeStd;

  if(pUserCfg->s32AccMaskStd != GET_FROM_SCIM)
    pCfg->s32AccMaskStd = pUserCfg->s32AccMaskStd;

  if(pUserCfg->s32AccCodeXtd != GET_FROM_SCIM)
    pCfg->s32AccCodeXtd = pUserCfg->s32AccCodeXtd;

  if(pUserCfg->s32AccMaskXtd != GET_FROM_SCIM)
    pCfg->s32AccMaskXtd = pUserCfg->s32AccMaskXtd;

  if(pUserCfg->s32OutputCtrl != GET_FROM_SCIM)
    pCfg->s32OutputCtrl = pUserCfg->s32OutputCtrl;

  if(pUserCfg->s32Prescaler != GET_FROM_SCIM)
  {
    if((pUserCfg->s32Prescaler > 32) || (pUserCfg->s32Prescaler < 1))
    {
      return CANL2_WRONG_PARAM;
    }
    pCfg->s32Prescaler = pUserCfg->s32Prescaler;
  }

  if(pUserCfg->s32Sam != GET_FROM_SCIM)
  {
    if(pUserCfg->s32Sam > 3)
    {
      return CANL2_WRONG_PARAM;
    }
    pCfg->s32Sam = pUserCfg->s32Sam;
  }

  if(pUserCfg->s32Sjw != GET_FROM_SCIM)
  {
    if((pUserCfg->s32Sjw > 4) || (pUserCfg->s32Sjw < 1))
    {
      return CANL2_WRONG_PARAM;
    }
    pCfg->s32Sjw = pUserCfg->s32Sjw;
  }

  if(pUserCfg->s32Tseg1 != GET_FROM_SCIM)
  {
    if((pUserCfg->s32Tseg1 > 16) || (pUserCfg->s32Tseg1 < 1))
    {
      return CANL2_WRONG_PARAM;
    }
    pCfg->s32Tseg1 = pUserCfg->s32Tseg1;
  }

  if(pUserCfg->s32Tseg2 != GET_FROM_SCIM)
  {
    if((pUserCfg->s32Tseg2 > 8) || (pUserCfg->s32Tseg2 < 1))
    {
      return CANL2_WRONG_PARAM;
    }
    
    pCfg->s32Tseg2 = pUserCfg->s32Tseg2;
  }

  if(pUserCfg->hEvent != INVALID_HANDLE_VALUE)
    pCfg->hEvent = pUserCfg->hEvent;

  return ret;
}


int CANL2_initialize_fifo_mode(CAN_HANDLE Can, L2CONFIG *pUserCfg)
{
  int ret = 0;
  L2CONFIG ScimConfig;
    
  // reset can chip
  ret = CANL2_reset_chip(Can);
  if(ret) return ret;
  
  // get the configuration build from SCIM 
  ret = getScimConfig(Can, &ScimConfig);
  
  if(ret)
  { // error handling use configuration from user
    // no configuration available or corrupt
    memcpy(&ScimConfig, pUserCfg, sizeof(L2CONFIG));
  }
  else
  {
    if(checkCfg(&ScimConfig, pUserCfg))
    {
      return CANL2_WRONG_PARAM;
    }
  }

  // set baudrate
  ret = CANL2_initialize_chip(Can, ScimConfig.s32Prescaler, ScimConfig.s32Sjw, 
                              ScimConfig.s32Tseg1, ScimConfig.s32Tseg2, ScimConfig.s32Sam);
  if(ret) return ret;

  // set acceptance filter
  ret = CANL2_set_acceptance(Can, ScimConfig.s32AccCodeStd, ScimConfig.s32AccMaskStd, 
                             ScimConfig.s32AccCodeXtd, ScimConfig.s32AccMaskXtd);
  if(ret) return ret;

  // define output control register
  ret = CANL2_set_output_control(Can, ScimConfig.s32OutputCtrl);
  if(ret) return ret;

  // select fifo mode
  ret = CANL2_enable_fifo(Can);
  if(ret) return ret;

  // should the appl. be informed, if a message is successful transmitted?
  if(ScimConfig.bEnableAck)
  {
    ret = CANL2_enable_fifo_transmit_ack(Can);
  }

  if(ret) return ret;

  // should the appl. be informed about error frames on the bus?
  if(ScimConfig.bEnableErrorframe)
  {
    ret = CANL2_enable_error_frame_detection(Can);
  }

  if(ret) return ret;

  // go online
  ret = CANL2_start_chip(Can);
  
  
  
  
  return ret;
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       get_CAN_channels                          */
/*                                                                          */
/* FUNCTIONAL_DESCRIPTION         retrives all channels in a specified proc */
/*                                directory                                 */
/*                                                                          */
/* INPUTS                                                                   */
/*                                pProcDir: path to the proc file           */
/*                                pBuffer:  buffer where the info about     */
/*                                          the CAN channels will be stored */
/*                                max:      number of free entries in       */
/*                                          pBuffer                         */
/*                                                                          */
/* OUTPUTS                        number of channels found in the directory */
/*                                                                          */
/*HEAD_END*******************************************************************/

static INT32 get_CAN_channels(char *pProcDir, PCHDSNAPSHOT pBuffer, INT32 max)
{
  
  struct dirent **namelist;
  int n, min=0;
  BOOL bCfgNotFound;

  n = scandir(pProcDir, &namelist, 0, alphasort);
  if (n < 0)
  {
    
  }
  else 
  {
    while(n--) 
    {
      FILE *pDev = NULL;
      char line[LINE_SIZE];
      
      
      if(
         strcmp(namelist[n]->d_name,"..") && 
         strcmp(namelist[n]->d_name,".")
        )
      {
        strcpy(line,pProcDir);
        strcat(line,namelist[n]->d_name);
        pDev = fopen(line,"r");
        if(pDev)
        {
          USIGN32 u32Serial,u32PhysCh,u32DeviceType,u32FwId,bIsOpen;
          
          memset(line,0,LINE_SIZE);
          fgets(line, LINE_SIZE, pDev);
          
          sscanf(namelist[n]->d_name,"%u",&u32Serial);            
          getIntVal(line,"%u", "channel:", (INT32*)&u32PhysCh);
          getIntVal(line,"%u", "type:", (INT32*)&u32DeviceType);
            
          getIntVal(line,"%u", "firmware:", (INT32*)&u32FwId);
          getIntVal(line,"%u", "open:", (INT32*)&bIsOpen);
          bCfgNotFound = getChConfig(u32Serial, u32PhysCh, u32DeviceType, line);
          
          if(min < max)
          {
            char *pCh;
            
            if(!bCfgNotFound)
            {
              pBuffer[min].u32Serial = u32Serial;            
              pBuffer[min].u32PhysCh = u32PhysCh;
              pBuffer[min].u32DeviceType = u32DeviceType;
            
              pBuffer[min].u32FwId = u32FwId;
              pBuffer[min].bIsOpen = bIsOpen;
             
              pCh = strstr(line,"\n");
              if(pCh) *pCh = 0;
            
              pCh = strstr(line,"name:");
            
              if(pCh)
              {
                pCh += sizeof("name:")-1;
                if(strlen(pCh) >= sizeof(pBuffer[min].ChannelName))
                {
                  memcpy(pBuffer[min].ChannelName, pCh, sizeof(pBuffer[min].ChannelName));
                }
                else
                {
                  memcpy(pBuffer[min].ChannelName, pCh, strlen(pCh)+1);
                }
              }
            }
          }
          fclose(pDev);
          if(!bCfgNotFound) min++;
        }
        else
        {
          //printf("error: can not open file [%s] for reading\n",line);
        }
      }
      
      free(namelist[n]);
    }
    free(namelist);
  }  
  
  return min;
  
}

/*HEAD_BEG*******************************************************************/
/*                                                                          */
/* FUNCTION                       CANL2_get_all_CAN_channels                */
/*                                                                          */
/* FUNCTIONAL_DESCRIPTION         retrives all plugged devices from the     */
/*                                driver                                    */
/*                                                                          */
/* INPUTS                         -                                         */
/* OUTPUTS                        OK_                                       */
/*                                CANL2_FW_TIMEOUT    timeout of firmware             */
/*                                                                          */
/*HEAD_END*******************************************************************/
int CANL2_get_all_CAN_channels(USIGN32 u32ProvidedBufferSize,
                                OUT PUSIGN32 pu32NeededBufferSize,
                                OUT PUSIGN32 pu32NumOfChannels,
                                OUT PCHDSNAPSHOT pBuffer)
                                   
{
  int s32Max = u32ProvidedBufferSize/sizeof(CHDSNAPSHOT);
  int s32Defined;
  
  // the information about CAN channels will be published by the driver
  // as the contents of proc-files in the /proc directory
  
  s32Defined = get_CAN_channels("/proc/softing_can_pci/",pBuffer,s32Max); 
  s32Defined += get_CAN_channels("/proc/softing_can_pcmcia/",&pBuffer[s32Defined],s32Max-s32Defined); 
  s32Defined += get_CAN_channels("/proc/softing_can_104/",&pBuffer[s32Defined],s32Max-s32Defined); 
  s32Defined += get_CAN_channels("/proc/softing_can_eco/",&pBuffer[s32Defined],s32Max-s32Defined); 
  s32Defined += get_CAN_channels("/proc/softing_can_pro/",&pBuffer[s32Defined],s32Max-s32Defined); 
  *pu32NumOfChannels = s32Defined;
  *pu32NeededBufferSize = s32Defined * sizeof(CHDSNAPSHOT);
  
  return 0;
}

int
    INIL2_initialize_channel(CAN_HANDLE *channel_handle, char *channel_name)
{
  int can_fd = -1;
  int rt;
  CANPC_RESOURCES crResources;
  char name[LINE_SIZE];
  
  // all device file names (channel names) start with Softing_AG_
  strcpy(name,DEVFILEPREFIX);
  strcat(name,channel_name);
  
  can_fd = open(name, O_RDONLY);
  //printf(" open %s\n",name);
  
  if(can_fd < 0)
    return -1;
  
  rt = ioctl(can_fd, CAN_IOC_INIL2_initialize_board, (USIGN32 *) &crResources);
  
  if(rt < 0)
    return -2;
  
  rt = ioctl(can_fd, CAN_IOC_CANL2_reset_board, NULL);
  
  *channel_handle = can_fd;
  
  return rt;
}

/* +--- INIPC_close_board         ------------------------------------------*/
int
INIL2_close_channel(CAN_HANDLE channel_handle)
{
  
  close(channel_handle);
 
  return 0;
}
